{$J+,Z4}
unit BitmapObject;
{------------------------------------------------------------------------------}
{       This code is copyright (C) 2006-2023 by Michael in der Wiesche;        }
{                         it is based on Hani Atassi's                         }
{          "Insert any HBITMAP (Bitmap) in your RichEdit Control" at           }
{      <http://www.codeguru.com/Cpp/controls/richedit/article.php/c5383>       }
{------------------------------------------------------------------------------}

interface

uses
  Windows, RichOle, RichEdit, OLE2, Graphics;

type
  IBitmapObject = class(IDataObject)
  private
    FRefCount: DWord;
    FRelease: Longbool;
    FFormatEtc: TFormatEtc;
    FStgMedium: TStgMedium;
  public
    constructor Create;
    destructor Destroy; override;
    function SetBitmap(Bitmap: TBitmap): HResult;
    function GetOleObject(OleClientSite: IOleClientSite; Storage: IStorage; var OleObject: IOleObject): HResult;

    // Methods of the IUnknown interface
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;

    // Methods of the IDataObject interface
    function GetData(var formatetcIn: TFormatEtc; var medium: TStgMedium): HResult; override;
    function GetDataHere(var formatetc: TFormatEtc; var medium: TStgMedium): HResult; override;
    function QueryGetData(var formatetc: TFormatEtc): HResult; override;
    function GetCanonicalFormatEtc(var formatetc: TFormatEtc; var formatetcOut: TFormatEtc): HResult; override;
    function SetData(var formatetc: TFormatEtc; var medium: TStgMedium; fRelease: BOOL): HResult; override;
    function EnumFormatEtc(dwDirection: Longint; var enumFormatEtc: IEnumFormatEtc): HResult; override;
    function DAdvise(var formatetc: TFormatEtc; advf: Longint; advSink: IAdviseSink; var dwConnection: Longint): HResult; override;
    function DUnadvise(dwConnection: Longint): HResult; override;
    function EnumDAdvise(var enumAdvise: IEnumStatData): HResult; override;
  end;


implementation

constructor IBitmapObject.Create;
begin
  inherited Create;
  AddRef;
end;

destructor IBitmapObject.Destroy;
begin
  if FRelease then ReleaseStgMedium(FStgMedium);
  inherited Destroy;
end;

function IBitmapObject.SetBitmap(Bitmap: TBitmap): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
begin
  with StgMedium do begin
    tyMed := TYMED_GDI;
    hBitmap := Bitmap.Handle;
    unkForRelease := nil;
  end;
  with FormatEtc do begin
    cfFormat := CF_BITMAP;
    ptd := nil;
    dwAspect := DVASPECT_CONTENT;
    lIndex := -1;
    tyMed := TYMED_GDI;
  end;
  Result := SetData(FormatEtc, StgMedium, false);
end;

function IBitmapObject.GetOleObject(OleClientSite: IOleClientSite; Storage: IStorage; var OleObject: IOleObject): HResult;
begin
  Result := OleCreateStaticFromData(Self, IID_IOleObject, OLERENDER_FORMAT, @FFormatEtc,
				    OleClientSite, Storage, OleObject);
end;

// Methods of the IUnknown interface

function IBitmapObject.QueryInterface(const iid: TIID; var obj): HResult;
begin
  if IsEqualIID(iid, IID_IUnknown) or IsEqualIID(iid, IID_IDataObject) then begin
    IUnknown(obj) := Self;
    Self.AddRef;
    Result := S_OK;
  end
  else Result := E_NOINTERFACE;
end;

function IBitmapObject.AddRef: Longint;
begin
  inc(FRefCount);
  Result := FRefCount;
end;

function IBitmapObject.Release: Longint;
begin
  dec(FRefCount);
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;

// Methods of the IDataObject Interface

function IBitmapObject.GetData(var formatetcIn: TFormatEtc; var medium: TStgMedium): HResult;
var
  hDest: THandle;
begin
  hDest := OleDuplicateData(FStgMedium.hBitmap, CF_BITMAP, 0);
  if hDest <> 0 then begin
    medium.tymed := TYMED_GDI;
    medium.hBitmap := hDest;
    medium.unkForRelease := nil;
    Result := S_OK;
  end
  else Result := E_HANDLE;
end;

function IBitmapObject.GetDataHere(var formatetc: TFormatEtc; var medium: TStgMedium): HResult;
begin
  Result := E_NOTIMPL;
end;

function IBitmapObject.QueryGetData(var formatetc: TFormatEtc): HResult;
begin
  Result := E_NOTIMPL;
end;

function IBitmapObject.GetCanonicalFormatEtc(var formatetc: TFormatEtc; var formatetcOut: TFormatEtc): HResult;
begin
  formatetcOut.ptd := nil;
  Result := E_NOTIMPL;
end;

function IBitmapObject.SetData(var formatetc: TFormatEtc; var medium: TStgMedium; fRelease: BOOL): HResult;
begin
  Self.FRelease := fRelease;
  FFormatEtc := formatetc;
  FStgMedium := medium;
  Result := S_OK;
end;

function IBitmapObject.EnumFormatEtc(dwDirection: Longint; var enumFormatEtc: IEnumFormatEtc): HResult;
begin
  Result := E_NOTIMPL;
end;

function IBitmapObject.DAdvise(var formatetc: TFormatEtc; advf: Longint; advSink: IAdviseSink; var dwConnection: Longint): HResult;
begin
  Result := E_NOTIMPL;
end;

function IBitmapObject.DUnadvise(dwConnection: Longint): HResult;
begin
  Result := E_NOTIMPL;
end;

function IBitmapObject.EnumDAdvise(var enumAdvise: IEnumStatData): HResult;
begin
  Result := E_NOTIMPL;
end;

end.

