{$J+,Z4}
unit CBExtend;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}
{$IFDEF CONDITIONALEXPRESSIONS}
{$WARN SUSPICIOUS_TYPECAST OFF}
{$WARN UNIT_PLATFORM OFF}
{$ENDIF}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls,
  FileCtrl, StdCtrls, StrUtil, ToolTip, Utf8, UxThemes;


type
// TPlainCBEx ------------------------------------------------------------------
  TOnListItemSelected = procedure(Sender: TObject; Index: Integer) of Object;
  TPlainCBEx = class(TComboBox)
  private
    FCBXTheme: THandle;
    FDroppedWidth: Integer;
    FListDropped: Boolean;
    FListMargin: Integer;
    FOnListDrop: TNotifyEvent;
    FOnListClose: TNotifyEvent;
    FOnListItemSelected: TOnListItemSelected;
    procedure ListDrop;
    procedure ListClose;
    function  OpenCBXTheme: THandle;
    procedure CloseCBXTheme;
    procedure ChangeOnShow; virtual;
    function  GetDroppedWidth: Integer;
    procedure SetDroppedWidth(Value: Integer);
    function  GetIndex(Index: Integer): Integer;
    procedure SetIndex(Index: Integer; const Value: Integer);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    property DroppedWidth: Integer read GetDroppedWidth write SetDroppedWidth;
    property Indices[Index: Integer]: Integer read GetIndex write SetIndex;
    property ListDropped: Boolean read FListDropped;
  published
    property OnListDrop: TNotifyEvent read FOnListDrop write FOnListDrop;
    property OnListClose: TNotifyEvent read FOnListClose write FOnListClose;
    property OnListItemSelected: TOnListItemSelected read FOnListItemSelected write FOnListItemSelected;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


// TFontCBEx -------------------------------------------------------------------
  TOnGetSysFontNames = procedure(Sender: TObject; FontList: TStrings; var PreselectedFont, NoDefaultFontString: String) of Object;
  TFontCBEx = class(TPlainCBEx)
  private
    FNoDefaultFontString: String;
    FOnGetSysFontNames: TOnGetSysFontNames;
    procedure ChangeOnShow; override;
    procedure SetNoDefaultFontString(const Value: String);
    procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property OnGetSysFontNames: TOnGetSysFontNames read FOnGetSysFontNames write FOnGetSysFontNames;
    property NoDefaultFontString: String read FNoDefaultFontString write SetNoDefaultFontString;
  end;


// TDriveCBEx ------------------------------------------------------------------
  TDriveCBEx = class(TDriveComboBox)
  private
    FListDropped: Boolean;
    FOnListDrop: TNotifyEvent;
    FOnListClose: TNotifyEvent;
    procedure ListDrop;
    procedure ListClose;
  protected
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  public
    property ListDropped: Boolean read FListDropped;
  published
    property OnListDrop: TNotifyEvent read FOnListDrop write FOnListDrop;
    property OnListClose: TNotifyEvent read FOnListClose write FOnListClose;
  end;


// TFilterCBEx -----------------------------------------------------------------
  TFilterCBEx = class(TFilterComboBox)
  private
    FListDropped: Boolean;
    FOnListDrop: TNotifyEvent;
    FOnListClose: TNotifyEvent;
    procedure ListDrop;
    procedure ListClose;
  protected
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  public
    property ListDropped: Boolean read FListDropped;
  published
    property OnListDrop: TNotifyEvent read FOnListDrop write FOnListDrop;
    property OnListClose: TNotifyEvent read FOnListClose write FOnListClose;
  end;


procedure Register;

implementation


const
  DRAWSTYLE = DT_EXTERNALLEADING or DT_NOCLIP or DT_HIDEPREFIX or DT_SINGLELINE or DT_VCENTER;


// TPlainCBEx ------------------------------------------------------------------

procedure TPlainCBEx.ListDrop;
begin
  FListDropped := true;
  if Assigned(FOnListDrop) then FOnListDrop(Self);
end;

procedure TPlainCBEx.ListClose;
begin
  FListDropped := false;
  if Assigned(FOnListClose) then FOnListClose(Self);
end;

function TPlainCBEx.OpenCBXTheme: THandle;
begin
  if UseThemes then begin
    Result := OpenThemeData(Handle, 'ComboBox');
    Exit;
  end;
  Result := 0;
end;

procedure TPlainCBEx.CloseCBXTheme;
begin
  if FCBXTheme <> 0 then begin
    CloseThemeData(FCBXTheme);
    FCBXTheme := 0;
  end;
end;

function TPlainCBEx.GetDroppedWidth: Integer;
begin
  Result := SendMessage(WindowHandle, CB_GETDROPPEDWIDTH, 0, 0);
end;

procedure TPlainCBEx.SetDroppedWidth(Value: Integer);
begin
  if Value > FDroppedWidth then begin
    SendMessage(WindowHandle, CB_SETDROPPEDWIDTH, Value, 0);
    FDroppedWidth := Value;
  end;
end;

function  TPlainCBEx.GetIndex(Index: Integer): Integer;
begin
  Result := Integer(Items.Objects[Index]);
end;

procedure TPlainCBEx.SetIndex(Index: Integer; const Value: Integer);
begin
  Items.Objects[Index] := Ptr(Index);
end;

procedure TPlainCBEx.ChangeOnShow;
begin
  if Showing then begin
    ParentFont := false;
    Canvas.Font.Size := Font.Size;
  end;
end;

procedure TPlainCBEx.CMColorChanged(var Message: TMessage);
begin
  inherited;
  if EnableThemes(WindowHandle, (Color = clWindow) or (Color = clBtnFace)) then begin
    FCBXTheme := OpenCBXTheme;
    Exit;
  end;
  FCBXTheme := 0;
end;

procedure TPlainCBEx.CMShowingChanged(var Message: TMessage);
begin
  ChangeOnShow;
  inherited;
end;

procedure TPlainCBEx.CNCommand(var Message: TWMCommand);
begin
  with Message do begin
    case NotifyCode of
      CBN_DROPDOWN: ListDrop;
      CBN_CLOSEUP: ListClose;
    end;
  end;
  inherited;
end;

procedure TPlainCBEx.CNMeasureItem(var Message: TWMMeasureItem);
var
  rText: TRect;
  sText: String;
  iVScrollBarWidth: Integer;
begin
  with Message.MeasureItemStruct^ do begin
    iVScrollBarWidth := GetSystemMetrics(SM_CXVSCROLL);
    itemHeight := Self.ItemHeight;
    itemWidth := ClientWidth;
    if Style = csOwnerDrawVariable then begin
      if Integer(itemID) >= 0 then with Canvas do begin
	inc(itemWidth, iVScrollBarWidth);
	sText := Items[itemID];
	rText := Rect(0, 0, itemWidth, itemHeight);
	if FCBXTheme <> 0 then begin
	  GetThemeTextExtent(FCBXTheme, Handle, CP_BACKGROUND, CBB_NORMAL,
			     PWideChar(AnsiToWideString(sText)), -1, DRAWSTYLE, @rText, rText);
	end
	else DrawText(Handle, PChar(sText), -1, rText, DRAWSTYLE or DT_CALCRECT);
	SetDroppedWidth(rText.Right + iVScrollBarWidth + FListMargin);
      end;
    end;
  end;
end;

procedure TPlainCBEx.WMCreate(var Message: TWMCreate);
begin
  inherited;
  FListMargin := 2;
  Perform(CM_COLORCHANGED, 0, 0);
  FDroppedWidth := GetDroppedWidth;
end;

procedure TPlainCBEx.WMDestroy(var Message: TWMDestroy);
begin
  if FCBXTheme <> 0 then CloseCBXTheme;
  inherited;
end;

procedure TPlainCBEx.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  sText: String;
  iState: Integer;
begin
  if Index >= 0 then with Canvas do begin
    SetBkMode(Handle, TRANSPARENT);
    FillRect(Rect);
    sText := Items[Index];
    if ([odSelected, odFocused] * State) <> [] then begin
      Brush.Color := GetSysColor(COLOR_HIGHLIGHT);
      Font.Color := GetSysColor(COLOR_HIGHLIGHTTEXT);
    end
    else begin
      Brush.Color := GetSysColor(COLOR_WINDOW);
      Font.Color := GetSysColor(COLOR_WINDOWTEXT);
    end;
    if FCBXTheme <> 0 then begin
      if ([odSelected, odFocused] * State) <> [] then
	iState := CBXS_HOT
      else iState := CBXS_NORMAL;
      DrawThemeText(FCBXTheme, Handle, CP_BACKGROUND, iState,
		    PWideChar(AnsiToWideString(sText)), -1, DRAWSTYLE, 0, Rect);
    end
    else DrawText(Handle, PChar(sText), -1, Rect, DRAWSTYLE);
  end;
  if (odSelected in State) and Assigned(FOnListItemSelected) then FOnListItemSelected(Self, Index);
end;


// TFontCBEx -------------------------------------------------------------------

procedure TFontCBEx.ChangeOnShow;
var
  sPreselectedFontName, sNewNoDefaultFontString: String;
begin
  if Showing then begin
    Items.BeginUpdate;
    try
      Sorted := true;
      ParentFont := false;
      Canvas.Font.Size := Font.Size;
      if Assigned(FOnGetSysFontNames) then begin
	if (ItemIndex > 0) and (ItemIndex < Items.Count) then
	  sPreselectedFontName := Items[ItemIndex]
	else sPreselectedFontName := E;
	sNewNoDefaultFontString := FNoDefaultFontString;
	FOnGetSysFontNames(Self, Items, sPreselectedFontName, sNewNoDefaultFontString);
      end;
    finally
      SetNoDefaultFontString(sNewNoDefaultFontString);
      if sPreselectedFontName <> E then ItemIndex := Items.IndexOf(sPreselectedFontName);
      if ItemIndex = -1 then ItemIndex := Items.IndexOf(FNoDefaultFontString);
      Items.EndUpdate;
    end;
  end;
end;

procedure TFontCBEx.SetNoDefaultFontString(const Value: String);
var
  iIndex: Integer;
begin
  if Value = E then Exit;
  iIndex := Items.IndexOf(FNoDefaultFontString);
  if (iIndex = -1) or (Value <> FNoDefaultFontString) then begin
    if Items.Count > 1 then begin
      if (iIndex = 0) or (Items[0] = E) then
	Items[0] := Value
      else Items.Insert(0, Value);
      FNoDefaultFontString := Value;
    end;
  end;
end;

procedure TFontCBEx.CMShowingChanged(var Message: TMessage);
begin
  inherited;
end;

procedure TFontCBEx.CNMeasureItem(var Message: TWMMeasureItem);
var
  rText: TRect;
  sText: String;
  fNoFont: Longbool;
  iVScrollBarWidth: Integer;
begin
  with Message.MeasureItemStruct^ do begin
    iVScrollBarWidth := GetSystemMetrics(SM_CXVSCROLL);
    itemHeight := Self.ItemHeight;
    itemWidth := ClientWidth;
    if Style = csOwnerDrawVariable then begin
      if Integer(itemID) >= 0 then with Canvas do begin
	inc(itemWidth, iVScrollBarWidth);
	sText := Items[itemID];
	fNoFont := CompareTxt(sText, FNoDefaultFontString);
	if not fNoFont then Font.Name := sText;
	rText := Rect(0, 0, itemWidth, itemHeight);
	if FCBXTheme <> 0 then begin
	  GetThemeTextExtent(FCBXTheme, Handle, CP_BACKGROUND, CBB_NORMAL,
			     PWideChar(AnsiToWideString(sText)), -1, DRAWSTYLE, @rText, rText);
	end
	else DrawText(Handle, PChar(sText), -1, rText, DRAWSTYLE or DT_CALCRECT);
	SetDroppedWidth(rText.Right + iVScrollBarWidth + FListMargin);
      end;
    end;
  end;
end;

procedure TFontCBEx.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  sText: String;
  dtto: TDTTOpts;
  iState: Integer;
  fNoFont: Longbool;
begin
  if Index >= 0 then with Canvas do begin
    SetBkMode(Handle, TRANSPARENT);
    FillRect(Rect);
    sText := Items[Index];
    fNoFont := CompareTxt(sText, FNoDefaultFontString);
    if not fNoFont then Font.Name := sText;
    with Rect do begin
      inc(Left, FListMargin);
      Font.Height := (Bottom - Top - FListMargin);
    end;
    FillChar(dtto, SizeOf(TDTTOpts), 0);
    with dtto do begin
      dwSize := SizeOf(TDTTOpts);
      dwFlags := DTT_TEXTCOLOR;
      if ([odSelected, odFocused] * State) <> [] then begin
	Brush.Color := GetSysColor(COLOR_HIGHLIGHT);
	crText := GetSysColor(COLOR_HIGHLIGHTTEXT);
      end
      else begin
	Brush.Color := GetSysColor(COLOR_WINDOW);
	if not fNoFont then
	  crText := GetSysColor(COLOR_WINDOWTEXT)
	else crText := GetSysColor(COLOR_GRAYTEXT);
      end;
    end;
    if FCBXTheme <> 0 then begin
      if ([odSelected, odFocused] * State) <> [] then
	iState := CBXS_HOT
      else iState := CBXS_NORMAL;
      if MainWinVersion >= 8 then begin
	DrawThemeTextEx(FCBXTheme, Handle, CP_BACKGROUND, iState,
			PWideChar(AnsiToWideString(sText)), -1, DRAWSTYLE, @Rect, dtto);
      end
      else begin
	Font.Color := dtto.crText;
	if not fNoFont then begin
	  DrawThemeText(FCBXTheme, Handle, CP_BACKGROUND, iState,
			PWideChar(AnsiToWideString(sText)), -1, DRAWSTYLE, 0, Rect);
	end
	else DrawText(Handle, PChar(sText), -1, Rect, DRAWSTYLE);
      end;
    end
    else begin
      Font.Color := dtto.crText;
      DrawText(Handle, PChar(sText), -1, Rect, DRAWSTYLE);
    end;
  end;
  if (odSelected in State) and Assigned(FOnListItemSelected) then FOnListItemSelected(Self, Index);
end;

constructor TFontCBEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawVariable;
end;

destructor TFontCBEx.Destroy;
begin
  inherited Destroy;
end;


// TDriveCBEx ------------------------------------------------------------------

procedure TDriveCBEx.ListDrop;
begin
  FListDropped := true;
  if Assigned(FOnListDrop) then FOnListDrop(Self);
end;

procedure TDriveCBEx.ListClose;
begin
  FListDropped := false;
  if Assigned(FOnListClose) then FOnListClose(Self);
end;

procedure TDriveCBEx.CNCommand(var Message: TWMCommand);
begin
  with Message do begin
    case NotifyCode of
      CBN_DROPDOWN: ListDrop;
      CBN_CLOSEUP: ListClose;
    end;
  end;
  inherited;
end;


// TFilterCBEx -----------------------------------------------------------------

procedure TFilterCBEx.ListDrop;
begin
  FListDropped := true;
  if Assigned(FOnListDrop) then FOnListDrop(Self);
end;

procedure TFilterCBEx.ListClose;
begin
  FListDropped := false;
  if Assigned(FOnListClose) then FOnListClose(Self);
end;

procedure TFilterCBEx.CNCommand(var Message: TWMCommand);
begin
  with Message do begin
    case NotifyCode of
      CBN_DROPDOWN: ListDrop;
      CBN_CLOSEUP: ListClose;
    end;
  end;
  inherited;
end;


procedure Register;
begin
  RegisterComponents('Modified', [TPlainCBEx, TFontCBEx, TDriveCBEx, TFilterCBEx]);
end;


end.

