{$J+,Z4}
unit ColGrip;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, Graphics, Forms, Controls, ComCtrls, UxThemes;

type
  TColoredGrip = class(TStatusBar)
  private
    FTheme: THandle;
    procedure OpenTheme;
    procedure CloseTheme;
    procedure DrawSizeGrip;
    function  DrawThemedGrip(R: TRect): Longbool;
    function  GetAlign: TAlign;
    procedure SetAlign(Value: TAlign);
    function  GetSimplePanel: Longbool;
    procedure SetSimplePanel(Value: Longbool);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMEraseBkGnd(var Message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align: TAlign read GetAlign write SetAlign;
    property SimplePanel: Longbool read GetSimplePanel write SetSimplePanel;
  end;

procedure Register;


implementation

procedure TColoredGrip.OpenTheme;
begin
  if UseThemes and (Color = clBtnFace) then FTheme := OpenThemeData(Handle, 'STATUS');
end;

procedure TColoredGrip.CloseTheme;
begin
  if FTheme <> 0 then begin
    CloseThemeData(FTheme);
    FTheme := 0;
  end;
end;

procedure PolyDraw(Canvas: TCanvas; Points: Array of TPoint; Types: Array of Byte);
type
  PPoints = ^TPoints;
  TPoints = Array[0..0] of TPoint;
  PTypes = ^TTypes;
  TTypes = Array[0..0] of Byte;
begin
  Windows.PolyDraw(Canvas.Handle, PPoints(@Points)^, PTypes(@Types)^, succ(High(Points)));
end;

procedure TColoredGrip.DrawSizeGrip;
var
  rClient: TRect;
begin
  rClient := ClientRect;
  with Canvas, rClient do begin
    Pen.Width := 1;
    Font.Assign(Self.Font);
    Brush.Assign(Self.Brush);
    FillRect(ClientRect);
    if GetParentForm(Self).WindowState <> wsMaximized then begin
      SizeGrip := true;
      if not DrawThemedGrip(rClient) then begin
	Pen.Color := clBtnHighlight;
	PolyDraw(Canvas,
		 [
		   Point(Right - 2, Bottom), Point(Right, Bottom - 2),
		   Point(Right - 13, Bottom), Point(Right, Bottom - 13),
		   Point(Right - 9, Bottom), Point(Right, Bottom - 9),
		   Point(Right - 5, Bottom), Point(Right, Bottom - 5),
		   Point(Right - 1, Bottom), Point(Right, Bottom)
		 ],
		 [
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO
		 ]);
	Pen.Color := clBtnShadow;
	PolyDraw(Canvas,
		 [
		   Point(Right - 11, Bottom), Point(Right, Bottom - 11),
		   Point(Right - 7, Bottom), Point(Right, Bottom - 7),
		   Point(Right - 3, Bottom), Point(Right, Bottom - 3)
		 ],
		 [
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO,
		   PT_MOVETO, PT_LINETO
		 ]);
      end;
      dec(Right, Height);
    end
    else SizeGrip := false;
    if Text <> '' then DrawText(Canvas.Handle, Pointer(Text), -1, rClient, DT_LEFT or DT_SINGLELINE or DT_VCENTER);
  end;
end;

function TColoredGrip.DrawThemedGrip(R: TRect): Longbool;
begin
  Result := false;
  if FTheme <> 0 then begin
    with R do begin
      Left := Right - Bottom;
      Top := 0;
    end;
    if GetThemeBackgroundContentRect(FTheme, Canvas.Handle, SP_GRIPPER, 0, R, @R) = S_OK then begin
      if IsThemeBackgroundPartiallyTransparent(FTheme, SP_GRIPPER, 0) then begin
	DrawThemeParentBackground(WindowHandle, Canvas.Handle, @R);
      end;
      Result := (DrawThemeBackground(FTheme, Canvas.Handle, SP_GRIPPER, 0, R, nil) = S_OK);
    end;
  end;
end;

function TColoredGrip.GetAlign: TAlign;
begin
  Result := inherited Align;
end;

procedure TColoredGrip.SetAlign(Value: TAlign);
begin
  inherited Align := alBottom;
end;

function TColoredGrip.GetSimplePanel: Longbool;
begin
  Result := inherited SimplePanel;
end;

procedure TColoredGrip.SetSimplePanel(Value: Longbool);
begin
  inherited SimplePanel := true;
end;

procedure TColoredGrip.CMColorChanged(var Message: TMessage);
begin
  inherited;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

procedure TColoredGrip.WMCreate(var Message: TWMCreate);
begin
  inherited;
  if EnableThemes(WindowHandle, (Color = clBtnFace)) then OpenTheme;
end;

procedure TColoredGrip.WMDestroy(var Message: TWMDestroy);
begin
  CloseTheme;
  inherited;
end;

procedure TColoredGrip.WMEraseBkGnd(var Message: TWMEraseBkGnd);
begin
  Message.Result := -1;
end;

procedure TColoredGrip.WMPaint(var Message: TWMPaint);
var
  iDC: Integer;
  psDC: TPaintStruct;
begin
  iDC := SaveDC(Canvas.Handle);
  Message.DC := BeginPaint(WindowHandle, psDC);
  try
    DrawSizeGrip;
  finally
    EndPaint(WindowHandle, psDC);
    RestoreDC(Canvas.Handle, iDC);
  end;
end;

procedure TColoredGrip.WMThemeChanged(var Message: TMessage);
begin
  CloseTheme;
  inherited;
  OpenTheme;
  Invalidate;
end;

constructor TColoredGrip.Create(AOwner: TComponent);
begin
  inherited;
  ParentColor := true;
end;

destructor TColoredGrip.Destroy;
begin
  inherited;
end;

procedure Register;
begin
  RegisterComponents('Modified', [TColoredGrip]);
end;

end.

