{$J+,Z4}
unit CustBtnCtrl;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, UxThemes;

type
  TBtnPart = BUTTONPARTS;
  TBtnState = Cardinal;
  TBtnType = (btCheck, btRadio);
  TCustBtnCtrl = class(TCustomControl)
  private
    FAlignment: TLeftRight;
    FAllowGrayed: Longbool;
    FBtnPart: TBtnPart;
    FCaptionRect: TRect;
    FCheckBoxState: TCheckBoxState;
    FChecked: Boolean;
    FClicked: Boolean;
    FDrawRect: TRect;
    FFocused: Longbool;
    FFocusRect: TRect;
    FTheme: THandle;
    FWindowColor: TColor;
    FOnChanged: TNotifyEvent;
    procedure OpenTheme;
    procedure CloseTheme;
    procedure GetCaptionRect;
    function  GetDrawRect: TRect;
    procedure CheckKeyPreview(var Message: TMessage);
    procedure DrawFocusRect(Focused: Longbool);
    function  DrawThemedBtn: Longbool;
    procedure InvalidateCaptionBkGnd;
    procedure InvalidateDrawRect;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetBtnType(Value: TBtnType);
    procedure SetCheckBoxState(Value: TCheckBoxState);
    function  GetMinimumWidth: Integer;
    procedure SetWindowColor(Value: TColor);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CNChar(var Message: TWMChar); message CN_CHAR;
    procedure CNKeyDown(var Message: TWMKeyDown); message CN_KEYDOWN;
    procedure CNKeyUp(var Message: TWMKeyUp); message CN_KEYUP;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMEraseBkGnd(var Message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMSysKeyUp(var Message: TWMSysKeyUp); message WM_SYSKEYUP;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
  protected
    procedure SetChecked(Value: Boolean); virtual;
    function  GetState: TBtnState; virtual; abstract;
    procedure DrawDefaultBtn(Canvas: TCanvas); virtual; abstract;
    procedure Paint; override;
    procedure SetName(const Value: TComponentName); override;
    property  AllowGrayed: Longbool read FAllowGrayed write FAllowGrayed;
    property  BtnType: TBtnType write SetBtnType;
    property  BtnPart: TBtnPart write FBtnPart;
    property  Clicked: Boolean read FClicked;
    property  DrawRect: TRect read GetDrawRect;
    property  State: TCheckBoxState read FCheckBoxState write SetCheckBoxState;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure CreateParams(var Params: TCreateParams); override;
    property  MinimumWidth: Integer read GetMinimumWidth;
  published
    property Caption;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Alignment: TLeftRight read FAlignment write SetAlignment;
    property Checked: Boolean read FChecked write SetChecked;
    property WindowColor: TColor read FWindowColor write SetWindowColor;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  end;


implementation

const
  TEXT_BORDER = 4;

procedure TCustBtnCtrl.GetCaptionRect;
var
  iTextHeight, iBtnWidth: Integer;
begin
  if Caption <> '' then begin
    FCaptionRect := ClientRect;
    with DrawRect do iBtnWidth := Right - Left;
    case Alignment of
      taLeftJustify: dec(FCaptionRect.Right, iBtnWidth);
      taRightJustify: inc(FCaptionRect.Left, iBtnWidth);
    end;
    InflateRect(FCaptionRect, -TEXT_BORDER, -TEXT_BORDER);
    iTextHeight := Canvas.TextHeight('W');
    with FCaptionRect do begin
      Top := ((Bottom + Top) - iTextHeight) div 2;
      Bottom := Top + iTextHeight;
    end;
    FFocusRect := FCaptionRect;
    DrawText(Canvas.Handle, Pointer(Caption), -1, FFocusRect, DT_CALCRECT or DT_NOCLIP or DT_SINGLELINE);
    InflateRect(FFocusRect, 2, 2);
    OffsetRect(FCaptionRect, 0, -1);
  end;
end;

function TCustBtnCtrl.GetDrawRect: TRect;
var
  szBox: TSize;
  iBoxHeight, iBoxWidth, iBorderHeight: Integer;
begin
  if IsRectEmpty(FDrawRect) then begin
    if (FTheme <> 0)
    and (GetThemePartSize(FTheme, Canvas.Handle, FBtnPart, GetState, nil, TS_DRAW, szBox) = S_OK) then begin
     with szBox do begin
       iBoxHeight := cx;
       iBoxWidth := cy;
     end;
    end
    else begin
      iBoxHeight := 13;
      iBoxWidth := 13;
    end;
    iBorderHeight := (Height - iBoxHeight) div 2;
    case Alignment of
      taLeftJustify: Result := Rect(Width - iBoxWidth, iBorderHeight, Width, iBorderHeight + iBoxWidth);
      taRightJustify: Result := Rect(0, iBorderHeight, iBoxWidth, iBorderHeight + iBoxWidth);
    end;
    FDrawRect := Result;
  end
  else Result := FDrawRect;
end;

procedure TCustBtnCtrl.OpenTheme;
begin
  if UseThemes and (Color = clBtnFace) then FTheme := OpenThemeData(Handle, 'BUTTON');
  SetRectEmpty(FDrawRect);
  GetCaptionRect;
end;

procedure TCustBtnCtrl.CloseTheme;
begin
  if FTheme <> 0 then begin
    CloseThemeData(FTheme);
    FTheme := 0;
  end;
end;

procedure TCustBtnCtrl.CheckKeyPreview(var Message: TMessage);
begin
  if (Owner <> nil) and (Owner is TForm) then with TForm(Owner) do begin
    if KeyPreview then with Message do Result := Perform(Msg, wParam, lParam);
  end;
end;

procedure TCustBtnCtrl.DrawFocusRect(Focused: Longbool);
begin
  with Canvas do begin
    if Focused <> FFocused then begin
      SetTextColor(Handle, clBlack);
      DrawFocusRect(FFocusRect);
      FFocused := Focused;
    end;
  end;
end;

function TCustBtnCtrl.DrawThemedBtn: Longbool;
var
  iState: Integer;
begin
  Result := (FTheme <> 0);
  if Result then begin
    iState := GetState;
    DrawThemeParentBackground(WindowHandle, Canvas.Handle, nil);
    DrawThemeBackground(FTheme, Canvas.Handle, FBtnPart, iState, FDrawRect, nil);
  end;
end;

procedure TCustBtnCtrl.InvalidateCaptionBkGnd;
var
  rBkGnd: TRect;
begin
  if not Showing then Exit;
  SubtractRect(rBkGnd, ClientRect, GetDrawRect);
  InvalidateRect(WindowHandle, @rBkGnd, false);
end;

procedure TCustBtnCtrl.InvalidateDrawRect;
begin
  if Showing then InvalidateRect(WindowHandle, @FDrawRect, false);
end;

procedure TCustBtnCtrl.SetAlignment(Value: TLeftRight);
begin
  if Value <> FAlignment then begin
    FAlignment := Value;
    SetRectEmpty(FDrawRect);
    GetCaptionRect;
    Invalidate;
  end;
end;

procedure TCustBtnCtrl.SetBtnType(Value: TBtnType);
begin
  case Value of
    btCheck: FBtnPart := BP_CHECKBOX;
    btRadio: FBtnPart := BP_RADIOBUTTON;
  end;
end;

procedure TCustBtnCtrl.SetCheckBoxState(Value: TCheckBoxState);
begin
  if FAllowGrayed then SetChecked(Boolean(Value));
end;

function TCustBtnCtrl.GetMinimumWidth: Integer;
begin
  with FDrawRect do Result := Right - Left;
  if Caption <> '' then with FFocusRect do inc(Result, Right - Left + (TEXT_BORDER shl 1));
end;

procedure TCustBtnCtrl.SetWindowColor(Value: TColor);
begin
  if Value <> FWindowColor then begin
    FWindowColor := Value;
    Invalidate;
  end;
end;

procedure TCustBtnCtrl.CMColorChanged(var Message: TMessage);
begin
  inherited;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

procedure TCustBtnCtrl.CMCtl3DChanged(var Message: TMessage);
begin
  Invalidate;
  inherited;
end;

procedure TCustBtnCtrl.CMDialogChar(var Message: TCMDialogChar);
begin
  if IsAccel(Message.CharCode, Caption) and CanFocus then begin
    Message.Result := 1;
    FClicked := true;
    Click;
  end
  else inherited;
end;

procedure TCustBtnCtrl.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TCustBtnCtrl.CMFontChanged(var Message: TMessage);
begin
  Canvas.Font.Assign(Font);
  GetCaptionRect;
  inherited;
end;

procedure TCustBtnCtrl.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  InvalidateDrawRect;
end;

procedure TCustBtnCtrl.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  InvalidateDrawRect;
end;

procedure TCustBtnCtrl.CMParentColorChanged(var Message: TMessage);
begin
  Canvas.Brush.Assign(Parent.Brush);
  inherited;
end;

procedure TCustBtnCtrl.CMTextChanged(var Message: TMessage);
begin
  if Parent <> NIL then begin
    InvalidateCaptionBkGnd;
    GetCaptionRect;
  end;
  inherited;
end;

procedure TCustBtnCtrl.CNChar(var Message: TWMChar);
begin
  CheckKeyPreview(TMessage(Message));
  if Message.CharCode = VK_SPACE then Click;
end;

procedure TCustBtnCtrl.CNKeyDown(var Message: TWMKeyDown);
begin
  CheckKeyPreview(TMessage(Message));
  if Message.CharCode = VK_SPACE then begin
    FClicked := true;
    InvalidateDrawRect;
  end;
end;

procedure TCustBtnCtrl.CNKeyUp(var Message: TWMKeyUp);
begin
  CheckKeyPreview(TMessage(Message));
  if Message.CharCode = VK_SPACE then begin
    FClicked := false;
    InvalidateDrawRect;
  end;
end;

procedure TCustBtnCtrl.WMCreate(var Message: TWMCreate);
begin
  inherited;
  if EnableThemes(WindowHandle, (Color = clBtnFace)) then OpenTheme;
end;

procedure TCustBtnCtrl.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  DrawFocusRect(true);
end;

procedure TCustBtnCtrl.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  DrawFocusRect(false);
end;

procedure TCustBtnCtrl.WMEraseBkGnd(var Message: TWMEraseBkGnd);
begin
  if FTheme = 0 then
    inherited
  else Message.Result := -1;
end;

procedure TCustBtnCtrl.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  SetFocus;
  FClicked := true;
  InvalidateDrawRect;
end;

procedure TCustBtnCtrl.WMLButtonUp(var Message: TWMLButtonUp);
begin
  inherited;
  if GetCapture = WindowHandle then Click;
  FClicked := false;
  InvalidateDrawRect;
end;

procedure TCustBtnCtrl.WMSize(var Message: TWMSize);
begin
  SetRectEmpty(FDrawRect);
  GetCaptionRect;
  Invalidate;
  inherited;
end;

procedure TCustBtnCtrl.WMSysKeyUp(var Message: TWMSysKeyUp);
begin
  inherited;
  if IsAccel(Message.CharCode, AnsiLowerCase(Caption)) then begin
    FClicked := false;
    InvalidateDrawRect;
  end;
end;

procedure TCustBtnCtrl.WMThemeChanged(var Message: TMessage);
begin
  CloseTheme;
  inherited;
  OpenTheme;
  Invalidate;
end;

procedure TCustBtnCtrl.SetChecked(Value: Boolean);
begin
  if TCheckBoxState(Value) = FCheckBoxState then Exit;
  FCheckBoxState := TCheckBoxState(Value);
  FChecked := (FCheckBoxState = cbChecked);
  InvalidateDrawRect;
  if Assigned(FOnChanged) then begin
    if FClicked then
      FOnChanged(Self)
    else FOnChanged(nil);
  end;
end;

procedure TCustBtnCtrl.Paint;
begin
  DrawFocusRect(false);
  try
    if not DrawThemedBtn then with Canvas do begin
      Brush.Color := Color;
      FillRect(ClientRect);
      Pen.Color := FWindowColor;
      if Enabled then begin
	if Color = clBtnFace then
	  Brush.Color := clWindow
	else Brush.Color := FWindowColor;
      end
      else Brush.Color := Color;
      DrawDefaultBtn(Canvas);
    end;
    if Caption <> '' then with Canvas do begin
      Font.Assign(Self.Font);
      Brush.Color := Color;
      SelectObject(Handle, Font.Handle);
      SetBkMode(Handle, Windows.TRANSPARENT);
      if not Enabled then
	SetTextColor(Handle, GetSysColor(COLOR_GRAYTEXT))
      else SetTextColor(Handle, ColorToRGB(Font.Color));
      DrawText(Handle, PChar(Caption), -1, FCaptionRect, DT_LEFT or DT_SINGLELINE);
    end;
  finally
    DrawFocusRect(Focused);
  end;
end;

procedure TCustBtnCtrl.SetName(const Value: TComponentName);
var
  fChangeText: Longbool;
begin
  fChangeText := (csSetCaption in ControlStyle) and (Name = Caption) and
		 ((Owner = nil) or not (Owner is TControl) or not (csLoading in TControl(Owner).ComponentState));
  inherited SetName(Value);
  if fChangeText then Caption := Value;
end;

constructor TCustBtnCtrl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csDoubleClicks, csOpaque];
  Alignment := taLeftJustify;
  ParentColor := true;
  TabStop := true;
  Height := 17;
  Width := 97;
  FWindowColor := clWindow;
end;

destructor TCustBtnCtrl.Destroy;
begin
  CloseTheme;
  inherited Destroy;
end;

procedure TCustBtnCtrl.Click;
begin
  SetFocus;
  SetChecked(not FChecked);
  inherited;
end;

procedure TCustBtnCtrl.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do ExStyle := ExStyle or WS_EX_TRANSPARENT;
end;

end.

