{$J+,Z4}
unit CustCheck;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Graphics, Forms, Classes, Controls, ExtCtrls, StdCtrls, CustBtnCtrl, UxThemes;

type
  TCustCheckBox = class(TCustBtnCtrl)
  private
  protected
    function  GetState: TBtnState; override;
    procedure DrawDefaultBtn(Canvas: TCanvas); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property AllowGrayed;
    property State;
  end;

procedure Register;


implementation

function TCustCheckBox.GetState: TBtnState;
const
  CheckedStates: Array[TCheckBoxState] of TBtnState = (
    CBS_UNCHECKEDPRESSED,
    CBS_CHECKEDPRESSED,
    CBS_MIXEDPRESSED
  );
  HotStates: Array[TCheckBoxState] of TBtnState = (
    CBS_UNCHECKEDHOT,
    CBS_CHECKEDHOT,
    CBS_MIXEDHOT
  );
  NormalStates: Array[TCheckBoxState] of TBtnState = (
    CBS_UNCHECKEDNORMAL,
    CBS_CHECKEDNORMAL,
    CBS_MIXEDNORMAL
  );
  DisabledStates: Array[TCheckBoxState] of TBtnState = (
    CBS_UNCHECKEDDISABLED,
    CBS_CHECKEDDISABLED,
    CBS_MIXEDDISABLED
  );
var
  CurPos: TPoint;
begin
  if Enabled then begin
    if Clicked then
      Result := CheckedStates[State]
    else begin
      GetCursorPos(CurPos);
      if not (csDesigning in ComponentState) and PtInRect(ClientRect, ScreenToClient(CurPos)) then
	Result := HotStates[State]
      else Result := NormalStates[State];
    end;
  end
  else Result := DisabledStates[State];
end;

procedure TCustCheckBox.DrawDefaultBtn(Canvas: TCanvas);
var
  dr: TRect;
begin
  dr := DrawRect;
  with Canvas, dr do begin
    if Ctl3D then begin
      Frame3D(Canvas, dr, clBtnShadow, GetSysColor(COLOR_3DHILIGHT), 1);
      Frame3D(Canvas, dr, GetSysColor(COLOR_3DDKSHADOW), clBtnFace, 1);
    end
    else begin
      Pen.Color := Font.Color;
      Rectangle(Left, Top, Right, Bottom);
      InflateRect(dr, -2, -2);
    end;
    FillRect(dr);
    Pen.Color := clGrayText;
    if State = cbUnchecked then Exit;
    if Enabled and (State = cbChecked) then Pen.Color := Font.Color;
    Canvas.PolyLine([Point(Left + 1, Top + 3), Point(Left + 3, Top + 5), Point(Left + 7, Top + 1),
		     Point(Left + 7, Top + 2), Point(Left + 3, Top + 6), Point(Left + 1, Top + 4),
		     Point(Left + 1, Top + 5), Point(Left + 3, Top + 7), Point(Left + 8, Top + 2)]);
  end;
end;

constructor TCustCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BtnType := btCheck;
end;

destructor TCustCheckBox.Destroy;
begin
  inherited Destroy;
end;

procedure TCustCheckBox.Click;
begin
  if AllowGrayed then begin
    SetFocus;
    case State of
      cbUnchecked: SetChecked(Boolean(cbGrayed));
      cbChecked: SetChecked(Boolean(cbUnchecked));
      cbGrayed: SetChecked(Boolean(cbChecked));
    end;
  end
  else begin
    inherited Click;
    Exit;
  end;
  if Assigned(OnClick) then OnClick(Self);
end;

procedure Register;
begin
  RegisterComponents('Modified', [TCustCheckBox]);
end;

end.

