{$J+,Z4}
unit CustRadio;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Graphics, Forms, Classes, Controls, ExtCtrls, StdCtrls, CustBtnCtrl, UxThemes;

type
  TCustRadioBtn = class(TCustBtnCtrl)
  private
  protected
    procedure SetChecked(Value: Boolean); override;
    function  GetState: TBtnState; override;
    procedure DrawDefaultBtn(Canvas: TCanvas); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
  end;

procedure Register;


implementation

procedure TCustRadioBtn.SetChecked(Value: Boolean);

  procedure TurnSiblingsOff;
  var
    Index: Integer;
    Sibling: TControl;
  begin
    if Parent <> nil then with Parent do begin
      for Index := 0 to pred(ControlCount) do begin
	Sibling := Controls[Index];
	if (Sibling <> Self) and (Sibling is TCustRadioBtn) then begin
	  TCustRadioBtn(Sibling).SetChecked(false);
	end;
      end;
    end;
  end;

begin
  if Value <> inherited Checked then begin
    if Value then TurnSiblingsOff;
    inherited;
  end;
end;

function TCustRadioBtn.GetState: TBtnState;
const
  CheckedStates: Array[Boolean] of TBtnState = (
    RBS_UNCHECKEDPRESSED,
    RBS_CHECKEDPRESSED
  );
  HotStates: Array[Boolean] of TBtnState = (
    RBS_UNCHECKEDHOT,
    RBS_CHECKEDHOT
  );
  NormalStates: Array[Boolean] of TBtnState = (
    RBS_UNCHECKEDNORMAL,
    RBS_CHECKEDNORMAL
  );
  DisabledStates: Array[Boolean] of TBtnState = (
    RBS_UNCHECKEDDISABLED,
    RBS_CHECKEDDISABLED
  );
var
  CurPos: TPoint;
begin
  if Enabled then begin
    if Clicked then
      Result := CheckedStates[Checked]
    else begin
      GetCursorPos(CurPos);
      if not (csDesigning in ComponentState) and PtInRect(ClientRect, ScreenToClient(CurPos)) then
	Result := HotStates[Checked]
      else Result := NormalStates[Checked]
    end;
  end
  else Result := DisabledStates[Checked];
end;

procedure TCustRadioBtn.DrawDefaultBtn(Canvas: TCanvas);
var
  dr: TRect;
begin
  dr := DrawRect;
  with Canvas, dr do begin
    Ellipse(Left, Top, Right - 2, Bottom - 2);
    Pen.Color := Font.Color;
    if Ctl3D then Pen.Color := clBtnShadow;
    PolyLine([Point(Left + 1, Top + 8), Point(Left + 1, Top + 7),
	      Point(Left, Top + 6), Point(Left, Top + 4),
	      Point(Left + 1, Top + 3), Point(Left + 1, Top + 2),
	      Point(Left + 2, Top + 1), Point(Left + 3, Top + 1),
	      Point(Left + 4, Top), Point(Left + 6, Top),
	      Point(Left + 7, Top + 1), Point(Left + 10, Top + 2)]);
    if Ctl3D then begin
      Pen.Color := GetSysColor(COLOR_3DHILIGHT);
      PolyLine([Point(Left + 10, Top + 2), Point(Left + 10, Top + 3),
		Point(Left + 11, Top + 4), Point(Left + 11, Top + 7),
		Point(Left + 10, Top + 8), Point(Left + 10, Top + 9),
		Point(Left + 9, Top + 10), Point(Left + 8, Top + 10),
		Point(Left + 7, Top + 11), Point(Left + 4, Top + 11),
		Point(Left + 3, Top + 10), Point(Left + 1, Top + 10)]);
      Pen.Color := GetSysColor(COLOR_3DDKSHADOW);
      PolyLine([Point(Left + 2, Top + 8), Point(Left + 1, Top + 7),
		Point(Left + 1, Top + 7), Point(Left + 1, Top + 4),
		Point(Left + 1, Top + 4), Point(Left + 2, Top + 3),
		Point(Left + 2, Top + 3), Point(Left + 2, Top + 2),
		Point(Left + 2, Top + 2), Point(Left + 3, Top + 2),
		Point(Left + 3, Top + 2), Point(Left + 4, Top + 1),
		Point(Left + 4, Top + 1), Point(Left + 7, Top + 1),
		Point(Left + 7, Top + 1), Point(Left + 8, Top + 2),
		Point(Left + 8, Top + 2), Point(Left + 10, Top + 2)]);
    end;
    if Ctl3D then Pen.Color := clBtnFace;
    PolyLine([Point(Left + 9, Top + 3), Point(Left + 10, Top + 4),
	      Point(Left + 10, Top + 4), Point(Left + 10, Top + 6),
	      Point(Left + 10, Top + 6), Point(Left + 9, Top + 8),
	      Point(Left + 8, Top + 9), Point(Left + 6, Top + 10),
	      Point(Left + 6, Top + 10), Point(Left + 4, Top + 10),
	      Point(Left + 4, Top + 10), Point(Left + 3, Top + 9),
	      Point(Left + 3, Top + 9), Point(Left + 1, Top + 9)]);
    if Enabled then
      Pen.Color := Font.Color
    else Pen.Color := clGrayText;
    if Checked then begin
      Brush.Color := Pen.Color;
      Rectangle(Left + 3 + ord(Ctl3D), Top + 4 + ord(Ctl3D), Left + 8, Top + 7);
      Rectangle(Left + 4 + ord(Ctl3D), Top + 3 + ord(Ctl3D), Left + 7, Top + 8);
    end;
  end;
end;

constructor TCustRadioBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BtnType := btRadio;
end;

destructor TCustRadioBtn.Destroy;
begin
  inherited Destroy;
end;

procedure TCustRadioBtn.Click;

  function HasSiblings: Longbool;
  var
    Index: Integer;
    Sibling: TControl;
  begin
    Result := false;
    if Parent <> nil then with Parent do begin
      for Index := 0 to pred(ControlCount) do begin
	Sibling := Controls[Index];
	if (Sibling <> Self) and (Sibling is TCustRadioBtn) then begin
	  Result := true;
	  Break;
	end;
      end;
    end;
  end;

begin
  if not (Checked and HasSiblings) then begin
    inherited Click;
    Exit;
  end
  else SetFocus;
  if Assigned(OnClick) then OnClick(Self);
end;

procedure Register;
begin
  RegisterComponents('Modified', [TCustRadioBtn]);
end;

end.

