{$J+,Z4}
unit DropFiles;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, ShellApi, Messages, Classes, Controls, Graphics, Forms;

type
  TDropFiles = class(TObject)
  private
    FAcceptFiles: Longbool;
    FFileList: TStringList;
    FNewWndProc: Pointer;
    FOldWndProc: Pointer;
    FParent: TWinControl;
    FOnFileDropped: TNotifyEvent;
    function  FilesDropped(Drop: THandle): Longint;
    procedure NewWndProc(var Message: TMessage);
    procedure ResetWndProc;
    procedure SetAcceptFiles(Value: Longbool);
  protected
  public
    constructor Create(Parent: TWinControl);
    destructor Destroy; override;
    property  FileList: TStringList read FFileList;
  published
    property  AcceptFiles: Longbool read FAcceptFiles write SetAcceptFiles;
    property  OnFileDropped: TNotifyEvent read FOnFileDropped write FOnFileDropped;
  end;


implementation

function TDropFiles.FilesDropped(Drop: THandle): Longint;
var
  iIndex: Integer;
  sFileName: String;
begin
  Result := 0;
  try
    for iIndex := pred(DragQueryFile(Drop, $FFFFFFFF, nil, 0)) downto 0 do begin
      SetString(sFileName, nil, DragQueryFile(Drop, iIndex, nil, 0));
      DragQueryFile(Drop, iIndex, Pointer(sFileName), succ(Length(sFileName)));
      FFileList.Add(sFileName);
    end;
  finally
    DragFinish(Drop);
  end;
  if (FFileList.Count <> 0) and Assigned(FOnFileDropped) then FOnFileDropped(Self);
end;

procedure TDropFiles.NewWndProc(var Message: TMessage);
begin
  with Message do begin
    case Msg of
      WM_DESTROY: ResetWndProc;
      WM_DROPFILES: begin
	Result := FilesDropped(THandle(wParam));
	Exit;
      end;
    end;
    Result := CallWindowProc(FOldWndProc, FParent.Handle, Msg, wParam, lParam);
  end;
end;

procedure TDropFiles.ResetWndProc;
begin
  if (FParent <> nil) and FParent.HandleAllocated and (FOldWndProc <> nil) then begin
    SetWindowLong(FParent.Handle, GWL_WNDPROC, Longint(FOldWndProc));
  end;
end;

procedure TDropFiles.SetAcceptFiles(Value: Longbool);
begin
  if (FParent <> nil) and FParent.HandleAllocated then begin
    if FOldWndProc = nil then FOldWndProc := ptr(SetWindowLong(FParent.Handle, GWL_WNDPROC, Longint(FNewWndProc)));
    DragAcceptFiles(FParent.Handle, Value);
    FAcceptFiles := Value;
  end;
end;

constructor TDropFiles.Create(Parent: TWinControl);
begin
  inherited Create;
  FParent := Parent;
  FFileList := TStringList.Create;
  {$IFDEF CONDITIONALEXPRESSIONS}
  FNewWndProc := Classes.MakeObjectInstance(NewWndProc);
  {$ELSE}
  FNewWndProc := MakeObjectInstance(NewWndProc);
  {$ENDIF}
end;

destructor TDropFiles.Destroy;
begin
  ResetWndProc;
  {$IFDEF CONDITIONALEXPRESSIONS}
  Classes.FreeObjectInstance(FNewWndProc);
  {$ELSE}
  FreeObjectInstance(FNewWndProc);
  {$ENDIF}
  FFileList.Free;
  FParent := nil;
  inherited Destroy;
end;

end.

