{$J+,Z4}
unit FileUtils;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, Controls, Graphics, Forms, ShellApi, ShlObj, Ole2, Menus,
  SysUtils, StrUtil, IniFiles, FileDialogs, GdiPlus, Processes, WindowUtils
  {$IFDEF PMPGP}, PMPGPCommon{$ENDIF}
  {$IFDEF PMSM}, PMSMCommon{$ENDIF};

const
  MIN_CONTEXT_MENU_ID	= 256;

const
  CF_RTF		= CF_PRIVATEFIRST + 1;
  CF_HTML		= CF_PRIVATEFIRST + 2;
  CLIP_NAME		= FileDialogs.CLIP_NAME;

type
  TFileStream = class(Classes.TFileStream)
  public
    procedure Truncate;
  end;

type
  TKeyringFileType = (kftSecring, kftPubring, kftGroups);
  TOnExecCmd = Procedure(const FileName: String; var Cancel: Longbool) of Object;

function  TempFileName(const Ext: String): String;
function  DelTempFiles(const Path: String): Integer;
function  UniqueFile(const FileName: String): String;
function  FileTypeInfo(const FileName, MimeType: String): String;
function  GetDefaultIcon(const FileName, ImageData, MimeType: String): THandle;
function  FileTypeIcon(const FileName, FileData: String; OpenIcon, Selected: Longbool): THandle;
function  ExtractIcon(const FileName: String; IconID, MaxBPP: Integer; Small: Longbool): THandle;
function  LegalFileName(const FileName: String): String;
function  GetUrlFileName(const UrlPath: String): String;
function  GetShortPath(const Path: String): String;
function  GetLongPath(const Path: String): String;
function  FileVersionInfo(const FileName: String): String;
function  FindFirst(const FileMask: String; var fdFound: TWin32FindData): THandle;
function  FindNext(hSearch: THandle; var fdFound: TWin32FindData): Longbool;
function  FindClose(hSearch: THandle): Longbool;
function  FindFile(const FileMask: String): String;
function  DeleteFiles(const FileMask: String): Longbool;
function  DeleteLinkFolder(const PathName: String): Longbool;
function  CopyFiles(const FileMask, NewPath: String; Overwrite, Move: Longbool): Longbool;
function  ReadClipboardText(var Text: String; Unicode: Longbool; ParentWindow: THandle): Longbool;
function  WriteClipboardData(const Data: Variant; Format: Integer; ParentWindow: THandle): Longbool;
function  ReadTextFromFile(const FileName: String; var Text: String): Longbool;
function  WriteTextToFile(const FileName: String; const Text: String): Longbool;
function  ReadIniFileSection(const FileName, Section: String): TStringList;
function  WriteIniFileSection(const FileName, Section: String; Items: TStringList): Longbool;
function  ReadIniFileSectionString(const FileName, Section: String): String;
function  WriteIniFileSectionString(const FileName, Section, Items: String): Longbool;
function  ReadIniFileString(const FileName, Section, Item: String): String;
function  WriteIniFileString(const FileName, Section, Item, Value: String): Longbool;
function  ReadIniFileInteger(const FileName, Section, Item: String): Longint;
function  WriteIniFileInteger(const FileName, Section, Item: String; Value: Longint): Longbool;
function  ReadIniFileBoolean(const FileName, Section, Item: String): Longbool;
function  WriteIniFileBoolean(const FileName, Section, Item: String; Value: Longbool): Longbool;
function  CreateLinkFile(const Link, Target, Arguments, Icon: String): Longbool;
function  GetTargetFileName(const LinkFile: String): String;
function  FindExecutable(const FileName: String): String;
function  FileExecute(Parent: THandle; const Command, FileName, Parameters: String; Show: Longbool): Longbool;
function  FindDirectory(OwnerWindow: THandle; const SearchPath, SearchFile, Prompt: String): String;
function  KeyFileOpen(const Caption, ClipboardName: String; OwnerWindow: THandle): String;
function  KeyFileSave(const Caption, ClipboardName, UserName: String; OwnerWindow: THandle): String;
function  CertFileOpen(const Caption, ClipboardName: String; OwnerWindow: THandle): String;
function  CertFileSave(const Caption, ClipboardName, UserName: String; OwnerWindow: THandle): String;
function  ViewerFileSave(const ViewerFile, InitialPath, Caption: String; OwnerWindow: THandle): String;
function  ImageFileSave(const ImageFile, Caption, Filters: String; OwnerWindow: THandle): String;
function  OpenKeyringFile(const Caption, PGPPath, KeyFile: String; KFT: TKeyringFileType; OwnerWindow: THandle): String;
function  ReloadModule(const FileName: String): THandle;
function  GetLocalizedPath(const Path: String): String;
function  GetSystemFolderPathByID(FolderID: Integer; Localize: Longbool): String;
function  GetSystemFolderPathByPID(FolderID: PItemIDList; Localize: Longbool): String;
procedure CreateFileContextMenu(const FileName, SubMenuName: String; ParentWindow, MainMenu: THandle;
				SubMenuPos, MinMenuID: Cardinal; PTrackPos: PPoint;
				PopupAlignment: TPopupAlignment;
				OnExecMenuCmd: TOnExecCmd);


function GetLongPathNameA(lpszShortPath: PAnsiChar; lpszLongPath: PAnsiChar;
  cchBuffer: DWORD): DWORD; stdcall;
function GetLongPathNameW(lpszShortPath: PWideChar; lpszLongPath: PWideChar;
  cchBuffer: DWORD): DWORD; stdcall;
function GetLongPathName(lpszShortPath: PChar; lpszLongPath: PChar;
  cchBuffer: DWORD): DWORD; stdcall;

function GetShortPathNameA(lpszLongPath: PAnsiChar; lpszShortPath: PAnsiChar;
  cchBuffer: DWORD): DWORD; stdcall;
function GetShortPathNameW(lpszLongPath: PWideChar; lpszShortPath: PWideChar;
  cchBuffer: DWORD): DWORD; stdcall;
function GetShortPathName(lpszLongPath: PChar; lpszShortPath: PChar;
  cchBuffer: DWORD): DWORD; stdcall;


implementation

{$IFNDEF VER90}
uses
  ComObj
{$IFDEF PMPGP}, TerHandler{$ENDIF}
{$IFDEF PMSM}, TerHandler{$ENDIF}
  ;

type
  TGUID = System.TGUID;
  TIID = TGUID;
{$ELSE}
{$IFDEF PMPGP}uses TerHandler;{$ENDIF}
{$IFDEF PMSM}uses TerHandler;{$ENDIF}
{$ENDIF}


// TFileStream -------------------------------------------------------------------------------------

procedure TFileStream.Truncate;
begin
  SetEndOfFile(Handle);
end;


// Other Routines ----------------------------------------------------------------------------------

const
  TMP_NAME_BEG = US;
  TMP_NAME_END = US + DOT;

function TempFileName(const Ext: String): String;
const
  DIGITS = 6;
var
  pResult: PChar;
  iNull, iPos, iLen: Integer;
  liCounter: TLargeInteger;
begin
  SetString(Result, nil, MAX_PATH + Length(Ext));
  iNull := succ(GetTempPath(Length(Result), Pointer(Result)));
  repeat
    QueryPerformanceCounter(liCounter);
    pResult := Pointer(Result);
    inc(pResult, pred(iNull));
    pResult^ := TMP_NAME_BEG;
    inc(pResult);
    {$IFDEF CONDITIONALEXPRESSIONS}
    Move(Pointer(HashedString(IntToHex(liCounter, DIGITS), DIGITS))^, pResult^, DIGITS);
    {$ELSE}
    Move(Pointer(HashedString(IntToHex(liCounter.LowPart, DIGITS), DIGITS))^, pResult^, DIGITS);
    {$ENDIF}
    inc(pResult, DIGITS);
    PDChar(pResult)^ := TMP_NAME_END;
    inc(pResult, 2);
    if Ext = E then begin
      PQChar(pResult)^ := 'tmp';
      inc(pResult, 3);
    end
    else begin
      iPos := 1;
      if Ext[1] = DOT then inc(iPos);
      iLen := Length(Ext) - pred(iPos);
      Move(Ext[iPos], pResult^, iLen);
      inc(pResult, iLen);
    end;
    pResult^ := NULL;
    if not FileExists(Result) then Break;
  until Result = E;
  if pResult <> nil then Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
end;

function DelTempFiles(const Path: String): Integer;
const
  SEARCH_PATTERN = TMP_NAME_BEG + AST + TMP_NAME_END + AST;
var
  sTemp: String;
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := 0;
  if Path <> E then begin
    if Path[Length(Path)] <> BS then
      sTemp := Path + BS
    else sTemp := Path;
  end
  else sTemp := GetTempDir;
  hSearch := FindFirstFile(Pointer(sTemp + SEARCH_PATTERN), fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    Screen.Cursor := crHourglass;
    try
      repeat
	DeleteFile(sTemp + fdFound.cFileName);
      until not FindNextFile(hSearch, fdFound);
    finally
      Windows.FindClose(hSearch);
      Screen.Cursor := crDefault;
    end;
  end;
end;

function UniqueFile(const FileName: String): String;
var
  iIndex: Integer;
  sFileExt, sPathName: String;
begin
  iIndex := 0;
  Result := FileName;
  sFileExt := ExtractFileExt(FileName);
  sPathName := Copy(FileName, 1, Length(FileName) - Length(sFileExt));
  while FileExists(Result) do begin
    inc(iIndex);
    Result := Format(sPathName + SP + '(%d)' + sFileExt, [iIndex]);
  end;
end;

function FileTypeInfo(const FileName, MimeType: String): String;
var
  sFileExt: String;
  fiFound: TSHFileInfo;
  iPos: Integer;
begin
  sFileExt := ExtractFileExt(FileName);
  if (sFileExt <> E) and (SHGetFileInfo(PChar(sFileExt), FILE_ATTRIBUTE_NORMAL, fiFound, SizeOf(TSHFileInfo),
					SHGFI_TYPENAME or SHGFI_USEFILEATTRIBUTES) <> 0) then begin
    if RegEntry(HKEY_CLASSES_ROOT, PChar(sFileExt)) then begin
      Result := fiFound.szTypeName;
      Exit;
    end
    else if MimeType <> E then begin
      iPos := FirstCharPos(SL, MimeType);
      if (iPos > 0) and (iPos < Length(MimeType)) then begin
	sFileExt := Copy(MimeType, iPos, MAXINT);
	if sFileExt <> E then begin
	  PChar(Pointer(sFileExt))[0] := DOT;
	  Result := FileTypeInfo(sFileExt, E);
	end;
      end;
    end;
  end;
  Result := E;
end;

function GetDefaultIcon(const FileName, ImageData, MimeType: String): THandle;
const
  DEF_ICON_KEY = '\DefaultIcon';
var
  sFileExt, sIconFile, sRegKey: String;
  iPos, iIndex: Integer;
begin
  Result := 0;
  if MimeType <> E then begin
   sIconFile := RegString(HKEY_CLASSES_ROOT, PChar(MimeType + DEF_ICON_KEY), E);
  end
  else begin
    sFileExt := ExtractFileExt(FileName);
    sIconFile := RegString(HKEY_CLASSES_ROOT, PChar(sFileExt + DEF_ICON_KEY), E);
    if sIconFile = E then begin
      sRegKey := RegString(HKEY_CLASSES_ROOT, sFileExt, E);
      if sRegKey <> E then sIconFile := RegString(HKEY_CLASSES_ROOT, PChar(sRegKey + DEF_ICON_KEY), E);
    end;
  end;
  if sIconFile = E then Exit;
  if (ImageData <> E) and (DelQuotes(sIconFile) = '%1') then begin
    Result := GetIconFromImageBuffer(ImageData);
    if Result <> 0 then Exit;
  end;
  iPos := LastCharPos(CM, sIconFile);
  iIndex := StrToIntDef(Copy(sIconFile, succ(iPos), MAXINT), 0);
  if iIndex < 0 then
    iIndex := -iIndex
   else inc(iIndex);
  Delete(sIconFile, iPos, MAXINT);
  if iIndex <> 0 then Result := ExtractIcon(sIconFile, iIndex, 32, true);
end;

function FileTypeIcon(const FileName, FileData: String; OpenIcon, Selected: Longbool): THandle;
const
  SMALL_ICONS_DEFAULT = SHGFI_ICON or SHGFI_SMALLICON or SHGFI_USEFILEATTRIBUTES;
var
  fiFound: TSHFileInfo;
  dwAttributes: DWORD;
  uFlags: UINT;
begin
  Result := 0;
  fiFound.hIcon := 0;
  if FileName <> FOLDER then
    dwAttributes := FILE_ATTRIBUTE_NORMAL
  else dwAttributes := FILE_ATTRIBUTE_DIRECTORY;
  uFlags := SMALL_ICONS_DEFAULT or (ord(OpenIcon = true) * SHGFI_OPENICON) or (ord(Selected = true) * SHGFI_SELECTED);
  if (SHGetFileInfo(PChar(FileName), dwAttributes, fiFound, SizeOf(TSHFileInfo), uFlags) = 0)
  or (fiFound.iIcon <= 3) then begin
    try
      if not Selected then Result := GetDefaultIcon(FileName, FileData, E);
    finally
      if Result <> 0 then
	DestroyIcon(fiFound.hIcon)
      else if not Selected then Result := fiFound.hIcon;
    end;
  end
  else Result := fiFound.hIcon;
end;

function ExtractIcon(const FileName: String; IconID, MaxBPP: Integer; Small: Longbool): THandle;
type
  PMemIconDirEntry = ^TMemIconDirEntry;
  TMemIconDirEntry = packed record
    bWidth: Byte;					// width of the image
    bHeight: Byte;					// height of the image (times 2)
    bColorCount: Byte;					// number of colors in image (0 if >= 8bpp)
    bReserved: Byte;					// reserved
    wPlanes: Word;					// color planes
    wBitCount: Word;					// bits per pixel
    dwBytesInRes: DWord;				// size in bytes
    nID: Word;						// resource ID
  end;
  PMemIconDir = ^TMemIconDir;
  TMemIconDir = packed record
    idReserved: Word;					// reserved
    idType: Word;					// resource type (1 for icons)
    idCount: Word;					// number of images
    idEntries: Array[0..0] of TMemIconDirEntry;		// the entries for each image
  end;
var
  hLib, hIcon: THandle;
  pmidGroup: PMemIconDir;
  iWidth, iHeight, iFound, iIndex: Integer;
begin
  Result := 0;
  hLib := LoadLibraryEx(PChar(ExpandEnvironmentStrings(FileName)), 0, LOAD_LIBRARY_AS_DATAFILE);
  if hLib <> 0 then begin
    try
      repeat
	pmidGroup := LockResource(LoadResource(hLib, FindResource(hLib, MakeIntResource(IconID), RT_GROUP_ICON)));
	if pmidGroup <> nil then begin
	  if Small then begin
	    iWidth := GetSystemMetrics(SM_CXSMICON);
	    iHeight := GetSystemMetrics(SM_CYSMICON);
	  end
	  else begin
	    iWidth := GetSystemMetrics(SM_CXICON);
	    iHeight := GetSystemMetrics(SM_CYICON);
	  end;
	  iFound := -1;
	  for iIndex := 0 to pred(pmidGroup^.idCount) do with pmidGroup^.idEntries[iIndex] do begin
	    if (bWidth = iWidth) and (bHeight = iHeight) and (wBitCount <= MaxBPP) then begin
	      if (iFound = -1) or (wBitCount > pmidGroup^.idEntries[iFound].wBitCount) then iFound := iIndex;
	    end;
	  end;
	  if iFound > -1 then begin
	    hIcon := FindResource(hLib, MakeIntResource(pmidGroup^.idEntries[iFound].nID), RT_ICON);
	    if hIcon <> 0 then begin
	      Result := CreateIconFromResource(LockResource(LoadResource(hLib, hIcon)),
					       SizeOfResource(hLib, hIcon), true, $00030000);
	      if Result <> 0 then Break;
	    end;
	  end;
	end;
	if IconID <> 1 then
	  IconID := 1
	else Break;
      until Result <> 0;
    finally
      FreeLibrary(hLib);
    end;
  end;
end;

function LegalFileName(const FileName: String): String;
const
  ILLEGAL_CHARS = ['*', '/', ':', '<', '>', '?', '\', '|'];
var
  iLen, iIndex: Integer;
  pResult: PChar;
begin
  if FileName <> E then begin
    iLen := Length(FileName);
    SetString(Result, nil, iLen);
    pResult := Pointer(Result);
    for iIndex := 1 to iLen do begin
      if FileName[iIndex] in ILLEGAL_CHARS then
	pResult^ := '_'
      else pResult^ := FileName[iIndex];
      inc(pResult);
    end;
  end
  else Result := E;
end;

function GetUrlFileName(const UrlPath: String): String;
begin
  Result := Copy(UrlPath, succ(LastCharPos(SL, UrlPath)), MAXINT);
end;

function GetShortPath(const Path: String): String;
var
  iLen: Integer;
begin
  if Path <> E then begin
    iLen := GetShortPathName(Pointer(Path), nil, 0);
    if iLen > 0 then begin
      SetString(Result, nil, pred(iLen));
      if GetShortPathName(Pointer(Path), Pointer(Result), iLen) > 0 then Exit;
    end;
  end;
  Result := Path;
end;

function GetLongPath(const Path: String): String;
var
  iLen: Integer;
begin
  if Path <> E then begin
    iLen := GetLongPathName(Pointer(Path), nil, 0);
    if iLen > 0 then begin
      SetString(Result, nil, pred(iLen));
      if GetLongPathName(Pointer(Path), Pointer(Result), iLen) > 0 then Exit;
    end;
  end;
  Result := Path;
end;

function FileVersionInfo(const FileName: String): String;
var
  dwSize: DWord;
  hInfo: THandle;
begin
  if FileName <> E then begin
    dwSize := succ(GetFileVersionInfoSize(Pointer(FileName), hInfo));
    if dwSize > 1 then begin
      SetString(Result, nil, dwSize);
      if GetFileVersionInfo(Pointer(FileName), hInfo, dwSize, Pointer(Result)) then Exit;
    end;
  end;
  Result := E;
end;

function FindFirst(const FileMask: String; var fdFound: TWin32FindData): THandle;
begin
  Result := FindFirstFile(PChar(FileMask), fdFound);
  if (Result <> INVALID_HANDLE_VALUE) and (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY <> 0) then begin
    if not FindNext(Result, fdFound) then begin
      Windows.FindClose(Result);
      Result := INVALID_HANDLE_VALUE;
    end;
  end;
end;

function FindNext(hSearch: THandle; var fdFound: TWin32FindData): Longbool;
begin
  Result := FindNextFile(hSearch, fdFound);
end;

function FindClose(hSearch: THandle): Longbool;
begin
  Result := Windows.FindClose(hSearch);
end;

function FindFile(const FileMask: String): String;
var
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  hSearch := FindFirstFile(PChar(FileMask), fdFound);
  try
    while hSearch <> INVALID_HANDLE_VALUE do begin
      if (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then begin
	Result := ExtractFilePath(FileMask) + fdFound.cFileName;
	Exit;
      end
      else if not FindNextFile(hSearch, fdFound) then Break;
    end;
  finally
    Windows.FindClose(hSearch);
  end;
  Result := E;
end;

function DeleteFiles(const FileMask: String): Longbool;
var
  sPath: String;
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := false;
  sPath := ExtractFilePath(FileMask);
  hSearch := FindFirstFile(PChar(FileMask), fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    try
      repeat
	if (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then begin
	  Result := DeleteFile(sPath + fdFound.cFileName);
	end;
      until not FindNextFile(hSearch, fdFound);
    finally
      Windows.FindClose(hSearch);
    end;
  end;
end;

function DeleteLinkFolder(const PathName: String): Longbool;
var
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := false;
  hSearch := FindFirstFile(PChar(PathName + '*.lnk'), fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    try
      repeat
	if (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then begin
	  Result := DeleteFile(PathName + fdFound.cFileName);
	end;
      until not FindNextFile(hSearch, fdFound);
    finally
      Windows.FindClose(hSearch);
    end;
  end;
  if Result then Result := RemoveDir(PathName);
end;

function CopyFiles(const FileMask, NewPath: String; Overwrite, Move: Longbool): Longbool;
var
  sPath: String;
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := false;
  sPath := ExtractFilePath(FileMask);
  hSearch := FindFirstFile(PChar(FileMask), fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    try
      repeat
	if (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then begin
	  if Move then begin
	    Result := MoveFileEx(Pointer(sPath + fdFound.cFileName), Pointer(NewPath + fdFound.cFileName),
				 MOVEFILE_REPLACE_EXISTING * ord(Overwrite = true) or MOVEFILE_COPY_ALLOWED);
	  end
	  else Result := CopyFile(Pointer(sPath + fdFound.cFileName), Pointer(NewPath + fdFound.cFileName), not Overwrite);
	end;
      until not FindNextFile(hSearch, fdFound);
    finally
      Windows.FindClose(hSearch);
    end;
  end;
end;

function ReadClipboardText(var Text: String; Unicode: Longbool; ParentWindow: THandle): Longbool;
var
  hText: THandle;
  pText: Pointer;
  iLen: Integer;
begin
  Result := false;
  if OpenClipboard(ParentWindow) then begin
    try
      if Unicode then
	hText := GetClipboardData(CF_UNICODETEXT)
      else hText := GetClipboardData(CF_TEXT);
      if hText <> 0 then begin
	try
	  pText := GlobalLock(hText);
	  if Unicode then
	    iLen := succ(lStrLenW(pText) shl 1)
	  else iLen := lStrLenA(pText);
	  SetString(Text, PChar(pText), iLen);
	  Result := (Text <> E);
	finally
	  GlobalUnlock(hText);
	end;
      end;
    finally
      CloseClipboard;
    end;
  end;
end;

function WriteClipboardData(const Data: Variant; Format: Integer; ParentWindow: THandle): Longbool;
var
  sText: String;
  bmpData: TBitmap;
begin
  Result := false;
  case Format of
    CF_TEXT, CF_UNICODETEXT, CF_RTF, CF_HTML: begin
      sText := Data;
      if sText <> E then begin
	if (Format in [CF_TEXT, CF_UNICODETEXT]) and OpenClipboard(ParentWindow) then begin
	  EmptyClipboard;
	  try
	    case Format of
	      CF_TEXT: if sText[Length(sText)] = NULL then Format := CF_UNICODETEXT;
	      CF_UNICODETEXT: if sText[Length(sText)] <> NULL then Format := CF_TEXT;
	    end;
	    case Format of
	      CF_TEXT: Result := (SetClipboardData(CF_TEXT, GetStringHandleA(sText, 0, false)) <> 0);
	      CF_UNICODETEXT: Result := (SetClipboardData(CF_UNICODETEXT, GetStringHandleW(sText, 0, false)) <> 0);
	    end;
	  finally
	    CloseClipboard;
	  end;
	  Exit;
	end;
	case Format of
	  CF_RTF: Result := RtfToClipboard(sText);
	  CF_HTML: Result := HtmlToClipboard(sText);
	end;
      end;
    end;
    CF_BITMAP: if OpenClipboard(ParentWindow) then begin
      bmpData := TBitmap(Longint(Data));
      EmptyClipboard;
      try
	with bmpData do begin
	  SetClipboardData(CF_BITMAP, Handle);
	  if Palette <> 0 then SetClipboardData(CF_PALETTE, Palette);
	end;
      finally
	CloseClipboard;
      end;
    end;
  end;
end;

function ReadTextFromFile(const FileName: String; var Text: String): Longbool;
begin
  Text := ReadTextFile(FileName);
  Result := (Text <> E);
end;

function WriteTextToFile(const FileName: String; const Text: String): Longbool;
begin
  Result := WriteTextFile(FileName, Text);
end;

function ReadIniFileSection(const FileName, Section: String): TStringList;
var
  iLen: Integer;
  sSection: String;
begin
  Result := nil;
  iLen := $FFFF;
  SetString(sSection, nil, iLen);
  iLen := GetPrivateProfileSection(PChar(Section), Pointer(sSection), iLen, PChar(FileName));
  Delete(sSection, succ(iLen), MAXINT);
  if iLen > 0 then begin
    XChgChars(sSection, NULL, LF);
    Result := TStringList.Create;
    if Result <> nil then Result.Text := sSection;
  end;
end;

function WriteIniFileSection(const FileName, Section: String; Items: TStringList): Longbool;
var
  iLen, iMax, iIndex: Integer;
  sSection: String;
  pSection: PChar;
begin
  Result := false;
  if Items <> nil then with Items do begin
    iLen := 0;
    iMax := pred(Count);
    if iMax >= 0 then begin
      for iIndex := 0 to iMax do inc(iLen, Length(Strings[iIndex]));
      if iLen > 0 then begin
	SetString(sSection, nil, iLen + succ(iMax));
	pSection := Pointer(sSection);
	for iIndex := 0 to iMax do begin
	  iLen := Length(Strings[iIndex]);
	  if iLen > 0 then begin
	    System.Move(Pointer(Strings[iIndex])^, pSection^, iLen);
	    inc(pSection, iLen);
	    pSection^ := NULL;
	    inc(pSection);
	  end;
	end;
	System.Delete(sSection, succ(pSection - Pointer(sSection)), MAXINT);
	Result := WritePrivateProfileSection(PChar(Section), PChar(sSection), PChar(FileName));
      end;
    end;
  end
  else Result := WritePrivateProfileSection(PChar(Section), nil, PChar(FileName));
end;

function ReadIniFileSectionString(const FileName, Section: String): String;
var
  iLen: Integer;
begin
  iLen := $FFFF;
  SetString(Result, nil, iLen);
  iLen := GetPrivateProfileSection(PChar(Section), Pointer(Result), iLen, PChar(FileName));
  Delete(Result, succ(iLen), MAXINT);
end;

function WriteIniFileSectionString(const FileName, Section, Items: String): Longbool;
begin
  if Section <> E then
    Result := WritePrivateProfileSection(PChar(Section), PChar(Items), PChar(FileName))
  else Result := WritePrivateProfileSection(PChar(Section), nil, PChar(FileName));
end;

function ReadIniFileString(const FileName, Section, Item: String): String;
var
  iLen: Integer;
begin
  iLen := 1023;
  SetString(Result, nil, iLen);
  iLen := GetPrivateProfileString(PChar(Section), PChar(Item), E, Pointer(Result), iLen, PChar(FileName));
  Delete(Result, succ(iLen), MAXINT);
end;

function WriteIniFileString(const FileName, Section, Item, Value: String): Longbool;
begin
  if Value <> E then
    Result := WritePrivateProfileString(PChar(Section), PChar(Item), PChar(Value), PChar(FileName))
  else Result := WritePrivateProfileString(PChar(Section), PChar(Item), nil, PChar(FileName));
end;

function ReadIniFileInteger(const FileName, Section, Item: String): Longint;
begin
  Result := GetPrivateProfileInt(PChar(Section), PChar(Item), 0, PChar(FileName));
end;

function WriteIniFileInteger(const FileName, Section, Item: String; Value: Longint): Longbool;
begin
  Result := WritePrivateProfileString(PChar(Section), PChar(Item), PChar(IntToStr(Value)), PChar(FileName));
end;

function ReadIniFileBoolean(const FileName, Section, Item: String): Longbool;
begin
  Result := (GetPrivateProfileInt(PChar(Section), PChar(Item), 0, PChar(FileName)) <> 0);
end;

function WriteIniFileBoolean(const FileName, Section, Item: String; Value: Longbool): Longbool;
begin
  Result := WritePrivateProfileString(PChar(Section), PChar(Item), PChar(IntToStr(ord(Value = true))), PChar(FileName));
end;

function CreateLinkFile(const Link, Target, Arguments, Icon: String): Longbool;
var
  slObject: IShellLink;
  pfObject: IPersistFile;
  iIcon, iPos: Integer;
  sIconFile: String;
begin
  Result := false;
  if (Link <> E) and (Target <> E) then begin
    iIcon := -1;
    if Icon <> E then begin
      iPos := LastCharPos(CM, Icon);
      if iPos > 0 then iIcon := StrToIntDef(Copy(Icon, succ(iPos), MAXINT), -1);
      if iIcon > -1 then sIconFile := Copy(Icon, 1, pred(iPos));
    end;
    {$IFNDEF VER90}
    slObject := CreateComObject(CLSID_ShellLink) as IShellLink;
    if slObject = nil then Exit;
    {$ELSE}
    if CoCreateInstance(CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER, IID_IShellLink, slObject) <> S_OK then Exit;
    pfObject := nil;
    try
    {$ENDIF}
      with slObject do begin
	if SetPath(PChar(Target)) <> S_OK then Exit;
	if SetArguments(PChar(Arguments)) <> S_OK then Exit;
	if (iIcon > -1) and (SetIconLocation(PChar(sIconFile), iIcon) <> S_OK) then Exit;
	if QueryInterface(TGUID(IID_IPersistFile), pfObject) <> S_OK then Exit;
      end;
      if pfObject.Save(Pointer(StrToWideStr(Link)), true) <> S_OK then Exit;
      Result := true;
    {$IFDEF VER90}
    finally
      pfObject.Release;
      slObject.Release;
    end;
    {$ENDIF}
  end;
end;

function GetTargetFileName(const LinkFile: String): String;
var
  slObject: IShellLink;
  pfObject: IPersistFile;
  fdLinkFile: TWin32FindData;
begin
  Result := E;
  if LinkFile <> E then begin
    {$IFNDEF VER90}
    slObject := CreateComObject(CLSID_ShellLink) as IShellLink;
    if slObject = nil then Exit;
    {$ELSE}
    if CoCreateInstance(CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER, IID_IShellLink, slObject) <> S_OK then Exit;
    pfObject := nil;
    try
    {$ENDIF}
      if slObject.QueryInterface(TGUID(IID_IPersistFile), pfObject) <> S_OK then Exit;
      SetString(Result, nil, MAX_PATH);
      if (pfObject.Load(Pointer(StrToWideStr(LinkFile)), STGM_READ) <> S_OK)
      or (slObject.Resolve(0, 1 shl 16 + SLR_NO_UI) <> S_OK)
      or (slObject.GetPath(Pointer(Result), MAX_PATH, fdLinkFile, 0) = S_OK) then begin
	Delete(Result, succ(FastStrLen(PChar(Result))), MAXINT);
	Delete(Result, 1, LastCharPos(BS, Result));
      end
      else Result := E;
    {$IFDEF VER90}
    finally
      pfObject.Release;
      slObject.Release;
    end;
    {$ENDIF}
  end;
end;

function FindExecutable(const FileName: String): String;
begin
  SetString(Result, nil, MAX_PATH);
  if ShellApi.FindExecutable(PChar(FileName), nil, Pointer(Result)) > 32 then
    Delete(Result, succ(FastStrLen(PChar(Result))), MAXINT)
  else Result := E;
end;

function FileExecute(Parent: THandle; const Command, FileName, Parameters: String; Show: Longbool): Longbool;
var
  iFlag: Integer;
  sApplication: String;
begin
  Result := false;
  if FileName <> E then begin
    iFlag := SW_SHOW;
    if not Show then iFlag := SW_HIDE;
    if LoStrStart(EXE, ExtractFileExt(FileName)) then
      sApplication := FileName
    else sApplication := FindExecutable(FileName);
    Result := (ShellExecuteAndWait(Parent, Command, FileName, Parameters, E, iFlag, 10) > 32);
    if Result and Show and (sApplication <> E) then SetForegroundWindow(FindApplicationWindow(sApplication, E, E));
  end;
end;

function SearchPathA(lpPath, lpFileName, lpExtension: PChar; nBufferLength: DWord;
		     lpBuffer: PChar; var lpFilePart: PChar): DWord; stdcall;
		     external kernel32 name 'SearchPathA';

function FindDirectory(OwnerWindow: THandle; const SearchPath, SearchFile, Prompt: String): String;

  function BrowseCallbackProc(hwnd: HWND; uMsg: UINT; lParam: LPARAM; lpData: LPARAM): Integer; stdcall;
  begin
    Result := 0;
    case uMsg of
      BFFM_INITIALIZED: if lpData <> 0 then SendMessage(hwnd, BFFM_SETSELECTION, 1, lpData);
    end;
  end;

var
  biFind: TBrowseInfo;
  pFileName: PChar;
  iPathLen: Integer;
  pidl: PItemIDList;
begin
  SetString(Result, nil, succ(MAX_PATH));
  FillByte(biFind, SizeOf(TBrowseInfo), 0);
  with biFind do begin
    hwndOwner := OwnerWindow;
    pszDisplayName := Pointer(Result);
    lpszTitle := PChar(Prompt);
    ulFlags := BIF_RETURNONLYFSDIRS;
    lpfn := @BrowseCallbackProc;
  end;
  if (SearchPath <> E) and (SearchFile <> E) then begin
    pFileName := nil;
    iPathLen := SearchPathA(PChar(SearchPath), PChar(SearchFile), nil, MAX_PATH, Pointer(Result), pFileName);
    if iPathLen > 0 then begin
      pFileName^ := NULL;
      biFind.lParam := LPARAM(Result);
    end;
  end;
  pidl := SHBrowseForFolder(biFind);
  try
    if (pidl <> nil) and SHGetPathFromIDList(pidl, Pointer(Result)) then begin
      iPathLen := FirstCharPos(NULL, Result);
      Delete(Result, succ(iPathLen), MAXINT);
      Result[iPathLen] := BS;
    end
    else Result := E;
  finally
    if pidl <> nil then CoTaskMemFree(pidl);
  end;
end;

function KeyFileOpen(const Caption, ClipboardName: String; OwnerWindow: THandle): String;
var
  odeKeyFiles: TOpenDialogEx;
begin
  Result := E;
  odeKeyFiles := TOpenDialogEx.Create(nil);
  if odeKeyFiles <> nil then with odeKeyFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      ClipBtnText := ClipboardName;
      Filter.Text := CKIMPORT_FILTER + LF + ALLFILE_FILTER;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function KeyFileSave(const Caption, ClipboardName, UserName: String; OwnerWindow: THandle): String;
var
  sdeKeyFiles: TSaveDialogEx;
begin
  Result := E;
  sdeKeyFiles := TSaveDialogEx.Create(nil);
  if sdeKeyFiles <> nil then with sdeKeyFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      ClipBtnText := ClipboardName;
      Filter.Text := KEYEXPORT_FILTER + LF + ALLFILE_FILTER;
      DefaultExtension := 'asc';
      FileName := LegalFileName(UserName) + DOT + DefaultExtension;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function CertFileOpen(const Caption, ClipboardName: String; OwnerWindow: THandle): String;
var
  odeKeyFiles: TOpenDialogEx;
begin
  Result := E;
  odeKeyFiles := TOpenDialogEx.Create(nil);
  if odeKeyFiles <> nil then with odeKeyFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      ClipBtnText := ClipboardName;
      Filter.Text := CERTIMPORT_FILTER + LF + ALLFILE_FILTER;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function CertFileSave(const Caption, ClipboardName, UserName: String; OwnerWindow: THandle): String;
var
  sdeCertFiles: TSaveDialogEx;
begin
  Result := E;
  sdeCertFiles := TSaveDialogEx.Create(nil);
  if sdeCertFiles <> nil then with sdeCertFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      ClipBtnText := ClipboardName;
      Filter.Text := CERTEXPORT_FILTER + LF + ALLFILE_FILTER;
      DefaultExtension := 'cer';
      FileName := LegalFileName(UserName) + DOT + DefaultExtension;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function ViewerFileSave(const ViewerFile, InitialPath, Caption: String; OwnerWindow: THandle): String;
var
  sdeViewerFiles: TSaveDialogEx;
begin
  Result := E;
  sdeViewerFiles := TSaveDialogEx.Create(nil);
  if sdeViewerFiles <> nil then with sdeViewerFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      Filter.Text := ALLFILE_FILTER;
      FileName := ViewerFile;
      InitialDir := InitialPath;
      if InitialDir = E then InitialDir := GetSystemFolderPathByID(CSIDL_MYDOCUMENTS, false);
      DefaultExtension := Copy(ExtractFileExt(ViewerFile), 2, MAXINT);
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function ImageFileSave(const ImageFile, Caption, Filters: String; OwnerWindow: THandle): String;
var
  sdeImageFiles: TSaveDialogEx;
  sFilterText: String;
begin
  Result := E;
  sdeImageFiles := TSaveDialogEx.Create(nil);
  if sdeImageFiles <> nil then with sdeImageFiles do begin
    try
      Title := Caption;
      ParentHandle := OwnerWindow;
      sFilterText := LowerCase(Filters);
      Filter.Text := sFilterText;
      XChgChars(sFilterText, SC, CM);
      Filter.Text := sFilterText + '|' + Filter.Text;
      DefaultExtension := Copy(ExtractFileExt(ImageFile), 2, MAXINT);
      FileName := ImageFile;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function OpenKeyringFile(const Caption, PGPPath, KeyFile: String; KFT: TKeyringFileType; OwnerWindow: THandle): String;
var
  odeKeyFiles: TOpenDialogEx;
  iPos: Integer;
begin
  Result := E;
  odeKeyFiles := TOpenDialogEx.Create(nil);
  if odeKeyFiles <> nil then with odeKeyFiles do begin
    try
      InitialDir := ExtractFilePath(KeyFile);
      DefaultExt := Copy(ExtractFileExt(KeyFile), 2, MAXINT);
      case KFT of
	kftSecring: Filter.Text := SECRING_FILTER;
	kftPubring: Filter.Text := PUBRING_FILTER;
	kftGroups: Filter.Text := GROUPS_FILTER;
      end;
      Title := Caption;
      ParentHandle := OwnerWindow;
      if not DirExists(InitialDir) then InitialDir := PGPPath;
      if (DefaultExt <> E) and (FirstLoStrPos(DefaultExt, Filter.Text) = 0) then begin
	iPos := FirstCharPos('|', Filter.Text);
	if iPos > 0 then begin
	  Filter.Text := Copy(Filter.Text, 1, iPos) + AST + DOT + AnsiLowerCase(DefaultExt) + SC +
			 Copy(Filter.Text, succ(iPos), MAXINT);
	end;
      end;
      if Execute then Result := FileName;
    finally
      Free;
    end;
  end;
end;

function ReloadModule(const FileName: String): THandle;
begin
  Result := LoadLibrary(PChar(GetModuleFileName(GetModuleHandle(PChar(FileName)))));
end;

function GetLocalizedPath(const Path: String): String;

  function IsQuotedStr(const Str: String): Integer;
  var
    iLen: Integer;
  begin
    Result := 0;
    iLen := Length(Str);
    if (iLen >= 1) and (Str[1] = QU) then inc(Result, 1);
    if (iLen >= 2) and (Str[2] = QU) then inc(Result, 2);
  end;

  function QuotedStr(const Str: String): String;
  begin
    case IsQuotedStr(Str) of
      0: Result := QU + Str + QU;
      1: Result := Str + QU;
      2: Result := QU + Str;
      3: Result := Str;
    end;
  end;

  function UnquotedStr(const Str: String): String;
  begin
    case IsQuotedStr(Str) of
      0: Result := Str;
      1: Result := Copy(Str, 2, Length(Str) - 1);
      2: Result := Copy(Str, 1, Length(Str) - 1);
      3: Result := Copy(Str, 2, Length(Str) - 2);
    end;
  end;

var
  sTrimPath: String;
  iLen, iPos: Integer;
  fQuoted, fBS: Longbool;
  paPath: TPathArray;
  shfi: TSHFileInfo;
begin
  sTrimPath := Trim(Path);
  if (sTrimPath <> E) and not StrAtPos(BS + BS, sTrimPath, 1) then begin
    iLen := Length(sTrimPath);
    fQuoted := ((sTrimPath[1] = QU) and (sTrimPath[iLen] = QU));
    if fQuoted then sTrimPath := Trim(UnquotedStr(sTrimPath));
    iLen := Length(sTrimPath);
    fBS := (sTrimPath[iLen] = BS);
    StrPLCopy(paPath, sTrimPath, MAX_PATH);
    if fBS then paPath[pred(iLen)] := NULL;
    if sTrimPath[2] = CL then begin
      Result := sTrimPath[1] + CL;
      iPos := 3;
    end
    else begin
      Result := E;
      iPos := 0;
    end;
    repeat
      inc(iPos);
      while not (sTrimPath[iPos] in [NULL, BS]) do inc(iPos);
      paPath[pred(iPos)] := NULL;
      if SHGetFileInfo(paPath, DWord(-1), shfi, SizeOf(shfi), SHGFI_DISPLAYNAME) <> 0 then begin
	if Result <> E then
	  Result := Result + BS + shfi.szDisplayName
	else Result := Result + shfi.szDisplayName;
      end
      else begin
	Result := sTrimPath;
	Exit;
      end;
      paPath[pred(iPos)] := sTrimPath[iPos];
    until iPos >= iLen;
    if fBS then AddBackslash(Result, StrLen(PChar(Result)));
    if fQuoted then Result := QuotedStr(Result);
  end
  else Result := sTrimPath;
end;

function GetSystemFolderPathByID(FolderID: Integer; Localize: Longbool): String;
var
  pidl: PItemIDList;
begin
  if SHGetSpecialFolderLocation(0, FolderID, pidl) = S_OK then
    Result := GetSystemFolderPathByPID(pidl, Localize)
  else if (FolderID = CSIDL_MYDOCUMENTS) and (SHGetSpecialFolderLocation(0, CSIDL_PERSONAL, pidl) = S_OK) then
    Result := GetSystemFolderPathByPID(pidl, Localize)
  else Result := E;
end;

function GetSystemFolderPathByPID(FolderID: PItemIDList; Localize: Longbool): String;
begin
  SetString(Result, nil, succ(MAX_PATH));
  if (FolderID <> nil) and SHGetPathFromIDList(FolderID, PChar(Result)) then begin
    AddBackslash(Result, StrLen(PChar(Result)));
    if Localize then Result := GetLocalizedPath(Result);
  end
  else Result := E;
end;

const
  CMF_NORMAL		= $00000000;
  CMF_DEFAULTONLY	= $00000001;
  CMF_VERBSONLY		= $00000002;
  CMF_EXPLORE		= $00000004;
  CMF_NOVERBS		= $00000008;
  CMF_CANRENAME		= $00000010;
  CMF_NODEFAULT		= $00000020;
  CMF_INCLUDESTATIC	= $00000040;
  CMF_EXTENDEDVERBS	= $00000100;
  CMF_RESERVED		= $FFFF0000;

const
  GCS_VERBA		= 00000000;
  GCS_HELPTEXTA		= 00000001;
  GCS_VALIDATEA		= 00000002;
  GCS_VERBW		= 00000004;
  GCS_HELPTEXTW		= 00000005;
  GCS_VALIDATEW		= 00000006;

const
  SFGAO_CANCOPY		= $00000001;		// Objects can be copied
  SFGAO_CANMOVE		= $00000002;		// Objects can be moved
  SFGAO_CANLINK		= $00000004;		// Objects can be linked
  SFGAO_STORAGE		= $00000008;		// supports BindToObject(IID_IStorage)
  SFGAO_CANRENAME	= $00000010;		// Objects can be renamed
  SFGAO_CANDELETE	= $00000020;		// Objects can be deleted
  SFGAO_HASPROPSHEET	= $00000040;		// Objects have property sheets
  SFGAO_DROPTARGET	= $00000100;		// Objects are drop target
  SFGAO_CAPABILITYMASK	= $00000177;
  SFGAO_ENCRYPTED	= $00002000;		// object is encrypted (use alt color)
  SFGAO_ISSLOW		= $00004000;		// 'slow' object
  SFGAO_GHOSTEDt	= $00008000;		// ghosted icon
  SFGAO_LINK		= $00010000;		// Shortcut (link)
  SFGAO_SHARE		= $00020000;		// shared
  SFGAO_READONLY	= $00040000;		// read-only
  SFGAO_HIDDEN		= $00080000;		// hidden object
  SFGAO_DISPLAYATTRMASK	= $000FC000;
  SFGAO_FILESYSANCESTOR	= $10000000;		// may contain children with SFGAO_FILESYSTEM
  SFGAO_FOLDER		= $20000000;		// support BindToObject(IID_IShellFolder)
  SFGAO_FILESYSTEM	= $40000000;		// is a win32 file system object (file/folder/root)
  SFGAO_HASSUBFOLDER	= $80000000;		// may contain children with SFGAO_FOLDER
  SFGAO_CONTENTSMASK	= $80000000;
  SFGAO_VALIDATE	= $01000000;		// invalidate cached information
  SFGAO_REMOVABLE	= $02000000;		// is this removeable media?
  SFGAO_COMPRESSED	= $04000000;		// Object is compressed (use alt color)
  SFGAO_BROWSABLE	= $08000000;		// supports IShellFolder, but only implements CreateViewObject() (non-folder view)
  SFGAO_NONENUMERATED	= $00100000;		// is a non-enumerated object
  SFGAO_NEWCONTENT	= $00200000;		// should show bold in explorer tree
  SFGAO_CANMONIKER	= $00400000;		// defunct
  SFGAO_HASSTORAGE	= $00400000;		// defunct
  SFGAO_STREAM		= $00400000;		// supports BindToObject(IID_IStream)
  SFGAO_STORAGEANCESTOR	= $00800000;		// may contain children with SFGAO_STORAGE or SFGAO_STREAM
  SFGAO_STORAGECAPMASK	= $70C50008;		// for determining storage capabilities, ie for open/save semantics

type
  PCMInvokeCommandInfo = ^TCMInvokeCommandInfo;
  TCMInvokeCommandInfo = Record
    cbSize: DWORD;				// SizeOf(CMINVOKECOMMANDINFO)
    fMask: DWORD;				// any combination of CMIC_MASK_*
    hwnd: HWND;					// might be NULL (indicating no owner window)
    lpVerb: PAnsiChar;				// either a string or MAKEINTRESOURCE(idOffset)
    lpParameters: PAnsiChar;			// might be NULL (indicating no parameter)
    lpDirectory: PAnsiChar;			// might be NULL (indicating no specific directory)
    nShow: Integer;				// one of SW_ values for ShowWindow() API
    dwHotKey: DWORD;
    hIcon: THandle;
  end;

const
  IID_IShellFolder: TGUID  = (D1: $000214E6; D2: $0000; D3: $0000; D4: ($C0, $00, $00, $00, $00, $00, $00, $46));
  IID_IContextMenu: TGUID  = (D1: $000214E4; D2: $0000; D3: $0000; D4: ($C0, $00, $00, $00, $00, $00, $00, $46));
  IID_IContextMenu2: TGUID = (D1: $000214F4; D2: $0000; D3: $0000; D4: ($C0, $00, $00, $00, $00, $00, $00, $46));
  IID_IContextMenu3: TGUID = (D1: $BCFCE0A0; D2: $EC17; D3: $11D0; D4: ($8D, $10, $00, $A0, $C9, $0F, $27, $19));

type
  IUnknown = class
  public
    function QueryInterface(const iid: TIID; var obj): HResult; virtual; stdcall; abstract;
    function AddRef: Longint; virtual; stdcall; abstract;
    function Release: Longint; virtual; stdcall; abstract;
  end;
  IShellFolder = class(IUnknown)
    function ParseDisplayName(hwndOwner: HWND;
      pbcReserved: Pointer; lpszDisplayName: POLESTR; var pchEaten: ULONG;
      var ppidl: PItemIDList; var dwAttributes: ULONG): HResult; virtual; stdcall; abstract;
    function EnumObjects(hwndOwner: HWND; grfFlags: DWORD;
      var EnumIDList: IEnumIDList): HResult; virtual; stdcall; abstract;
    function BindToObject(pidl: PItemIDList; pbcReserved: Pointer;
      const riid: TIID; var ppvOut: Pointer): HResult; virtual; stdcall; abstract;
    function BindToStorage(pidl: PItemIDList; pbcReserved: Pointer;
      const riid: TIID; var ppvObj: Pointer): HResult; virtual; stdcall; abstract;
    function CompareIDs(lParam: LPARAM;
      pidl1, pidl2: PItemIDList): HResult; virtual; stdcall; abstract;
    function CreateViewObject(hwndOwner: HWND; const riid: TIID;
      var ppvOut: Pointer): HResult; virtual; stdcall; abstract;
    function GetAttributesOf(cidl: UINT; var apidl: PItemIDList;
      var rgfInOut: UINT): HResult; virtual; stdcall; abstract;
    function GetUIObjectOf(hwndOwner: HWND; cidl: UINT; var apidl: PItemIDList;
      const riid: TIID; prgfInOut: PUINT; var ppvOut: Pointer): HResult; virtual; stdcall; abstract;
    function GetDisplayNameOf(pidl: PItemIDList; uFlags: DWORD;
      var lpName: TStrRet): HResult; virtual; stdcall; abstract;
    function SetNameOf(hwndOwner: HWND; pidl: PItemIDList; lpszName: POLEStr;
      uFlags: DWORD; var ppidlOut: PItemIDList): HResult; virtual; stdcall; abstract;
  end;
  IContextMenu = class(IUnknown)
    function QueryContextMenu(Menu: HMENU;
      indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult; virtual; stdcall; abstract;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; virtual; stdcall; abstract;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; virtual; stdcall; abstract;
  end;
  IContextMenu2 = class(IContextMenu)
    function HandleMenuMsg(uMsg: UINT; wParam: WPARAM;
      lParam: LPARAM): HResult; virtual; stdcall; abstract;
  end;
  IContextMenu3 = class(IContextMenu2)
    function HandleMenuMsg2(uMsg: UINT; wParam: WPARAM;
      lParam: LPARAM; var plResult: LRESULT): HResult; virtual; stdcall; abstract;
  end;

function SHGetDesktopFolder(var ppshf: IShellFolder): HResult; stdcall;
			    external Shell32 name 'SHGetDesktopFolder';

function SameMenuText(const WCText: PWideChar; const StrText: String): Longbool;
begin
  Result := (CompareText(WideCharToString(WCText), StrText) = 0);
end;

const
  CONTEXT_MENU_DATA_PROP = 'ContextMenuDataProp';

type
  TMenuTrigger = (mtUnknown, mtMouse, mtKey);
  PContextMenuData = ^TContextMenuData;
  TContextMenuData = Record
    OnExecCmd: TOnExecCmd;
    ExecCmdFileName: String;
    MenuTrigger: TMenuTrigger;
    MenuParentHandle: THandle;
    MenuParentWndProc: TFNWndProc;
    MenuObject: IContextMenu;
    MenuObject2: IContextMenu2;
    MenuObject3: IContextMenu3;
    MenuHandle: THandle;
    MinItemID: Cardinal;
    ParentMenu: THandle;
    MenuPos: Cardinal;
  end;

procedure CleanupContextMenuData(var PCMData: PContextMenuData);
begin
  if PCMData <> nil then with PCMData^ do begin
    KillTimer(MenuParentHandle, WM_MENUSELECT);
    KillTimer(MenuParentHandle, WM_EXITMENULOOP);
    if MenuParentWndProc <> nil then SetWindowLong(MenuParentHandle, GWL_WNDPROC, Longint(MenuParentWndProc));
    if MenuParentHandle <> 0 then RemoveProp(MenuParentHandle, CONTEXT_MENU_DATA_PROP);
    if MenuObject3 <> nil then MenuObject3.Release;
    if MenuObject2 <> nil then MenuObject2.Release;
    if MenuObject <> nil then MenuObject.Release;
    if MenuHandle <> 0 then begin
      if ParentMenu = 0 then begin
	while DestroyMenu(MenuHandle) do;
      end
      else while DeleteMenu(ParentMenu, MenuHandle, MF_BYCOMMAND) do;
      DrawMenuBar(MenuParentHandle);
    end;
    ExecCmdFileName := E;
    FreeMem(PCMData);
    PCMData := nil;
  end;
end;

function GetMenuShowDelay: Cardinal;
const
  SPI_GETMENUSHOWDELAY	= $006A;
begin
  Result := 0;
  SystemParametersInfo(SPI_GETMENUSHOWDELAY, 0, @Result, 0);
end;

procedure SetMenuShowDelay(Value: Cardinal);
const
  SPI_SETMENUSHOWDELAY	= $006B;
begin
  SystemParametersInfo(SPI_SETMENUSHOWDELAY, 0, @Value, 0);
end;

function OnSubMenuItem(PCMData: PContextMenuData; const CursorPos: TPoint): Longbool;
begin
  with PCMData^ do Result := (Cardinal(MenuItemFromPoint(MenuParentHandle, ParentMenu, CursorPos)) = MenuPos);
end;

procedure PopulateContextMenu(PCMData: PContextMenuData);
var
  ptMouse: TPoint;
begin
  if PCMData <> nil then with PCMData^ do begin
    MenuObject.QueryContextMenu(MenuHandle, 0, MinItemID, MAXINT, CMF_NODEFAULT or CMF_VERBSONLY);
    case MenuTrigger of
      mtMouse: begin
	ptMouse := GetCursorPos;
	if OnSubMenuItem(PCMData, ptMouse) then with ptMouse do begin
	  if OnSubMenuItem(PCMData, Point(succ(X), Y)) then
	    SetCursorPos(succ(X), Y)
	  else if OnSubMenuItem(PCMData, Point(pred(X), Y)) then
	    SetCursorPos(pred(X), Y);
	end;
      end;
      mtKey: PostMessage(MenuParentHandle, WM_KEYDOWN, VK_RIGHT, 0);
    end;
  end;
end;

function ContextMenuCommandHandler(PCMData: PContextMenuData; ItemID: Cardinal): LRESULT;
var
  iciMenu: TCMInvokeCommandInfo;
  fCancelCommand: Longbool;
begin
  Result := E_FAIL;
  fCancelCommand := false;
  try
    if PCMData <> nil then with PCMData^ do begin
      if (ItemID >= MinItemID) and (ItemID <= UINT(MAXINT)) then with iciMenu do begin
	if Assigned(OnExecCmd) then OnExecCmd(ExecCmdFileName, fCancelCommand);
	if fCancelCommand then Exit;
	FillByte(iciMenu, SizeOf(TCMInvokeCommandInfo), 0);
	cbSize := SizeOf(TCMInvokeCommandInfo);
	lpVerb := Ptr(ItemID - MinItemID);
	nShow := SW_SHOWNORMAL;
	Result := PCMData^.MenuObject.InvokeCommand(iciMenu);
      end;
    end;
  finally
    CleanupContextMenuData(PCMData);
  end;
end;

procedure CustomizeContextMenu(PCMData: PContextMenuData);
const
  MAX_VERB_LEN = $FF;
var
  iIndex: Integer;
  cItem: Cardinal;
  wVerb: Array[0..MAX_VERB_LEN] of WideChar;
begin
  if PCMData <> nil then with PCMData^ do begin
    for iIndex := 0 to pred(GetMenuItemCount(MenuHandle)) do begin
      FillChar(wVerb, SizeOf(wVerb), 0);
      cItem := GetMenuItemID(MenuHandle, iIndex) - MinItemID;
      if MenuString(MenuHandle, iIndex, MF_BYPOSITION) = E then Continue;
      if MenuObject.GetCommandString(cItem, GCS_VERBW, nil, @wVerb, MAX_VERB_LEN) = S_OK then begin
	if SameMenuText(wVerb, 'cut') or SameMenuText(wVerb, 'delete') or SameMenuText(wVerb, 'link')
	or SameMenuText(wVerb, 'paste') or SameMenuText(wVerb, 'properties') or SameMenuText(wVerb, 'rename') then begin
	  EnableMenuItem(MenuHandle, iIndex, MF_BYPOSITION or MF_GRAYED);
	end;
      end;
    end;
  end;
end;

function ContextMenuMsgHandler(PCMData: PContextMenuData; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
const
  MAX_VERB_LEN = $FF;
begin
  Result := ord(uMsg <> WM_INITMENUPOPUP);
  if PCMData <> nil then with PCMData^ do begin
    if (uMsg = WM_INITMENUPOPUP) and ((THandle(wParam) = ParentMenu) or (THandle(wParam) = MenuHandle)) then begin
      if GetMenuItemCount(MenuHandle) <> 0 then
	//CustomizeContextMenu(PCMData)
      else if GetKeyState(VK_RIGHT) < 0 then MenuTrigger := mtKey;
    end
    else if (uMsg <> WM_MENUCHAR) and (MenuObject2 <> nil) then
      MenuObject2.HandleMenuMsg(uMsg, wParam, lParam)
    else if MenuObject3 <> nil then MenuObject3.HandleMenuMsg2(uMsg, wParam, lParam, Result);
  end;
end;

function ContextMenuParentWndProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  PCMData: PContextMenuData;
begin
  PCMData := PContextMenuData(GetProp(hWnd, CONTEXT_MENU_DATA_PROP));
  if PCMData <> nil then begin
    case uMsg of
      WM_DRAWITEM, WM_MEASUREITEM, WM_INITMENUPOPUP, WM_MENUCHAR: if (uMsg <> WM_MEASUREITEM) or (wParam = 0) then begin
	ContextMenuMsgHandler(PCMData, uMsg, wParam, lParam);
      end;
      WM_COMMAND: if (lParam = 0) and (HiWord(wParam) = 0) then begin
	KillTimer(hWnd, WM_EXITMENULOOP);
	Result := ContextMenuCommandHandler(PCMData, wParam);
	if Result = S_OK then Exit;
      end;
    end;
    try
      Result := CallWindowProc(PCMData^.MenuParentWndProc, hWnd, uMsg, wParam, lParam);
    finally
      case uMsg of
	WM_DESTROY, WM_CANCELMODE: CleanupContextMenuData(PCMData);
	WM_TIMER: begin
	  case wParam of
	    WM_MENUSELECT: begin
	      KillTimer(hWnd, WM_MENUSELECT);
	      PopulateContextMenu(PCMData);
	    end;
	    WM_EXITMENULOOP: begin
	      KillTimer(hWnd, WM_EXITMENULOOP);
	      CleanupContextMenuData(PCMData);
	    end;
	  end;
	end;
	WM_MENUSELECT: with PCMData^ do begin
	  if (THandle(lParam) = ParentMenu)
	  and (HiWord(wParam) and MF_POPUP <> 0)
	  and (GetMenuItemCount(MenuHandle) = 0)
	  and (SetTimer(hWnd, WM_MENUSELECT, GetMenuShowDelay, nil) <> 0) then begin
	    if HiWord(wParam) and MF_MOUSESELECT <> 0 then MenuTrigger := mtMouse;
	    Result := 0;
	  end
	  else KillTimer(hWnd, WM_MENUSELECT);
	end;
	WM_EXITMENULOOP: SetTimer(hWnd, WM_EXITMENULOOP, 1, nil);
      end;
    end;
  end
  else Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
end;

procedure CreateFileContextMenu(const FileName, SubMenuName: String; ParentWindow, MainMenu: THandle;
				SubMenuPos, MinMenuID: Cardinal; PTrackPos: PPoint;
				PopupAlignment: TPopupAlignment;
				OnExecMenuCmd: TOnExecCmd);
const
  AAlignments: Array[TPopupAlignment] of Word = (TPM_LEFTALIGN, TPM_RIGHTALIGN, TPM_CENTERALIGN);
var
  PCMData: PContextMenuData;
  sfDesktop, sfTarget: IShellFolder;
  pidlFolder, pidlFile: PItemIDList;
  ulEaten, ulAttributes: ULONG;
  cmObject: IContextMenu;
  uiFlags: UINT;
begin
  PCMData := Ptr(GetProp(ParentWindow, CONTEXT_MENU_DATA_PROP));
  if PCMData <> nil then CleanupContextMenuData(PCMData);
  if not FileExists(FileName) then Exit;
  try
    if SHGetDesktopFolder(sfDesktop) = S_OK then begin
      sfTarget := nil;
      try
	ulEaten := 0;
	ulAttributes := 0;
	with sfDesktop do begin
	  if ParseDisplayName(ParentWindow, nil, Pointer(StrToWideStr(ExtractFilePath(FileName))),
			      ulEaten, pidlFolder, ulAttributes) <> S_OK then Exit;
	  if BindToObject(pidlFolder, nil, IID_IShellFolder, Pointer(sfTarget)) <> S_OK then Exit;
	end;
	ulEaten := 0;
	ulAttributes := 0;
	with sfTarget do begin
	  if ParseDisplayName(ParentWindow, nil, Pointer(StrToWideStr(ExtractFileName(FileName))),
			      ulEaten, pidlFile, ulAttributes) <> S_OK then Exit;
	  if GetUIObjectOf(ParentWindow, 1, pidlFile, IID_IContextMenu,
			   nil, Pointer(cmObject)) <> S_OK then Exit;
	end;
      finally
	if sfTarget <> nil then sfTarget.Release;
	if sfDesktop <> nil then sfDesktop.Release;
      end;
      PCMData := AllocMem(SizeOf(TContextMenuData));
      if PCMData = nil then begin
	cmObject.Release;
	Exit;
      end;
      try
	with PCMData^ do begin
	  OnExecCmd := OnExecMenuCmd;
	  ExecCmdFileName := FileName;
	  MenuObject := cmObject;
	  if MenuObject <> nil then with MenuObject do begin
	    if QueryInterface(IID_IContextMenu2, MenuObject2) <> S_OK then MenuObject2 := nil;
	    if QueryInterface(IID_IContextMenu3, MenuObject3) <> S_OK then MenuObject3 := nil;
	  end;
	  MenuParentHandle := ParentWindow;
	  MenuHandle := CreatePopupMenu;
	  MinItemID := MinMenuID;
	  ParentMenu := MainMenu;
	  MenuPos := SubMenuPos;
	  if MainMenu = 0 then
	    PopulateContextMenu(PCMData)
	  else InsertMenu(MainMenu, SubMenuPos, MF_BYPOSITION or MF_POPUP, MenuHandle, PChar(SubMenuName));
	  MenuParentWndProc := Ptr(SetWindowLong(ParentWindow, GWL_WNDPROC, Longint(@ContextMenuParentWndProc)));
	end;
	SetProp(ParentWindow, CONTEXT_MENU_DATA_PROP, THandle(PCMData));
	if PTrackPos <> nil then begin
	  uiFlags := TPM_RIGHTBUTTON or AAlignments[PopupAlignment];
	  with PTrackPos^ do begin
	    if (X < 0) or (Y < 0) then PTrackPos^ := GetCursorPos;
	    if MainMenu <> 0 then
	      TrackPopupMenu(MainMenu, uiFlags, X, Y, 0, ParentWindow, nil)
	    else TrackPopupMenu(PCMData^.MenuHandle, uiFlags, X, Y, 0, ParentWindow, nil);
	  end;
	end
	else DrawMenuBar(ParentWindow);
      except
	CleanupContextMenuData(PCMData);
	Raise;
      end;
    end;
  finally
    if FileSize(FileName) = 0 then DeleteFile(FileName);
  end;
end;

function GetLongPathNameA; external kernel32 name 'GetLongPathNameA';
function GetLongPathNameW; external kernel32 name 'GetLongPathNameW';
function GetLongPathName; external kernel32 name 'GetLongPathNameA';

function GetShortPathNameA; external kernel32 name 'GetShortPathNameA';
function GetShortPathNameW; external kernel32 name 'GetShortPathNameW';
function GetShortPathName; external kernel32 name 'GetShortPathNameA';


end.

