{$J+,Z4}
unit FlatBtn;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, CommCtrl, Classes, Graphics, Forms,
  Controls, ExtCtrls, Buttons, ToolTip, UxThemes;

type
  TToolBarStyle = (tbsNone, tbsPushButton, tbsToolbarButton);
  TFlatBtn = class(TBitBtn)
  private
    FBtnDefault: Longbool;
    FBtnDown: Longbool;
    FBtnFocused: Longbool;
    FBtnPressed: Longbool;
    FBtnTouched: Longbool;
    FBtnUp: Longbool;
    FCanvas: TCanvas;
    FFace: TBitmap;
    FShortCutOnly: Longbool;
    FShowHint: Longbool;
    FSpacing: Integer;
    FTheme: THandle;
    FToolBarStyle: TToolBarStyle;
    FToolTip: TToolTip;
    FOnToggleToolbarButton: TNotifyEvent;
    procedure SetBtnDown(Value: Longbool);
    function  GetCancel: Longbool;
    procedure SetCancel(Value: Longbool);
    function  GetDefault: Longbool;
    procedure SetDefault(Value: Longbool);
    function  GetHint: String;
    procedure SetHint(const Value: String);
    procedure SetSpacing(Value: Integer);
    procedure SetToolbarStyle(Value: TToolBarStyle);
    procedure OpenTheme;
    procedure CloseTheme;
    function  GetTextSize: TSize;
    function  GetPBState: PUSHBUTTONSTATES;
    function  GetTBState: TOOLBARSTATES;
    procedure CopyGlyphTransparent(const RImage: TRect);
    procedure DrawButtonBackground(const BtnCanvas: TCanvas; const RFace: TRect);
    procedure DrawButtonText(const BtnCanvas: TCanvas; RText: TRect; Alignment: Integer);
    procedure DrawFace(RFace: TRect);
    procedure DrawItem(const BtnDrawStruct: TDrawItemStruct);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMEraseBkGnd(var Message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    property  ToolTip: TToolTip read FToolTip;
  published
    property  BtnDown: Longbool read FBtnDown write SetBtnDown;
    property  Cancel: Longbool read GetCancel write SetCancel;
    property  Default: Longbool read GetDefault write SetDefault;
    property  Hint: String read GetHint write SetHint;
    property  ShowHint: Longbool read FShowHint write FShowHint;
    property  Spacing: Integer read FSpacing write SetSpacing;
    property  ToolbarStyle: TToolBarStyle read FToolbarStyle write SetToolbarStyle;
    property  OnToggleToolbarButton: TNotifyEvent read FOnToggleToolbarButton write FOnToggleToolbarButton;
  end;

procedure Register;


implementation

procedure TFlatBtn.SetBtnDown(Value: Longbool);
begin
  if Value <> FBtnDown then begin
    Invalidate;
    FBtnDown := Value;
    if Assigned(FOnToggleToolbarButton) then FOnToggleToolbarButton(Self);
  end;
end;

function TFlatBtn.GetCancel: Longbool;
begin
  Result := inherited Cancel;
end;

procedure TFlatBtn.SetCancel(Value: Longbool);
begin
  if FToolbarStyle = tbsToolbarButton then Value := false;
  inherited Cancel := Value;
end;

function TFlatBtn.GetDefault: Longbool;
begin
  Result := inherited Default;
end;

procedure TFlatBtn.SetDefault(Value: Longbool);
begin
  if FToolbarStyle = tbsToolbarButton then Value := false;
  inherited Default := Value;
end;

function TFlatBtn.GetHint: String;
begin
  Result := inherited Hint;
end;

procedure TFlatBtn.SetHint(const Value: String);
begin
  inherited Hint := Value;
  if FToolTip <> nil then FToolTip.AnsiText := Value;
end;

procedure TFlatBtn.SetSpacing(Value: Integer);
begin
  if Value <> FSpacing then begin
    FSpacing := Value;
    Invalidate;
  end;
end;

procedure TFlatBtn.SetToolbarStyle(Value: TToolBarStyle);
begin
  FToolbarStyle := Value;
  if FToolbarStyle = tbsToolbarButton then begin
    SetCancel(false);
    SetDefault(false);
  end;
  PostMessage(Handle, WM_THEMECHANGED, 0, 0);
end;

procedure TFlatBtn.OpenTheme;
begin
  if UseThemes and (Color = clBtnFace) then begin
    if FToolbarStyle <> tbsNone then
      FTheme := OpenThemeData(Handle, 'TOOLBAR')
    else FTheme := OpenThemeData(Handle, 'BUTTON');
  end;
end;

procedure TFlatBtn.CloseTheme;
begin
  if FTheme <> 0 then begin
    CloseThemeData(FTheme);
    FTheme := 0;
  end;
end;

function TFlatBtn.GetTextSize: TSize;
var
  rCalc: TRect;
begin
  DrawText(FCanvas.Handle, PChar(Caption), -1, rCalc, DT_CALCRECT);
  with Result, rCalc do begin
    cx := Right - Left;
    cy := Bottom - Top;
  end;
end;

function TFlatBtn.GetPBState: PUSHBUTTONSTATES;
begin
  if Enabled then begin
    if FBtnPressed then
      Result := PBS_PRESSED
    else if FBtnTouched then
      Result := PBS_HOT
    else if FBtnFocused or FBtnDefault then
      Result := PBS_DEFAULTED
    else Result := PBS_NORMAL;
  end
  else Result := PBS_DISABLED;
end;

function TFlatBtn.GetTBState: TOOLBARSTATES;
begin
  if Enabled then begin
    if FBtnPressed then
      Result := TS_PRESSED
    else begin
      if FToolbarStyle = tbsToolbarButton then begin
	if FBtnTouched or FBtnFocused then begin
	  if FBtnDown then
	    Result := TS_HOTCHECKED
	  else Result := TS_HOT;
	end
	else begin
	  if FBtnDown then
	    Result := TS_CHECKED
	  else Result := TS_NORMAL;
	end;
      end
      else if FBtnTouched then
	Result := TS_HOTCHECKED
      else if FBtnFocused or FBtnDefault then
	Result := TS_HOT
      else Result := TS_NORMAL;
    end;
  end
  else Result := TS_DISABLED;
end;

procedure TFlatBtn.CopyGlyphTransparent(const RImage: TRect);
var
  iOffset, iCol, iRow, iWidth, iHeight: Integer;
  rgbTransparent, rgbPix: TColorRef;
begin
  with RImage do begin
    iOffset := 0;
    iWidth := pred(Right - Left);
    iHeight := pred(Bottom - Top);
    if (NumGlyphs > 1) and not Enabled then iOffset := iWidth;
    rgbTransparent := TColorRef(Glyph.TransparentColor and $00FFFFFF);
    if Enabled or (NumGlyphs > 1) then begin
      for iRow := 0 to iWidth do begin
	for iCol := 0 to iHeight do begin
	  rgbPix := Glyph.Canvas.Pixels[iOffset + iCol, iRow];
	  if rgbPix <> rgbTransparent then FFace.Canvas.Pixels[Left + iCol, Top + iRow] := rgbPix;
	end;
      end;
    end
    else begin
      for iRow := 0 to iWidth do begin
	for iCol := 0 to iHeight do begin
	  rgbPix := Glyph.Canvas.Pixels[iCol, iRow];
	  if rgbPix <> rgbTransparent then begin
	    rgbPix := (((rgbPix and $FF) + ((rgbPix shr 8) and $FF) + ((rgbPix shr 16) and $FF)) div 3) shr 1 + $80;
	    FFace.Canvas.Pixels[Left + iCol, Top + iRow] := (rgbPix shl 16) + (rgbPix shl 8) + rgbPix;
	  end;
	end;
      end;
    end;
  end;
end;

procedure TFlatBtn.DrawButtonBackground(const BtnCanvas: TCanvas; const RFace: TRect);
var
  iPart, iState: Integer;
begin
  with BtnCanvas do begin
    FillRect(ClipRect);
    if FTheme <> 0 then begin
      if FToolbarStyle <> tbsNone then begin
	iPart := TP_BUTTON;
	iState := GetTBState;
      end
      else begin
	iPart := BP_PUSHBUTTON;
	iState := GetPBState;
      end;
      if IsThemeBackgroundPartiallyTransparent(FTheme, iPart, iState) then begin
	DrawThemeParentBackground(WindowHandle, Handle, @RFace);
      end;
      DrawThemeBackground(FTheme, Handle, iPart, iState, RFace, nil);
    end;
  end;
end;

procedure TFlatBtn.DrawButtonText(const BtnCanvas: TCanvas; RText: TRect; Alignment: Integer);
begin
  with BtnCanvas do begin
    SetBkMode(Handle, TRANSPARENT);
    DrawText(Handle, PChar(Caption), -1, RText, Alignment or DT_SINGLELINE);
    SetBkMode(Handle, OPAQUE);
  end;
end;

procedure TFlatBtn.DrawFace(RFace: TRect);
var
  iPos, iWSquare, iHSquare, iWMid, iHMid, iTextWidth, iTextHeight, iOffset: Integer;
  rGraph, rText: TRect;
begin
  rGraph := RFace;
  rText := RFace;
  iPos := 0;
  if not Glyph.Empty then begin
    with rGraph do begin
      with Glyph do begin
	iWSquare := Width div NumGlyphs;
	iHSquare := Height;
      end;
      if (Caption <> '') and not FShortCutOnly then begin
	iWMid := Right - Left - iWSquare;
	iHMid := Bottom - Top - iHSquare;
	with GetTextSize do begin
	  iTextWidth := cx;
	  iTextHeight := cy;
	end;
	case Layout of
	  blGlyphLeft: begin
	    iPos := DT_Left or DT_VCenter;
	    inc(Left, (iWMid - iTextWidth - FSpacing) div 2);
	    Right := Left + Glyph.Width;
	    inc(Top, iHMid div 2);
	    Bottom := Top + Glyph.Height;
	    rText.Left := Right + FSpacing;
	  end;
	  blGlyphRight: begin
	    iPos := DT_Right or DT_VCenter;
	    dec(Right, (iWMid - iTextWidth - FSpacing) div 2);
	    Left := Right - Glyph.Width;
	    inc(Top, iHMid div 2);
	    Bottom := Top + Glyph.Height;
	    rText.Right := Left - FSpacing;
	  end;
	  blGlyphTop: begin
	    iPos := DT_Center or DT_Top;
	    inc(Top, (iHMid - iTextHeight - FSpacing) div 2);
	    Bottom := Top + Glyph.Height;
	    inc(Left, iWMid div 2);
	    Right := Left + Glyph.Width;
	    rText.Top := Bottom + FSpacing;
	  end;
	  blGlyphBottom: begin
	    iPos := DT_Center or DT_Bottom;
	    dec(Bottom, (iHMid - iTextHeight - FSpacing) div 2);
	    Top := Bottom - Glyph.Height;
	    inc(Left, iWMid div 2);
	    Right := Left + Glyph.Width;
	    rText.Bottom := Top - FSpacing;
	  end;
	end;
      end
      else with rGraph do begin
	iWMid := (Right - Left - iWSquare) div 2;
	iHMid := (Bottom - Top - iHSquare) div 2;
      end;
    end;
  end
  else begin
    iWMid := 0;
    iHMid := 0;
    iPos := DT_Center or DT_VCenter;
  end;
  with FFace do begin
    if FToolbarStyle = tbsToolbarButton then
      iOffset := Ord(FBtnDown = TRUE)
    else iOffset := Ord(FBtnPressed = TRUE);
    if (Caption <> '') and not FShortCutOnly then begin
      OffsetRect(rText, iOffset, iOffset);
      DrawButtonText(Canvas, rText, iPos);
    end
    else if not Glyph.Empty then with rGraph do begin
      Left := Left + iWMid;
      Right := Left + Glyph.Width;
      Top := Top + iHMid;
      Bottom := Top + Glyph.Height;
    end;
    if not Glyph.Empty then begin
      OffsetRect(rGraph, iOffset, iOffset);
      CopyGlyphTransparent(rGraph);
    end;
    with FCanvas do BitBlt(Handle, 0, 0, Width, Height, Canvas.Handle, 0, 0, SRCCOPY);
  end;
end;

procedure TFlatBtn.DrawItem(const BtnDrawStruct: TDrawItemStruct);
var
  rBack, rFace, rFocus: TRect;
  iWDiff, iHDiff: Integer;
begin
  with BtnDrawStruct do begin
    FBtnDefault := (Default or (itemState and ODS_DEFAULT <> 0));
    FBtnFocused := (itemState and ODS_FOCUS <> 0);
    FBtnPressed := (itemState and ODS_SELECTED <> 0);
    FCanvas.Handle := hDC;
    rBack := rcItem;
  end;
  with FFace, rBack do begin
    Width := Right - Left;
    Height := Bottom - Top;
  end;
  with FFace.Canvas do begin
    Font.Assign(Self.Font);
    Pen.Color := Color;
    Brush.Color := Color;
    if not Enabled then Font.Color := clGrayText;
  end;
  if FTheme = 0 then
    rFace := rBack
  else begin
    if FToolbarStyle <> tbsNone then
      GetThemeBackgroundContentRect(FTheme, FCanvas.Handle, TP_BUTTON, GetTBState, rBack, @rFace)
    else GetThemeBackgroundContentRect(FTheme, FCanvas.Handle, BP_PUSHBUTTON, GetPBState, rBack, @rFace);
  end;
  DrawButtonBackground(FFace.Canvas, rBack);
  iWDiff := rBack.Right - rFace.Right - 4;
  iHDiff := rBack.Bottom - rFace.Bottom - 4;
  if FTheme <> 0 then begin
    InflateRect(rFace, iWDiff, iHDiff);
    if not (FBtnUp and FBtnDown) then
      DrawFace(rFace)
    else if (FBtnUp xor FBtnTouched) then begin
      DrawFace(rFace);
      FBtnUp := not FBtnUp;
    end;
    InflateRect(rFace, -iWDiff, -iHDiff);
    if FBtnFocused and (FToolbarStyle <> tbsToolbarButton) then begin
      rFocus := rFace;
      if FToolbarStyle <> tbsNone then InflateRect(rFocus, -2, -2);
      FCanvas.DrawFocusRect(rFocus);
    end;
  end
  else begin
    with FCanvas do begin
      Brush.Style := bsClear;
      Brush.Color := Color;
      Pen.Width := 1;
      InflateRect(rFace, iWDiff, iHDiff);
      if not (FBtnUp and FBtnDown) then
	DrawFace(rFace)
      else if (FBtnUp xor FBtnTouched) then begin
	DrawFace(rFace);
	FBtnUp := not FBtnUp;
      end;
      if FBtnFocused then DrawFocusRect(rFace);
      InflateRect(rFace, -iWDiff, -iHDiff);
      if FToolbarStyle = tbsToolbarButton then begin
	if FBtnDown then
	  Frame3D(FCanvas, rFace, clBtnShadow, clWhite, 1)
	else Frame3D(FCanvas, rFace, clWhite, clBtnShadow, 1);
      end
      else if FBtnPressed then
	Frame3D(FCanvas, rFace, clBtnShadow, clWhite, 1)
      else if FBtnTouched then
	Frame3D(FCanvas, rFace, clWhite, clBtnShadow, 1)
      else Frame3D(FCanvas, rFace, Color, Color, 1);
    end;
  end;
  FCanvas.Handle := 0;
end;

procedure TFlatBtn.CMColorChanged(var Message: TMessage);
begin
  inherited;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

procedure TFlatBtn.CMDialogChar(var Message: TCMDialogChar);
begin
  if IsAccel(Message.CharCode, Caption) and CanFocus then begin
    Message.Result := 1;
    Click;
  end
  else inherited;
end;

procedure TFlatBtn.CMMouseEnter(var Message: TMessage);
begin
  if FToolTip <> nil then FToolTip.Activate := true;
  FBtnTouched := true;
  Invalidate;
  inherited;
end;

procedure TFlatBtn.CMMouseLeave(var Message: TMessage);
begin
  if FToolTip <> nil then FToolTip.Activate := false;
  FBtnTouched := false;
  Invalidate;
  inherited;
end;

procedure TFlatBtn.CMTextChanged(var Message: TMessage);
begin
  FShortCutOnly := ((Length(Caption) = 2) and (Caption[1] = '&') and (Caption[2] <> '&'));
  Invalidate;
  inherited;
end;

procedure TFlatBtn.CMParentColorChanged(var Message: TMessage);
begin
  FCanvas.Brush.Assign(Parent.Brush);
  inherited;
end;

procedure TFlatBtn.CNDrawItem(var Message: TWMDrawItem);
begin
  DrawItem(Message.DrawItemStruct^);
end;

procedure TFlatBtn.WMCreate(var Message: TWMCreate);
begin
  inherited;
  if EnableThemes(WindowHandle, (Color = clBtnFace)) then OpenTheme;
  if not (csDesigning in ComponentState) then FToolTip := TToolTip.Create(Self);
  if FToolTip <> nil then with FToolTip do begin
    ToolTipStyle := ttsUnderscore;
    AnsiText := Self.Hint;
  end;
end;

procedure TFlatBtn.WMDestroy(var Message: TWMDestroy);
begin
  FToolTip.Free;
  CloseTheme;
  inherited;
end;

procedure TFlatBtn.WMEraseBkGnd(var Message: TWMEraseBkGnd);
begin
  Message.Result := -1;
end;

procedure TFlatBtn.WMSize(var Message: TWMSize);
begin
  inherited;
  if FToolTip <> nil then FToolTip.ToolRect := ClientRect;
end;

procedure TFlatBtn.WMThemeChanged(var Message: TMessage);
begin
  CloseTheme;
  inherited;
  OpenTheme;
  Invalidate;
end;

procedure TFlatBtn.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do begin
    Style := Style or BS_OWNERDRAW;
    ExStyle := ExStyle or WS_EX_TRANSPARENT;
  end;
end;

constructor TFlatBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TCanvas.Create;
  FFace := TBitmap.Create;
  ParentColor := true;
  FSpacing := 4;
  Height := 33;
  Width := 75;
end;

destructor TFlatBtn.Destroy;
begin
  FFace.Free;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TFlatBtn.Click;
begin
  inherited;
  if FToolbarStyle = tbsToolbarButton then SetBtnDown(not FBtnDown);
end;

procedure Register;
begin
  RegisterComponents('New', [TFlatBtn]);
end;

end.

