{$J+,Z4}
unit GroupBoxEx;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, UxThemes;

type
  TGroupBoxEx = class(TGroupBox)
  private
    FTheme: THandle;
    procedure OpenTheme;
    procedure CloseTheme;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CNCtlColorStatic(var Message: TWMCtlColorStatic); message CN_CTLCOLORSTATIC;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMEraseBkGnd(var Message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
  protected
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure PaintWindow(DC: HDC); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  end;

procedure Register;


implementation

procedure TGroupBoxEx.OpenTheme;
begin
  if UseThemes and (Color = clBtnFace) then FTheme := OpenThemeData(Handle, 'BUTTON');
end;

procedure TGroupBoxEx.CloseTheme;
begin
  if FTheme <> 0 then begin
    CloseThemeData(FTheme);
    FTheme := 0;
  end;
end;

procedure TGroupBoxEx.CMColorChanged(var Message: TMessage);
begin
  inherited;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

procedure TGroupBoxEx.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
  Realign;
end;

procedure TGroupBoxEx.CMParentColorChanged(var Message: TMessage);
begin
  Canvas.Brush.Assign(Parent.Brush);
  inherited;
  Realign;
end;

procedure TGroupBoxEx.CNCtlColorStatic(var Message: TWMCtlColorStatic);
begin
  Canvas.Brush.Style := bsClear;
  Message.Result := Canvas.Brush.Handle;
end;

procedure TGroupBoxEx.WMCreate(var Message: TWMCreate);
begin
  inherited;
  if EnableThemes(WindowHandle, (Color = clBtnFace)) then OpenTheme;
end;

procedure TGroupBoxEx.WMDestroy(var Message: TWMDestroy);
begin
  CloseTheme;
  inherited;
end;

procedure TGroupBoxEx.WMEraseBkGnd(var Message: TWMEraseBkGnd);
begin
  if FTheme <> 0 then with Message do begin
    DrawThemeParentBackground(WindowHandle, DC, nil);
    Result := -1;
  end
  else inherited;
end;

procedure TGroupBoxEx.WMThemeChanged(var Message: TMessage);
begin
  CloseTheme;
  inherited;
  OpenTheme;
  Realign;
end;

procedure TGroupBoxEx.AlignControls(AControl: TControl; var Rect: TRect);
begin
  if FTheme <> 0 then begin
    InflateRect(Rect, 1, 1);
    if Ctl3D then InflateRect(Rect, 1, 1);
    GetThemeBackgroundContentRect(FTheme, Canvas.Handle, BP_GROUPBOX, 0, Rect, @Rect);
  end;
  inherited AlignControls(AControl, Rect);
end;

procedure TGroupBoxEx.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do Style := Style or BS_GROUPBOX;
end;

procedure TGroupBoxEx.PaintWindow(DC: HDC);
var
  msgPaint: TMessage;
begin
  if FTheme <> 0 then with msgPaint do begin
    Msg := WM_PAINT;
    wParam := DC;
    lParam := 0;
    Result := 0;
    DefaultHandler(msgPaint);
  end
  else inherited;
end;

constructor TGroupBoxEx.Create(AOwner: TComponent);
begin
  inherited;
  Width := 185;
  Height := 105;
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csDoubleClicks, csReplicatable, csSetCaption];
end;

destructor TGroupBoxEx.Destroy;
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Modified', [TGroupBoxEx]);
end;

end.

