{$J+,Z4}
unit PageCtrlEx;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2008-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, CommCtrl, SysUtils, Classes, Graphics, Forms, Controls, ComCtrls, UxThemes;

type
  TPageControlEx = class(TPageControl)
  private
    FTheme: THandle;
    FButtonStyle: Longbool;
    FTabSheetWndProcs: TStringList;
    procedure OpenTheme;
    procedure CloseTheme;
    function  GetState: TABITEMSTATES;
    procedure SetButtonStyle(Value: Longbool);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMEraseBkGnd(var Message: TWMEraseBkGnd); message WM_ERASEBKGND;
    procedure WMParentNotify(var Message: TWMParentNotify); message WM_PARENTNOTIFY;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color;
    property ParentColor;
    property ButtonStyle: Longbool read FButtonStyle write SetButtonStyle;
  end;

procedure Register;


implementation

const
  PCEX_NAME = 'TPageControlEx';

type
  PSheetData = ^TSheetData;
  TSheetData = Record
    WndProc: TFNWndProc;
    Theme: THandle;
  end;

procedure FinitTabSheet(SheetHandle: THandle; var SheetData: TSheetData);
begin
  with SheetData do begin
    SetWindowLong(SheetHandle, GWL_WNDPROC, Longint(WndProc));
    RemoveProp(SheetHandle, PCEX_NAME);
    if Theme <> 0 then begin
      CloseThemeData(Theme);
      Theme := 0;
    end;
  end;
end;

function TabSheetWndProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  pcex: TPageControlEx;
  iIndex: Integer;
begin
  pcex := TPageControlEx(GetProp(hWnd, PCEX_NAME));
  with pcex.FTabSheetWndProcs do begin
    case uMsg of
      WM_ERASEBKGND: if (pcex.FTheme <> 0) and (DrawThemeParentBackground(hWnd, wParam, nil) = S_OK) then begin
	Result := -1;
	Exit;
      end;
    end;
    iIndex := IndexOf(IntToStr(hWnd));
    Result := CallWindowProc(PSheetData(Objects[iIndex])^.WndProc, hWnd, uMsg, wParam, lParam);
    if uMsg = WM_DESTROY then begin
      FinitTabSheet(hWnd, PSheetData(Objects[iIndex])^);
      Delete(iIndex);
    end;
  end;
end;

constructor TPageControlEx.Create(AOwner: TComponent);
begin
  inherited;
  FTabSheetWndProcs := TStringList.Create;
end;

destructor TPageControlEx.Destroy;
begin
  FTabSheetWndProcs.Free;
  inherited;
end;

procedure TPageControlEx.OpenTheme;
begin
  if UseThemes and (Color = clBtnFace) then FTheme := OpenThemeData(Handle, 'TAB');
end;

procedure TPageControlEx.CloseTheme;
begin
  if FTheme <> 0 then begin
    CloseThemeData(FTheme);
    FTheme := 0;
  end;
end;

function TPageControlEx.GetState: TABITEMSTATES;
const
  ATabItemStates: Array[Boolean] of TABITEMSTATES = (TIS_DISABLED, TIS_NORMAL);
begin
  Result := ATabItemStates[Enabled];
end;

procedure TPageControlEx.SetButtonStyle(Value: Longbool);
const
  TCS_FLATBUTTONS = $0008;
  TAB_BTN_STYLE = TCS_BUTTONS or TCS_FLATBUTTONS;
var
  iStyle: Integer;
begin
  iStyle := GetWindowLong(WindowHandle, GWL_STYLE);
  if Value <> (iStyle and TAB_BTN_STYLE <> 0) then begin
    iStyle := iStyle and not TAB_BTN_STYLE or (ord(Value = true) * TAB_BTN_STYLE);
    SetWindowLong(WindowHandle, GWL_STYLE, iStyle);
    Perform(CM_COLORCHANGED, 0, 0);
    FButtonStyle := Value;
  end;
end;

procedure TPageControlEx.CMColorChanged(var Message: TMessage);
begin
  inherited;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

procedure TPageControlEx.WMCreate(var Message: TWMCreate);
begin
  inherited;
  SetButtonStyle(FButtonStyle);
  if EnableThemes(WindowHandle, (Color = clBtnFace)) then OpenTheme;
end;

procedure TPageControlEx.WMDestroy(var Message: TWMDestroy);
var
  iIndex: Integer;
begin
  with FTabSheetWndProcs do for iIndex := 0 to pred(Count) do begin
    FinitTabSheet(StrToInt(Strings[iIndex]), PSheetData(Objects[iIndex])^);
  end;
  CloseTheme;
  inherited;
end;

procedure TPageControlEx.WMEraseBkGnd(var Message: TWMEraseBkGnd);
var
  rClient, rSheet, rTabs: TRect;
  iState: Integer;
begin
  if FTheme <> 0 then begin
    rClient := ClientRect;
    if Tabs.Count > 0 then begin
      rSheet := DisplayRect;
      rTabs := rClient;
      rClient.Top := rSheet.Top;
      rTabs.Bottom := rSheet.Top;
      FillRect(Message.DC, rTabs, Brush.Handle);
    end;
    iState := GetState;
    if IsThemeBackgroundPartiallyTransparent(FTheme, TABP_BODY, iState) then begin
      DrawThemeParentBackground(WindowHandle, Handle, @rClient);
    end;
    DrawThemeBackground(FTheme, Message.DC, TABP_BODY, GetState, rClient, nil);
  end
  else FillRect(Message.DC, ClientRect, Brush.Handle);
  Message.Result := -1;
end;

procedure TPageControlEx.WMParentNotify(var Message: TWMParentNotify);
var
  psd: PSheetData;
begin
  inherited;
  with Message do case Event of
    WM_CREATE: begin
      if UseThemes then begin
	New(psd);
	if (psd = nil) or (FTabSheetWndProcs.AddObject(IntToStr(ChildWnd), TObject(psd)) = -1) then begin
	  if psd <> nil then Dispose(psd);
	  Exit;
	end;
	with psd^ do begin
	  WndProc := TFNWndProc(SetWindowLong(ChildWnd, GWL_WNDPROC, Longint(@TabSheetWndProc)));
	  Theme := OpenThemeData(ChildWnd, 'TAB');
	end;
	SetProp(ChildWnd, PCEX_NAME, THandle(Self));
      end;
    end;
  end;
end;

procedure TPageControlEx.WMThemeChanged(var Message: TMessage);
begin
  CloseTheme;
  inherited;
  OpenTheme;
  SetWindowPos(WindowHandle, 0, 0, 0, 0, 0, SWP_FRAMECHANGED or SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE or SWP_NOZORDER);
end;

procedure Register;
begin
  RegisterComponents('Modified', [TPageControlEx]);
end;

end.

