{$J+,Z4}
unit PrintDlg;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2008-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, WinSpool, Messages, OLE2, Classes, CommCtrl, CommDlg, Dialogs, Graphics, Printers, SysUtils;


// IPrintDialogCallback --------------------------------------------------------

const
  IID_IObjectWithSite: TGUID = (D1: $FC4801A3; D2: $2BA9; D3: $11CF; D4: ($A2, $29, $00, $AA, $00, $3D, $73, $52));
  IID_IPrintDialogCallback: TGUID = (D1: $5852A2C3; D2: $6530; D3: $11D1; D4: ($B6, $A3, $00, $00, $F8, $75, $7B, $F9));
  IID_IPrintDialogServices: TGUID = (D1: $509AAEDA; D2: $5639; D3: $11D1; D4: ($B6, $A1, $00, $00, $F8, $75, $7B, $F9));

type
  TUnknown = class(IUnknown)
  private
    FRefCount: DWord;
  public
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
  end;

  IObjectWithSite = class(TUnknown)
  public
    function SetSite(const pUnkSite: IUnknown): HResult; virtual; stdcall; abstract;
    function GetSite(const iid: TIID; var obj): HResult; virtual; stdcall; abstract;
  end;

  IPrintDialogCallback = class(TUnknown)
  public
    function InitDone: HResult; virtual; stdcall; abstract;
    function SelectionChange: HResult; virtual; stdcall; abstract;
    function HandleMessage(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM;
			   var pResult: LRESULT): HResult; virtual; stdcall; abstract;
  end;

  IPrintDialogServices = class(TUnknown)
  public
    function GetCurrentDevMode(lpDevMode: PDevMode; var pcbSize: UINT): HResult; virtual; stdcall; abstract;
    function GetCurrentPrinterName(lpPrinterName: PChar; var pcchSize: UINT): HResult; virtual; stdcall; abstract;
    function GetCurrentPortName(lpPortName: PChar; var pcchSize: UINT): HResult; virtual; stdcall; abstract;
  end;


// TPrintDialogEx --------------------------------------------------------------

const
  START_PAGE_GENERAL		= $FFFFFFFF;

  PD_RESULT_CANCEL		= 0;
  PD_RESULT_PRINT		= 1;
  PD_RESULT_APPLY		= 2;

  PD_CURRENTPAGE		= $00400000;
  PD_NOCURRENTPAGE		= $00800000;
  PD_EXCLUSIONFLAGS		= $01000000;
  PD_USELARGETEMPLATE		= $10000000;

  PD_EXCL_COPIESANDCOLLATE	= DM_COPIES or DM_COLLATE;

type
  PPrintPageRange = ^TPrintPageRange;
  TPrintPageRange = Packed Record
    nFromPage: DWORD;
    nToPage: DWORD;
  end;

  TPrintDlgExA = Packed Record
    lStructSize: DWORD;			// size of structure in bytes
    hwndOwner: HWND;			// caller's window handle
    hDevMode: HGLOBAL;			// handle to DevMode
    hDevNames: HGLOBAL;			// handle to DevNames
    hDC: HDC;				// printer DC/IC or NULL
    Flags: DWORD;			// PD_ flags
    Flags2: DWORD;			// reserved
    ExclusionFlags: DWORD;		// items to exclude from driver pages
    nPageRanges: DWORD;			// number of page ranges
    nMaxPageRanges: DWORD;		// max number of page ranges
    lpPageRanges: PPrintPageRange;	// array of page ranges
    nMinPage: DWORD;			// min page number
    nMaxPage: DWORD;			// max page number
    nCopies: DWORD;			// number of copies
    hInstance: THandle;			// instance handle
    lpPrintTemplateName: PAnsiChar;	// template name for app specific area
    lpCallback: Pointer;		// app callback interface
    nPropertyPages: DWORD;		// number of app property pages in lphPropertyPages
    lphPropertyPages: Pointer;		// array of app property page handles
    nStartPage: DWORD;			// start page id
    dwResultAction: DWORD;		// result action if S_OK is returned
  end;

  TPrintDlgExW = Packed Record
    lStructSize: DWORD;			// size of structure in bytes
    hwndOwner: HWND;			// caller's window handle
    hDevMode: HGLOBAL;			// handle to DevMode
    hDevNames: HGLOBAL;			// handle to DevNames
    hDC: HDC;				// printer DC/IC or NULL
    Flags: DWORD;			// PD_ flags
    Flags2: DWORD;			// reserved
    ExclusionFlags: DWORD;		// items to exclude from driver pages
    nPageRanges: DWORD;			// number of page ranges
    nMaxPageRanges: DWORD;		// max number of page ranges
    lpPageRanges: PPrintPageRange;	// array of page ranges
    nMinPage: DWORD;			// min page number
    nMaxPage: DWORD;			// max page number
    nCopies: DWORD;			// number of copies
    hInstance: THandle;			// instance handle
    lpPrintTemplateName: PWideChar;	// template name for app specific area
    lpCallback: Pointer;		// app callback interface
    nPropertyPages: DWORD;		// number of app property pages in lphPropertyPages
    lphPropertyPages: Pointer;		// array of app property page handles
    nStartPage: DWORD;			// start page id
    dwResultAction: DWORD;		// result action if S_OK is returned
  end;

  TPrintDlgEx = TPrintDlgExA;

  TPrintDialogEx = Class;

  TObjectWithSite = class(IObjectWithSite)
  private
    FSite: IUnknown;
  public

    // Methods of the IUnknown interface
    function QueryInterface(const iid: TIID; var obj): HResult; override;

    // Methods of the IObjectWithSite interface
    function SetSite(const pUnkSite: IUnknown): HResult; override;
    function GetSite(const iid: TIID; var obj): HResult; override;

    property Site: IUnknown read FSite;
  end;

  TPDCallback = class(IPrintDialogCallback)
  private
    FPageSize: TSize;
    FPrintDialogEx: TPrintDialogEx;
    FObjectWithSite: TObjectWithSite;
    function PDServices: IPrintDialogServices;
    function GetCurrentDevMode: TDevMode;
    function GetCurrentPortName: String;
    function GetCurrentPrinterName: String;
    function GetPrinterDC(const Printer: String): THandle;
    procedure OnPrinterSelectionChange;
  public
    constructor Create(PrintDialogEx: TPrintDialogEx);
    destructor Destroy; override;

    // Methods of the IUnknown interface
    function QueryInterface(const iid: TIID; var obj): HResult; override;

    // Methods of the IPrintDialogCallback interface
    function InitDone: HResult; override;
    function SelectionChange: HResult; override;
    function HandleMessage(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM;
			   var pResult: LRESULT): HResult; override;

    property CurrentDevMode: TDevMode read GetCurrentDevMode;
    property CurrentPortName: String read GetCurrentPortName;
    property CurrentPrinterName: String read GetCurrentPrinterName;
    property PageSize: TSize read FPageSize;
  end;

  TPrinterName = Array[0..MAX_PATH] of Char;
  TFontName = Array[0..LF_FACESIZE] of Char;
  TMarginOption = (moPrintHeader, moPrintFooter);
  TMarginOptions = Set of TMarginOption;

  TFontNameSize = Record
    Name: String;
    Size: Integer;
  end;

  PEnumFontParams = ^TEnumFontParams;
  TEnumFontParams = Record
    PDE: TPrintDialogEx;
    ComboBox: THandle;
    ScreenDC: THandle;
    ScreenResolution: Integer;
    DefaultFont: TFontNameSize;
    Preselect: TFontNameSize;
    ItemCount: Integer;
  end;

  TPrinterPrefs = Record
    Initialized: Longbool;
    ApplySettings: Longbool;
    Printer: TPrinterName;
    Copies: Integer;
    Collate: Longbool;
    CustomOption: Longbool;
    MarginFontStyles: TFontStyles;
    MarginOptions: TMarginOptions;
    Margins: TRect;
    Orientation: TPrinterOrientation;
    PrinterFontName: TFontName;
    PrinterFontSize: Integer;
    PrintRange: TPrintRange;
  end;

  TPrefsCaptions = Record
    PropsSheet: String;
    OrientationGroup: String;
    PagePortrait: String;
    PageLandscape: String;
    MarginsGroup: String;
    HMargins: String;
    VMargins: String;
    FontGroup: String;
    FontName: String;
    FontSize: String;
    HeadFootGroup: String;
    PrintHeader: String;
    PrintFooter: String;
    CustomOption: String;
  end;

  TPrinterData = Record
    OwnerWindow: THandle;
    PrintJobName: String;
    PrefsCaptions: TPrefsCaptions;
    case Integer of
      0: (
	Initialized: Longbool;
	ApplySettings: Longbool;
	Printer: TPrinterName;
	Copies: Integer;
	Collate: Longbool;
	CustomOption: Longbool;
	MarginFontStyles: TFontStyles;
	MarginOptions: TMarginOptions;
	Margins: TRect;
	Orientation: TPrinterOrientation;
	PrinterFontName: TFontName;
	PrinterFontSize: Integer;
	PrintRange: TPrintRange;
      );
      1: (PrinterPrefs: TPrinterPrefs);
  end;

  {$HINTS OFF}
  TPrintDialogEx = class(TPrintDialog)
  private
    FMarginsSheet: THandle;
    FPDCallback: TPDCallback;
    FPrintDlgEx: TPrintDlgEx;
    FPrinterData: TPrinterData;
    PrintDlgEx: function(var PrintDlgEx: TPrintDlgEx): HResult; stdcall;
    procedure ShowControl(CTL_ID: Integer; Show: Longbool);
    procedure SetControlText(CTL_ID: Integer; Text: String);
    procedure ClickButton(BTN_ID: Integer);
    function  GetButtonState(BTN_ID: Integer): Longbool;
    procedure SetButtonState(BTN_ID: Integer; Check: Longbool);
    procedure SetOrientation;
    procedure InitOrientationControls;
    procedure SetUpDnRange(UD_ID: Integer);
    function  GetMaxMargin(ID: Integer): Cardinal;
    function  GetMargin(ED_ID: Integer): Cardinal;
    procedure SetMargin(ED_ID: Integer; Value: Cardinal);
    procedure InitMarginControl(ED_ID: Integer);
    procedure InitMarginControls;
    function  AddItem(ComboBox: THandle; const Item: String): Integer;
    function  FindItem(ComboBox: THandle; const Item: String): Integer;
    function  GetIndexedItem(ComboBox: THandle; Index: Integer): String;
    function  GetItemCount(ComboBox: THandle): Integer;
    function  GetSelectedIndex(ComboBox: THandle): Integer;
    procedure SetSelectedIndex(ComboBox: THandle; Index: Integer);
    function  GetSelectedItem(ComboBox: THandle): String;
    procedure SetSelectedItem(ComboBox: THandle; const Item: String);
    procedure InitEnumFontParams(var EnumFontParams: TEnumFontParams);
    procedure GetFontNames(ComboBox: THandle; EnumFontParams: PEnumFontParams);
    procedure GetFontSizes(ComboBox: THandle; EnumFontParams: PEnumFontParams);
    procedure ResizeFontControl(CBX_ID: Integer);
    procedure InitFontControl(CBX_ID: Integer);
    procedure InitFontControls;
    procedure InitCheckBox(CB_ID: Integer);
    procedure InitHeadFootControls;
    procedure OnInitDialogSheet;
    procedure OnNotifyDialogSheet(const NMHdr: TNMHdr);
    procedure OnOptionsChangeCommand(hControl: THandle; CTL_ID: Integer);
  protected
    function CreateMarginsSheet(const Name: String): HPropSheetPage;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {$IFDEF CONDITIONALEXPRESSIONS}
    function Execute: Boolean; override;
    {$ELSE}
    function Execute: Boolean;
    {$ENDIF}
    property MarginsSheet: THandle read FMarginsSheet;
    property PDCallback: TPDCallback read FPDCallback;
    property PrinterData: TPrinterData read FPrinterData write FPrinterData;
  published
  end;
  {$HINTS ON}

function ExecutePrintDialog(var PrinterData: TPrinterData): Longbool;


implementation

{$R *.RES}
{$I *.INC}


// Dialog utilities ------------------------------------------------------------

const
  E = '';

function Str2WideStr(const Str: String): String;
var
  iPCharLen: Integer;
begin
  iPCharLen := succ(Length(Str));
  SetString(Result, nil, iPCharLen shl 1);
  StringToWideChar(Str, Pointer(Result), iPCharLen);
end;

function ClassString(Window: HWND): String;
var
  iLen: Integer;
begin
  SetString(Result, nil, MAX_PATH);
  iLen := GetClassName(Window, Pointer(Result), MAX_PATH);
  if iLen > 0 then begin
    Delete(Result, succ(iLen), MAXINT);
    CharUpper(Pointer(Result));
  end
  else Result := E;
end;

function IsDialog(Window: THandle): Longbool;
const
  DIALOG = '#32770';
begin
  Result := (ClassString(Window) = DIALOG) and (GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_DLGMODALFRAME <> 0);
end;

function IsPrintDialog(Window: THandle): THandle;
begin
  if IsDialog(Window) and not IsDialog(GetWindow(Window, GW_OWNER)) then
    Result := Window
  else Result := 0;
end;

procedure CenterDialog(Dialog, Owner: THandle);
var
  rOwner, rDialog: TRect;
begin
  GetWindowRect(Owner, rOwner);
  with rOwner do begin
    dec(Right, pred(Left));
    dec(Bottom, pred(Top));
  end;
  GetWindowRect(Dialog, rDialog);
  with rDialog do begin
    dec(Right, pred(Left));
    dec(Bottom, pred(Top));
    Left := rOwner.Left + (rOwner.Right - Right) div 2;
    Top := rOwner.Top + (rOwner.Bottom - Bottom) div 2;
    MoveWindow(Dialog, Left, Top, Right, Bottom, false);
  end;
end;


// TUnknown --------------------------------------------------------------------

function TUnknown.QueryInterface(const iid: TIID; var obj): HResult;
begin
  Result := S_OK;
  if IsEqualIID(iid, IID_IUnknown) then begin
    IUnknown(obj) := Self;
    AddRef;
  end
  else Result := E_NOINTERFACE;
end;

function TUnknown.AddRef: Longint;
begin
  inc(FRefCount);
  Result := FRefCount;
end;

function TUnknown.Release: Longint;
begin
  dec(FRefCount);
  Result := FRefCount;
end;


// TObjectWithSite -------------------------------------------------------------

function TObjectWithSite.QueryInterface(const iid: TIID; var obj): HResult;
begin
  Result := S_OK;
  if IsEqualIID(iid, IID_IObjectWithSite) then begin
    IUnknown(obj) := Self;
    AddRef;
  end
  else Result := inherited QueryInterface(iid, obj);
end;

function TObjectWithSite.SetSite(const pUnkSite: IUnknown): HResult;
begin
  Result := S_OK;
  if pUnkSite <> nil then pUnkSite.AddRef;
  if FSite <> nil then FSite.Release;
  FSite := pUnkSite;
end;

function TObjectWithSite.GetSite(const iid: TIID; var obj): HResult;
begin
  Result := E_FAIL;
  IUnknown(obj) := nil;
  if FSite = nil then Exit;
  Result := FSite.QueryInterface(iid, Obj);
end;


// TPDCallback -----------------------------------------------------------------

constructor TPDCallback.Create(PrintDialogEx: TPrintDialogEx);
begin
  inherited Create;
  FPrintDialogEx := PrintDialogEx;
  FObjectWithSite := TObjectWithSite.Create;
end;

destructor TPDCallback.Destroy;
begin
  FObjectWithSite.Free;
  inherited Destroy;
end;

function TPDCallback.QueryInterface(const iid: TIID; var obj): HResult;
begin
  Result := S_OK;
  if IsEqualIID(iid, IID_IPrintDialogCallback) then begin
    IUnknown(obj) := Self;
    AddRef;
  end
  else if (FObjectWithSite = nil) or (FObjectWithSite.QueryInterface(iid, obj) <> S_OK) then begin
    Result := inherited QueryInterface(iid, obj);
  end;
end;

function TPDCallback.InitDone: HResult;
begin
  Result := S_FALSE;
end;

function TPDCallback.SelectionChange: HResult;
begin
  Result := S_FALSE;
  if FPrintDialogEx.MarginsSheet <> 0 then OnPrinterSelectionChange;
end;

function TPDCallback.HandleMessage(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM; var pResult: LRESULT): HResult;
begin
  Result := S_FALSE;
end;

function TPDCallback.PDServices: IPrintDialogServices;
begin
  Result := nil;
  if FObjectWithSite = nil then Exit;
  Result := IPrintDialogServices(FObjectWithSite.Site);
end;

function TPDCallback.GetCurrentDevMode: TDevMode;
var
  pdm: PDevMode;
  uSize: UINT;
begin
  FillChar(Result, SizeOf(Result), 0);
  if PDServices <> nil then with PDServices do begin
    uSize := 0;
    if GetCurrentDevMode(nil, uSize) = S_OK then begin
      pdm := AllocMem(uSize);
      if pdm = nil then Exit;
      try
	if GetCurrentDevMode(pdm, uSize) = S_OK then Result := pdm^;
      finally
	FreeMem(pdm);
      end;
    end;
  end;
end;

function TPDCallback.GetCurrentPortName: String;
var
  uSize: UINT;
begin
  if PDServices <> nil then with PDServices do begin
    uSize := 0;
    if GetCurrentPortName(nil, uSize) = S_OK then begin
      SetString(Result, nil, uSize);
      if GetCurrentPortName(Pointer(Result), uSize) = S_OK then begin
	Delete(Result, uSize, MAXINT);
	Exit;
      end;
    end;
  end;
  Result := E;
end;

function TPDCallback.GetCurrentPrinterName: String;
var
  uSize: UINT;
begin
  if PDServices <> nil then with PDServices do begin
    uSize := 0;
    if GetCurrentPrinterName(nil, uSize) = S_OK then begin
      SetString(Result, nil, uSize);
      if GetCurrentPrinterName(Pointer(Result), uSize) = S_OK then begin
	Delete(Result, uSize, MAXINT);
	Exit;
      end;
    end;
  end;
  Result := E;
end;

function TPDCallback.GetPrinterDC(const Printer: String): THandle;
begin
  Result := 0;
  if Printer <> E then begin
    Result := CreateDC(nil, Pointer(Printer), nil, nil);
    if Result = 0 then CreateDC('WINSPOOL', Pointer(Printer), nil, nil);
  end;
end;

procedure TPDCallback.OnPrinterSelectionChange;
var
  hDC: THandle;
begin
  with FPageSize do begin
    hDC := GetPrinterDC(GetCurrentPrinterName);
    if hDC <> 0 then begin
      try
	cx := GetDeviceCaps(hDC, HORZSIZE);
	cy := GetDeviceCaps(hDC, VERTSIZE);
      finally
	DeleteDC(hDC);
      end;
    end;
  end;
  with FPrintDialogEx do begin
    InitMarginControls;
    InitFontControls;
  end;
end;


// Printer utilities -----------------------------------------------------------

function GetCurrentPrinter(Orientation: TPrinterOrientation; var DeviceNames: THandle): THandle;
const
  AOrientations: Array[TPrinterOrientation] of Integer = (DMORIENT_PORTRAIT, DMORIENT_LANDSCAPE);
type
  TPrinterData = Array[0..MAX_PATH] of Char;
  PDevData = ^TDevData;
  TDevData = Packed Record
    DevName: TDevNames;
    Driver, Device, Output: TPrinterData;
  end;
var
  pdd: PDevData;
  hprn: THandle;
  pdm: PDevMode;
begin
  DeviceNames := GlobalAlloc(GHND, SizeOf(TDevData));
  pdd := PDevData(GlobalLock(DeviceNames));
  if pdd <> nil then with pdd^ do begin
    try
      Printer.GetPrinter(Device, Driver, Output, Result);
      Result := 0;
      if Device <> E then begin
	pdm := nil;
	if OpenPrinter(Device, hprn, nil) then begin
	  Result := GlobalAlloc(GHND, DocumentProperties(0, hprn, Device, pdm^, pdm^, 0));
	  try
	    if Result = 0 then Exit;
	    pdm := GlobalLock(Result);
	    if pdm <> nil then begin
	      DocumentProperties(0, hprn, Device, pdm^, pdm^, DM_OUT_BUFFER);
	      pdm^.dmOrientation := AOrientations[Orientation];
	    end;
	  finally
	    GlobalUnlock(Result);
	    if hprn = 0 then begin
	      GlobalFree(Result);
	      Result := 0;
	    end
	    else ClosePrinter(hprn);
	  end;
	end;
	with DevName do begin
	  wDriverOffset := Longint(@Driver) - Longint(@DevName);
	  wDeviceOffset := Longint(@Device) - Longint(@DevName);
	  wOutputOffset := Longint(@Output) - Longint(@DevName);
	  wDefault := 0;
	end;
      end;
    finally
      GlobalUnlock(DeviceNames);
    end;
  end;
end;

procedure SetPrinter(DeviceNames, DeviceMode: THandle);
var
  DevNames: PDevNames;
begin
  DevNames := PDevNames(GlobalLock(DeviceNames));
  try
    with DevNames^ do begin
      Printer.SetPrinter(PChar(DevNames) + wDeviceOffset,
			 PChar(DevNames) + wDriverOffset,
			 PChar(DevNames) + wOutputOffset,
			 DeviceMode);
    end;
  finally
    GlobalUnlock(DeviceNames);
    GlobalFree(DeviceNames);
  end;
end;


// TPrintDialogEx --------------------------------------------------------------

procedure SetWndProcProp(hWnd: HWND);
begin
  SetProp(hWnd, PChar(IntToHex(hWnd, 8)), GetWindowLong(hWnd, GWL_WNDPROC));
end;

procedure RemoveWndProcProp(hWnd: HWND);
begin
  RemoveProp(hWnd, PChar(IntToHex(hWnd, 8)));
end;

function GetWndProc(hWnd: HWND): TFNWndProc;
begin
  Result := TFNWndProc(GetProp(hWnd, PChar(IntToHex(hWnd, 8))));
end;

procedure SetWndProc(hWnd: HWND; NewProc: TFNWndProc);
begin
  SetWindowLong(hWnd, GWL_WndProc, Longint(NewProc));
end;

procedure SetUserData(hWnd: HWND; Data: Longint);
begin
  if hWnd <> 0 then SetWindowLong(hWnd, GWL_USERDATA, Data);
end;

function GetUserData(hWnd: HWND): TPrintDialogEx;
begin
  Result := TPrintDialogEx(GetWindowLong(hWnd, GWL_USERDATA));
end;

constructor TPrintDialogEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPDCallback := TPDCallback.Create(Self);
  PrintDlgEx := GetProcAddress(GetModulehandle('ComDlg32.dll'), 'PrintDlgExA');
end;

destructor TPrintDialogEx.Destroy;
begin
  FPDCallback.Free;
  inherited Destroy;
end;

function TPrintDialogEx.Execute: Boolean;
const
  APrintRanges: Array[TPrintRange] of Integer = (PD_ALLPAGES, PD_SELECTION, PD_PAGENUMS);
var
  ppr: TPrintPageRange;
  psp: Array[0..0] of HPropSheetPage;
begin
  with Printer do PrinterIndex := Printers.IndexOf(FPrinterData.Printer);
  Copies := FPrinterData.Copies;
  Collate := FPrinterData.Collate;
  PrintRange := FPrinterData.PrintRange;
  case FPrinterData.PrintRange of
    prSelection: Options := Options + [poSelection];
    prPageNums: Options := Options + [poPageNums];
  end;
  if Assigned(PrintDlgEx) then begin
    with ppr do begin
      nFromPage := FromPage;
      nToPage := ToPage;
    end;
    FillChar(FPrintDlgEx, SizeOf(TPrintDlgEx), 0);
    with FPrintDlgEx do begin
      lStructSize := SizeOf(TPrintDlgEx);
      hWndOwner := FPrinterData.OwnerWindow;
      hDevMode := GetCurrentPrinter(FPrinterData.Orientation, hDevNames);
      Flags := APrintRanges[PrintRange] or PD_NOCURRENTPAGE or PD_EXCLUSIONFLAGS;
      if Collate then inc(Flags, PD_COLLATE);
      if not (poPageNums in Options) then inc(Flags, PD_NOPAGENUMS);
      if not (poSelection in Options) then inc(Flags, PD_NOSELECTION);
      if PrintToFile then inc(Flags, PD_PRINTTOFILE);
      if not (poPrintToFile in Options) then inc(Flags, PD_HIDEPRINTTOFILE);
      if poDisablePrintToFile in Options then inc(Flags, PD_DISABLEPRINTTOFILE);
      if poHelp in Options then inc(Flags, PD_SHOWHELP);
      if not (poWarning in Options) then inc(Flags, PD_NOWARNING);
      ExclusionFlags := PD_EXCL_COPIESANDCOLLATE;
      if (poPageNums in Options) then begin
	nPageRanges := 1;
	nMaxPageRanges := 1;
      end;
      lpPageRanges := @ppr;
      nMinPage := MinPage;
      nMaxPage := MaxPage;
      nCopies := Copies;
      lpCallback := FPDCallback;
      psp[0] := CreateMarginsSheet(FPrinterData.PrefsCaptions.PropsSheet);
      if psp[0] <> nil then nPropertyPages := 1;
      lphPropertyPages := @psp;
      nStartPage := START_PAGE_GENERAL;
      Result := (PrintDlgEx(FPrintDlgEx) = S_OK) and (dwResultAction = PD_RESULT_PRINT);
      FPrinterData.ApplySettings := (dwResultAction <> PD_RESULT_CANCEL);
      if FPrinterData.ApplySettings then begin
	SetPrinter(hDevNames, hDevMode);
	Printer.Orientation := FPrinterData.Orientation;
	Collate := (Flags and PD_COLLATE <> 0);
	PrintToFile := (Flags and PD_PRINTTOFILE <> 0);
	PrintRange := prAllPages;
	if (Flags and PD_SELECTION) <> 0 then
	  PrintRange := prSelection
	else if (Flags and PD_PAGENUMS) <> 0 then PrintRange := prPageNums;
	with ppr do begin
	  FromPage := nFromPage;
	  ToPage := nToPage;
	end;
	if nCopies = 1 then
	  Copies := Printer.Copies
	else Copies := nCopies;
      end
      else begin
	if hDevMode <> 0 then GlobalFree(hDevMode);
	if hDevNames <> 0 then GlobalFree(hDevNames);
      end;
    end;
  end
  else Result := inherited Execute;
  with Printer do StrPLCopy(FPrinterData.Printer, Printers[PrinterIndex], pred(SizeOf(TPrinterName)));
  FPrinterData.Copies := Copies;
  FPrinterData.Collate := Collate;
  FPrinterData.PrintRange := PrintRange;
end;

function MarginsSheetProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  pde: TPrintDialogEx;
begin
  if uMsg = WM_INITDIALOG then SetUserData(hWnd, PPropSheetPage(lParam).lParam);
  if GetWindowLong(hWnd, GWL_USERDATA) = 0 then SetWndProcProp(hWnd);
  pde := GetUserData(hWnd);
  if pde <> nil then with pde do begin
    FMarginsSheet := hWnd;
    Result := 0;
    case uMsg of
      WM_INITDIALOG: OnInitDialogSheet;
      WM_NOTIFY: OnNotifyDialogSheet(pnmhdr(lParam)^);
      WM_COMMAND: if IsWindowVisible(lParam) then begin
	case HiWord(wParam) of
	  BN_CLICKED, CBN_SELENDOK, EN_CHANGE: OnOptionsChangeCommand(lParam, LoWord(wParam));
	end;
      end;
    end;
  end
  else Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
  if uMsg = WM_DESTROY then RemoveWndProcProp(hWnd);
end;

function TPrintDialogEx.CreateMarginsSheet(const Name: String): HPropSheetPage;
var
  psp: TPropSheetPageA;
begin
  FillChar(psp, SizeOf(TPropSheetPageA), 0);
  psp.hInstance := hInstance;
  with psp do begin
    dwSize := SizeOf(TPropSheetPageA);
    dwFlags := PSP_USETITLE;
    pszTemplate := MakeIntResourceA(SHEET_FORMATTING);
    pszTitle := Pointer(Name);
    pfnDlgProc := @MarginsSheetProc;
    lParam := Longint(Self);
  end;
  Result := CreatePropertySheetPageA(psp);
end;

procedure TPrintDialogEx.ShowControl(CTL_ID: Integer; Show: Longbool);
begin
  if Show then
    ShowWindow(GetDlgItem(FMarginsSheet, CTL_ID), SW_SHOW)
  else ShowWindow(GetDlgItem(FMarginsSheet, CTL_ID), SW_HIDE);
end;

procedure TPrintDialogEx.SetControlText(CTL_ID: Integer; Text: String);
begin
  SetDlgItemText(FMarginsSheet, CTL_ID, PChar(Text));
end;

procedure TPrintDialogEx.ClickButton(BTN_ID: Integer);
begin
  SendDlgItemMessage(FMarginsSheet, BTN_ID, BM_CLICK, 0, 0);
end;

function TPrintDialogEx.GetButtonState(BTN_ID: Integer): Longbool;
begin
  Result := (SendDlgItemMessage(FMarginsSheet, BTN_ID, BM_GETCHECK, 0, 0) = BST_CHECKED)
end;

procedure TPrintDialogEx.SetButtonState(BTN_ID: Integer; Check: Longbool);
begin
  SendDlgItemMessage(FMarginsSheet, BTN_ID, BM_SETCHECK, ord(Check = true) * BST_CHECKED, 0);
end;

procedure TPrintDialogEx.SetOrientation;
const
  AButtons: Array[TPrinterOrientation] of Integer = (RB_PORTRAIT, RB_LANDSCAPE);
begin
  ClickButton(AButtons[FPrinterData.Orientation]);
end;

procedure TPrintDialogEx.InitOrientationControls;
begin
  with FPrinterData.PrefsCaptions do begin
    SetControlText(GROUP_ORIENTATION, OrientationGroup);
    SetControlText(RB_PORTRAIT, PagePortrait);
    SetControlText(RB_LANDSCAPE, PageLandscape);
    SetOrientation;
  end;
end;

procedure TPrintDialogEx.SetUpDnRange(UD_ID: Integer);
begin
  SendDlgItemMessage(FMarginsSheet, UD_ID, UDM_SETRANGE, 0, MakeLong(GetMaxMargin(UD_ID), 0));
end;

function TPrintDialogEx.GetMaxMargin(ID: Integer): Cardinal;
begin
  with FPDCallback.PageSize do begin
    case ID of
      EDIT_HORZ, UPDN_HORZ: Result := cx div 3;
      EDIT_VERT, UPDN_VERT: Result := cy div 3;
    else
      Result := 0;
    end;
  end;
  if Result > 99 then Result := 99;
end;

function TPrintDialogEx.GetMargin(ED_ID: Integer): Cardinal;
var
  fSuccess: Longbool;
begin
  Result := GetDlgItemInt(FMarginsSheet, ED_ID, fSuccess, false);
end;

procedure TPrintDialogEx.SetMargin(ED_ID: Integer; Value: Cardinal);
begin
  SetDlgItemInt(FMarginsSheet, ED_ID, Value, false);
end;

function EditCtrlProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  pde: TPrintDialogEx;
  lED_ID: Longint;
  cMax, cValue: Cardinal;
begin
  pde := GetUserData(hWnd);
  if (pde <> nil) and (uMsg = WM_KEYDOWN) and (wParam in [VK_UP, VK_DOWN]) then with pde do begin
    Result := 0;
    lED_ID := GetWindowLong(hWnd, GWL_ID);
    cMax := GetMaxMargin(lED_ID);
    cValue := GetMargin(lED_ID);
    case wParam of
      VK_UP: begin
	if cValue < cMax then
	  SetMargin(lED_ID, succ(cValue))
	else Beep;
      end;
      VK_DOWN: begin
	if cValue > 0 then
	  SetMargin(lED_ID, pred(cValue))
	else Beep;
      end;
    end;
  end
  else Result := CallWindowProc(GetWndProc(hWnd), hWnd, uMsg, wParam, lParam);
  if uMsg = WM_DESTROY then RemoveWndProcProp(hWnd);
end;

procedure TPrintDialogEx.InitMarginControl(ED_ID: Integer);
var
  hEdit: THandle;
begin
  hEdit := GetDlgItem(FMarginsSheet, ED_ID);
  case ED_ID of
    EDIT_HORZ: begin
      SetMargin(ED_ID, FPrinterData.Margins.Left);
      SetUpDnRange(UPDN_HORZ);
    end;
    EDIT_VERT: begin
      SetMargin(ED_ID, FPrinterData.Margins.Top);
      SetUpDnRange(UPDN_VERT);
    end;
  else
    Exit;
  end;
  if GetWndProc(hEdit) = nil then begin
    SetWndProcProp(hEdit);
    SetWndProc(hEdit, @EditCtrlProc);
    SetUserData(hEdit, Longint(GetUserData(FMarginsSheet)));
  end;
end;

procedure TPrintDialogEx.InitMarginControls;
begin
  with FPrinterData.PrefsCaptions do begin
    SetControlText(GROUP_MARGINS, MarginsGroup);
    SetControlText(TEXT_HORZ, HMargins);
    SetControlText(TEXT_VERT, VMargins);
  end;
  InitMarginControl(EDIT_HORZ);
  InitMarginControl(EDIT_VERT);
end;

function TPrintDialogEx.AddItem(ComboBox: THandle; const Item: String): Integer;
begin
  Result := SendMessage(ComboBox, CB_ADDSTRING, 0, Longint(Item));
end;

function TPrintDialogEx.GetItemCount(ComboBox: THandle): Integer;
begin
  Result := SendMessage(ComboBox, CB_GETCOUNT, 0, 0);
end;

function TPrintDialogEx.GetSelectedIndex(ComboBox: THandle): Integer;
begin
  Result := SendMessage(ComboBox, CB_GETCURSEL, 0, 0);
end;

procedure TPrintDialogEx.SetSelectedIndex(ComboBox: THandle; Index: Integer);
begin
  SendMessage(ComboBox, CB_SETCURSEL, Index, 0);
end;

function TPrintDialogEx.FindItem(ComboBox: THandle; const Item: String): Integer;
begin
  Result := SendMessage(ComboBox, CB_FINDSTRINGEXACT, -1, Longint(Item));
end;

function TPrintDialogEx.GetIndexedItem(ComboBox: THandle; Index: Integer): String;
begin
  if (Index > -1) and (Index < GetItemCount(ComboBox)) then begin
    SetString(Result, nil, SendMessage(ComboBox, CB_GETLBTEXTLEN, Index, 0));
    Delete(Result, succ(SendMessage(ComboBox, CB_GETLBTEXT, Index, Longint(Result))), MAXINT);
  end
  else Result := E;
end;

function TPrintDialogEx.GetSelectedItem(ComboBox: THandle): String;
begin
  Result := GetIndexedItem(ComboBox, GetSelectedIndex(ComboBox));
end;

procedure TPrintDialogEx.SetSelectedItem(ComboBox: THandle; const Item: String);
begin
  SetSelectedIndex(ComboBox, FindItem(ComboBox, Item));
end;

procedure TPrintDialogEx.InitEnumFontParams(var EnumFontParams: TEnumFontParams);

  function GetDefaultFont(ScreenResolution: Integer): TFontNameSize;
  var
    hFont: THandle;
    lfDefault: TLogFont;
  begin
    with Result do begin
      hFont := GetStockObject(ANSI_VAR_FONT);
      if (hFont <> 0) and (GetObject(hFont, SizeOf(TLogFont), @lfDefault) <> 0) then with lfDefault do begin
	Name := lfFaceName;
	Size := MulDiv(lfHeight, 72, ScreenResolution);
      end
      else begin
	Name := E;
	Size := 0;
      end;
    end;
  end;

begin
  with EnumFontParams do begin
    PDE := Self;
    ScreenDC := GetDC(0);
    ScreenResolution := GetDeviceCaps(ScreenDC, LOGPIXELSY);
    DefaultFont := GetDefaultFont(ScreenResolution);
    ItemCount := 0;
  end;
end;

procedure TPrintDialogEx.GetFontNames(ComboBox: THandle; EnumFontParams: PEnumFontParams);

  function EnumFontsProc(const ELFE: TEnumLogFontEx; const TM: TTextMetric;
			 FontType: Cardinal; var EnumFontParams: TEnumFontParams): Integer; stdcall;
  begin
    Result := 1;
    with ELFE.elfLogFont, EnumFontParams do begin
      if PDE.FindItem(ComboBox, lfFaceName) = -1 then begin
	PDE.AddItem(ComboBox, lfFaceName);
	inc(ItemCount);
      end;
    end;
  end;

var
  lfEnum: TLogFont;
  efpLocal: TEnumFontParams;
  iPreselectIndex: Integer;
begin
  FillChar(lfEnum, SizeOf(TLogFont), 0);
  lfEnum.lfCharSet := DEFAULT_CHARSET;
  if EnumFontParams = nil then begin
    efpLocal.Preselect.Name := GetSelectedItem(ComboBox);
    efpLocal.ComboBox := ComboBox;
    InitEnumFontParams(efpLocal);
    EnumFontParams := @efpLocal;
  end;
  with EnumFontParams^ do begin
    try
      SendMessage(ComboBox, CB_RESETCONTENT, 0, 0);
      EnumFontFamiliesEx(ScreenDC, lfEnum, @EnumFontsProc, Longint(EnumFontParams), 0);
    finally
      ReleaseDC(WindowFromDC(ScreenDC), ScreenDC);
    end;
    if ItemCount > 0 then begin
      iPreselectIndex := FindItem(ComboBox, Preselect.Name);
      if iPreselectIndex = -1 then iPreselectIndex := FindItem(ComboBox, DefaultFont.Name);
      SetSelectedIndex(ComboBox, iPreselectIndex);
    end;
  end;
end;

procedure TPrintDialogEx.GetFontSizes(ComboBox: THandle; EnumFontParams: PEnumFontParams);

  function EnumSizesProc(const ELFE: TEnumLogFontEx; const TM: TTextMetric;
			 FontType: Cardinal; var EnumFontParams: TEnumFontParams): Integer; stdcall;
  const
    ATT_SIZES: Array[0..17] of String = ('6', '8', '9', '10', '11', '12', '14', '16', '18',
					 '20', '22', '24', '26', '28', '36', '48', '60', '72');
  var
    iIndex: Integer;
    sPoints: String;
  begin
    with EnumFontParams do begin
      if (FontType and RASTER_FONTTYPE) = 0 then begin
	Result := 0;
	for iIndex := 0 to High(ATT_SIZES) do begin
	  PDE.AddItem(ComboBox, ATT_SIZES[iIndex]);
	  inc(ItemCount);
	end;
      end
      else begin
	Result := 1;
	with TM do sPoints := IntToStr(MulDiv(tmHeight - tmInternalLeading, 72, ScreenResolution));
	if PDE.FindItem(ComboBox, sPoints) = -1 then begin
	  PDE.AddItem(ComboBox, sPoints);
	  inc(ItemCount);
	end;
      end;
    end;
  end;

var
  lfEnum: TLogFont;
  efpLocal: TEnumFontParams;
  iPreselectIndex: Integer;
begin
  FillChar(lfEnum, SizeOf(TLogFont), 0);
  lfEnum.lfCharSet := DEFAULT_CHARSET;
  StrPLCopy(lfEnum.lfFaceName, GetSelectedItem(GetDlgItem(FMarginsSheet, CBX_NAME)), LF_FACESIZE);
  if EnumFontParams = nil then begin
    efpLocal.Preselect.Size := StrToIntDef(GetSelectedItem(ComboBox), efpLocal.DefaultFont.Size);
    efpLocal.ComboBox := ComboBox;
    InitEnumFontParams(efpLocal);
    EnumFontParams := @efpLocal;
  end;
  with EnumFontParams^ do begin
    try
      SendMessage(ComboBox, CB_RESETCONTENT, 0, 0);
      EnumFontFamiliesEx(EnumFontParams^.ScreenDC, lfEnum, @EnumSizesProc, Longint(EnumFontParams), 0);
    finally
      ReleaseDC(WindowFromDC(ScreenDC), ScreenDC);
    end;
    if ItemCount > 0 then begin
      iPreselectIndex := FindItem(ComboBox, IntToStr(Preselect.Size));
      if iPreselectIndex = -1 then for iPreselectIndex := 0 to pred(ItemCount) do begin
	if StrToIntDef(GetIndexedItem(ComboBox, iPreselectIndex), 0) >= Preselect.Size then Break;
      end;
      SetSelectedIndex(ComboBox, iPreselectIndex);
    end;
  end;
end;

procedure TPrintDialogEx.ResizeFontControl(CBX_ID: Integer);
var
  hComboBox: THandle;
  rControl: TRect;
begin
  hComboBox := GetDlgItem(FMarginsSheet, CBX_ID);
  GetWindowRect(hComboBox, rControl);
  with rControl do begin
    dec(Right, Left);
    dec(Bottom, Top);
    inc(Bottom, SendMessage(hComboBox, CB_GETITEMHEIGHT, 0, 0) shl 3 + 2);
    SetWindowPos(hComboBox, 0, 0, 0, Right, Bottom, SWP_NOACTIVATE or SWP_HIDEWINDOW or SWP_NOMOVE or
		 SWP_NOREDRAW or SWP_NOZORDER);
    SetWindowPos(hComboBox, 0, 0, 0, 0, 0, SWP_NOACTIVATE or SWP_SHOWWINDOW or SWP_NOMOVE or
		 SWP_NOREDRAW or SWP_NOSIZE or SWP_NOZORDER);
  end;
end;

procedure TPrintDialogEx.InitFontControl(CBX_ID: Integer);
var
  EnumFontParams: TEnumFontParams;
begin
  with EnumFontParams do begin
    ComboBox := GetDlgItem(FMarginsSheet, CBX_ID);
    Preselect.Name := FPrinterData.PrinterFontName;
    Preselect.Size := FPrinterData.PrinterFontSize;
    InitEnumFontParams(EnumFontParams);
    case CBX_ID of
      CBX_NAME: GetFontNames(ComboBox, @EnumFontParams);
      CBX_SIZE: GetFontSizes(ComboBox, @EnumFontParams);
    end;
  end;
end;

procedure TPrintDialogEx.InitFontControls;
begin
  with FPrinterData.PrefsCaptions do begin
    SetControlText(GROUP_FONT, FontGroup);
    SetControlText(TEXT_NAME, FontName);
    SetControlText(TEXT_SIZE, FontSize);
  end;
  InitFontControl(CBX_NAME);
  InitFontControl(CBX_SIZE);
  ResizeFontControl(CBX_NAME);
  ResizeFontControl(CBX_SIZE);
end;

procedure TPrintDialogEx.InitCheckBox(CB_ID: Integer);
var
  sText: String;
  fCheck: Longbool;
begin
  with FPrinterData do begin
    case CB_ID of
      CB_HEADER: begin
	sText := PrefsCaptions.PrintHeader;
	fCheck := moPrintHeader in MarginOptions;
      end;
      CB_FOOTER: begin
	sText := PrefsCaptions.PrintFooter;
	fCheck := moPrintFooter in MarginOptions;
      end;
      CB_CUSTOM: begin
	sText := PrefsCaptions.CustomOption;
	fCheck := CustomOption;
      end;
    else
      Exit;
    end;
  end;
  if sText <> E then begin
    SetControlText(CB_ID, sText);
    SetButtonState(CB_ID, fCheck);
  end;
end;

procedure TPrintDialogEx.InitHeadFootControls;
begin
  with FPrinterData.PrefsCaptions do begin
    if HeadFootGroup <> E then begin
      SetControlText(GROUP_HEAD_FOOT, HeadFootGroup);
      InitCheckBox(CB_HEADER);
      InitCheckBox(CB_FOOTER);
    end
    else begin
      ShowControl(GROUP_HEAD_FOOT, false);
      ShowControl(CB_HEADER, false);
      ShowControl(CB_FOOTER, false);
    end;
  end;
end;

procedure TPrintDialogEx.OnInitDialogSheet;
begin
  InitOrientationControls;
  FPDCallback.OnPrinterSelectionChange;
  InitHeadFootControls;
  if FPrinterData.PrefsCaptions.CustomOption <> E then
    InitCheckBox(CB_CUSTOM)
  else ShowControl(CB_CUSTOM, false);
end;

procedure TPrintDialogEx.OnNotifyDialogSheet(const NMHdr: TNMHdr);
const
  AOrientations: Array[DMORIENT_PORTRAIT..DMORIENT_LANDSCAPE] of TPrinterOrientation = (poPortrait, poLandscape);
begin
  if NMHdr.code = PSN_SETACTIVE then begin
    FPrinterData.Orientation := AOrientations[FPDCallback.GetCurrentDevMode.dmOrientation];
    SetOrientation;
  end;
end;

procedure TPrintDialogEx.OnOptionsChangeCommand(hControl: THandle; CTL_ID: Integer);

  function EnableApplyButton(hChild: THandle; Enable: Longbool): BOOL; stdcall;
  begin
    Result := true;
    if not IsWindowEnabled(hChild) and (IsPrintDialog(GetParent(hChild)) <> 0)
    and (ClassString(hChild) = 'BUTTON') and (GetWindowLong(hChild, GWL_STYLE) and $F = BS_PUSHBUTTON) then begin
      EnableWindow(hChild, Enable);
      Result := false;
    end;
  end;

begin
  with FPrinterData do begin
    case CTL_ID of
      RB_PORTRAIT: if GetButtonState(RB_PORTRAIT) then Orientation := poPortrait;
      RB_LANDSCAPE: if GetButtonState(RB_LANDSCAPE) then Orientation := poLandscape;
      EDIT_HORZ: with Margins do begin
	Left := GetMargin(CTL_ID);
	Right := Left;
      end;
      EDIT_VERT: with Margins do begin
	Top := GetMargin(CTL_ID);
	Bottom := Top;
      end;
      CBX_NAME: begin
	StrPLCopy(PrinterFontName, GetSelectedItem(hControl), pred(SizeOf(TFontName)));
	GetFontSizes(GetDlgItem(FMarginsSheet, CBX_SIZE), nil);
      end;
      CBX_SIZE: PrinterFontSize := StrToIntDef(GetSelectedItem(hControl), 0);
      CB_HEADER: if GetButtonState(CTL_ID) then
	Include(MarginOptions, moPrintHeader)
      else Exclude(MarginOptions, moPrintHeader);
      CB_FOOTER: if GetButtonState(CTL_ID) then
	Include(MarginOptions, moPrintFooter)
      else Exclude(MarginOptions, moPrintFooter);
      CB_CUSTOM: CustomOption := GetButtonState(CTL_ID);
    else
      Exit;
    end;
    EnumChildWindows(GetParent(FMarginsSheet), @EnableApplyButton, ord(true));
  end;
end;


// ExecutePrintDialog ----------------------------------------------------------

function ExecutePrintDialog(var PrinterData: TPrinterData): Longbool;
const
  hOwner: HWND = 0;
  hCallWndHook: HHOOK = 0;

  function CallWndRetProc(nCode: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
  begin
    if nCode = HC_ACTION then with PCWPRetStruct(lParam)^ do begin
      Result := 0;
      case Message of
	WM_SHOWWINDOW: if (wParam <> 0) and IsDialog(hWnd) then begin
	  if IsPrintDialog(hWnd) <> 0 then
	    CenterDialog(hWnd, hOwner)
	  else CenterDialog(hWnd, GetWindow(hWnd, GW_OWNER));
	end;
      end;
    end
    else Result := CallNextHookEx(hCallWndHook, nCode, wParam, lParam);
  end;

var
  pde: TPrintDialogEx;
begin
  Result := false;
  pde := TPrintDialogEx.Create(nil);
  if Assigned(pde) then begin
    hOwner := PrinterData.OwnerWindow;
    hCallWndHook := SetWindowsHookEx(WH_CALLWNDPROCRET, @CallWndRetProc, 0, GetCurrentThreadID);
    try
      pde.PrinterData := PrinterData;
      Result := pde.Execute;
      if pde.PrinterData.ApplySettings then PrinterData.PrinterPrefs := pde.PrinterData.PrinterPrefs;
    finally
      UnhookWindowsHookEx(hCallWndHook);
      pde.Free;
    end;
  end;
end;


end.

