{$J+,Z4}
unit ProBarEx;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ComCtrls, UxThemes;

type
  TProgressBarEx = class(TProgressBar)
  private
    FLastValue: Longint;
    FLongMax: Longint;
    FPercent: String;
    FOnProgress: TNotifyEvent;
    procedure ProgressNotify;
    procedure SetMax(Value: Longint);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Reset;
    procedure StepBy(Value: Longint);
    procedure StepIt;
  published
    property  LongMax: Longint read FLongMax write SetMax Default 100;
    property  Percent: String read FPercent;
    property  OnProgress: TNotifyEvent read FOnProgress write FOnProgress;
  end;

procedure Register;


implementation

procedure TProgressBarEx.ProgressNotify;
begin
  if Assigned(FOnProgress) then FOnProgress(Self);
end;

procedure TProgressBarEx.SetMax(Value: Longint);
begin
  if Value <> FLongMax then begin
    FLongMax := Value;
    Max := 100;
  end;
end;

procedure TProgressBarEx.CMColorChanged(var Message: TMessage);
begin
  inherited;
  RecreateWnd;
  EnableThemes(WindowHandle, (Color = clBtnFace));
end;

constructor TProgressBarEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLongMax := 100;
end;

destructor TProgressBarEx.Destroy;
begin
  inherited Destroy;
end;

procedure TProgressBarEx.Reset;
begin
  FLastValue := 0;
  FPercent := '0';
  Position := 0;
  ProgressNotify;
end;

procedure TProgressBarEx.StepBy(Value: Longint);
var
  LongPos: Integer;
begin
  FLastValue := FLastValue + Value;
  LongPos := Round(FLastValue * 100 / FLongMax);
  if LongPos > Position then begin
    Position := LongPos;
    FPercent := IntToStr(Position);
    ProgressNotify;
  end;
end;

procedure TProgressBarEx.StepIt;
var
  LongPos: Integer;
begin
  inc(FLastValue, Step);
  LongPos := Round(FLastValue * 100 / FLongMax);
  if LongPos > Position then begin
    Position := LongPos;
    FPercent := IntToStr(Position);
    ProgressNotify;
  end;
end;

procedure Register;
begin
  RegisterComponents('Modified', [TProgressBarEx]);
end;

end.

