{$J+,Z4}
unit RichEd20;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, RichEdit, Classes, Graphics, GdiPlus, SysUtils, StrUtil, WinInet, WindowUtils;

const // URLs
  DS = SL + SL;
  PROT_ABOUT = 'about:';
  PROT_CID   = 'cid:';
  PROT_DATA  = 'data:';
  PROT_FILE  = 'file:';
  PROT_MAIL  = 'mailto:';
  PROT_RES   = 'res:';

const // URLs
  PROT_A_DS  = PROT_ABOUT + DS;
  PROT_C_DS  = PROT_CID + DS;
  PROT_D_DS  = PROT_DATA + DS;
  PROT_M_DS  = PROT_MAIL + DS;
  PROT_F_DS  = PROT_FILE + DS;
  PROT_R_DS  = PROT_RES + DS;

const // UTF-8
  BOM_STR = #$EF#$BB#$BF;
  BOM_NUM = $BFBBEF;
  BOM_SIZE = 3;

const // UTF-8
  UTF8_FACTOR = 4;

const // Ansi
  aNUL = #00;
  aCR = #13;
  aLF = #10;
  aSP = #32;
  aCRLF = aCR + aLF;
  aPARA = aCRLF + aCRLF;

const // Wide
  wNUL = #00#00;
  wCR = #13#00;
  wLF = #10#00;
  wSP = #32#00;
  wCRLF = wCR + wLF;
  wPARA = wCRLF + wCRLF;

  // from WinNLS.h
  CP_UTF7		= 65000;	// UTF-7 translation
  CP_UTF8		= 65001;	// UTF-8 translation

  MB_PRECOMPOSED	= $00000001;	// use precomposed chars
  MB_COMPOSITE		= $00000002;	// use composite chars
  MB_USEGLYPHCHARS	= $00000004;	// use glyph chars, not ctrl chars
  MB_ERR_INVALID_CHARS	= $00000008;	// error for invalid chars

  WC_COMPOSITECHECK	= $00000200;	// convert composite to precomposed
  WC_DISCARDNS		= $00000010;	// discard non-spacing chars
  WC_SEPCHARS		= $00000020;	// generate separate chars
  WC_DEFAULTCHAR	= $00000040;	// replace w/ default char

const
  TWIPS = 1440;

  CP_UNICODE		= 1200;

  RTF_MARKER		= '{\rtf';

  // coming with RichEdit v2.0 resp. v3.0 or later
  LIB_MSFTEDIT		= 'MSFTEdit.dll';
  LIB_RICHED20		= 'RichEd20.dll';
  LIB_RICHED10		= 'Riched32.dll';
  RICHEDIT50W		= 'RichEdit50W';
  RICHEDIT20W		= 'RichEdit20W';
  RICHEDIT20A		= 'RichEdit20A';
  RICHEDIT		= 'RICHEDIT';

  // editor messages
  EM_REDO		= WM_USER + 84;	// wParam: unused zero; lParam: unused zero
  EM_CANREDO		= WM_USER + 85;	// wParam: unused zero; lParam: unused zero
  EM_AUTOURLDETECT	= WM_USER + 91;	// wParam: 1 to enable, 0 to disable URL detection; lParam: unused zero
  EM_GETAUTOURLDETECT	= WM_USER + 92;	// wParam: unused zero; lParam: unused zero
  EM_GETTEXTEX		= WM_USER + 94;	// wParam: pointer to GETTEXTEX; lParam: pointer to the text buffer
  EM_GETTEXTLENGTHEX	= WM_USER + 95;	// wParam: pointer to GETTEXTLENGTHEX; lParam: unused zero
  EM_SETTEXTEX		= WM_USER + 97;	// wParam: pointer to SETTEXTEX; lParam: pointer to null-terminated text
  EM_FINDTEXTW		= WM_USER + 123;// wParam: search parameters (the FR_XXX values); lParam: pointer to FINDTEXTW
  EM_FINDTEXTEXW	= WM_USER + 124;// wParam: search parameters (the FR_XXX values); lParam: pointer to FINDTEXTEXW

  // extended edit style specific messages
  EM_SETEDITSTYLE       = WM_USER + 204;// wParam & lParam taking the same SES_XXX parameters, the latter working as selector
  EM_GETEDITSTYLE       = WM_USER + 205;// wParam & lParam set to zero, returning the SES_XXX values

  // outline mode message
  EM_OUTLINE		= (WM_USER + 220);

  // message for getting and restoring scroll pos
  EM_GETSCROLLPOS 	= (WM_USER + 221);
  EM_SETSCROLLPOS 	= (WM_USER + 222);

  // change fontsize in current selection by wParam
  EM_SETFONTSIZE	= (WM_USER + 223);
  EM_GETZOOM		= (WM_USER + 224);
  EM_SETZOOM		= (WM_USER + 225);
  EM_GETVIEWKIND	= (WM_USER + 226);
  EM_SETVIEWKIND	= (WM_USER + 227);

  // parent notification codes
  EN_OLEOPFAILED	= $0709;
  EN_OBJECTPOSITIONS	= $070A;
  EN_LINK		= $070B;
  EN_DRAGDROPDONE	= $070C;
  EN_PARAGRAPHEXPANDED	= $070D;
  EN_ALIGNLTR		= $0710;
  EN_ALIGNRTL		= $0711;

  // flags for sending notifications
  ENM_SCROLLEVENTS	= $00000008;
  ENM_DRAGDROPDONE	= $00000010;
  ENM_PARAGRAPHEXPANDED	= $00000020;
  ENM_LANGCHANGE	= $01000000;
  ENM_OBJECTPOSITIONS	= $02000000;
  ENM_LINK		= $04000000;

  // dwMask flags for CHARFORMAT data structure
  CFM_BOLD		= $00000001;
  CFM_ITALIC		= $00000002;
  CFM_UNDERLINE		= $00000004;
  CFM_STRIKEOUT		= $00000008;

  CFM_PROTECTED		= $00000010;
  CFM_LINK		= $00000020;

  CFM_SIZE		= $80000000;
  CFM_COLOR		= $40000000;
  CFM_FACE		= $20000000;
  CFM_OFFSET		= $10000000;

  CFM_CHARSET		= $08000000;

  CFM_SMALLCAPS		= $00000040;
  CFM_ALLCAPS		= $00000080;

  CFM_HIDDEN		= $00000100;
  CFM_OUTLINE		= $00000200;
  CFM_SHADOW		= $00000400;
  CFM_EMBOSS		= $00000800;

  CFM_IMPRINT		= $00001000;
  CFM_DISABLED		= $00002000;
  CFM_REVISED		= $00004000;

  CFM_BACKCOLOR		= $04000000;
  CFM_LCID		= $02000000;

  CFM_UNDERLINETYPE	= $00800000;
  CFM_WEIGHT		= $00400000;
  CFM_SPACING		= $00200000;
  CFM_KERNING		= $00100000;

  CFM_STYLE		= $00080000;
  CFM_ANIMATION		= $00040000;

  CFM_REVAUTHOR		= $00008000;

  // dwEffects flags for CHARFORMAT data structure
  CFE_BOLD		= $00000001;
  CFE_ITALIC		= $00000002;
  CFE_UNDERLINE		= $00000004;
  CFE_STRIKEOUT		= $00000008;

  CFE_PROTECTED		= $00000010;
  CFE_LINK		= $00000020;

  CFE_AUTOCOLOR		= $40000000;

  CFE_SMALLCAPS		= CFM_SMALLCAPS;
  CFE_ALLCAPS		= CFM_ALLCAPS;

  CFE_HIDDEN		= CFM_HIDDEN;
  CFE_OUTLINE		= CFM_OUTLINE;
  CFE_SHADOW		= CFM_SHADOW;
  CFE_EMBOSS		= CFM_EMBOSS;

  CFE_IMPRINT		= CFM_IMPRINT;
  CFE_DISABLED		= CFM_DISABLED;
  CFE_REVISED		= CFM_REVISED;

  CFE_AUTOBACKCOLOR	= CFM_BACKCOLOR;

  // flags for the GETEXTLENGTHEX data structure
  GTL_DEFAULT		= 0;		// do the default (return # of chars)
  GTL_USECRLF		= 1;		// compute answer using CRLFs for paragraphs
  GTL_PRECISE		= 2;		// compute a precise answer
  GTL_CLOSE		= 4;		// fast computation of a "close" answer
  GTL_NUMCHARS		= 8;		// return the number of characters
  GTL_NUMBYTES		= 16;		// return the number of _bytes_

  // flags for the GETEXTEX data structure
  GT_DEFAULT		= 0;		// no CR translation
  GT_USECRLF		= 1;		// each CR should be translated into a CRLF
  GT_SELECTION		= 2;		// retrieves the text for the current selection

  // flags for the SETEXTEX data structure
  ST_DEFAULT		= 0;		// deletes the undo stack, discards rich-text formatting, replaces all text
  ST_KEEPUNDO		= 1;		// keeps the undo stack
  ST_SELECTION		= 2;		// replaces selection and keeps rich-text formatting

  // search option flags
  FT_DOWN		= 1;
  FT_WHOLEWORD		= 2;
  FT_MATCHCASE		= 4;

  // EM_GETCHARFORMAT & EM_SETCHARFORMAT wParam masks
  SCF_DEFAULT		= $0000;
  SCF_SELECTION		= $0001;
  SCF_WORD		= $0002;	// only valid in conjunction SCF_SELECTION
  SCF_ALL		= $0004;	// not valid with SCF_SELECTION or SCF_WORD

  // stream formats
  SF_UNICODE		= $0010;	// Unicode file (UCS2 little endian)
  SF_USECODEPAGE	= $0020;	// CodePage given by high word

  // extended edit style masks
  SES_EMULATESYSEDIT     = $00000001;
  SES_BEEPONMAXTEXT      = $00000002;
  SES_EXTENDBACKCOLOR    = $00000004;
  SES_MAPCPS             = $00000008;   // (obsolete)
  SES_EMULATE10          = $00000010;   // (obsolete)
  SES_USECRLF            = $00000020;   // (obsolete)
  SES_NOXLTSYMBOLRANGE   = $00000020;
  SES_USEAIMM            = $00000040;
  SES_NOIME              = $00000080;

  SES_ALLOWBEEPS         = $00000100;
  SES_UPPERCASE          = $00000200;
  SES_LOWERCASE          = $00000400;
  SES_NOINPUTSEQUENCECHK = $00000800;
  SES_BIDI               = $00001000;
  SES_SCROLLONKILLFOCUS  = $00002000;
  SES_XLTCRCRLFTOCR      = $00004000;
  SES_DRAFTMODE          = $00008000;

  SES_USECTF             = $00010000;
  SES_HIDEGRIDLINES      = $00020000;
  SES_USEATFONT          = $00040000;
  SES_CUSTOMLOOK         = $00080000;
  SES_LBSCROLLNOTIFY     = $00100000;
  SES_CTFALLOWEMBED      = $00200000;
  SES_CTFALLOWSMARTTAG   = $00400000;
  SES_CTFALLOWPROOFING   = $00800000;

type // fixes DWord declarations
  TCharFormat = Record
    cbSize: UINT;
    dwMask: DWord;
    dwEffects: DWord;
    yHeight: Longint;
    yOffset: Longint;
    crTextColor: TColorRef;
    bCharSet: Byte;
    bPitchAndFamily: Byte;
    szFaceName: Array[0..LF_FACESIZE - 1] of Char;
  end;

type
  TCharFormat2 = Record
    cbSize: UINT;
    dwMask: DWord;
    dwEffects: DWord;
    yHeight: Longint;
    yOffset: Longint;
    crTextColor: TColorRef;
    bCharSet: Byte;
    bPitchAndFamily: Byte;
    szFaceName: Array[0..LF_FACESIZE - 1] of Char;
    wWeight: Word;
    sSpacing: Smallint;
    crBackColor: TColorRef;
    lid: LCID;
    dwReserved: DWord;
    sStyle: Smallint;
    wKerning: Word;
    bUnderlineType: Byte;
    bAnimation: Byte;
    bRevAuthor: Byte;
    bReserved1: Byte;
  end;

type
  TRtfFont = Record
    Name: String;
    Size: Integer;
    Height: Integer;
    Styles: Cardinal;
  end;

  // used by EN_LINK notification message
  TENLink = Record
    nmhdr: TNMHDR;
    msg: UINT;
    wParam: WPARAM;
    lParam: LPARAM;
    chrg: TCharRange;
  end;
  PENLink = ^TENLink;

  // EM_GETTEXTLENGTHEX info; this struct is passed in the wparam of the message
  TGetTextLengthEx = Record
    flags: DWORD;			// flags (see GTL_XXX defines)
    codepage: UINT;			// code page for translation (CP_ACP for default, 1200 for Unicode)
  end;

  // EM_GETTEXTEX info; this struct is passed in the wparam of the message
  TGetTextEx = Record
    cb: DWORD;				// size of the buffer provided in lParam
    flags: DWORD;			// flags (see the GT_XXX defines)
    codepage: UINT;			// code page for translation (CP_ACP for sys default, 1200 for Unicode)
    lpDefaultChar: LPCSTR;		// replacement for unmappable chars
    lpUsedDefChar: PBOOL;		// pointer to flag set when def char used
  end;

  // EM_SETTEXTEX info; this struct is passed in the wparam of the message
  TSetTextEx = Record
    flags: DWORD;			// flags (see the ST_XXX defines)
    codepage: UINT;			// code page for translation (CP_ACP for sys default, 1200 for Unicode)
  end;

  // EM_FINDTEXTA info; this struct is passed in the lparam of the message
  TFindTextA = Record
    chrg: TCharRange;			// a CHARRANGE structure that specifies the range of characters to search
    lpstrText: PChar;		// a pointer to the null-terminated string to find
  end;

  // EM_FINDTEXTW info; this struct is passed in the lparam of the message
  TFindTextW = Record
    chrg: TCharRange;			// a CHARRANGE structure that specifies the range of characters to search
    lpstrText: PWideChar;		// a pointer to the null-terminated string to find
  end;

  // EM_FINDTEXTEXW info; this struct is passed in the lparam of the message
  TFindTextExW = record
    chrg: TCharRange;			// a CHARRANGE structure that specifies the range of characters to search
    lpstrText: PWideChar;		// a pointer to the null-terminated string to find
    chrgText: TCharRange;		// a CHARRANGE structure that specifies the range of characters in which text is found
  end;

// EM_GETTEXTRANGEA info; this struct is passed in the lparam of the message
  TTextRangeA = record
    chrg: TCharRange;			// a CHARRANGE structure that specifies the range of characters to retrieve
    lpstrText: PChar;		// pointer to buffer that receives the text
  end;

  // EM_GETTEXTRANGEW info; this struct is passed in the lparam of the message
  TTextRangeW = record
    chrg: TCharRange;			// a CHARRANGE structure that specifies the range of characters to retrieve
    lpstrText: PWideChar;		// pointer to buffer that receives the text
  end;

const
  CFM_EFFECTS = CFM_BOLD or CFM_ITALIC or CFM_UNDERLINE or CFM_STRIKEOUT;
  CFM_FONT = CFM_BOLD or CFM_ITALIC or CFM_UNDERLINE or CFM_STRIKEOUT or CFM_SIZE or CFM_COLOR or CFM_FACE;


type
  TStringStream = class(TMemoryStream)
  private
    FMemoryString: String;
    function  GetCapacity: Longint;
    procedure SetCapacity(Value: Longint);
  protected
    function  Realloc(var NewCapacity: Longint): Pointer; override;
  public
    constructor Create(const Str: String);
    destructor Destroy; override;
    function  ReadAndClear: String;
    property  Capacity: Longint read GetCapacity write SetCapacity;
    property  MemoryString: String read FMemoryString;
  end;


type
  TRtfTxtConverter = Class
  private
    FRtfWindow: THandle;
  public
    constructor Create;
    destructor Destroy; override;
    function RtfToText(const RtfText: String; var ReturnUnicode: Longbool): String;
    function TextToRtf(const PlainText: String; const RtfFont: TRtfFont; Utf8: Longbool): String;
  end;


function  OffScreenPos: TPoint;
function  MakePoints(Pos: DWord): TPoint;
function  CountChars(Find: Char; const Text: String): Integer;
function  IntToPascalHex(Value: Integer): String;
function  IsWideString(const WideString: String): Longbool;
procedure SetWSString(var WideString: String; NewLength: Integer);
function  WCPos(WC: WideChar; const WideString: String): Integer;
function  WSChar(const WideString: String; Pos: Integer): WideChar;
function  WSConcat(const WideStrings: Array of String): String;
function  WSCopy(const WideString: String; Pos, Len: Integer): String;
procedure WSDelete(var WideString: String; Index, Count: Integer);
function  WSCompareStr(const WideStr1, WideStr2: String): Boolean;
function  WSCompareTxt(const WideStr1, WideStr2: String): Boolean;
procedure WSInsert(const Source: String; var Target: String; Index: Integer);
function  WSLength(const WideString: String): Integer;
function  WSPos(const Find, Dest: String): Integer;
function  AnsiToWideString(const Ansi: String): String;
function  WideStringToAnsi(const Wide: String): String;
function  Utf8ToWideString(const Utf8: String): String;
function  WideStringToUtf8(const Wide: String): String;
function  AnsiToUtf8String(const Ansi: String): String;
function  Utf8StringToAnsi(const Utf8: String): String;
function  NoBomUtf8ToWideString(const Utf8: String): String;
function  NoBomWideStringToUtf8(const Wide: String): String;
function  NoBomUtf8ToAnsiString(const Utf8: String): String;
function  NoBomAnsiStringToUtf8(const Ansi: String): String;
function  GetRichEditVersion: Longint;
function  IsBrowserUrl(const Url: String): Longbool;
function  IsAboutUrl(const Url: String): Longbool;
function  IsCIDUrl(const Url: String): Longbool;
function  IsDataUrl(const Url: String): Longbool;
function  IsFileUrl(const Url: String): Longbool;
function  IsMailToUrl(const Url: String): Longbool;
function  IsResUrl(const Url: String): Longbool;
function  GetFilePathFromUrl(const Url: String): String;
function  IsRichText(const Text: String): Longbool;
function  RtfUnescape(const Text: String): String;
function  GetImageDataAsRtf(const Data: String; AsBitmap: Longbool): String;
function  CreateEmbeddedRtfVersion(const Text, DefaultBitmap: String; ImageList: TStringList): String;
function  GetRtfSectionEnd(const Text: String; Pos, Max: Integer): Integer;
function  GetRtfTextHeight(RichEdit: THandle; Version, FontSize, VMargins: Integer): Integer;
function  GetRtfTextLength(RichEdit: THandle; Version: Integer): Integer;
function  GetRtfText(RichEdit: THandle; Version: Integer): String;
function  CreateFontFromCharFormat(const CharFormat: TCharFormat): TFont;
function  CreateFontFromHandle(Font: THandle; const Face: String; Size: Integer; Styles: TFontStyles): THandle;
function  GetValueFromCharFormat(const CharFormat: TCharFormat; Mask: DWord): Variant;
function  GetCharFormatFromFont(const Font: TFont; const NewFace: String; NewSize: Integer; Mask: DWord): TCharFormat;
function  GetCharFormatFromLogFont(const LogFont: TLogFont; const NewName: String; NewSize: Integer; Mask: DWord): TCharFormat;
function  DecodeIdns(const Url: String): String;


function RtfToText(const RtfText: String; var ReturnUnicode: Longbool): String;
function TextToRtf(const PlainText: String; const RtfFont: TRtfFont; Utf8: Longbool): String;


implementation

uses
  RawByteStringList;

type
  DChar = Array[0..1] of Char;
  PDChar = ^DChar;
  QChar = Array[0..3] of Char;
  PQChar = ^QChar;


const // for IDN functions
  ACE = 'xn--';
  IDN = [$2D, $30..$39, $41..$5A, $61..$7A];

var
  IdnToAscii: function(dwFlags: DWord; lpUnicodeCharStr: PWideChar; cchUnicodeChar: Integer;
		       lpASCIICharStr: PWideChar; cchASCIIChar: Integer): Integer; stdcall;
  IdnToUnicode: function(dwFlags: DWord; lpASCIICharStr: PWideChar; cchASCIIChar: Integer;
			 lpUnicodeCharStr: PWideChar; cchUnicodeChar: Integer): Integer; stdcall;


const // Url handling stuff
  ICU_ENCODE_PERCENT		= $00001000;  // Encode any percent (ASCII25) signs encountered
  ICU_BROWSER_MODE		= $02000000;  // Special encode/decode rules for browser
  ICU_ENCODE_SPACES_ONLY	= $04000000;  // Encode spaces only
  ICU_NO_META			= $08000000;  // Don't convert .. etc. meta path sequences
  ICU_DECODE			= $10000000;  // Convert %XX escape sequences to characters
  ICU_NO_ENCODE			= $20000000;  // Don't convert unsafe characters to escape sequence
  ICU_USERNAME			= $40000000;  // use internal username & password
  ICU_ESCAPE			= $80000000;  // (un)escape URL characters

  INTERNET_SCHEME_PARTIAL				= -2;
  INTERNET_SCHEME_UNKNOWN				= -1;
  INTERNET_SCHEME_DEFAULT				= 00;
  INTERNET_SCHEME_FTP					= 01;
  INTERNET_SCHEME_GOPHER				= 02;
  INTERNET_SCHEME_HTTP					= 03;
  INTERNET_SCHEME_HTTPS					= 04;
  INTERNET_SCHEME_FILE					= 05;
  INTERNET_SCHEME_NEWS					= 06;
  INTERNET_SCHEME_MAILTO				= 07;
  INTERNET_SCHEME_SOCKS					= 08;
  INTERNET_SCHEME_JAVASCRIPT				= 09;
  INTERNET_SCHEME_VBSCRIPT				= 10;
  INTERNET_SCHEME_RES					= 11;
  INTERNET_SCHEME_FIRST					= INTERNET_SCHEME_FTP;
  INTERNET_SCHEME_LAST					= INTERNET_SCHEME_RES;

type
  PInternetScheme = ^TInternetScheme;
  TInternetScheme = Integer;

  PURLComponents = ^TURLComponents;
  TURLComponents = Record
    dwStructSize: DWORD;	{ size of this structure. Used in version check }
    lpszScheme: PChar;		{ pointer to scheme name }
    dwSchemeLength: DWORD;	{ length of scheme name }
    nScheme: TInternetScheme;	{ enumerated scheme type (if known) }
    lpszHostName: PChar;	{ pointer to host name }
    dwHostNameLength: DWORD;	{ length of host name }
    nPort: INTERNET_PORT;	{ converted port number }
    pad: WORD;			{ force correct allignment regardless of comp. flags}
    lpszUserName: PChar;	{ pointer to user name }
    dwUserNameLength: DWORD;	{ length of user name }
    lpszPassword: PChar;	{ pointer to password }
    dwPasswordLength: DWORD;	{ length of password }
    lpszUrlPath: PChar;		{ pointer to URL-path }
    dwUrlPathLength: DWORD;	{ length of URL-path }
    lpszExtraInfo: PChar;	{ pointer to extra information (e.g. ?foo or #foo) }
    dwExtraInfoLength: DWORD;	{ length of extra information }
  end;

var
  InternetCanonicalizeUrl: function(lpszUrl, lpszBuffer: PChar; var lpdwBufferLength: DWORD; dwFlags: DWORD): BOOL; stdcall;
  InternetCrackUrl: function(lpszUrl: PChar; dwUrlLength, dwFlags: DWORD; var lpUrlComponents: TURLComponents): BOOL; stdcall;


function InitIdnSupport: THandle;
const
  NORMALIZ = 'Normaliz.dll';
begin
  Result := GetModuleHandleA(NORMALIZ);
  if Result = 0 then Result := LoadLibraryA(NORMALIZ);
  IdnToAscii := GetProcAddress(Result, 'IdnToAscii');
  IdnToUnicode := GetProcAddress(Result, 'IdnToUnicode');
end;

function InitWinINetLib: THandle;
const
  WININET = 'WinINet.dll';
begin
  Result := GetModuleHandleA(WININET);
  if Result = 0 then Result := LoadLibraryA(WININET);
  InternetCanonicalizeUrl := GetProcAddress(Result, 'InternetCanonicalizeUrlA');
  InternetCrackUrl := GetProcAddress(Result, 'InternetCrackUrlA');
end;

function TextPos(Find, Dest: PChar): PChar; register; assembler;
asm	// EAX = Find, EDX = Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  PUSH	EBX
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  MOV	AL,[ESI]
  OR	AL,AL
  JZ	@NOT
  @LOOP:
  MOV	AH,[EDI]
  OR	AH,AH
  JE	@NOT
  XOR	AH,AL
  JE	@MATCH
  TEST	BYTE PTR [ESI],040h
  JE	@NEXT
  TEST	BYTE PTR [EDI],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	EDI
  JMP	@LOOP
  @MATCH:
  MOV	EBX,ESI
  MOV	ECX,EDI
  @CMP:
  INC	EBX
  INC	ECX
  CMP	BYTE PTR [EBX],0
  JE	@YES
  CMP	BYTE PTR [ECX],0
  JE	@NOT
  MOV	AH,[EBX]
  XOR	AH,[ECX]
  JE	@CMP
  TEST	BYTE PTR [EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [ECX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  INC	EDI
  JMP	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  RET
  @YES:
  MOV	EAX,EDI
  POP	ESI
  POP	EDI
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
end;

function PunyToUtf8(PunyStr: PChar; PunyLen: Integer; Utf8Str: PChar; Utf8Len: Integer): Integer;
var
  wsIdn, wsUnicode: String;
  iLen: Integer;
begin
  if (PunyStr <> nil) and Assigned(IdnToUnicode) then begin
    SetString(wsIdn, nil, succ(PunyLen) shl 1);
    if MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, PunyStr, PunyLen, Pointer(wsIdn), PunyLen) > 0 then begin
      iLen := IdnToUnicode(0, Pointer(wsIdn), PunyLen, nil, 0);
      if iLen > 0 then begin
	SetString(wsUnicode, nil, succ(iLen) shl 1);
	iLen := IdnToUnicode(0, Pointer(wsIdn), PunyLen, Pointer(wsUnicode), iLen);
	if iLen > 0 then begin
	  Result := WideCharToMultiByte(CP_UTF8, 0, Pointer(wsUnicode), iLen, Utf8Str, Utf8Len, nil, nil);
	  Exit;
	end;
      end;
    end;
  end;
  Move(PunyStr^, Utf8Str^, PunyLen);
  Utf8Str[PunyLen] := NULL;
  Result := PunyLen;
end;

function DecodeIdns(const Url: String): String;
var
  pResult, pMax, pUrl, pIdn: PChar;
  iLen: Integer;
begin
  if Url <> E then begin
    iLen := Length(Url) * 128;
    SetString(Result, nil, iLen);
    pResult := Pointer(Result);
    pMax := pResult + iLen;
    pUrl := Pointer(Url);
    repeat
      pIdn := TextPos(ACE, pUrl);
      if pIdn <> nil then begin
	iLen := pIdn - pUrl;
	Move(pUrl^, pResult^, iLen);
	inc(pResult, iLen);
	pUrl := pIdn + Length(ACE);
	while ord(pUrl^) in IDN do inc(pUrl);
	inc(pResult, PunyToUtf8(pIdn, pUrl - pIdn, pResult, pMax - pResult));
      end
      else begin
        iLen := Length(Url) - (pUrl - Pointer(Url));
	Move(pUrl^, pResult^, iLen);
	inc(pResult, iLen);
      end;
    until pIdn = nil;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end
  else Result := E;
end;


// Converter functions -----------------------------------------------------------------------------

function RtfToText(const RtfText: String; var ReturnUnicode: Longbool): String;
var
  rtc: TRtfTxtConverter;
begin
  rtc := TRtfTxtConverter.Create;
  if rtc <> nil then begin
    try
      Result := rtc.RtfToText(RtfText, ReturnUnicode);
    finally
      rtc.Free;
    end;
  end;
end;

function TextToRtf(const PlainText: String; const RtfFont: TRtfFont; Utf8: Longbool): String;
var
  rtc: TRtfTxtConverter;
begin
  rtc := TRtfTxtConverter.Create;
  if rtc <> nil then begin
    try
      Result := rtc.TextToRtf(PlainText, RtfFont, Utf8);
    finally
      rtc.Free;
    end;
  end;
end;


// TRftTxtConverter ------------------------------------------------------------

function GetWindowStringA(Window: HWND): String;
var
  iLen: Integer;
begin
  iLen := succ(GetWindowTextLengthA(Window));
  SetString(Result, nil, iLen);
  iLen := GetWindowTextA(Window, Pointer(Result), iLen);
  if iLen > 0 then
    Delete(Result, succ(iLen), MAXINT)
  else Result := E;
end;

function GetWindowStringW(Window: HWND): String;
var
  iLen: Integer;
begin
  iLen := succ(GetWindowTextLengthW(Window));
  SetWSString(Result, iLen);
  iLen := GetWindowTextW(Window, Pointer(Result), iLen);
  if iLen > 0 then
    WSDelete(Result, succ(iLen), MAXINT)
  else Result := E;
end;

function OffScreenPos: TPoint;
begin
  with Result do begin
    X := -1;
    Y := -1;
  end;
end;

function MakePoints(Pos: DWord): TPoint;
begin
  with Result do begin
    X := Pos and $FFFF;
    Y := Pos shr 16;
  end;
end;

function CountChars(Find: Char; const Text: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Text
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EBX
  MOV	ECX,[EDX - 04h]
  XOR	EBX,EBX
  @LOOP:
  DEC	ECX
  JL	@END
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  INC	EBX
  JMP	@LOOP
  @END:
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function IntToPascalHex(Value: Integer): String;
begin
  Result := '$' + IntToHex(Value, 8);
end;

function IsWideString(const WideString: String): Longbool;
var
  iLen: Integer;
begin
  iLen := Length(WideString);
  Result := (iLen > 1) and (WideString[iLen] = aNUL);
end;

procedure SetWSString(var WideString: String; NewLength: Integer);
begin
  SetString(WideString, nil, succ(NewLength) shl 1);
end;

function WCPos(WC: WideChar; const WideString: String): Integer; register; assembler;
asm	// AX = WC, EDX = @WideString
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EDX
  MOV	ECX,[EDX - 04h]
  SUB	ECX,2
  ADD	EDX,ECX
  NEG	ECX
  @LOOP:
  CMP	[EDX + ECX],AX
  JE	@MATCH
  ADD	ECX,2
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  POP	EDX
  MOV	EAX,[EDX - 04h]
  ADD	EAX,ECX
  SHR	EAX,1
end;

function WSChar(const WideString: String; Pos: Integer): WideChar;
begin
  Pos := pred(Pos shl 1);
  if (Pos > 0) and (Pos < Length(WideString)) then
    Result := WideChar(WideString[Pos])
  else Result := aNUL;
end;

function WSConcat(const WideStrings: Array of String): String;
var
  iLength, iIndex: Integer;
  pResult: PChar;
begin
  if High(WideStrings) > 0 then begin
    iLength := 0;
    for iIndex := 0 to High(WideStrings) do inc(iLength, Length(WideStrings[iIndex]));
    SetString(Result, nil, iLength);
    pResult := Pointer(Result);
    for iIndex := 0 to High(WideStrings) do begin
      iLength := Length(WideStrings[iIndex]) - Length(wNUL);
      if iLength > 0 then begin
	Move(Pointer(WideStrings[iIndex])^, pResult^, iLength);
	inc(pResult, iLength);
      end;
    end;
    iLength := succ(pResult - Pointer(Result));
    if iLength > 0 then begin
      Delete(Result, iLength + Length(wNUL), MAXINT);
      PWideChar(pResult)^ := aNUL;
    end
    else Result := E;
  end
  else Result := WideStrings[0];
end;

function WSCopy(const WideString: String; Pos, Len: Integer): String;
begin
  if (Len > 0) and (Pos > 0) and (Pos <= Length(WideString) shr 1) then begin
    if Len <= MAXINT shr 1 then
      Len := Len shl 1 + Length(wNUL)
    else Len := MAXINT;
    Result := Copy(WideString, pred(Pos shl 1), Len);
    Len := Length(Result) shr 1;
    if Len > 1 then
      PWideChar(Pointer(Result))[pred(Len)] := aNUL
    else Result := E;
  end
  else Result := E;
end;

procedure WSDelete(var WideString: String; Index, Count: Integer);
var
  iLen: Integer;
begin
  if (Count > 0) and (Index > 0) and (Index <= Length(WideString) shr 1) then begin
    Index := succ(Index shl 1);
    if Count <= MAXINT shr 1 then
      Count := Count shl 1
    else Count := MAXINT;
    Delete(WideString, Index, Count);
    iLen := Length(WideString) shr 1;
    if iLen > 1 then
      PWideChar(Pointer(WideString))[pred(iLen)] := aNUL
    else WideString := E;
  end;
end;

function WSCompareStr(const WideStr1, WideStr2: String): Boolean;
begin
  Result := (CompareStringW(LOCALE_USER_DEFAULT, 0, Pointer(WideStr1), WSLength(WideStr1),
			    Pointer(WideStr2), WSLength(WideStr2)) = 2);
end;

function WSCompareTxt(const WideStr1, WideStr2: String): Boolean;
begin
  Result := (CompareStringW(LOCALE_USER_DEFAULT, NORM_IGNORECASE, Pointer(WideStr1), WSLength(WideStr1),
			    Pointer(WideStr2), WSLength(WideStr2)) = 2);
end;

procedure WSInsert(const Source: String; var Target: String; Index: Integer);
var
  iSourceLen, iTargetLen, iLen: Integer;
  pSource, pTarget, pResult: PChar;
  sResult: String;
begin
  iSourceLen := Length(Source);
  iTargetLen := Length(Target);
  if (iSourceLen = 0) or (iTargetLen = 0) or (Index <= 1) then
    Target := WSConcat([Source, Target])
  else if Index > iTargetLen then
    Target := WSConcat([Target, Source])
  else begin
    pSource := Pointer(Source);
    pTarget := Pointer(Target);
    SetString(sResult, nil, iSourceLen + iTargetLen - Length(wNUL));
    pResult := Pointer(sResult);
    iLen := pred(Index) shl 1;
    Move(pTarget^, pResult^, iLen);
    inc(pResult, iLen);
    dec(iSourceLen, Length(wNUL));
    Move(pSource^, pResult^, iSourceLen);
    inc(pResult, iSourceLen);
    dec(iTargetLen, iLen);
    inc(pTarget, iLen);
    Move(pTarget^, pResult^, iTargetLen);
    Target := sResult;
  end;
end;

function WSLength(const WideString: String): Integer;
begin
  Result := pred(Length(WideString) shr 1);
end;

function WSPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  MOV	ECX,[EDX - 04h]
  SUB	ECX,4
  JB	@END
  CMP	DWORD PTR [EAX - 04h],4
  JE	@CHAR
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,ECX
  MOV	AX,[ESI]
  NEG	ECX
  @LOOP:
  CMP	[EDI + ECX],AX
  JE	@MATCH
  SUB	ECX,2
  ADD	ECX,[ESI - 04h]
  JG	@NOT
  SUB	ECX,[ESI - 04h]
  ADD	ECX,4
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  SUB	EBX,2
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  ADD	EBX,2
  JE	@YES
  PUSH	EAX
  MOV	AX,[EDI + EBX]
  CMP	[EBP + EBX],AX
  POP	EAX
  JE	@CMP
  POP	EDI
  ADD	ECX,2
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - 04h]
  POP	EDI
  MOV	EAX,ECX
  SUB	EAX,2
  SHR	EAX,1
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  PUSH	EDX
  ADD	EDX,ECX
  NEG	ECX
  MOV	AX,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AX
  JE	@@MATCH
  ADD	ECX,2
  JNG	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  POP	EDX
  SUB	ECX,2
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
  SHR	EAX,1
end;

function AnsiToWideString(const Ansi: String): String;
var
  iCharCount: Integer;
begin
  iCharCount := Length(Ansi);
  if iCharCount > 0 then begin
    SetWSString(Result, iCharCount);
    iCharCount := MultiByteToWideChar(CP_ACP, MB_ERR_INVALID_CHARS, Pointer(Ansi), -1, Pointer(Result), succ(iCharCount));
    if iCharCount > 0 then
      Delete(Result, succ(iCharCount shl 1), MAXINT)
    else Result := E;
  end
  else Result := E;
end;

function WideStringToAnsi(const Wide: String): String;
var
  iCharCount: Integer;
  fError: Bool;
begin
  iCharCount := WSLength(Wide);
  if iCharCount > 1 then begin
    SetString(Result, nil, iCharCount);
    iCharCount := WideCharToMultiByte(CP_ACP, 0, Pointer(Wide), -1, Pointer(Result), succ(iCharCount), nil, @fError);
    if not fError and (iCharCount > 0) then
      Delete(Result, iCharCount, MAXINT)
    else Result := E;
  end
  else Result := E;
end;

function Utf8ToWide(const Utf8: String): String; register; assembler;
asm	// EAX = @Utf8, EDX = @Result
  OR	EAX,EAX
  JE	@EXIT
  CMP	DWORD PTR [EAX - SLO],BOM_SIZE
  JBE	@START
  MOV	ECX,[EAX]
  AND	ECX,00FFFFFFh
  CMP	ECX,BOM_NUM
  MOV	ECX,0
  JNE	@START
  MOV	ECX,BOM_SIZE
  @START:
  PUSH	ESI
  PUSH	EDI
  PUSH	EBX
  MOV	ESI,EAX
  MOV	EDI,EDX
  ADD	ESI,ECX
  MOV	EAX,[EAX - SLO]
  SUB	EAX,ECX
  SHL	EAX,1
  ADD	EAX,2
  CALL	System.@NewAnsiString
  MOV	[EDI],EAX
  MOV	EBX,EDI
  MOV	EDI,EAX
  @STR_LOOP:
  XOR	EAX,EAX
  MOV	AL,[ESI]
  CMP	AL,0FDh
  JA	@ZERO
  CMP	AL,080h
  JB	@ASCII
  XOR	ECX,ECX
  @COUNT:
  INC	CL
  SHL	AL,1
  JC	@COUNT
  CMP	CL,3
  JB	@ZERO
  SHR	AL,CL
  DEC	ECX
  ADD	ESI,ECX
  NEG	ECX
  INC	ECX
  @LOOP:
  MOV	DL,[ESI + ECX]
  CMP	DL,080h
  JB	@ZERO
  AND	DL,07Fh
  CMP	DL,040h
  JNB	@ZERO
  SHL	EAX,6
  OR	AL,DL
  INC	ECX
  JNE	@LOOP
  MOV	[EDI],AX
  ADD	EDI,2
  JMP	@STR_LOOP
  @ASCII:
  MOV	[EDI],AX
  ADD	EDI,2
  INC	ESI
  OR	EAX,EAX
  JNE	@STR_LOOP
  @DONE:
  MOV	EBX,[EBX]
  SUB	EDI,EBX
  MOV	DWORD PTR [EBX - SLO],EDI
  POP	EBX
  POP	EDI
  POP	ESI
  RET
  @ZERO:
  MOV	EAX,EBX
  CALL	System.@LStrClr
  POP	EBX
  POP	EDI
  POP	ESI
  RET
  @EXIT:
  MOV	EAX,EDX
  CALL	System.@LStrClr
end;

function WideToUtf8(const Wide: String; AddBom: Longbool): String; register; assembler;
asm	// EAX = @Wide, EDX = AddBom, ECX = @Result
  OR	EAX,EAX
  JE	@EXIT
  PUSH	EAX
  PUSH	ECX
  OR	EDX,EDX
  MOV	ECX,0
  JE	@START
  MOV	ECX,BOM_NUM
  @START:
  PUSH	ECX
  MOV	EAX,[EAX - SLO]
  MOV	EDX,UTF8_FACTOR / 2
  MUL	EDX
  ADD	EAX,BOM_SIZE
  CALL	System.@NewAnsiString
  POP	ECX
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDX,EAX
  POP	EAX
  PUSH	EBX
  PUSH	EDX
  XOR	EBX,EBX
  OR	ECX,ECX
  JE	@STR_LOOP
  MOV	DWORD PTR [EDX],BOM_NUM
  ADD	EDX,BOM_SIZE
  @STR_LOOP:
  MOV	BX,[EAX]
  OR	EBX,EBX
  JE	@DONE
  ADD	EDX,UTF8_FACTOR - 1
  XOR	ECX,ECX
  DEC	EDX
  CMP	EBX,00000800h
  JNB	@3BYTES
  DEC	EDX
  CMP	EBX,00000080h
  JNB	@2BYTES
  DEC	EDX
  JMP	@ASCII
  @3BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],BL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EBX,6
  DEC	EDX
  @2BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],BL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EBX,6
  DEC	EDX
  INC	ECX
  MOV	[EDX],BL
  MOV	BX,00FFh
  ROR	BX,CL
  OR	[EDX],BH
  ADD	EDX,ECX
  ADD	EAX,2
  JMP	@STR_LOOP
  @ASCII:
  MOV	[EDX],BL
  ADD	EAX,2
  INC	EDX
  JMP	@STR_LOOP
  @DONE:
  MOV	BYTE PTR [EDX],0
  POP	EAX
  SUB	EDX,EAX
  MOV	DWORD PTR [EAX - SLO],EDX
  POP	EBX
  RET
  @EXIT:
  MOV	EAX,ECX
  CALL	System.@LStrClr
end;

function Utf8ToWideString(const Utf8: String): String;
begin
  Result := Utf8ToWide(Utf8);
  if Result = E then Result := AnsiToWideString(Utf8);
end;

function WideStringToUtf8(const Wide: String): String;
begin
  Result := WideToUtf8(Wide, true);
  if Result = E then Result := WideStringToAnsi(Wide);
end;

function AnsiToUtf8String(const Ansi: String): String;
begin
  Result := WideStringToUtf8(AnsiToWideString(Ansi));
end;

function Utf8StringToAnsi(const Utf8: String): String;
begin
  Result := WideStringToAnsi(Utf8ToWideString(Utf8));
end;

function NoBomUtf8ToWideString(const Utf8: String): String;
begin
  Result := Utf8ToWide(Utf8);
  if Result = E then Result := AnsiToWideString(Utf8);
end;

function NoBomWideStringToUtf8(const Wide: String): String;
begin
  Result := WideToUtf8(Wide, false);
  if Result = E then Result := WideStringToAnsi(Wide);
end;

function NoBomUtf8ToAnsiString(const Utf8: String): String;
begin
  Result := WideStringToAnsi(NoBomUtf8ToWideString(Utf8));
end;

function NoBomAnsiStringToUtf8(const Ansi: String): String;
begin
  Result := NoBomWideStringToUtf8(AnsiToWideString(Ansi));
end;


// Url tools ---------------------------------------------------------------------------------------

function TrimUrl(const Url: String): String;
const
  WSP = [#1..#32];
var
  pEnd, pBeg: PChar;
begin
  if Url <> E then begin
    pBeg := Pointer(Url);
    pEnd := pBeg + Length(Url) - 1;
    while (pEnd > pBeg) and (pEnd^ in WSP) do dec(pEnd);
    while (pBeg < pEnd) and (pBeg^ in WSP) do inc(pBeg);
    while (pBeg + 1 < pEnd) and (pBeg^ = PC) and (HexToChar(pBeg + 1) in WSP) do inc(pBeg, 3);
    while (pEnd - 1 > pBeg) and ((pEnd - 2)^ = PC) and (HexToChar(pEnd - 1) in WSP) do dec(pEnd, 3);
    SetString(Result, pBeg, succ(pEnd - pBeg));
    Exit;
  end;
  Result := Url;
end;

function CanonicalizeUrl(const Url: String; Decode: Longbool): String;
var
  dwLen, dwFlags: DWord;
  sUrl: String;
begin
  sUrl := DelAngles(Url);
  if not IsDataUrl(sUrl) then begin
    dwLen := Length(NULL);
    if Decode then begin
      sUrl := Unescape(Url, false, PC);
      dwFlags := ICU_DECODE or ICU_NO_ENCODE;
    end
    else dwFlags := ICU_BROWSER_MODE or ICU_DECODE;
    if DWord(Length(sUrl)) > dwLen then begin
      if LoStrAtPos(PROT_ABOUT, sUrl, 1) then
	Delete(sUrl, 1, Length(PROT_ABOUT))
      else if FirstLoStrPos(TS, sUrl) > 0 then
	sUrl := XChgStrings(sUrl, TS, DS)
      else UniqueString(sUrl);
      InternetCanonicalizeUrl(Pointer(sUrl), E, dwLen, dwFlags);
      if GetLastError = ERROR_INSUFFICIENT_BUFFER then begin
	SetString(Result, NIL, pred(dwLen));
	if InternetCanonicalizeUrl(Pointer(sUrl), Pointer(Result), dwLen, dwFlags) then begin
	  Result := TrimUrl(Result);
	  Exit;
	end;
      end;
      Result := TrimUrl(sUrl);
    end
    else Result := E;
  end
  else Result := Url;
end;

function GetUrlProtocol(const Url: String): Integer;
var
  sDummy: String;
  uc: TUrlComponents;
begin
  if Url <> E then begin
    sDummy := CanonicalizeUrl(Url, false);
    FillChar(uc, SizeOf(TUrlComponents), 0);
    uc.dwStructSize := SizeOf(TUrlComponents);
    if InternetCrackUrl(PChar(sDummy), 0, 0, uc)
    or InternetCrackUrl(PChar(Unescape(sDummy, false, PC)), 0, 0, uc) then begin
      Result := uc.nScheme;
      Exit;
    end;
  end;
  Result := INTERNET_SCHEME_UNKNOWN;
end;

function IsBrowserProtocol(Scheme: Integer): Longbool;
begin
  Result := (Scheme in [INTERNET_SCHEME_FTP..INTERNET_SCHEME_HTTPS,
			INTERNET_SCHEME_SOCKS..INTERNET_SCHEME_VBSCRIPT]);
end;

function IsBrowserUrl(const Url: String): Longbool;
begin
  Result := IsBrowserProtocol(GetUrlProtocol(Url));
end;

function IsAboutUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_ABOUT, Url, 1);
end;

function IsCIDUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_CID, Url, 1);
end;

function IsDataUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_DATA, Url, 1);
end;

function IsFileUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_FILE, Url, 1);
end;

function IsMailToUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_MAIL, Url, 1);
end;

function IsResUrl(const Url: String): Longbool;
begin
  Result := LoStrAtPos(PROT_RES, Url, 1);
end;

function GetFilePathFromUrl(const Url: String): String;
var
  uc: TUrlComponents;
begin
  Result := CanonicalizeUrl(Url, false);
  if Result <> E then begin
    FillChar(uc, SizeOf(TUrlComponents), 0);
    with uc do begin
      dwStructSize := SizeOf(TUrlComponents);
      dwUrlPathLength := 1;
      dwExtraInfoLength := 1;
      if InternetCrackUrl(PChar(Result), 0, 0, uc) and (dwUrlPathLength > 1) then begin
	SetString(Result, lpszUrlPath, dwUrlPathLength);
	Result := Unescape(Result, false, PC);
	XChgChars(Result, SL, BS);
	if (Result[1] = BS) and (Result[2] <> BS) then Result := BS + Result;
      end
      else Result := E;
    end;
  end;
end;


// Rich Text tools ---------------------------------------------------------------------------------

const
  RTF_BEG = LC;
  RTF_END = RC;
  RTF_AST = [AST, BS];
  RTF_NEXT = [NULL, BS];
  RTF_DELIM = [LC, RC];
  RTF_BLANK = [LF, CR, SP];

const // back scan stopper
  RTF_PLAIN = '\plain';

const // shape control words
  // w&h in pixels
  RTF_LINKPICT = '\sslinkpict';

const // field control words
  RTF_FIELD = '\field';
  // leading \*
  RTF_FLDINST = '\fldinst';
  // leading \*
  RTF_FLDRSLT = '\fldrslt';

const // picture instructions
  RTF_INCPICT = 'INCLUDEPICTURE';
  RTF_FLDCODE = '\d';

const // shape results (optional)
  RTF_SHPP = '\shppict';
  RTF_NSHPP = '\nonshppict';

const // picture results
  RTF_PICT = '\pict';

  // pixels
  RTF_PICW = '\picw';
  RTF_PICH = '\pich';

  // twips
  RTF_GOALW = '\picwgoal';
  RTF_GOALH = '\pichgoal';

const // picture properties (description, e.g.)
  RTF_PICPROP = '\picprop';
  RTF_SP = '\sp';
  RTF_SN = '\sn';
  RTF_SV = '\sv';

function GetRichEditVersion: Longint;
var
  hInfo: THandle;
  dwSize: DWord;
  sInfo: String;
  dwFFSize: DWord;
  pFFInfo: Pointer;
begin
  Result := 0;
  dwSize := succ(GetFileVersionInfoSizeA(PChar(LIB_RICHED20), hInfo));
  if dwSize > 1 then begin
    SetString(sInfo, nil, dwSize);
    if GetFileVersionInfoA(PChar(LIB_RICHED20), hInfo, dwSize, PChar(sInfo)) then begin
      if VerQueryValueA(PChar(sInfo), '\StringFileInfo\000004B0\ProductVersion', pFFInfo, dwFFSize) then begin
	if dwFFSize > 0 then Result := StrToCardDef(PChar(pFFInfo)[0], 0);
      end;
    end;
  end;
end;

function IsRichText(const Text: String): Longbool;
begin
  Result := LoStrAtPos(RTF_MARKER, Text, 1);
end;

function RtfUnescape(const Text: String): String;
const
  ESC = '\''';
  BS4 = '\\\\';
var
  pResult, pText, pPos, pEnd: PChar;
  iLen: Integer;
begin
  if Text <> E then begin
    SetString(Result, nil, Length(Text));
    pResult := Pointer(Result);
    pText := Pointer(Text);
    pEnd := StrEnd(pText);
    pPos := pText;
    repeat
      while not (pPos^ in RTF_NEXT) do inc(pPos);
      if pPos^ = BS then begin
	if (pEnd - pPos >= Length(ESC) + 2) and (PDChar(pPos)^ = ESC) then begin
	  iLen := pPos - pText;
	  Move(pText^, pResult^, iLen);
	  inc(pResult, iLen);
	  pResult^ := Char(HexByteToDigit(pPos + Length(ESC), false));
	  inc(pText, iLen + Length(ESC) + 2);
	  inc(pResult);
	  pPos := pText;
	end
	else if (pEnd - pPos  >= pred(Length(BS4))) and (PQChar(pPos)^ = BS4) then begin
	  iLen := pPos - pText;
	  Move(pText^, pResult^, succ(iLen));
	  inc(pText, iLen + Length(BS4));
	  inc(pResult, succ(iLen));
	  pPos := pText;
	end
	else inc(pPos);
      end
      else Break;
    until pPos >= pEnd;
    iLen := pPos - pText;
    Move(pText^, pResult^, iLen);
    inc(pResult, iLen);
    System.Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
    Exit;
  end;
  Result := E;
end;

function GetImageDataFromSource(GDIPImage: TGdipImage; const ImageData: String; AsBitmap: Longbool;
				var PixelSizes, TargetSizes: TSize): String;

  function ExtractResourceName(const ImageData: String): String;
  begin
    if LoStrAtPos(PROT_R_DS, ImageData, 1) then begin
      Result := Copy(ImageData, succ(LastCharPos(SL, ImageData)), MAXINT);
      Exit;
    end;
    Result := E;
  end;

begin
  if (GDIPImage <> nil) and (ImageData <> E) then with GDIPImage do begin
    if FileExists(ImageData) then
      LoadFromFile(ImageData)
    else if not (LoadResourceBitmap(HInstance, ExtractResourceName(ImageData), true) or LoadFromStrBuffer(ImageData)) then Exit;
    if NullSize(TargetSizes) then TargetSizes := GetTwipsDimensions(0, MakeSize(ImageWidth, ImageHeight));
    if not IsResUrl(ImageData) then with TargetSizes do begin
      if ((cx * cy <> 0) and (ImageWidth * ImageHeight <> 0)) and (cx div cy <> ImageWidth div ImageHeight) then begin
	cx := MulDiv(cy, ImageWidth, ImageHeight);
      end;
    end;
    PixelSizes := GetPixDimsFromTwips(0, TargetSizes);
    if AsBitmap then
      Result := GetDIBitstream(PixelSizes.cx, PixelSizes.cy, TargetSizes.cx, TargetSizes.cy)
    else SaveAsPngBuffer(Result);
  end;
end;

function ImageDataToPictureRtf(GDIPImage: TGdipImage; const ImageSource: String;
			       const DimGoals: TSize; AsBitmap: Longbool): String;
const
  RTF_DIB = '\dibitmap';
  RTF_PNG = '\pngblip';
  PICT_BEG: AnsiString = RTF_BEG + RTF_PICT;
var
  szTwips, szPixels: TSize;
  rsPictData, rsDimData: String;
  iPictSize, iDimSize: Integer;
  pResult: PChar;
begin
  szTwips := DimGoals;
  szPixels := MakeSize(0, 0);
  if AsBitmap then
    PICT_BEG := PICT_BEG + RTF_DIB
  else PICT_BEG := PICT_BEG + RTF_PNG;
  rsPictData := GetImageDataFromSource(GDIPImage, ImageSource, AsBitmap, szPixels, szTwips);
  if rsPictData <> E then begin
    rsDimData := RTF_PICW + CardToStr(szPixels.cx) + RTF_PICH + CardToStr(szPixels.cy) +
		 RTF_GOALW + CardToStr(szTwips.cx) + RTF_GOALH + CardToStr(szTwips.cy);
    iPictSize := Length(rsPictData) shl 1;
    iDimSize := Length(rsDimData);
    SetString(Result, nil, Length(PICT_BEG) + iDimSize + Length(SP) + iPictSize + Length(RTF_END));
    pResult := Pointer(Result);
    Move(Pointer(PICT_BEG)^, pResult^, Length(PICT_BEG));
    inc(pResult, Length(PICT_BEG));
    Move(Pointer(rsDimData)^, pResult^, iDimSize);
    inc(pResult, iDimSize);
    pResult^ := SP;
    inc(pResult, Length(SP));
    StrToHexChars(rsPictData, pResult, iPictSize);
    inc(pResult, iPictSize);
    pResult^ := RTF_END;
    Exit;
  end;
  Result := E;
end;

function GetImageDataAsRtf(const Data: String; AsBitmap: Longbool): String;
var
  gdipi: TGdipImage;
begin
  if Data <> E then begin
    gdipi := TGdipImage.Create;
    if gdipi <> nil then begin
      try
	Result := ImageDataToPictureRtf(gdipi, Data, MakeSize(0, 0), AsBitmap);
	Exit;
      finally
	gdipi.Free;
      end;
    end;
  end;
  Result := E;
end;

type
  PRtfPos = ^TRtfPos;
  TRtfPos = Record
    BegPos: PChar;
    EndPos: PChar;
  end;

function MakeRtfPos(BPos, EPos: Pointer): TRtfPos;
begin
  with Result do begin
    BegPos := BPos;
    EndPos := EPos;
  end;
end;

type
  TRtfPictProps = class(TRawByteStrings)
  private
    function GetElementPos(Index: Integer): TRtfPos;
    procedure SetElementPos(Index: Integer; const RtfPos: TRtfPos);
    function GetElementBeg(Index: Integer): PChar;
    procedure SetElementBeg(Index: Integer; BPos: PChar);
    function GetElementEnd(Index: Integer): PChar;
    procedure SetElementEnd(Index: Integer; EPos: PChar);
    function GetElement(Index: Integer): RawByteString;
    procedure SetElement(Index: Integer; const S: RawByteString);
    function GetElementsPosText: RawByteString;
  protected
    procedure InsertItem(Index: Integer; const S: RawByteString; AObject: TObject); override;
  public
    destructor Destroy; override;
    function Add(const S: RawByteString): Integer; override;
    function AddRtfObject(const S: RawByteString; const RtfPos: TRtfPos): Integer;
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Insert(Index: Integer; const S: RawByteString); override;
    procedure InsertRtfObject(Index: Integer; const S: RawByteString; const RtfPos: TRtfPos);
    property Elements[Index: Integer]: RawByteString read GetElement write SetElement;
    property ElementsPos[Index: Integer]: TRtfPos read GetElementPos write SetElementPos;
    property ElementsBeg[Index: Integer]: PChar read GetElementBeg write SetElementBeg;
    property ElementsEnd[Index: Integer]: PChar read GetElementEnd write SetElementEnd;
    property ElementsPosText: RawByteString read GetElementsPosText;
  end;

function TRtfPictProps.GetElementPos(Index: Integer): TRtfPos;
begin
  if (Index > -1) and (Index < Count) then begin
    Result := PRtfPos(Objects[Index])^;
    Exit;
  end;
  Result := MakeRtfPos(nil, nil);
end;

procedure TRtfPictProps.SetElementPos(Index: Integer; const RtfPos: TRtfPos);
begin
  if (Index > -1) and (Index < Count) then PRtfPos(Objects[Index])^ := RtfPos;
end;

function TRtfPictProps.GetElementBeg(Index: Integer): PChar;
begin
  if (Index > -1) and (Index < Count) then begin
    Result := PChar(PRtfPos(Objects[Index])^.BegPos);
    Exit;
  end;
  Result := nil;
end;

procedure TRtfPictProps.SetElementBeg(Index: Integer; BPos: PChar);
begin
  if (Index > -1) and (Index < Count) then PChar(PRtfPos(Objects[Index])^.BegPos) := BPos;
end;

function TRtfPictProps.GetElementEnd(Index: Integer): PChar;
begin
  if (Index > -1) and (Index < Count) then begin
    Result := PChar(PRtfPos(Objects[Index])^.EndPos);
    Exit;
  end;
  Result := nil;
end;

procedure TRtfPictProps.SetElementEnd(Index: Integer; EPos: PChar);
begin
  if (Index > -1) and (Index < Count) then PChar(PRtfPos(Objects[Index])^.EndPos) := EPos;
end;

function TRtfPictProps.GetElement(Index: Integer): RawByteString;
begin
  if (Index > -1) and (Index < Count) then begin
    Result := Strings[Index];
    Exit;
  end;
  Result := E;
end;

procedure TRtfPictProps.SetElement(Index: Integer; const S: RawByteString);
begin
  if (Index > -1) and (Index < Count) then Strings[Index] := S;
end;

function TRtfPictProps.GetElementsPosText: RawByteString;
const
  RTF_L = RTF_BEG + SP;
  RTF_R = SP + RTF_END;
var
  iIndex, iLen: Integer;
  pResult: PChar;
begin
  iLen := 0;
  if Count > 0 then begin
    for iIndex := 0 to pred(Count) do begin
      inc(iLen, Length(RTF_L) + Length(Strings[iIndex]) + Length(RTF_R) + Length(CRLF));
    end;
    SetString(Result, nil, iLen);
    pResult := Pointer(Result);
    for iIndex := 0 to pred(Count) do begin
      if ElementsBeg[iIndex] <> nil then
	pResult^ := ElementsBeg[iIndex]^
      else pResult^ := '';
      inc(pResult);
      pResult^ := SP;
      inc(pResult);
      iLen := Length(Strings[iIndex]);
      System.Move(Pointer(Strings[iIndex])^, pResult^, iLen);
      inc(pResult, iLen);
      pResult^ := SP;
      inc(pResult);
      if ElementsEnd[iIndex] <> nil then
	pResult^ := ElementsEnd[iIndex]^
      else pResult^ := '';
      inc(pResult);
      PDChar(pResult)^ := CRLF;
      inc(pResult, Length(CRLF));
    end;
    Exit;
  end;
  Result := E;
end;

procedure TRtfPictProps.InsertItem(Index: Integer; const S: RawByteString; AObject: TObject);
var
  ptrRtfPos: PRtfPos;
begin
  New(ptrRtfPos);
  if ptrRtfPos <> nil then begin
    if AObject <> nil then ptrRtfPos^ := PRtfPos(AObject)^;
    AObject := TObject(ptrRtfPos);
  end;
  inherited;
end;

destructor TRtfPictProps.Destroy;
var
  iIndex: Integer;
begin
  for iIndex := 0 to pred(Count) do if Objects[iIndex] <> nil then begin
    Dispose(PRtfPos(Objects[iIndex]));
    Objects[iIndex] := nil;
  end;
  inherited;
end;

function TRtfPictProps.Add(const S: RawByteString): Integer;
begin
  Result := inherited Add(S);
end;

function TRtfPictProps.AddRtfObject(const S: RawByteString; const RtfPos: TRtfPos): Integer;
begin
  Result := inherited Add(S);
  SetElementPos(Result, RtfPos);
end;

procedure TRtfPictProps.Clear;
var
  iIndex: Integer;
begin
  for iIndex := 0 to pred(Count) do if Objects[iIndex] <> nil then begin
    Dispose(PRtfPos(Objects[iIndex]));
    Objects[iIndex] := nil;
  end;
  inherited;
end;

procedure TRtfPictProps.Delete(Index: Integer);
begin
  if (Index > -1) and (Index < Count) then begin
    if Objects[Index] <> nil then begin
      Dispose(PRtfPos(Objects[Index]));
      Objects[Index] := nil;
    end;
  end;
  inherited;
end;

procedure TRtfPictProps.Insert(Index: Integer; const S: RawByteString);
begin
  inherited Insert(Index, S);
end;

procedure TRtfPictProps.InsertRtfObject(Index: Integer; const S: RawByteString; const RtfPos: TRtfPos);
begin
  inherited InsertObject(Index, S, TObject(@RtfPos));
end;

function GetIncludePictureName(rppPict: TRtfPictProps): String;
var
  iIndex, iLen: Integer;
  pBeg, pEnd, pPos: PChar;
begin
  with rppPict do begin
    iIndex := IndexOf(RTF_FLDINST);
    if iIndex > -1 then begin
      with ElementsPos[iIndex] do begin
	pBeg := BegPos;
	pEnd := EndPos;
      end;
      inc(pBeg);
      while (pBeg < pEnd) and (pBeg^ in RTF_AST) do inc(pBeg);
      inc(pBeg, Length(RTF_FLDINST));
      while (pBeg < pEnd) and (pBeg^ <= SP) do inc(pBeg);
      inc(pBeg, Length(RTF_INCPICT));
      repeat
	while (pBeg < pEnd) and (pBeg^ <= SP) do inc(pBeg);
	if pBeg^ = BS then while (pBeg < pEnd) and not (pBeg^ in RTF_BLANK) do inc(pBeg);
      until not (pBeg^ in RTF_BLANK + [BS]);
      pPos := pBeg + 1;
      if pBeg^ = QU then begin
	while (pPos < pEnd) and (pPos^ <> QU) do inc(pPos);
	inc(pBeg);
      end
      else while (pPos < pEnd) and not (pPos^ in RTF_BLANK + [RTF_END]) do inc(pPos);
      pEnd := pPos;
      pPos := pBeg;
      while (pPos < pEnd) and (pPos^ <> SL) do inc(pPos);
      if (pPos^ = SL) then while (pPos < pEnd) and (pPos^ = SL) do inc(pPos);
      iLen := pPos - pBeg;
      if (iLen > 0) and (StrLIComp(pBeg, PROT_F_DS, Length(PROT_F_DS)) = 0) then inc(pBeg, iLen);
      iLen := pEnd - pBeg;
      if iLen > 0 then begin
	Result := RtfUnescape(StrLPas(pBeg, iLen));
	Exit;
      end;
    end;
  end;
  Result := E;
end;

function GetDesciptionPictureName(rppPict: TRtfPictProps): String;
const
  RTF_SNAME = 'wzDescription';
var
  pPos: PChar;
  iIndex: Integer;
begin
  with rppPict do begin
    iIndex := IndexOf(RTF_SN);
    if (iIndex > -1) and (Elements[succ(iIndex)] = RTF_SV) then begin
      if TextPos(PChar(RTF_SNAME), ElementsBeg[iIndex]) <> nil then begin
        inc(iIndex);
	pPos := ElementsBeg[iIndex];
	while (pPos < ElementsEnd[iIndex]) and (pPos^ <> SP) do inc(pPos);
	while (pPos < ElementsEnd[iIndex]) and (pPos^ in RTF_BLANK) do inc(pPos);
	Result := RtfUnescape(StrLPas(pPos, ElementsEnd[iIndex] - pPos));
      end;
    end;
  end;
end;

function GetImageDimensions(rppPict: TRtfPictProps): TSize;

  function GetDimension(var BegPos, EndPos: PChar): Cardinal;
  begin
    EndPos := BegPos + 1;
    while EndPos^ in DIGITS do inc(EndPos);
    Result := StrToCardDef(StrLPas(BegPos, EndPos - BegPos), Cardinal(-1));
    BegPos := EndPos;
  end;

var
  pDimNameW, pDimNameH, pDimsBeg, pDimsEnd: PChar;
  iIndex: Integer;
begin
  pDimNameW := nil;
  pDimNameH := nil;
  with rppPict do begin
    iIndex := IndexOf(RTF_LINKPICT);
    if iIndex <> -1 then begin
      pDimNameW := RTF_LINKPICT + 'w';
      pDimNameH := RTF_LINKPICT + 'h';
    end
    else begin
      iIndex := IndexOf(RTF_PICT);
      if iIndex <> -1 then begin
	pDimNameW := RTF_GOALW;
	pDimNameH := RTF_GOALH;
      end;
    end;
    if iIndex <> -1 then begin
      with ElementsPos[iIndex] do begin
	pDimsBeg := BegPos;
	pDimsEnd := EndPos;
      end;
      repeat
	while not (pDimsBeg^ in [NULL, SP, BS]) do inc(pDimsBeg);
	if pDimsBeg^ = BS then begin
	  if StrLIComp(pDimsBeg, pDimNameW, Length(pDimNameW)) = 0 then begin
	    inc(pDimsBeg, Length(pDimNameW));
	    if pDimsBeg^ in DIGITS then Result.cx := GetDimension(pDimsBeg, pDimsEnd);
	    Continue;
	  end;
	  if StrLIComp(pDimsBeg, pDimNameH, Length(pDimNameH)) = 0 then begin
	    inc(pDimsBeg, Length(pDimNameH));
	    if pDimsBeg^ in DIGITS then Result.cy := GetDimension(pDimsBeg, pDimsEnd);
	    Continue;
	  end;
	  inc(pDimsBeg);
	end;
      until ((Result.cx > 0) and (Result.cy > 0)) or (pDimsBeg^ in RTF_BLANK + RTF_DELIM);
      Exit;
    end;
  end;
  Result := MakeSize(0, 0);
end;

function ParseRtfPictElement(PictElement: PChar; rppPict: TRtfPictProps): PChar;
var
  pPos: PChar;
  iIndex: Integer;
begin
  Result := PictElement;
  while not (Result^ in RTF_NEXT) do inc(Result);
  if (StrLIComp(Result, RTF_PICT, Length(RTF_PICT)) = 0) then begin
    rppPict.AddRtfObject(RTF_PICT, MakeRtfPos(ScanBack(Result, RTF_BEG), nil));
    inc(Result, succ(Length(RTF_PICT)));
    repeat
      while not (Result^ in [NULL] + RTF_DELIM) or ((Result^ in RTF_DELIM) and ((Result - 1)^ = BS)) do inc(Result);
      case Result^ of
	NULL: Break;
	RTF_BEG: if ((Result + 1)^ = BS) and ((Result + 2)^ > NULL) then begin
	  if (Result + 2)^ <> AST then begin
	    pPos := Result + 2;
	    while not (pPos^ in [NULL] + RTF_BLANK + RTF_DELIM) do inc(pPos);
	    if (pPos^ > NULL) then rppPict.AddRtfObject(StrLPas(Result + 1, pPos - Result - 1), MakeRtfPos(Result, nil));
	    while not (pPos^ in [NULL] + RTF_DELIM) or ((pPos^ in RTF_DELIM) and ((pPos - 1)^ = BS)) do inc(pPos);
	    if pPos^ = NULL then begin
	      Result := NULL;
	      Break;
	    end;
	    Result := pPos;
	  end
	  else if StrLIComp(Result + 3, RTF_PICPROP, Length(RTF_PICPROP)) = 0 then begin
	    rppPict.AddRtfObject(RTF_PICPROP, MakeRtfPos(Result, nil));
	    inc(Result, Length(RTF_PICPROP) + 3);
	  end;
	end;
	RTF_END: with rppPict do begin
	  for iIndex := pred(Count) downto 0 do begin
	    if ElementsEnd[iIndex] = nil then begin
	      ElementsEnd[iIndex] := Result;
	      Break;
	    end;
	  end;
	  while (iIndex > -1) and (ElementsEnd[iIndex] <> nil) do dec(iIndex);
	  if (iIndex > -1) and (ElementsEnd[iIndex] = nil) then
	    inc(Result)
	  else Break;
	end;
      end;
    until Result^ = NULL;
  end
  else Result := PictElement;
end;

function CreateEmbeddedRtfVersion(const Text, DefaultBitmap: String; ImageList: TStringList): String;
const
  SHPP_CONTROLS: Array[0..1] of String = (RTF_SHPP, RTF_NSHPP);
var
  szDims: TSize;
  gdipi: TGdipImage;
  rppPict: TRtfPictProps;
  rbsData: TRawByteStrings;
  rsPicture: RawByteString;
  rsPictControl: RawByteString;
  iIndex, iCount, iLength, iCID: Integer;
  pText, pBeg, pEnd, pIndex, pResult: PChar;
begin
  if StrLIComp(PChar(Text), RTF_MARKER, Length(RTF_MARKER)) = 0 then begin
    gdipi := TGdipImage.Create;
    rppPict := TRtfPictProps.Create;
    rbsData := TRawByteStrings.Create;
    if (gdipi <> nil) and (rppPict <> nil) and (rbsData <> nil) then try
      with rppPict do begin
	Duplicates := dupAccept;
	Sorted := false;
	BeginUpdate;
      end;
      with rbsData do begin
	Duplicates := dupAccept;
	Sorted := false;
	BeginUpdate;
      end;
      pText:= Pointer(Text);
      pBeg := pText;
      repeat
	rppPict.Clear;
	szDims := MakeSize(-1, -1);
	// find field instructions and include picture name
	repeat
	  while not (pBeg^ in RTF_NEXT) do inc(pBeg);
	  if pBeg^ = BS then begin
	    if StrLIComp(pBeg, RTF_FLDINST, Length(RTF_FLDINST)) = 0 then Break;
	    inc(pBeg);
	  end;
	until pBeg^ = NULL;
	if pBeg^ = NULL then Break;
	pEnd := pBeg + Length(RTF_FLDINST);
	while not (pEnd^ in [NULL, RTF_END]) do inc(pEnd);
	if (pEnd^ = NULL) then Break;
	pIndex := pBeg + Length(RTF_FLDINST);
	while (pIndex^ > NULL) and (pIndex^ <= SP) do inc(pIndex);
	if StrLIComp(pIndex, RTF_INCPICT, Length(RTF_INCPICT)) <> 0 then begin
	  pBeg := pEnd + Length(RTF_END);
	  Continue;
	end;
	rppPict.AddRtfObject(RTF_FLDINST, MakeRtfPos(ScanBack(pBeg, RTF_BEG), pEnd));
	rsPicture := GetIncludePictureName(rppPict);
	// scan back to field definition starting point
	dec(pBeg, Length(RTF_FIELD));
	while not (pBeg^ in [BS, RTF_BEG]) do dec(pBeg);
	if (StrLIComp(pBeg, RTF_FIELD, Length(RTF_FIELD)) = 0) then begin
	  rppPict.InsertRtfObject(0, RTF_FIELD, MakeRtfPos(ScanBack(pBeg, RTF_BEG), nil));
	  pBeg := rppPict.ElementsBeg[0];
	end;
	pIndex := pBeg - 1;
	// scan back to link picture definition if any
	while (pIndex^ in RTF_BLANK + [RTF_BEG]) do dec(pIndex);
	if pIndex^ <> RTF_END then begin
	  dec(pIndex, Length(RTF_PLAIN));
	  while not (pIndex^ in [BS, RTF_BEG]) do dec(pIndex);
	  if (StrLIComp(pIndex, RTF_PLAIN, Length(RTF_PLAIN)) <> 0)
	  and (StrLIComp(pIndex, RTF_LINKPICT, Length(RTF_LINKPICT)) = 0) then begin
	    rppPict.InsertRtfObject(0, RTF_LINKPICT, MakeRtfPos(ScanBack(pIndex, RTF_BEG), nil));
	    pBeg := rppPict.ElementsBeg[0];
	  end;
	end;
	if pBeg = NULL then Exit;
	while not (pBeg^ in [RTF_BEG]) do dec(pBeg);
	while not (pEnd^ in [RTF_BEG]) do inc(pEnd);
	// find field result element and complete it if empty
	repeat
	  while not (pEnd^ in RTF_NEXT) do inc(pEnd);
	  if pEnd^ = BS then begin
	    if StrLIComp(pEnd, RTF_FLDRSLT, Length(RTF_FLDRSLT)) = 0 then begin
	      rppPict.AddRtfObject(RTF_FLDRSLT, MakeRtfPos(ScanBack(pEnd, RTF_BEG), nil));
	      inc(pEnd, Length(RTF_FLDRSLT));
	      while pEnd^ <= SP do inc(pEnd);
	      if pEnd^ = RTF_END then with rppPict do begin
		iIndex := pred(Count);
		while (pEnd^ in RTF_BLANK + [RTF_END]) do begin
		  if (pEnd^ = RTF_END) and (iIndex > -1) then begin
		    if ElementsEnd[iIndex] = nil then begin
		      ElementsEnd[iIndex] := pEnd;
		      inc(pEnd);
		    end;
		    dec(iIndex);
		  end;
		  if iIndex = -1 then Break;
		end;
	      end;
	      Break;
	    end;
	    inc(pEnd);
	  end;
	until pEnd^ = NULL;
	if pEnd^ = NULL then Break;
	// find shape control elements if any
	for iIndex := Low(SHPP_CONTROLS) to High(SHPP_CONTROLS) do begin
	  while not (pIndex^ in RTF_NEXT) do inc(pIndex);
	    if pIndex^ = BS then begin
	    for iCount := Low(SHPP_CONTROLS) to High(SHPP_CONTROLS) do begin
	      rsPictControl := SHPP_CONTROLS[iCount];
	      iLength := Length(rsPictControl);
	      if (StrLIComp(pIndex, Pointer(rsPictControl), iLength) = 0)
	      or (StrLIComp(pIndex + 2, Pointer(rsPictControl), iLength) = 0) then begin
		rppPict.AddRtfObject(rsPictControl, MakeRtfPos(ScanBack(pIndex, RTF_BEG), nil));
		inc(pIndex, iLength);
		pEnd := pIndex;
		Break;
	      end;
	    end;
	  end;
	end;
	if pEnd^ = NULL then Break;
	// parse and complete the whole picture section
	pEnd := ParseRtfPictElement(pEnd, rppPict);
	if rppPict.IndexOf(RTF_PICT) > -1 then begin
	  while not (pEnd^ in RTF_NEXT) do inc(pEnd);
	  if pEnd^ = NULL then Break;
	end;
	// check filename from instructions and use the one from description if any
	if (rsPicture = E) or SameText(rsPicture, DefaultBitmap) then begin
	  rsPicture := GetDesciptionPictureName(rppPict);
	  iCID := ImageList.IndexOf(rsPicture);
	end
	else iCID := -1;
	// extract picture dimensions from link picture or picture result respectively
	szDims := GetImageDimensions(rppPict);
	// include any link definiton into the source section to copy
	with rppPict do begin
	  iIndex := IndexOf(RTF_LINKPICT);
	  if iIndex > -1 then pBeg := ElementsBeg[succ(iIndex)];
	end;
	// compose the resulting destination rich text
	iLength := pBeg - pText;
	rbsData.SetListString(pText, iLength);
	if iCID > -1 then
	  rsPicture := PString(ImageList.Objects[iCID])^
	else rsPicture := ImageDataToPictureRtf(gdipi, RtfUnescape(rsPicture), szDims, true);
	rbsData.Add(rsPicture);
	if iIndex > -1 then rbsData.Add(RTF_END);
	rsPicture := E;
	pText := pEnd;
	pBeg := pEnd;
      until pBeg^ = NULL;
      iLength := pBeg - pText;
      rbsData.SetListString(pText, iLength);
      Exit;
    finally
      // cleanup and copy to result
      if gdipi <> nil then gdipi.Free;
      if rppPict <> nil then rppPict.Free;
      if rbsData <> nil then with rbsData do begin
	try
	  iLength := 0;
	  for iIndex := 0 to pred(Count) do begin
	    inc(iLength, Length(Strings[iIndex]));
	  end;
	  SetString(Result, nil, iLength);
	  pResult := Pointer(Result);
	  for iIndex := 0 to pred(Count) do begin
	    iLength := Length(Strings[iIndex]);
	    System.Move(Pointer(Strings[iIndex])^, pResult^, iLength);
	    inc(pResult, iLength);
	  end;
	finally
	  rbsData.Free;
	end;
      end;
    end;
  end;
  if Result = E then Result := Text;
end;

function GetRtfSectionEnd(const Text: String; Pos, Max: Integer): Integer;

  function IsEscaped(const Text: String; Pos: Integer): Longbool;
  var
    iCount: Integer;
  begin
    iCount := 0;
    while (Pos > 1) and (Text[pred(Pos)] = BS) do begin
      inc(iCount);
      dec(Pos);
    end;
    Result := Odd(iCount);
  end;

var
  iCount: Integer;
begin
  Result := 0;
  if Pos < Max then begin
    iCount := 1;
    Result := succ(Pos);
    while (Result <= Max) and (iCount > 0) do begin
      case Text[Result] of
	RTF_BEG: if not IsEscaped(Text, Result) then inc(iCount);
	RTF_END: if not IsEscaped(Text, Result) then dec(iCount);
      end;
      inc(Result);
    end;
    dec(Result);
  end;
end;

function GetRtfTextHeight(RichEdit: THandle; Version, FontSize, VMargins: Integer): Integer;
var
  iLineCount, iLineIndex: Integer;
  ptLastChar, ptFirstChar: TPoint;
begin
  ptLastChar := OffScreenPos;
  ptFirstChar := OffScreenPos;
  iLineCount := SendMessage(RichEdit, EM_GETLINECOUNT, 0, 0);
  iLineIndex := SendMessage(RichEdit, EM_LINEINDEX, pred(iLineCount), 0);
  if Version <> 2 then begin
    SendMessage(RichEdit, EM_POSFROMCHAR, Longint(@ptLastChar), iLineIndex);
    SendMessage(RichEdit, EM_POSFROMCHAR, Longint(@ptFirstChar), 0);
  end
  else begin
    ptLastChar := MakePoints(SendMessage(RichEdit, EM_POSFROMCHAR, iLineIndex, 0));
    ptFirstChar := MakePoints(SendMessage(RichEdit, EM_POSFROMCHAR, 0, 0));
  end;
  Result := ptLastChar.Y - ptFirstChar.Y + FontSize + VMargins;
end;

function GetRtfTextLength(RichEdit: THandle; Version: Integer): Integer;
var
  gtle: TGetTextLengthEx;
begin
  if Version > 1 then begin
    with gtle do begin
      flags := GTL_DEFAULT;
      codepage := CP_UNICODE;
    end;
    Result := SendMessage(RichEdit, EM_GETTEXTLENGTHEX, Longint(@gtle), 0);
  end
  else Result := SendMessage(RichEdit, WM_GETTEXTLENGTH, 0, 0);
end;

function GetRtfText(RichEdit: THandle; Version: Integer): String;
var
  iLen: Integer;
  gteUnicode: TGetTextEx;
begin
  if Version > 1 then begin
    iLen := GetRtfTextLength(RichEdit, Version);
    if iLen > 0 then begin
      SetWSString(Result, iLen);
      with gteUnicode do begin
	cb := Length(Result);
	flags := GTL_DEFAULT;
	codepage := CP_UNICODE;
	lpDefaultChar := nil;
	lpUsedDefChar := nil;
      end;
      iLen := SendMessage(RichEdit, EM_GETTEXTEX, Longint(@gteUnicode), Longint(Result));
      WSDelete(Result, succ(iLen), MAXINT);
      Exit;
    end;
  end
  else begin
    iLen := GetRtfTextLength(RichEdit, Version);
    if iLen > 0 then begin
      SetString(Result, nil, iLen);
      iLen := SendMessage(RichEdit, WM_GETTEXT, iLen, Longint(Result));
      Delete(Result, succ(iLen), MAXINT);
      Exit;
    end;
  end;
  Result := E;
end;

function CreateFontFromCharFormat(const CharFormat: TCharFormat): TFont;
var
  fs: TFontStyles;
begin
  Result := TFont.Create;
  if Result <> nil then with Result, CharFormat do begin
    Color := GetValueFromCharFormat(CharFormat, CFM_COLOR);
    Name := szFaceName;
    Pitch := fpDefault;
    case bPitchAndFamily of
      FIXED_PITCH: Pitch := fpFixed;
      VARIABLE_PITCH: Pitch := fpVariable
    end;
    Size := yHeight div 20;
    fs := [];
    if (dwEffects and CFE_BOLD) <> 0 then Include(fs, fsBold);
    if (dwEffects and CFE_ITALIC) <> 0 then Include(fs, fsItalic);
    if (dwEffects and CFE_UNDERLINE) <> 0 then Include(fs, fsUnderline);
    if (dwEffects and CFE_STRIKEOUT) <> 0 then Include(fs, fsStrikeOut);
    Style := fs;
  end;
end;

function CreateFontFromHandle(Font: THandle; const Face: String; Size: Integer; Styles: TFontStyles): THandle;
var
  lf: TLogFont;
  hDC: THandle;
begin
  if GetObject(Font, SizeOf(TLogFont), @lf) <> 0 then with lf do begin
    hDC := GetDC(0);
    try
      if Size > -1 then lfHeight := -MulDiv(Size, GetDeviceCaps(hDC, LOGPIXELSY), 72);
      if Face <> E then StrPLCopy(lfFaceName, Face, LF_FACESIZE);
      if Styles <> [] then begin
	lfItalic := Byte(fsItalic in Styles);
	lfStrikeOut := Byte(fsStrikeout in Styles);
	lfUnderline := Byte(fsUnderline in Styles);
	lfWeight := Byte(fsBold in Styles) * FW_BOLD;
      end;
      Result := CreateFontIndirect(lf);
    finally
      if hDC <> 0 then ReleaseDC(0, hDC);
    end;
    Exit;
  end;
  Result := 0;
end;

function GetValueFromCharFormat(const CharFormat: TCharFormat; Mask: DWord): Variant;
begin
  with CharFormat do begin
    if Mask = CFM_EFFECTS then begin
      Result := (dwEffects and CFE_BOLD) or
		(dwEffects and CFE_ITALIC) or
		(dwEffects and CFE_UNDERLINE) or
		(dwEffects and CFE_STRIKEOUT);
    end
    else case Mask of
      CFM_BOLD: Result := dwEffects and CFE_BOLD;
      CFM_ITALIC: Result := dwEffects and CFE_ITALIC;
      CFM_UNDERLINE: Result := dwEffects and CFE_UNDERLINE;
      CFM_STRIKEOUT: Result := dwEffects and CFE_STRIKEOUT;
      CFM_SIZE: Result := yHeight div 20;
      CFM_COLOR: if (dwEffects and CFE_AUTOCOLOR) <> 0 then
	Result := GetSysColor(COLOR_WINDOWTEXT)
      else Result := crTextColor;
      CFM_FACE: Result := String(szFaceName);
      CFM_OFFSET: Result := yOffset;
      CFM_CHARSET: Result := bCharSet;
    end;
  end;
end;

function GetCharFormatFromFont(const Font: TFont; const NewFace: String; NewSize: Integer; Mask: DWord): TCharFormat;
var
  fs: TFontStyles;
begin
  FillChar(Result, SizeOf(TCharFormat), 0);
  with Font, Result do begin
    cbSize := SizeOf(TCharFormat);
    if (NewFace <> E) and (NewSize > 0) then
      dwMask := CFM_FACE or CFM_SIZE
    else dwMask := Mask;
    dwEffects := 0;
    fs := Style;
    if fsBold in fs then dwEffects := dwEffects or CFE_BOLD;
    if fsItalic in fs then dwEffects := dwEffects or CFE_ITALIC;
    if fsUnderline in fs then dwEffects := dwEffects or CFE_UNDERLINE;
    if fsStrikeout in fs then dwEffects := dwEffects or CFE_STRIKEOUT;
    if NewSize > 0 then
      yHeight := NewSize * 20
    else yHeight := Size;
    if Color = clWindowText then
      dwEffects := CFE_AUTOCOLOR
    else crTextColor := ColorToRGB(Color);
    case Pitch of
      fpVariable: bPitchAndFamily := VARIABLE_PITCH;
      fpFixed: bPitchAndFamily := FIXED_PITCH;
    else
      bPitchAndFamily := DEFAULT_PITCH;
    end;
    if NewFace <> E then
      StrPLCopy(szFaceName, NewFace, LF_FACESIZE)
    else StrPLCopy(szFaceName, Name, LF_FACESIZE);
  end;
end;

function GetCharFormatFromLogFont(const LogFont: TLogFont; const NewName: String; NewSize: Integer; Mask: DWord): TCharFormat;
begin
  FillChar(Result, SizeOf(TCharFormat), 0);
  with LogFont, Result do begin
    cbSize := SizeOf(TCharFormat);
    if (NewName <> E) and (NewSize > 0) then
      dwMask := CFM_FACE or CFM_SIZE
    else dwMask := Mask;
    dwEffects := CFE_AUTOCOLOR;
    if lfWeight >= FW_BOLD then dwEffects := dwEffects or CFE_BOLD;
    if lfItalic <> 0 then dwEffects := dwEffects or CFE_ITALIC;
    if lfUnderline  <> 0 then dwEffects := dwEffects or CFE_UNDERLINE;
    if lfStrikeout  <> 0 then dwEffects := dwEffects or CFE_STRIKEOUT;
    if NewSize > 0 then
      yHeight := NewSize * 20
    else yHeight := lfHeight * 20;
    bCharset := lfCharset;
    bPitchAndFamily := lfPitchAndFamily;
    if NewName <> E then
      StrPLCopy(szFaceName, NewName, LF_FACESIZE)
    else StrLCopy(szFaceName, lfFaceName, LF_FACESIZE);
  end;
end;


// TStringStream ---------------------------------------------------------------

constructor TStringStream.Create(const Str: String);
begin
  inherited Create;
  SetPointer(Pointer(Str), Length(Str));
end;

destructor TStringStream.Destroy;
begin
  inherited Destroy;
end;

function TStringStream.ReadAndClear: String;
begin
  Delete(FMemoryString, succ(Size), MAXINT);
  Result := FMemoryString;
  Clear;
end;

function TStringStream.GetCapacity: Longint;
begin
  Result := Length(FMemoryString);
end;

procedure TStringStream.SetCapacity(Value: Longint);
begin
  inherited Capacity := Value;
end;

function TStringStream.Realloc(var NewCapacity: Longint): Pointer;
const
  MEMORY_DELTA = $10000; // must be a power of 2
begin
  if NewCapacity > 0 then NewCapacity := (NewCapacity + pred(MEMORY_DELTA)) and not pred(MEMORY_DELTA);
  Result := Memory;
  if NewCapacity <> Length(FMemoryString) then begin
    SetLength(FMemoryString, NewCapacity);
    Result := Pointer(FMemoryString);
  end;
end;


// TRftTxtConverter --------------------------------------------------------------------------------

constructor TRtfTxtConverter.Create;
begin
  inherited;
  if GetModuleHandleA(LIB_MSFTEDIT) = 0 then LoadLibraryA(LIB_MSFTEDIT);
  if GetModuleHandleA(LIB_RICHED20) = 0 then LoadLibraryA(LIB_RICHED20);
  if GetModuleHandleA(LIB_RICHED10) = 0 then LoadLibraryA(LIB_RICHED10);
  if GetRichEditVersion >= 3 then begin
    if GetModuleHandle(LIB_MSFTEDIT) <> 0 then
      FRtfWindow := CreateWindow(RICHEDIT50W, nil, ES_MULTILINE or WS_DISABLED, 0, 0, 0, 0, 0, 0, hInstance, nil)
    else FRtfWindow := CreateWindow(RICHEDIT20W, nil, ES_MULTILINE or WS_DISABLED, 0, 0, 0, 0, 0, 0, hInstance, nil);
  end
  else FRtfWindow := CreateWindow(RichEd20.RICHEDIT, nil, ES_MULTILINE or WS_DISABLED, 0, 0, 0, 0, 0, 0, hInstance, nil);
  SendMessage(FRtfWindow, EM_EXLIMITTEXT, 0, MAXINT);
end;

destructor TRtfTxtConverter.Destroy;
begin
  DestroyWindow(FRtfWindow);
  inherited;
end;

function TRtfTxtConverter.RtfToText(const RtfText: String; var ReturnUnicode: Longbool): String;
var
  ste: TSetTextEx;
  gtle: TGetTextLengthEx;
  iLen: Integer;
  gte: TGetTextEx;
begin
  if (FRtfWindow = 0) or (RtfText = E) then begin
    Result := RtfText;
    Exit;
  end;
  if GetRichEditVersion >= 3 then begin
    with ste do begin
      flags := ST_DEFAULT;
      codepage := CP_ACP;
    end;
    if SendMessage(FRtfWindow, EM_SETTEXTEX, Longint(@ste), Longint(Pointer(RtfText))) > 0 then begin
      with gtle do begin
	flags := GTL_USECRLF;
	codepage := CP_ACP
      end;
      iLen := SendMessage(FRtfWindow, EM_GETTEXTLENGTHEX, Longint(@gtle), 0);
      if iLen > 0 then with gte do begin
	if ReturnUnicode then begin
	  inc(iLen, SizeOf(PWideChar));
	  SetWSString(Result, iLen);
	  codepage := CP_UNICODE;
	end
	else begin
	  inc(iLen, SizeOf(PChar));
	  SetString(Result, nil, iLen);
	  codepage := CP_ACP;
	end;
	cb := iLen;
	flags := GT_USECRLF;
	lpDefaultChar := nil;
	lpUsedDefChar := nil;
	iLen := SendMessage(FRtfWindow, EM_GETTEXTEX, Longint(@gte), Longint(Result));
	if iLen > 0 then begin
	  if ReturnUnicode then
	    WSDelete(Result, succ(iLen), MAXINT)
	  else Delete(Result, succ(iLen), MAXINT);
	  Exit;
	end;
      end;
    end;
    Result := E;
  end
  else if SetWindowText(FRtfWindow, Pointer(RtfText)) then begin
    if ReturnUnicode then
      Result := GetWindowStringW(FRtfWindow)
    else Result := GetWindowStringA(FRtfWindow);
  end
  else Result := E;
end;

type
  TDataBufferA = Record
    Data: String;
    CurrentPos: Integer;
  end;

function RtfStreamSaveA(var DestBuffer: TDataBufferA; SourceBuffer: PByte; BytesToSave: Longint; var BytesSaved: Longint): DWord; stdcall;
var
  iLen: Integer;
  pCurrent: PChar;
begin
  Result := 0;
  try
    with DestBuffer do begin
      iLen := CurrentPos + BytesToSave;
      if iLen > (Length(Data) * SizeOf(Char)) then SetLength(Data, iLen);
      pCurrent := Pointer(Data);
      inc(pCurrent, CurrentPos);
      inc(CurrentPos, BytesToSave);
    end;
    Move(SourceBuffer^, pCurrent^, BytesToSave);
    BytesSaved := BytesToSave;
  except
    Result := $FFFFFFFF;
  end;
end;

function TRtfTxtConverter.TextToRtf(const PlainText: String; const RtfFont: TRtfFont; Utf8: Longbool): String;

  procedure SetDefaultRtfFont(RichEdit: THandle; const RtfFont: TRtfFont);
  var
    cfText: TCharFormat;
  begin
    FillChar(cfText, SizeOf(TCharFormat), 0);
    cfText.cbSize := SizeOf(TCharFormat);
    SendMessage(RichEdit, EM_GETCHARFORMAT, SCF_DEFAULT, Longint(@cfText));
    with RtfFont, cfText do begin
      dwMask := CFM_BOLD or CFM_ITALIC or CFM_UNDERLINE or CFM_STRIKEOUT or CFM_SIZE or CFM_FACE;
      if RtfFont.Styles = 0 then
	dwEffects := CFE_AUTOCOLOR
      else dwEffects := Styles;
      if Size <> 0 then yHeight := Size * 20;
      if Name <> E then StrPLCopy(szFaceName, Name, LF_FACESIZE);
    end;
    SendMessage(RichEdit, EM_SETCHARFORMAT, SCF_ALL, Longint(@cfText));
  end;

var
  sInput: String;
  ste: TSetTextEx;
  iLen: Integer;
  dbaRtf: TDataBufferA;
  esRTF: TEditStream;
begin
  if (FRtfWindow = 0) or (PlainText = E) then begin
    Result := PlainText;
    Exit;
  end;
  SetDefaultRtfFont(FRtfWindow, RtfFont);
  if GetRichEditVersion >= 3 then begin
    if not Utf8 then
      sInput := PlainText
    else sInput := NoBomUtf8ToWideString(PlainText);
    with ste do begin
      flags := ST_DEFAULT;
      if Utf8 or IsWideString(sInput) then begin
	iLen := Length(sInput);
	codepage := CP_UNICODE;
      end
      else begin
	iLen := Length(sInput) shl 1;
	codepage := CP_ACP;
      end;
    end;
    SendMessage(FRtfWindow, EM_SETTEXTEX, Longint(@ste), Longint(Pointer(sInput)));
  end
  else begin
    if Utf8 then
      sInput := NoBomUtf8ToAnsiString(PlainText)
    else if IsWideString(PlainText) then
      sInput := WideStringToAnsi(PlainText)
    else sInput := PlainText;
    SetWindowText(FRtfWindow, Pointer(sInput));
    iLen := Length(sInput) shl 1;
  end;
  if iLen = 0 then begin
    Result := PlainText;
    Exit;
  end;
  with dbaRtf do begin
    Data := Result;
    CurrentPos := 0;
  end;
  with esRTF do begin
    dwCookie := DWord(@dbaRtf);
    dwError := 0;
    pfnCallback := @RtfStreamSaveA;
  end;
  if SendMessage(FRtfWindow, EM_STREAMOUT, SF_RTF, Longint(@esRTF)) > 0 then with dbaRtf do begin
    Delete(Data, CurrentPos, MAXINT);
    Result := Data;
  end;
end;

begin
  InitIdnSupport;
  InitWinINetLib;
end.

