{$J+,Z4}
unit RichOle;

(*
 *	RICHOLE.H
 *
 *	Purpose:
 *		OLE Extensions to the Rich Text Editor
 *
 *	Copyright (c) 1985-1999, Microsoft Corporation
 *)

interface

uses
  Windows, RichEdit, OLE2, {$IFDEF VER90} OleAuto {$ELSE} ComObj {$ENDIF}, OleConst;


// Structure passed to GetObject and InsertObject
type
TREObject = packed record
  cbStruct	: DWord;			// Size of structure
  cp		: Longint;			// Character position of object
  Clsid		: TClsid;			// Class ID of object
  OleObj	: IOleObject;			// OLE object interface
  Stg		: IStorage;			// Associated storage interface
  OleSite	: IOleClientSite;		// Associated client site interface
  Size		: TSize;			// Size of object (may be 0,0)
  dvAspect	: DWord;			// Display aspect to use
  dwFlags	: DWord;			// Object status flags
  dwUser	: DWord;			// Dword for user's use
end;


// Flags to specify which interfaces should be returned in the structure above
const
  REO_GETOBJ_NO_INTERFACES	= $00000000;
  REO_GETOBJ_POleObj		= $00000001;
  REO_GETOBJ_PStg		= $00000002;
  REO_GETOBJ_POLESITE		= $00000004;
  REO_GETOBJ_ALL_INTERFACES	= $00000007;

  // Place object at selection
  REO_CP_SELECTION		= -1;

  // Use character position to specify object instead of index
  REO_IOB_SELECTION		= -1;
  REO_IOB_USE_CP		= -2;

  // Object flags
  REO_NULL			= $00000000;	// No flags
  REO_READWRITEMASK		= $0000003F;	// Mask out RO bits
  REO_DONTNEEDPALETTE		= $00000020;	// Object doesn't need palette
  REO_BLANK			= $00000010;	// Object is blank
  REO_DYNAMICSIZE		= $00000008;	// Object defines size always
  REO_INVERTEDSELECT		= $00000004;	// Object drawn all inverted if sel
  REO_BELOWBASELINE		= $00000002;	// Object sits below the baseline
  REO_RESIZABLE			= $00000001;	// Object may be resized
  REO_LINK			= $80000000;	// Object is a link (RO)
  REO_STATIC			= $40000000;	// Object is static (RO)
  REO_SELECTED			= $08000000;	// Object selected (RO)
  REO_OPEN			= $04000000;	// Object open in its server (RO)
  REO_INPLACEACTIVE		= $02000000;	// Object in place active (RO)
  REO_HILITED			= $01000000;	// Object is to be hilited (RO)
  REO_LINKAVAILABLE		= $00800000;	// Link believed available (RO)
  REO_GETMETAFILE		= $00400000;	// Object requires metafile (RO)

  // flags for IRichEditOle::GetClipboardData(),
  // IRichEditOleCallback::GetClipboardData() and
  // IRichEditOleCallback::QueryAcceptData()
  RECO_PASTE			= $00000000;	// paste from clipboard
  RECO_DROP			= $00000001;	// drop
  RECO_COPY			= $00000002;	// copy to the clipboard
  RECO_CUT			= $00000003;	// cut to the clipboard
  RECO_DRAG			= $00000004;	// drag


(*
 *	IRichEditOle
 *
 *	Purpose:
 *		Interface used by the client of RichEdit to perform OLE-related
 *		operations.
 *)
type
  IRichEditOle = class(IUnknown)
  public
    function GetClientSite(var OleSite: IOleClientSite): HResult; virtual; stdcall; abstract;
    function GetObjectCount: Longint; virtual; stdcall; abstract;
    function GetLinkCount: Longint; virtual; stdcall; abstract;
    function GetObject(iOb: Longint; var REObject: TREObject; dwFlags: DWord): HResult; virtual; stdcall; abstract;
    function InsertObject(var REObject: TREObject): HResult; virtual; stdcall; abstract;
    function ConvertObject(iOb: Longint; const ClsidNew: TClsid;
			   lpstrUserTypeNew: POleStr): HResult; virtual; stdcall; abstract;
    function ActivateAs(const rClsid, rClsidAs: TClsid): HResult; virtual; stdcall; abstract;
    function SetHostNames(lpstrContainerApp, lpstrContainerObj: POleStr): HResult; virtual; stdcall; abstract;
    function SetLinkAvailable(iOb: Longint; fAvailable: Bool): HResult; virtual; stdcall; abstract;
    function SetDvaspect(iOb: Longint; dvaspect: DWord): HResult; virtual; stdcall; abstract;
    function HandsOffStorage(iOb: Longint): HResult; virtual; stdcall; abstract;
    function SaveCompleted(iOb: Longint; Stg: IStorage): HResult; virtual; stdcall; abstract;
    function InPlaceDeactivate: HResult; virtual; stdcall; abstract;
    function ContextSensitiveHelp(fEnterMode: Bool): HResult; virtual; stdcall; abstract;
    function GetClipboardData(var Chrg: TCharRange; reco: DWord;
			      var DataObj: IDataObject): HResult; virtual; stdcall; abstract;
    function ImportDataObject(DataObj: IDataObject; cf: TClipFormat;
			      hMetaPict: HGLOBAL): HResult; virtual; stdcall; abstract;
  end;

// IID_IRichEditOle interface GUID
const
  IID_IRichEditOle: TGUID = (
    D1:$00020D00;D2:$0000;D3:$0000;D4:($C0,$00,$00,$00,$00,$00,$00,$46)
  );


(*
 *	IRichEditOleCallback
 *
 *	Purpose:
 *		Interface used by the RichEdit to get OLE-related stuff from the
 *		application using RichEdit.
 *)
type
  IRichEditOleCallback = class(IUnknown)
    function GetNewStorage(var Stg: IStorage): HResult; virtual; stdcall; abstract;
    function GetInPlaceContext(var Frame: IOleInPlaceFrame; var Doc: IOleInPlaceUIWindow;
			       FrameInfo: TOleInPlaceFrameInfo): HResult; virtual; stdcall; abstract;
    function ShowContainerUI(fShow: Bool): HResult; virtual; stdcall; abstract;
    function QueryInsertObject(const Clsid: TCLSID; Stg: IStorage; cp: Longint): HResult; virtual; stdcall; abstract;
    function DeleteObject(OleObj: IOleObject): HResult; virtual; stdcall; abstract;
    function QueryAcceptData(DataObj: IDataObject; var cfFormat: TClipFormat; reco: DWord;
			     fReally: Bool; hMetaPict: HGLOBAL): HResult; virtual; stdcall; abstract;
    function ContextSensitiveHelp(fEnterMode: Bool): HResult; virtual; stdcall; abstract;
    function GetClipboardData(var Chrg: TCharRange; reco: DWord;
			      var DataObj: IDataObject): HResult; virtual; stdcall; abstract;
    function GetDragDropEffect(fDrag: Bool; grfKeyState: DWord;
			       var dwEffect: DWord): HResult; virtual; stdcall; abstract;
    function GetContextMenu(SelType: Word; OleObj: IOleObject; var Chrg: TCharRange;
			    var hMenu: HMENU): HResult; virtual; stdcall; abstract;
  end;

// IID_IRichEditOleCallback interface GUID
const
  IID_IRichEditOleCallback: TGUID = (
    D1:$00020D03;D2:$0000;D3:$0000;D4:($C0,$00,$00,$00,$00,$00,$00,$46)
  );


(* from TOM.h *)

// TOM flags
const
  tomFalse			= 0;
  tomTrue			= -1;

  tomUndefined			= -9999999;
  tomToggle			= -9999998;
  tomAutoColor			= -9999997;
  tomDefault			= -9999996;
  tomSuspend			= -9999995;
  tomResume			= -9999994;

  tomApplyNow			= 0;
  tomApplyLater			= 1;
  tomTrackParms			= 2;
  tomCacheParms			= 3;

  tomBackward			= $c0000001;
  tomForward			= $3fffffff;

  tomMove			= 0;
  tomExtend			= 1;

  tomNoSelection		= 0;
  tomSelectionIP		= 1;
  tomSelectionNormal		= 2;
  tomSelectionFrame		= 3;
  tomSelectionColumn		= 4;
  tomSelectionRow		= 5;
  tomSelectionBlock		= 6;
  tomSelectionInlineShape	= 7;
  tomSelectionShape		= 8;

  tomSelStartActive		= 1;
  tomSelAtEOL			= 2;
  tomSelOvertype		= 4;
  tomSelActive			= 8;
  tomSelReplace			= 16;

  tomEnd			= 0;
  tomStart			= 32;

  tomCollapseEnd		= 0;
  tomCollapseStart		= 1;

  tomClientCoord		= 256;

  tomNone			= 0;
  tomSingle			= 1;
  tomWords			= 2;
  tomDouble			= 3;
  tomDotted			= 4;
  tomDash			= 5;
  tomDashDot			= 6;
  tomDashDotDot			= 7;
  tomWave			= 8;
  tomThick			= 9;
  tomHair			= 10;

  tomLineSpaceSingle		= 0;
  tomLineSpace1pt5		= 1;
  tomLineSpaceDouble		= 2;
  tomLineSpaceAtLeast		= 3;
  tomLineSpaceExactly		= 4;
  tomLineSpaceMultiple		= 5;

  tomAlignLeft			= 0;
  tomAlignCenter		= 1;
  tomAlignRight			= 2;
  tomAlignJustify		= 3;

  tomAlignDecimal		= 3;
  tomAlignBar			= 4;

  tomAlignInterWord		= 3;
  tomAlignInterLetter		= 4;
  tomAlignScaled		= 5;
  tomAlignGlyphs		= 6;
  tomAlignSnapGrid		= 7;

  tomSpaces			= 0;
  tomDots			= 1;
  tomDashes			= 2;
  tomLines			= 3;
  tomThickLines			= 4;
  tomEquals			= 5;

  tomTabBack			= -3;
  tomTabNext			= -2;
  tomTabHere			= -1;

  tomListNone			= 0;
  tomListBullet			= 1;
  tomListNumberAsArabic		= 2;
  tomListNumberAsLCLetter	= 3;
  tomListNumberAsUCLetter	= 4;
  tomListNumberAsLCRoman	= 5;
  tomListNumberAsUCRoman	= 6;
  tomListNumberAsSequence	= 7;

  tomListParentheses		= $10000;
  tomListPeriod			= $20000;
  tomListPlain			= $30000;

  tomCharacter			= 1;
  tomWord			= 2;
  tomSentence			= 3;
  tomParagraph			= 4;
  tomLine			= 5;
  tomStory			= 6;
  tomScreen			= 7;
  tomSection			= 8;
  tomColumn			= 9;
  tomRow			= 10;
  tomWindow			= 11;
  tomCell			= 12;
  tomCharFormat			= 13;
  tomParaFormat			= 14;
  tomTable			= 15;
  tomObject			= 16;

  tomMatchWord			= 2;
  tomMatchCase			= 4;
  tomMatchPattern		= 8;

  tomUnknownStory		= 0;
  tomMainTextStory		= 1;
  tomFootnotesStory		= 2;
  tomEndnotesStory		= 3;
  tomCommentsStory		= 4;
  tomTextFrameStory		= 5;
  tomEvenPagesHeaderStory	= 6;
  tomPrimaryHeaderStory		= 7;
  tomEvenPagesFooterStory	= 8;
  tomPrimaryFooterStory		= 9;
  tomFirstPageHeaderStory	= 10;
  tomFirstPageFooterStory	= 11;

  tomNoAnimation		= 0;
  tomLasVegasLights		= 1;
  tomBlinkingBackground		= 2;
  tomSparkleText		= 3;
  tomMarchingBlackAnts		= 4;
  tomMarchingRedAnts		= 5;
  tomShimmer			= 6;
  tomWipeDown			= 7;
  tomWipeRight			= 8;
  tomAnimationMax		= 8;

  tomLowerCase			= 0;
  tomUpperCase			= 1;
  tomTitleCase			= 2;
  tomSentenceCase		= 4;
  tomToggleCase			= 5;

  tomReadOnly			= $100;
  tomShareDenyRead		= $200;
  tomShareDenyWrite		= $400;

  tomPasteFile			= $1000;

  tomCreateNew			= $10;
  tomCreateAlways		= $20;
  tomOpenExisting		= $30;
  tomOpenAlways			= $40;
  tomTruncateExisting		= $50;

  tomRTF			= $1;
  tomText			= $2;
  tomHTML			= $3;
  tomWordDocument		= $4;

  tomBold			= $80000001;
  tomItalic			= $80000002;
  tomUnderline			= $80000004;
  tomStrikeout			= $80000008;
  tomProtected			= $80000010;
  tomLink			= $80000020;
  tomSmallCaps			= $80000040;
  tomAllCaps			= $80000080;
  tomHidden			= $80000100;
  tomOutline			= $80000200;
  tomShadow			= $80000400;
  tomEmboss			= $80000800;
  tomImprint			= $80001000;
  tomDisabled			= $80002000;
  tomRevised			= $80004000;

  tomNormalCaret		= 0;
  tomKoreanBlockCaret		= $1;

  tomIncludeInset		= $1;

  tomIgnoreCurrentFont		= 0;

  tomMatchFontCharset		= $1;
  tomMatchFontSignature		= $2;

  tomCharset			= $80000000;

  tomRE10Mode			= $1;

  tomNoIME			= $80000;
  tomSelfIME			= $40000;


// ITextFont interface
type
  ITextFont = class(IDispatch)
    function GetDuplicate(var Font: ITextFont): HResult; virtual; stdcall; abstract;
    function SetDuplicate(Font: ITextFont): HResult; virtual; stdcall; abstract;
    function CanChange(var B: Longint): HResult; virtual; stdcall; abstract;
    function IsEqual(Font: ITextFont; var B: Longint): HResult; virtual; stdcall; abstract;
    function Reset(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetStyle(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetStyle(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetAllCaps(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetAllCaps(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetAnimation(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetAnimation(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetBackColor(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetBackColor(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetBold(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetBold(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetEmboss(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetEmboss(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetForeColor(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetForeColor(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetHidden(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetHidden(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetEngrave(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetEngrave(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetItalic(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetItalic(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetKerning(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetKerning(Value: Single): HResult; virtual; stdcall; abstract;
    function GetLanguageID(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetLanguageID(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetName(var bstr: TBStr): HResult; virtual; stdcall; abstract;
    function SetName(bstr: TBStr): HResult; virtual; stdcall; abstract;
    function GetOutline(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetOutline(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetPosition(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetPosition(Value: Single): HResult; virtual; stdcall; abstract;
    function GetProtected(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetProtected(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetShadow(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetShadow(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetSize(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetSize(Value: Single): HResult; virtual; stdcall; abstract;
    function GetSmallCaps(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetSmallCaps(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetSpacing(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetSpacing(Value: Single): HResult; virtual; stdcall; abstract;
    function GetStrikeThrough(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetStrikeThrough(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetSubscript(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetSubscript(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetSuperscript(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetSuperscript(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetUnderline(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetUnderline(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetWeight(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetWeight(Value: Longint): HResult; virtual; stdcall; abstract;
  end;

// ITextFont interface GUID
const
  IID_ITextFont: TGUID = (
    D1:$8CC497C3;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


// ITextPara interface
type
  ITextPara = class(IDispatch)
    function GetDuplicate(var Para: ITextPara): HResult; virtual; stdcall; abstract;
    function SetDuplicate(Para: ITextPara): HResult; virtual; stdcall; abstract;
    function CanChange(var B: Longint): HResult; virtual; stdcall; abstract;
    function IsEqual(Para: ITextPara; var B: Longint): HResult; virtual; stdcall; abstract;
    function Reset(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetStyle(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetStyle(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetAlignment(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetAlignment(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetHyphenation(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetHyphenation(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetFirstLineIndent(var Value: Single): HResult; virtual; stdcall; abstract;
    function GetKeepTogether(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetKeepTogether(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetKeepWithNext(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetKeepWithNext(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetLeftIndent(var Value: Single): HResult; virtual; stdcall; abstract;
    function GetLineSpacing(var Value: Single): HResult; virtual; stdcall; abstract;
    function GetLineSpacingRule(var Value: Longint): HResult; virtual; stdcall; abstract;
    function GetListAlignment(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetListAlignment(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetListLevelIndex(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetListLevelIndex(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetListStart(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetListStart(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetListTab(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetListTab(Value: Single): HResult; virtual; stdcall; abstract;
    function GetListType(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetListType(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetNoLineNumber(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetNoLineNumber(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetPageBreakBefore(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetPageBreakBefore(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetRightIndent(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetRightIndent(Value: Single): HResult; virtual; stdcall; abstract;
    function SetIndents(StartIndent, LeftIndent, RightIndent: Single): HResult; virtual; stdcall; abstract;
    function SetLineSpacing(LineSpacingRule: Longint; LineSpacing: Single): HResult; virtual; stdcall; abstract;
    function GetSpaceAfter(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetSpaceAfter(Value: Single): HResult; virtual; stdcall; abstract;
    function GetSpaceBefore(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetSpaceBefore(Value: Single): HResult; virtual; stdcall; abstract;
    function GetWidowControl(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetWidowControl(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetTabCount(var Count: Longint): HResult; virtual; stdcall; abstract;
    function AddTab(tbPos: Single; tbAlign, tbLeader: Longint): HResult; virtual; stdcall; abstract;
    function ClearAllTabs: HResult; virtual; stdcall; abstract;
    function DeleteTab(tbPos: Single): HResult; virtual; stdcall; abstract;
    function GetTab(iTab: Longint; var tbPos: Single; var tbAlign, tbLeader: Longint): HResult; virtual; stdcall; abstract;
  end;

// ITextPara interface GUID
const
  IID_ITextPara: TGUID = (
    D1:$8CC497C4;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


// ITextRange interface
type
  ITextRange = class(IDispatch)
    function GetText(var bstr: TBStr): HResult; virtual; stdcall; abstract;
    function SetText(bstr: TBStr): HResult; virtual; stdcall; abstract;
    function GetChar(var ch: Longint): HResult; virtual; stdcall; abstract;
    function SetChar(ch: Longint): HResult; virtual; stdcall; abstract;
    function GetDuplicate(var Range: ITextRange): HResult; virtual; stdcall; abstract;
    function GetFormattedText(var Range: ITextRange): HResult; virtual; stdcall; abstract;
    function SetFormattedText(Range: ITextRange): HResult; virtual; stdcall; abstract;
    function GetStart(var cpFirst: Longint): HResult; virtual; stdcall; abstract;
    function SetStart(cpFirst: Longint): HResult; virtual; stdcall; abstract;
    function GetEnd(var cpLim: Longint): HResult; virtual; stdcall; abstract;
    function SetEnd(cpLim: Longint): HResult; virtual; stdcall; abstract;
    function GetFont(var Font: ITextFont): HResult; virtual; stdcall; abstract;
    function SetFont(Font: ITextFont): HResult; virtual; stdcall; abstract;
    function GetPara(var Para: ITextPara): HResult; virtual; stdcall; abstract;
    function SetPara(Para: ITextPara): HResult; virtual; stdcall; abstract;
    function GetStoryLength(var cch: Longint): HResult; virtual; stdcall; abstract;
    function GetStoryType(var Value: Longint): HResult; virtual; stdcall; abstract;
    function Collapse(bStart: Longint): HResult; virtual; stdcall; abstract;
    function Expand(TextUnit: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function GetIndex(TextUnit: Longint; var Index: Longint): HResult; virtual; stdcall; abstract;
    function SetIndex(TextUnit, Index, Extend: Longint): HResult; virtual; stdcall; abstract;
    function SetRange(cpActive, cpOther: Longint): HResult; virtual; stdcall; abstract;
    function InRange(Range: ITextRange; var B: Longint): HResult; virtual; stdcall; abstract;
    function InStory(Range: ITextRange; var B: Longint): HResult; virtual; stdcall; abstract;
    function IsEqual(Range: ITextRange; var B: Longint): HResult; virtual; stdcall; abstract;
    function Select: HResult; virtual; stdcall; abstract;
    function StartOf(TextUnit, Extend: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function EndOf(TextUnit, Extend: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function Move(TextUnit, Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveStart(TextUnit, Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveEnd(TextUnit, Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveWhile(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveStartWhile(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveEndWhile(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveUntil(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveStartUntil(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function MoveEndUntil(CSet: PVarData; Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function FindText(bstr: TBStr; cch: Longint; Flags: Longint; var Length: Longint): HResult; virtual; stdcall; abstract;
    function FindTextStart(bstr: TBStr; cch: Longint; Flags: Longint; var Length: Longint): HResult; virtual; stdcall; abstract;
    function FindTextEnd(bstr: TBStr; cch: Longint; Flags: Longint; var Length: Longint): HResult; virtual; stdcall; abstract;
    function Delete(TextUnit, Count: Longint; var Delta: Longint): HResult; virtual; stdcall; abstract;
    function Cut(pVar: PVarData): HResult; virtual; stdcall; abstract;
    function Copy(pVar: PVarData): HResult; virtual; stdcall; abstract;
    function Paste(pVar: PVarData; Format: Longint): HResult; virtual; stdcall; abstract;
    function CanPaste(pVar: PVarData; Format: Longint; var B: Longint): HResult; virtual; stdcall; abstract;
    function CanEdit(var bCanEdit: Longint): HResult; virtual; stdcall; abstract;
    function ChangeCase(CaseType: Longint): HResult; virtual; stdcall; abstract;
    function GetPoint(RangeType: Longint; var x, y: Longint): HResult; virtual; stdcall; abstract;
    function SetPoint(x, y: Longint; RangeType, Extend: Longint): HResult; virtual; stdcall; abstract;
    function ScrollIntoView(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetEmbeddedObject(var Obj: IUnknown): HResult; virtual; stdcall; abstract;
  end;

// ITextRange interface GUID
const
  IID_ITextRange: TGUID = (
    D1:$8CC497C2;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


// ITextStoryRanges interface
type
  ITextStoryRanges = class(IDispatch)
    function _NewEnum(var unkEnum: IUnknown): HResult; virtual; stdcall; abstract;
    function Item(Index: Longint; var Range: ITextRange): HResult; virtual; stdcall; abstract;
    function GetCount(var Count: Longint): HResult; virtual; stdcall; abstract;
  end;

// ITextStoryRanges interface GUID
const
  IID_ITextStoryRanges: TGUID = (
    D1:$8CC497C5;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


// ITextSelection interface
type
  ITextSelection = class(ITextRange)
    function GetName(var Name: TBStr): HResult; virtual; stdcall; abstract;
    function GetSelection(var Sel: ITextSelection): HResult; virtual; stdcall; abstract;
    function GetStoryCount(var Count: Longint): HResult; virtual; stdcall; abstract;
    function GetStoryRanges(var Stories: ITextStoryRanges): HResult; virtual; stdcall; abstract;
    function GetSaved(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetSaved(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetDefaultTabStop(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetDefaultTabStop(Value: Single): HResult; virtual; stdcall; abstract;
    function New: HResult; virtual; stdcall; abstract;
    function Open(pVar: PVarData; Flags, CodePage: Longint): HResult; virtual; stdcall; abstract;
    function Save(pVar: PVarData; Flags, CodePage: Longint): HResult; virtual; stdcall; abstract;
    function Freeze(var Count: Longint): HResult; virtual; stdcall; abstract;
    function Unfreeze(var Count: Longint): HResult; virtual; stdcall; abstract;
    function BeginEditCollection: HResult; virtual; stdcall; abstract;
    function EndEditCollection: HResult; virtual; stdcall; abstract;
    function Undo(Count: Longint; var prop: Longint): HResult; virtual; stdcall; abstract;
    function Redo(Count: Longint; var prop: Longint): HResult; virtual; stdcall; abstract;
    function Range(cp1, cp2: Longint; var Range: ITextRange): HResult; virtual; stdcall; abstract;
    function RangeFromPoint(x, y: Longint; var Range: ITextRange): HResult; virtual; stdcall; abstract;
  end;

// ITextSelection interface GUID
const
  IID_ITextSelection: TGUID = (
    D1:$8CC497C1;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


// ITextDocument interface
type
  ITextDocument = class(IDispatch)
    function GetName(var Name: TBStr): HResult; virtual; stdcall; abstract;
    function GetSelection(var Sel: ITextSelection): HResult; virtual; stdcall; abstract;
    function GetStoryCount(var Count: Longint): HResult; virtual; stdcall; abstract;
    function GetStoryRanges(var Stories: ITextStoryRanges): HResult; virtual; stdcall; abstract;
    function GetSaved(var Value: Longint): HResult; virtual; stdcall; abstract;
    function SetSaved(Value: Longint): HResult; virtual; stdcall; abstract;
    function GetDefaultTabStop(var Value: Single): HResult; virtual; stdcall; abstract;
    function SetDefaultTabStop(Value: Single): HResult; virtual; stdcall; abstract;
    function New: HResult; virtual; stdcall; abstract;
    function Open(pVar: PVarData; Flags: Longint; CodePage: Longint): HResult; virtual; stdcall; abstract;
    function Save(pVar: PVarData; Flags: Longint; CodePage: Longint): HResult; virtual; stdcall; abstract;
    function Freeze(var Count: Longint): HResult; virtual; stdcall; abstract;
    function Unfreeze(var Count: Longint): HResult; virtual; stdcall; abstract;
    function BeginEditCollection: HResult; virtual; stdcall; abstract;
    function EndEditCollection: HResult; virtual; stdcall; abstract;
    function Undo(Count: Longint; var prop: Longint): HResult; virtual; stdcall; abstract;
    function Redo(Count: Longint; var prop: Longint): HResult; virtual; stdcall; abstract;
    function Range(cp1: Longint; cp2: Longint; var Range: ITextRange): HResult; virtual; stdcall; abstract;
    function RangeFromPoint(x: Longint; y: Longint; var Range: ITextRange): HResult; virtual; stdcall; abstract;
  end;

// ITextDocument interface GUID
const
  IID_ITextDocument: TGUID = (
    D1:$8CC497C0;D2:$A1DF;D3:$11CE;D4:($80,$98,$00,$AA,$00,$47,$BE,$5D)
  );


(* from Platform SDK Collection for Windows Server 2003 *)

// Paragraph styles
const
  StyleNormal			= -1;
  StyleHeading1			= -2;
  StyleHeading2			= -3;
  StyleHeading3			= -4;
  StyleHeading4			= -5;
  StyleHeading5			= -6;
  StyleHeading6			= -7;
  StyleHeading7			= -8;
  StyleHeading8			= -9;
  StyleHeading9			= -10;
  StyleIndex1			= -11;
  StyleIndex2			= -12;
  StyleIndex3			= -13;
  StyleIndex4			= -14;
  StyleIndex5			= -15;
  StyleIndex6			= -16;
  StyleIndex7			= -17;
  StyleIndex8			= -18;
  StyleIndex9			= -19;
  StyleTOC1			= -20;
  StyleTOC2			= -21;
  StyleTOC3			= -22;
  StyleTOC4			= -23;
  StyleTOC5			= -24;
  StyleTOC6			= -25;
  StyleTOC7			= -26;
  StyleTOC8			= -27;
  StyleTOC9			= -28;
  StyleNormalIndent		= -29;
  StyleFootnoteText		= -30;
  StyleAnnotationText		= -31;
  StyleHeader			= -32;
  StyleFooter			= -33;
  StyleIndexHeading		= -34;
  StyleCaption			= -35;
  StyleTableofFigures		= -36;
  StyleEnvelopeAddress		= -37;
  StyleEnvelopeReturn		= -38;
  StyleFootnoteReference	= -39;
  StyleAnnotationReference	= -40;
  StyleLineNumber		= -41;
  StylePageNumber		= -42;
  StyleEndnoteReference		= -43;
  StyleEndnoteText		= -44;
  StyleTableofAuthorities	= -45;
  StyleMacroText		= -46;
  StyleTOAHeading		= -47;
  StyleList			= -48;
  StyleListBullet		= -49;
  StyleListNumber		= -50;
  StyleList2			= -51;
  StyleList3			= -52;
  StyleList4			= -53;
  StyleList5			= -54;
  StyleListBullet2		= -55;
  StyleListBullet3		= -56;
  StyleListBullet4		= -57;
  StyleListBullet5		= -58;
  StyleListNumber2		= -59;
  StyleListNumber3		= -60;
  StyleListNumber4		= -61;
  StyleListNumber5		= -62;
  StyleTitle			= -63;
  StyleClosing			= -64;
  StyleSignature		= -65;
  StyleDefaultParagraphFont	= -66;
  StyleBodyText			= -67;
  StyleBodyTextIndent		= -68;
  StyleListContinue		= -69;
  StyleListContinue2		= -70;
  StyleListContinue3		= -71;
  StyleListContinue4		= -72;
  StyleListContinue5		= -73;
  StyleMessageHeader		= -74;
  StyleSubtitle			= -75;
  StyleSalutation		= -76;
  StyleDate			= -77;
  StyleBodyTextFirstIndent	= -78;
  StyleBodyTextFirstIndent2	= -79;
  StyleNoteHeading		= -80;
  StyleBodyText2		= -81;
  StyleBodyText3		= -82;
  StyleBodyTextIndent2		= -83;
  StyleBodyTextIndent3		= -84;
  StyleBlockQuotation		= -85;
  StyleHyperlink		= -86;
  StyleHyperlinkFollowed	= -87;


(* created by idw *)

procedure OleException(Error: Longint);


implementation

procedure OleException(Error: Longint);
var
  pOleError: PChar;
  dwStrLen: DWord;
begin
  dwStrLen := FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM,
			    nil, DWord(Error), 0, @pOleError, 0, nil);
  if dwStrLen > 0 then begin
    try
      dec(dwStrLen);
      while (dwStrLen > 0) and (pOleError[dwStrLen] in [#0..#32, '.']) do dec(dwStrLen);
      pOleError[succ(dwStrLen)] := #0;
      Raise EOleError.Create(pOleError);
    finally
      LocalFree(HLocal(pOleError));
    end;
  end
  {$IFDEF CONDITIONALEXPRESSIONS}
  else Raise EOleError.CreateFmt(SOleError, [Error]);
  {$ELSE}
  else Raise EOleError.CreateResFmt(SOleError, [Error]);
  {$ENDIF}
end;

end.

