{$J+,Z4}
unit RichOleCallback;
{------------------------------------------------------------------------------}
{       This code is copyright (C) 2006-2023 by Michael in der Wiesche;        }
{                        it is based on Mike O'Neill's                         }
{    "A Rich Edit Control that Displays Bitmaps and Other OLE Objects" at      }
{         <http://www.codeproject.com/richedit/COleRichEditCtrl.asp>           }
{------------------------------------------------------------------------------}

interface

uses
  Windows, RichOle, RichEdit, OLE2, SysUtils;

type
  IREOCallback = class(IRichEditOleCallback)
  private
    FRefCount: DWord;
    FLockBytes: ILockBytes;
    FStorage: IStorage;
    FObjectCount: Integer;
    FShowObjects: Longbool;
  public
    constructor Create;
    destructor Destroy; override;

    // Methods of the IUnknown interface
    function QueryInterface(const iid: TIID; var obj): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;

    // Methods of the IRichEditOleCallback interface
    function GetNewStorage(var Stg: IStorage): HResult; override;
    function GetInPlaceContext(var Frame: IOleInPlaceFrame; var Doc: IOleInPlaceUIWindow;
			       FrameInfo: TOleInPlaceFrameInfo): HResult; override;
    function ShowContainerUI(fShow: Bool): HResult; override;
    function QueryInsertObject(const Clsid: TCLSID; Stg: IStorage; cp: Longint): HResult; override;
    function DeleteObject(OleObj: IOleObject): HResult; override;
    function QueryAcceptData(DataObj: IDataObject; var cfFormat: TClipFormat; reco: DWord;
			     fReally: Bool; hMetaPict: HGLOBAL): HResult; override;
    function ContextSensitiveHelp(fEnterMode: Bool): HResult; override;
    function GetClipboardData(var Chrg: TCharRange; reco: DWord;
			      var DataObj: IDataObject): HResult; override;
    function GetDragDropEffect(fDrag: Bool; grfKeyState: DWord;
			       var dwEffect: DWord): HResult; override;
    function GetContextMenu(SelType: Word; OleObj: IOleObject; var Chrg: TCharRange;
			    var hMenu: HMENU): HResult; override;
    property ObjectCount: Integer read FObjectCount;
    property ShowObjects: Longbool read FShowObjects write FShowObjects;
  end;


implementation

constructor IREOCallback.Create;
var
  OleError: HResult;
begin
  OleError := E_OUTOFMEMORY;
  inherited Create;
  try
    OleError := CreateILockBytesOnHGlobal(0, true, FLockBytes);
    if OleError <> S_OK then Exit;
    OleError := StgCreateDocfileOnILockBytes(FLockBytes, STGM_SHARE_EXCLUSIVE or STGM_CREATE or STGM_READWRITE,
					     0, FStorage);
  finally
    if OleError <> S_OK then Free;
  end;
end;

destructor IREOCallback.Destroy;
begin
  if FStorage <> nil then FStorage.Release;
  if FLockBytes <> nil then FLockBytes.Release;
  inherited Destroy;
end;

// Methods of the IUnknown interface

function IREOCallback.QueryInterface(const iid: TIID; var obj): HResult;
begin
  if IsEqualIID(iid, IID_IUnknown) or IsEqualIID(iid, IID_IRichEditOleCallback) then begin
    IUnknown(obj) := Self;
    Result := S_OK;
  end
  else Result := E_NOINTERFACE;
end;

function IREOCallback.AddRef: Longint;
begin
  Result := FRefCount;
end;

function IREOCallback.Release: Longint;
begin
  Result := FRefCount;
end;

// Methods of the IRichEditOleCallback Interface

function IREOCallback.GetNewStorage(var Stg: IStorage): HResult;
begin
  Result := E_NOTIMPL;
  if not FShowObjects then Exit;
  Result := FStorage.CreateStorage('', STGM_SHARE_EXCLUSIVE or STGM_CREATE or STGM_READWRITE, 0, 0, Stg);
end;

function IREOCallback.GetInPlaceContext(var Frame: IOleInPlaceFrame; var Doc: IOleInPlaceUIWindow;
					FrameInfo: TOleInPlaceFrameInfo): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.ShowContainerUI(fShow: Bool): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.QueryInsertObject(const Clsid: TCLSID; Stg: IStorage; cp: Longint): HResult;
begin
  if FShowObjects then begin
    inc(FObjectCount);
    Result := S_OK;
  end
  else Result := S_FALSE;
end;

function IREOCallback.DeleteObject(OleObj: IOleObject): HResult;
begin
  dec(FObjectCount);
  Result := E_NOTIMPL;
end;

function IREOCallback.QueryAcceptData(DataObj: IDataObject; var cfFormat: TClipFormat; reco: DWord;
				      fReally: Bool; hMetaPict: HGLOBAL): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.ContextSensitiveHelp(fEnterMode: Bool): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.GetClipboardData(var Chrg: TCharRange; reco: DWord; var DataObj: IDataObject): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.GetDragDropEffect(fDrag: Bool; grfKeyState: DWord; var dwEffect: DWord): HResult;
begin
  Result := E_NOTIMPL;
end;

function IREOCallback.GetContextMenu(SelType: Word; OleObj: IOleObject; var Chrg: TCharRange; var hMenu: HMENU): HResult;
begin
  Result := E_NOTIMPL;
end;

end.

