{$J+,Z4}
unit StrUtil;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2005-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, ShlObj, RichEdit, Classes, SysUtils;

const // 32bit string length offset
  SLO = 4;

type
  TTrimSet = Set of Char;

  DChar = Array[0..1] of Char;
  PDChar = ^DChar;
  QChar = Array[0..3] of Char;
  PQChar = ^QChar;

  PDigitArrayA = ^TDigitArrayA;
  TDigitArrayA = Array[0..2] of Char;

  PPathArray = ^TPathArray;
  TPathArray = Array[0..MAX_PATH] of Char;

  PAStrings = ^TAStrings;
  TAStrings = Array[0..0] of String;
  TStringAdjust = function(const Str: String): String;
  TStringArray = Object
  private
    FCount: Integer;
    FStrings: PAStrings;
    FStringAdjust: TStringAdjust;
    function Grow(Items: Integer): Integer;
  protected
    function AdjustString(const Str: String): String;
   public
    procedure New;
    procedure Clear;
    function  Count: Integer;
    function  GetString(Index: Integer): String;
    procedure SetString(Index: Integer; const Value: String);
    procedure AddLines(const Values: Array of String);
    procedure AddText(const Value: String);
    function  GetText: String;
    property  StringAdjust: TStringAdjust read FStringAdjust write FStringAdjust;
  end;

  TFVIParameter = (
    fviFixed,
    fviComments,
    fviCompanyName,
    fviFileDescription,
    fviFileVersion,
    fviInternalName,
    fviLegalCopyright,
    fviLegalTrademarks,
    fviOriginalFilename,
    fviPrivateBuild,
    fviProductName,
    fviProductVersion,
    fviSpecialBuild
  );
  TFVIParameters = Set of TFVIParameter;
  TFileVersionInfo = Record
    QueryFile: String;
    QueryLanguage: String;
    QueryParameters: TFVIParameters;
    Fixed: TVSFixedFileInfo;
    Comments: String;
    CompanyName: String;
    FileDescription: String;
    FileVersion: String;
    InternalName: String;
    LegalCopyright: String;
    LegalTrademarks: String;
    OriginalFilename: String;
    PrivateBuild: String;
    ProductName: String;
    ProductVersion: String;
    SpecialBuild: String;
  end;

  TDlgFont = Record
    Name: String;
    Size: Integer;
  end;

const
  E = '';
  SP = ' ';
  CL = ':';
  CM = ',';
  SC = ';';
  DA = '-';
  EQ = '=';
  UL = '_';
  QU = '"';
  SQ = '''';
  DC = '#';
  US = '$';
  AM = '&';
  PC = '%';

  SL = '/';
  BS = '\';

  LT = '<';
  GT = '>';

  LB = '(';
  RB = ')';

  LS = '[';
  RS = ']';

  LC = '{';
  RC = '}';

  AT = '@';
  QM = '?';
  EM = '!';
  AST = '*';
  DOT = '.';
  DOTS = ' ...';
  NULL = #0;
  TAB = #9;
  LF = #10;
  CR = #13;

  DS = SL + SL;
  TS = DS + SL;

  CRLF = CR + LF;
  PARA = CRLF + CRLF;

  TRIMCHARS = [#01..#32];
  DIGITS = ['0'..'9'];
  MAX_UTF8_BYTES = 6;

const
  CSIDL_DESKTOP			= $0000;	// <desktop>
  CSIDL_INTERNET		= $0001;	// Internet Explorer (icon on desktop)
  CSIDL_PROGRAMS		= $0002;	// Start Menu\Programs
  CSIDL_CONTROLS		= $0003;	// My Computer\Control Panel
  CSIDL_PRINTERS		= $0004;	// My Computer\Printers
  CSIDL_PERSONAL		= $0005;	// My Documents
  CSIDL_FAVORITES		= $0006;	// <user name>\Favorites
  CSIDL_STARTUP			= $0007;	// Start Menu\Programs\Startup
  CSIDL_RECENT			= $0008;	// <user name>\Recent
  CSIDL_SENDTO			= $0009;	// <user name>\SendTo
  CSIDL_BITBUCKET		= $000A;	// <desktop>\Recycle Bin
  CSIDL_STARTMENU		= $000B;	// <user name>\Start Menu
  CSIDL_MYDOCUMENTS		= $000C;	// logical "My Documents" desktop icon
  CSIDL_MYMUSIC			= $000D;	// "My Music" folder
  CSIDL_MYVIDEO			= $000E;	// "My Videos" folder
  CSIDL_DESKTOPDIRECTORY	= $0010;	// <user name>\Desktop
  CSIDL_DRIVES			= $0011;	// My Computer
  CSIDL_NETWORK			= $0012;	// Network Neighborhood (My Network Places)
  CSIDL_NETHOOD			= $0013;	// <user name>\nethood
  CSIDL_FONTS			= $0014;	// windows\fonts
  CSIDL_TEMPLATES		= $0015;
  CSIDL_COMMON_STARTMENU	= $0016;	// All Users\Start Menu
  CSIDL_COMMON_PROGRAMS		= $0017;	// All Users\Start Menu\Programs
  CSIDL_COMMON_STARTUP		= $0018;	// All Users\Startup
  CSIDL_COMMON_DESKTOPDIRECTORY	= $0019;	// All Users\Desktop
  CSIDL_APPDATA			= $001A;	// <user name>\Application Data
  CSIDL_PRINTHOOD		= $001B;	// <user name>\PrintHood
  CSIDL_LOCAL_APPDATA		= $001c;	// <user name>\Local Settings\Application Data (non roaming)
  CSIDL_ALTSTARTUP		= $001d;	// non localized startup
  CSIDL_COMMON_ALTSTARTUP	= $001e;	// non localized common startup
  CSIDL_COMMON_FAVORITES	= $001f;
  CSIDL_INTERNET_CACHE		= $0020;
  CSIDL_COOKIES			= $0021;
  CSIDL_HISTORY			= $0022;
  CSIDL_COMMON_APPDATA		= $0023;	// All Users\Application Data
  CSIDL_WINDOWS			= $0024;	// GetWindowsDirectory()
  CSIDL_SYSTEM			= $0025;	// GetSystemDirectory()
  CSIDL_PROGRAM_FILES		= $0026;	// C:\Program Files
  CSIDL_MYPICTURES		= $0027;	// C:\Program Files\My Pictures
  CSIDL_PROFILE			= $0028;	// USERPROFILE
  CSIDL_SYSTEMX86		= $0029;	// x86 system directory on RISC
  CSIDL_PROGRAM_FILESX86	= $002a;	// x86 C:\Program Files on RISC
  CSIDL_PROGRAM_FILES_COMMON	= $002b;	// C:\Program Files\Common
  CSIDL_PROGRAM_FILES_COMMONX86	= $002c;	// x86 Program Files\Common on RISC
  CSIDL_COMMON_TEMPLATES	= $002d;	// All Users\Templates
  CSIDL_COMMON_DOCUMENTS	= $002e;	// All Users\Documents
  CSIDL_COMMON_ADMINTOOLS	= $002f;	// All Users\Start Menu\Programs\Administrative Tools
  CSIDL_ADMINTOOLS		= $0030;	// <user name>\Start Menu\Programs\Administrative Tools
  CSIDL_CONNECTIONS		= $0031;	// Network and Dial-up Connections
  CSIDL_COMMON_MUSIC		= $0035;	// All Users\My Music
  CSIDL_COMMON_PICTURES		= $0036;	// All Users\My Pictures
  CSIDL_COMMON_VIDEO		= $0037;	// All Users\My Video
  CSIDL_RESOURCES		= $0038;	// Resource Direcotry
  CSIDL_RESOURCES_LOCALIZED	= $0039;	// Localized Resource Direcotry
  CSIDL_COMMON_OEM_LINKS	= $003a;	// Links to All Users OEM specific apps
  CSIDL_CDBURN_AREA		= $003b;	// USERPROFILE\Local Settings\Application Data\Microsoft\CD Burning
  // unused			= $003c;
  CSIDL_COMPUTERSNEARME		= $003d;	// Computers Near Me (computered from Workgroup membership)
  CSIDL_FLAG_CREATE		= $8000;	// combine with CSIDL_ value to force folder creation in SHGetFolderPath()
  CSIDL_FLAG_DONT_VERIFY	= $4000;	// combine with CSIDL_ value to return an unverified folder path
  CSIDL_FLAG_NO_ALIAS		= $1000;	// combine with CSIDL_ value to insure non-alias versions of the pidl
  CSIDL_FLAG_PER_USER_INIT	= $0800;	// combine with CSIDL_ value to indicate per-user init (eg. upgrade)
  CSIDL_FLAG_MASK		= $FF00;	// mask for all possible flag values

function AsciiLoCase(C: Char): Char;
function AsciiUpCase(C: Char): Char;
procedure ByteToHex(Byte: Byte; Hex: PChar);
procedure WordToHex(Word: Word; Hex: PChar);
function  CardToHex(Card: Cardinal): String;
function  CardToStr(Card: Cardinal): String;
function  StrToCard(const Str: String): Cardinal;
function  StrToCardDef(const Str: String; Def: Cardinal): Cardinal;
function  HexToDigit(Hex: Char): Byte;
function  DigitToHex(Digit: Byte): Char;
function  HexToChar(Hex: PChar): Char;
function  HexByteToDigit(HexByte: PChar; NullPad: Longbool): Byte;
function  ByteToDigitChars(Value: Byte; PResult: PChar): Byte;
function  StrToHexChars(const Str: String; Hex: PChar; Size: Cardinal): PChar;
function  StrToHex(const Str: String; Size: Integer): String;
function  StrLAppend(Dest, Source: PChar; Len: Integer; AddLineBreak: Longbool): PChar;
function  StrLPas(Source: PChar; Length: Integer): String;
function  ScanBack(const Str: PChar; Char: Char): PChar;
function  Unwrap(const Str: String): String;
function  Unescape(const Str: String; SkipSpaces: Longbool; EscapeChar: Char): String;
procedure FillByte(var X; Count: Cardinal; Value: Byte);
function  FastStrLen(Str: PChar): Cardinal;
function  WideStrLen(Str: PWideChar): Cardinal;
function  AnsiStrLCheck(Str: PChar; MaxLen: Cardinal): Boolean;
function  WideStrLCheck(Str: PWideChar; MaxLen: Cardinal): Boolean;
function  WideCharToUtf8(Wide: WideChar): String;
function  Utf8ToWideChar(Utf8: PChar; var CharCount: Integer): WideChar;
function  PWideCharToWideStr(Wide: PWideChar): String;
function  StrToWideStr(const Str: String): String;
function  Trim(const Str: String): String;
function  TrimLeft(const Str: String): String;
function  TrimRight(const Str: String): String;
function  TrimCustom(const Str: String; const Chars: TTrimSet): String;
function  InsertStrings(const Format: String; const Strings: Array of String): String;
function  DateString(const ST: TSystemTime; const Format: String): String;
function  TimeString(const ST: TSystemTime; const Format: String): String;
function  HashedString(const Str: String; Len: Integer): String;
function  RtfToAnsiString(const RtfText: String): String;
function  GetPCharHandleA(Text: PAnsiChar; Size: Integer; ReallocHandle: THandle; Local: Longbool): THandle;
function  GetPCharHandleW(Text: PWideChar; Size: Integer; ReallocHandle: THandle; Local: Longbool): THandle;
function  GetStringHandleA(const Str: String; ReallocHandle: THandle; Local: Longbool): THandle;
function  GetStringHandleW(const Str: String; ReallocHandle: THandle; Local: Longbool): THandle;
function  GetHandleStringA(Handle: THandle; Size: Integer; Free: Longbool): String;
function  GetHandleStringW(Handle: THandle; Size: Integer; Free: Longbool): String;
function  CompareStr(const Str1, Str2: String): Boolean;
function  CompareTxt(const Str1, Str2: String): Boolean;
function  CompareStrMax(const Str1, Str2: String; MaxLen: Integer): Boolean;
function  CompareTxtMax(const Str1, Str2: String; MaxLen: Integer): Boolean;
function  LowerFirstChar(const Str: String): String;
function  UpperFirstChar(const Str: String): String;
procedure LowerChar(var Str: String; Pos: Integer);
procedure UpperChar(var Str: String; Pos: Integer);
procedure LowerChars(var Str: String; Count: Integer);
procedure UpperChars(var Str: String; Count: Integer);
function  LowerStr(const Str: String): String;
function  UpperStr(const Str: String): String;
function  DelAmp(const Text: String): String;
function  DelAngles(const Text: String): String;
function  DelQuotes(const Text: String): String;
function  Unquote(const Str: String; CBeg, CEnd: Char): String;
function  CharCopy(const Text: String; Limit: Char): String;
function  CharExtract(var Text: String; Limit: Char): String;
function  ColonExtract(var Text: String): String;
function  CommaExtract(var Text: String): String;
function  SpaceExtract(var Text: String): String;
function  TabExtract(var Text: String): String;
function  LineExtract(var Text: String): String;
function  TrimmedLine(var Text: String): String;
function  GetFirstLine(var Text: String): String;
function  CopyLine(const Str: String; Num: Integer): String;
function  AddSpaceLine(const Text: String): String;
function  AdjustLineBreaks(const Str: String): String;
function  ExtractBaseName(const Name: String): String;
function  ExtractFileExt(const Name: String): String;
function  ExtractFileName(const Name: String): String;
function  ExtractFilePath(const Name: String): String;
function  ExtractFileDrive(const Name: String): String;
function  ExtractShortPath(const Name: String): String;
function  ChangeFileExt(const Name, Ext: String): String;
function  GetLineCount(const Text: String): Integer;
function  GetCharCount(Find: Char; const Text: String): Integer;
function  GetCharCountFrom(Find: Char; const Text: String; Pos: Integer): Integer;
function  GetCharCountUpTo(Find: Char; const Text: String; Pos: Integer): Integer;
function  NextNonSpaceChar(const Text: String; BegPos: Integer): Integer;
function  PrevNonSpaceChar(const Text: String; EndPos: Integer): Integer;
function  FirstCharPos(Find: Char; const Dest: String): Integer;
function  LastCharPos(Find: Char; const Dest: String): Integer;
function  NextCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer;
function  PrevCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer;
function  FirstRangeCharPos(Find: Char; const Dest: String; LoPos, HiPos: Integer): Integer;
function  LastRangeCharPos(Find: Char; const Dest: String; HiPos, LoPos: Integer): Integer;
function  FirstLoCharPos(Find: Char; const Dest: String): Integer;
function  LastLoCharPos(Find: Char; const Dest: String): Integer;
function  NextLoCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer;
function  PrevLoCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer;
function  FirstRangeLoCharPos(Find: Char; const Dest: String; LoPos, HiPos: Integer): Integer;
function  LastRangeLoCharPos(Find: Char; const Dest: String; HiPos, LoPos: Integer): Integer;
function  GetStrCount(const Find, Text: String): Integer;
function  FirstStrPos(const Find, Dest: String): Integer;
function  LastStrPos(const Find, Dest: String): Integer;
function  NextStrPos(const Find, Dest: String; BegPos: Integer): Integer;
function  PrevStrPos(const Find, Dest: String; EndPos: Integer): Integer;
function  FirstStrPartInRange(const Find, Dest: String; PartLen, LoPos, HiPos: Integer): Integer;
function  FirstRangeStrPos(const Find, Dest: String; LoPos, HiPos: Integer): Integer;
function  LastRangeStrPos(const Find, Dest: String; HiPos, LoPos: Integer): Integer;
function  StrStart(const Find, Dest: String): Boolean;
function  StrAtPos(const Find, Dest: String; Pos: Integer): Boolean;
function  FirstLoStrPos(const Find, Dest: String): Integer;
function  LastLoStrPos(const Find, Dest: String): Integer;
function  NextLoStrPos(const Find, Dest: String; BegPos: Integer): Integer;
function  PrevLoStrPos(const Find, Dest: String; EndPos: Integer): Integer;
function  FirstLoStrPartInRange(const Find, Dest: String; PartLen, LoPos, HiPos: Integer): Integer;
function  FirstRangeLoStrPos(const Find, Dest: String; LoPos, HiPos: Integer): Integer;
function  LastRangeLoStrPos(const Find, Dest: String; HiPos, LoPos: Integer): Integer;
function  LoStrStart(const Find, Dest: String): Boolean;
function  LoStrAtPos(const Find, Dest: String; Pos: Integer): Boolean;
procedure DeleteChar(Ch: Char; var Str: String);
procedure XChgChars(var Str: String; Old, New: Char);
function  XChgStrings(const Str, Old, New: String): String;
function  XChgLoStrings(const Str, Old, New: String): String;
function  XChgStrAtPos(const Str, New: String; OldPos, OldLen: Integer): String;
function  EnclosedStr(const Str: String; Left, Right: Char; Pos: Integer): String;
function  ExtractStr(var Source: String; Pos, Len: Integer): String;
function  AlphaCompareStr(const Str1, Str2: String): Integer;
function  AlphaCompareText(const Str1, Str2: String): Integer;
function  SameStr(const Str1, Str2: String): Longbool;
function  SameText(const Str1, Str2: String): Longbool;
function  RegEntry(KeyIndex: HKEY; const KeyPath: String): BOOL;
function  RegDWord(KeyIndex: HKEY; const KeyPath, KeyEntry: String): DWORD;
function  RegString(KeyIndex: HKEY; const KeyPath, KeyEntry: String): String;
function  IniDWord(const IniFile, Section, Item: String): DWORD;
function  IniString(const IniFile, Section, Item: String): String;
function  ExpandEnvironmentStrings(const Str: String): String;
function  ModuleString(Module: HMODULE; ID: UINT): String;
function  ResString(ID: UINT): String;
function  MenuString(Menu: HMENU; ID, Flags: UINT): String;
function  ClassString(Window: HWND): String;
function  WndString(Window: HWND): String;
function  SysErrorString(Code: Cardinal): String;
function  WriteToConsole(const Text: String): Longbool;
function  DialogItemInt(Dialog: HWND; ItemID: Integer): Integer;
function  DialogItemString(Dialog: HWND; ItemID: Integer): String;
function  DialogFontInfo(Module: HMODULE; DialogID: PChar): TDlgFont;
function  DialogCaptionString(Module: HMODULE; DialogID: PChar): String;
function  DialogControlString(Module: HMODULE; DialogID: PChar; ControlID: UINT): String;
function  GetSystemFileVersion: Cardinal;
function  GetSimpleFileVersion(const FileName: String; MainOnly: Longbool): Cardinal;
function  GetWinVersion(Simple, AddNumber, AddSvcPack, AddCpuType: Longbool): String;
function  IsMinWinVersion(const MinWinVersion: String): Longbool;
function  MainWinVersion: Cardinal;
function  GetComputerName: String;
function  GetUserName: String;
function  GetAppDir: String;
function  GetCurDir: String;
function  GetWinDir: String;
function  GetSysDir: String;
function  GetTempDir: String;
function  GetTempFile: String;
function  GetModuleFileName(Module: THandle): String;
function  GetModuleFilePath(Module: THandle): String;
function  GetSystemFolderPath(FolderID: Integer): String;
procedure AddBackslash(var Path: String; PathLen: Integer);
function  FileVersionInfo(var FileVersionInfo: TFileVersionInfo): Longbool;
function  MoveFile(const Source, Dest: String; Overwrite: Longbool): Longbool;
function  WriteTextFile(const Name, Text: String): Longbool;
function  ReadTextFile(const Name: String): String;
function  CreateWriteFile(const Name: String): THandle;
function  OpenReadFile(const Name: String): THandle;
function  CloseFile(const Handle: THandle): Longbool;
function  DeleteFile(const Name: String): Longbool;
function  FileSize(const Name: String): Cardinal;
function  FileExists(const Name: String): Longbool;
function  DirExists(const Name: String): Longbool;
function  SetCurDir(Name: String): Longbool;

function SendKeys(const Keys: Array of Cardinal): Longbool;

function RtfToClipboard(const RtfString: String): Longbool;
function HtmlToClipboard(const HtmlString: String): Longbool;

function HashData(pbData: PBYTE; cbData: DWORD; pbHash: PBYTE; cbHash: DWORD): HRESULT; stdcall;
	 external 'shlwapi.dll' name 'HashData';


implementation

uses
  UTF8;


// TStringArray ------------------------------------------------------------------------------------

function TStringArray.Grow(Items: Integer): Integer;
begin
  Result := FCount;
  FCount := Result + Items;
  ReallocMem(FStrings, SizeOf(TAStrings) * FCount);
  Initialize(FStrings^[Result], Items);
end;

procedure TStringArray.New;
begin
  FCount := 0;
  FStrings := nil;
end;

function TStringArray.AdjustString(const Str: String): String;
begin
  if Assigned(FStringAdjust) then
    Result := FStringAdjust(Str)
  else Result := Str;
end;

procedure TStringArray.Clear;
begin
  if FStrings <> nil then begin
    if FCount > 0 then Finalize(FStrings^, FCount);
    FreeMem(FStrings);
    FStrings := nil;
    FCount := 0;
  end;
end;

function TStringArray.Count: Integer;
begin
  Result := FCount;
end;

function TStringArray.GetString(Index: Integer): String;
begin
  if (Index >= 0) and (Index < FCount) then
    Result := FStrings^[Index]
  else Result := E;
end;

procedure TStringArray.SetString(Index: Integer; const Value: String);
begin
  if (Index >= 0) and (Index < FCount) then FStrings^[Index] := Value;
end;

function TStringArray.GetText: String;
var
  iIndex, iLen: Integer;
  pResult: PChar;
begin
  if FCount > 0 then begin
    iLen := 0;
    for iIndex := 0 to pred(FCount) do inc(iLen, Length(FStrings^[iIndex]) + Length(CRLF));
    System.SetString(Result, nil, iLen);
    pResult := Pointer(Result);
    for iIndex := 0 to pred(FCount) do begin
      iLen := Length(FStrings^[iIndex]);
      if iLen > 0 then begin
	Move(Pointer(FStrings^[iIndex])^, pResult^, iLen);
	inc(pResult, iLen);
      end;
      PDChar(pResult)^ := CRLF;
      inc(pResult, Length(CRLF));
    end;
  end
  else Result := E;
end;

procedure TStringArray.AddLines(const Values: Array of String);
var
  iCurrent, iIndex: Integer;
begin
  iCurrent := Grow(succ(High(Values)));
  for iIndex := 0 to High(Values) do FStrings^[iCurrent + iIndex] := AdjustString(Values[iIndex]);
end;

procedure TStringArray.AddText(const Value: String);
var
  iOld, iNew, iBeg, iEnd, iIndex: Integer;
begin
  if Value <> E then begin
    iNew := GetCharCount(LF, Value);
    if not (Value[Length(Value)] in [LF, CR]) then inc(iNew);
    iOld := Grow(iNew);
    iBeg := 1;
    iEnd := 1;
    for iIndex := iOld to pred(FCount) do begin
      while not (Value[iEnd] in [NULL, LF, CR]) do inc(iEnd);
      if iEnd > iBeg then FStrings^[iIndex] := AdjustString(Copy(Value, iBeg, iEnd - iBeg));
      while Value[iEnd] in [LF, CR] do inc(iEnd);
      iBeg := iEnd;
    end;
  end;
end;


// NonObject routines ------------------------------------------------------------------------------

function AsciiLoCase(C: Char): Char; register; assembler;
asm	// AL = Hex
  CMP	AL,'A'
  JB    @EXIT
  CMP	AL,'Z'
  JA	@EXIT
  OR	EAX,20h
  RET
  @EXIT:
  MOVZX	EAX,AL
end;

function AsciiUpCase(C: Char): Char; register; assembler;
asm	// AL = Hex
  CMP	AL,'a'
  JB    @EXIT
  CMP	AL,'z'
  JA	@EXIT
  AND	EAX,0DFh
  RET
  @EXIT:
  MOVZX	EAX,AL
end;

procedure ByteToHex(Byte: Byte; Hex: PChar); register; assembler;
asm	// EAX = Byte, EDX = @Hex
  OR	EDX,EDX
  JZ	@EXIT
  MOV	AH,AL
  SHR	AH,4
  AND	AH,0Fh
  ADD	AH,'0'
  CMP	AH,'9'
  JBE	@HI_NIBBLE
  ADD	AH,'A' - '0' - 0Ah
  @HI_NIBBLE:
  MOV	[EDX],AH
  AND	AL,0Fh
  ADD	AL,'0'
  CMP	AL,'9'
  JBE	@LO_NIBBLE
  ADD	AL,'A' - '0' - 0Ah
  @LO_NIBBLE:
  MOV	[EDX + 01h],AL
  @EXIT:
end;

procedure WordToHex(Word: Word; Hex: PChar); register; assembler;
asm	// EAX = Word, EDX = @Hex
  OR	EDX,EDX
  JZ	@EXIT
  MOV	CH,AL
  MOV	AL,AH
  MOV	CL,2
  @LOOP:
  MOV	AH,AL
  SHR	AH,4
  AND	AH,0Fh
  ADD	AH,'0'
  CMP	AH,'9'
  JBE	@HI_NIBBLE
  ADD	AH,'A' - '0' - 0Ah
  @HI_NIBBLE:
  MOV	[EDX],AH
  INC	EDX
  AND	AL,0Fh
  ADD	AL,'0'
  CMP	AL,'9'
  JBE	@LO_NIBBLE
  ADD	AL,'A' - '0' - 0Ah
  @LO_NIBBLE:
  MOV	[EDX],AL
  INC	EDX
  MOV	AL,CH
  DEC	CL
  JNE	@LOOP
  @EXIT:
end;

function CardToHex(Card: Cardinal): String; register; assembler;
asm	// EAX = Card, EDX = @Result
  PUSH	EAX
  PUSH	EDX
  MOV	EAX,8
  CALL	System.@NewAnsiString
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDX,EAX
  MOV	EAX,[ESP]
  MOV	ECX,4
  DEC	EDX
  @LOOP:
  MOV	AH,AL
  AND	AL,0Fh
  ADD	AL,'0'
  CMP	AL,'9'
  JBE	@LO_NIBBLE
  ADD	AL,'A' - '0' - 0Ah
  @LO_NIBBLE:
  MOV	[EDX + ECX * 2],AL
  SHR	AH,4
  AND	AH,0Fh
  ADD	AH,'0'
  CMP	AH,'9'
  JBE	@HI_NIBBLE
  ADD	AH,'A' - '0' - 0Ah
  @HI_NIBBLE:
  MOV	[EDX + ECX * 2 - 1],AH
  POP	EAX
  SHR	EAX,8
  PUSH	EAX
  DEC	CL
  JNE	@LOOP
  POP	EAX
end;

function CardToStr(Card: Cardinal): String; register; assembler;
asm	// EAX = Card, EDX = @Result
  PUSH	EAX
  PUSH	EDX
  MOV	EAX,10
  CALL	System.@NewAnsiString
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDX,EAX
  MOV	ECX,EAX
  POP	EAX
  PUSH	EBX
  MOV	EBX,EDX
  ADD	EBX,9
  CMP	EAX,1000000000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,100000000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,10000000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,1000000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,100000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,10000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,1000
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,100
  JNB	@BEGIN
  dec	EBX
  CMP	EAX,10
  JNB	@BEGIN
  dec	EBX
  @BEGIN:
  SUB	ECX,EBX
  PUSH	EDX
  NEG	ECX
  INC	ECX
  MOV	[EDX - SLO],ECX
  MOV	BYTE PTR [EBX + 01h],0
  MOV	ECX,10
  @LOOP:
  XOR	EDX,EDX
  DIV	ECX
  ADD	DL,'0'
  MOV	[EBX],DL
  DEC	EBX
  OR	EAX,EAX
  JNE	@LOOP
  @END:
  POP	EDX
  POP	EBX
end;

function StrToCard(const Str: String): Cardinal; register; assembler;
const
  P_HEX = ord('$');
  C_HEX = ord('X') shl 8 + ord('0');
asm	// EAX = @Str
  PUSH	EBX
  OR	EAX,EAX
  JE	@ILLEGAL
  MOV	ECX,[EAX - SLO]
  DEC	ECX
  MOV	EBX,EAX
  JE	@CHAR
  ADD	EBX,ECX
  NEG	ECX
  CMP	ECX,-1
  JA	@DECZERO
  CMP	BYTE PTR [EAX],P_HEX
  JE	@PHEX
  CMP	ECX,-2
  JA	@DECZERO
  MOV	AX,[EAX]
  AND	AX,0DFFFh
  CMP	AX,C_HEX
  JE	@CHEX
  @DECZERO:
  CMP	BYTE PTR [EBX + ECX],'0'
  JNE	@DECBEG
  INC	ECX
  JL	@DECZERO
  @DECBEG:
  CMP	ECX,-9
  JL	@OVERFLOW
  @CHAR:
  XOR	EDX,EDX
  XOR	EAX,EAX
  @DECLOOP:
  MOV	DL,[EBX + ECX]
  SUB	DL,'0'
  JB	@ILLEGAL
  CMP	DL,0Ah
  JA	@ILLEGAL
  @DECNEXT:
  PUSH	EDX
  MOV	EDX,0Ah
  MUL	EDX
  POP	EDX
  JC	@OVERFLOW
  ADD	EAX,EDX
  JC	@OVERFLOW
  INC	ECX
  JLE	@DECLOOP
  POP	EBX
  RET
  @CHEX:
  INC	ECX
  @PHEX:
  INC	ECX
  @HEXZERO:
  CMP	BYTE PTR [EBX + ECX],'0'
  JNE	@HEXBEG
  INC	ECX
  JLE	@HEXZERO
  XOR	EAX,EAX
  POP	EBX
  RET
  @HEXBEG:
  CMP	ECX,-7
  JL	@OVERFLOW
  XOR	EDX,EDX
  XOR	EAX,EAX
  @HEXLOOP:
  MOV	DL,[EBX + ECX]
  SUB	DL,'0'
  JB	@ILLEGAL
  CMP	DL,0Ah
  JB	@HEXNEXT
  AND	DL,0DFh
  SUB	DL,'A' - '0'
  JB	@ILLEGAL
  CMP	DL,05h
  JA	@ILLEGAL
  ADD	DL,0Ah
  @HEXNEXT:
  SHL	EAX,4
  OR	AL,DL
  INC	ECX
  JLE	@HEXLOOP
  POP	EBX
  RET
  @ILLEGAL:
  MOV	EAX,EXCEPTION_ILLEGAL_INSTRUCTION
  JMP	@EXCEPTION
  @OVERFLOW:
  MOV	EAX,EXCEPTION_INT_OVERFLOW
  @EXCEPTION:
  POP	EBX
  PUSH	0
  PUSH	0
  PUSH	0
  PUSH	EAX
  CALL	RaiseException
end;

function StrToCardDef(const Str: String; Def: Cardinal): Cardinal;
begin
  try
    Result := StrToCard(Str);
  except
    Result := Def;
  end;
end;

function HexToDigit(Hex: Char): Byte; register; assembler;
asm	// EAX = Hex
  SUB	AL,'0'
  JB	@ERROR
  CMP	AL,0Ah
  JB	@EXIT
  AND	AL,0DFh
  SUB	AL,'A' - '0'
  JB	@ERROR
  CMP	AL,05h
  JA	@ERROR
  ADD	AL,0Ah
  @EXIT:
  RET
  @ERROR:
  XOR	EAX,EAX
end;

function DigitToHex(Digit: Byte): Char; register; assembler;
asm	// EAX = Digit
  AND	AL,0Fh
  ADD	AL,'0'
  CMP	AL,'9'
  JBE	@EXIT
  ADD	AL,'A' - '0' - 0Ah
  @EXIT:
end;

function HexToChar(Hex: PChar): Char; register; assembler;
asm	// EAX = Hex
  MOV	DX,[EAX]
  @FIRSTBYTE:
  SUB	DL,'0'
  JB	@ERROR
  CMP	DL,09h
  JA	@FIRSTCHAR
  JMP	@NEXTBYTE
  @FIRSTCHAR:
  AND	DL,0DFh
  SUB	DL,'A' - '0'
  JB	@ERROR
  CMP	DL,05h
  JA	@ERROR
  ADD	DL,0Ah
  @NEXTBYTE:
  SUB	DH,'0'
  JB	@ERROR
  CMP	DH,09h
  JA	@NEXTCHAR
  JMP	@DECODE
  @NEXTCHAR:
  AND	DH,0DFh
  SUB	DH,'A' - '0'
  JB	@ERROR
  CMP	DH,05h
  JA	@ERROR
  ADD	DH,0Ah
  @DECODE:
  XOR	EAX,EAX
  SHL	DL,4
  ADD	DL,DH
  MOV	AL,DL
  RET
  @ERROR:
  XOR	EAX,EAX
end;

function HexByteToDigit(HexByte: PChar; NullPad: Longbool): Byte; register; assembler;
asm	// EAX = @HexByte, EDX = NullPad (two hex chars, returns NUL on error)
  MOV	EAX,[EAX]
  OR	EDX,EDX
  JZ	@START
  MOV	AH,AL
  MOV	AL,'0'
  @START:
  SUB	AL,'0'
  JB	@ERROR
  CMP	AL,0Ah
  JB	@NEXT
  AND	AL,0DFh
  SUB	AL,'A' - '0'
  JB	@ERROR
  CMP	AL,05h
  JA	@ERROR
  ADD	AL,0Ah
  @NEXT:
  SUB	AH,'0'
  JB	@ERROR
  CMP	AH,0Ah
  JB	@EXIT
  AND	AH,0DFh
  SUB	AH,'A' - '0'
  JB	@ERROR
  CMP	AH,05h
  JA	@ERROR
  ADD	AH,0Ah
  @EXIT:
  SHL	AL,04h
  OR	AL,AH
  MOVZX	EAX,AL
  RET
  @ERROR:
  XOR	EAX,EAX
end;

function ByteToDigitChars(Value: Byte; PResult: PChar): Byte; register; assembler;
asm	// EAX = Value, EDX = PResult, ECX = Result (Count)
  PUSH	EBX
  XOR	ECX,ECX
  MOV	EBX,EDX
  @LOOP:
  PUSH	EBX
  MOV	EBX,10
  XOR	EDX,EDX
  DIV	EBX
  ADD	EDX,'0'
  INC	ECX
  POP	EBX
  CMP	EAX,10
  JG	@100
  CMP	EAX,00
  JG	@10
  MOV	[EBX],DL
  JMP	@EXIT
  @10:
  MOV	[EBX + 01h],DL
  JMP	@LOOP
  @100:
  MOV	[EBX + 02h],DL
  JMP	@LOOP
  @EXIT:
  MOV	EAX,ECX
  SUB	ECX,Type(TDigitArrayA)
  JZ	@END
  ADD	EBX,Type(TDigitArrayA)
  @ZERO:
  MOV	BYTE PTR [EBX + ECX],0
  INC	ECX
  JNZ	@ZERO
  @END:
  POP	EBX
end;

function StrToHexChars(const Str: String; Hex: PChar; Size: Cardinal): PChar; register; assembler;
asm	// EAX = @Str, EDX = Hex, ECX = Size
  PUSH	EDX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  SHR	ECX,1
  CMP	ECX,[EAX - 04h]
  JB	@EXCEPTION
  PUSH	EBX
  MOV	ECX,[EAX - 04h]
  ADD	EAX,ECX
  NEG	ECX
  @LOOP:
  MOV	BL,[EAX + ECX]
  MOV	BH,BL
  SHR	BH,4
  AND	BH,0Fh
  ADD	BH,'0'
  CMP	BH,'9'
  JBE	@HI_NIBBLE
  ADD	BH,'A' - '0' - 0Ah
  @HI_NIBBLE:
  MOV	[EDX],BH
  INC	EDX
  AND	BL,0Fh
  ADD	BL,'0'
  CMP	BL,'9'
  JBE	@LO_NIBBLE
  ADD	BL,'A' - '0' - 0Ah
  @LO_NIBBLE:
  MOV	[EDX],BL
  INC	EDX
  INC	ECX
  JNE	@LOOP
  POP	EBX
  @EXIT:
  POP	EAX
  RET
  @EXCEPTION:
  POP	EAX
  PUSH	0
  PUSH	0
  PUSH	0
  PUSH	EXCEPTION_ACCESS_VIOLATION
  CALL	RaiseException
end;

function StrToHex(const Str: String; Size: Integer): String;
const
  HEXCHARS: Array[0..15] of Char = String('0123456789ABCDEF');
var
  pResult, pStr: PChar;
  iIndex: Integer;
begin
  if Str <> E then begin
    SetString(Result, nil, Size);
    pResult := Pointer(Result);
    pStr := Pointer(Str);
    for iIndex := 0 to pred(Size shr 1) do begin
      pResult[0] := HEXCHARS[Byte(pStr[iIndex]) shr 04];
      pResult[1] := HEXCHARS[Byte(pStr[iIndex]) and $F];
      inc(pResult, 2);
    end;
  end;
end;

function StrLAppend(Dest, Source: PChar; Len: Integer; AddLineBreak: Longbool): PChar;
begin
  if Len > 0 then begin
    Move(Source^, Dest^, Len);
    inc(Dest, Len);
    if AddLineBreak then begin
      Dest^ := CR;
      inc(Dest);
      Dest^ := LF;
      inc(Dest);
    end;
    Result := Dest;
  end
  else Result := Dest;
end;

function StrLPas(Source: PChar; Length: Integer): String;
begin
  if (Source <> nil) and (Length > 0) then begin
    SetString(Result, Source, Length);
    Exit;
  end;
  Result := E;
end;

function ScanBack(const Str: PChar; Char: Char): PChar;
begin
  Result := Str;
  if Str <> nil then while Result^ <> Char do dec(Result);
end;

function Unwrap(const Str: String): String;
var
  iBeg, iPos, iEnd, iLen: Integer;
  pResult: PChar;
begin
  if Str <> E then begin
    iBeg := 1;
    iPos := iBeg;
    iEnd := Length(Str);
    SetString(Result, nil, iEnd);
    pResult := Pointer(Result);
    repeat
      while not (Str[iPos] in [LF, CR, NULL]) do inc(iPos);
      iLen := iPos - iBeg;
      if iLen > 0 then begin
	Move(Str[iBeg], pResult^, iLen);
	inc(pResult, iLen);
      end;
      while Str[iPos] in [LF, CR] do inc(iPos);
      iBeg := iPos;
    until iPos > iEnd;
    iLen := succ(iEnd - iBeg);
    if iLen > 0 then begin
      Move(Str[iBeg], pResult^, iLen);
      inc(pResult, iLen);
    end;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end
  else Result := E;
end;

function Unescape(const Str: String; SkipSpaces: Longbool; EscapeChar: Char): String;
var
  pSource: PAnsiChar;
  asUtf8: AnsiString;
  pUtf8: PAnsiChar;
begin
  if Str <> E then begin
    SetString(asUtf8, NIL, Length(Str));
    pUtf8 := Pointer(asUtf8);
    pSource := Pointer(Str);
    repeat
      while not (pSource^ in [NULL, EscapeChar]) do begin
	pUtf8^ := pSource^;
	inc(pSource);
	inc(pUtf8);
      end;
      if pSource^ = EscapeChar then begin
	pUtf8^ := HexToChar(pSource + 1);
	if (pUtf8^ > NULL) and (not SkipSpaces or (pUtf8^ <> SP)) then begin
	  inc(pSource, 3);
	  inc(pUtf8);
	end
	else begin
	  pUtf8^ := EscapeChar;
	  inc(pSource);
	end;
      end
      else Break;
    until pSource^ = NULL;
    Delete(asUtf8, succ(pUtf8 - Pointer(asUtf8)), MAXINT);
    if Length(asUtf8) < Length(Str) then
      Result := asUtf8
    else Result := Str;
  end
  else Result := Str;
end;

procedure FillByte(var X; Count: Cardinal; Value: Byte); register; assembler;
asm	// EAX = @X, EDX = Count, CL = Value
  OR	EAX, EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EBX
  MOV	CH,CL
  MOV	BX,CX
  SHL	EBX,16
  ADD	BX,CX
  MOV	ECX,EDX
  AND	EDX,3
  SUB	ECX,EDX
  CMP	ECX,64
  JB	@LOOP
  CLD
  PUSH	EDI
  MOV	EDI,EAX
  MOV	EAX,EBX
  SHR	ECX,2
  REP	STOSD
  MOV	ECX,EDX
  REP	STOSB
  POP	EDI
  POP	EBX
  RET
  @LOOP:
  ADD	EAX,ECX
  SHR	ECX,2
  JE	@REST
  NEG	ECX
  @DWLOOP:
  MOV	[EAX + ECX * 4],EBX
  INC	ECX
  JNE	@DWLOOP
  @REST:
  ADD	EAX,EDX
  NEG	EDX
  JE	@END
  @BLOOP:
  MOV	[EAX + EDX],BL
  INC	EDX
  JNE	@BLOOP
  @END:
  POP	EBX
  @EXIT:
end;

function FastStrLen(Str: PChar): Cardinal; register; assembler;
asm	// EAX = @Str
  OR	EAX,EAX
  JE	@EXIT
  XOR	EDX,EDX
  JMP	@CMP
  @INC:
  INC	EDX
  @CMP:
  CMP	BYTE PTR [EAX + EDX],0
  JNE	@INC
  @END:
  MOV	EAX,EDX
  @EXIT:
end;

function WideStrLen(Str: PWideChar): Cardinal; register; assembler;
asm	// EAX = @Str
  OR	EAX,EAX
  JE	@EXIT
  XOR	EDX,EDX
  JMP	@CMP
  @INC:
  ADD	EDX,2
  @CMP:
  CMP	WORD PTR [EAX + EDX],0
  JNE	@INC
  @END:
  MOV	EAX,EDX
  @EXIT:
end;

function AnsiStrLCheck(Str: PChar; MaxLen: Cardinal): Boolean; register; assembler;
asm	// EAX = @Str, EDX = MaxLen
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  ADD	EAX,EDX
  NEG	EDX
  JMP	@CMP
  @INC:
  INC	EDX
  JE	@END
  @CMP:
  CMP	BYTE PTR [EAX + EDX],0
  JNE	@INC
  @EXIT:
  XOR	EAX,EAX
  RET
  @END:
  MOV	EAX,true
end;

function WideStrLCheck(Str: PWideChar; MaxLen: Cardinal): Boolean; register; assembler;
asm	// EAX = @Str, EDX = MaxLen
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  ADD	EAX,EDX
  NEG	EDX
  JMP	@CMP
  @INC:
  ADD	EDX,2
  JE	@END
  @CMP:
  CMP	BYTE PTR [EAX + EDX],0
  JNE	@INC
  @EXIT:
  XOR	EAX,EAX
  RET
  @END:
  MOV	EAX,true
end;

function WideCharToUtf8(Wide: WideChar): String; register; assembler;
asm	// EAX = Wide, EDX = @Result
  PUSH	EAX
  PUSH	EDX
  MOV	EAX,MAX_UTF8_BYTES
  CALL	System.@NewAnsiString
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDX,EAX
  POP	EAX
  ADD	EDX,MAX_UTF8_BYTES
  AND	EAX,0000FFFFh
  XOR	ECX,ECX
  DEC	EDX
  CMP	EAX,04000000h
  JNB	@6BYTES
  DEC	EDX
  CMP	EAX,00200000h
  JNB	@5BYTES
  DEC	EDX
  CMP	EAX,00010000h
  JNB	@4BYTES
  DEC	EDX
  CMP	EAX,00000800h
  JNB	@3BYTES
  DEC	EDX
  CMP	EAX,00000080h
  JNB	@2BYTES
  DEC	EDX
  JMP	@ASCII
  @6BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],AL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EAX,6
  DEC	EDX
  @5BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],AL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EAX,6
  DEC	EDX
  @4BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],AL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EAX,6
  DEC	EDX
  @3BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],AL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EAX,6
  DEC	EDX
  @2BYTES:
  INC	ECX
  MOV	BYTE PTR [EDX],AL
  AND	BYTE PTR [EDX],03Fh
  OR	BYTE PTR [EDX],080h
  SHR	EAX,6
  DEC	EDX
  INC	ECX
  MOV	[EDX],AL
  MOV	AX,00FFh
  ROR	AX,CL
  OR	[EDX],AH
  MOV	BYTE PTR [EDX + ECX],0
  MOV	DWORD PTR [EDX - SLO],ECX
  RET
  @ASCII:
  MOV	DWORD PTR [EDX - SLO],1
  MOV	[EDX],AX
end;

function Utf8ToWideChar(Utf8: PChar; var CharCount: Integer): WideChar; register; assembler;
asm	// EAX = @Utf8, EDX = @CharCount
  PUSH	EBX
  OR	EAX,EAX
  JE	@ZERO
  XOR	EBX,EBX
  MOV	BL,[EAX]
  CMP	BL,0FDh
  JA	@ZERO
  CMP	BL,080h
  JB	@ASCII
  XOR	ECX,ECX
  @COUNT:
  INC	CL
  SHL	BL,1
  JC	@COUNT
  CMP	CL,3
  JB	@ZERO
  SHR	BL,CL
  DEC	ECX
  MOV	[EDX],ECX
  ADD	EAX,ECX
  PUSH	EDX
  NEG	ECX
  INC	ECX
  @LOOP:
  MOV	DL,[EAX + ECX]
  CMP	DL,080h
  JB	@EXIT
  AND	DL,07Fh
  CMP	DL,040h
  JNB	@EXIT
  SHL	EBX,6
  OR	BL,DL
  INC	ECX
  JNE	@LOOP
  MOV	EAX,EBX
  POP	EDX
  POP	EBX
  RET
  @ASCII:
  MOV	DWORD PTR [EDX],1
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  POP	EDX
  @ZERO:
  POP	EBX
  XOR	EAX,EAX
  MOV	[EDX],EAX
end;

function PWideCharToWideStr(Wide: PWideChar): String;
var
  iPCharLen: Integer;
begin
  iPCharLen := WideStrLen(Wide);
  SetString(Result, nil, iPCharLen shl 1);
  Move(Wide^, Pointer(Result)^, iPCharLen);
end;

function StrToWideStr(const Str: String): String;
var
  iPCharLen: Integer;
begin
  iPCharLen := succ(Length(Str));
  SetString(Result, nil, iPCharLen shl 1);
  StringToWideChar(Str, Pointer(Result), iPCharLen);
end;

function Trim(const Str: String): String;
var
  iLen, iEnd, iBeg: Integer;
begin
  iLen := Length(Str);
  iEnd := iLen;
  while (iEnd >= 1) and (Str[iEnd] <= SP) do dec(iEnd);
  iBeg := 1;
  while (iBeg <= iEnd) and (Str[iBeg] <= SP) do inc(iBeg);
  Result := Copy(Str, iBeg, succ(iEnd - iBeg));
end;

function TrimLeft(const Str: String): String;
var
  iPos: Integer;
begin
  if Str <> E then begin
    iPos := 1;
    while (Str[iPos] > NULL) and (Str[iPos] <= SP) do inc(iPos);
    Result := Copy(Str, iPos, MAXINT);
  end
  else Result := E;
end;

function TrimRight(const Str: String): String;
var
  iPos: Integer;
begin
  if Str <> E then begin
    iPos := Length(Str);
    while (iPos >= 1) and (Str[iPos] <= SP) do dec(iPos);
    Result := Copy(Str, 1, iPos);
  end
  else Result := E;
end;

function TrimCustom(const Str: String; const Chars: TTrimSet): String;
var
  iLen, iEnd, iBeg: Integer;
begin
  iLen := Length(Str);
  iEnd := iLen;
  while (iEnd >= 1) and (Str[iEnd] in Chars) do dec(iEnd);
  iBeg := 1;
  while (iBeg <= iEnd) and (Str[iBeg] in Chars) do inc(iBeg);
  Result := Copy(Str, iBeg, succ(iEnd - iBeg));
end;

function InsertStrings(const Format: String; const Strings: Array of String): String;
var
  iResult, iIndex, iBeg, iEnd, iLen: Integer;
  pResult: PChar;
begin
  if Format <> E then begin
    iResult := Length(Format);
    for iIndex := 0 to High(Strings) do inc(iResult, Length(Strings[iIndex]));
    SetString(Result, nil, iResult);
    pResult := Pointer(Result);
    iIndex := 0;
    iBeg := 1;
    iEnd := 1;
    repeat
      while not (Format[iEnd] in [NULL, PC]) do inc(iEnd);
      if Format[iEnd] = PC then begin
	inc(iEnd);
	if Format[iEnd] <> NULL then begin
	  if Format[iEnd] in [PC, 'S', 's'] then begin
	    iLen := pred(iEnd - iBeg);
	    if iLen > 0 then begin
	      Move(Format[iBeg], pResult^, iLen);
	      inc(pResult, iLen);
	    end;
	    if Format[iEnd] <> PC then begin
	      if iIndex <= High(Strings) then begin
		iLen := Length(Strings[iIndex]);
		if iLen > 0 then begin
		  Move(Pointer(Strings[iIndex])^, pResult^, iLen);
		  inc(pResult, iLen);
		end;
		inc(iIndex);
	      end;
	    end
	    else begin
	      pResult^ := PC;
	      inc(pResult);
	    end;
	    inc(iEnd);
	    iBeg := iEnd;
	  end
	  else inc(iEnd);
	  if Format[iEnd] <> NULL then Continue;
	end;
      end;
      iLen := iEnd - iBeg;
      if iLen > 0 then begin
	Move(Format[iBeg], pResult^, iLen);
	inc(pResult, iLen);
      end;
      Break;
    until Format[iEnd] = NULL;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end
  else Result := E;
end;

function DateString(const ST: TSystemTime; const Format: String): String;
var
  iLen: Integer;
begin
  iLen := GetDateFormat(LOCALE_USER_DEFAULT, 0, @ST, Pointer(Format), nil, 0);
  SetString(Result, nil, iLen);
  Delete(Result, GetDateFormat(LOCALE_USER_DEFAULT, 0, @ST, Pointer(Format), Pointer(Result), iLen), MAXINT);
end;

function TimeString(const ST: TSystemTime; const Format: String): String;
var
  iLen: Integer;
begin
  iLen := GetTimeFormat(LOCALE_USER_DEFAULT, 0, @ST, Pointer(Format), nil, 0);
  SetString(Result, nil, iLen);
  Delete(Result, GetTimeFormat(LOCALE_USER_DEFAULT, 0, @ST, Pointer(Format), Pointer(Result), iLen), MAXINT);
end;

function HashedString(const Str: String; Len: Integer): String;
var
  sHash: String;
begin
  if Str <> E then begin
    Len := Len div 2;
    SetLength(sHash, Len);
    if HashData(Pointer(Str), Length(Str) * SizeOf(Char), Pointer(sHash), Len) = S_OK then begin
      Result := StrToHex(sHash, Len * 2);
      Exit;
    end;
  end;
  Result := E;
end;

function RtfToAnsiString(const RtfText: String): String;
const
  RTF_MARKER = '{\rtf';
var
  hRichEdit: THandle;
  iLength: Integer;
begin
  if LoStrAtPos(RTF_MARKER, RtfText, 1) then begin
    hRichEdit := CreateWindow('RICHEDIT', nil, ES_MULTILINE, 0, 0, 0, 0, 0, 0, hInstance, nil);
    if hRichEdit <> 0 then begin
      try
	SendMessage(hRichEdit, EM_EXLIMITTEXT, 0, MAXINT);
	SetWindowText(hRichEdit, Pointer(RtfText));
	iLength := GetWindowTextLength(hRichEdit);
	SetString(Result, nil, iLength);
	Delete(Result, succ(GetWindowText(hRichEdit, Pointer(Result), succ(iLength))), MAXINT);
	Exit;
      finally
	DestroyWindow(hRichEdit);
      end;
    end;
  end;
  Result := RtfText;
end;

function GetPCharHandleA(Text: PAnsiChar; Size: Integer; ReallocHandle: THandle; Local: Longbool): THandle;
var
  pResult: PAnsiChar;
  cMemSize: Cardinal;
begin
  Result := 0;
  if Text <> NIL then begin
    if Size = -1 then begin
      Size := succ(lstrlena(Text));
      cMemSize := Size * (SizeOf(Text[0]));
    end
    else cMemSize := Size;
    if Local then begin
      if ReallocHandle <> 0 then
	Result := LocalReAlloc(ReallocHandle, GHND, cMemSize)
      else Result := LocalAlloc(GHND, cMemSize);
    end
    else begin
      if ReallocHandle <> 0 then
	Result := GlobalReAlloc(ReallocHandle, GHND, cMemSize)
      else Result := GlobalAlloc(GHND, cMemSize);
    end;
    if Result <> 0 then begin
      if Local then
	pResult := LocalLock(Result)
      else pResult := GlobalLock(Result);
      try
	Move(Pointer(Text)^, pResult^, cMemSize);
      finally
	if Local then
	  LocalUnlock(Result)
	else GlobalUnlock(Result);
      end;
    end;
  end;
end;

function GetPCharHandleW(Text: PWideChar; Size: Integer; ReallocHandle: THandle; Local: Longbool): THandle;
var
  pResult: PWideChar;
  cMemSize: Cardinal;
begin
  Result := 0;
  if Text <> NIL then begin
    if Size = -1 then begin
      Size := succ(lstrlenw(Text));
      cMemSize := Size * (SizeOf(Text[0]));
    end
    else cMemSize := Size;
    if Local then begin
      if ReallocHandle <> 0 then
	Result := LocalReAlloc(ReallocHandle, GHND, cMemSize)
      else Result := LocalAlloc(GHND, cMemSize);
    end
    else begin
      if ReallocHandle <> 0 then
	Result := GlobalReAlloc(ReallocHandle, GHND, cMemSize)
      else Result := GlobalAlloc(GHND, cMemSize);
    end;
    if Result <> 0 then begin
      if Local then
	pResult := LocalLock(Result)
      else pResult := GlobalLock(Result);
      try
	Move(Pointer(Text)^, pResult^, cMemSize);
      finally
	if Local then
	  LocalUnlock(Result)
	else GlobalUnlock(Result);
      end;
    end;
  end;
end;

function GetStringHandleA(const Str: String; ReallocHandle: THandle; Local: Longbool): THandle;
begin
  Result := GetPCharHandleA(Pointer(Str), succ(Length(Str)), ReallocHandle, Local);
end;

function GetStringHandleW(const Str: String; ReallocHandle: THandle; Local: Longbool): THandle;
begin
  Result := GetPCharHandleW(Pointer(Str), Length(Str), ReallocHandle, Local);
end;

function GetHandleStringA(Handle: THandle; Size: Integer; Free: Longbool): String;
var
  pc: PAnsiChar;
begin
  Result := E;
  if Handle <> 0 then begin
    try
      if Size = 0 then begin
	Size := GlobalSize(Handle);
	if Size > 0 then begin
	  pc := Pointer(GlobalLock(Handle));
	  dec(Size, Ord((pc + Size - 1)^ = NULL));
	  SetString(Result, pc, Size div SizeOf(AnsiChar));
	  Exit;
	end;
      end
      else begin
	pc := Pointer(GlobalLock(Handle));
	if pc <> nil then begin
	  if Size = -1 then
	    SetString(Result, PAnsiChar(pc), lstrlena(pc) * SizeOf(AnsiChar))
	  else if Size > 0 then begin
	    while pc[pred(Size)] = NULL do dec(Size);
	    SetString(Result, pc, Size);
	  end;
	end;
	Exit;
      end;
    finally
      GlobalUnlock(Handle);
      if Free then GlobalFree(Handle);
    end;
  end;
  Result := E;
end;

function GetHandleStringW(Handle: THandle; Size: Integer; Free: Longbool): String;
var
  pc: PWideChar;
  fHasBOM: Boolean;
begin
  Result := E;
  if Handle <> 0 then begin
    try
      if Size = 0 then begin
	Size := GlobalSize(Handle);
	if Size > 0 then SetString(Result, PAnsiChar(GlobalLock(Handle)), Size);
	Exit;
      end
      else begin
	pc := Pointer(GlobalLock(Handle));
	if pc <> nil then begin
	  fHasBOM := (pc^ = UnicodeBomChar);
	  inc(pc, Ord(fHasBOM));
	  if Size = -1 then
	    SetString(Result, PAnsiChar(pc), succ(lstrlenw(pc)) * SizeOf(WideChar))
	  else if Size > 0 then begin
	    while pc[pred(Size)] = NULL do dec(Size);
	    SetString(Result, PAnsiChar(pc), succ(Size - Ord(fHasBOM)) * SizeOf(WideChar));
	  end;
	end;
	Exit;
      end;
    finally
      GlobalUnlock(Handle);
      if Free then GlobalFree(Handle);
    end;
  end;
  Result := E;
end;

function CompareStr(const Str1, Str2: String): Boolean;
begin
  Result := (CompareString(LOCALE_USER_DEFAULT, 0, Pointer(Str1), -1, Pointer(Str2), -1) = 2);
end;

function CompareTxt(const Str1, Str2: String): Boolean;
begin
  Result := (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, Pointer(Str1), -1, Pointer(Str2), -1) = 2);
end;

function CompareStrMax(const Str1, Str2: String; MaxLen: Integer): Boolean;
begin
  Result := (CompareString(LOCALE_USER_DEFAULT, 0, Pointer(Str1), MaxLen, Pointer(Str2), MaxLen) = 2);
end;

function CompareTxtMax(const Str1, Str2: String; MaxLen: Integer): Boolean;
begin
  Result := (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, Pointer(Str1), MaxLen, Pointer(Str2), MaxLen) = 2);
end;


function LowerFirstChar(const Str: String): String;
begin
  if Str <> E then begin
    Result := Str;
    UniqueString(Result);
    CharLowerBuff(Pointer(Result), 1);
  end
  else Result := E;
end;

function UpperFirstChar(const Str: String): String;
begin
  if Str <> E then begin
    Result := Str;
    UniqueString(Result);
    CharUpperBuff(Pointer(Result), 1);
  end
  else Result := E;
end;

procedure LowerChar(var Str: String; Pos: Integer);
begin
  if (Pos > 0) and (Pos <= Length(Str)) then CharLowerBuff(@Str[Pos], 1);
end;

procedure UpperChar(var Str: String; Pos: Integer);
begin
  if (Pos > 0) and (Pos <= Length(Str)) then CharUpperBuff(@Str[Pos], 1);
end;

procedure LowerChars(var Str: String; Count: Integer);
begin
  if (Count > 0) and (Count <= Length(Str)) then CharLowerBuff(Pointer(Str), Count);
end;

procedure UpperChars(var Str: String; Count: Integer);
begin
  if (Count > 0) and (Count <= Length(Str)) then CharUpperBuff(Pointer(Str), Count);
end;

function LowerStr(const Str: String): String;
begin
  Result := Str;
  if Result <> E then CharLower(Pointer(Result));
end;

function UpperStr(const Str: String): String;
begin
  Result := Str;
  if Result <> E then CharUpper(Pointer(Result));
end;

function DelAmp(const Text: String): String;
var
  iPos, iLen: Integer;
  pResult: PChar;
begin
  iPos := FirstCharPos(AM, Text);
  if iPos > 0 then begin
    iLen := pred(Length(Text));
    SetString(Result, nil, iLen);
    dec(iPos);
    pResult := Pointer(Result);
    Move(Text[1], pResult^, iPos);
    inc(pResult, iPos);
    Move(Text[iPos + 2], pResult^, iLen - iPos);
  end
  else Result := Text;
end;

function Unquote(const Str: String; CBeg, CEnd: Char): String;
var
  pBeg, pEnd: PChar;
begin
  if Str <> E then begin
    pBeg := Pointer(Str);
    pEnd := pBeg;
    inc(pEnd, Length(Str));
    while (pEnd > pBeg) and (pEnd^ <= SP) do dec(pEnd);
    while (pBeg < pEnd) and (pBeg^ <= SP) do inc(pBeg);
    while pEnd > pBeg do begin
      if (pBeg^ = CBeg) and (pEnd^ = CEnd) then begin
	inc(pBeg);
	dec(pEnd);
      end
      else Break;
    end;
    SetString(Result, pBeg, succ(pEnd - pBeg));
    Exit;
  end;
  Result := E;
end;

function DelAngles(const Text: String): String;
begin
  Result := Unquote(Text, LT, GT);
end;

function DelQuotes(const Text: String): String;
begin
  Result := Unquote(Text, QU, QU);
end;

function CharCopy(const Text: String; Limit: Char): String;
begin
  Result := Copy(Text, 1, pred(FirstCharPos(Limit, Text)));
  if Result = E then Result := Text;
end;

function CharExtract(var Text: String; Limit: Char): String;
var
  iCharPos: Integer;
begin
  iCharPos := FirstCharPos(Limit, Text);
  if iCharPos > 0 then begin
    Result := Copy(Text, 1, pred(iCharPos));
    Delete(Text, 1, iCharPos);
  end
  else begin
    Result := Text;
    Text := E;
  end;
end;

function ColonExtract(var Text: String): String;
begin
  Result := CharExtract(Text, CL);
end;

function CommaExtract(var Text: String): String;
begin
  Result := CharExtract(Text, CM);
end;

function SpaceExtract(var Text: String): String;
begin
  Result := Trim(CharExtract(Text, SP));
end;

function TabExtract(var Text: String): String;
begin
  Result := Trim(CharExtract(Text, TAB));
end;

function LineExtract(var Text: String): String;
begin
  Result := TrimRight(CharExtract(Text, LF));
end;

function TrimmedLine(var Text: String): String;
var
  iPos: Integer;
begin
  iPos := FirstCharPos(CR, Text);
  if iPos > 0 then begin
    Result := Copy(Text, 1, pred(iPos));
    Delete(Text, 1, succ(iPos));
  end
  else begin
    Result := Text;
    Text := E;
  end;
end;

function GetFirstLine(var Text: String): String;
begin
  Result := CopyLine(Text, 1);
end;

function CopyLine(const Str: String; Num: Integer): String;
var
  pStr, pPos: PChar;
  iPos: Integer;
begin
  if (Str <> E) and (Num > 0) then begin
    pStr := Pointer(Str);
    pPos := pStr;
    iPos := 1;
    while Num > 0 do begin
      while not (pStr^ in [NULL, CR]) do inc(pStr);
      dec(Num);
      if Num = 0 then begin
	Result := Copy(Str, iPos, pStr - pPos);
	Exit;
      end
      else if pStr^ = NULL then begin
	Result := Copy(Str, iPos, pStr - pPos);
	Exit;
      end
      else begin
	while pStr^ in [LF, CR] do inc(pStr);
	if (pStr^ = NULL) and (Num > 0) then begin
	  while (pStr > pPos) and (pStr^ < SP) do dec(pStr);
	  Result := Copy(Str, iPos, succ(pStr - pPos));
	  Exit;
	end
	else begin
	  iPos := succ(pStr - Pointer(Str));
	  pPos := pStr;
	end;
      end;
    end;
  end;
end;

function AddSpaceLine(const Text: String): String;
var
  iLen: Integer;
  sSpaceLine: String;
begin
  iLen := Length(Text);
  if iLen > 0 then begin
    if StrAtPos(PARA, Text, iLen - pred(Length(PARA))) then
      sSpaceLine := E
    else if StrAtPos(CRLF, Text, iLen - pred(Length(CRLF))) then
      sSpaceLine := CRLF
    else sSpaceLine := PARA;
    Result := Text + sSpaceLine;
  end
  else Result := Text;
end;

function AdjustLineBreaks(const Str: String): String;
var
  pSource, pLast, pDest: PChar;
  iDestLen: Integer;
begin
  pSource := Pointer(Str);
  iDestLen := Length(Str);
  pLast := pSource + iDestLen;
  while pSource < pLast do begin
    case pSource^ of
      LF: inc(iDestLen);
      CR: if pSource[1] = LF then
	inc(pSource)
      else inc(iDestLen);
    end;
    inc(pSource);
  end;
  if iDestLen = Length(pSource) then
    Result := Str
  else begin
    pSource := Pointer(Str);
    SetString(Result, NIL, iDestLen);
    pDest := Pointer(Result);
    while pSource < pLast do begin
      case pSource^ of
	LF: begin
	  pDest^ := CR;
	  inc(pDest);
	  pDest^ := LF;
	  inc(pDest);
	  inc(pSource);
	end;
	CR: begin
	  pDest^ := CR;
	  inc(pDest);
	  pDest^ := LF;
	  inc(pDest);
	  inc(pSource);
	  if pSource^ = LF then inc(pSource);
	end;
      else
	pDest^ := pSource^;
	inc(pSource);
	inc(pDest);
      end;
    end;
  end;
end;

function ExtractBaseName(const Name: String): String;
var
  iPos: Integer;
begin
  Result := ExtractFileName(Name);
  iPos := LastCharPos(DOT, Result);
  if iPos <> 0 then Delete(Result, iPos, Length(Result))
end;

function ExtractFileExt(const Name: String): String;
var
  iPos: Integer;
begin
  iPos := LastCharPos(DOT, Name);
  if iPos <> 0 then
    Result := Copy(Name, iPos, MAXINT)
  else Result := E;
end;

function ExtractFileName(const Name: String): String;
var
  iPos, iLen: Integer;
begin
  iPos := LastCharPos(BS, Name);
  if iPos <> 0 then begin
    iLen := Length(Name);
    if iLen > iPos then
      Result := Copy(Name, succ(iPos), iLen - iPos)
    else Result := E;
  end
  else Result := Name;
end;

function ExtractFilePath(const Name: String): String;
var
  iPos: Integer;
begin
  iPos := LastCharPos(BS, Name);
  if iPos <> 0 then
    Result := Copy(Name, 1, iPos)
  else Result := E;
end;

function ExtractFileDrive(const Name: String): String;
begin
  if (Length(Name) >= 2) and (Name[2] = CL) then
    Result := Copy(Name, 1, 2)
  else Result := E;
end;

function ExtractShortPath(const Name: String): String;
var
  iFPos, iLPos: Integer;
  sPathBuf, sNameBuf: String;
begin
  iFPos := FirstCharPos(BS, Name);
  iLPos := LastCharPos(BS, Name);
  if (iFPos <> 0) and (iFPos <> iLPos) then begin
    sPathBuf := ExtractFilePath(Name);
    sNameBuf := ExtractFileName(Name);
    Result := ExtractStr(sPathBuf, 1, iFPos);
    if (Length(Result) + Length(sPathBuf) + Length(sNameBuf)) > 63 then begin
      Result := Result + Copy(sPathBuf, 1, 63 - Length(sNameBuf) - Length(Result)) + '...' + BS + sNameBuf;
    end
    else Result := Result + sPathBuf + sNameBuf;
  end
  else Result := Name;
end;

function ChangeFileExt(const Name, Ext: String): String;
var
  iName, iExt, iPos: Integer;
  pResult: PChar;
begin
  if Name <> E then begin
    iName := Length(Name);
    iExt := Length(Ext);
    SetString(Result, nil, succ(iName + iExt));
    pResult := Pointer(Result);
    iPos := LastCharPos(DOT, Name);
    if iPos = 0 then iPos := iName;
    Move(Pointer(Name)^, pResult^, iPos);
    inc(pResult, iPos);
    if Result[iPos] <> DOT then begin
      pResult^ := DOT;
      inc(pResult);
    end;
    if iExt > 0 then begin
      iPos := 1;
      if Ext[1] = DOT then begin
	inc(iPos);
	dec(iExt);
      end;
      Move(Ext[iPos], pResult^, iExt);
      inc(pResult, iExt);
    end;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end
  else Result := E;
end;

function GetLineCount(const Text: String): Integer;
var
  pText: PChar;
begin
  Result := 0;
  if Text = E then Exit;
  pText := Pointer(Text);
  repeat
    if pText^ = CR then inc(Result);
    inc(pText);
  until pText^ = NULL;
  dec(pText);
  if pText^ <> LF then inc(Result);
end;

function GetCharCount(Find: Char; const Text: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Text
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EBX
  MOV	ECX,[EDX - SLO]
  XOR	EBX,EBX
  @LOOP:
  DEC	ECX
  JL	@END
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  INC	EBX
  JMP	@LOOP
  @END:
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function GetCharCountFrom(Find: Char; const Text: String; Pos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Text, ECX = Pos
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JE	@EXIT
  SUB	ECX,[EDX - SLO]
  JG	@EXIT
  PUSH	EBX
  XOR	EBX,EBX
  ADD	EDX,[EDX - SLO]
  DEC	ECX
  DEC	EDX
  @LOOP:
  INC	ECX
  JG	@END
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  INC	EBX
  JMP	@LOOP
  @END:
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function GetCharCountUpTo(Find: Char; const Text: String; Pos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Text, ECX = Pos
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JE	@EXIT
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  PUSH	EBX
  XOR	EBX,EBX
  @LOOP:
  DEC	ECX
  JL	@END
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  INC	EBX
  JMP	@LOOP
  @END:
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function NextNonSpaceChar(const Text: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = @Text, EDX = BegPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JLE	@NULL
  SUB	EDX,[EAX - SLO]
  JG	@NULL
  MOV	ECX,[EAX - SLO]
  ADD	EAX,ECX
  DEC	EAX
  @LOOP:
  CMP	BYTE PTR[EAX + EDX],' '
  JG	@BREAK
  INC	EDX
  JLE	@LOOP
  JG	@NULL
  @BREAK:
  ADD	ECX,EDX
  MOV	EAX,ECX
  RET
  @NULL:
  XOR	EAX,EAX
  @EXIT:
end;

function PrevNonSpaceChar(const Text: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = @Text, EDX = EndPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JLE	@NULL
  CMP	EDX,[EAX - SLO]
  JG	@NULL
  MOV	ECX,EAX
  MOV	EAX,EDX
  DEC	EAX
  @LOOP:
  CMP	BYTE PTR[ECX + EAX],' '
  JG	@BREAK
  DEC	EAX
  JGE	@LOOP
  JL	@NULL
  @BREAK:
  INC	EAX
  RET
  @NULL:
  XOR	EAX,EAX
  @EXIT:
end;

function FirstCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EDX
  MOV	ECX,[EDX - SLO]
  ADD	EDX,[EDX - SLO]
  NEG	ECX
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function LastCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  MOV	ECX,[EDX - SLO]
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function NextCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = BegPos
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  SUB	ECX,[EDX - SLO]
  JG	@EXIT
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - SLO]
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function PrevCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = EndPos
  OR	EDX,EDX
  JE	@EXIT
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function FirstRangeCharPos(Find: Char; const Dest: String; LoPos, HiPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = LoPos
  PUSH	EBX
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  SUB	ECX,HiPos
  JG	@EXIT
  DEC	ECX
  ADD	EDX,HiPos
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @MATCH:
  POP	EBX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function LastRangeCharPos(Find: Char; const Dest: String; HiPos, LoPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = HiPos
  OR	EDX,EDX
  JE	@EXIT
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  OR	LoPos,0
  JG	@CONTINUE
  MOV	LoPos,1
  @CONTINUE:
  SUB	ECX,LoPos
  JL	@EXIT
  ADD	EDX,LoPos
  DEC	EDX
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  DEC	ECX
  JNL	@LOOP
  @EXIT:
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @MATCH:
  MOV	EAX,ECX
  ADD	EAX,LoPos
end;

function FirstLoCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EDX
  MOV	ECX,[EDX - SLO]
  ADD	EDX,[EDX - SLO]
  NEG	ECX
  @LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function LastLoCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  MOV	ECX,[EDX - SLO]
  @LOOP:
  DEC	ECX
  JL	@EXIT
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@LOOP
  AND	AH,0DFh
  JNE	@LOOP
  @MATCH:
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function NextLoCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = BegPos
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  SUB	ECX,[EDX - SLO]
  JG	@EXIT
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - SLO]
  @LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function PrevLoCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = EndPos
  OR	EDX,EDX
  JE	@EXIT
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  @LOOP:
  DEC	ECX
  JL	@EXIT
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@LOOP
  AND	AH,0DFh
  JNE	@LOOP
  @MATCH:
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function FirstRangeLoCharPos(Find: Char; const Dest: String; LoPos, HiPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = LoPos
  PUSH	EBX
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  SUB	ECX,HiPos
  JG	@EXIT
  DEC	ECX
  ADD	EDX,HiPos
  @LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @MATCH:
  POP	EBX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function LastRangeLoCharPos(Find: Char; const Dest: String; HiPos, LoPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = HiPos
  OR	EDX,EDX
  JE	@EXIT
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  OR	LoPos,0
  JG	@CONTINUE
  MOV	LoPos,1
  @CONTINUE:
  SUB	ECX,LoPos
  JL	@EXIT
  ADD	EDX,LoPos
  DEC	EDX
  @LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  DEC	ECX
  JNL	@LOOP
  @EXIT:
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @MATCH:
  MOV	EAX,ECX
  ADD	EAX,LoPos
end;

function GetStrCount(const Find, Text: String): Integer; register; assembler;
var
  Count: Integer;
asm	// EAX = @Find, EDX = @Text
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EBX
  MOV	ECX,1
  MOV	Count,0
  MOV	EBX,[EAX - SLO]
  SUB	ECX,[EDX - SLO]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@END
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,[EDX - SLO]
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  INC	ECX
  JL	@LOOP
  @FINISH:
  POP	ESI
  POP	EDI
  POP	EBX
  MOV	EAX,Count
  JMP	@@EXIT
  @MATCH:
  PUSH	EBP
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[EBP + EBX],AH
  JE	@CMP
  POP	EDI
  POP	EBP
  INC	ECX
  JL	@LOOP
  JMP	@FINISH
  @YES:
  POP	EDI
  POP	EBP
  INC	Count
  ADD	ECX,[ESI - SLO]
  JL	@LOOP
  JMP	@FINISH
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
  JMP	@@EXIT
  @CHAR:
  PUSH	EBX
  MOV	AL,[EAX]
  MOV	ECX,[EDX - SLO]
  XOR	EBX,EBX
  @@LOOP:
  DEC	ECX
  JL	@@END
  CMP	[EDX + ECX],AL
  JNE	@@LOOP
  INC	EBX
  JMP	@@LOOP
  @@END:
  MOV	EAX,EBX
  POP	EBX
  @@EXIT:
end;

function FirstStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  MOV	ECX,[EDX - SLO]
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,ECX
  MOV	AL,[ESI]
  NEG	ECX
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  ADD	ECX,[ESI - SLO]
  JE	@NOT
  SUB	ECX,[ESI - SLO]
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[EBP + EBX],AH
  JE	@CMP
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  PUSH	EDX
  MOV	ECX,[EDX - SLO]
  ADD	EDX,[EDX - SLO]
  NEG	ECX
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function LastStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  MOV	ECX,[EDX - SLO]
  CMP	ECX,[EAX - SLO]
  JL	@END
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  CMP	[EDI + ECX - 01h],AL
  JE	@MATCH
  DEC	ECX
  CMP	ECX,[ESI - SLO]
  JNL	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  CMP	[EBP + EBX - 01h],AH
  JE	@CMP
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	ECX,[EDX - SLO]
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  CMP	[EDX + ECX],AL
  JNE	@@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function NextStrPos(const Find, Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = BegPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EAX - SLO]
  SUB	ECX,[EDX - SLO]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,[EDX - SLO]
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[EBP + EBX],AH
  JE	@CMP
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - SLO]
  POP	EDI
  MOV	EAX,ECX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  POP	EBX
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - SLO]
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function PrevStrPos(const Find, Dest: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = EndPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  ADD	ECX,[EAX - SLO]
  CMP	ECX,[EDX - SLO]
  JNG	@CONTINUE
  MOV	ECX,[EDX - SLO]
  INC	ECX
  @CONTINUE:
  DEC	ECX
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  CMP	[EDX - SLO],EBX
  JL	@END
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  CMP	[EDI + ECX - 01h],AL
  JE	@MATCH
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  CMP	[EBP + EBX - 01h],AH
  JE	@CMP
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  @END:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  CMP	[EDX + ECX],AL
  JNE	@@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function FirstStrPartInRange(const Find, Dest: String; PartLen, LoPos, HiPos: Integer): Integer;
asm	// EAX = @Find, EDX = @Dest, ECX = PartLen
  PUSH	ECX
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  OR	LoPos,0
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  CMP	ECX,[EAX - SLO]
  JG	@EXIT
  MOV	EBX,ECX
  MOV	ECX,LoPos
  SUB	ECX,HiPos
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@EXIT
  CMP	DWORD PTR [EBP - 4],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,HiPos
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @MATCH:
  MOV	EBX,[EBP - 4]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[EBP - 4]
  MOV	ECX,ESI
  ADD	EDI,EBX
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[ECX + EBX],AH
  JE	@CMP
  POP	ECX
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @YES:
  POP	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  ADD	ECX,HiPos
  MOV	EAX,ECX
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @EXIT:
  POP	EBX
  POP	EDX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos + TYPE HiPos
  @CHAR:
  DEC	ECX
  ADD	EDX,HiPos
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EBX
  POP	EDX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos + TYPE HiPos
  @@MATCH:
  POP	EBX
  POP	EDX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function FirstRangeStrPos(const Find, Dest: String; LoPos, HiPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = LoPos
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  MOV	EBX,[EAX - SLO]
  SUB	ECX,HiPos
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@EXIT
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,HiPos
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @MATCH:
  MOV	EBX,[ESI - SLO]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[ESI - SLO]
  MOV	ECX,ESI
  ADD	EDI,EBX
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[ECX + EBX],AH
  JE	@CMP
  POP	ECX
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @YES:
  POP	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  ADD	ECX,HiPos
  MOV	EAX,ECX
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @CHAR:
  DEC	ECX
  ADD	EDX,HiPos
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @@MATCH:
  POP	EBX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function LastRangeStrPos(const Find, Dest: String; HiPos, LoPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = HiPos
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	LoPos,0
  JG	@CONTINUE
  MOV	LoPos,1
  @CONTINUE:
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  SUB	ECX,LoPos
  JL	@EXIT
  ADD	EDX,LoPos
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  CMP	[EDI + ECX - 01h],AL
  JE	@MATCH
  DEC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @MATCH:
  MOV	EBX,[ESI - SLO]
  SUB	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[ESI - SLO]
  MOV	ECX,ESI
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  CMP	[ECX + EBX - 01h],AH
  JE	@CMP
  POP	ECX
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @YES:
  POP	ECX
  SUB	ECX,[ESI - SLO]
  ADD	ECX,LoPos
  INC	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  MOV	EAX,ECX
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @CHAR:
  DEC	EDX
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  DEC	ECX
  JNL	@@LOOP
  @@EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @@MATCH:
  MOV	EAX,LoPos
  ADD	EAX,ECX
  POP	EBX
end;

function StrStart(const Find, Dest: String): Boolean;
begin
  Result := StrAtPos(Find, Dest, 1);
end;

function StrAtPos(const Find, Dest: String; Pos: Integer): Boolean; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = Pos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EDX - SLO]
  SUB	EBX,ECX
  INC	EBX
  CMP	EBX,[EAX - SLO]
  JL	@END
  ADD	EDX,ECX
  MOV	ECX,[EAX - SLO]
  DEC	EDX
  @LOOP:
  DEC	ECX
  JL	@MATCH
  MOV	BL,[EAX + ECX]
  CMP	BL,[EDX + ECX]
  JE	@LOOP
  POP	EBX
  XOR	EAX,EAX
  RET
  @MATCH:
  POP	EBX
  MOV	EAX,true
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
end;

function FirstLoStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  MOV	ECX,[EDX - SLO]
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,ECX
  MOV	AL,[ESI]
  NEG	ECX
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  ADD	ECX,[ESI - SLO]
  JE	@NOT
  SUB	ECX,[ESI - SLO]
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[EBP + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  PUSH	EDX
  MOV	ECX,[EDX - SLO]
  ADD	EDX,[EDX - SLO]
  NEG	ECX
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function LastLoStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  MOV	ECX,[EDX - SLO]
  CMP	ECX,[EAX - SLO]
  JL	@END
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  MOV	AH,[EDI + ECX - 01h]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX - 01h],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  DEC	ECX
  CMP	ECX,[ESI - SLO]
  JNL	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  XOR	AH,[EBP + EBX - 01h]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX - 01h],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX - 01h],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	ECX,[EDX - SLO]
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@LOOP
  AND	AH,0DFh
  JNE	@@LOOP
  @@MATCH:
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function NextLoStrPos(const Find, Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = BegPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EAX - SLO]
  SUB	ECX,[EDX - SLO]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@END
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,[EDX - SLO]
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[EBP + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - SLO]
  POP	EDI
  MOV	EAX,ECX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  POP	EBX
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - SLO]
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - SLO]
end;

function PrevLoStrPos(const Find, Dest: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = EndPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  ADD	ECX,[EAX - SLO]
  CMP	ECX,[EDX - SLO]
  JNG	@CONTINUE
  MOV	ECX,[EDX - SLO]
  INC	ECX
  @CONTINUE:
  DEC	ECX
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  CMP	[EDX - SLO],EBX
  JL	@END
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  MOV	AH,[EDI + ECX - 01h]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX - 01h],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - SLO]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  XOR	AH,[EBP + EBX - 01h]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX - 01h],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX - 01h],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  @END:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - SLO]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@LOOP
  AND	AH,0DFh
  JNE	@@LOOP
  @@MATCH:
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function FirstLoStrPartInRange(const Find, Dest: String; PartLen, LoPos, HiPos: Integer): Integer;
asm	// EAX = @Find, EDX = @Dest, ECX = PartLen
  PUSH	ECX
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  OR	LoPos,0
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  CMP	ECX,[EAX - SLO]
  JG	@EXIT
  MOV	EBX,ECX
  MOV	ECX,LoPos
  SUB	ECX,HiPos
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@EXIT
  CMP	DWORD PTR [EBP - 4],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,HiPos
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @MATCH:
  MOV	EBX,[EBP - 4]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[EBP - 4]
  MOV	ECX,ESI
  ADD	EDI,EBX
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[ECX + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [ECX + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	ECX
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @YES:
  POP	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  ADD	ECX,HiPos
  MOV	EAX,ECX
  POP	EBX
  POP	EDX
  POP	EBP
  RET	TYPE LoPos + TYPE HiPos
  @EXIT:
  POP	EBX
  POP	EDX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos + TYPE HiPos
  @CHAR:
  DEC	ECX
  ADD	EDX,HiPos
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EBX
  POP	EDX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos + TYPE HiPos
  @@MATCH:
  POP	EBX
  POP	EDX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function FirstRangeLoStrPos(const Find, Dest: String; LoPos, HiPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = LoPos
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  MOV	EBX,[EDX - SLO]
  CMP	EBX,HiPos
  JGE	@CONTINUE
  MOV	HiPos,EBX
  @CONTINUE:
  MOV	EBX,[EAX - SLO]
  SUB	ECX,HiPos
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@EXIT
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,HiPos
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @MATCH:
  MOV	EBX,[ESI - SLO]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[ESI - SLO]
  MOV	ECX,ESI
  ADD	EDI,EBX
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[ECX + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [ECX + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	ECX
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @YES:
  POP	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  ADD	ECX,HiPos
  MOV	EAX,ECX
  POP	EBX
  POP	EBP
  RET	TYPE HiPos
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @CHAR:
  DEC	ECX
  ADD	EDX,HiPos
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE HiPos
  @@MATCH:
  POP	EBX
  INC	ECX
  MOV	EAX,ECX
  ADD	EAX,HiPos
end;

function LastRangeLoStrPos(const Find, Dest: String; HiPos, LoPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = HiPos
  PUSH	EBX
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	LoPos,0
  JG	@CONTINUE
  MOV	LoPos,1
  @CONTINUE:
  CMP	ECX,[EDX - SLO]
  JG	@EXIT
  SUB	ECX,LoPos
  JL	@EXIT
  ADD	EDX,LoPos
  CMP	DWORD PTR [EAX - SLO],1
  JE	@CHAR
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - SLO]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  MOV	AH,[EDI + ECX - 01h]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX - 01h],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  DEC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @MATCH:
  MOV	EBX,[ESI - SLO]
  SUB	EBX,ECX
  CMP	EBX,1
  JG	@NOT
  PUSH	EDI
  PUSH	ECX
  ADD	EDI,ECX
  MOV	EBX,[ESI - SLO]
  MOV	ECX,ESI
  ADD	ECX,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  XOR	AH,[ECX + EBX - 01h]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX - 01h],040h
  JE	@BREAK
  TEST	BYTE PTR [ECX + EBX - 01h],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	ECX
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @YES:
  POP	ECX
  SUB	ECX,[ESI - SLO]
  ADD	ECX,LoPos
  INC	ECX
  POP	EDI
  POP	ESI
  POP	EDI
  MOV	EAX,ECX
  POP	EBX
  POP	EBP
  RET	TYPE LoPos
  @EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @CHAR:
  DEC	EDX
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  DEC	ECX
  JNL	@@LOOP
  @@EXIT:
  POP	EBX
  POP	EBP
  XOR	EAX,EAX
  RET	TYPE LoPos
  @@MATCH:
  MOV	EAX,LoPos
  ADD	EAX,ECX
  POP	EBX
end;

function LoStrStart(const Find, Dest: String): Boolean;
begin
  Result := LoStrAtPos(Find, Dest, 1);
end;

function LoStrAtPos(const Find, Dest: String; Pos: Integer): Boolean; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = Pos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EDX - SLO]
  SUB	EBX,ECX
  INC	EBX
  CMP	EBX,[EAX - SLO]
  JL	@END
  ADD	EDX,ECX
  MOV	ECX,[EAX - SLO]
  DEC	EDX
  @LOOP:
  DEC	ECX
  JL	@MATCH
  MOV	BL,[EDX + ECX]
  XOR	BL,[EAX + ECX]
  JE	@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@END
  TEST	BYTE PTR [EAX + ECX],040h
  JE	@END
  AND	BL,0DFh
  JE	@LOOP
  POP	EBX
  XOR	EAX,EAX
  RET
  @MATCH:
  POP	EBX
  MOV	EAX,true
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
end;

procedure DeleteChar(Ch: Char; var Str: String); register; assembler;
asm	// EAX = Ch, EDX = pStr
  OR	DWORD PTR [EDX],0
  JE	@EXIT
  PUSH	EAX
  PUSH	EDX
  PUSH	ECX
  MOV	EAX,EDX
  CALL	UniqueString
  POP	ECX
  POP	EDX
  POP	EAX
  PUSH	EBX
  MOV	EDX,[EDX]
  MOV	ECX,[EDX - SLO]
  PUSH	EDX
  MOV	EBX,EDX
  ADD	EDX,ECX
  NEG	ECX
  @LOOP:
  MOV	AH,[EDX + ECX]
  CMP	AL,AH
  JE	@NEXT
  MOV	[EBX],AH
  INC	EBX
  @NEXT:
  INC	ECX
  JNE	@LOOP
  MOV	BYTE PTR [EBX],0
  @STOP:
  POP	EDX
  SUB	EBX,EDX
  MOV	[EDX - SLO],EBX
  POP	EBX
  @EXIT:
end;

procedure XChgChars(var Str: String; Old, New: Char); register; assembler;
asm	// EAX = @pStr, EDX = Old, ECX = New
  OR	DWORD PTR [EAX],0
  JE	@EXIT
  PUSH	EAX
  PUSH	EDX
  PUSH	ECX
  CALL	UniqueString
  POP	ECX
  POP	EDX
  POP	EAX
  MOV	DH,CL
  MOV	EAX,[EAX]
  MOV	ECX,[EAX - SLO]
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	DL,[EAX + ECX]
  JNE	@LOOP
  MOV	[EAX + ECX],DH
  JMP	@LOOP
  @EXIT:
end;

function XChgStrings(const Str, Old, New: String): String;
var
  iStrLen, iOldLen, iNewLen, iBeg, iEnd, iLen: Integer;
  pResult: PChar;
begin
  iStrLen := Length(Str);
  iOldLen := Length(Old);
  iNewLen := Length(New);
  if (iStrLen > 0) and (iOldLen > 0) then begin
    if iNewLen > iOldLen then
      SetString(Result, nil, iStrLen * iNewLen - iOldLen)
    else SetString(Result, nil, iStrLen);
    pResult := Pointer(Result);
    iBeg := 1;
    repeat
      iEnd := NextStrPos(Old, Str, iBeg);
      iLen := iEnd - iBeg;
      if iLen >= 0 then begin
	if iLen > 0 then begin
	  Move(Str[iBeg], pResult^, iLen);
	  inc(pResult, iLen);
	end;
	if iNewLen > 0 then begin
	  Move(Pointer(New)^, pResult^, iNewLen);
	  inc(pResult, iNewLen);
	end;
	iBeg := iEnd + iOldLen;
      end
      else begin
	iLen := succ(iStrLen) - iBeg;
	if iLen > 0 then begin
	  Move(Str[iBeg], pResult^, iLen);
	  inc(pResult, iLen);
	end;
	Break;
      end;
    until iBeg > iStrLen;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end;
end;

function XChgLoStrings(const Str, Old, New: String): String;
var
  iStrLen, iOldLen, iNewLen, iBeg, iEnd, iLen: Integer;
  pResult: PChar;
begin
  iStrLen := Length(Str);
  iOldLen := Length(Old);
  iNewLen := Length(New);
  if (iStrLen > 0) and (iOldLen > 0) then begin
    if iNewLen > iOldLen then
      SetString(Result, nil, iStrLen * iNewLen - iOldLen)
    else SetString(Result, nil, iStrLen);
    pResult := Pointer(Result);
    iBeg := 1;
    repeat
      iEnd := NextLoStrPos(Old, Str, iBeg);
      iLen := iEnd - iBeg;
      if iLen >= 0 then begin
	if iLen > 0 then begin
	  Move(Str[iBeg], pResult^, iLen);
	  inc(pResult, iLen);
	end;
	if iNewLen > 0 then begin
	  Move(Pointer(New)^, pResult^, iNewLen);
	  inc(pResult, iNewLen);
	end;
	iBeg := iEnd + iOldLen;
      end
      else begin
	iLen := succ(iStrLen) - iBeg;
	if iLen > 0 then begin
	  Move(Str[iBeg], pResult^, iLen);
	  inc(pResult, iLen);
	end;
	Break;
      end;
    until iBeg > iStrLen;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end;
end;

function XChgStrAtPos(const Str, New: String; OldPos, OldLen: Integer): String;
var
  iStrLen, iNewLen, iCopy: Integer;
  pStr, pNew, pResult: PChar;
begin
  iStrLen := Length(Str);
  iNewLen := Length(New);
  if (iStrLen > 0) and (iNewLen > 0) and (OldPos > 0) then begin
    pStr := Pointer(Str);
    pNew := Pointer(New);
    if OldPos > iStrLen then begin
      SetString(Result, nil, iStrLen + iNewLen);
      pResult := Pointer(Result);
      Move(pStr^, pResult^, iStrLen);
      inc(pResult, iStrLen);
      Move(pNew^, pResult^, iNewLen);
    end
    else begin
      if OldPos + OldLen > succ(iStrLen) then OldLen := succ(iStrLen - OldPos);
      SetString(Result, nil, iStrLen + iNewLen - OldLen);
      pResult := Pointer(Result);
      iCopy := pred(OldPos);
      if iCopy > 0 then begin
	Move(pStr^, pResult^, iCopy);
	inc(pResult, iCopy);
      end;
      inc(iCopy, OldLen);
      inc(pStr, iCopy);
      Move(pNew^, pResult^, iNewLen);
      inc(pResult, iNewLen);
      iCopy := iStrLen - iCopy;
      if iCopy > 0 then Move(pStr^, pResult^, iCopy);
    end;
  end
  else Result := Str;
end;

function EnclosedStr(const Str: String; Left, Right: Char; Pos: Integer): String;
var
  iLPos, iRPos, iCount: Integer;
begin
  Result := E;
  if Pos < 1 then
    iLPos := 1
  else iLPos := Pos;
  while not (Str[iLPos] in [NULL, Left]) do inc(iLPos);
  if Str[iLPos] = NULL then Exit;
  iRPos := succ(iLPos);
  iCount := 1;
  repeat
    while not (Str[iRPos] in [NULL, Left, Right]) do inc(iRPos);
    if Str[iRPos] = NULL then Exit;
    if Str[iRPos] = Left then
      inc(iCount)
    else dec(iCount);
    if iCount = 0 then begin
      inc(iLPos);
      Result := Copy(Str, iLPos, iRPos - iLPos);
      Break;
    end
    else inc(iRPos);
  until Str = E;
end;

function ExtractStr(var Source: String; Pos, Len: Integer): String;
begin
  Result := Copy(Source, Pos, Len);
  Delete(Source, Pos, Len);
end;

function AlphaCompareStr(const Str1, Str2: String): Integer;
begin
  Result := CompareString(LOCALE_USER_DEFAULT, NORM_IGNORENONSPACE or NORM_IGNORESYMBOLS,
			  Pointer(Str1), Length(Str1), Pointer(Str2), Length(Str2)) - 2;
end;

function AlphaCompareText(const Str1, Str2: String): Integer;
begin
  Result := CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE or NORM_IGNORENONSPACE or NORM_IGNORESYMBOLS,
			  Pointer(Str1), Length(Str1), Pointer(Str2), Length(Str2)) - 2;
end;

function SameStr(const Str1, Str2: String): Longbool;
begin
  Result := (AlphaCompareStr(Str1, Str2) = 0);
end;

function SameText(const Str1, Str2: String): Longbool;
begin
  Result := (AlphaCompareText(Str1, Str2) = 0);
end;

function RegEntry(KeyIndex: HKEY; const KeyPath: String): BOOL;
var
  hRegKey: HKEY;
begin
  Result := (RegOpenKeyEx(KeyIndex, PChar(KeyPath), 0, KEY_NOTIFY, hRegKey) = ERROR_SUCCESS);
  if Result then RegCloseKey(hRegKey);
end;

function RegDWord(KeyIndex: HKEY; const KeyPath, KeyEntry: String): DWORD;
var
  hRegKey: HKEY;
  dwType, dwBufSize: DWORD;
begin
  dwType := 0;
  dwBufSize := 0;
  if RegOpenKeyEx(KeyIndex, PChar(KeyPath), 0, KEY_QUERY_VALUE, hRegKey) = ERROR_SUCCESS then begin
    try
      if (RegQueryValueEx(hRegKey, PChar(KeyEntry), nil, @dwType, nil, @dwBufSize) = ERROR_SUCCESS) and (dwType = REG_DWORD)
      and (RegQueryValueEx(hRegKey, PChar(KeyEntry), nil, nil, @Result, @dwBufSize) = ERROR_SUCCESS) then begin
	Exit;
      end;
    finally
      RegCloseKey(hRegKey);
    end;
  end;
  Result := 0;
end;

function RegString(KeyIndex: HKEY; const KeyPath, KeyEntry: String): String;
var
  hRegKey: HKEY;
  dwBufSize: DWORD;
begin
  dwBufSize := 0;
  if RegOpenKeyEx(KeyIndex, PChar(KeyPath), 0, KEY_QUERY_VALUE, hRegKey) = ERROR_SUCCESS then begin
    try
      if RegQueryValueEx(hRegKey, PChar(KeyEntry), nil, nil, nil, @dwBufSize) = ERROR_SUCCESS then begin
	SetString(Result, nil, pred(dwBufSize));
	if RegQueryValueEx(hRegKey, PChar(KeyEntry), nil, nil, Pointer(Result), @dwBufSize) = ERROR_SUCCESS then Exit;
      end;
    finally
      RegCloseKey(hRegKey);
    end;
  end;
  Result := E;
end;

function IniDWord(const IniFile, Section, Item: String): DWORD;
begin
  Result := GetPrivateProfileInt(PChar(Section), PChar(Item), 0, PChar(IniFile));
end;

function IniString(const IniFile, Section, Item: String): String;
var
  iLen: Integer;
begin
  iLen := 1023;
  SetString(Result, nil, iLen);
  iLen := GetPrivateProfileString(PChar(Section), PChar(Item), E, Pointer(Result), iLen, PChar(IniFile));
  Delete(Result, succ(iLen), MAXINT);
end;

function ExpandEnvironmentStrings(const Str: String): String;
var
  iLen: Integer;
begin
  if Str <> E then begin
    iLen := Windows.ExpandEnvironmentStrings(Pointer(Str), nil, 0);
    SetString(Result, nil, iLen);
    Delete(Result, Windows.ExpandEnvironmentStrings(Pointer(Str), Pointer(Result), succ(iLen)), MAXINT);
  end
  else Result := E;
end;

function ModuleString(Module: HMODULE; ID: UINT): String;
var
  iLen: Integer;
begin
  iLen := 4097;
  SetString(Result, nil, iLen);
  iLen := LoadString(Module, ID, Pointer(Result), iLen);
  if iLen > 0 then
    Delete(Result, succ(iLen), MAXINT)
  else Result := E;
end;

function ResString(ID: UINT): String;
begin
  Result := ModuleString(hInstance, ID);
end;

function MenuString(Menu: HMENU; ID, Flags: UINT): String;
var
  iLen: Integer;
begin
  iLen := GetMenuString(Menu, ID, nil, 0, Flags);
  if iLen > 0 then begin
    SetString(Result, nil, iLen);
    GetMenuString(Menu, ID, Pointer(Result), iLen, Flags);
    Exit;
  end;
  Result := E;
end;

function ClassString(Window: HWND): String;
var
  iLen: Integer;
begin
  SetString(Result, nil, MAX_PATH);
  iLen := GetClassName(Window, Pointer(Result), MAX_PATH);
  if iLen > 0 then begin
    Delete(Result, succ(iLen), MAXINT);
    CharUpper(Pointer(Result));
  end
  else Result := E;
end;

function WndString(Window: HWND): String;
var
  iLen: Integer;
begin
  iLen := succ(GetWindowTextLength(Window));
  SetString(Result, nil, iLen);
  iLen := GetWindowText(Window, Pointer(Result), iLen);
  if iLen > 0 then
    Delete(Result, succ(iLen), MAXINT)
  else Result := E;
end;

function SysErrorString(Code: Cardinal): String;
var
  pText: PChar;
  iPos: Integer;
begin
  if FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM,
		   nil, DWord(Code), 0, @pText, 0, nil) <> 0 then begin
    try
      iPos := pred(FastStrLen(pText));
      while (iPos >= 0) and (pText[iPos] in [NULL..SP, DOT]) do dec(iPos);
      if iPos > 0 then begin
	SetString(Result, pText, succ(iPos));
	Exit;
      end;
    finally
      LocalFree(HLOCAL(pText));
    end;
  end;
  if Code <> 0 then
    Result := DC + CardToHex(Code)
  else Result := E;
end;

function WriteToConsole(const Text: String): Longbool;
var
  hConsole: THandle;
  dwSize, dwResult: DWord;
begin
  Result := false;
  hConsole := GetStdHandle(STD_OUTPUT_HANDLE);
  if hConsole <> INVALID_HANDLE_VALUE then begin
    dwSize := Length(Text);
    Result := WriteFile(hConsole, Pointer(Text)^, dwSize, dwResult, nil) and (dwResult = dwSize);
  end;
end;

function DialogItemInt(Dialog: HWND; ItemID: Integer): Integer;
var
  fSuccess: Bool;
begin
  Result := GetDlgItemInt(Dialog, ItemID, fSuccess, true);
end;

function DialogItemString(Dialog: HWND; ItemID: Integer): String;
begin
  SetString(Result, nil, MAX_PATH);
  Delete(Result, succ(GetDlgItemText(Dialog, ItemID, Pointer(Result), MAX_PATH)), MAXINT);
end;

type
  TDialogItemParams = Record
    Extended: Longbool;
    Offset: Pointer;
    Count: Integer;
  end;

function GetNextPWideChar(var p: PChar): PWideChar;
var
  pw: PWideChar;
begin
  pw := PWideChar(p);
  case pw^ of
    WideChar(0): inc(pw);
    WideChar($FFFF): inc(pw, 2);
  else
    Result := pw;
    inc(pw, succ(lstrlenW(pw)));
    p := PChar(pw);
    Exit;
  end;
  p := PChar(pw);
  Result := E;
end;

type
  TTypeFace = Array[0..0] of WideChar;
  PDlgTemplate = ^TDlgTemplate;
  TDlgTemplate = Packed Record
    style: DWord;
    exStyle: DWord;
    cDlgItems: Word;
    x: Short;
    y: Short;
    cx: Short;
    cy: Short;
  end;
  PFontDef = ^TFontDef;
  TFontDef = Record
    pointsize: Word;
    typeface: TTypeFace;
  end;
  PDlgTemplateEx = ^TDlgTemplateEx;
  TDlgTemplateEx = Packed Record
    dlgVer: Word;
    signature: Word;
    helpID: DWord;
    exStyle: DWord;
    style: DWord;
    cDlgItems: Word;
    x: Short;
    y: Short;
    cx: Short;
    cy: Short;
  end;
  PFontDefEx = ^TFontDefEx;
  TFontDefEx = Packed Record
    pointsize: Word;
    weight: Word;
    italic: Byte;
    charset: Byte;
    typeface: TTypeFace;
  end;

function GetDialogCaption(hModule: THandle; DialogID: PChar; var DialogItemParams: TDialogItemParams): String;
var
  pDialog: PChar;
  fFontDef: Longbool;
begin
  pDialog := LockResource(LoadResource(hModule, FindResource(hModule, DialogID, RT_DIALOG)));
  if pDialog <> nil then begin
    DialogItemParams.Extended := PDlgTemplateEx(pDialog)^.signature = $FFFF;
    DialogItemParams.Offset := nil;
    if DialogItemParams.Extended then with PDlgTemplateEx(pDialog)^ do begin
      fFontDef := (style and DS_SETFONT <> 0);
      DialogItemParams.Count := cDlgItems;
      inc(pDialog, SizeOf(TDlgTemplateEx));
    end
    else with PDlgTemplate(pDialog)^ do begin
      fFontDef := (style and DS_SETFONT <> 0);
      DialogItemParams.Count := cDlgItems;
      inc(pDialog, SizeOf(TDlgTemplate));
    end;
    // menu
    GetNextPWideChar(pDialog);
    // class name
    GetNextPWideChar(pDialog);
    // dialog caption
    Result := WideCharToString(GetNextPWideChar(pDialog));
    if fFontDef then begin
      // numeric font params
      if DialogItemParams.Extended then
	inc(pDialog, SizeOf(TFontDefEx) - SizeOf(TTypeFace))
      else inc(pDialog, SizeOf(TFontDef) - SizeOf(TTypeFace));
      // font type face name
      GetNextPWideChar(pDialog);
    end;
    with DialogItemParams do if Count > 0 then Offset := pDialog;
    Exit;
  end;
  Result := E;
end;

type
  TExtraCount = Word;
  PDlgItemTemplate = ^TDlgItemTemplate;
  TDlgItemTemplate = Packed Record
    style: DWord;
    exStyle: DWord;
    x: Short;
    y: Short;
    cx: Short;
    cy: Short;
    id: Word;
  end;
  PDlgItemTemplateEx = ^TDlgItemTemplateEx;
  TDlgItemTemplateEx = Packed Record
    helpID: DWord;
    exStyle: DWord;
    style: DWord;
    x: Short;
    y: Short;
    cx: Short;
    cy: Short;
    id: DWord;
  end;

function GetDialogControlText(const DialogItemParams: TDialogItemParams; ItemID: Integer): String;
var
  pItem: PChar;
  iIndex, iItemID, iExtraCount: Integer;
begin
  with DialogItemParams do if Count > 0 then begin
    pItem := Offset;
    for iIndex := 1 to Count do begin
      inc(pItem, (4 - (DWord(pItem) and 3)) and 3);
      if Extended then begin
	iItemID := PDlgItemTemplateEx(pItem)^.id;
	inc(pItem, SizeOf(TDlgItemTemplateEx));
      end
      else begin
	iItemID := PDlgItemTemplate(pItem)^.id;
	inc(pItem, SizeOf(TDlgItemTemplate));
      end;
      // control class
      GetNextPWideChar(pItem);
      // control text
      if iItemID = ItemID then begin
	Result := WideCharToString(GetNextPWideChar(pItem));
	Exit;
      end
      else GetNextPWideChar(pItem);
      // extra count
      iExtraCount := Word(pItem^);
      inc(pItem, SizeOf(TExtraCount));
      // creation data
      if iExtraCount > 0 then begin
	if Extended then
	  inc(pItem, iExtraCount)
	else inc(pItem, iExtraCount + SizeOf(TExtraCount));
      end
    end;
  end;
  Result := E;
end;

function DialogFontInfo(Module: HMODULE; DialogID: PChar): TDlgFont;
var
  pDialog: PChar;
  dip: TDialogItemParams;
  fFontDef: Longbool;
begin
  pDialog := LockResource(LoadResource(Module, FindResource(Module, DialogID, RT_DIALOG)));
  if pDialog <> nil then begin
    dip.Extended := PDlgTemplateEx(pDialog)^.signature = $FFFF;
    dip.Offset := nil;
    if dip.Extended then with PDlgTemplateEx(pDialog)^ do begin
      fFontDef := (style and DS_SETFONT <> 0);
      dip.Count := cDlgItems;
      inc(pDialog, SizeOf(TDlgTemplateEx));
    end
    else with PDlgTemplate(pDialog)^ do begin
      fFontDef := (style and DS_SETFONT <> 0);
      dip.Count := cDlgItems;
      inc(pDialog, SizeOf(TDlgTemplate));
    end;
    // menu
    GetNextPWideChar(pDialog);
    // class name
    GetNextPWideChar(pDialog);
    // dialog caption
    GetNextPWideChar(pDialog);
    if fFontDef then begin
      // numeric font params
      Result.Size := PFontDef(pDialog)^.pointsize;
      if dip.Extended then
	inc(pDialog, SizeOf(TFontDefEx) - SizeOf(TTypeFace))
      else inc(pDialog, SizeOf(TFontDef) - SizeOf(TTypeFace));
      // font type face name
      Result.Name := WideCharToString(GetNextPWideChar(pDialog));
      Exit;
    end;
  end;
  with Result do begin
    Name := E;
    Size := 0;
  end;
end;

function DialogCaptionString(Module: HMODULE; DialogID: PChar): String;
var
  dip: TDialogItemParams;
begin
  Result := GetDialogCaption(Module, DialogID, dip);
end;

function DialogControlString(Module: HMODULE; DialogID: PChar; ControlID: UINT): String;
var
  dip: TDialogItemParams;
begin
  GetDialogCaption(Module, DialogID, dip);
  Result := GetDialogControlText(dip, ControlID);
end;

const
  PROCESSOR_ARCHITECTURE_INTEL		=     0;
  PROCESSOR_ARCHITECTURE_MIPS		=     1;
  PROCESSOR_ARCHITECTURE_ALPHA		=     2;
  PROCESSOR_ARCHITECTURE_PPC		=     3;
  PROCESSOR_ARCHITECTURE_SHX		=     4;
  PROCESSOR_ARCHITECTURE_ARM		=     5;
  PROCESSOR_ARCHITECTURE_IA64		=     6;
  PROCESSOR_ARCHITECTURE_ALPHA64	=     7;
  PROCESSOR_ARCHITECTURE_MSIL		=     8;
  PROCESSOR_ARCHITECTURE_AMD64		=     9;
  PROCESSOR_ARCHITECTURE_IA32_ON_WIN64	=    10;
  PROCESSOR_ARCHITECTURE_UNKNOWN	= $FFFF;

type
  TOSVersionInfoEx = Record
    dwOSVersionInfoSize: DWORD;
    dwMajorVersion: DWORD;
    dwMinorVersion: DWORD;
    dwBuildNumber: DWORD;
    dwPlatformId: DWORD;
    szCSDVersion: Array[0..127] of Char;
    wServicePackMajor: Word;
    wServicePackMinor: Word;
    wSuiteMask: Word;
    wProductType: Byte;
    wReserved: Byte;
  end;

function GetVersionEx(var lpVersionInformation: TOSVersionInfoEx): Bool; stdcall;
		      external kernel32 name 'GetVersionExA';
procedure GetNativeSystemInfo(var lpSystemInfo: TSystemInfo); stdcall;
			      external kernel32 name 'GetNativeSystemInfo';

function GetSystemFileVersion: Cardinal;
begin
  Result := GetSimpleFileVersion(GetSysDir + 'Kernel32.dll', false);
end;

function GetSimpleFileVersion(const FileName: String; MainOnly: Longbool): Cardinal;
var
  fvi: TFileVersionInfo;
  iFirstSepPos, iNextSepPos: Integer;
begin
  FillChar(fvi, SizeOf(TFileVersionInfo), 0);
  with fvi do begin
    QueryFile := FileName;
    Include(QueryParameters, fviFileVersion);
    if FileVersionInfo(fvi) then begin
      iFirstSepPos := 1;
      while FileVersion[iFirstSepPos] in DIGITS do inc(iFirstSepPos);
      if iFirstSepPos > 1 then begin
	iNextSepPos := succ(iFirstSepPos);
	while FileVersion[iNextSepPos] in DIGITS do inc(iNextSepPos);
	Result := StrToIntDef(Copy(FileVersion, 1, pred(iFirstSepPos)), 0);
	if MainOnly then Exit;
	Result := (Result shl 16) or
		   Cardinal(StrToIntDef(Copy(FileVersion, succ(iFirstSepPos), pred(iNextSepPos - iFirstSepPos)), 0));
	Exit;
      end;
    end;
  end;
  Result := 0;
end;

function GetWinVersion(Simple, AddNumber, AddSvcPack, AddCpuType: Longbool): String;
var
  vi: TOSVersionInfoEx;
  si: TSystemInfo;
  fv: Cardinal;
begin
  with vi do begin
    FillChar(vi, SizeOf(TOSVersionInfoEx), 0);
    dwOSVersionInfoSize := SizeOf(TOSVersionInfoEx);
    if not GetVersionEx(vi) then begin
      dec(dwOSVersionInfoSize, 8);
      GetVersionEx(vi);
    end;
    if Simple then begin
      if dwPlatformId = VER_PLATFORM_WIN32_NT then begin
	if dwMajorVersion >= 6 then begin
	  fv := GetSystemFileVersion;
	  dwMajorVersion := fv shr 16;
	  dwMinorVersion := fv and $FFFF;
	end;
      end;
      Result := IntToStr(dwMajorVersion) + DOT + IntToStr(dwMinorVersion);
      Exit;
    end;
    case dwPlatformId of
      VER_PLATFORM_WIN32s: Result := '32s';
      VER_PLATFORM_WIN32_WINDOWS: begin
	case dwMinorVersion of
	   0: Result := '95';
	  10: Result := '98';
	  90: Result := 'ME';
	end;
      end;
      VER_PLATFORM_WIN32_NT: begin
	if dwMajorVersion >= 6 then begin
	  fv := GetSystemFileVersion;
	  dwMajorVersion := fv shr 16;
	  dwMinorVersion := fv and $FFFF;
	end;
	case dwMajorVersion of
	  10: case dwMinorVersion of
	    0: Result := '10.0/S16';
	  end;
	  08: case dwMinorVersion of
	    0: Result := '8.0/S12';
	    1: Result := '8.1/S12R';
	  end;
	  06: case dwMinorVersion of
	    0: Result := 'VI/S8';
	    1: Result := '7.0/S8R';
	  end;
	  05: case dwMinorVersion of
	    0: Result := '2K';
	    1: Result := 'XP';
	    2: Result := 'S3';
	  end;
	else
	  Result := 'NT';
	end;
      end;
    end;
    Result := 'Win' + Result;
    if AddNumber then Result := Result + SP + LB + IntToStr(dwMajorVersion) + DOT + IntToStr(dwMinorVersion) + RB;
    if AddSvcPack then Result := Result + ' SP ' + IntToStr(wServicePackMajor);
    if AddCpuType then begin
      if dwMajorVersion >= 5 then
	GetNativeSystemInfo(si)
      else GetSystemInfo(si);
      case si.wProcessorArchitecture of
	PROCESSOR_ARCHITECTURE_INTEL..PROCESSOR_ARCHITECTURE_ARM: Result := Result + ' - 32bit';
	PROCESSOR_ARCHITECTURE_IA64..PROCESSOR_ARCHITECTURE_ALPHA64,
	PROCESSOR_ARCHITECTURE_AMD64..PROCESSOR_ARCHITECTURE_IA32_ON_WIN64: Result := Result + ' - 64bit';
	PROCESSOR_ARCHITECTURE_MSIL: Result := Result + ' - MSIL';
      else
	Result := Result + ' - ????';
      end;
    end;
  end;
end;

function IsMinWinVersion(const MinWinVersion: String): Longbool;

  function GetMajorVersion(const Version: String; DotPos: Integer): Cardinal;
  begin
    Result := StrToIntDef(Copy(Version, 1, pred(DotPos)), 0);
  end;

  function GetMinorVersion(const Version: String; DotPos: Integer): Cardinal;
  begin
    Result := StrToIntDef(Copy(Version, succ(DotPos), MAXINT), 0);
  end;

var
  sCurrentVersion: String;
  iMinPos, iCurPos: Integer;
  cMinVersion, cCurVersion: Cardinal;
begin
  sCurrentVersion := GetWinVersion(true, false, false, false);
  if (MinWinVersion <> E) and (sCurrentVersion <> E) then begin
    iMinPos := 1;
    while MinWinVersion[iMinPos] in DIGITS do inc(iMinPos);
    iCurPos := 1;
    while sCurrentVersion[iCurPos] in  DIGITS do inc(iCurPos);
    cMinVersion := GetMajorVersion(MinWinVersion, iMinPos) shl 16 + GetMinorVersion(MinWinVersion, iMinPos);
    cCurVersion := GetMajorVersion(sCurrentVersion, iCurPos) shl 16 + GetMinorVersion(sCurrentVersion, iCurPos);
    Result := (cCurVersion >= cMinVersion);
    Exit;
  end;
  Result := false;
end;

function MainWinVersion: Cardinal;
var
  vi: TOSVersionInfoEx;
  fv: Cardinal;
begin
  with vi do begin
    FillChar(vi, SizeOf(TOSVersionInfoEx), 0);
    dwOSVersionInfoSize := SizeOf(TOSVersionInfoEx);
    if not GetVersionEx(vi) then begin
      dec(dwOSVersionInfoSize, 8);
      GetVersionEx(vi);
    end;
    if dwPlatformId = VER_PLATFORM_WIN32_NT then begin
      if dwMajorVersion >= 6 then begin
	fv := GetSystemFileVersion;
	dwMajorVersion := fv shr 16;
	dwMinorVersion := fv and $FFFF;
      end;
    end;
    Result := dwMajorVersion;
  end;
end;

function GetComputerName: String;
var
  dwLen: DWord;
begin
  dwLen := MAX_COMPUTERNAME_LENGTH;
  SetString(Result, nil, dwLen);
  if Windows.GetComputerName(Pointer(Result), dwLen) then
    Delete(Result, succ(dwLen), MAXINT)
  else Result := E;
end;

function GetUserName: String;
var
  dwLen: DWord;
begin
  dwLen := 0;
  WNetGetUser(nil, nil, dwLen);
  SetString(Result, nil, pred(dwLen));
  WNetGetUser(nil, Pointer(Result), dwLen);
end;

function GetAppDir: String;
var
  iIndex: Integer;
begin
  Result := ParamStr(0);
  iIndex := Length(Result);
  while Result[iIndex] <> BS do dec(iIndex);
  Delete(Result, succ(iIndex), MAXINT);
end;

function GetCurDir: String;
begin
  SetString(Result, nil, GetCurrentDirectory(0, nil));
  AddBackslash(Result, GetCurrentDirectory(Length(Result), Pointer(Result)));
end;

function GetWinDir: String;
begin
  SetString(Result, nil, GetWindowsDirectory(nil, 0));
  AddBackslash(Result, GetWindowsDirectory(Pointer(Result), Length(Result)));
end;

function GetSysDir: String;
begin
  SetString(Result, nil, GetSystemDirectory(nil, 0));
  AddBackslash(Result, GetSystemDirectory(Pointer(Result), Length(Result)));
end;

function GetTempDir: String;
begin
  SetString(Result, nil, GetTempPath(0, nil));
  AddBackslash(Result, GetTempPath(Length(Result), Pointer(Result)));
end;

function GetTempFile: String;
begin
  try
    SetString(Result, nil, MAX_PATH);
    if (GetTempPath(MAX_PATH, Pointer(Result)) = 0)
    or (GetTempFileName(Pointer(Result), 'TMP', 0, Pointer(Result)) = 0) then begin
      Result := E;
    end
    else Delete(Result, succ(FastStrLen(PChar(Result))), MAXINT);
  except
    Result := E;
  end;
end;

function GetModuleFileName(Module: THandle): String;
var
  iLen: Integer;
begin
  SetString(Result, nil, MAX_PATH);
  iLen := Windows.GetModuleFileName(Module, Pointer(Result), MAX_PATH);
  Delete(Result, succ(iLen), MAXINT);
end;

function GetModuleFilePath(Module: THandle): String;
var
  iLen: Integer;
begin
  Result := GetModuleFileName(Module);
  iLen := Length(Result);
  if iLen <> 0 then while (iLen > 0) and (Result[iLen] <> BS) do dec(iLen);
  Delete(Result, succ(iLen), MAXINT);
end;

function GetSystemFolderPath(FolderID: Integer): String;
var
  idlSystem: PItemIDList;
begin
  SetString(Result, nil, MAX_PATH);
  if (SHGetSpecialFolderLocation(0, FolderID, idlSystem) = S_OK)
  and (SHGetPathFromIDList(idlSystem, Pointer(Result))) then begin
    AddBackslash(Result, FastStrLen(Pointer(Result)));
  end
  else Result := E;
end;

procedure AddBackslash(var Path: String; PathLen: Integer);
begin
  if PathLen > 0 then begin
    if Path[PathLen] <> BS then begin
      if PathLen < Length(Path) then
	Path[succ(PathLen)] := BS
      else Path := Path + BS;
      Delete(Path, PathLen + 2, MAXINT);
    end
    else Delete(Path, succ(PathLen), MAXINT);
  end
  else Path := E;
end;

function FileVersionInfo(var FileVersionInfo: TFileVersionInfo): Longbool;
const
  COMMENTS_		= '\Comments';
  COMPANY_NAME		= '\CompanyName';
  FILE_DESCRIPTION	= '\FileDescription';
  FILE_VERSION		= '\FileVersion';
  INTERNAL_NAME		= '\InternalName';
  LEGAL_COPYRIGHT	= '\LegalCopyright';
  LEGAL_TRADEMARKS	= '\LegalTrademarks';
  ORIGINAL_FILENAME	= '\OriginalFilename';
  PRIVATE_BUILD		= '\PrivateBuild';
  PRODUCT_NAME		= '\ProductName';
  PRODUCT_VERSION	= '\ProductVersion';
  SPECIAL_BUILD		= '\SpecialBuild';

  function VerFixed(const Info: String): TVSFixedFileInfo;
  var
    pData: PVSFixedFileInfo;
    uSize: UInt;
  begin
    if VerQueryValue(Pointer(Info), BS, Pointer(pData), uSize) and (uSize > 0) then begin
      Result := pData^;
      Exit;
    end;
    FillByte(Result, SizeOf(TVSFixedFileInfo), 0);
  end;

  function VerTranslation(const Info: String): String;
  const
    VFI_TRANSLATION = '\VarFileInfo\Translation';
  type
    PTranslationInfo = ^TTranslationInfo;
    TTranslationInfo = Record
      wLanguage: Word;
      wCodePage: Word;
    end;
  var
    pti: PTranslationInfo;
    uSize: UInt;
  begin
    if VerQueryValue(Pointer(Info), VFI_TRANSLATION, Pointer(pti), uSize) and (uSize > 0) then begin
      with pti^ do Result := CardToHex(wLanguage shl 16 + wCodePage);
      Exit;
    end;
    Result := E;
  end;

  function VerString(const Info, Language, Value: String): String;
  const
    FILE_INFO = '\StringFileInfo\';
  var
    pData: Pointer;
    uSize: UInt;
  begin
    if VerQueryValue(Pointer(Info), Pointer(FILE_INFO + Language + Value), pData, uSize) and (uSize > 0) then begin
      SetString(Result, PChar(pData), pred(uSize));
      Exit;
    end;
    Result := E;
  end;

var
  dwSize: DWord;
  hInfo: THandle;
  sInfo: String;
begin
  Result := false;
  with FileVersionInfo do begin
    if QueryFile <> E then begin
      dwSize := GetFileVersionInfoSize(Pointer(QueryFile), hInfo);
      if dwSize > 0 then begin
	SetString(sInfo, nil, dwSize);
	Result := GetFileVersionInfo(Pointer(QueryFile), hInfo, dwSize, Pointer(sInfo));
      end;
    end;
    if not Result then Exit;
    if fviFixed in QueryParameters then Fixed := VerFixed(sInfo);
    if QueryLanguage = E then QueryLanguage := VerTranslation(sInfo);
    if fviComments in QueryParameters then Comments := VerString(sInfo, QueryLanguage, COMMENTS_);
    if fviCompanyName in QueryParameters then CompanyName := VerString(sInfo, QueryLanguage, COMPANY_NAME);
    if fviFileDescription in QueryParameters then FileDescription := VerString(sInfo, QueryLanguage, FILE_DESCRIPTION);
    if fviFileVersion in QueryParameters then FileVersion := VerString(sInfo, QueryLanguage, FILE_VERSION);
    if fviInternalName in QueryParameters then InternalName := VerString(sInfo, QueryLanguage, INTERNAL_NAME);
    if fviLegalCopyright in QueryParameters then LegalCopyright := VerString(sInfo, QueryLanguage, LEGAL_COPYRIGHT);
    if fviLegalTrademarks in QueryParameters then LegalTrademarks := VerString(sInfo, QueryLanguage, LEGAL_TRADEMARKS);
    if fviOriginalFilename in QueryParameters then OriginalFilename := VerString(sInfo, QueryLanguage, ORIGINAL_FILENAME);
    if fviPrivateBuild in QueryParameters then PrivateBuild := VerString(sInfo, QueryLanguage, PRIVATE_BUILD);
    if fviProductName in QueryParameters then ProductName := VerString(sInfo, QueryLanguage, PRODUCT_NAME);
    if fviProductVersion in QueryParameters then ProductVersion := VerString(sInfo, QueryLanguage, PRODUCT_VERSION);
    if fviSpecialBuild in QueryParameters then SpecialBuild := VerString(sInfo, QueryLanguage, SPECIAL_BUILD);
  end;
end;

function MoveFile(const Source, Dest: String; Overwrite: Longbool): Longbool;
begin
  Result := MoveFileEx(Pointer(Source), Pointer(Dest), MOVEFILE_REPLACE_EXISTING * ord(Overwrite = true) +
  						       MOVEFILE_COPY_ALLOWED);
end;

function WriteTextFile(const Name, Text: String): Longbool;
var
  hFile: THandle;
  dwSize, dwResult: DWord;
begin
  Result := false;
  hFile := CreateWriteFile(Name);
  if hFile <> INVALID_HANDLE_VALUE then begin
    try
      dwSize := Length(Text);
      Result := WriteFile(hFile, Pointer(Text)^, dwSize, dwResult, nil) and (dwResult = dwSize);
    finally
      CloseHandle(hFile);
    end;
  end;
end;

function ReadTextFile(const Name: String): String;
var
  hFile: THandle;
  dwSize, dwResult: DWord;
begin
  hFile := OpenReadFile(Name);
  if hFile <> INVALID_HANDLE_VALUE then begin
    try
      dwSize := SetFilePointer(hFile, 0, nil, FILE_END);
      SetString(Result, nil, dwSize);
      SetFilePointer(hFile, 0, nil, FILE_BEGIN);
      if ReadFile(hFile, Pointer(Result)^, dwSize, dwResult, nil) and (dwResult = dwSize) then Exit;
    finally
      CloseHandle(hFile);
    end;
  end;
  Result := E;
end;

function CreateWriteFile(const Name: String): THandle;
begin
  Result := CreateFile(Pointer(Name), GENERIC_WRITE, FILE_SHARE_READ, nil, CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH, 0);
end;

function OpenReadFile(const Name: String): THandle;
begin
  Result := CreateFile(Pointer(Name), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, 0);
end;

function CloseFile(const Handle: THandle): Longbool;
begin
  Result := CloseHandle(Handle);
end;

function DeleteFile(const Name: String): Longbool;
begin
  Result := Windows.DeleteFile(Pointer(Name));
end;

function FileSize(const Name: String): Cardinal;
var
  hFile: THandle;
begin
  hFile := OpenReadFile(Name);
  if hFile <> INVALID_HANDLE_VALUE then begin
    Result := GetFileSize(hFile, nil);
    CloseHandle(hFile);
  end
  else Result := Cardinal(-1);
end;

function FileExists(const Name: String): Longbool;
var
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := false;
  if Name <> E then begin
    hSearch := FindFirstFile(Pointer(Name), fdFound);
    if hSearch <> INVALID_HANDLE_VALUE then begin
      Result := (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY = 0);
      Windows.FindClose(hSearch);
    end;
  end;
end;

function DirExists(const Name: String): Longbool;
var
  sDir: String;
  iLen: Integer;
  hSearch: THandle;
  fdFound: TWin32FindData;
begin
  Result := false;
  sDir := Name;
  if sDir <> E then begin
    iLen := Length(sDir);
    if sDir[iLen] = BS then Delete(sDir, iLen, 1);
    hSearch := FindFirstFile(Pointer(sDir), fdFound);
    if hSearch <> INVALID_HANDLE_VALUE then begin
      Result := (fdFound.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY <> 0);
      Windows.FindClose(hSearch);
    end;
  end;
end;

function SetCurDir(Name: String): Longbool;
begin
  Result:= SetCurrentDirectory(Pointer(Name));
end;

const
  MAPVK_VK_TO_VSC    = 0;
  MAPVK_VSC_TO_VK    = 1;
  MAPVK_VK_TO_CHAR   = 2;
  MAPVK_VSC_TO_VK_EX = 3;
  MAPVK_VK_TO_VSC_EX = 4;

const
  KEYEVENTF_EXTENDEDKEY = 1;
  KEYEVENTF_KEYUP       = 2;
  KEYEVENTF_UNICODE     = 4;
  KEYEVENTF_SCANCODE    = 8;

const
  MOUSEEVENTF_MOVE            = $0001; { mouse move }
  MOUSEEVENTF_LEFTDOWN        = $0002; { left button down }
  MOUSEEVENTF_LEFTUP          = $0004; { left button up }
  MOUSEEVENTF_RIGHTDOWN       = $0008; { right button down }
  MOUSEEVENTF_RIGHTUP         = $0010; { right button up }
  MOUSEEVENTF_MIDDLEDOWN      = $0020; { middle button down }
  MOUSEEVENTF_MIDDLEUP        = $0040; { middle button up }
  MOUSEEVENTF_WHEEL           = $0800; { wheel button rolled }
  MOUSEEVENTF_HWHEEL          = $1000; { hwheel button rolled }
  MOUSEEVENTF_MOVE_NOCOALESCE = $2000; { hwheel button rolled }
  MOUSEEVENTF_ABSOLUTE        = $8000; { absolute move }

type
  PMouseInput = ^TMouseInput;
  tagMOUSEINPUT = packed record
    dx: Longint;
    dy: Longint;
    mouseData: DWORD;
    dwFlags: DWORD;
    time: DWORD;
    dwExtraInfo: DWORD;
  end;
  TMouseInput = tagMOUSEINPUT;

  PKeybdInput = ^TKeybdInput;
  tagKEYBDINPUT = packed record
    wVk: WORD;
    wScan: WORD;
    dwFlags: DWORD;
    time: DWORD;
    dwExtraInfo: DWORD;
  end;
  TKeybdInput = tagKEYBDINPUT;

  PHardwareInput = ^THardwareInput;
  tagHARDWAREINPUT = packed record
    uMsg: DWORD;
    wParamL: WORD;
    wParamH: WORD;
  end;
  THardwareInput = tagHARDWAREINPUT;

const
  INPUT_MOUSE = 0;
  INPUT_KEYBOARD = 1;
  INPUT_HARDWARE = 2;

type
  PInput = ^TInput;
  tagINPUT = packed record
    Itype: DWORD;
    case Integer of
      0: (mi: TMouseInput);
      1: (ki: TKeybdInput);
      2: (hi: THardwareInput);
  end;
  TInput = tagINPUT;

procedure keybd_event(bVk: Byte; bScan: Byte; dwFlags, dwExtraInfo: DWORD); stdcall;
	  external user32 name 'keybd_event';
procedure mouse_event(dwFlags, dx, dy, dwData, dwExtraInfo: DWORD); stdcall;
	  external user32 name 'mouse_event';
function SendInput(cInputs: UINT; var pInputs: TInput; cbSize: Integer): UINT; stdcall;
	 external user32 name 'SendInput';

function SendKeys(const Keys: Array of Cardinal): Longbool;
const
  MAXSIZE = MAXINT div SizeOf(TInput);
type
  PInputArray = ^TInputArray;
  TInputArray = Array[0..pred(MAXSIZE)] of TInput;
var
  iCount, iIndex, iItems, iKeyUp: Integer;
  pia: PInputArray;
begin
  iCount := succ(High(Keys));
  if iCount > 0 then begin
    iItems := iCount * 2;
    pia := AllocMem(iItems * SizeOf(TInput));
    try
      for iIndex := 0 to pred(iCount) do begin
	iKeyUp := pred(iItems - iIndex);
	with pia^[iIndex] do begin
	  Itype := INPUT_KEYBOARD;
	  with ki do begin
	    wVk := Keys[iIndex];
	    wScan := MapVirtualKey(wVk, MAPVK_VK_TO_VSC);
	    dwExtraInfo := Cardinal(-1);
	  end;
	end;
	pia^[iKeyUp] := pia^[iIndex];
	pia^[iKeyUp].ki.dwFlags := KEYEVENTF_KEYUP;
      end;
      Result := (SendInput(iItems, pia[0], SizeOf(TInput)) <> 0);
      Exit;
    finally
      FreeMem(pia);
    end;
  end;
  Result := false;
end;

function RtfToClipboard(const RtfString: String): Longbool;
var
  cfRtf: UINT;
  hRtf, hText: HGLOBAL;
begin
  Result := false;
  cfRtf := RegisterClipboardFormat(RichEdit.CF_RTF);
  if cfRtf = 0 then Exit;
  if not OpenClipboard(0) then Exit;
  try
    EmptyClipboard;
    hRtf := GetStringHandleA(RtfString, 0, false);
    hText := GetStringHandleA(RtfString, 0, false);
    Result := (hRtf <> 0) and (SetClipboardData(cfRtf, hRtf) <> 0) and
	      (hText <> 0) and (SetClipboardData(CF_TEXT, hText) <> 0);
  finally
    CloseClipboard;
  end;
end;

function HtmlToClipboard(const HtmlString: String): Longbool;
var
  cfHtml, uHtmlBeg, uHtmlEnd: UINT;
  sHtmlBeg, sHtmlEnd: String;
  hHtml, hText: HGLOBAL;
begin
  Result := false;
  cfHtml := RegisterClipboardFormat('HTML Format');
  if cfHtml = 0 then Exit;
  if not OpenClipboard(0) then Exit;
  try
    EmptyClipboard;
    uHtmlBeg := 105 + pred(Pos('<html', LowerCase(HtmlString)));
    if uHtmlBeg < 105 then uHtmlBeg := 105;
    uHtmlEnd := uHtmlBeg + UINT(Length(HtmlString));
    sHtmlBeg := Format('%.10d', [uHtmlBeg]) + CRLF;
    sHtmlEnd := Format('%.10d', [uHtmlEnd]) + CRLF;
    hHtml := GetStringHandleA(
      'Version:0.9' + CRLF +
      'StartHTML:' + sHtmlBeg +
      'EndHTML:' + sHtmlEnd +
      'StartFragment:' + sHtmlBeg +
      'EndFragment:' + sHtmlEnd +
      HtmlString, 0, false);
    hText := GetStringHandleA(HtmlString, 0, false);
    Result := (hHtml <> 0) and (SetClipboardData(cfHtml, hHtml) <> 0) and
	      (hText <> 0) and (SetClipboardData(CF_TEXT, hText) <> 0);
  finally
    CloseClipboard;
  end;
end;


end.

