{$J+,Z4}
unit WebInterface;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2007-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, WinSock, TimeUtils, SysUtils, Classes;

const
  WININET = 'WinInet.dll';

//
// common per-API flags (new APIs)
//
  WININET_API_FLAG_ASYNC			= $00000001;  // force async operation
  WININET_API_FLAG_SYNC				= $00000004;  // force sync operation
  WININET_API_FLAG_USE_CONTEXT			= $00000008;  // use value supplied in dwContext (even if 0)

//
// flags for INTERNET_CONNECTED_INFO dwFlags
//
  ISO_FORCE_DISCONNECTED			= $00000001;

//
// the following can be indicated in a state change notification:
//
  INTERNET_STATE_CONNECTED			= $00000001;  // connected state (mutually exclusive with disconnected)
  INTERNET_STATE_DISCONNECTED			= $00000002;  // disconnected from network
  INTERNET_STATE_DISCONNECTED_BY_USER		= $00000010;  // disconnected by user request
  INTERNET_STATE_IDLE				= $00000100;  // no network requests being made (by Wininet)
  INTERNET_STATE_BUSY				= $00000200;  // network requests being made (by Wininet)

//
// if this value is presented as the dwContext parameter
// then no call-backs will be made for that API
//
  INTERNET_NO_CALLBACK				= 0;

//
// if the following value is returned by InternetSetStatusCallback, then
// probably an invalid (non-code) address was supplied for the callback
//
  INTERNET_INVALID_STATUS_CALLBACK 		= $FFFFFFFF;

//
// status manifests for Internet status callback
//
  INTERNET_STATUS_RESOLVING_NAME		=  10;
  INTERNET_STATUS_NAME_RESOLVED			=  11;
  INTERNET_STATUS_CONNECTING_TO_SERVER		=  20;
  INTERNET_STATUS_CONNECTED_TO_SERVER		=  21;
  INTERNET_STATUS_SENDING_REQUEST		=  30;
  INTERNET_STATUS_REQUEST_SENT			=  31;
  INTERNET_STATUS_RECEIVING_RESPONSE		=  40;
  INTERNET_STATUS_RESPONSE_RECEIVED		=  41;
  INTERNET_STATUS_CTL_RESPONSE_RECEIVED		=  42;
  INTERNET_STATUS_PREFETCH			=  43;
  INTERNET_STATUS_CLOSING_CONNECTION		=  50;
  INTERNET_STATUS_CONNECTION_CLOSED		=  51;
  INTERNET_STATUS_HANDLE_CREATED		=  60;
  INTERNET_STATUS_HANDLE_CLOSING		=  70;
  INTERNET_STATUS_DETECTING_PROXY		=  80;
  INTERNET_STATUS_REQUEST_COMPLETE		= 100;
  INTERNET_STATUS_REDIRECT			= 110;
  INTERNET_STATUS_INTERMEDIATE_RESPONSE		= 120;
  INTERNET_STATUS_USER_INPUT_REQUIRED		= 140;
  INTERNET_STATUS_STATE_CHANGE			= 200;
  INTERNET_STATUS_COOKIE_SENT			= 320;
  INTERNET_STATUS_COOKIE_RECEIVED		= 321;
  INTERNET_STATUS_PRIVACY_IMPACTED		= 324;
  INTERNET_STATUS_P3P_HEADER			= 325;
  INTERNET_STATUS_P3P_POLICYREF			= 326;
  INTERNET_STATUS_COOKIE_HISTORY		= 327;

//
// options manifests for Internet{Query|Set}Option
//
  INTERNET_OPTION_CALLBACK			= 1;
  INTERNET_OPTION_CONNECT_TIMEOUT		= 2;
  INTERNET_OPTION_CONNECT_RETRIES		= 3;
  INTERNET_OPTION_CONNECT_BACKOFF		= 4;
  INTERNET_OPTION_SEND_TIMEOUT			= 5;
  INTERNET_OPTION_CONTROL_SEND_TIMEOUT		= INTERNET_OPTION_SEND_TIMEOUT;
  INTERNET_OPTION_RECEIVE_TIMEOUT		= 6;
  INTERNET_OPTION_CONTROL_RECEIVE_TIMEOUT	= INTERNET_OPTION_RECEIVE_TIMEOUT;
  INTERNET_OPTION_DATA_SEND_TIMEOUT		= 7;
  INTERNET_OPTION_DATA_RECEIVE_TIMEOUT		= 8;
  INTERNET_OPTION_HANDLE_TYPE			= 9;
  INTERNET_OPTION_READ_BUFFER_SIZE		= 12;
  INTERNET_OPTION_WRITE_BUFFER_SIZE		= 13;
  INTERNET_OPTION_ASYNC_ID			= 15;
  INTERNET_OPTION_ASYNC_PRIORITY		= 16;
  INTERNET_OPTION_PARENT_HANDLE			= 21;
  INTERNET_OPTION_KEEP_CONNECTION		= 22;
  INTERNET_OPTION_REQUEST_FLAGS			= 23;
  INTERNET_OPTION_EXTENDED_ERROR		= 24;
  INTERNET_OPTION_OFFLINE_MODE			= 26;
  INTERNET_OPTION_CACHE_STREAM_HANDLE		= 27;
  INTERNET_OPTION_USERNAME			= 28;
  INTERNET_OPTION_PASSWORD			= 29;
  INTERNET_OPTION_ASYNC				= 30;
  INTERNET_OPTION_SECURITY_FLAGS		= 31;
  INTERNET_OPTION_SECURITY_CERTIFICATE_STRUCT	= 32;
  INTERNET_OPTION_DATAFILE_NAME			= 33;
  INTERNET_OPTION_URL				= 34;
  INTERNET_OPTION_SECURITY_CERTIFICATE		= 35;
  INTERNET_OPTION_SECURITY_KEY_BITNESS		= 36;
  INTERNET_OPTION_REFRESH			= 37;
  INTERNET_OPTION_PROXY				= 38;
  INTERNET_OPTION_SETTINGS_CHANGED		= 39;
  INTERNET_OPTION_VERSION			= 40;
  INTERNET_OPTION_USER_AGENT			= 41;
  INTERNET_OPTION_END_BROWSER_SESSION		= 42;
  INTERNET_OPTION_PROXY_USERNAME		= 43;
  INTERNET_OPTION_PROXY_PASSWORD		= 44;
  INTERNET_OPTION_CONTEXT_VALUE			= 45;
  INTERNET_OPTION_CONNECT_LIMIT			= 46;
  INTERNET_OPTION_SECURITY_SELECT_CLIENT_CERT	= 47;
  INTERNET_OPTION_POLICY			= 48;
  INTERNET_OPTION_DISCONNECTED_TIMEOUT		= 49;
  INTERNET_OPTION_CONNECTED_STATE		= 50;
  INTERNET_OPTION_IDLE_STATE			= 51;
  INTERNET_OPTION_OFFLINE_SEMANTICS		= 52;
  INTERNET_OPTION_SECONDARY_CACHE_KEY		= 53;
  INTERNET_OPTION_CALLBACK_FILTER		= 54;
  INTERNET_OPTION_CONNECT_TIME			= 55;
  INTERNET_OPTION_SEND_THROUGHPUT		= 56;
  INTERNET_OPTION_RECEIVE_THROUGHPUT		= 57;
  INTERNET_OPTION_REQUEST_PRIORITY		= 58;
  INTERNET_OPTION_HTTP_VERSION			= 59;
  INTERNET_OPTION_RESET_URLCACHE_SESSION	= 60;

//
// access types for InternetOpen()
//
  INTERNET_OPEN_TYPE_PRECONFIG			= 0;  // use registry configuration
  PRE_CONFIG_INTERNET_ACCESS			= INTERNET_OPEN_TYPE_PRECONFIG;
  INTERNET_OPEN_TYPE_DIRECT			= 1;  // direct to net
  LOCAL_INTERNET_ACCESS				= INTERNET_OPEN_TYPE_DIRECT;
  GATEWAY_INTERNET_ACCESS			= 2;  // Internet via gateway
  INTERNET_OPEN_TYPE_PROXY			= 3;  // via named ProxyServer
  CERN_PROXY_INTERNET_ACCESS			= INTERNET_OPEN_TYPE_PROXY;
  INTERNET_OPEN_TYPE_PRECONFIG_WITH_NO_AUTOPROXY= 4;  // prevent using java/script/INS

//
// flags for InternetAutodial
//
  INTERNET_AUTODIAL_FORCE_ONLINE		= $00000001;  // forces an online Internet connection
  INTERNET_AUTODIAL_FORCE_UNATTENDED		= $00000002;  // forces an unattended Internet dial-up
  INTERNET_AUTODIAL_FAILIFSECURITYCHECK		= $00000004;  // causes InternetAutodial to fail if
							      // file and printer sharing is disabled
  INTERNET_AUTODIAL_OVERRIDE_NET_PRESENT	= $00000008;  // dial the modem connection even when a
							      // network connection to the Internet exists

//
// flags for InternetOpen()
//
  INTERNET_FLAG_NEED_FILE			= $00000010;  // need a file for this request
  INTERNET_FLAG_FORMS_SUBMIT			= $00000040;  // this is a forms submit
  INTERNET_FLAG_CACHE_ASYNC			= $00000080;  // ok to perform lazy cache-write
  INTERNET_FLAG_PRAGMA_NOCACHE			= $00000100;  // asking wininet to add "pragma: no-cache"
  INTERNET_FLAG_NO_UI				= $00000200;  // no cookie popup
  INTERNET_FLAG_HYPERLINK			= $00000400;  // asking wininet to do hyperlinking semantic which works right for scripts
  INTERNET_FLAG_RESYNCHRONIZE			= $00000800;  // asking wininet to update an item if it is newer
  INTERNET_FLAG_CACHE_IF_NET_FAIL		= $00010000;  // return cache file if net request fails
  INTERNET_FLAG_NO_AUTH				= $00040000;  // no automatic authentication handling
  INTERNET_FLAG_NO_COOKIES			= $00080000;  // no automatic cookie handling
  INTERNET_FLAG_READ_PREFETCH			= $00100000;  // do background read prefetch
  INTERNET_FLAG_NO_AUTO_REDIRECT		= $00200000;  // don't handle redirections automatically
  INTERNET_FLAG_KEEP_CONNECTION			= $00400000;  // use keep-alive semantics
  INTERNET_FLAG_SECURE				= $00800000;  // use PCT/SSL if applicable (HTTP)
  INTERNET_FLAG_FROM_CACHE			= $01000000;  // use offline semantics
  INTERNET_FLAG_OFFLINE				= INTERNET_FLAG_FROM_CACHE;
  INTERNET_FLAG_MAKE_PERSISTENT			= $02000000;  // make this item persistent in cache
  INTERNET_FLAG_NO_CACHE_WRITE			= $04000000;  // don't write this item to the cache
  INTERNET_FLAG_DONT_CACHE			= INTERNET_FLAG_NO_CACHE_WRITE;
  INTERNET_FLAG_PASSIVE				= $08000000;  // used for FTP connections
  INTERNET_FLAG_ASYNC				= $10000000;  // this request is asynchronous (where supported)

//
// flags for InternetOpenUrl()
//
  INTERNET_FLAG_EXISTING_CONNECT		= $20000000;  // do not create new connection object
  INTERNET_FLAG_RAW_DATA			= $40000000;  // receive the item as raw data
  INTERNET_FLAG_RELOAD				= $80000000;  // retrieve the original item

//
// HttpQueryInfo info levels. Generally, there is one info level
// for each potential RFC822/HTTP/MIME header that an HTTP server
// may send as part of a request response.
//
// The HTTP_QUERY_RAW_HEADERS info level is provided for clients
// that choose to perform their own header parsing.
//
  HTTP_QUERY_MIME_VERSION			=  0;
  HTTP_QUERY_CONTENT_TYPE			=  1;
  HTTP_QUERY_CONTENT_TRANSFER_ENCODING		=  2;
  HTTP_QUERY_CONTENT_ID				=  3;
  HTTP_QUERY_CONTENT_DESCRIPTION		=  4;
  HTTP_QUERY_CONTENT_LENGTH			=  5;
  HTTP_QUERY_CONTENT_LANGUAGE			=  6;
  HTTP_QUERY_ALLOW				=  7;
  HTTP_QUERY_PUBLIC				=  8;
  HTTP_QUERY_DATE				=  9;
  HTTP_QUERY_EXPIRES				= 10;
  HTTP_QUERY_LAST_MODIFIED			= 11;
  HTTP_QUERY_MESSAGE_ID				= 12;
  HTTP_QUERY_URI				= 13;
  HTTP_QUERY_DERIVED_FROM			= 14;
  HTTP_QUERY_COST				= 15;
  HTTP_QUERY_LINK				= 16;
  HTTP_QUERY_PRAGMA				= 17;
  HTTP_QUERY_VERSION				= 18;  // special: part of status line
  HTTP_QUERY_STATUS_CODE			= 19;  // special: part of status line
  HTTP_QUERY_STATUS_TEXT			= 20;  // special: part of status line
  HTTP_QUERY_RAW_HEADERS			= 21;  // special: all headers as ASCIIZ
  HTTP_QUERY_RAW_HEADERS_CRLF			= 22;  // special: all headers
  HTTP_QUERY_CONNECTION				= 23;

//
// HTTP Response Status Codes
//
  HTTP_STATUS_CONTINUE				= 100;  // OK to continue with request
  HTTP_STATUS_SWITCH_PROTOCOLS			= 101;  // server has switched protocols in upgrade header
  HTTP_STATUS_OK				= 200;  // request completed
  HTTP_STATUS_CREATED				= 201;  // object created, reason =  new URI
  HTTP_STATUS_ACCEPTED				= 202;  // async completion (TBS)
  HTTP_STATUS_PARTIAL				= 203;  // partial completion
  HTTP_STATUS_NO_CONTENT			= 204;  // no info to return
  HTTP_STATUS_RESET_CONTENT			= 205;  // request completed, but clear form
  HTTP_STATUS_PARTIAL_CONTENT			= 206;  // partial GET furfilled
  HTTP_STATUS_AMBIGUOUS				= 300;  // server couldn't decide what to return
  HTTP_STATUS_MOVED				= 301;  // object permanently moved
  HTTP_STATUS_REDIRECT				= 302;  // object temporarily moved
  HTTP_STATUS_REDIRECT_METHOD			= 303;  // redirection w/ new access method
  HTTP_STATUS_NOT_MODIFIED			= 304;  // if-modified-since was not modified
  HTTP_STATUS_USE_PROXY				= 305;  // redirection to ProxyServer, location header specifies ProxyServer to use
  HTTP_STATUS_REDIRECT_KEEP_VERB		= 307;  // HTTP/1.1: keep same verb
  HTTP_STATUS_BAD_REQUEST			= 400;  // invalid syntax
  HTTP_STATUS_DENIED				= 401;  // access denied
  HTTP_STATUS_PAYMENT_REQ			= 402;  // payment required
  HTTP_STATUS_FORBIDDEN				= 403;  // request forbidden
  HTTP_STATUS_NOT_FOUND				= 404;  // object not found
  HTTP_STATUS_BAD_METHOD			= 405;  // method is not allowed
  HTTP_STATUS_NONE_ACCEPTABLE			= 406;  // no response acceptable to client found
  HTTP_STATUS_PROXY_AUTH_REQ			= 407;  // ProxyServer authentication required
  HTTP_STATUS_REQUEST_TIMEOUT			= 408;  // server timed out waiting for request
  HTTP_STATUS_CONFLICT				= 409;  // user should resubmit with more info
  HTTP_STATUS_GONE				= 410;  // the resource is no longer available
  HTTP_STATUS_AUTH_REFUSED			= 411;  // couldn't authorize client
  HTTP_STATUS_PRECOND_FAILED			= 412;  // precondition given in request failed
  HTTP_STATUS_REQUEST_TOO_LARGE			= 413;  // request entity was too large
  HTTP_STATUS_URI_TOO_LONG			= 414;  // request URI too long
  HTTP_STATUS_UNSUPPORTED_MEDIA			= 415;  // unsupported media type
  HTTP_STATUS_SERVER_ERROR			= 500;  // internal server error
  HTTP_STATUS_NOT_SUPPORTED			= 501;  // required not supported
  HTTP_STATUS_BAD_GATEWAY			= 502;  // error response received from gateway
  HTTP_STATUS_SERVICE_UNAVAIL			= 503;  // temporarily overloaded
  HTTP_STATUS_GATEWAY_TIMEOUT			= 504;  // timed out waiting for gateway
  HTTP_STATUS_VERSION_NOT_SUP			= 505;  // HTTP version not supported

//
// HTTP_QUERY_FLAG_COALESCE - combine the values from several headers of the
// same name into the output buffer
//
  HTTP_QUERY_FLAG_COALESCE			= $10000000;

//
// HTTP_QUERY_FLAG_NUMBER - if this bit is set in the dwInfoLevel parameter of
// HttpQueryInfo(), then the value of the header will be converted to a number
// before being returned to the caller, if applicable
//
  HTTP_QUERY_FLAG_NUMBER			= $20000000;

//
// HTTP_QUERY_FLAG_SYSTEMTIME - if this bit is set in the dwInfoLevel parameter
// of HttpQueryInfo() AND the header being queried contains date information,
// e.g. the "Expires:" header then lpBuffer will contain a SYSTEMTIME structure
// containing the date and time information converted from the header string
//
  HTTP_QUERY_FLAG_SYSTEMTIME			= $40000000;

//
// HTTP_QUERY_FLAG_REQUEST_HEADERS - if this bit is set in the dwInfoLevel
// parameter of HttpQueryInfo() then the request headers will be queried for the
// request information
//
  HTTP_QUERY_FLAG_REQUEST_HEADERS		= $80000000;

//
// flags for InternetCanonicalizeUrl() and InternetCombineUrl()
//
  ICU_ENCODE_PERCENT				= $00001000;  // Encode any percent (ASCII25) signs encountered
  ICU_BROWSER_MODE				= $02000000;  // Special encode/decode rules for browser
  ICU_ENCODE_SPACES_ONLY			= $04000000;  // Encode spaces only
  ICU_NO_META					= $08000000;  // Don't convert .. etc. meta path sequences
  ICU_DECODE					= $10000000;  // Convert %XX escape sequences to characters
  ICU_NO_ENCODE					= $20000000;  // Don't convert unsafe characters to escape sequence

//
// flags for InternetCrackUrl() and InternetCreateUrl()
//
  ICU_USERNAME					= $40000000;  // use internal username & password
  ICU_ESCAPE					= $80000000;  // (un)escape URL characters

//
// Internet API error returns
//
  INTERNET_ERROR_BASE					=  12000;

  ERROR_INTERNET_OUT_OF_HANDLES				=  (INTERNET_ERROR_BASE + 1);
  ERROR_INTERNET_TIMEOUT				=  (INTERNET_ERROR_BASE + 2);
  ERROR_INTERNET_EXTENDED_ERROR				=  (INTERNET_ERROR_BASE + 3);
  ERROR_INTERNET_INTERNAL_ERROR				=  (INTERNET_ERROR_BASE + 4);
  ERROR_INTERNET_INVALID_URL				=  (INTERNET_ERROR_BASE + 5);
  ERROR_INTERNET_UNRECOGNIZED_SCHEME			=  (INTERNET_ERROR_BASE + 6);
  ERROR_INTERNET_NAME_NOT_RESOLVED			=  (INTERNET_ERROR_BASE + 7);
  ERROR_INTERNET_PROTOCOL_NOT_FOUND			=  (INTERNET_ERROR_BASE + 8);
  ERROR_INTERNET_INVALID_OPTION				=  (INTERNET_ERROR_BASE + 9);
  ERROR_INTERNET_BAD_OPTION_LENGTH			=  (INTERNET_ERROR_BASE + 10);
  ERROR_INTERNET_OPTION_NOT_SETTABLE			=  (INTERNET_ERROR_BASE + 11);
  ERROR_INTERNET_SHUTDOWN				=  (INTERNET_ERROR_BASE + 12);
  ERROR_INTERNET_INCORRECT_USER_NAME			=  (INTERNET_ERROR_BASE + 13);
  ERROR_INTERNET_INCORRECT_PASSWORD			=  (INTERNET_ERROR_BASE + 14);
  ERROR_INTERNET_LOGIN_FAILURE				=  (INTERNET_ERROR_BASE + 15);
  ERROR_INTERNET_INVALID_OPERATION			=  (INTERNET_ERROR_BASE + 16);
  ERROR_INTERNET_OPERATION_CANCELLED			=  (INTERNET_ERROR_BASE + 17);
  ERROR_INTERNET_INCORRECT_HANDLE_TYPE			=  (INTERNET_ERROR_BASE + 18);
  ERROR_INTERNET_INCORRECT_HANDLE_STATE			=  (INTERNET_ERROR_BASE + 19);
  ERROR_INTERNET_NOT_PROXY_REQUEST			=  (INTERNET_ERROR_BASE + 20);
  ERROR_INTERNET_REGISTRY_VALUE_NOT_FOUND		=  (INTERNET_ERROR_BASE + 21);
  ERROR_INTERNET_BAD_REGISTRY_PARAMETER			=  (INTERNET_ERROR_BASE + 22);
  ERROR_INTERNET_NO_DIRECT_ACCESS			=  (INTERNET_ERROR_BASE + 23);
  ERROR_INTERNET_NO_CONTEXT				=  (INTERNET_ERROR_BASE + 24);
  ERROR_INTERNET_NO_CALLBACK				=  (INTERNET_ERROR_BASE + 25);
  ERROR_INTERNET_REQUEST_PENDING			=  (INTERNET_ERROR_BASE + 26);
  ERROR_INTERNET_INCORRECT_FORMAT			=  (INTERNET_ERROR_BASE + 27);
  ERROR_INTERNET_ITEM_NOT_FOUND				=  (INTERNET_ERROR_BASE + 28);
  ERROR_INTERNET_CANNOT_CONNECT				=  (INTERNET_ERROR_BASE + 29);
  ERROR_INTERNET_CONNECTION_ABORTED			=  (INTERNET_ERROR_BASE + 30);
  ERROR_INTERNET_CONNECTION_RESET			=  (INTERNET_ERROR_BASE + 31);
  ERROR_INTERNET_FORCE_RETRY				=  (INTERNET_ERROR_BASE + 32);
  ERROR_INTERNET_INVALID_PROXY_REQUEST			=  (INTERNET_ERROR_BASE + 33);
  ERROR_INTERNET_NEED_UI				=  (INTERNET_ERROR_BASE + 34);

  ERROR_INTERNET_HANDLE_EXISTS				=  (INTERNET_ERROR_BASE + 36);
  ERROR_INTERNET_SEC_CERT_DATE_INVALID			=  (INTERNET_ERROR_BASE + 37);
  ERROR_INTERNET_SEC_CERT_CN_INVALID			=  (INTERNET_ERROR_BASE + 38);
  ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR			=  (INTERNET_ERROR_BASE + 39);
  ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR			=  (INTERNET_ERROR_BASE + 40);
  ERROR_INTERNET_MIXED_SECURITY				=  (INTERNET_ERROR_BASE + 41);
  ERROR_INTERNET_CHG_POST_IS_NON_SECURE			=  (INTERNET_ERROR_BASE + 42);
  ERROR_INTERNET_POST_IS_NON_SECURE			=  (INTERNET_ERROR_BASE + 43);
  ERROR_INTERNET_CLIENT_AUTH_CERT_NEEDED		=  (INTERNET_ERROR_BASE + 44);
  ERROR_INTERNET_INVALID_CA				=  (INTERNET_ERROR_BASE + 45);
  ERROR_INTERNET_CLIENT_AUTH_NOT_SETUP			=  (INTERNET_ERROR_BASE + 46);
  ERROR_INTERNET_ASYNC_THREAD_FAILED			=  (INTERNET_ERROR_BASE + 47);
  ERROR_INTERNET_REDIRECT_SCHEME_CHANGE			=  (INTERNET_ERROR_BASE + 48);
  ERROR_INTERNET_DIALOG_PENDING				=  (INTERNET_ERROR_BASE + 49);
  ERROR_INTERNET_RETRY_DIALOG				=  (INTERNET_ERROR_BASE + 50);
  ERROR_INTERNET_HTTPS_HTTP_SUBMIT_REDIR		=  (INTERNET_ERROR_BASE + 52);
  ERROR_INTERNET_INSERT_CDROM				=  (INTERNET_ERROR_BASE + 53);
  ERROR_INTERNET_FORTEZZA_LOGIN_NEEDED			=  (INTERNET_ERROR_BASE + 54);
  ERROR_INTERNET_SEC_CERT_ERRORS			=  (INTERNET_ERROR_BASE + 55);
  ERROR_INTERNET_SEC_CERT_NO_REV			=  (INTERNET_ERROR_BASE + 56);
  ERROR_INTERNET_SEC_CERT_REV_FAILED			=  (INTERNET_ERROR_BASE + 57);

  //
  // FTP API errors
  //
  ERROR_FTP_TRANSFER_IN_PROGRESS			=  (INTERNET_ERROR_BASE + 110);
  ERROR_FTP_DROPPED					=  (INTERNET_ERROR_BASE + 111);
  ERROR_FTP_NO_PASSIVE_MODE				=  (INTERNET_ERROR_BASE + 112);

  //
  // gopher API errors
  //
  ERROR_GOPHER_PROTOCOL_ERROR				=  (INTERNET_ERROR_BASE + 130);
  ERROR_GOPHER_NOT_FILE					=  (INTERNET_ERROR_BASE + 131);
  ERROR_GOPHER_DATA_ERROR				=  (INTERNET_ERROR_BASE + 132);
  ERROR_GOPHER_END_OF_DATA				=  (INTERNET_ERROR_BASE + 133);
  ERROR_GOPHER_INVALID_LOCATOR				=  (INTERNET_ERROR_BASE + 134);
  ERROR_GOPHER_INCORRECT_LOCATOR_TYPE			=  (INTERNET_ERROR_BASE + 135);
  ERROR_GOPHER_NOT_GOPHER_PLUS				=  (INTERNET_ERROR_BASE + 136);
  ERROR_GOPHER_ATTRIBUTE_NOT_FOUND			=  (INTERNET_ERROR_BASE + 137);
  ERROR_GOPHER_UNKNOWN_LOCATOR				=  (INTERNET_ERROR_BASE + 138);

  //
  // HTTP API errors
  //
  ERROR_HTTP_HEADER_NOT_FOUND				=  (INTERNET_ERROR_BASE + 150);
  ERROR_HTTP_DOWNLEVEL_SERVER				=  (INTERNET_ERROR_BASE + 151);
  ERROR_HTTP_INVALID_SERVER_RESPONSE			=  (INTERNET_ERROR_BASE + 152);
  ERROR_HTTP_INVALID_HEADER				=  (INTERNET_ERROR_BASE + 153);
  ERROR_HTTP_INVALID_QUERY_REQUEST			=  (INTERNET_ERROR_BASE + 154);
  ERROR_HTTP_HEADER_ALREADY_EXISTS			=  (INTERNET_ERROR_BASE + 155);
  ERROR_HTTP_REDIRECT_FAILED				=  (INTERNET_ERROR_BASE + 156);
  ERROR_HTTP_NOT_REDIRECTED				=  (INTERNET_ERROR_BASE + 160);
  ERROR_HTTP_COOKIE_NEEDS_CONFIRMATION			=  (INTERNET_ERROR_BASE + 161);
  ERROR_HTTP_COOKIE_DECLINED				=  (INTERNET_ERROR_BASE + 162);
  ERROR_HTTP_REDIRECT_NEEDS_CONFIRMATION		=  (INTERNET_ERROR_BASE + 168);

  //
  // additional Internet API error codes
  //
  ERROR_INTERNET_SECURITY_CHANNEL_ERROR			=  (INTERNET_ERROR_BASE + 157);
  ERROR_INTERNET_UNABLE_TO_CACHE_FILE			=  (INTERNET_ERROR_BASE + 158);
  ERROR_INTERNET_TCPIP_NOT_INSTALLED			=  (INTERNET_ERROR_BASE + 159);
  ERROR_INTERNET_DISCONNECTED				=  (INTERNET_ERROR_BASE + 163);
  ERROR_INTERNET_SERVER_UNREACHABLE			=  (INTERNET_ERROR_BASE + 164);
  ERROR_INTERNET_PROXY_SERVER_UNREACHABLE		=  (INTERNET_ERROR_BASE + 165);

  ERROR_INTERNET_BAD_AUTO_PROXY_SCRIPT			=  (INTERNET_ERROR_BASE + 166);
  ERROR_INTERNET_UNABLE_TO_DOWNLOAD_SCRIPT		=  (INTERNET_ERROR_BASE + 167);
  ERROR_INTERNET_SEC_INVALID_CERT			=  (INTERNET_ERROR_BASE + 169);
  ERROR_INTERNET_SEC_CERT_REVOKED			=  (INTERNET_ERROR_BASE + 170);

  //
  // InternetAutodial specific errors
  //
  ERROR_INTERNET_FAILED_DUETOSECURITYCHECK		=  (INTERNET_ERROR_BASE + 171);
  ERROR_INTERNET_NOT_INITIALIZED			=  (INTERNET_ERROR_BASE + 172);
  ERROR_INTERNET_NEED_MSN_SSPI_PKG			=  (INTERNET_ERROR_BASE + 173);
  ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY	=  (INTERNET_ERROR_BASE + 174);

  INTERNET_ERROR_LAST					=  ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY;

//
// InternetErrorDlg - provides UI for certain errors
//
  FLAGS_ERROR_UI_FILTER_FOR_ERRORS			= $01;
  FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS			= $02;
  FLAGS_ERROR_UI_FLAGS_GENERATE_DATA			= $04;
  FLAGS_ERROR_UI_FLAGS_NO_UI				= $08;
  FLAGS_ERROR_UI_SERIALIZE_DIALOGS			= $10;

type
  HInternet = Pointer;
  TInternetPort = Word;

//
// HTTP_VERSION_INFO - query or set global HTTP version (1.0 or 1.1)
//
  THttpVersionInfo = Record
    dwMajorVersion: DWord;
    dwMinorVersion: DWord;
  end;

//
// INTERNET_CONNECTED_INFO - information used to set the global connected state
//
  TInternetConnectedInfo = Record
    dwConnectedState: DWord;	// new connected/disconnected state
    dwFlags: DWord;		// flags controlling transition
  end;

//
// INTERNET_PROXY_INFO - structure supplied with INTERNET_OPTION_PROXY
//
  TInternetProxyInfo = Record
    dwAccessType: DWord;	// INTERNET_OPEN_TYPE_DIRECT, INTERNET_OPEN_TYPE_PROXY, INTERNET_OPEN_TYPE_PRECONFIG
    lpszProxy: PChar;		// proxy server list
    lpszProxyBypass: PChar;	// proxy bypass list
  end;

//
// URL_COMPONENTS - the constituent parts of an URL
//
  PUrlComponents = ^TUrlComponents;
  TUrlComponents = Record
    dwStructSize: DWord;	// size of this structure
    lpszScheme: PChar;		// pointer to scheme name
    dwSchemeLength: DWord;	// length of scheme name
    nScheme: Integer;		// enumerated scheme type (if known)
    lpszHostName: PChar;	// pointer to host name
    dwHostNameLength: DWord;	// length of host name
    nPort: Integer;		// converted port number
    lpszUserName: PChar;	// pointer to user name
    dwUserNameLength: DWord;	// length of user name
    lpszPassword: PChar;	// pointer to password
    dwPasswordLength: DWord;	// length of password
    lpszUrlPath: PChar;		// pointer to URL-path
    dwUrlPathLength: DWord;	// length of URL-path
    lpszExtraInfo: PChar;	// pointer to extra information (e.g. ?foo or #foo)
    dwExtraInfoLength: DWord;	// length of extra information
  end;

//
// INTERNET_ASYNC_RESULT - this structure is returned to the application via
// the callback with INTERNET_STATUS_REQUEST_COMPLETE. It is not sufficient to
// just return the result of the async operation. If the API failed then the
// app cannot call GetLastError() because the thread context will be incorrect.
// Both the value returned by the async API and any resultant error code are
// made available. The app need not check dwError if dwResult indicates that
// the API succeeded (in this case dwError will be ERROR_SUCCESS)
//

  PInternetAsyncResult = ^TInternetAsyncResult;
  TInternetAsyncResult = Record
    dwResult: DWord;		// HINTERNET, DWORD or BOOL return code from an async API
    dwError: DWord;		// the error code if the API failed
  end;

  PFNInternetStatusCallback = ^TFNInternetStatusCallback;
  TFNInternetStatusCallback = procedure(hInet: HInternet; dwContext: DWord; dwInternetStatus: DWord;
					lpvStatusInformation: Pointer; dwStatusInformationLength: DWord); stdcall;

var
  InternetGetConnectedState: function(var dwFlags: DWord; dwReserved: DWord): Bool; stdcall;
  InternetQueryOption: function(hInet: HInternet; dwOption: DWord; lpBuffer: Pointer;
				var dwBufferLength: DWord): Bool; stdcall;
  InternetSetOption: function(hInet: HInternet; dwOption: DWord; lpBuffer: Pointer;
			      dwBufferLength: DWord): Bool; stdcall;
  InternetAutodial: function(dwFlags: DWord; hwndParent: HWnd): Bool; stdcall;
  InternetOpen: function(lpszCallerName: PChar; dwAccessType: DWord; lpszServerName: PChar;
			 nServerPort: TInternetPort; dwFlags: DWord): HInternet; stdcall;
  InternetOpenUrl: function(hInet: HInternet; lpszUrl, lpszHeaders: PChar;
			    dwHeadersLength, dwFlags, dwContext: DWord): HInternet; stdcall;
  InternetSetStatusCallback: function(hInet: HInternet; lpfnInternetCallback: PFNInternetStatusCallback)
				     : PFNInternetStatusCallback; stdcall;
  InternetQueryDataAvailable: function(hFile: HInternet; lpdwNumberOfBytesAvailable: PDWord;
				       dwFlags, dwContext: DWord): Bool; stdcall;
  InternetReadFile: function(hFile: HInternet; lpBuffer: Pointer; dwNumberOfBytesToRead: DWord;
			     var lpdwNumberOfBytesRead: DWord): Bool; stdcall;
  InternetCloseHandle: function(hInet: HInternet): Bool; stdcall;
  InternetAutodialHangup: function(dwReserved: DWord): Bool; stdcall;
  HttpQueryInfo: function(hHttpRequest: HInternet; dwInfoLevel: DWord;
			  lpvBuffer: Pointer; var lpdwBufferLength: DWord;
			  lpdwIndex: PDWord): Bool; stdcall;
  HttpSendRequest: function(hHttpRequest: HInternet; lpszHeaders: PChar;
			    dwHeadersLength: DWord; lpOptional: Pointer;
			    dwOptionalLength: DWord): Bool; stdcall;
  InternetCanonicalizeUrl: function(lpszUrl: PChar; lpszBuffer: PChar; var lpdwBufferLength: DWord;
				    dwFlags: DWord): Bool; stdcall;
  InternetCrackUrl: function(lpszUrl: PChar; dwUrlLength: DWord; dwFlags: DWord;
			     lpUrlComponents: PUrlComponents): Bool; stdcall;
  DeleteUrlCacheEntry: function(lpszUrlName: PChar): Bool; stdcall;
  InternetErrorDlg: function(hWnd: HWND; hRequest: HInternet; dwError, dwFlags: DWord;
			     lppvData: Pointer): DWord; stdcall;

const
  HTTP10: THttpVersionInfo = (
    dwMajorVersion: 1;
    dwMinorVersion: 0;
  );

type
  TWinInetError = (wieNone, wieWinInet, wieNoUrl, wieConnect, wieLoadPage, wieReadPage, wieWriteFile, wieCancelled);
  TInetHandles = Record
    AccessType: DWord;
    AccessFlags: DWord;
    Connection: HInternet;
    Page: HInternet;
  end;
  TWinInetThread = Class(TThread)
  private
    FConnectTimeout: DWord;
    FInetHandles: TInetHandles;
    FWebData: String;
    FWebDomain: String;
    FWebTargets: String;
    FWebUserAgent: String;
    FWinInetError: TWinInetError;
    FOnConnect: TNotifyEvent;
    procedure CallOnConnect;
    procedure CallOnTerminate;
    procedure ReadWebData;
  protected
    procedure DoConnect;
    procedure Execute; override;
    procedure DoTerminate; override;
  public
    constructor Create(const WebUserAgent: String);
    destructor Destroy; override;
    procedure Cancel;
    property  ConnectTimeout: DWord read FConnectTimeout;
    property  WebData: String read FWebData;
    property  WebDomain: String read FWebDomain;
    property  WebTargets: String write FWebTargets;
    property  WinInetError: TWinInetError read FWinInetError;
    property  OnConnect: TNotifyEvent read FOnConnect write FOnConnect;
  end;

function LoadWinInet(Reload: Longbool): Longbool;
function WhoisData(const Target, Service: String): String;
function GetAbuseEmailAddress(const Data: String): String;
function NTPTime(const Server: String; Port: Integer): TDateTime;


implementation

const
  E = '';
  AT = '@';
  SP = #32;
  CR = #13;
  LF = #10;
  TAB = #9;
  NUL = #0;
  DOT = '.';
  CRLF = CR + LF;


// WinInet utilities -----------------------------------------------------------

function TabExtract(var Text: String): String;
var
  iCharPos: Integer;
begin
  Result := E;
  iCharPos := pos(TAB, Text);
  if iCharPos > 0 then begin
    Result := Copy(Text, 1, pred(iCharPos));
    Delete(Text, 1, iCharPos);
  end
  else begin
    Result := Text;
    Text := E;
  end;
end;

function CheckConnectedState: DWord;
var
  dwSize: DWord;
  iciState: TInternetConnectedInfo;
begin
  Result := 0;
  dwSize := SizeOf(Result);
  if InternetQueryOption(nil, INTERNET_OPTION_CONNECTED_STATE, @Result, dwSize)
  and ((Result and INTERNET_STATE_DISCONNECTED_BY_USER) <> 0) then begin
    with iciState do begin
      dwConnectedState := INTERNET_STATE_CONNECTED;
      dwFlags := 0;
    end;
    InternetSetOption(nil, INTERNET_OPTION_CONNECTED_STATE, @iciState, SizeOf(iciState));
  end;
end;

procedure ResetConnectedState(PreviousState: DWord);
var
  iciState: TInternetConnectedInfo;
begin
  if (PreviousState and INTERNET_STATE_DISCONNECTED_BY_USER) <> 0 then begin
    with iciState do begin
      dwConnectedState := INTERNET_STATE_DISCONNECTED_BY_USER;
      dwFlags := ISO_FORCE_DISCONNECTED;
    end;
    InternetSetOption(nil, INTERNET_OPTION_CONNECTED_STATE, @iciState, SizeOf(iciState));
  end;
end;

function GetConnectTimeout(Connection: HInternet): DWord; // seconds
var
  dwSize: DWord;
begin
  Result := 0;
  dwSize := SizeOf(Result);
  InternetQueryOption(Connection, INTERNET_OPTION_CONNECT_TIMEOUT, @Result, dwSize);
  Result := Result div 1000;
end;

function GetUsesProxy(Connection: HInternet): Longbool;
var
  dwSize: DWord;
  ipiConnection: TInternetProxyInfo;
begin
  Result := false;
  dwSize := SizeOf(TInternetProxyInfo);
  if InternetQueryOption(Connection, INTERNET_OPTION_PROXY, @ipiConnection, dwSize) then with ipiConnection do begin
    Result := (dwAccessType = INTERNET_OPEN_TYPE_PROXY) and (lpszProxy <> nil) and (lpszProxy[0] <> NUL);
  end;
end;

function GetStatusCode(WebPage: HInternet): DWord;
var
  dwSize: DWord;
begin
  Result := 0;
  dwSize := SizeOf(Result);
  InternetQueryDataAvailable(WebPage, nil, 0, 0);
  HttpQueryInfo(WebPage, HTTP_QUERY_STATUS_CODE or HTTP_QUERY_FLAG_NUMBER, @Result, dwSize, nil);
end;

function GetContentLength(WebPage: HInternet; MaxSize: DWord): DWord;
var
  dwSize: DWord;
begin
  Result := MaxSize;
  dwSize := SizeOf(Result);
  InternetQueryDataAvailable(WebPage, nil, 0, 0);
  HttpQueryInfo(WebPage, HTTP_QUERY_CONTENT_LENGTH or HTTP_QUERY_FLAG_NUMBER, @Result, dwSize, nil);
end;

function CanonicalizeUrl(const Url: String): String;
var
  dwLen: DWord;
begin
  dwLen := Length(NUL);
  if Url <> E then begin
    InternetCanonicalizeUrl(Pointer(Url), NUL, dwLen, ICU_DECODE or ICU_NO_ENCODE);
    if GetLastError = ERROR_INSUFFICIENT_BUFFER then begin
      SetString(Result, nil, dwLen);
      if not InternetCanonicalizeUrl(Pointer(Url), Pointer(Result), dwLen, ICU_DECODE or ICU_NO_ENCODE) then begin
	Result := Url;
      end;
    end
    else Result := Url;
  end
  else Result := E;
end;

function GetWebDomain(const TargetUrl: String): String;
var
  ucTarget: TUrlComponents;
begin
  Result := CanonicalizeUrl(TargetUrl);
  if Result <> E then begin
    FillChar(ucTarget, SizeOf(TUrlComponents), 0);
    with ucTarget do begin
      dwStructSize := SizeOf(TUrlComponents);
      dwHostNameLength := 1;
    end;
    if InternetCrackUrl(Pointer(Result), 0, 0, @ucTarget) then begin
      with ucTarget do SetString(Result, lpszHostName, dwHostNameLength);
    end;
  end;
end;

function Access(WIT: TWinInetThread; const Target: String): TWinInetError;
begin
  Result := wieNone;
  with WIT.FInetHandles do begin
    Connection := InternetOpen(Pointer(WIT.FWebUserAgent), AccessType, nil, 0, 0);
    if Connection <> nil then begin
      WIT.FConnectTimeout := GetConnectTimeout(Connection);
      WIT.FWebDomain := GetWebDomain(Target);
      WIT.DoConnect;
      Page := InternetOpenUrl(Connection, Pointer(Target), nil, 0, AccessFlags, 0);
      if Page = nil then begin
	if GetLastError = ERROR_INTERNET_OPERATION_CANCELLED then
	  Result := wieCancelled
	else Result := wieLoadPage;
	InternetCloseHandle(Connection);
	Connection := nil;
      end;
    end
    else Result := wieConnect;
  end;
end;

function Connect(WIT: TWinInetThread; const Target: String): TWinInetError;
begin
  with WIT.FInetHandles do begin
    AccessFlags := INTERNET_FLAG_RELOAD or INTERNET_FLAG_DONT_CACHE;
    Connection := nil;
    Page := nil;
    if GetUsesProxy(nil) then
      AccessType := PRE_CONFIG_INTERNET_ACCESS
    else AccessType := LOCAL_INTERNET_ACCESS;
    Result := Access(WIT, Target);
    if (Result in [wieConnect, wieLoadPage]) and (AccessType = PRE_CONFIG_INTERNET_ACCESS) then begin
      AccessType := LOCAL_INTERNET_ACCESS;
      Result := Access(WIT, Target);
    end;
  end;
end;

procedure CloseConnection(var InetHandles: TInetHandles);
begin
  with InetHandles do begin
    InternetCloseHandle(Page);
    InternetCloseHandle(Connection);
    Page := nil;
    Connection := nil;
  end;
end;

function ReadData(WebPage: HInternet; var Data: String): TWinInetError;
var
  dwLen, dwSize: DWord;
  pData: PChar;
begin
  Result := wieReadPage;
  Data := E;
  if GetStatusCode(WebPage) = HTTP_STATUS_OK then begin
    dwLen := GetContentLength(WebPage, $10000);
    SetString(Data, nil, dwLen);
    pData := Pointer(Data);
    dwSize := dwLen;
    while (dwLen > 0) and InternetQueryDataAvailable(WebPage, @dwSize, 0, 0) and (dwSize > 0)
    and InternetReadFile(WebPage, pData, dwSize, dwSize) and (dwSize > 0) do begin
      inc(pData, dwSize);
      dec(dwLen, dwSize);
    end;
    Delete(Data, succ(pData - Pointer(Data)), MAXINT);
    if Data <> E then Result := wieNone;
  end;
end;

// TWinInetThread --------------------------------------------------------------

constructor TWinInetThread.Create(const WebUserAgent: String);
begin
  inherited Create(true);
  FreeOnTerminate := true;
  FWebUserAgent := WebUserAgent;
end;

destructor TWinInetThread.Destroy;
begin
  if Suspended then begin
    Terminate;
    Resume;
  end;
  inherited Destroy;
end;

procedure TWinInetThread.Cancel;
begin
  CloseConnection(FInetHandles);
end;

procedure TWinInetThread.DoConnect;
begin
  Synchronize(CallOnConnect);
end;

procedure TWinInetThread.Execute;
begin
  if not Terminated then ReadWebData;
end;

procedure TWinInetThread.DoTerminate;
begin
  Synchronize(CallOnTerminate);
end;

procedure TWinInetThread.CallOnConnect;
begin
  if Assigned(FOnConnect) then FOnConnect(Self);
end;

procedure TWinInetThread.CallOnTerminate;
begin
  if Assigned(OnTerminate) then OnTerminate(Self);
end;

procedure TWinInetThread.ReadWebData;
var
  dwConnectedState: DWord;
begin
  FWinInetError := wieWinInet;
  if GetModuleHandle(WININET) <> 0 then begin
    dwConnectedState := CheckConnectedState;
    try
      if FWebTargets <> E then begin
	repeat
	  FWinInetError := Connect(Self, TabExtract(FWebTargets));
	  if FWinInetError = wieNone then begin
	    try
	      FWinInetError := ReadData(FInetHandles.Page, FWebData);
	    finally
	      CloseConnection(FInetHandles);
	    end;
	  end;
	until (FWinInetError in [wieNone, wieCancelled]) or (FWebTargets = E);
      end;
    finally
      ResetConnectedState(dwConnectedState);
    end;
  end;
end;

function LoadWinInet(Reload: Longbool): Longbool;
var
  hWinInet: HModule;
begin
  Result := false;
  if Reload then begin
    // enforce unloading WININET
    while FreeLibrary(GetModuleHandle(WININET)) do;
    // get a valid handle by re-loading WININET
    hWinInet := LoadLibrary(WININET);
  end
  else begin
    hWinInet := GetModuleHandle(WININET);
    if hWinInet = 0 then hWinInet := LoadLibrary(WININET);
  end;
  if hWinInet <> 0 then begin
    // dynamically load WININET functions
    InternetGetConnectedState := GetProcAddress(hWinInet, 'InternetGetConnectedState');
    InternetQueryOption := GetProcAddress(hWinInet, 'InternetQueryOptionA');
    InternetSetOption := GetProcAddress(hWinInet, 'InternetSetOptionA');
    InternetAutodial := GetProcAddress(hWinInet, 'InternetAutodial');
    InternetOpen := GetProcAddress(hWinInet, 'InternetOpenA');
    InternetOpenUrl := GetProcAddress(hWinInet, 'InternetOpenUrlA');
    InternetSetStatusCallback := GetProcAddress(hWinInet, 'InternetSetStatusCallback');
    InternetQueryDataAvailable := GetProcAddress(hWinInet, 'InternetQueryDataAvailable');
    InternetReadFile := GetProcAddress(hWinInet, 'InternetReadFile');
    InternetCloseHandle := GetProcAddress(hWinInet, 'InternetCloseHandle');
    InternetAutodialHangup := GetProcAddress(hWinInet, 'InternetAutodialHangup');
    HttpQueryInfo := GetProcAddress(hWinInet, 'HttpQueryInfoA');
    HttpSendRequest := GetProcAddress(hWinInet, 'HttpSendRequestA');
    InternetCanonicalizeUrl := GetProcAddress(hWinInet, 'InternetCanonicalizeUrlA');
    InternetCrackUrl := GetProcAddress(hWinInet, 'InternetCrackUrlA');
    DeleteUrlCacheEntry := GetProcAddress(hWinInet, 'DeleteUrlCacheEntryA');
    Result := true;
  end;
end;


// Other tools -----------------------------------------------------------------

function GetInAddrOfDomain(const Domain: String): TInAddr;
var
  phe: PHostEnt;
begin
  Result := TInAddr(inet_addr(PChar(Domain)));
  if Longint(Result) = INADDR_NONE then begin
    phe := GetHostByName(PChar(Domain));
    if phe <> nil then begin
      if phe.h_length = SizeOf(TInAddr) then
	Result := PInAddr(phe.h_addr_list^)^
      else WSASetLastError(WSANO_DATA);
    end;
  end;
end;

function GetIPOfDomain(const Domain: String): String;
var
  ia: TInAddr;
  phe: PHostEnt;
begin
  ia := TInAddr(inet_addr(PChar(Domain)));
  if Longint(ia) = INADDR_NONE then begin
    phe := GetHostByName(PChar(Domain));
    if phe <> nil then begin
      if phe.h_length = SizeOf(TInAddr) then
	ia := PInAddr(phe.h_addr_list^)^
      else WSASetLastError(WSANO_DATA);
    end;
  end;
  Result := inet_ntoa(ia);
end;

function GetIPFromAny(const Target: String): String;
begin
  Result := GetIPOfDomain(GetWebDomain(Target));
end;

function WhoisData(const Target, Service: String): String;
const
  IP_PORT_WHOIS_SERVER = 43;
  TRANSMIT_SIZE = $100;
  MAX_SIZE = $10000;
var
  wsad: TWSAData;
  sock: TSocket;
  sai: TSockAddrIn;
  sWhois: String;
  pBuffer, pPosition: PChar;
  iResult: Integer;
begin
  if LoadWinInet(false) and (WSAStartup($0101, wsad) = 0) then begin
    try
      Result := GetIPFromAny(Target);
      sock := WinSock.socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
      if sock <> 0 then begin
	try
	  FillChar(sai, SizeOf(TSockAddr), 0);
	  with sai do begin
	    sin_family := PF_INET;
	    sin_port := WinSock.htons(IP_PORT_WHOIS_SERVER);
	    sin_addr.S_addr := WinSock.inet_addr(PChar(GetIPFromAny(Service)));
	  end;
	  if WinSock.connect(sock, sai, SizeOf(TSockAddrIn)) = 0 then begin
	    sWhois := Target + CRLF;
	    pBuffer := StrAlloc(MAX_SIZE);
	    if pBuffer <> nil then begin
	      try
		StrPLCopy(pBuffer, sWhois, pred(MAX_SIZE));
		if WinSock.send(sock, pBuffer^, TRANSMIT_SIZE, 0) <> SOCKET_ERROR then begin
		  pPosition := pBuffer;
		  repeat
		    iResult := WinSock.recv(sock, pPosition^, TRANSMIT_SIZE, 0);
		    inc(pPosition, iResult);
		  until (iResult = 0) or (pPosition - pBuffer + TRANSMIT_SIZE >= MAX_SIZE) or (iResult = SOCKET_ERROR);
		end;
		Result := pBuffer;
		Exit;
	      finally
		StrDispose(pBuffer);
	      end;
	    end;
	  end;
	finally
	  closesocket(sock);
	end;
      end;
    finally
      WSACleanup;
    end;
  end;
  Result := E;
end;

function GetAbuseEmailAddress(const Data: String): String;
const
  ABUSE_ADDR = 'abuse@';
  EMAIL_ADDR = 'e-mail:';
var
  iBeg, iEnd: Integer;
begin
  iBeg := pos(ABUSE_ADDR, Data);
  if iBeg > 0 then begin
    iEnd := iBeg;
    while not (Data[iEnd] in [NUL, LF, CR, SP]) do inc(iEnd);
    if iEnd > iBeg then begin
      Result := Copy(Data, iBeg, iEnd - iBeg);
      Exit;
    end;
  end
  else begin
    iBeg := pos(EMAIL_ADDR, Data);
    if iBeg > 0 then begin
      inc(iBeg, Length(EMAIL_ADDR));
      while not (Data[iBeg] in [NUL, LF, CR, AT]) do inc(iBeg);
      iEnd := succ(iBeg);
      if Data[iBeg] = AT then begin
        dec(iBeg);
	while (iBeg > 0) and not (Data[iBeg] in [LF, CR, SP]) do dec(iBeg);
	inc(iBeg);
	while not (Data[iEnd] in [NUL, LF, CR, SP]) do inc(iEnd);
	if iEnd > iBeg then begin
	  Result := Copy(Data, iBeg, iEnd - iBeg);
	  Exit;
	end;
      end;
    end;
  end;
  Result := E;
end;

function NTPTime(const Server: String; Port: Integer): TDateTime;
type
  TNTPDataGram = Record
    Header: Array[0..3] of Byte;
    RootDelay: Cardinal;
    RootDispersion: Cardinal;
    RefID: Cardinal;
    ClientRef1: Cardinal;
    ClientRef2: Cardinal;
    ClientSent1: Cardinal;
    ClientSent2: Cardinal;
    ServerReceived1: Cardinal;
    ServerReceived2: Cardinal;
    ClientReceived1: Cardinal;
    ClientReceived2: Cardinal;
  end;
const
  TRANSMIT_SIZE = SizeOf(TNTPDataGram);
var
  err: Cardinal;
  wsad: TWSAData;
  sock: TSocket;
  sai: TSockAddrIn;
  ntpdg: TNTPDatagram;
  dtSent, dtReceived: TDateTime;
begin
  err := 0;
  Result := 0;
  try
    if WSAStartup($0101, wsad) = 0 then begin
      try
	sock := WinSock.socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if sock <> 0 then begin
	  try
	    FillChar(sai, SizeOf(TSockAddr), 0);
	    FillChar(ntpdg, SizeOf(TNTPDataGram), 0);
	    with sai do begin
	      sin_family := PF_INET;
	      sin_port := WinSock.htons(Port);
	      sin_addr.S_addr := Longint(GetInAddrOfDomain(Server));
	    end;
	    ntpdg.Header[0] := $1B;
	    if WinSock.connect(sock, sai, SizeOf(TSockAddrIn)) = 0 then begin
	      dtSent := Now;
	      if (WinSock.send(sock, ntpdg, TRANSMIT_SIZE, 0) <> SOCKET_ERROR)
	      and (WinSock.recv(sock, ntpdg, TRANSMIT_SIZE, 0) <> SOCKET_ERROR) then begin
		dtReceived := Now;
		with ntpdg do if ((Header[0] and 4) <> 0) and (Header[1] <> 0) and (ServerReceived2 <> 0) then begin
		  Result := (dtReceived - dtSent) / 2 + NTPTimeToLocalTime(ServerReceived1, ServerReceived2);
		  SetLastError(0);
		end
		else err := ERROR_INVALID_DATA;
	      end
	      else err := WSAGetLastError;
	    end
	    else err := WSAGetLastError;
	  finally
	    closesocket(sock);
	  end;
	end
	else err := WSAGetLastError;
      finally
	WSACleanup;
      end;
    end
    else err := WSAGetLastError;
  finally
    SetLastError(err);
  end;
end;


end.

