{$J+,Z4}
unit WindowUtils;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, Controls, Forms, Graphics, SysUtils, StrUtil, Processes, UxThemes;

{$IFDEF CONDITIONALEXPRESSIONS}
const
  CW_USEDEFAULT		= Integer(CW_USEDEFAULT);
{$ENDIF}

const
  CLASS_BUTTON		= 'BUTTON';
  CLASS_CBX		= 'COMBOBOX';
  CLASS_DIALOG		= '#32770';
  CLASS_EDIT		= 'EDIT';
  CLASS_LISTBOX		= 'LISTBOX';
  CLASS_STATIC		= 'STATIC';

const
  RT_ANICURSOR		= MakeIntResource(21);
  RT_ANIICON		= MakeIntResource(22);
  RT_HTML		= MakeIntResource(23);
  RT_MANIFEST		= MakeIntResource(24);

const // Resource flags
  LR_DEFAULTCOLOR	=$0000;
  LR_MONOCHROME		=$0001;
  LR_COLOR		=$0002;
  LR_COPYRETURNORG	=$0004;
  LR_COPYDELETEORG	=$0008;
  LR_LOADFROMFILE	=$0010;
  LR_LOADTRANSPARENT	=$0020;
  LR_DEFAULTSIZE	=$0040;
  LR_VGACOLOR		=$0080;
  LR_LOADMAP3DCOLORS	=$1000;
  LR_CREATEDIBSECTION	=$2000;
  LR_COPYFROMRESOURCE	=$4000;
  LR_SHARED		=$8000;

const // Cursor resources
  OCR_NORMAL		= 32512;
  OCR_IBEAM		= 32513;
  OCR_WAIT		= 32514;
  OCR_CROSS		= 32515;
  OCR_UP		= 32516;
  OCR_SIZE		= 32640;  { OBSOLETE: use OCR_SIZEALL }
  OCR_ICON		= 32641;  { OBSOLETE: use OCR_NORMAL }
  OCR_SIZENWSE		= 32642;
  OCR_SIZENESW		= 32643;
  OCR_SIZEWE		= 32644;
  OCR_SIZENS		= 32645;
  OCR_SIZEALL		= 32646;
  OCR_ICOCUR		= 32647;  { OBSOLETE: use OIC_WINLOGO }
  OCR_NO		= 32648;
  OCR_HAND		= 32649;
  OCR_APPSTARTING	= 32650;

const // Icon resources
  OIC_SAMPLE		= 32512;
  OIC_HAND		= 32513;
  OIC_QUESTION		= 32514;
  OIC_BANG		= 32515;
  OIC_NOTE		= 32516;
  OIC_WINLOGO		= 32517;
  OIC_WARNING		= OIC_BANG;
  OIC_ERROR		= OIC_HAND;
  OIC_INFORMATION	= OIC_NOTE;

const // Static Styles
  SS_ENDELLIPSIS	= $4000;
  SS_PATHELLIPSIS	= $8000;
  SS_WORDELLIPSIS	= $C000;
  SS_ELLIPSISMASK	= $C000;

const // from comctl32
  LIM_SMALL		= 0; // corresponds to SM_CXSMICON/SM_CYSMICON
  LIM_LARGE		= 1; // corresponds to SM_CXICON/SM_CYICON

type
  COLOR16 = Word;

  PTriVertex = ^TTriVertex;
  TTriVertex = Packed Record
    x: Longint;
    y: Longint;
    Red: COLOR16;
    Green: COLOR16;
    Blue: COLOR16;
    Alpha: COLOR16;
  end;

  PGradientRect = ^TGradientRect;
  TGradientRect = Packed Record
    UpperLeft: ULONG;
    LowerRight: ULONG;
  end;

const
  GRADIENT_FILL_RECT_H = $00000000;
  GRADIENT_FILL_RECT_V = $00000001;

const
  EMPTY_RECT: TRect = (Left: 0; Top: 0; Right: 0; Bottom: 0);

type
  TNCLFontType = (ftCaption, ftSmCaption, ftMenu, ftStatus, ftMessage);
  TNCLFontNameSize = Record
    fnsName: String;
    fnsSize: Integer;
  end;

var
  GradientFill: function(DC: HDC; Vertex: PTriVertex; NumVertex: ULONG;
			 Mesh: Pointer; NumMesh, Mode: ULONG): BOOL; stdcall;
  LoadIconMetric: function(hinst: HINST; pszName: LPCWSTR; lims: Integer;
  			   var phico: HICON): HResult; stdcall;

procedure ProcessMessages;
function  GetCaretPos: TPoint;
function  SetCaretPos(const Pos: TPoint): Longbool;
function  OffScreenPos: TPoint;
function  GetCursorPos: TPoint;
function  MakeSize(CX, CY: Integer): TSize;
function  MakePoints(Pos: Cardinal): TPoint;
function  NullSize(const Size: TSize): Longbool;
function  PointsToPos(Point: TPoint): Cardinal;
function  SamePoints(Point1, Point2: TPoint): Longbool;
function  GetWidthTwips(Window: THandle; const Data: Variant): Integer;
function  GetSysScreenRect: TRect;
function  GetSysScreenDimensions: TSize;
function  GetWindowRect(Window: THandle): TRect;
function  GetClientRect(Window: THandle): TRect;
function  GetClientWidth(Window: THandle): Integer;
function  GetClientHeight(Window: THandle): Integer;
function  IsAllRectEmpty(const Rect: TRect): Longbool;
function  Get_X_lParam(lParam: Integer): SmallInt;
function  Get_Y_lParam(lParam: Integer): SmallInt;
function  Make_lParam(Low, High: Word): Longint;
function  GetAsyncMouseBtnUnswitched(VKCode: Integer): SmallInt;
function  GetMaxWindowTextWidth(Window: THandle): Integer;
function  GetFontHeightFromPoints(Points: Integer): Integer;
function  GetFontPointsFromHeight(Height: Integer): Integer;
function  PointsFromPixels(Pixels: Integer): Integer;
function  PixelsFromPoints(Points: Integer): Integer;
function  PixelsFromTwips(Twips: Integer): Integer;
function  MilliMetersFromTwips(Twips: Integer): Integer;
function  TwipsFromMilliMeters(MilliMeters: Integer): Integer;
function  ShellDlgFont(Theme: THandle; FontID: Integer): String;
function  SysFontNameAndSize(NCLFontType: TNCLFontType): TNCLFontNameSize;
function  GetHtmlSysFontFormat(const Html: String): String;
function  CheckQMRegEx(const Target: String; RegEx: PChar; Len: Integer): Longbool;
function  ListMDIWindows(Window: THandle; const MDIWindows: TList): Bool; stdcall;
function  FindApplicationWindow(const FileName, ChildClass, ChildText: String): THandle;
function  FindAllChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
function  FindDisabledChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
function  FindInvisibleChild(ParentWindow: THandle; const ClassName, WindowText: String): THandle;
function  FindFirstChild(ParentWindow: THandle; const ClassName, WindowText: String; CtlID: Integer; RegExClass: Longbool): THandle;
function  FindNextChild(ParentWindow: THandle; const ClassName, WindowText: String; CtlID: Integer; RegExClass: Longbool): THandle;
function  CenterInClientRect(const ChildRect: TRect; Child, Parent: THandle; Draw: Longbool): Longbool;
function  FindButton(ParentWindow: THandle; const WindowText: String; Style: DWord): THandle;
function  FindCheckBox(Dialog: THandle; const WindowText: String): THandle;
function  FindComboBox(Dialog: THandle): THandle;
function  FindCtrlByID(Dialog: THandle; CtlID: Integer): THandle;
function  FindEditControl(Dialog: THandle): THandle;
function  FindStatic(Dialog: THandle; const WindowText: String): THandle;
procedure ClickDlgBtn(Dialog: THandle; BtnID: Integer);
function  GetDlgBtnChecked(Dialog: THandle; BtnID: Integer): Integer;
function  SetDialogCheckBox(CheckBox: THandle; Check: Longbool): Longbool;
function  IsDialogCheckBoxChecked(CheckBox: THandle): Longbool;
function  ClickDialogCheckBoxChecked(CheckBox: THandle): Longbool;
function  CheckMenuItem(Menu: THandle; ItemID: Integer; Check: Longbool): Longbool;
function  SelectCBXItemIndex(ComboBox: THandle; Index: Integer): Longbool;
function  GetCBXItemIndex(ComboBox: THandle; const Item: String): Integer;
function  DeleteCBXIndex(ComboBox: THandle; Index: Integer): Integer;
function  SelectCBXIndex(ComboBox: THandle; Index: Integer): Integer;
function  GetSelectedCBXIndex(ComboBox: THandle): Integer;
function  GetSelectedCBXItem(ComboBox: THandle): String;
function  GetLBXText(ListBox: THandle): String;
function  GetMDIParent(Window: THandle): THandle;
function  IsMDIWindowChild(MDIClient, Window: THandle): Longbool;
function  GetWindowModuleName(Window: THandle): String;
function  IsModuleWindow(Window: THandle): Longbool;
procedure SendKey(Window: THandle; Key: Integer);

function LoadSysIcon(ID: Cardinal): HICON;

function GetWindowModuleFileNameA(hwnd: HWND; pszFileName: PAnsiChar; cchFileNameMax: UINT): UINT; stdcall;
function GetWindowModuleFileNameW(hwnd: HWND; pszFileName: PWideChar; cchFileNameMax: UINT): UINT; stdcall;


implementation

{$IFDEF CONDITIONALEXPRESSIONS}
uses
  Math, Variants;

const
  NULL = StrUtil.NULL;
{$ENDIF}

type
  TFindCase = (fcFirst, fcNext, fcAll);
  PWindowParams = ^TWindowParams;
  TWindowParams = Record
    wpWindowText: String;
    wpClassName: String;
    wpWindowList: TList;
    wpButtonStyle: DWord;
    wpControlID: Integer;
    wpDisabled: Longbool;
    wpInvisible: Longbool;
    wpCheckRegEx: Longbool;
    fcType: TFindCase;
    wpHandle: THandle;
  end;

{$IFDEF VER90}
type
  TFPURoundingMode = (rmNearest, rmDown, rmUp, rmTruncate);

function SetRoundMode(const RoundMode: TFPURoundingMode): TFPURoundingMode;

  function Get8087CW: Word;
  asm
    PUSH    0
    FNSTCW  [ESP].Word
    POP     EAX
  end;

  procedure Set8087CW(NewCW: Word);
  var
    CtlWord: Word;
  begin
    CtlWord := NewCW;
    asm
      FNCLEX
      FLDCW   CtlWord
    end;
  end;

var
  CtlWord: Word;
begin
  CtlWord := Get8087CW;
  Set8087CW((CtlWord and $F3FF) or (Ord(RoundMode) shl 10));
  Result := TFPURoundingMode((CtlWord shr 10) and 3);
end;
{$ENDIF}

procedure ProcessMessages;
var
  msg: TMsg;
begin
  while PeekMessage(msg, 0, 0, 0, PM_REMOVE) do begin
    Windows.TranslateMessage(msg);
    DispatchMessage(msg);
  end;
end;

function GetCaretPos: TPoint;
begin
  if not Windows.GetCaretPos(Result) then Result := Point(-1, -1);
end;

function SetCaretPos(const Pos: TPoint): Longbool;
begin
  with Pos do Result := Windows.SetCaretPos(X, Y);
end;

function OffScreenPos: TPoint;
begin
  with Result do begin
    X := -1;
    Y := -1;
  end;
end;

function GetCursorPos: TPoint;
begin
  if not Windows.GetCursorPos(Result) then Result := Point(-1, -1);
end;

function MakeSize(CX, CY: Integer): TSize;
begin
  Result.CX := CX;
  Result.CY := CY;
end;

function MakePoints(Pos: Cardinal): TPoint;
begin
  with Result do begin
    X := Pos and $FFFF;
    Y := Pos shr 16;
  end;
end;

function NullSize(const Size: TSize): Longbool;
begin
  with Size do Result := (cx = 0) and (cy = 0);
end;

function PointsToPos(Point: TPoint): Cardinal;
begin
  with Point do Result := (Y shl 16) or (X and $FFFF);
end;

function SamePoints(Point1, Point2: TPoint): Longbool;
begin
  Result := (Point1.X = Point2.X) and (Point1.Y = Point2.Y);
end;

function GetWidthTwips(Window: THandle; const Data: Variant): Integer;
var
  hDC, hFont: THandle;
  sData: String;
  szText: TSize;
begin
  hDC := GetDC(Window);
  hFont := 0;
  try
    if VarType(Data) and varTypeMask = varString then begin
      sData := Data;
      hFont := SelectObject(hDC, SendMessage(Window, WM_GETFONT, 0, 0));
      GetTextExtentPoint32(hDC, PChar(sData), Length(sData), szText);
    end
    else szText.cx := Data;
    Result := MulDiv(szText.cx, 1440, GetDeviceCaps(hDC, LOGPIXELSX));
  finally
    if hFont <> 0 then SelectObject(hDC, hFont);
    ReleaseDC(Window, hDC);
  end;
end;

function GetSysScreenRect: TRect;
begin
  SystemParametersInfo(SPI_GETWORKAREA, 0, @Result, 0);
end;

function GetSysScreenDimensions: TSize;
begin
  with GetSysScreenRect do Result := MakeSize(Right - Left, Bottom - Top);
end;

function GetWindowRect(Window: THandle): TRect;
begin
  if not Windows.GetWindowRect(Window, Result) then Result := EMPTY_RECT;
end;

function GetClientRect(Window: THandle): TRect;
begin
  if not Windows.GetClientRect(Window, Result) then Result := EMPTY_RECT;
end;

function GetClientWidth(Window: THandle): Integer;
begin
  Result := GetClientRect(Window).Right;
end;

function  GetClientHeight(Window: THandle): Integer;
begin
  Result := GetClientRect(Window).Bottom;
end;

function IsAllRectEmpty(const Rect: TRect): Longbool;
begin
   Result := EqualRect(Rect, EMPTY_RECT);
end;

function Get_X_lParam(lParam: Integer): SmallInt;
begin
  Result := LoWord(lParam);
end;

function Get_Y_lParam(lParam: Integer): SmallInt;
begin
  Result := HiWord(lParam);
end;

function Make_lParam(Low, High: Word): Longint;
begin
  Result := High shl 16 or Low;
end;

function GetAsyncMouseBtnUnswitched(VKCode: Integer): SmallInt;
begin
  if GetSystemMetrics(SM_SWAPBUTTON) <> 0 then begin
    case VKCode of
      VK_LBUTTON: VKCode := VK_RBUTTON;
      VK_RBUTTON: VKCode := VK_LBUTTON;
    end;
  end;
  Result := GetAsyncKeyState(VKCode);
end;

function GetMaxWindowTextWidth(Window: THandle): Integer;
begin
  Result := GetClientRect(Window).Right - GetSystemMetrics(SM_CXVSCROLL);
end;

function GetFontHeightFromPoints(Points: Integer): Integer;
var
  hDC: THandle;
begin
  hDC := GetDC(0);
  try
    Result := Abs(MulDiv(Points, GetDeviceCaps(hDC, LOGPIXELSY), 72));
  finally
    ReleaseDC(0, hDC);
  end;
end;

function GetFontPointsFromHeight(Height: Integer): Integer;
var
  hDC: THandle;
begin
  hDC := GetDC(0);
  try
    Result := Abs(MulDiv(Height, 72, GetDeviceCaps(hDC, LOGPIXELSY)));
  finally
    ReleaseDC(0, hDC);
  end;
end;

function PointsFromPixels(Pixels: Integer): Integer;
var
  hDC: THandle;
begin
  hDC := GetDC(0);
  try
    Result := Abs(MulDiv(Pixels, 72, GetDeviceCaps(hDC, LOGPIXELSY)));
  finally
    ReleaseDC(0, hDC);
  end;
end;

function PixelsFromPoints(Points: Integer): Integer;
var
  hDC: THandle;
begin
  hDC := GetDC(0);
  try
    Result := Abs(MulDiv(Points, GetDeviceCaps(hDC, LOGPIXELSY), 72));
  finally
    ReleaseDC(0, hDC);
  end;
end;

function PixelsFromTwips(Twips: Integer): Integer;
var
  hDC: THandle;
begin
  hDC := GetDC(0);
  try
    Result := MulDiv(Twips, GetDeviceCaps(hDC, LOGPIXELSY), 1440);
  finally
    ReleaseDC(0, hDC);
  end;
end;

function MilliMetersFromTwips(Twips: Integer): Integer;
var
  rm: TFPURoundingMode;
begin
  rm := SetRoundMode(rmNearest);
  try
    Result := Round(25.4 * Twips / 1440);
  finally
    SetRoundMode(rm);
  end;
end;

function TwipsFromMilliMeters(MilliMeters: Integer): Integer;
var
  rm: TFPURoundingMode;
begin
  rm := SetRoundMode(rmNearest);
  try
    Result := Round(1440 * MilliMeters / 25.4);
  finally
    SetRoundMode(rm);
  end;
end;

function ShellDlgFont(Theme: THandle; FontID: Integer): String;
const
  MsShellDlg = 'MS Shell Dlg';
  MsShellDlg2 = MsShellDlg + SP + '2';
var
  lfSystem: TLogFontW;
  hRegKey: HKEY;
  dwBufSize: DWORD;
begin
  if Theme <> 0 then begin
    FillChar(lfSystem, SizeOf(TLogFont), 0);
    if GetThemeSysFont(Theme, FontID, lfSystem) = S_OK then begin
      Result := WideCharToString(lfSystem.lfFaceName);
      if Result <> E then Exit;
    end;
  end;
  hRegKey := 0;
  dwBufSize := 0;
  if RegOpenKeyEx(HKEY_LOCAL_MACHINE, 'Software\Microsoft\Windows NT\CurrentVersion\FontSubstitutes', 0,
		  KEY_QUERY_VALUE, hRegKey) = ERROR_SUCCESS then begin
    try
      if RegQueryValueEx(hRegKey, MsShellDlg2, nil, nil, nil, @dwBufSize) = ERROR_SUCCESS then begin
	SetString(Result, nil, pred(dwBufSize));
	if RegQueryValueEx(hRegKey, MsShellDlg2, nil, nil, Pointer(Result), @dwBufSize) = ERROR_SUCCESS then Exit;
      end;
    finally
      RegCloseKey(hRegKey);
    end;
  end;
  Result := MsShellDlg;
end;

function SysFontNameAndSize(NCLFontType: TNCLFontType): TNCLFontNameSize;
var
  nclm: TNonClientMetrics;
begin
  with Result do begin
    fnsName := E;
    fnsSize := 0;
  end;
  nclm.cbSize := SizeOf(TNonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(TNonClientMetrics), @nclm, 0) then with nclm, Result do begin
    case NCLFontType of
      ftCaption: begin
	fnsName := lfCaptionFont.lfFaceName;
	fnsSize := lfCaptionFont.lfheight;
      end;
      ftSmCaption: begin
	fnsName := lfSmCaptionFont.lfFaceName;
	fnsSize := lfSmCaptionFont.lfheight;
      end;
      ftMenu: begin
	fnsName := lfMenuFont.lfFaceName;
	fnsSize := lfMenuFont.lfheight;
      end;
      ftStatus: begin
	fnsName := lfStatusFont.lfFaceName;
	fnsSize := lfStatusFont.lfheight;
      end;
      ftMessage: begin
	fnsName := lfMessageFont.lfFaceName;
	fnsSize := lfMessageFont.lfheight;
      end;
    end;
  end;
  with Result do if fnsSize <> 0 then fnsSize := GetFontPointsFromHeight(fnsSize);
end;

function GetHtmlSysFontFormat(const Html: String): String;
const
  DB = '<div style="font-family: %s; font-size: %dpt;">';
  DE = '</div>';
  FB = '<font face="%s"; size="2">';
  FE = '</font>';
begin
  if MainWinVersion < 6 then
    Result := Format(FB, [SysFontNameAndSize(ftStatus).fnsName]) + Html + FE
  else with SysFontNameAndSize(ftStatus) do Result := Format(DB, [fnsName, succ(fnsSize)]) + Html + DE;
end;

function CheckQMRegEx(const Target: String; RegEx: PChar; Len: Integer): Longbool;
var
  pTarget: PChar;
  iIndex: Integer;
begin
  Result := false;
  if Len = Length(Target) then begin
    Result := true;
    pTarget := Pointer(Target);
    for iIndex := 0 to pred(Len) do begin
      if (RegEx[iIndex] <> QM) and (pTarget[iIndex] <> RegEx[iIndex]) then begin
	Result := false;
	Break;
      end;
    end;
  end;
end;

function ListMDIWindows(Window: THandle; const MDIWindows: TList): Bool; stdcall;
begin
  Result := true;
  with MDIWindows do begin
    if IsWindowVisible(Window)
    and ((GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_MDICHILD) <> 0)
    and (IndexOf(Ptr(Window)) = -1) then begin
      Insert(0, Ptr(Window));
    end;
  end;
end;

function FindApplicationWindow(const FileName, ChildClass, ChildText: String): THandle;
begin
  Result := FindProcessInfo(ExtractFileName(FileName), true).piWindow;
  if (Result = 0) or ((ChildClass = E) and (ChildText = E)) then Exit;
  Result := FindFirstChild(Result, ChildClass, ChildText, 0, false);
end;

function CheckChildWindows(FoundWindow: THandle; WP: PWindowParams): Bool; stdcall;

  function IsDisabled(FoundWindow: THandle): Longbool;
  begin
    Result := IsWindowVisible(FoundWindow) and not IsWindowEnabled(FoundWindow);
  end;

var
  sFound: String;
  pSearch: PChar;
  iSearch, iBeg, iEnd, iLen: Integer;
begin
  Result := true;
  with WP^ do begin
    if (not wpDisabled or IsDisabled(FoundWindow)) and not (wpInvisible and IsWindowVisible(FoundWindow)) then begin
      if (wpControlID <> 0) and (GetDlgCtrlID(FoundWindow) = wpControlID) then begin
	wpHandle := FoundWindow;
	Result := false;
	Exit;
      end;
      if wpWindowText <> E then begin
	if FirstStrPos(wpWindowText, DelAmp(WndString(FoundWindow))) <> 0 then begin
	  wpHandle := FoundWindow;
	  if wpClassName = E then begin
	    Result := false;
	    Exit;
	  end;
	end;
      end;
      if wpClassName <> E then begin
	sFound := ClassString(FoundWindow);
	pSearch := Pointer(wpClassName);
	iSearch := Length(wpClassName);
	iBeg := 1;
	while iBeg <= iSearch do begin
	  inc(pSearch, pred(iBeg));
	  iEnd := NextCharPos(CM, wpClassName, iBeg);
	  if iEnd = 0 then iEnd := succ(iSearch);
	  iLen := iEnd - iBeg;
	  if iLen = Length(sFound) then begin
	    if (wpCheckRegEx and CheckQMRegEx(sFound, pSearch, iLen))
	    or (CompareString(LOCALE_USER_DEFAULT, 0, Pointer(sFound), -1, pSearch, iLen) = 2) then begin
	      if (wpHandle <> 0) or ((wpWindowText = E) and (fcType = fcFirst)) then begin
		wpHandle := FoundWindow;
		Result := false;
	      end
	      else begin
		case fcType of
		  fcNext: begin
		    fcType := fcFirst;
		    wpHandle := 0;
		  end;
		  fcAll: begin
		    if wpWindowList <> nil then wpWindowList.Add(Ptr(FoundWindow));
		    wpHandle := 0;
		  end;
		end;
	      end;
	      Exit;
	    end;
	  end;
	  dec(pSearch, pred(iBeg));
	  iBeg := succ(iEnd);
	end;
	if Result = true then wpHandle := 0;
      end;
    end;
  end;
end;

function FindAllChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowList := Children;
      fcType := fcAll;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpWindowList.Count;
  end;
end;

function FindDisabledChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowList := Children;
      wpDisabled := true;
      fcType := fcAll;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpWindowList.Count;
  end;
end;

function FindInvisibleChild(ParentWindow: THandle; const ClassName, WindowText: String): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowText := WindowText;
      wpInvisible := true;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function FindFirstChild(ParentWindow: THandle; const ClassName, WindowText: String;
			CtlID: Integer; RegExClass: Longbool): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0 then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpWindowText := DelAmp(WindowText);
      wpClassName := UpperCase(ClassName);
      wpControlID := CtlID;
      wpCheckRegEx := RegExClass;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function FindNextChild(ParentWindow: THandle; const ClassName, WindowText: String;
		       CtlID: Integer; RegExClass: Longbool): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0 then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpWindowText := DelAmp(WindowText);
      wpClassName := UpperCase(ClassName);
      wpControlID := CtlID;
      wpCheckRegEx := RegExClass;
      fcType := fcNext;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function CheckButtons(FoundWindow: THandle; WP: PWindowParams): Bool; stdcall;
begin
  Result := true;
  with WP^ do begin
    if (GetWindowLong(FoundWindow, GWL_STYLE) and wpButtonStyle <> 0) and (ClassString(FoundWindow) = CLASS_BUTTON)
    and ((wpWindowText = E) or (FirstStrPos(wpWindowText, DelAmp(WndString(FoundWindow))) <> 0)) then begin
      wpHandle := FoundWindow;
      Result := false;
    end;
  end;
end;

function CenterInClientRect(const ChildRect: TRect; Child, Parent: THandle; Draw: Longbool): Longbool;
var
  rParent, rChild: TRect;
begin
  Result := false;
  if Parent = 0 then Parent := GetDesktopWindow;
  if not IsAllRectEmpty(ChildRect) and (Parent <> 0) then begin
    rParent := GetClientRect(Parent);
    if not IsAllRectEmpty(rParent) then begin
      rChild := ChildRect;
      with rChild do begin
	Left := rParent.Left + (rParent.Right - Right) div 2;
	Top := rParent.Top + (rParent.Bottom - Bottom) div 2;
	Result := MoveWindow(Child, Left, Top, Right, Bottom, Draw);
      end;
    end;
  end;
end;

function FindButton(ParentWindow: THandle; const WindowText: String; Style: DWord): THandle;
var
  wpSearch: TWindowParams;
begin
  if WindowText <> E then
    Result := FindFirstChild(ParentWindow, CLASS_BUTTON, WindowText, 0, false)
  else begin
    Result := 0;
    if ParentWindow <> 0 then begin
      FillByte(wpSearch, SizeOf(TWindowParams), 0);
      with wpSearch do begin
	wpWindowText := DelAmp(WindowText);
	wpButtonStyle := Style;
      end;
      EnumChildWindows(ParentWindow, @CheckButtons, LPARAM(@wpSearch));
      Result := wpSearch.wpHandle;
    end;
  end;
end;

function FindCheckBox(Dialog: THandle; const WindowText: String): THandle;
begin
  Result := FindButton(Dialog, WindowText, BS_CHECKBOX);
end;

function FindComboBox(Dialog: THandle): THandle;
begin
  Result := FindFirstChild(Dialog, CLASS_CBX, E, 0, false);
end;

function  FindCtrlByID(Dialog: THandle; CtlID: Integer): THandle;
begin
  Result := FindFirstChild(Dialog, CLASS_EDIT, E, 0, false);
end;

function FindEditControl(Dialog: THandle): THandle;
begin
  Result := FindFirstChild(Dialog, CLASS_EDIT, E, 0, false);
end;

function FindStatic(Dialog: THandle; const WindowText: String): THandle;
begin
  Result := FindFirstChild(Dialog, CLASS_STATIC, WindowText, 0, false);
end;

procedure ClickDlgBtn(Dialog: THandle; BtnID: Integer);
begin
  SendDlgItemMessage(Dialog, BtnID, BM_CLICK, 0, 0);
end;

function GetDlgBtnChecked(Dialog: THandle; BtnID: Integer): Integer;
begin
  Result := SendDlgItemMessage(Dialog, BtnID, BM_GETCHECK, 0, 0);
end;

function SetDialogCheckBox(CheckBox: THandle; Check: Longbool): Longbool;
begin
  Result := true;
  if Check then
    SendMessage(CheckBox, BM_SETCHECK, BST_CHECKED, 0)
  else SendMessage(CheckBox, BM_SETCHECK, BST_UNCHECKED, 0);
end;

function IsDialogCheckBoxChecked(CheckBox: THandle): Longbool;
begin
  Result := (SendMessage(CheckBox, BM_GETCHECK, 0, 0) = BST_CHECKED);
end;

function ClickDialogCheckBoxChecked(CheckBox: THandle): Longbool;
begin
  Result := true;
  if SendMessage(CheckBox, BM_GETCHECK, 0, 0) = BST_CHECKED then SendMessage(CheckBox, BM_SETCHECK, BST_UNCHECKED, 0);
  SendMessage(CheckBox, BM_CLICK, 0, 0);
end;

function CheckMenuItem(Menu: THandle; ItemID: Integer; Check: Longbool): Longbool;
begin
  Result := (Windows.CheckMenuItem(Menu, ItemID, MF_BYCOMMAND or (MF_CHECKED * (ord(Check = true)))) <> $FFFFFFFF)
end;

function SelectCBXItemIndex(ComboBox: THandle; Index: Integer): Longbool;
begin
  Result := (SendMessage(ComboBox, CB_SETCURSEL, Index, 0) <> CB_ERR);
end;

function GetCBXItemIndex(ComboBox: THandle; const Item: String): Integer;
begin
  Result := CB_ERR;
  if Item <> E then Result := ord(SendMessage(ComboBox, CB_FINDSTRINGEXACT, -1, Longint(Item)));
end;

function DeleteCBXIndex(ComboBox: THandle; Index: Integer): Integer;
begin
  Result := CB_ERR;
  if Index <> CB_ERR then Result := SendMessage(ComboBox, CB_DELETESTRING, Index, 0);
end;

function SelectCBXIndex(ComboBox: THandle; Index: Integer): Integer;
begin
  Result := CB_ERR;
  if Index <> CB_ERR then Result := SendMessage(ComboBox, CB_SETCURSEL, Index, 0);
end;

function GetSelectedCBXIndex(ComboBox: THandle): Integer;
begin
  Result := SendMessage(ComboBox, CB_GETCURSEL, 0, 0);
end;

function GetSelectedCBXItem(ComboBox: THandle): String;
var
  iSelected: Integer;
begin
  iSelected := SendMessage(ComboBox, CB_GETCURSEL, 0, 0);
  if iSelected > 0 then begin
    SetString(Result, nil, SendMessage(ComboBox, CB_GETLBTEXTLEN, iSelected, 0));
    Delete(Result, succ(SendMessage(ComboBox, CB_GETLBTEXT, iSelected, Longint(Result))), MAXINT);
  end;
end;

function GetLBXText(ListBox: THandle): String;
var
  iCount, iLength, iIndex: Integer;
  pBuffer: PChar;
begin
  iCount := SendMessage(ListBox, LB_GETCOUNT, 0, 0);
  if iCount > 0 then begin
    iLength := 0;
    for iIndex := 0 to pred(iCount) do inc(iLength, SendMessage(ListBox, LB_GETTEXTLEN, iIndex, 0));
    SetString(Result, nil, iLength + iCount);
    pBuffer := Pointer(Result);
    for iIndex := 0 to pred(iCount) do begin
      iLength := SendMessage(ListBox, LB_GETTEXT, iIndex, Longint(pBuffer));
      if iLength <> LB_ERR then begin
	inc(pBuffer, iLength);
	pBuffer^ := NULL;
	inc(pBuffer);
      end;
    end;
    Delete(Result, succ(pBuffer - Pointer(Result)), MAXINT);
    Exit;
  end;
  Result := E;
end;

function GetMDIParent(Window: THandle): THandle;
begin
  Result := Window;
  while (Result <> 0) and (GetWindowLong(Result, GWL_EXSTYLE) and WS_EX_MDICHILD = 0) do begin
    Result := GetParent(Result);
  end;
end;

function IsMDIWindowChild(MDIClient, Window: THandle): Longbool;
begin
  Result := (GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_MDICHILD <> 0) or
	    IsChild(SendMessage(MDIClient, WM_MDIGETACTIVE, 0, 0), Window);

end;

function GetWindowModuleName(Window: THandle): String;
begin
  SetString(Result, NIL, MAX_PATH);
  Delete(Result, succ(GetWindowModuleFileNameA(Window, Pointer(Result), MAX_PATH)), MAXINT);
end;

function IsModuleWindow(Window: THandle): Longbool;
begin
  Result := (THandle(GetWindowLong(Window, GWL_HINSTANCE)) = hInstance);
end;

procedure SendKey(Window: THandle; Key: Integer);
begin
  SendMessage(Window, WM_KEYDOWN, Key, 0);
  SendMessage(Window, WM_KEYUP, Key, LPARAM($C0000000));
end;

function LoadSysIcon(ID: Cardinal): HICON;
begin
  if Assigned(LoadIconMetric) and (LoadIconMetric(0, MakeIntResourceW(ID), LIM_SMALL, Result) = S_OK) then Exit;
  Result := LoadImageW(0, MakeIntResourceW(ID), IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_SHARED);
end;

function GetWindowModuleFileNameA; external user32 name 'GetWindowModuleFileNameA';
function GetWindowModuleFileNameW; external user32 name 'GetWindowModuleFileNameW';

begin
  GradientFill := GetProcAddress(GetModuleHandle('MSIMG32.DLL'), 'GradientFill');
  LoadIconMetric := GetProcAddress(GetModuleHandle(comctl32), 'LoadIconMetric');
end.

