{$J+,Z4}
unit AutoListDialog;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Buttons,
  ExtCtrlsEx, Grids, DropGrid, ToolTip, CBExtend, FlatBtn, StrUtil, FileUtils, PMPGPForm, PMPGPCommon,
  PMPGPResources, KeyPropTypes, AutoListHandler, PMHandler, PGPHandler;

const
  ADD_ENTRY_COL	= 0;
  ENCRYPT_COL	= 1;
  SIGN_COL	= 2;
  ASK_COL	= 3;
  PMIME_COL	= 4;
  SMIME_COL	= 5;
  DISABLED_COL	= 6;
  ALGORITHM_COL	= 7;
  HEX_ID_COL	= 8;
  USER_ID_COL	= 9;

const
  OPT_ENABLED	= 'I';
  OPT_DISABLED	= 'O';

const
  RSA		= 'RSA';
  DH_DSS	= 'DH/DSS';

type
  TDialogOption = (doMultiKeySelect, doSetOptions, doAddKeyIDs, doAddAddresses);
  TAutoListDialog = class(TPMPGPForm)
    pnTop: TPanel;
    lblSelect: TLabel;
    pnLeft: TPanel;
    pnGrid: TColoredFramePanel;
    dgAddresses: TDropGrid;
    edUserID: TEdit;
    pnToolTips: TPanel;
    pnSizeFrame: TPanel;
    pnSize: TPanel;
    pnAlgorithmFrame: TPanel;
    pnAlgorithm: TPanel;
    pnHexIDs: TPanel;
    cbxHexIDs: TPlainCBEx;
    pnUserIDFrame: TPanel;
    pnUserID: TPanel;
    pnRight: TPanel;
    pnButtons: TPanel;
    fbOk: TFlatBtn;
    fbFind: TFlatBtn;
    fbHelp: TFlatBtn;
    fbDelete: TFlatBtn;
    fbCancel: TFlatBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure dgAddressesDblClick(Sender: TObject);
    procedure dgAddressesDrawCell(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
    procedure dgAddressesEnter(Sender: TObject);
    procedure dgAddressesGetToolTipText(ACol, ARow: Longint; Fixed: Longbool; var Text: String);
    procedure sgAdressesKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure dgAddressesSelectCell(Sender: TObject; Col, Row: Longint; var CanSelect: Boolean);
    procedure dgAddressesTopLeftChanged(Sender: TObject);
    procedure pnSizeFrameResize(Sender: TObject);
    procedure pnHexIDsResize(Sender: TObject);
    procedure cbxHexIDsEnter(Sender: TObject);
    procedure cbxHexIDsExit(Sender: TObject);
    procedure cbxHexIDsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure cbxHexIDsListItemSelected(Sender: TObject; Index: Integer);
    procedure edUserIDEnter(Sender: TObject);
    procedure edUserIDExit(Sender: TObject);
    procedure edUserIDKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure fbHelpClick(Sender: TObject);
    procedure fbFindClick(Sender: TObject);
    procedure fbDeleteClick(Sender: TObject);
  private
    DialogOption: TDialogOption;
    SMimeEnabled: Longbool;
    Editable: TColor;
    NewCount: Integer;
    OldList, NewList: TAutoList;
    StartList, SearchList: String;
    function  DataChanged: Longbool;
    function  ExtractEntryOptions(Flags: Integer): String;
    function  ExtractListEntry(const Item: TALItem; var KeyIndex: Integer): String;
    procedure InvalidateOptionCells(ARow: Longint);
    function  IsMutableOptionCell(const Selected: TGridCoord): Longbool;
    procedure LoadHexIDBoxEntries;
    function  RetrieveEntryOptions(ARow: Longint): Integer;
    function  RetrieveListEntry(ARow: Longint): String;
    procedure SearchNames(Again: Longbool);
    procedure ShowHexIDBox(ARow: Longint);
    procedure ShowUserIDEdit(ARow: Longint);
    procedure SwitchCell;
    procedure UpdateOldList;
  protected
    procedure SetColors(Default: Longbool); override;
  public
    procedure LoadAutoList(AutoList: TAutoList);
  end;

procedure MultiKeySelect(MainClass: TObject; const TabbedRecipients: String; AutoList: TAutoList;
			 AddressList: TRecipientsData; SMimeReadWrite: Longbool);
procedure SetOptions(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool);
function  AddKeyring(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;
function  AddAddressBook(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;
function  AddNewAddresses(MainClass: TObject; const KIDD: TKeyIDData;
			  AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;


implementation

uses
  MessageDialogs,
  SearchDialog;

{$R *.DFM}

procedure MultiKeySelect(MainClass: TObject; const TabbedRecipients: String; AutoList: TAutoList;
			 AddressList: TRecipientsData; SMimeReadWrite: Longbool);
var
  frmAutoListDlg: TAutoListDialog;
  alEnabled: TAutoList;
  iLen, iBeg, iEnd, iFlags: Integer;
  sRecipient: String;
  aliNew: TALItem;
begin
  if TabbedRecipients <> E then begin
    frmAutoListDlg := TAutoListDialog.Create(MainClass);
    if frmAutoListDlg <> nil then with frmAutoListDlg do begin
      try
	AutoList.LoadList;
	fbFind.Visible := false;
	fbDelete.Visible := false;
	SMimeEnabled := SMimeReadWrite;
	DialogOption := doMultiKeySelect;
	alEnabled := AutoList.EnabledItemListCreate;
	if alEnabled <> nil then begin
	  try
	    iLen := Length(TabbedRecipients);
	    iBeg := 1;
	    repeat
	      NewList.Clear;
	      iEnd := NextCharPos(TAB, TabbedRecipients, iBeg);
	      if iEnd = 0 then iEnd := succ(iLen);
	      sRecipient := Copy(TabbedRecipients, iBeg, iEnd - iBeg);
	      alEnabled.AddOccurencesToNewList(sRecipient, PGPInterface.KeyPropsList, NewList);
	      case NewList.Count of
		0: with aliNew do begin
		  UserID := sRecipient;
		  HexID := HEX_ID_NULL;
		  ALFlags := 0;
		  ALObject := nil;
		  with AddressList do begin
		    UnknownKeys := UnknownKeys + 1;
		    AddListItem(aliNew);
		  end;
		end;
		1: AddressList.AddListItem(NewList.Items[0]);
	      else
		LoadAutoList(NewList);
		lblSelect.Caption := sRecipient;
		if ShowModal = mrOk then
		  with dgAddresses do AddressList.AddListItem(NewList.Items[Row - FixedRows])
		else begin
		  AddressList.Clear;
		  Exit;
		end;
	      end;
	      with AddressList do if Count > 0 then begin
		iFlags := Items[pred(Count)].ALFlags;
		if not MixOptions and (Count > 1) then begin
		  MixOptions := (iFlags and AUTO_FLAG_ENCRYPT) <> (DefaultFlags and AUTO_FLAG_ENCRYPT);
		end;
		if Count <= AddressCounts.ToRecipients then DefaultFlags := DefaultFlags or iFlags;
	      end;
	      iBeg := succ(iEnd);
	    until iEnd > iLen;
	  finally
	    alEnabled.Free;
	  end;
	end;
      finally
	Release;
      end;
    end;
  end;
end;

procedure SetOptions(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool);
var
  frmAutoListDlg: TAutoListDialog;
begin
  frmAutoListDlg := TAutoListDialog.Create(MainClass);
  if frmAutoListDlg <> nil then with frmAutoListDlg do begin
    try
      AutoList.LoadList;
      if AutoList.Count > 0 then begin
	SMimeEnabled := SMimeReadWrite;
	DialogOption := doSetOptions;
	OldList := AutoList;
	LoadAutoList(AutoList);
	ShowModal;
      end
      else Info(PMPGPStrings[NO_AUTOLIST_ENTRIES]);
    finally
      Release;
    end;
  end;
end;

function AddKeyring(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;
var
  frmAutoListDlg: TAutoListDialog;
  iIndex, iItem, iUser: Integer;
  ALItem: TALItem;
begin
  Result := false;
  frmAutoListDlg := TAutoListDialog.Create(MainClass);
  if frmAutoListDlg <> nil then with frmAutoListDlg do begin
    try
      AutoList.LoadList;
      with ALItem do begin
	ALObject := nil;
	ALFlags := AutoList.DefaultFlags;
      end;
      with PGPInterface.KeyPropsList do for iIndex := 0 to pred(Count) do begin
	with ALItem do begin
	  HexID := Strings[iIndex];
	  UserID := KeyProps[iIndex].kUserID;
	end;
	if AutoList.ListItemExists(ALItem, iItem) then with KeyProps[iIndex].kUserIDList do begin
	  for iUser := 0 to pred(Count) do begin
	    ALItem.UserID := Strings[iUser];
	    if not AutoList.ListItemExists(ALItem, iItem) then NewList.AddListItem(ALItem);
	  end;
	end
	else NewList.AddListItem(ALItem);
      end;
      if NewList.Count = 0 then begin
	Info(PMPGPStrings[NO_NEW_ADDRESSES_INFO]);
	Exit;
      end;
      SMimeEnabled := SMimeReadWrite;
      DialogOption := doAddKeyIDs;
      OldList := AutoList;
      LoadAutoList(NewList);
      ShowModal;
      Result := (NewCount > 0);
    finally
      Release;
    end;
  end;
end;

function AddAddressBook(MainClass: TObject; AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;
var
  frmAutoListDlg: TAutoListDialog;
begin
  Result := false;
  frmAutoListDlg := TAutoListDialog.Create(MainClass);
  if frmAutoListDlg <> nil then with frmAutoListDlg do begin
    try
      AutoList.LoadList;
      with AutoList.AddressBookList do begin
	if AddBookIndex = Count then
	  NewList.Text := AllAddresses
	else NewList.Text := AddressList[AddBookIndex];
      end;
      if NewList.Count = 0 then begin
	Info(PMPGPStrings[NO_NEW_ADDRESSES_INFO]);
	Exit;
      end;
      SMimeEnabled := SMimeReadWrite;
      DialogOption := doAddAddresses;
      OldList := AutoList;
      LoadAutoList(NewList);
      ShowModal;
      Result := (NewCount > 0);
    finally
      Release;
    end;
  end;
end;

function AddNewAddresses(MainClass: TObject; const KIDD: TKeyIDData;
			 AutoList: TAutoList; SMimeReadWrite: Longbool): Longbool;
var
  frmAutoListDlg: TAutoListDialog;
begin
  Result := false;
  frmAutoListDlg := TAutoListDialog.Create(MainClass);
  if frmAutoListDlg <> nil then with frmAutoListDlg do begin
    try
      AutoList.LoadList;
      AutoList.AddKeyIDDataToNewList(KIDD, PGPInterface.KeyPropsList, NewList);
      if NewList.Count = 0 then begin
	Info(PMPGPStrings[NO_NEW_ADDRESSES_INFO]);
	Exit;
      end;
      SMimeEnabled := SMimeReadWrite;
      if KIDD.HexIDCommaList = E then
	DialogOption := doAddAddresses
      else DialogOption := doAddKeyIDs;
      OldList := AutoList;
      LoadAutoList(NewList);
      ShowModal;
      Result := (NewCount > 0);
    finally
      Release;
    end;
  end;
end;

function EditWordBreakProc(lpch: PChar; ichCurrent, cch, code: Integer): Integer; stdcall;
const
  DELIMITERS = [#32..#47, #58..#64, #91..#95, #123..#137, #139, #145..#153, #155, #160..#191, #215, #247];
var
  sAnsi: String;
begin
  Result := ichCurrent;
  sAnsi := WideCharToString(PWideChar(lpch));
  if sAnsi = E then sAnsi := lpch;
  case code of
    WB_LEFT: begin
      inc(Result);
      if (GetKeyState(VK_LEFT) and $80) <> 0 then begin
	while (Result > 1) and (sAnsi[pred(Result)] in DELIMITERS) do dec(Result);
	while (Result > 1) and not (sAnsi[pred(Result)] in DELIMITERS) do dec(Result);
      end
      else begin
	if sAnsi[Result] in DELIMITERS then
	  while (Result > 1) and (sAnsi[pred(Result)] in DELIMITERS) do dec(Result)
	else while (Result > 1) and not (sAnsi[pred(Result)] in DELIMITERS) do dec(Result);
      end;
      dec(Result);
    end;
    WB_RIGHT: begin
      inc(Result);
      if (GetKeyState(VK_RIGHT) and $80) <> 0 then begin
	if not (sAnsi[pred(Result)] in DELIMITERS) then begin
	  while (Result <= cch) and not (sAnsi[Result] in DELIMITERS) do inc(Result);
	end;
	while (Result <= cch) and (sAnsi[Result] in DELIMITERS) do inc(Result);
      end
      else begin
	if not (sAnsi[pred(Result)] in DELIMITERS) then begin
	  while (Result <= cch) and not (sAnsi[Result] in DELIMITERS) do inc(Result);
	end
	else while (Result <= cch) and (sAnsi[Result] in DELIMITERS) do inc(Result);
      end;
      dec(Result);
    end;
    WB_ISDELIMITER: Result := ord(lpch[ichCurrent] in DELIMITERS);
  end;
end;

procedure TAutoListDialog.FormCreate(Sender: TObject);
var
  iHeightAdjustment: Integer;
begin
  NewList := TAutoList.Create(E);

  PGPInterface.GetEncryptKeys;

  lblSelect.Caption := PMPGPStrings[OPTIONS_SWITCH_LABEL];

  fbOk.Hint := PMPGPStrings[ACCEPT_HINT];
  fbFind.Hint := PMPGPStrings[OPTIONS_FIND_HINT];
  fbHelp.Hint := PMPGPStrings[OPTIONS_HELP_HINT];
  fbDelete.Hint := PMPGPStrings[OPTIONS_DELETE_HINT];
  fbCancel.Hint := PMPGPStrings[CANCEL_HINT];

  fbOk.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_OK);
  fbHelp.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_HELP);
  fbCancel.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CANCEL);

  with dgAddresses do iHeightAdjustment := (DefaultRowHeight + GridLineWidth) * 12 - (GridLineWidth * 2) - Height;
  with pnGrid do Height := Height + iHeightAdjustment;
  Height := Height + iHeightAdjustment;
  dgAddresses.HelpContext := ord(IDH_AUTOLIST_OPTIONS_DLG);
  cbxHexIDs.HelpContext := ord(IDH_AUTOLIST_OPTIONS_DLG);
  edUserID.HelpContext := ord(IDH_AUTOLIST_OPTIONS_DLG);
  with dgAddresses do begin
    SearchColumn := USER_ID_COL;
    GetSearchText := GridSearchText;
  end;
end;

procedure TAutoListDialog.FormShow(Sender: TObject);
begin
  SendMessage(edUserID.Handle, EM_SETWORDBREAKPROC, 0, Longint(@EditWordBreakProc));
  ActiveControl := dgAddresses;
  with dgAddresses do begin
    LoadArrowImages(IDW_SORT_ARROWS);
    SetImageProperties(succ(DISABLED_COL), DefaultColWidth, DefaultRowHeight);
    SetColumnData(ADD_ENTRY_COL, taCenter, taCenter, OPT_ENABLED, ADD_ENTRY_BITMAP);
    SetColumnData(ENCRYPT_COL, taCenter, taCenter, OPT_ENABLED, ENCRYPT_BITMAP);
    SetColumnData(SIGN_COL, taCenter, taCenter, OPT_ENABLED, SIGN_BITMAP);
    SetColumnData(ASK_COL, taCenter, taCenter, OPT_ENABLED, ASK_BITMAP);
    SetColumnData(PMIME_COL, taCenter, taCenter, OPT_ENABLED, PMIME_BITMAP);
    SetColumnData(SMIME_COL, taCenter, taCenter, OPT_ENABLED, SMIME_BITMAP);
    SetColumnData(DISABLED_COL, taCenter, taCenter, OPT_ENABLED, DISABLED_BITMAP);
    SetColumnData(ALGORITHM_COL, taCenter, taCenter, 'ALG', 0);
    SetColumnData(HEX_ID_COL, taCenter, taLeftJustify, 'ID', 0);
    SetColumnData(USER_ID_COL, taLeftJustify, taLeftJustify, 'ADR', 0);
  end;
  with dgAddresses do begin
    ObjectColumn := HEX_ID_COL;
    ToolTip.HintPause := 1500;
    if not SMimeEnabled then ColWidths[SMIME_COL] := 0;
    case DialogOption of
      doMultiKeySelect: begin
	AllowHeaderClick := false;
	GridLineWidth := 0;
	SpaceBarMoves := sbmNone;
	HelpContext := ord(IDH_MULTI_ID_DLG);
	ColWidths[ADD_ENTRY_COL] := 0;
	ColWidths[DISABLED_COL] := 0;
	ColWidths[USER_ID_COL] := dgAddresses.Width - ColWidths[ENCRYPT_COL] - ColWidths[SIGN_COL] -
				  ColWidths[ASK_COL] - ColWidths[PMIME_COL] - ColWidths[SMIME_COL] -
				  ColWidths[ALGORITHM_COL] - ColWidths[HEX_ID_COL] -
				  ord(BorderStyle = bsSingle) * 4;
	Options := Options - [goFixedVertLine, goVertLine, goHorzLine, goColSizing] + [goRowSelect];
	Caption := PMPGPStrings[SELECT_KEY_FOR_RECIPIENT_DIALOG];
      end;
      doSetOptions: begin
	SpaceBarMoves := sbmVertical;
	ColWidths[ADD_ENTRY_COL] := 0;
	StartList := SortRows(HEX_ID_COL, true, false);
	Caption := PMPGPStrings[SET_OPTIONS_DIALOG];
      end;
      doAddKeyIDs: begin
	SortRows(USER_ID_COL, false, false);
	Caption := PMPGPStrings[ADD_KEYRING_DIALOG];
      end;
      doAddAddresses: begin
	SortRows(USER_ID_COL, false, false);
	Caption := PMPGPStrings[ADD_ADDRESS_DIALOG];
      end;
    end;
    TopRow := FixedRows;
    if ColWidths[ADD_ENTRY_COL] = 0 then LeftCol := ENCRYPT_COL;
    if DialogOption = doMultiKeySelect then
      Selection := TGridRect(Rect(LeftCol, FixedRows, USER_ID_COL, FixedRows))
    else Selection := TGridRect(Rect(LeftCol, FixedRows, LeftCol, FixedRows));
  end;
  if (DialogOption <> doSetOptions) or (OldList.Count > 0) then LoadHexIDBoxEntries;
end;

procedure TAutoListDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key in [VK_RETURN, VK_ESCAPE])
  and not (dgAddresses.ToolTip.Showing or pnToolTips.Visible or edUserID.Visible) then begin
    if (DialogOption <> doMultiKeySelect) and DataChanged then
      ModalResult := mrIgnore
    else begin
      case Key of
	VK_RETURN: ModalResult := mrOk;
	VK_ESCAPE: ModalResult := mrCancel;
      end;
    end;
  end;
end;

procedure TAutoListDialog.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if not dgAddresses.Searching and (DialogOption <> doMultiKeySelect) and (ModalResult <> mrCancel) then begin
    if DataChanged then begin
      if ModalResult = mrIgnore then ModalResult := DefYesCancel(PMPGPStrings[SAVE_CONFIRMATION]);
      case ModalResult of
	mrCancel: CanClose := false;
	mrYes: begin
	  try
	    UpdateOldList;
	  except
	    Error(PMPGPStrings[SAVE_ERROR]);
	  end;
	end;
      end;
    end
    else if (ModalResult in [mrOk, mrYes]) and (DialogOption > doSetOptions) then begin
      Info(PMPGPStrings[SELECT_OR_CANCEL_INFO]);
      CanClose := false;
    end;
  end;
end;

procedure TAutoListDialog.FormDestroy(Sender: TObject);
begin
  PGPInterface.KeyPropsList.Clear;
  NewList.Free;
end;

procedure TAutoListDialog.dgAddressesDblClick(Sender: TObject);
var
  ptMouse: TPoint;
  iCol, iRow: Longint;
begin
  with dgAddresses do begin
    GetCursorPos(ptMouse);
    ptMouse := ScreenToClient(ptMouse);
    with ptMouse do MouseToCell(X, Y, iCol, iRow);
    if iRow > 0 then begin
      if DialogOption = doMultiKeySelect then
	ModalResult := mrOk
      else if not Searching and (Cells[USER_ID_COL, Row] <> E) then begin
	if (DialogOption = doAddKeyIDs) or not (Col in [HEX_ID_COL, USER_ID_COL]) then
	  SwitchCell
	else begin
	  case Col of
	    HEX_ID_COL: ShowHexIDBox(Row);
	    USER_ID_COL: ShowUserIDEdit(Row);
	  end;
	end;
      end;
    end;
  end;
end;

procedure TAutoListDialog.dgAddressesDrawCell(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
var
  rCheck: TRect;
begin
  if Row = 0 then
    dgAddresses.DrawHeaderCell(Col, Rect)
  else begin
    if Col = ADD_ENTRY_COL then begin
      if (DialogOption <> doAddAddresses) or (dgAddresses.Indices[HEX_ID_COL, Row] > -1) then
	dgAddresses.DrawCellBackground(Col, Rect, State, Editable)
      else dgAddresses.DrawCellBackground(Col, Rect, State, Color);
      dgAddresses.DrawCellImage(ADD_ENTRY_COL, Rect, State);
      if dgAddresses.Cells[Col, Row] = OPT_DISABLED then begin
	rCheck := Rect;
	InflateRect(rCheck, -4, -4);
	dgAddresses.Canvas.FillRect(rCheck);
      end;
    end
    else if Col < ALGORITHM_COL then begin
      with dgAddresses.Canvas do begin
	if DialogOption <> doMultiKeySelect then begin
	  if IsMutableOptionCell(TGridCoord(Point(Col, Row))) then
	    dgAddresses.DrawCellBackground(Col, Rect, State, Editable)
	  else dgAddresses.DrawCellBackground(Col, Rect, State, Color);
	end
	else dgAddresses.DrawCellBackground(Col, Rect, State, Color);
      end;
      if dgAddresses.Cells[Col, Row] = OPT_ENABLED then dgAddresses.DrawCellImage(Col, Rect, State);
    end
    else begin
      dgAddresses.DrawCellBackground(Col, Rect, State, Color);
      dgAddresses.DrawCellText(Col, Row, Rect, State);
    end;
    if dgAddresses.Focused and (gdSelected in State) then dgAddresses.DrawGridFocus(Rect);
  end;
end;

procedure TAutoListDialog.dgAddressesEnter(Sender: TObject);
var
  fCanSelect: Boolean;
begin
  with dgAddresses do dgAddressesSelectCell(Sender, Col, Row, fCanSelect);
end;

procedure TAutoListDialog.dgAddressesGetToolTipText(ACol, ARow: Longint; Fixed: Longbool; var Text: String);
var
  iIndex: Integer;
begin
  RemoveBoldStyleFromFont(dgAddresses.ToolTip.Font);
  if dgAddresses.RowCount > 1 then begin
    if ARow = 0 then begin
      case ACol of
	ADD_ENTRY_COL: Text := PMPGPStrings[ADD_ENTRY_COL_HINT];
	ENCRYPT_COL: Text := PMPGPStrings[ENCRYPT_COL_HINT];
	SIGN_COL: Text := PMPGPStrings[SIGN_COL_HINT];
	ASK_COL: Text := PMPGPStrings[ASK_COL_HINT];
	PMIME_COL: Text := PMPGPStrings[PMIME_COL_HINT];
	SMIME_COL: Text := PMPGPStrings[SMIME_COL_HINT];
	DISABLED_COL: Text := PMPGPStrings[DISABLED_COL_HINT];
	ALGORITHM_COL: Text := PMPGPStrings[ALGORITHM_COL_HINT];
	HEX_ID_COL: Text := PMPGPStrings[HEX_ID_COL_HINT];
	USER_ID_COL: Text := PMPGPStrings[USER_ID_COL_HINT];
      else
	Text := E;
      end;
    end
    else begin
      iIndex := -1;
      if ARow < 0 then begin
	if not pnToolTips.Showing then begin
	  Text := E;
	  Exit;
	end
	else with cbxHexIDs do if ItemIndex > -1 then iIndex := Indices[ItemIndex];
      end
      else with dgAddresses do if Cells[HEX_ID_COL, ARow] <> E then iIndex := Indices[HEX_ID_COL, ARow];
      if iIndex = -1 then begin
	RemoveBoldStyleFromFont(dgAddresses.ToolTip.Font);
	if dgAddresses.Cells[ALGORITHM_COL, ARow] = PMPGPStrings[NONE_SELECTED] then begin
	  Text := PMPGPStrings[SELECT_KEY_ID_INFO] +
		  PMPGPStrings[FOR_ADDRESS] + QU +
		  dgAddresses.Cells[USER_ID_COL, ARow] + QU;
	end
	else with PGPInterface do Text := GetInvalidKeyProperties(dgAddresses.Cells[HEX_ID_COL, ARow]);
      end
      else Text := TrimRight(PGPInterface.GetFingerprintData(iIndex, Handle, true));
    end;
  end;
end;

procedure TAutoListDialog.sgAdressesKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  iRow, iIndex: Integer;
begin
  if not dgAddresses.Searching then begin
    if (Shift = [ssCtrl]) and (chr(Key) in [chr(VK_DELETE), 'A', 'C', 'F', 'G']) then begin
      case Key of
	VK_DELETE: if DialogOption > doSetOptions then with dgAddresses do begin
	  for iRow := FixedRows to (RowCount - FixedRows) do Cells[ADD_ENTRY_COL, iRow] := OPT_DISABLED;
	  InvalidateOptionCells(RowCount);
	end;
	ord('A'): if DialogOption > doSetOptions then with dgAddresses do begin
	  for iRow := FixedRows to (RowCount - FixedRows) do if DialogOption = doAddAddresses then begin
	    if Indices[HEX_ID_COL, iRow] > -1 then Cells[ADD_ENTRY_COL, iRow] := OPT_ENABLED;
	  end
	  else Cells[ADD_ENTRY_COL, iRow] := OPT_ENABLED;
	  InvalidateOptionCells(RowCount);
	end;
	ord('C'): if DialogOption >= doSetOptions then with dgAddresses do begin
	  case Col of
	    HEX_ID_COL: begin
	      iIndex := Indices[HEX_ID_COL, Row];
	      if iIndex > -1 then WriteClipboardData(PGPInterface.KeyPropsList[iIndex], CF_TEXT, Self.Handle);
	    end;
	    USER_ID_COL: WriteClipboardData(Trim(Cells[Col, Row]), CF_TEXT, Self.Handle);
	  end;
	end;
	ord('F'): SearchNames(false);
	ord('G'): SearchNames(true);
      end;
    end
    else if DialogOption <> doMultiKeySelect then begin
      case Key of
	VK_DELETE: fbDelete.Click;
	VK_SPACE: with dgAddresses do begin
	  if Row > 0 then begin
	    if (DialogOption = doAddKeyIDs) or not (Col in [HEX_ID_COL, USER_ID_COL]) then
	      SwitchCell
	    else begin
	      case Col of
		HEX_ID_COL: ShowHexIDBox(Row);
		USER_ID_COL: ShowUserIDEdit(Row);
	      end;
	      Key := 0;
	    end;
	  end;
	end;
	VK_F2: if DialogOption <> doAddKeyIDs then begin
	  with dgAddresses do begin
	    if Row > 0 then begin
	      case Col of
		HEX_ID_COL: ShowHexIDBox(Row);
		USER_ID_COL: ShowUserIDEdit(Row);
	      end;
	    end;
	  end;
	end;
	VK_F3: SearchNames(true);
      end;
    end;
  end;
end;

procedure TAutoListDialog.dgAddressesSelectCell(Sender: TObject; Col, Row: Longint; var CanSelect: Boolean);
begin
  if (DialogOption <= doSetOptions) and (Col = ADD_ENTRY_COL) then begin
    if DialogOption = doMultiKeySelect then
      dgAddresses.Selection := TGridRect(Rect(ENCRYPT_COL, Row, USER_ID_COL, Row))
    else dgAddresses.Selection := TGridRect(Rect(ENCRYPT_COL, Row, ENCRYPT_COL, Row));
    dgAddresses.LeftCol := ENCRYPT_COL;
    CanSelect := false;
    with dgAddresses do with CellRect(LeftCol, Row) do if (Top = 0) and (Bottom = 0) then TopRow := Row;
  end
  else if DialogOption = doAddAddresses then lblSelect.Caption := '=> ' + dgAddresses.Cells[USER_ID_COL, Row];
  if (DialogOption = doMultiKeySelect) and (dgAddresses.Row = Row) then CanSelect := false;
end;

procedure TAutoListDialog.dgAddressesTopLeftChanged(Sender: TObject);
begin
  with dgAddresses do if (DialogOption <= doSetOptions) and (LeftCol = ADD_ENTRY_COL) then LeftCol := ENCRYPT_COL;
end;

procedure TAutoListDialog.pnSizeFrameResize(Sender: TObject);
begin
  pnSize.Width := pred(pnSizeFrame.Width);
end;

procedure TAutoListDialog.pnHexIDsResize(Sender: TObject);
begin
  cbxHexIDs.Width := pnHexIDs.Width;
end;

procedure TAutoListDialog.cbxHexIDsEnter(Sender: TObject);
var
  iItem, iIndex: Integer;
begin
  with dgAddresses do iItem := Indices[HEX_ID_COL, Row];
  if iItem > -1 then begin
    with cbxHexIDs do for iIndex := 0 to pred(Items.Count) do begin
      if Indices[iIndex] = iItem then begin
	ItemIndex := iIndex;
	Break;
      end;
    end;
  end
  else cbxHexIDs.ItemIndex := 0;
end;

procedure TAutoListDialog.cbxHexIDsExit(Sender: TObject);
var
  rRedraw: TRect;
begin
  pnToolTips.Hide;
  pnUserID.Caption := E;
  pnAlgorithm.Caption := E;
  cbxHexIDs.ItemIndex := -1;
  with dgAddresses do begin
    rRedraw := dgAddresses.ClientRect;
    InvalidateRect(Handle, @rRedraw, false);
  end;
end;

procedure TAutoListDialog.cbxHexIDsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  iRow, iIndex: Integer;
begin
  with dgAddresses.ToolTip do begin
    if Showing and (Key = VK_ESCAPE) then Key := 0;
    AnsiText := E;
  end;
  case Key of
    VK_RETURN: begin
      with dgAddresses, cbxHexIDs do if ItemIndex > -1 then begin
	iRow := Row;
	Cells[ALGORITHM_COL, iRow] := pnAlgorithm.Caption;
	if (DialogOption <> doAddAddresses) or (ItemIndex > 0) then begin
	  Cells[HEX_ID_COL, iRow] := PGPInterface.KeyPropsList.KeyProps[cbxHexIDs.Indices[ItemIndex]].kHexID;
	  dgAddresses.Indices[HEX_ID_COL, iRow] := cbxHexIDs.Indices[ItemIndex];
	  Cells[ADD_ENTRY_COL, iRow] := OPT_ENABLED;
	end
	else begin
	  Cells[HEX_ID_COL, iRow] := PMPGPStrings[SELECT_KEY_ID];
	  dgAddresses.Indices[HEX_ID_COL, iRow] := -1;
	  Cells[ADD_ENTRY_COL, iRow] := OPT_DISABLED;
	end;
      end;
      RemoveKeyMessage;
      pnToolTips.Hide;
      with dgAddresses do begin
	SetFocus;
	if DialogOption = doAddAddresses then
	  LeftCol := ADD_ENTRY_COL
	else LeftCol := ENCRYPT_COL;
	InvalidateOptionCells(RowCount);
      end;
    end;
    VK_ESCAPE: if not cbxHexIDs.ListDropped then begin
      RemoveKeyMessage;
      pnToolTips.Hide;
      with dgAddresses do begin
	SetFocus;
	if DialogOption = doAddAddresses then
	  LeftCol := ADD_ENTRY_COL
	else LeftCol := ENCRYPT_COL;
	InvalidateOptionCells(Row);
      end;
    end;
  else
    if (Shift = []) and (Key > VK_SPACE) then with cbxHexIDs do begin
      if SearchList[succ(ItemIndex)] > chr(Key) then
	iIndex := NextCharPos(chr(Key), SearchList, 1)
      else iIndex := NextCharPos(chr(Key), SearchList, ItemIndex + 2);
      if iIndex > 0 then ItemIndex := pred(iIndex);
    end;
  end;
end;

procedure TAutoListDialog.cbxHexIDsListItemSelected(Sender: TObject; Index: Integer);
begin
  with cbxHexIDs do if Index > -1 then begin
    if (DialogOption = doAddAddresses) and (Index = 0) then begin
      pnSize.Caption := E;
      pnAlgorithm.Caption := PMPGPStrings[NONE_SELECTED];
      pnUserID.Caption := PMPGPStrings[SELECT_KEY_ID_INFO];
    end
    else with PGPInterface.KeyPropsList.KeyProps[Indices[Index]] do begin
      pnSize.Caption := Copy(kSize, 1, pred(FirstCharPos(SP, kSize)));
      case kKeyAlgorithm of
	KeyAlgorithm_RSA..KeyAlgorithm_RSASignOnly: pnAlgorithm.Caption := RSA;
	KeyAlgorithm_DH..KeyAlgorithm_DHDSS: pnAlgorithm.Caption := DH_DSS;
      else
	pnAlgorithm.Caption := PMPGPStrings[INVALID_ALGORITHM];
      end;
      pnUserID.Caption := SP + kUserID;
    end;
  end;
end;

procedure TAutoListDialog.edUserIDEnter(Sender: TObject);
begin
  with dgAddresses do edUserID.Text := Cells[USER_ID_COL, Row];
end;

procedure TAutoListDialog.edUserIDExit(Sender: TObject);
begin
  edUserID.Hide;
end;

procedure TAutoListDialog.edUserIDKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN: begin
      RemoveKeyMessage;
      edUserID.Hide;
      dgAddresses.SetFocus;
      with dgAddresses, edUserID do if Text <> E then Cells[USER_ID_COL, Row] := Text;
    end;
    VK_ESCAPE: begin
      RemoveKeyMessage;
      edUserID.Hide;
      dgAddresses.SetFocus;
    end;
  end;
end;

procedure TAutoListDialog.fbFindClick(Sender: TObject);
begin
  SearchNames(false);
end;

procedure TAutoListDialog.fbHelpClick(Sender: TObject);
begin
  with dgAddresses do if not Searching then ShowHelp(Self, HelpContext, false);
end;

procedure TAutoListDialog.fbDeleteClick(Sender: TObject);
var
  sConfirm: String;
begin
  with dgAddresses do begin
    if (DialogOption <= doSetOptions) and not Searching then begin
      if Cells[USER_ID_COL, pred(RowCount)] <> E then begin
	sConfirm := PMPGPStrings[DELETE_ADDRESS_CONFIRMATION];
	if Confirm(Format(sConfirm, [Cells[USER_ID_COL, Row]])) then DeleteRow(Row);
      end;
      SetFocus;
    end
    else Beep;
  end;
end;

procedure TAutoListDialog.LoadAutoList(AutoList: TAutoList);
var
  iIndex, iRow, iKey: Integer;
begin
  with dgAddresses, AutoList do begin
    RowCount := FixedRows + Count;
    for iIndex := 0 to pred(Count) do begin
      iRow := FixedRows + iIndex;
      Rows[iRow].Text := ExtractListEntry(Items[iIndex], iKey);
      if DialogOption = doMultiKeySelect then dgAddresses.Objects[USER_ID_COL, iRow] := Objects[iIndex];
      dgAddresses.Indices[HEX_ID_COL, iRow] := iKey;
      AdjustColWidths(ColCount, iRow);
    end;
  end;
end;

procedure TAutoListDialog.SetColors(Default: Longbool);
begin
  inherited;
  Editable := AWindowColors[Default];
  dgAddresses.Color := AWindowColors[Default];
end;

function TAutoListDialog.DataChanged: Longbool;
var
  iRow: Integer;
begin
  Result := false;
  if DialogOption = doSetOptions then
    Result := (AnsiCompareText(dgAddresses.SortRows(HEX_ID_COL, true, true), StartList) <> 0)
  else if DialogOption > doSetOptions then with dgAddresses do begin
    for iRow := FixedRows to pred(RowCount) do begin
      if Cells[ADD_ENTRY_COL, iRow] = OPT_ENABLED then begin
	Result := true;
	Break;
      end;
    end;
  end;
end;

function TAutoListDialog.ExtractEntryOptions(Flags: Integer): String;
const
  ALL_DISABLED = OPT_DISABLED + LF + OPT_DISABLED + LF +
		 OPT_DISABLED + LF + OPT_DISABLED + LF +
		 OPT_DISABLED + LF + OPT_DISABLED + LF +
		 OPT_DISABLED + LF;
var
  pResult: PChar;
begin
  SetString(Result, PChar(ALL_DISABLED), Length(ALL_DISABLED));
  pResult := Pointer(Result);
  if (Flags and AUTO_FLAG_ENCRYPT) <> 0 then pResult[ENCRYPT_COL * 2] := OPT_ENABLED;
  if (Flags and AUTO_FLAG_SIGN) <> 0 then pResult[SIGN_COL * 2] := OPT_ENABLED;
  if (Flags and AUTO_FLAG_ASK) <> 0 then pResult[ASK_COL * 2] := OPT_ENABLED;
  if (Flags and AUTO_FLAG_PMIME) <> 0 then pResult[PMIME_COL * 2] := OPT_ENABLED;
  if (Flags and AUTO_FLAG_SMIME) <> 0 then pResult[SMIME_COL * 2] := OPT_ENABLED;
  if (Flags and AUTO_FLAG_DISABLED) <> 0 then pResult[DISABLED_COL * 2] := OPT_ENABLED;
end;

function TAutoListDialog.ExtractListEntry(const Item: TALItem; var KeyIndex: Integer): String;
var
  sAlgorithm, sHexID, sUserID: String;
  iALFlags: Integer;
begin
  KeyIndex := -1;
  if DialogOption < doAddAddresses then with PGPInterface do begin
    sHexID := Item.HexID;
    sUserID := Item.UserID;
    iALFlags := Item.ALFlags;
    KeyIndex := KeyPropsList.IndexOf(sHexID);
    if KeyIndex > -1 then with KeyPropsList.KeyProps[KeyIndex] do begin
      if (DialogOption > doSetOptions) and (kX509Cert in [StdCert..RootCert]) then begin
	iALFlags := Item.ALFlags or AUTO_FLAG_SMIME
      end;
      case kKeyAlgorithm of
	KeyAlgorithm_RSA..KeyAlgorithm_RSASignOnly: sAlgorithm := RSA;
	KeyAlgorithm_DH..KeyAlgorithm_DHDSS: sAlgorithm := DH_DSS;
      else
	sAlgorithm := PMPGPStrings[INVALID_ALGORITHM];
      end;
      sHexID := kHexID;
    end
    else begin
      sAlgorithm := PMPGPStrings[MISSING_KEY_INFO];
      sHexID := PGPInterface.GetShortHexID(sHexID);
    end;
    Result := ExtractEntryOptions(iALFlags) + sAlgorithm + LF + sHexID + LF + sUserID;
  end
  else begin
    sAlgorithm := PMPGPStrings[NONE_SELECTED];
    sHexID := PMPGPStrings[SELECT_KEY_ID];
    sUserID := Item.UserID;
    Result := ExtractEntryOptions(OldList.DefaultFlags) + sAlgorithm + LF + sHexID + LF + sUserID;
  end;
end;

procedure TAutoListDialog.InvalidateOptionCells(ARow: Longint);
var
  rInvalidate: TRect;
begin
  with dgAddresses do begin
    if ARow < RowCount then begin
      rInvalidate.TopLeft := CellRect(LeftCol, Col).TopLeft;
      rInvalidate.BottomRight := CellRect(DISABLED_COL, Col).BottomRight;
    end
    else begin
      rInvalidate.TopLeft := CellRect(LeftCol, TopRow).TopLeft;
      rInvalidate.BottomRight := CellRect(DISABLED_COL, BottomRow).BottomRight;
    end;
    InvalidateRect(Handle, @rInvalidate, false);
  end;
end;

function TAutoListDialog.IsMutableOptionCell(const Selected: TGridCoord): Longbool;
var
  kpX509Cert: TX509Type;
  iIndex: Integer;
begin
  Result := false;
  with dgAddresses, Selected do begin
    kpX509Cert := NoCert;
    iIndex := Indices[HEX_ID_COL, Y];
    if iIndex > -1 then kpX509Cert  :=  PGPInterface.KeyPropsList.KeyProps[iIndex].kX509Cert;
    if (DialogOption > doSetOptions) and (X <> ADD_ENTRY_COL) and (Cells[ADD_ENTRY_COL, Y] = OPT_DISABLED) then Exit;
    if (X in [ADD_ENTRY_COL, DISABLED_COL]) or (Cells[DISABLED_COL, Y] = OPT_DISABLED) then begin
      if ((X <> PMIME_COL) or (Cells[SMIME_COL, Y] <> OPT_ENABLED))
      and ((X <> SMIME_COL) or (kpX509Cert > BadCert)) then begin
	Result := true;
      end;
    end;
  end;
end;

procedure TAutoListDialog.LoadHexIDBoxEntries;
var
  iIndex: Integer;
  pSearchList: PChar;
begin
  // not in Object Inspector!
  cbxHexIDs.Sorted := false;
  with PGPInterface.KeyPropsList do begin
    if DialogOption = doAddAddresses then begin
      cbxHexIDs.Items.AddObject(SP + PMPGPStrings[SELECT_KEY_ID], Ptr(-1));
      SetString(SearchList, nil, succ(Count));
      pSearchList := Pointer(SearchList);
      pSearchList^ := SP;
      inc(pSearchList);
    end
    else begin
      SetString(SearchList, nil, Count);
      pSearchList := Pointer(SearchList);
    end;
    for iIndex := 0 to pred(Count) do with KeyProps[iIndex] do begin
      if cbxHexIDs.Items.AddObject(SP + kHexID, Ptr(iIndex)) > -1 then begin
	if kUserID <> E then
	  pSearchList[iIndex] := kUserID[1]
	else pSearchList[iIndex] := SP;
      end;
    end;
    SearchList := AnsiUpperCase(SearchList);
  end;
end;

function TAutoListDialog.RetrieveEntryOptions(ARow: Longint): Integer;
begin
  with dgAddresses do begin
    Result := (
		(ord(Cells[ENCRYPT_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_ENCRYPT) or
		(ord(Cells[SIGN_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_SIGN) or
		(ord(Cells[ASK_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_ASK) or
		(ord(Cells[PMIME_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_PMIME) or
		(ord(Cells[SMIME_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_SMIME) or
		(ord(Cells[DISABLED_COL, ARow] = OPT_ENABLED) * AUTO_FLAG_DISABLED)
	      );
  end;
end;

function TAutoListDialog.RetrieveListEntry(ARow: Longint): String;
var
  iIndex: Integer;
  NewItem: TALItem;
begin
  iIndex := dgAddresses.Indices[HEX_ID_COL, ARow];
  if iIndex > -1 then with NewItem do begin
    UserID := dgAddresses.Cells[USER_ID_COL, ARow];
    HexID := PGPInterface.KeyPropsList[iIndex];
    ALFlags := RetrieveEntryOptions(ARow);
    ALObject := Ptr(iIndex);
    NewList.AddListItem(NewItem);
  end
end;

procedure TAutoListDialog.SearchNames(Again: Longbool);
begin
  with dgAddresses do begin
    if (DialogOption <> doMultiKeySelect) and not Searching then begin
      ReSearch := Again;
      if not SearchGrid then
	Info(PMPGPStrings[TEXT_NOT_FOUND_INFO])
      else if ReSearch then begin
	if Confirm(PMPGPStrings[TEXT_NOT_FOUND_INFO] + PMPGPStrings[RESTART_SEARCH_CONFIRMATION]) then begin
	  Selection := TGridRect(Rect(LeftCol, FixedRows, ENCRYPT_COL, FixedRows));
	  PostMessage(Handle, WM_KEYDOWN, VK_F3, 0);
	end;
      end;
    end;
  end;
end;

procedure TAutoListDialog.ShowHexIDBox(ARow: Longint);

  function RowWidth: Integer;
  var
    iIndex: Integer;
  begin
    with dgAddresses do begin
      Result := 0;
      for iIndex := 0 to pred(ColCount) do inc(Result, ColWidths[iIndex] + GridLineWidth);
    end;
  end;

var
  iGridLineWidth, iBorderWidth: Integer;
  iScrollBarWidth, iScrollBarHeight: Integer;
  iMaxGridWidth: Integer;
  rToolTips: TRect;
  ptMouse: TPoint;
begin
  with dgAddresses do begin
    if RowWidth > ClientWidth then LeftCol := PMIME_COL;
    iGridLineWidth := GridLineWidth shl 1;
    iBorderWidth := ord(BorderStyle = bsSingle) shl 1;
    iScrollBarWidth := succ(GetSystemMetrics(SM_CXVSCROLL));
    iScrollBarHeight := succ(GetSystemMetrics(SM_CYHSCROLL));
    iMaxGridWidth := ClientWidth + iScrollBarWidth * ord((GetWindowLong(Handle, GWL_STYLE) and WS_VSCROLL) <> 0);
    rToolTips.TopLeft := CellRect(PMIME_COL, ARow).TopLeft;
    inc(rToolTips.Left, pred(Left + iBorderWidth));
    inc(rToolTips.Top, pred(Top + iBorderWidth));
    rToolTips.Right := ColWidths[PMIME_COL] + ColWidths[SMIME_COL] + ColWidths[DISABLED_COL] + ColWidths[ALGORITHM_COL] +
		       ColWidths[HEX_ID_COL] + ColWidths[USER_ID_COL] + succ(GridLineWidth * 6);
    if rToolTips.Right > iMaxGridWidth then rToolTips.Right := iMaxGridWidth;
    rToolTips.Bottom := RowHeights[HEX_ID_COL] + iGridLineWidth;
    with rToolTips do pnToolTips.SetBounds(Left, Top, Right, Bottom);
    pnSizeFrame.Width := ColWidths[PMIME_COL] + ColWidths[SMIME_COL] + ColWidths[DISABLED_COL] + (GridLineWidth * 3);
    pnAlgorithmFrame.Width := ColWidths[ALGORITHM_COL] + iGridLineWidth;
    with pnHexIDs do begin
      Width := ColWidths[HEX_ID_COL] + iScrollBarWidth;
      ptMouse := ClientToScreen(Point(Width - succ(iScrollBarWidth shr 1), Top + succ(iScrollBarHeight shr 1)));
    end;
    cbxHexIDs.Color := Editable;
    pnToolTips.Show;
    cbxHexIDs.SetFocus;
    with ptMouse do SetCursorPos(X, Y);
  end;
end;

procedure TAutoListDialog.ShowUserIDEdit(ARow: Longint);
var
  iGridLineWidth, iBorderWidth: Integer;
  rNameCell: TRect;
begin
  if DialogOption = doSetOptions then with dgAddresses do begin
    iGridLineWidth := GridLineWidth shl 1;
    iBorderWidth := ord(BorderStyle = bsSingle) shl 1;
    rNameCell.TopLeft := CellRect(USER_ID_COL, ARow).TopLeft;
    inc(rNameCell.Left, pred(Left + iBorderWidth));
    inc(rNameCell.Top, pred(Top + iBorderWidth));
    rNameCell.Right := ColWidths[USER_ID_COL] + iGridLineWidth;
    rNameCell.Bottom := RowHeights[ARow] + iGridLineWidth;
    with edUserID, rNameCell do begin
      SetBounds(Left, Top, Right, Bottom);
      Color := Editable;
      Show;
      SetFocus;
      SelectAll;
    end;
  end;
end;

procedure TAutoListDialog.SwitchCell;
var
  iRow, iIndex: Integer;
  cNewValue: Char;
begin
  with dgAddresses do begin
    if Col <= DISABLED_COL then begin
      if IsMutableOptionCell(TGridCoord(Point(Col, Row))) then begin
	iIndex := Indices[HEX_ID_COL, Row];
	if Cells[Col, Row] <> OPT_ENABLED then
	  cNewValue := OPT_ENABLED
	else cNewValue := OPT_DISABLED;
	if iIndex > -1 then begin
	  if (GetKeyState(VK_CONTROL) and $80) <> 0 then begin
	    iRow := Row;
	    Cells[Col, iRow] := cNewValue;
	    if (Col = SMIME_COL) and (cNewValue = OPT_ENABLED) then Cells[PMIME_COL, iRow] := OPT_DISABLED;
	    if Col in [ADD_ENTRY_COL, DISABLED_COL] then InvalidateOptionCells(iRow);
	  end
	  else for iRow := FixedRows to pred(RowCount) do begin
	    if Indices[HEX_ID_COL, iRow] = iIndex then begin
	      Cells[Col, iRow] := cNewValue;
	      if (Col = SMIME_COL) and (cNewValue = OPT_ENABLED) then Cells[PMIME_COL, iRow] := OPT_DISABLED;
	    end;
	    if Col in [ADD_ENTRY_COL, DISABLED_COL] then InvalidateOptionCells(RowCount);
	  end;
	end;
      end
      else Beep;
    end
    else Beep;
  end;
end;

procedure TAutoListDialog.UpdateOldList;
var
  iRow, iCount, iIndex: Integer;
  ALItem: TALItem;
  sUser: String;
begin
  NewList.Clear;
  with dgAddresses do NewList.Capacity := RowCount - FixedRows;
  if DialogOption = doSetOptions then begin
    with dgAddresses do for iRow := FixedRows to pred(RowCount) do RetrieveListEntry(iRow);
    with OldList do begin
      Clear;
      Capacity := NewList.Count;
      AddListItems(NewList);
      SaveList;
    end;
  end
  else if DialogOption > doSetOptions then begin
    with dgAddresses do for iRow := FixedRows to pred(RowCount) do begin
      if Cells[ADD_ENTRY_COL, iRow] = OPT_ENABLED then RetrieveListEntry(iRow);
    end;
    iCount := OldList.Count;
    if DialogOption = doAddKeyIDs then begin
      for iIndex := 0 to pred(NewList.Count) do begin
	ALItem := NewList.Items[iIndex];
	OldList.AddListItem(ALItem);
      end;
    end
    else begin
      for iIndex := 0 to pred(NewList.Count) do begin
	ALItem := NewList.Items[iIndex];
	with ALItem do begin
	  sUser := UserID;
	  UserID := ExtractName(sUser);
	  if UserID <> E then OldList.AddListItem(ALItem);
	  UserID := ExtractEmail(sUser);
	  if UserID <> E then OldList.AddListItem(ALItem);
	end;
      end;
    end;
    OldList.SaveList;
    NewCount := OldList.Count - iCount;
    Info(IntToStr(NewCount) + PMPGPStrings[LIST_ENTRIES_ADDED]);
  end;
end;


end.

