{$J+,Z4}
unit ConfigDialog;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Buttons, Mask,
  ExtCtrlsEx, TreeViewEx, PageCtrlEx, GroupBoxEx, CustBtnCtrl, CustCheck, CustRadio,
  FlatBtn, HotLabel, CBExtend, MaskEdEx, StrUtil, FileUtils, PMPGPForm, PMPGPCommon,
  PMPGPResources, KeyPropTypes, ConfigHandler, PMHandler, PGPHandler;

type
  TConfigDialog = class(TPMPGPForm)
    tvNavigation: TTreeViewEx;
    asVertical: TAlignedShape;
    pcConfiguration: TPageControlEx;
    tsPGPPaths: TTabSheet;
    hlSecring: THotLabel;
    hlPubring: THotLabel;
    hlGroups: THotLabel;
    hlKeyserver: THotLabel;
    cbxKeyserverProtocol: TPlainCBEx;
    edKeyserverName: TEditEx;
    meKeyserverPort: TMaskEdEx;
    edSecring: TEditEx;
    edPubring: TEditEx;
    edGroups: TEditEx;
    tsPGPMiscellaneous: TTabSheet;
    hlDefaultUser: THotLabel;
    hlKeyAlgorithm: THotLabel;
    shDefaultKey: TShape;
    hlSignAlgorithm: THotLabel;
    shSignAlgorithm: TShape;
    hlComment: THotLabel;
    cbxDefaultUser: TPlainCBEx;
    cbUseAlways: TCustCheckBox;
    cbxSignAlgorithm: TPlainCBEx;
    cbUseRIPEMD: TCustCheckBox;
    edComment: TEditEx;
    tsPMToolbar: TTabSheet;
    gbToolbar: TGroupBoxEx;
    hlEncryptSignOptions: THotLabel;
    cbWipePassphrase: TCustCheckBox;
    cbxEncryptSignOptions: TPlainCBEx;
    cbDecryptMessage: TCustCheckBox;
    cbVerifySignature: TCustCheckBox;
    cbManageKeys: TCustCheckBox;
    cbLaunchPGP: TCustCheckBox;
    tsPMMiscellaneous: TTabSheet;
    gbPMMiscellaneous: TGroupBoxEx;
    hlScanLines: THotLabel;
    shPMMiscellaneous: TShape;
    hlEditMargin: THotLabel;
    seScanLines: TSpinEdEx;
    fbCheckHeaderLines: TFlatBtn;
    seEditMargin: TSpinEdEx;
    pnNoChange: TPanel;
    lblNoChange: TLabel;
    tsPMPGPEncoding: TTabSheet;
    gbPMPGPEncode: TGroupBoxEx;
    cbUseAutoList: TCustCheckBox;
    cbHideKeyDialog: TCustCheckBox;
    cbEncryptToSelf: TCustCheckBox;
    cbEncryptAttachments: TCustCheckBox;
    cbConfirmEncryption: TCustCheckBox;
    cbMixOptionsWarning: TCustCheckBox;
    cbADKWarning: TCustCheckBox;
    tsPMPGPDecoding: TTabSheet;
    gbPMPGPDecode: TGroupBoxEx;
    shAutoVerify: TShape;
    cbCheckForKeyData: TCustCheckBox;
    cbKeyserverLookup: TCustCheckBox;
    cbPreDecodePSMime: TCustCheckBox;
    cbAutoVerify: TCustCheckBox;
    rbNoInfo: TCustRadioBtn;
    rbInReaderWindow: TCustRadioBtn;
    rbInExtraWindow: TCustRadioBtn;
    tsPMPGPMiscellaneous: TTabSheet;
    gbPMPGPMiscellaneous: TGroupBoxEx;
    hlSeconds: THotLabel;
    hlWipeFilePasses: THotLabel;
    hlCachePassphrase: THotLabel;
    hlDateTime: THotLabel;
    seSeconds: TSpinEdEx;
    sePasses: TSpinEdEx;
    cbStandardColors: TCustCheckBox;
    cbShowTime: TCustCheckBox;
    hlTimeServer: THotLabel;
    cbxTimeServerName: TPlainCBEx;
    meTimeServerPort: TMaskEdEx;
    cbTimeAutoSync: TCustCheckBox;
    asHorizontal: TAlignedShape;
    pnButtons: TColoredFramePanel;
    fbOk: TFlatBtn;
    fbHelp: TFlatBtn;
    fbCancel: TFlatBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure tvNavigationCollapsing(Sender: TObject; Node: TTreeNode; var AllowCollapse: Boolean);
    procedure tvNavigationChange(Sender: TObject; Node: TTreeNode);
    procedure tvNavigationKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure edFileMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure edSecringKeyPress(Sender: TObject; var Key: Char);
    procedure edSecringClick(Sender: TObject);
    procedure edPubringKeyPress(Sender: TObject; var Key: Char);
    procedure edPubringClick(Sender: TObject);
    procedure edGroupsKeyPress(Sender: TObject; var Key: Char);
    procedure edGroupsClick(Sender: TObject);
    procedure cbxDefaultUserChange(Sender: TObject);
    procedure hlKeyAlgorithmTextHover(Sender: THotLabel; Active: Longbool);
    procedure fbCheckHeaderLinesClick(Sender: TObject);
    procedure cbAutoVerifyChanged(Sender: TObject);
    procedure seSecondsChange(Sender: TObject);
    procedure cbStandardColorsChanged(Sender: TObject);
    procedure cbShowTimeChanged(Sender: TObject);
    procedure fbHelpClick(Sender: TObject);
  private
    CacheWarning: Longbool;
    NavigateUpwards: Longbool;
    RunningStandalone: Longbool;
    ToolbarChangeRestart: Longbool;
    OldOptions, NewOptions: TOptions;
    procedure LoadResources;
    procedure LoadHelpIDs;
    procedure LoadData;
    procedure ReadData;
    function  ToolbarChanged: Longbool;
    procedure GreySignatureInfoOptions(Grey: Longbool);
  protected
    procedure SetColors(Default: Longbool); override;
  public
  end;

function Configure(MainClass: TObject; Standalone: Longbool): Longbool;


implementation

{$R *.DFM}

uses
  MessageDialogs;

function Configure(MainClass: TObject; Standalone: Longbool): Longbool;
var
  frmConfigDlg: TConfigDialog;
begin
  Result := false;
  frmConfigDlg := TConfigDialog.Create(MainClass);
  if frmConfigDlg <> nil then with frmConfigDlg do begin
    try
      RunningStandalone := Standalone;
      ShowModal;
      Result := (ToolbarChangeRestart and Confirm(PMPGPStrings[RESTART_CONFIRMATION]));
    finally
      Release;
    end;
  end;
end;

procedure TConfigDialog.FormCreate(Sender: TObject);
begin
  OldOptions := TOptions(MainClass);
  NewOptions := TOptions.Create(E);
  LoadResources;
  LoadHelpIDs;
  LoadData;
end;

procedure TConfigDialog.FormShow(Sender: TObject);
begin
  if not FileExists(edSecring.Text) then
    ActiveControl := edSecring
  else if not FileExists(edPubring.Text) then
    ActiveControl := edPubring
  else ActiveControl := tvNavigation;
  with tvNavigation do Selected := Items[1];
  if not PGPInterface.SHA2Enabled then begin
    hlSignAlgorithm.Enabled := false;
    cbxSignAlgorithm.Enabled := false;
    cbxSignAlgorithm.ItemIndex := 0;
  end;
end;

procedure TConfigDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN, VK_ESCAPE: begin
      if not (cbxDefaultUser.ListDropped or cbxSignAlgorithm.ListDropped or
	      cbxKeyserverProtocol.ListDropped or cbxEncryptSignOptions.ListDropped or cbxTimeServerName.ListDropped or
	      meKeyserverPort.Focused or meTimeServerPort.Focused or seScanLines.Focused or seEditMargin.Focused or
	      seSeconds.Focused or sePasses.Focused or HelpShowing) then begin
	RemoveKeyMessage;
	ModalResult := mrIgnore;
      end;
    end;
    $FFFF: Warn(PMPGPStrings[CACHE_WARNING]);
  end;
end;

procedure TConfigDialog.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  ReadData;
  if not OldOptions.EqualConfiguration(NewOptions) and ((ModalResult <> mrCancel) or RunningStandalone) then begin
    if RunningStandalone then
      ModalResult := mrYes
    else if ModalResult = mrIgnore then ModalResult := DefYesCancel(PMPGPStrings[SAVE_CONFIRMATION]);
    if ModalResult = mrYes then begin
      try
	with OldOptions do begin
	  ToolbarChangeRestart := ToolbarChanged;
	  CopyConfiguration(NewOptions);
	  WriteConfiguration;
	end;
      except
	Error(PMPGPStrings[SAVE_ERROR]);
      end;
    end
    else if ModalResult = mrCancel then CanClose := false;
  end;
end;

procedure TConfigDialog.FormDestroy(Sender: TObject);
begin
  PGPInterface.KeyPropsList.Clear;
  NewOptions.Free;
end;

procedure TConfigDialog.tvNavigationCollapsing(Sender: TObject; Node: TTreeNode; var AllowCollapse: Boolean);
begin
  AllowCollapse := false;
end;

procedure TConfigDialog.tvNavigationChange(Sender: TObject; Node: TTreeNode);
begin
  if Node = nil then Exit;
  if Node.Level = 0 then with Node do begin
    if NavigateUpwards and (Node.AbsoluteIndex > 0) then
      tvNavigation.Selected := GetPrevSibling.GetLastChild
    else tvNavigation.Selected := GetFirstChild;
  end
  else if pcConfiguration <> nil then with pcConfiguration do begin
    ActivePage := TTabSheet(Node.Data);
    if ActivePage = tsPMMiscellaneous then with pnNoChange do begin
      if RunningStandalone then Parent := tsPMMiscellaneous;
      Visible := RunningStandalone;
    end;
  end;
end;

procedure TConfigDialog.tvNavigationKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_LEFT: Key := 0;
    VK_UP: NavigateUpwards := true;
    VK_RIGHT: Key := 0;
    VK_DOWN: NavigateUpwards := false;
  end;
end;

procedure TConfigDialog.edFileMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  TEditEx(Sender).Cursor := crHand;
end;

procedure TConfigDialog.edSecringKeyPress(Sender: TObject; var Key: Char);
var
  sSecringFile: String;
begin
  sSecringFile := OpenKeyringFile(PMPGPStrings[KEYFILE_DIALOG] + QU + DelAmp(hlSecring.Caption) + QU,
				  PGPInterface.Path, edSecring.Text, kftSecring, Self.Handle);
  if sSecringFile <> E then edSecring.Text := sSecringFile;
  Key := NULL;
end;

procedure TConfigDialog.edSecringClick(Sender: TObject);
var
  cKey: Char;
begin
  cKey := NULL;
  edSecringKeyPress(Sender, cKey);
end;

procedure TConfigDialog.edPubringKeyPress(Sender: TObject; var Key: Char);
var
  sPubringFile: String;
begin
  sPubringFile := OpenKeyringFile(PMPGPStrings[KEYFILE_DIALOG] + QU + DelAmp(hlPubring.Caption) + QU,
				  PGPInterface.Path, edPubring.Text, kftPubring, Self.Handle);
  if sPubringFile <> E then edPubring.Text := sPubringFile;
  Key := NULL;
end;

procedure TConfigDialog.edPubringClick(Sender: TObject);
var
  cKey: Char;
begin
  cKey := NULL;
  edPubringKeyPress(Sender, cKey);
end;

procedure TConfigDialog.edGroupsKeyPress(Sender: TObject; var Key: Char);
var
  sGroupsFile: String;
begin
  sGroupsFile := OpenKeyringFile(PMPGPStrings[KEYFILE_DIALOG] + QU + DelAmp(hlGroups.Caption) + QU,
				 PGPInterface.Path, edGroups.Text, kftGroups, Self.Handle);
  if sGroupsFile <> E then edGroups.Text := sGroupsFile;
  Key := NULL;
end;

procedure TConfigDialog.edGroupsClick(Sender: TObject);
var
  cKey: Char;
begin
  cKey := NULL;
  edGroupsKeyPress(Sender, cKey);
end;

procedure TConfigDialog.cbxDefaultUserChange(Sender: TObject);
begin
  with cbxDefaultUser do if ItemIndex > -1 then begin
    with PGPInterface.KeyPropsList.KeyProps[ItemIndex] do begin
      hlKeyAlgorithm.Caption := PMPGPStrings.KeyAlgorithms[kKeyAlgorithm] + SP + kHexID;
    end;
  end;
end;

procedure TConfigDialog.hlKeyAlgorithmTextHover(Sender: THotLabel; Active: Longbool);
begin
  with cbxDefaultUser do if ItemIndex > -1 then with PGPInterface.KeyPropsList do begin
    if Active then
      hlKeyAlgorithm.Caption := Strings[ItemIndex]
    else with KeyProps[ItemIndex] do hlKeyAlgorithm.Caption := PMPGPStrings.KeyAlgorithms[kKeyAlgorithm] + SP + kHexID;
  end;
end;

procedure TConfigDialog.fbCheckHeaderLinesClick(Sender: TObject);
var
  iLen, iSet: Integer;
begin
  Enabled := false;
  try
    iLen := PMInterface.MessageHeaderLineCount;
  finally
    Enabled := true;
  end;
  if iLen > seScanLines.Value then begin
    iSet := ((iLen + 9) div 10) * 10;
    if Confirm(PMPGPStrings[ESTIMATED_HEADER_LENGHT] + IntToStr(iLen) +
	       Format(PMPGPStrings[SCAN_LINES_ADJUST], [hlScanLines.Caption]) + IntToStr(iSet) + '?') then begin
      seScanLines.Value := iSet;
    end;
  end
  else if iLen > 0 then Info(PMPGPStrings[ESTIMATED_HEADER_LENGHT] + IntToStr(iLen) + PMPGPStrings[SCAN_LINES_OK]);
end;

procedure TConfigDialog.cbAutoVerifyChanged(Sender: TObject);
begin
  GreySignatureInfoOptions(not cbAutoVerify.Checked);
end;

procedure TConfigDialog.seSecondsChange(Sender: TObject);
begin
  with seSeconds, hlSeconds do begin
    if CacheWarning and (Value > -1) then begin
      CacheWarning := false;
      PostMessage(Handle, WM_KEYDOWN, $FFFF, 0);
    end;
    case Value of
      -1: Caption := PMPGPStrings[CACHE_DISABLED];
       0: Caption := PMPGPStrings[CACHE_INFINITE];
    else
      Caption := PMPGPStrings[CACHE_SECONDS];
    end;
  end;
end;

procedure TConfigDialog.cbStandardColorsChanged(Sender: TObject);
begin
  with cbStandardColors do begin
    NewOptions.StandardColors := Checked;
    SetColors(Checked);
  end;
end;

procedure TConfigDialog.cbShowTimeChanged(Sender: TObject);
begin
  with hlDateTime do begin
    case cbShowTime.State of
      cbUnchecked: Caption := E;
      cbChecked: Caption := PMPGPStrings[DATE_TIME];
      cbGrayed: Caption := PMPGPStrings[TIME_ONLY];
    end;
  end;
end;

procedure TConfigDialog.fbHelpClick(Sender: TObject);
begin
  ShowHelp(Self, HelpContext, false);
end;

procedure TConfigDialog.SetColors(Default: Longbool);
var
  clWnd: TColor;
  iIndex: Integer;
begin
  inherited;
  clWnd := AWindowColors[Default];
  with tvNavigation do begin
    Color := clWnd;
    FullExpand;
  end;
  pcConfiguration.Color := Color;
  for iIndex := 0 to pred(ComponentCount) do begin
    if (Components[iIndex] is TEditEx) then
      TEditEx(Components[iIndex]).Color := clWnd
    else if (Components[iIndex] is TMaskEdEx) then
      TMaskEdEx(Components[iIndex]).Color := clWnd
    else if (Components[iIndex] is TSpinEdEx) then
      TSpinEdEx(Components[iIndex]).Color := clWnd
    else if (Components[iIndex] is TPlainCBEx) then begin
      TPlainCBEx(Components[iIndex]).Color := clWnd;
    end;
  end;
end;

procedure TConfigDialog.LoadResources;
begin

  Caption := PMPGPStrings[CONFIG_DIALOG];

  fbOk.Hint := PMPGPStrings[ACCEPT_HINT];
  fbHelp.Hint := PMPGPStrings[CONFIG_HELP_HINT];
  fbCancel.Hint := PMPGPStrings[CANCEL_HINT];

  fbOk.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_OK);
  fbHelp.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_HELP);
  fbCancel.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CANCEL);

  with tvNavigation do begin

    with Items[1] do begin
      Text := PMPGPStrings[CONFIG_PGP_PATHS];
      Data := pcConfiguration.Pages[0];
    end;

    with Items[2] do begin
      Text := PMPGPStrings[CONFIG_MISCELLANEOUS];
      Data := pcConfiguration.Pages[1];
    end;

    with Items[4] do begin
      Text := PMPGPStrings[CONFIG_PM_TOOLBAR];
      Data := pcConfiguration.Pages[2];
    end;

    with Items[5] do begin
      Text := PMPGPStrings[CONFIG_MISCELLANEOUS];
      Data := pcConfiguration.Pages[3];
    end;

    with Items[7] do begin
      Text := PMPGPStrings[CONFIG_PMPGP_ENCODING];
      Data := pcConfiguration.Pages[4];
    end;

    with Items[8] do begin
      Text := PMPGPStrings[CONFIG_PMPGP_DECODING];
      Data := pcConfiguration.Pages[5];
    end;

    with Items[9] do begin
      Text := PMPGPStrings[CONFIG_MISCELLANEOUS];
      Data := pcConfiguration.Pages[6];
    end;

  end;

  hlKeyServer.Caption := PMPGPStrings[KEYSERVER_LABEL];
  hlSecring.Caption := PMPGPStrings[SECRING_LABEL];
  hlPubring.Caption := PMPGPStrings[PUBRING_LABEL];
  hlGroups.Caption := PMPGPStrings[GROUPS_LABEL];

  hlDefaultUser.Caption := PMPGPStrings[DEFAULT_USER_LABEL];
  cbUseAlways.Caption := PMPGPStrings[USE_ALWAYS_CHECKBOX];
  hlSignAlgorithm.Caption := PMPGPStrings[SIGN_ALGORITHM_LABEL];
  cbUseRIPEMD.Caption := PMPGPStrings[USE_RIPEMD_CHECKBOX];
  hlComment.Caption := PMPGPStrings[COMMENT_LABEL];

  cbWipePassphrase.Caption := PMPGPStrings[WIPE_PASSPHRASE_CHECKBOX];
  hlEncryptSignOptions.Caption := PMPGPStrings[ENCRYPT_SIGN_OPTIONS_LABEL];
  cbxEncryptSignOptions.Items.Text := PMPGPStrings[ENCRYPT_SIGN_OPTIONS_COMBOBOX];
  cbDecryptMessage.Caption := PMPGPStrings[DECRYPT_MESSAGE_CHECKBOX];
  cbVerifySignature.Caption := PMPGPStrings[VERIFY_SIGNATURE_CHECKBOX];
  cbManageKeys.Caption := PMPGPStrings[MANAGE_KEYS_CHECKBOX];
  cbLaunchPGP.Caption := PMPGPStrings[LAUNCH_PGP_CHECKBOX];

  lblNoChange.Caption := PMPGPStrings[NO_CHANGE_LABEL];
  hlScanLines.Caption := PMPGPStrings[SCAN_LINES_LABEL];
  fbCheckHeaderLines.Caption := LT + PMPGPStrings[CHECK_HEADER_LINES_LABEL] + GT;
  hlEditMargin.Caption := PMPGPStrings[EDIT_MARGIN_LABEL];

  cbUseAutoList.Caption := PMPGPStrings[USE_AUTOLIST_CHECKBOX];
  cbHideKeyDialog.Caption := PMPGPStrings[HIDE_KEY_DIALOG_CHECKBOX];
  cbEncryptToSelf.Caption := PMPGPStrings[ENCRYPT_TO_SELF_CHECKBOX];
  cbEncryptAttachments.Caption := PMPGPStrings[ENCRYPT_ATTACHMENTS_CHECKBOX];
  cbConfirmEncryption.Caption := PMPGPStrings[CONFIRM_ENCRYPTION_CHECKBOX];
  cbMixOptionsWarning.Caption := PMPGPStrings[MIX_OPTIONS_WARNING_CHECKBOX];
  cbADKWarning.Caption := PMPGPStrings[ADK_WARNING_CHECKBOX];

  cbCheckForKeyData.Caption := PMPGPStrings[CHECK_FOR_KEY_DATA_CHECKBOX];
  cbKeyserverLookup.Caption := PMPGPStrings[KEYSERVER_LOOKUP_CHECKBOX];
  cbPreDecodePSMime.Caption := PMPGPStrings[PRE_DECODE_PSMIME_CHECKBOX];
  cbAutoVerify.Caption := PMPGPStrings[AUTO_VERIFY_CHECKBOX];
  rbNoInfo.Caption := PMPGPStrings[NO_INFO_RADIOBOX];
  rbInReaderWindow.Caption := PMPGPStrings[IN_READER_WINDOW_RADIOBOX];
  rbInExtraWindow.Caption := PMPGPStrings[IN_EXTRA_WINDOW_RADIOBOX];

  hlCachePassphrase.Caption := PMPGPStrings[CACHE_PASSPHRASE_LABEL];
  hlWipeFilePasses.Caption := PMPGPStrings[WIPE_FILE_PASSES_LABEL];
  cbStandardColors.Caption := PMPGPStrings[STANDARD_COLORS_CHECKBOX];
  cbShowTime.Caption := PMPGPStrings[SHOW_TIME_CHECKBOX];
  hlTimeServer.Caption := PMPGPStrings[TIMESERVER_LABEL];
  cbTimeAutoSync.Caption := PMPGPStrings[AUTO_SYNC_CHECKBOX];

end;

procedure TConfigDialog.LoadHelpIDs;
begin

  HelpContext := ord(IDH_CONFIGURATION_DLG);

  tvNavigation.HelpContext := ord(IDH_CONFIGURATION_DLG);

  cbxKeyServerProtocol.HelpContext := ord(IDH_KEYSERVER);
  edKeyServerName.HelpContext := ord(IDH_KEYSERVER);
  meKeyServerPort.HelpContext := ord(IDH_KEYSERVER);
  edSecring.HelpContext := ord(IDH_SECRING_FILE);
  edPubring.HelpContext := ord(IDH_PUBRING_FILE);
  edGroups.HelpContext := ord(IDH_GROUPS_FILE);

  cbxDefaultUser.HelpContext := ord(IDH_DEFAULT_USER);
  cbUseAlways.HelpContext := ord(IDH_USE_ALWAYS);
  cbxSignAlgorithm.HelpContext := ord(IDH_USE_RIPEMD);
  cbUseRIPEMD.HelpContext := ord(IDH_USE_RIPEMD);
  edComment.HelpContext := ord(IDH_COMMENT);

  cbWipePassphrase.HelpContext := ord(IDH_MAINMENU);
  cbxEncryptSignOptions.HelpContext := ord(IDH_MAINMENU);
  cbDecryptMessage.HelpContext := ord(IDH_MAINMENU);
  cbVerifySignature.HelpContext := ord(IDH_MAINMENU);
  cbManageKeys.HelpContext := ord(IDH_MAINMENU);
  cbLaunchPGP.HelpContext := ord(IDH_MAINMENU);

  seScanLines.HelpContext := ord(IDH_SCAN_LINES);
  fbCheckHeaderLines.HelpContext := ord(IDH_SCAN_LINES);
  seEditMargin.HelpContext := ord(IDH_EDIT_MARGIN);

  cbUseAutoList.HelpContext := ord(IDH_USE_AUTOLIST);
  cbHideKeyDialog.HelpContext := ord(IDH_HIDE_KEY_DIALOG);
  cbEncryptToSelf.HelpContext := ord(IDH_ENCRYPT_TO_SELF);
  cbEncryptAttachments.HelpContext := ord(IDH_ENCRYPT_ATTACHMENTS);
  cbConfirmEncryption.HelpContext := ord(IDH_CONFIRM_ENCRYPTION);
  cbMixOptionsWarning.HelpContext := ord(IDH_MIX_OPTIONS_WARN);
  cbADKWarning.HelpContext := ord(IDH_ADK_WARN);

  cbCheckForKeyData.HelpContext := ord(IDH_CHECK_FOR_KEY_DATA);
  cbKeyserverLookup.HelpContext := ord(IDH_KEYSERVER_LOOKUP);
  cbPreDecodePSMime.HelpContext := ord(IDH_PRE_DECODE_PSMIME);
  cbAutoVerify.HelpContext := ord(IDH_AUTO_VERIFY);
  rbNoInfo.HelpContext := ord(IDH_AUTO_VERIFY);
  rbInReaderWindow.HelpContext := ord(IDH_AUTO_VERIFY);
  rbInExtraWindow.HelpContext := ord(IDH_AUTO_VERIFY);

  seSeconds.HelpContext := ord(IDH_PASSPHRASE_CACHE);
  sePasses.HelpContext := ord(IDH_WIPE_FILE_PASSES);
  cbShowTime.HelpContext := ord(IDH_SHOW_TIME);
  cbxTimeServerName.HelpContext := ord(IDH_TIME_SERVER);
  meTimeServerPort.HelpContext := ord(IDH_TIME_SERVER);
  cbTimeAutoSync.HelpContext := ord(IDH_TIME_AUTO_SYNC);

end;

procedure TConfigDialog.LoadData;
begin
  NewOptions.CopyConfiguration(OldOptions);
  PGPInterface.GetPrivateKeys(E);
  with NewOptions do begin

    with KeyServer do begin
      if Protocol > -1 then begin
	cbxKeyServerProtocol.ItemIndex := Protocol;
	edKeyServerName.Text := Name;
	meKeyServerPort.Text := Port;
      end;
    end;
    edSecring.Text := Secring;
    edPubring.Text := Pubring;
    edGroups.Text := Groups;

    with cbxDefaultUser do begin
      ItemIndex := PGPInterface.AddBasicKeyPropsToStringList(DefaultUser, Items);
      if Items.Count in [1..8] then DropDownCount := Items.Count;
      cbxDefaultUserChange(nil);
    end;
    cbUseAlways.Checked := UseAlways;
    cbxSignAlgorithm.ItemIndex := SignAlgorithm;
    cbUseRIPEMD.Checked := UseRipeMD;
    edComment.Text := Comment;

    cbWipePassphrase.Checked := WipePassphraseButton;
    cbxEncryptSignOptions.ItemIndex := Integer(EncryptSignButton);
    cbDecryptMessage.Checked := DecryptMessageButton;
    cbVerifySignature.Checked := VerifySignatureButton;
    cbManageKeys.Checked := ManageKeysButton;
    cbLaunchPGP.Checked := LaunchPGPButton;

    seScanLines.Value := PMScanLines;
    seEditMargin.Value := PMEditMargin;

    cbUseAutoList.Checked := UseAutoList;
    cbHideKeyDialog.Checked := HideKeyDialog;
    cbEncryptToSelf.Checked := EncryptToSelf;
    cbEncryptAttachments.Checked := EncryptAttachments;
    cbConfirmEncryption.Checked := ConfirmEncryption;
    cbMixOptionsWarning.Checked := MixOptionsWarning;
    cbADKWarning.Checked := ADKWarning;

    cbCheckForKeyData.Checked := CheckForKeyData;
    cbKeyserverLookup.Checked := KeyServerLookup;
    cbPreDecodePSMime.Checked := PreDecodePSMime;
    cbAutoVerify.Checked := AutoVerify;
    case InfoOption of
      vsOnStatusBarOnly: rbNoInfo.Checked := true;
      vsInReaderWindow: rbInReaderWindow.Checked := true;
      vsInExtraWindow: rbInExtraWindow.Checked := true;
    end;
    cbAutoVerifyChanged(nil);

    seSeconds.Value := PassphraseCacheSeconds;
    sePasses.Value := WipeFilePasses;
    cbStandardColors.Checked := StandardColors;
    cbShowTime.State := TCheckBoxState(ShowTime);
    with cbxTimeServerName do begin
      GetNTPServerList(Items);
      ItemIndex := Items.IndexOf(TimeServerName);
      if  ItemIndex = -1 then begin
	Items.Insert(0, TimeServerName);
	ItemIndex := 0;
      end;
    end;
    meTimeServerPort.Text := IntToStr(TimeServerPort);
    cbTimeAutoSync.Checked := TimeAutoSync;

  end;
  CacheWarning := (seSeconds.Value = -1)
end;

procedure TConfigDialog.ReadData;
begin
  with NewOptions do begin

    with KeyServer do begin
      if Protocol > -1 then begin
	Protocol := cbxKeyServerProtocol.ItemIndex;
	Name := edKeyServerName.Text;
	Port := meKeyServerPort.Text;
      end;
    end;
    Secring := edSecring.Text;
    Pubring := edPubring.Text;
    Groups := edGroups.Text;

    with cbxDefaultUser do if ItemIndex > -1 then with PGPInterface.KeyPropsList do begin
      DefaultUser := KeyProps[ItemIndex].kUserID + BS + Strings[ItemIndex];
    end;
    UseAlways := cbUseAlways.Checked;
    SignAlgorithm := cbxSignAlgorithm.ItemIndex;
    UseRipeMD := cbUseRIPEMD.Checked;
    Comment := edComment.Text;

    WipePassphraseButton := cbWipePassphrase.Checked;
    EncryptSignButton := TEncryptSignButtonOption(cbxEncryptSignOptions.ItemIndex);
    DecryptMessageButton := cbDecryptMessage.Checked;
    VerifySignatureButton := cbVerifySignature.Checked;
    ManageKeysButton := cbManageKeys.Checked;
    LaunchPGPButton := cbLaunchPGP.Checked;

    PMScanLines := seScanLines.Value;
    PMEditMargin := seEditMargin.Value;

    UseAutoList := cbUseAutoList.Checked;
    HideKeyDialog := cbHideKeyDialog.Checked;
    EncryptToSelf := cbEncryptToSelf.Checked;
    EncryptAttachments := cbEncryptAttachments.Checked;
    ConfirmEncryption := cbConfirmEncryption.Checked;
    MixOptionsWarning := cbMixOptionsWarning.Checked;
    ADKWarning := cbADKWarning.Checked;

    CheckForKeyData := cbCheckForKeyData.Checked;
    KeyServerLookup := cbKeyserverLookup.Checked;
    PreDecodePSMime := cbPreDecodePSMime.Checked;
    AutoVerify := cbAutoVerify.Checked;
    InfoOption := TVerifySignatureInfoOption(
		    ord(rbNoInfo.Checked = true) * ord(vsOnStatusBarOnly) +
		    ord(rbInReaderWindow.Checked = true) * ord(vsInReaderWindow) +
		    ord(rbInExtraWindow.Checked = true) * ord(vsInExtraWindow)
		  );

    PassphraseCacheSeconds := seSeconds.Value;
    WipeFilePasses := sePasses.Value;
    StandardColors := cbStandardColors.Checked;
    ShowTime := ord(cbShowTime.State);
    TimeServerName := cbxTimeServerName.Text;
    TimeServerPort := StrToIntDef(meTimeServerPort.Text, 0);
    TimeAutoSync := cbTimeAutoSync.Checked;


  end;
end;

function TConfigDialog.ToolbarChanged: Longbool;
begin
  Result := (
       (NewOptions.WipePassphraseButton <> OldOptions.WipePassphraseButton)
    or (NewOptions.EncryptSignButton <> OldOptions.EncryptSignButton)
    or (NewOptions.DecryptMessageButton <> OldOptions.DecryptMessageButton)
    or (NewOptions.VerifySignatureButton <> OldOptions.VerifySignatureButton)
    or (NewOptions.ManageKeysButton <> OldOptions.ManageKeysButton)
    or (NewOptions.LaunchPGPButton <> OldOptions.LaunchPGPButton)
  );
end;

procedure TConfigDialog.GreySignatureInfoOptions(Grey: Longbool);
begin
  rbNoInfo.Enabled := not Grey;
  rbInReaderWindow.Enabled := not Grey;
  rbInExtraWindow.Enabled := not Grey;
end;


end.

