{$J+,Z4}
unit ConfigHandler;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, IniFiles, Forms, HelpCtrl, FileUtils, WindowUtils, Processes,
  PMPGPCommon, PMPGPResources, PMOptions, PMHandler, PGPHandler, TERHandler, AutoListHandler, Tidy, UxThemes;

const // Text file extensions
  SINGLE_CHAR_EXTENSIONS	= '.c/.h/.j';
  DOUBLE_CHAR_EXTENSIONS	= '.js/.rc/.vb';
  TRIPLE_CHAR_EXTENSIONS	= '.asc/.txt/.css/.xml/.xsl/.inc/.ini/.inl/.asm/.bas/.cpp/.dpr/.jav/.hpp/.pas/.vbs';

const // Global data
  NEW_LINK_FOLDER		= 'Pegasus Mail PGP Utilities\';
  OLD_LINK_FOLDER		= 'Pegasus PGP Utilities\';
  PGP_ERROR_STRINGS		= 'PGPErrorStrings';
  PMPGP_STRINGS			= 'PMPGPStrings';
  RESOURCES_DIR			= 'Resources\';
  DEFAULT_USER			= 'DefaultUser';
  NEW_USERS_CFG			= 'PMPGPUsers.cfg';
  OLD_USERS_CFG			= 'Users.cfg';
  PMPGP_CFG			= PMPGP + CFG;
  PMPGP_HELP			= 'PMPGPHelp';
  PMPGP_MANUAL			= 'PMPGPManual';
  PMPGP_LICENCE			= 'PMPGPLicence';
  CRYPTOR_NAME			= 'PMPGPCryptor';
  HELPER_NAME			= 'PMPGPHelper';
  ADMIN_NAME			= 'PMPGPAdmin';
  MAIN_NAME			= 'PMPGPMain';
  POPUPS			= 'POPUPS';
  PGPSERV			= 'PGPSERV';
  PGPVIEW			= 'PGPVIEW';
  PGPVSTR			= 'PGPVSTR';
  AUTOPUSH			= 'AUTOPUSH';
  LANGUAGE_CODE			= '040904B0';


type // Help handler
  TPMPGPHelp = Class(THelpHandler)
    FParentWindow: THandle;
    FHelpManualMain: String;
    FHelpManualFile: String;
  public
    constructor Create(Caller: THandle; TERInterface: TTERInterface; const ResourcesPath, LanguageCode: String);
    function ShowManual(const Command: String): Longbool;
    function ManualApplication: String;
  end;

const // Configuration
  secFilePaths			= 'FilePaths';
    idSecRing			= 'SecRing';
    idPubRing			= 'PubRing';
    idPGPgroup			= 'PGPgroup';

  secDefaultUser		= 'DefaultUser';
    idName			= 'Name';
    idAlways			= 'Always';
    idSignAlgorithm		= 'SignAlg';
    idRipeMD			= 'RipeMD';

  secDefaultComment		= 'DefaultComment';
    idComment			= 'Comment';

  secToolbar			= 'Toolbar';
    idWipePass			= 'WipePass';
    idEncSign			= 'EncSign';
    idDecrypt			= 'Decrypt';
    idVerify			= 'Verify';
    idManage			= 'Manage';
    idPGPkeys			= 'PGPkeys';

  secAutoPush			= 'AutoPush';
    idAuto			= 'Auto';

  secHideKeyDialog		= 'HideKeyDialog';
    idHide			= 'Hide';

  secEncryptToSelf		= 'EncryptToSelf';
    idSelf			= 'Self';

  secEncryptAttach		= 'EncryptAttach';
    idEncrypt			= 'Encrypt';

  secConfirmEncrypt		= 'ConfirmEncrypt';
    idConfirm			= 'Confirm';

  secMixedOptions		= 'MixedOptions';
    idMIXWarn			= 'Warn';

  secADKWarning			= 'ADKWarning';
    idADKWarn			= 'Warn';

  secAIOWarning			= 'AIOWarning';
    idAIOWarn			= 'Warn';

  secMMWarning			= 'MMWarning';
    idMMWarn			= 'Warn';

  secCheckMessage		= 'CheckMessage';
    idCheckKey			= 'CheckKey';
    idLookup			= 'Lookup';
    idPreDecode			= 'PreDecode';
    idCheckSig			= 'CheckSig';
    idInclude			= 'Include';

  secStorePassphrase		= 'StorePassphrase';
    idStore			= 'Store';

  secWipeCount			= 'WipeCount';
    idCount			= 'Count';

  secStandardColor		= 'StandardColor';
    idStandard			= 'Standard';

  secShowTime			= 'ShowTime';
    idShow			= 'Show';
    idServer			= 'Server';
    idPort			= 'Port';
    idAutoSync			= 'AutoSync';

  secMenuOptions		= 'MenuOptions';
    idPGPMime			= 'PGPMime';
    idPGPExt			= 'PGPExt';

  secFullAuto			= 'FullAuto';
    idChecked			= 'Checked';
    idSigned			= 'Signed';

  secSMime			= 'SMime';
    idDefault			= 'Default';

  secVersion			= 'Version';
    idNo			= 'No';

  secLicenceWarn		= 'LicenceWarn';
    idWarn			= 'Warn';

  secPlainTextExtensions	= 'PlainTextExtensions';
    idSingle			= 'Single';
    idDouble			= 'Double';
    idTriple			= 'Triple';

type
  TEncryptSignButtonOption = (esNoButton, esEncrypt, esSign, esEncryptSign, esShowMenu);
  TVerifySignatureInfoOption = (vsOnStatusBarOnly, vsInReaderWindow, vsInExtraWindow);
  TOptions = Class(TIniFile)
  private
    FLastChecked: Integer;
    FNewConfig: Longbool;
    FUninstall: Longbool;
    // PGP paths
    FKeyServer: TKeyServer;
    FSecring: String;
    FPubring: String;
    FGroups: String;
    // PGP defaults
    FDefaultUser: String;
    FUseAlways: Longbool;
    FSignAlgorithm: Integer;
    FUseRipeMD: Longbool;
    FComment: String;
    // Toolbar buttons
    FWipePassphraseButton: Longbool;
    FEncryptSignButton: TEncryptSignButtonOption;
    FDecryptMessageButton: Longbool;
    FVerifySignatureButton: Longbool;
    FManageKeysButton: Longbool;
    FLaunchPGPButton: Longbool;
    // Encoding
    FUseAutoList: Longbool;
    FHideKeyDialog: Longbool;
    FEncryptToSelf: Longbool;
    FEncryptAttachments: Longbool;
    FConfirmEncryption: Longbool;
    FMixOptionsWarning: Longbool;
    FADKWarning: Longbool;
    FAIOWarning: Longbool;
    FMMWarning: Longbool;
    // Decoding
    FCheckForKeyData: Longbool;
    FKeyServerLookup: Longbool;
    FPreDecodePSMime: Longbool;
    FAutoVerify: Longbool;
    FInfoOption: TVerifySignatureInfoOption;
    // Paranoia
    FPassphraseCacheSeconds: Integer;
    FWipeFilePasses: Integer;
    // Miscellaneous
    FStandardColors: Longbool;
    FShowTime: Integer;
    FTimeServerName: String;
    FTimeServerPort: Integer;
    FTimeAutoSync: Longbool;
    // Menu options
    FUsePGPMime: Longbool;
    FAppendPGPExtension: Longbool;
    FAutoEncrypt: Longbool;
    FAutoSign: Longbool;
    // Various
    FPreferSMime: Longbool;
    FStoredVersion: Integer;
    FShowLicenceWarning: Longbool;
    // Mime part handling
    FSingleCharExtensions: String;
    FDoubleCharExtensions: String;
    FTripleCharExtensions: String;
    // Pegasus Mail options
    FPMScanLines: Integer;
    FPMEditMargin: Integer;
    // PGP S/Mime
    FPGPSMimeCapability: TSMimeCapability;
    // PGP paths
    procedure	SetSecring(const Value: String);
    procedure	SetPubring(const Value: String);
    procedure	SetGroups(const Value: String);
    // PGP defaults
    function	GetDefaultKey: String;
    procedure	SetDefaultUser(const Value: String);
    procedure	SetUseAlways(Value: Longbool);
    procedure	SetSignAlgorithm(Value: Integer);
    procedure	SetUseRipeMD(Value: Longbool);
    procedure	SetComment(const Value: String);
    // Toolbar buttons
    procedure	SetWipePassphraseButton(Value: Longbool);
    procedure	SetEncryptSignButton(Value: TEncryptSignButtonOption);
    procedure	SetDecryptMessageButton(Value: Longbool);
    procedure	SetVerifySignatureButton(Value: Longbool);
    procedure	SetManageKeysButton(Value: Longbool);
    procedure	SetLaunchPGPButton(Value: Longbool);
    // Encoding
    procedure	SetUseAutoList(Value: Longbool);
    procedure	SetHideKeyDialog(Value: Longbool);
    procedure	SetEncryptToSelf(Value: Longbool);
    procedure	SetEncryptAttachments(Value: Longbool);
    procedure	SetConfirmEncryption(Value: Longbool);
    procedure	SetMixOptionsWarning(Value: Longbool);
    procedure	SetADKWarning(Value: Longbool);
    procedure	SetAIOWarning(Value: Longbool);
    procedure	SetMMWarning(Value: Longbool);
    // Decoding
    procedure	SetCheckForKeyData(Value: Longbool);
    procedure	SetKeyServerLookup(Value: Longbool);
    procedure	SetPreDecodePSMime(Value: Longbool);
    procedure	SetAutoVerify(Value: Longbool);
    procedure	SetInfoOption(Value: TVerifySignatureInfoOption);
    // Paranoia
    procedure	SetPassphraseCacheSeconds(Value: Integer);
    procedure	SetWipeFilePasses(Value: Integer);
    // Miscellaneous
    procedure	SetStandardColors(Value: Longbool);
    procedure	SetShowTime(Value: Integer);
    function	GetTimeServerName: String;
    procedure	SetTimeServerName(const Value: String);
    function	GetTimeServerPort: Integer;
    procedure	SetTimeServerPort(Value: Integer);
    procedure	SetTimeAutoSync(Value: Longbool);
    // Menu options
    procedure	SetUsePGPMime(Value: Longbool);
    procedure	SetAppendPGPExtension(Value: Longbool);
    procedure	SetAutoEncrypt(Value: Longbool);
    procedure	SetAutoSign(Value: Longbool);
    // Various
    procedure	SetPreferSMime(Value: Longbool);
    procedure	SetStoredVersion(Value: Integer);
    procedure	SetShowLicenceWarning(Value: Longbool);
    // Mime part handling
    procedure	SetSingleCharExtensions(const Value: String);
    procedure	SetDoubleCharExtensions(const Value: String);
    procedure	SetTripleCharExtensions(const Value: String);
  protected
    procedure	UpdatePGPPreferences; virtual;
    procedure	ExecShowTime; virtual;
    procedure	GetKeyServer; virtual;
    procedure	SetKeyServer; virtual;
    procedure	GetPMPrefs; virtual;
    procedure	SetPMPrefs; virtual;
    procedure	SwitchUsePGPMime; virtual;
    procedure	SwitchAppendPGPExtension; virtual;
    procedure	SwitchAutoEncrypt(AutoSign: Longbool); virtual;
  public
    constructor	Create(const ConfigPath: String);
    destructor	Destroy; override;
    {$IFDEF VER90}
    procedure	WriteString(const Section, Ident, Value: String);
    procedure	WriteBool(const Section, Ident: String; Value: Boolean);
    procedure	WriteInteger(const Section, Ident: String; Value: Longint);
    {$ELSE}
    procedure	WriteString(const Section, Ident, Value: String); override;
    procedure	WriteBool(const Section, Ident: String; Value: Boolean); override;
    procedure	WriteInteger(const Section, Ident: String; Value: Longint); override;
    {$ENDIF}
    procedure	ReadConfiguration;
    procedure	WriteConfiguration;
    procedure	CopyConfiguration(const Source: TOptions);
    function	EqualConfiguration(const Source: TOptions): Longbool;
    // PGP paths
    property	KeyServer: TKeyServer read FKeyServer;
    property	Secring: String read FSecring write SetSecring;
    property	Pubring: String read FPubring write SetPubring;
    property	Groups: String read FGroups write SetGroups;
    // PGP defaults
    property	DefaultKey: String read GetDefaultKey;
    property	DefaultUser: String read FDefaultUser write SetDefaultUser;
    property	UseAlways: Longbool read FUseAlways write SetUseAlways;
    property	SignAlgorithm: Integer read FSignAlgorithm write SetSignAlgorithm;
    property	UseRipeMD: Longbool read FUseRipeMD write SetUseRipeMD;
    property	Comment: String read FComment write SetComment;
    // Toolbar buttons
    property	WipePassphraseButton: Longbool read FWipePassphraseButton write SetWipePassphraseButton;
    property	EncryptSignButton: TEncryptSignButtonOption read FEncryptSignButton write SetEncryptSignButton;
    property	DecryptMessageButton: Longbool read FDecryptMessageButton write SetDecryptMessageButton;
    property	VerifySignatureButton: Longbool read FVerifySignatureButton write SetVerifySignatureButton;
    property	ManageKeysButton: Longbool read FManageKeysButton write SetManageKeysButton;
    property	LaunchPGPButton: Longbool read FLaunchPGPButton write SetLaunchPGPButton;
    // Encoding
    property	UseAutoList: Longbool read FUseAutoList write SetUseAutoList;
    property	HideKeyDialog: Longbool read FHideKeyDialog write SetHideKeyDialog;
    property	EncryptToSelf: Longbool read FEncryptToSelf write SetEncryptToSelf;
    property	EncryptAttachments: Longbool read FEncryptAttachments write SetEncryptAttachments;
    property	ConfirmEncryption: Longbool read FConfirmEncryption write SetConfirmEncryption;
    property	MixOptionsWarning: Longbool read FMixOptionsWarning write SetMixOptionsWarning;
    property	ADKWarning: Longbool read FADKWarning write SetADKWarning;
    property	AIOWarning: Longbool read FAIOWarning write SetAIOWarning;
    property	MMWarning: Longbool read FMMWarning write SetMMWarning;
    // Decoding
    property	CheckForKeyData: Longbool read FCheckForKeyData write SetCheckForKeyData;
    property	KeyServerLookup: Longbool read FKeyServerLookup write SetKeyServerLookup;
    property	PreDecodePSMime: Longbool read FPreDecodePSMime write SetPreDecodePSMime;
    property	AutoVerify: Longbool read FAutoVerify write SetAutoVerify;
    property	InfoOption: TVerifySignatureInfoOption read FInfoOption write SetInfoOption;
    // Paranoia
    property	PassphraseCacheSeconds: Integer read FPassphraseCacheSeconds write SetPassphraseCacheSeconds;
    property	WipeFilePasses: Integer read FWipeFilePasses write SetWipeFilePasses;
    // Miscellaneous
    property	StandardColors: Longbool read FStandardColors write SetStandardColors;
    property	ShowTime: Integer read FShowTime write SetShowTime;
    property	TimeServerName: String read GetTimeServerName write SetTimeServerName;
    property	TimeServerPort: Integer read GetTimeServerPort write SetTimeServerPort;
    property	TimeAutoSync: Longbool read FTimeAutoSync write SetTimeAutoSync;
    // Menu options
    property	UsePGPMime: Longbool read FUsePGPMime write SetUsePGPMime;
    property	AppendPGPExtension: Longbool read FAppendPGPExtension write SetAppendPGPExtension;
    property	AutoEncrypt: Longbool read FAutoEncrypt write SetAutoEncrypt;
    property	AutoSign: Longbool read FAutoSign write SetAutoSign;
    // Various
    property	PreferSMime: Longbool read FPreferSMime write SetPreferSMime;
    property	StoredVersion: Integer read FStoredVersion write SetStoredVersion;
    property	ShowLicenceWarning: Longbool read FShowLicenceWarning write SetShowLicenceWarning;
    // Mime part handling
    property	SingleCharExtensions: String read FSingleCharExtensions write SetSingleCharExtensions;
    property	DoubleCharExtensions: String read FDoubleCharExtensions write SetDoubleCharExtensions;
    property	TripleCharExtensions: String read FTripleCharExtensions write SetTripleCharExtensions;
    // Pegasus Mail options
    property	PMScanLines: Integer read FPMScanLines write FPMScanLines;
    property	PMEditMargin: Integer read FPMEditMargin write FPMEditMargin;
    // PGP S/Mime capability
    property	PGPSMimeCapability: TSMimeCapability read FPGPSMimeCapability;
  end;

type
  TSetupError = (seNone, seUser, seOther);
  TConfiguration = Class(TOptions)
  private
    FPMPGPWindow, FParentWindow: THandle;
    FRestartingPegasusMail: Longbool;
    FApplicationPath: String;
    FResourcesPath: String;
    FAppDataPath: String;
    FWinUser: String;
    FWinVersion: String;
    FWinVersionNumber: Integer;
    FWinFixedFontName: String;
    FPMPGPVersion: String;
    FPMPGPVersionNumber: Integer;
    FPMPGPProductInfo: TPMPGPProductInfo;
    FPMPGPStrings: TPMPGPStrings;
    FPGPErrorStrings: TPGPErrorStrings;
    FTERInterface: TTERInterface;
    FPGPInterface: TPGPInterface;
    FPMInterface: TPMInterface;
    FALInterface: TALInterface;
    FPMPGPHelp: TPMPGPHelp;
    procedure	GetWindowsData;
    procedure	DeleteObsoleteFiles;
    function	LoadCryptorLibrary: DWord;
    function	Impersonate: THandle;
    function	RevertUser: Longbool;
    function	CopyUserFiles(const SourcePath, DestPath: String): TSetupError;
    function    DeleteUserFiles(const MailBoxPath: String): Longbool;
    function	InsertPMPGPPath(const FormFactFile: String): Longbool;
    function	CopyFormFactFiles(const SourcePath, DestPath: String): Longbool;
    function	DeleteFormFactFiles(const MailBoxPath, Name1, Name2: String): Longbool;
    function	CreateStartMenuEntries(const FolderName: String; AllUsers: Longbool): Longbool;
    function	DeleteStartMenuEntries(const FolderName: String; AllUsers: Longbool): Longbool;
  protected
    function	Configure: Longbool; virtual; abstract;
    procedure	UpdatePGPPreferences; override;
    procedure	GetKeyServer; override;
    procedure	SetKeyServer; override;
    procedure	GetPMPrefs; override;
    procedure	SetPMPrefs; override;
  public
    constructor	Create;
    destructor	Destroy; override;
    function	GetSupportInfo: String;
    function	Initialize(PMPGPWindow, ParentWindow: THandle): DWord;
    procedure	RestartPegasusMail(const Command: String; Frame: THandle);
    procedure	SendPMPGPMessage(Msg: UINT; wParam: WPARAM; lParam: LPARAM);
    procedure	SyncSysTimeCommand(Frame: THandle; Show: Longbool);
    procedure	LaunchBrowser(const Command: String);
    procedure	LaunchPGPHelp(const Command: String);
    procedure	ShowHelpManual(const Command: String);
    function	UserSetup(const Commands: String): Longbool;
    function	UserUpdate(const Commands: String): Longbool;
    function	UserUninstall: Longbool;
    procedure	UninstallCleanup;
    function	WriteUserEntry(AddNew: Longbool): Longbool;
    procedure	Update;
    property	RestartingPegasusMail: Longbool read FRestartingPegasusMail;
    property	WinFixedFontName: String read FWinFixedFontName;
    property	PMPGPStrings: TPMPGPStrings read FPMPGPStrings;
    property	PGPErrorStrings: TPGPErrorStrings read FPGPErrorStrings;
    property	TERInterface: TTERInterface read FTERInterface;
    property	PGPInterface: TPGPInterface read FPGPInterface;
    property	PMInterface: TPMInterface read FPMInterface;
    property	ALInterface: TALInterface read FALInterface;
    property	PMPGPHelp: TPMPGPHelp read FPMPGPHelp;
    property	PMPGPVersion: String read FPMPGPVersion;
    property	PMPGPVersionNumber: Integer read FPMPGPVersionNumber;
    property	PMPGPProductInfo: TPMPGPProductInfo read FPMPGPProductInfo;
  end;


implementation

uses
  MessageDialogs;


// TPMPGPHelp --------------------------------------------------------------------------------------

constructor TPMPGPHelp.Create(Caller: THandle; TERInterface: TTERInterface; const ResourcesPath, LanguageCode: String);
var
  sConfigHelpFile: String;
begin
  FParentWindow := Caller;
  FHelpManualMain := ResourcesPath + PMPGP_HELP + LanguageCode + HTM;
  if not FileExists(FHelpManualMain) then FHelpManualMain := ResourcesPath + PMPGP_HELP + EN + HTM;
  FHelpManualFile := ResourcesPath + PMPGP_MANUAL + LanguageCode + HTM;
  if not FileExists(FHelpManualFile) then FHelpManualFile := ResourcesPath + PMPGP_MANUAL + EN + HTM;
  sConfigHelpFile := ResourcesPath + PMPGP + LanguageCode + CHM;
  if not FileExists(sConfigHelpFile) then begin
    sConfigHelpFile := ResourcesPath + PMPGP + EN + CHM;
    if not FileExists(sConfigHelpFile) then begin
      sConfigHelpFile := ResourcesPath + PMPGP + LanguageCode + HLP;
      if not FileExists(sConfigHelpFile) then sConfigHelpFile := ResourcesPath + PMPGP + EN + HLP;
    end;
  end;
  inherited Create(Caller, sConfigHelpFile, ResourcesPath + POPUPS + LanguageCode + HTM, TERInterface.HtmlToRtf, 444);
  HelpTitle := DelAmp(FRONTEND_ITEM);
  LoadHelpTopicList(TopicList);
  TopicWindow := HELP_WINDOW;
end;

function TPMPGPHelp.ShowManual(const Command: String): Longbool;
const
  HTML_BEG = '<html>';
  HTML_HEAD = '<head><meta http-equiv="refresh" content="0; URL=%s"></head>';
  HTML_BODY = '<body><a href="%s">%s</a></body>';
  HTML_END = '</html>';
var
  sManualLink, sFileLink, sTempFile: String;
begin
  if Command <> E then begin
    sManualLink := FHelpManualFile + Command;
    sFileLink := FILE_PROT + sManualLink;
    if LoStrAtPos('IExplore.exe', ManualApplication, succ(LastCharPos(BS, ManualApplication))) then begin
      sTempFile := TempFileName(HTM);
      try
	if RegDWord(HKEY_CURRENT_USER, REG_KEY_IE_ZONE_MY_COMP, REG_ENTRY_ALLOW_META_REFRESH) <> 0 then begin
	  WriteTextToFile(sTempFile, Format(HTML_BEG + HTML_HEAD + HTML_BODY + HTML_END,
					    [sFileLink, sFileLink, sManualLink]));
	end
	else WriteTextToFile(sTempFile, Format(HTML_BEG + HTML_HEAD + HTML_END, [sFileLink]));
	Result := FileExecute(0, E, ManualApplication, sTempFile, true);
	if Result then Sleep(1000);
      finally
	DeleteFile(sTempFile);
      end;
    end
    else Result := FileExecute(0, E, ManualApplication, sFileLink, true);
  end
  else Result := FileExecute(0, E, ManualApplication, FILE_PROT + FHelpManualMain, true);
end;

function TPMPGPHelp.ManualApplication: String;
begin
  Result := FindExecutable(FHelpManualMain);
end;


// TOptions ----------------------------------------------------------------------------------------

constructor TOptions.Create(const ConfigPath: String);
var
  sConfigFile: String;
begin
  FKeyServer.Protocol := -1;
  if ConfigPath <> E then begin
    sConfigFile := ConfigPath + PMPGP_CFG;
    FNewConfig := not (FileExists(sConfigFile) or RenameFile(ConfigPath + PGPVIEW + CFG, sConfigFile));
    inherited Create(sConfigFile);
    ReadConfiguration;
  end;
end;

destructor TOptions.Destroy;
begin
  inherited;
end;

procedure TOptions.WriteString(const Section, Ident, Value: String);
begin
  if FileName <> E then begin
    WritePrivateProfileString(PChar(Section), PChar(Ident), PChar(Value), PChar(FileName));
  end;
end;

procedure TOptions.WriteBool(const Section, Ident: String; Value: Boolean);
begin
  if FileName <> E then begin
    WritePrivateProfileString(PChar(Section), PChar(Ident), PChar(IntToStr(ord(Value = true))), PChar(FileName));
  end;
end;

procedure TOptions.WriteInteger(const Section, Ident: String; Value: Longint);
begin
  if FileName <> E then begin
    WritePrivateProfileString(PChar(Section), PChar(Ident), PChar(IntToStr(Value)), PChar(FileName));
  end;
end;

procedure TOptions.ReadConfiguration;
var
  iFileAge: Integer;
begin
  GetKeyServer;
  iFileAge := FileAge(FileName);
  if (FLastChecked = 0) or (iFileAge > FLastChecked) then begin
    FSecring := ReadString(secFilePaths, idSecRing, E);
    FPubring := ReadString(secFilePaths, idPubRing, E);
    FGroups := ReadString(secFilePaths, idPGPgroup, E);
    FDefaultUser := ReadString(secDefaultUser, idName, E);
    FUseAlways := ReadBool(secDefaultUser, idAlways, false);
    FSignAlgorithm := ReadInteger(secDefaultUser, idSignAlgorithm, 0);
    FUseRipeMD := ReadBool(secDefaultUser, idRipeMD, false);
    FComment := ReadString(secDefaultComment, idComment, E);
    FWipePassphraseButton := ReadBool(secToolbar, idWipePass, false);
    FEncryptSignButton := TEncryptSignButtonOption(ReadInteger(secToolbar, idEncSign, 2));
    FDecryptMessageButton := ReadBool(secToolbar, idDecrypt, true);
    FVerifySignatureButton := ReadBool(secToolbar, idVerify, true);
    FManageKeysButton := ReadBool(secToolbar, idManage, true);
    FLaunchPGPButton := ReadBool(secToolbar, idPGPkeys, true);
    FUseAutoList := ReadBool(secAutoPush, idAuto, true);
    FHideKeyDialog := ReadBool(secHideKeyDialog, idHide, false);
    FEncryptToSelf := ReadBool(secEncryptToSelf, idSelf, true);
    FEncryptAttachments := ReadBool(secEncryptAttach, idEncrypt, true);
    FConfirmEncryption := ReadBool(secConfirmEncrypt, idConfirm, false);
    FMixOptionsWarning := ReadBool(secMixedOptions, idMIXWarn, true);
    FADKWarning := ReadBool(secADKWarning, idADKWarn, true);
    FAIOWarning := ReadBool(secAIOWarning, idAIOWarn, true);
    FMMWarning := ReadBool(secMMWarning, idMMWarn, true);
    FCheckForKeyData := ReadBool(secCheckMessage, idCheckKey, false);
    FKeyServerLookup := ReadBool(secCheckMessage, idLookup, false);
    FPreDecodePSMime := ReadBool(secCheckMessage, idPreDecode, false);
    FAutoVerify := ReadBool(secCheckMessage, idCheckSig, false);
    FInfoOption := TVerifySignatureInfoOption(ReadInteger(secCheckMessage, idInclude, 0));
    FPassphraseCacheSeconds := ReadInteger(secStorePassphrase, idStore, -1);
    FWipeFilePasses := ReadInteger(secWipeCount, idCount, 3);
    FStandardColors := ReadBool(secStandardColor, idStandard, false);
    FShowTime := ReadInteger(secShowTime, idShow, 1);
    FTimeServerName := ReadString(secShowTime, idServer, E);
    FTimeServerPort := ReadInteger(secShowTime, idPort, 123);
    FTimeAutoSync := ReadBool(secShowTime, idAutoSync, false);
    FUsePGPMime := ReadBool(secMenuOptions, idPGPMime, false);
    FAppendPGPExtension := ReadBool(secMenuOptions, idPGPExt, false);
    FAutoEncrypt := ReadBool(secFullAuto, idChecked, false);
    FAutoSign := ReadBool(secFullAuto, idSigned, true);
    FPreferSMime := ReadBool(secSMime, idDefault, false);
    FStoredVersion := ReadInteger(secVersion, idNo, 0);
    FShowLicenceWarning := ReadBool(secLicenceWarn, idWarn, true);
    FSingleCharExtensions := ReadString(secPlainTextExtensions, idSingle, SINGLE_CHAR_EXTENSIONS);
    FDoubleCharExtensions := ReadString(secPlainTextExtensions, idDouble, DOUBLE_CHAR_EXTENSIONS);
    FTripleCharExtensions := ReadString(secPlainTextExtensions, idTriple, TRIPLE_CHAR_EXTENSIONS);
    if FNewConfig and UseThemes then FStandardColors := true;
    FLastChecked := iFileAge;
  end;
  GetPMPrefs;
end;

procedure TOptions.WriteConfiguration;
begin
  SetKeyServer;
  WriteString(secFilePaths, idSecRing, QU + FSecring + QU);
  WriteString(secFilePaths, idPubRing, QU + FPubring + QU);
  WriteString(secFilePaths, idPGPgroup, QU + FGroups + QU);
  WriteString(secDefaultUser, idName, QU + FDefaultUser + QU);
  WriteBool(secDefaultUser, idAlways, FUseAlways);
  WriteInteger(secDefaultUser, idSignAlgorithm, FSignAlgorithm);
  WriteBool(secDefaultUser, idRipeMD, FUseRipeMD);
  WriteString(secDefaultComment, idComment, QU + FComment + QU);
  WriteBool(secToolbar, idWipePass, FWipePassphraseButton);
  WriteInteger(secToolbar, idEncSign, ord(FEncryptSignButton));
  WriteBool(secToolbar, idDecrypt, FDecryptMessageButton);
  WriteBool(secToolbar, idVerify, FVerifySignatureButton);
  WriteBool(secToolbar, idManage, FManageKeysButton);
  WriteBool(secToolbar, idPGPkeys, FLaunchPGPButton);
  WriteBool(secAutoPush, idAuto, FUseAutoList);
  WriteBool(secHideKeyDialog, idHide, FHideKeyDialog);
  WriteBool(secEncryptToSelf, idSelf, FEncryptToSelf);
  WriteBool(secEncryptAttach, idEncrypt, FEncryptAttachments);
  WriteBool(secConfirmEncrypt, idConfirm, FConfirmEncryption);
  WriteBool(secMixedOptions, idMIXWarn, FMixOptionsWarning);
  WriteBool(secADKWarning, idADKWarn, FADKWarning);
  WriteBool(secAIOWarning, idAIOWarn, FAIOWarning);
  WriteBool(secMMWarning, idMMWarn, FMMWarning);
  WriteBool(secCheckMessage, idCheckKey, FCheckForKeyData);
  WriteBool(secCheckMessage, idLookup, FKeyServerLookup);
  WriteBool(secCheckMessage, idPreDecode, FPreDecodePSMime);
  WriteBool(secCheckMessage, idCheckSig, FAutoVerify);
  WriteInteger(secCheckMessage, idInclude, ord(FInfoOption));
  WriteInteger(secStorePassphrase, idStore, FPassphraseCacheSeconds);
  WriteInteger(secWipeCount, idCount, FWipeFilePasses);
  WriteBool(secStandardColor, idStandard, FStandardColors);
  WriteInteger(secShowTime, idShow, FShowTime);
  WriteString(secShowTime, idServer, FTimeServerName);
  WriteInteger(secShowTime, idPort, FTimeServerPort);
  WriteBool(secShowTime, idAutoSync, FTimeAutoSync);
  WriteBool(secMenuOptions, idPGPMime, FUsePGPMime);
  WriteBool(secMenuOptions, idPGPExt, FAppendPGPExtension);
  WriteBool(secFullAuto, idChecked, FAutoEncrypt);
  WriteBool(secFullAuto, idSigned, FAutoSign);
  WriteBool(secSMime, idDefault, FPreferSMime);
  WriteInteger(secVersion, idNo, FStoredVersion);
  WriteBool(secLicenceWarn, idWarn, FShowLicenceWarning);
  WriteString(secPlainTextExtensions, idSingle, QU + FSingleCharExtensions + QU);
  WriteString(secPlainTextExtensions, idDouble, QU + FDoubleCharExtensions + QU);
  WriteString(secPlainTextExtensions, idTriple, QU + FTripleCharExtensions + QU);
  FLastChecked := FileAge(FileName);
  UpdatePGPPreferences;
  ExecShowTime;
  SetPMPrefs;
end;

procedure TOptions.CopyConfiguration(const Source: TOptions);
begin
  FKeyServer := Source.FKeyServer;
  FSecring := Source.FSecring;
  FPubring := Source.FPubring;
  FGroups := Source.FGroups;
  FDefaultUser := Source.FDefaultUser;
  FUseAlways := Source.FUseAlways;
  FSignAlgorithm := Source.FSignAlgorithm;
  FUseRipeMD := Source.FUseRipeMD;
  FComment := Source.FComment;
  FWipePassphraseButton := Source.FWipePassphraseButton;
  FEncryptSignButton := Source.FEncryptSignButton;
  FDecryptMessageButton := Source.FDecryptMessageButton;
  FVerifySignatureButton := Source.FVerifySignatureButton;
  FManageKeysButton := Source.FManageKeysButton;
  FLaunchPGPButton := Source.FLaunchPGPButton;
  FUseAutoList := Source.FUseAutoList;
  FHideKeyDialog := Source.FHideKeyDialog;
  FEncryptToSelf := Source.FEncryptToSelf;
  FEncryptAttachments := Source.FEncryptAttachments;
  FConfirmEncryption := Source.FConfirmEncryption;
  FMixOptionsWarning := Source.FMixOptionsWarning;
  FADKWarning := Source.FADKWarning;
  FAIOWarning := Source.FAIOWarning;
  FMMWarning := Source.FMMWarning;
  FCheckForKeyData := Source.FCheckForKeyData;
  FKeyServerLookup := Source.FKeyServerLookup;
  FPreDecodePSMime := Source.FPreDecodePSMime;
  FAutoVerify := Source.FAutoVerify;
  FInfoOption := Source.FInfoOption;
  FPassphraseCacheSeconds := Source.FPassphraseCacheSeconds;
  FWipeFilePasses := Source.FWipeFilePasses;
  FStandardColors := Source.FStandardColors;
  FShowTime := Source.FShowTime;
  FTimeServerName := Source.FTimeServerName;
  FTimeServerPort := Source.FTimeServerPort;
  FTimeAutoSync := Source.FTimeAutoSync;
  FUsePGPMime := Source.FUsePGPMime;
  FAppendPGPExtension := Source.FAppendPGPExtension;
  FAutoEncrypt := Source.FAutoEncrypt;
  FAutoSign := Source.FAutoSign;
  FPreferSMime := Source.FPreferSMime;
  FStoredVersion := Source.FStoredVersion;
  FShowLicenceWarning := Source.FShowLicenceWarning;
  FSingleCharExtensions := Source.FSingleCharExtensions;
  FDoubleCharExtensions := Source.FDoubleCharExtensions;
  FTripleCharExtensions := Source.FTripleCharExtensions;
  FPMScanLines := Source.FPMScanLines;
  FPMEditMargin := Source.FPMEditMargin;
end;

function TOptions.EqualConfiguration(const Source: TOptions): Longbool;
begin
  Result := (
    (FKeyServer.Protocol = Source.FKeyServer.Protocol) and
    (FKeyServer.Name = Source.FKeyServer.Name) and
    (FKeyServer.Port = Source.FKeyServer.Port) and
    (FKeyServer.Flags = Source.FKeyServer.Flags) and
    (FSecring = Source.FSecring) and
    (FPubring = Source.FPubring) and
    (FGroups = Source.FGroups) and
    (FDefaultUser = Source.FDefaultUser) and
    (FUseAlways = Source.FUseAlways) and
    (FSignAlgorithm = Source.FSignAlgorithm) and
    (FUseRipeMD = Source.FUseRipeMD) and
    (FComment = Source.FComment) and
    (FWipePassphraseButton = Source.FWipePassphraseButton) and
    (FEncryptSignButton = Source.FEncryptSignButton) and
    (FDecryptMessageButton = Source.FDecryptMessageButton) and
    (FVerifySignatureButton = Source.FVerifySignatureButton) and
    (FManageKeysButton = Source.FManageKeysButton) and
    (FLaunchPGPButton = Source.FLaunchPGPButton) and
    (FUseAutoList = Source.FUseAutoList) and
    (FHideKeyDialog = Source.FHideKeyDialog) and
    (FEncryptToSelf = Source.FEncryptToSelf) and
    (FEncryptAttachments = Source.FEncryptAttachments) and
    (FConfirmEncryption = Source.FConfirmEncryption) and
    (FMixOptionsWarning = Source.FMixOptionsWarning) and
    (FADKWarning = Source.FADKWarning) and
    (FAIOWarning = Source.FAIOWarning) and
    (FMMWarning = Source.FMMWarning) and
    (FCheckForKeyData = Source.FCheckForKeyData) and
    (FKeyServerLookup = Source.FKeyServerLookup) and
    (FPreDecodePSMime = Source.FPreDecodePSMime) and
    (FAutoVerify = Source.FAutoVerify) and
    (FInfoOption = Source.FInfoOption) and
    (FPassphraseCacheSeconds = Source.FPassphraseCacheSeconds) and
    (FWipeFilePasses = Source.FWipeFilePasses) and
    (FStandardColors = Source.FStandardColors) and
    (FShowTime = Source.FShowTime) and
    (FTimeServerName = Source.FTimeServerName) and
    (FTimeServerPort = Source.FTimeServerPort) and
    (FTimeAutoSync = Source.FTimeAutoSync) and
    (FUsePGPMime = Source.FUsePGPMime) and
    (FAppendPGPExtension = Source.FAppendPGPExtension) and
    (FAutoEncrypt = Source.FAutoEncrypt) and
    (FAutoSign = Source.FAutoSign) and
    (FPreferSMime = Source.FPreferSMime) and
    (FStoredVersion = Source.FStoredVersion) and
    (FShowLicenceWarning = Source.FShowLicenceWarning) and
    (FSingleCharExtensions = Source.FSingleCharExtensions) and
    (FDoubleCharExtensions = Source.FDoubleCharExtensions) and
    (FTripleCharExtensions = Source.FTripleCharExtensions) and
    (FPMScanLines = Source.FPMScanLines) and
    (FPMEditMargin = Source.FPMEditMargin)
  );
end;

procedure TOptions.UpdatePGPPreferences;
begin
end;

procedure TOptions.ExecShowTime;
begin
end;

procedure TOptions.GetKeyServer;
begin
end;

procedure TOptions.SetKeyServer;
begin
end;

procedure TOptions.GetPMPrefs;
begin
end;

procedure TOptions.SetPMPrefs;
begin
end;

procedure TOptions.SwitchUsePGPMime;
begin
  FUsePGPMime := not FUsePGPMime;
  WriteBool(secMenuOptions, idPGPMime, FUsePGPMime);
end;

procedure TOptions.SwitchAppendPGPExtension;
begin
  FAppendPGPExtension := not FAppendPGPExtension;
  WriteBool(secMenuOptions, idPGPExt, FAppendPGPExtension);
end;

procedure TOptions.SwitchAutoEncrypt(AutoSign: Longbool);
begin
  FAutoEncrypt := not FAutoEncrypt;
  if FAutoEncrypt then FAutoSign := AutoSign;
  WriteBool(secFullAuto, idChecked, FAutoEncrypt);
  WriteBool(secFullAuto, idSigned, FAutoSign);
end;

// PGP paths

procedure TOptions.SetSecring(const Value: String);
begin
  if Value <> FSecring then begin
    FSecring := Value;
    WriteString(secFilePaths, idSecRing, FSecring);
  end;
end;

procedure TOptions.SetPubring(const Value: String);
begin
  if Value <> FPubring then begin
    FPubring := Value;
    WriteString(secFilePaths, idPubRing, FPubring);
  end;
end;

procedure TOptions.SetGroups(const Value: String);
begin
  if Value <> FGroups then begin
    FGroups := Value;
    WriteString(secFilePaths, idPGPgroup, FGroups);
  end;
end;

// PGP defaults

function TOptions.GetDefaultKey: String;
begin
  Result := Copy(FDefaultUser, LastLoStrPos(HEX_MARKER, FDefaultUser), MAXINT);
end;

procedure TOptions.SetDefaultUser(const Value: String);
begin
  if Value <> FDefaultUser then begin
    FDefaultUser := Value;
    WriteString(secDefaultUser, idName, FDefaultUser);
  end;
end;

procedure TOptions.SetUseAlways(Value: Longbool);
begin
  if Value <> FUseAlways then begin
    FUseAlways := Value;
    WriteBool(secDefaultUser, idAlways, FUseAlways);
  end;
end;

procedure TOptions.SetSignAlgorithm(Value: Integer);
begin
  if Value <> FSignAlgorithm then begin
    FSignAlgorithm := Value;
    WriteInteger(secDefaultUser, idSignAlgorithm, FSignAlgorithm);
  end;
end;

procedure TOptions.SetUseRipeMD(Value: Longbool);
begin
  if Value <> FUseRipeMD then begin
    FUseRipeMD := Value;
    WriteBool(secDefaultUser, idRipeMD, FUseRipeMD);
  end;
end;

procedure TOptions.SetComment(const Value: String);
begin
  if Value <> FComment then begin
    FComment := Value;
    WriteString(secDefaultComment, idComment, FComment);
  end;
end;

// Toolbar buttons

procedure TOptions.SetWipePassphraseButton(Value: Longbool);
begin
  if Value <> FWipePassphraseButton then begin
    FWipePassphraseButton := Value;
    WriteBool(secToolbar, idWipePass, FWipePassphraseButton);
  end;
end;

procedure TOptions.SetEncryptSignButton(Value: TEncryptSignButtonOption);
begin
  if Value <> FEncryptSignButton then begin
    FEncryptSignButton := Value;
    WriteInteger(secToolbar, idEncSign, ord(FEncryptSignButton));
  end;
end;

procedure TOptions.SetDecryptMessageButton(Value: Longbool);
begin
  if Value <> FDecryptMessageButton then begin
    FDecryptMessageButton := Value;
    WriteBool(secToolbar, idDecrypt, FDecryptMessageButton);
  end;
end;

procedure TOptions.SetVerifySignatureButton(Value: Longbool);
begin
  if Value <> FVerifySignatureButton then begin
    FVerifySignatureButton := Value;
    WriteBool(secToolbar, idVerify, FVerifySignatureButton);
  end;
end;

procedure TOptions.SetManageKeysButton(Value: Longbool);
begin
  if Value <> FManageKeysButton then begin
    FManageKeysButton := Value;
    WriteBool(secToolbar, idManage, FManageKeysButton);
  end;
end;

procedure TOptions.SetLaunchPGPButton(Value: Longbool);
begin
  if Value <> FLaunchPGPButton then begin
    FLaunchPGPButton := Value;
    WriteBool(secToolbar, idPGPkeys, FLaunchPGPButton);
  end;
end;

// Encoding

procedure TOptions.SetUseAutoList(Value: Longbool);
begin
  if Value <> FUseAutoList then begin
    FUseAutoList := Value;
    WriteBool(secAutoPush, idAuto, FUseAutoList);
  end;
end;

procedure TOptions.SetHideKeyDialog(Value: Longbool);
begin
  if Value <> FHideKeyDialog then begin
    FHideKeyDialog := Value;
    WriteBool(secHideKeyDialog, idHide, FHideKeyDialog);
  end;
end;

procedure TOptions.SetEncryptToSelf(Value: Longbool);
begin
  if Value <> FEncryptToSelf then begin
    FEncryptToSelf := Value;
    WriteBool(secEncryptToSelf, idSelf, FEncryptToSelf);
  end;
end;

procedure TOptions.SetEncryptAttachments(Value: Longbool);
begin
  if Value <> FEncryptAttachments then begin
    FEncryptAttachments := Value;
    WriteBool(secEncryptAttach, idEncrypt, FEncryptAttachments);
  end;
end;

procedure TOptions.SetConfirmEncryption(Value: Longbool);
begin
  if Value <> FConfirmEncryption then begin
    FConfirmEncryption := Value;
    WriteBool(secConfirmEncrypt, idConfirm, FConfirmEncryption);
  end;
end;

procedure TOptions.SetMixOptionsWarning(Value: Longbool);
begin
  if Value <> FMixOptionsWarning then begin
    FMixOptionsWarning := Value;
    WriteBool(secMixedOptions, idMIXWarn, FMixOptionsWarning);
  end;
end;

procedure TOptions.SetADKWarning(Value: Longbool);
begin
  if Value <> FADKWarning then begin
    FADKWarning := Value;
    WriteBool(secADKWarning, idADKWarn, FADKWarning);
  end;
end;

procedure TOptions.SetAIOWarning(Value: Longbool);
begin
  if Value <> FAIOWarning then begin
    FAIOWarning := Value;
    WriteBool(secAIOWarning, idAIOWarn, FAIOWarning);
  end;
end;

procedure TOptions.SetMMWarning(Value: Longbool);
begin
  if Value <> FMMWarning then begin
    FMMWarning := Value;
    WriteBool(secMMWarning, idMMWarn, FMMWarning);
  end;
end;

// Decoding

procedure TOptions.SetCheckForKeyData(Value: Longbool);
begin
  if Value <> FCheckForKeyData then begin
    FCheckForKeyData := Value;
    WriteBool(secCheckMessage, idCheckKey, FCheckForKeyData);
  end;
end;

procedure TOptions.SetKeyServerLookup(Value: Longbool);
begin
  if Value <> FKeyServerLookup then begin
    FKeyServerLookup := Value;
    WriteBool(secCheckMessage, idLookup, FKeyServerLookup);
  end;
end;

procedure TOptions.SetPreDecodePSMime(Value: Longbool);
begin
  if Value <> FPreDecodePSMime then begin
    FPreDecodePSMime := Value;
    WriteBool(secCheckMessage, idPreDecode, FPreDecodePSMime);
  end;
end;

procedure TOptions.SetAutoVerify(Value: Longbool);
begin
  if Value <> FAutoVerify then begin
    FAutoVerify := Value;
    WriteBool(secCheckMessage, idCheckSig, FAutoVerify);
  end;
end;

procedure TOptions.SetInfoOption(Value: TVerifySignatureInfoOption);
begin
  if Value <> FInfoOption then begin
    FInfoOption := Value;
    WriteInteger(secCheckMessage, idInclude, ord(FInfoOption));
  end;
end;

// Paranoia

procedure TOptions.SetPassphraseCacheSeconds(Value: Integer);
begin
  if Value <> FPassphraseCacheSeconds then begin
    FPassphraseCacheSeconds := Value;
    WriteInteger(secStorePassphrase, idStore, FPassphraseCacheSeconds);
  end;
end;

procedure TOptions.SetWipeFilePasses(Value: Integer);
begin
  if Value <> FWipeFilePasses then begin
    FWipeFilePasses := Value;
    WriteInteger(secWipeCount, idCount, FWipeFilePasses);
  end;
end;

// Miscellaneous

procedure TOptions.SetStandardColors(Value: Longbool);
begin
  if Value <> FStandardColors then begin
    FStandardColors := Value;
    WriteBool(secStandardColor, idStandard, FStandardColors);
  end;
end;

procedure TOptions.SetShowTime(Value: Integer);
begin
  if Value <> FShowTime then begin
    FShowTime := Value;
    WriteInteger(secShowTime, idShow, FShowTime);
  end;
end;

function TOptions.GetTimeServerName: String;
begin
  Result := FTimeServerName;
  if Result = E then Result := GetDefaultNTPServer;
end;

procedure TOptions.SetTimeServerName(const Value: String);
begin
  if Value <> FTimeServerName then begin
    FTimeServerName := Value;
    WriteString(secShowTime, idServer, FTimeServerName);
  end;
end;

function TOptions.GetTimeServerPort: Integer;
begin
  Result := FTimeServerPort;
  if Result <= 0 then Result := 123;
end;

procedure TOptions.SetTimeServerPort(Value: Integer);
begin
  if Value <> FTimeServerPort then begin
    FTimeServerPort := Value;
    WriteInteger(secShowTime, idPort, FTimeServerPort);
  end;
end;

procedure TOptions.SetTimeAutoSync(Value: Longbool);
begin
  if Value <> FTimeAutoSync then begin
    FTimeAutoSync := Value;
    WriteBool(secShowTime, idAutoSync, FTimeAutoSync);
  end;
end;

// Menu options

procedure TOptions.SetUsePGPMime(Value: Longbool);
begin
  if Value <> FUsePGPMime then begin
    FUsePGPMime := Value;
    WriteBool(secMenuOptions, idPGPMime, FUsePGPMime);
  end;
end;

procedure TOptions.SetAppendPGPExtension(Value: Longbool);
begin
  if Value <> FAppendPGPExtension then begin
    FAppendPGPExtension := Value;
    WriteBool(secMenuOptions, idPGPExt, FAppendPGPExtension);
  end;
end;

procedure TOptions.SetAutoEncrypt(Value: Longbool);
begin
  if Value <> FAutoEncrypt then begin
    FAutoEncrypt := Value;
    WriteBool(secFullAuto, idChecked, FAutoEncrypt);
  end;
end;

procedure TOptions.SetAutoSign(Value: Longbool);
begin
  if Value <> FAutoSign then begin
    FAutoSign := Value;
    WriteBool(secFullAuto, idSigned, FAutoSign);
  end;
end;

// Various

procedure TOptions.SetPreferSMime(Value: Longbool);
begin
  if Value <> FPreferSMime then begin
    FPreferSMime := Value;
    WriteBool(secSMime, idDefault, FPreferSMime);
  end;
end;

procedure TOptions.SetStoredVersion(Value: Integer);
begin
  if Value <> FStoredVersion then begin
    FStoredVersion := Value;
    WriteInteger(secVersion, idNo, FStoredVersion);
  end;
end;

procedure TOptions.SetShowLicenceWarning(Value: Longbool);
begin
  if Value <> FShowLicenceWarning then begin
    FShowLicenceWarning := Value;
    WriteBool(secLicenceWarn, idWarn, FShowLicenceWarning);
  end;
end;

// Mime part handling

procedure TOptions.SetSingleCharExtensions(const Value: String);
begin
  if Value <> FSingleCharExtensions then begin
    FSingleCharExtensions := Value;
    WriteString(secPlainTextExtensions, idSingle, FSingleCharExtensions);
  end;
end;

procedure TOptions.SetDoubleCharExtensions(const Value: String);
begin
  if Value <> FDoubleCharExtensions then begin
    FDoubleCharExtensions := Value;
    WriteString(secPlainTextExtensions, idDouble, FDoubleCharExtensions);
  end;
end;

procedure TOptions.SetTripleCharExtensions(const Value: String);
begin
  if Value <> FTripleCharExtensions then begin
    FTripleCharExtensions := Value;
    WriteString(secPlainTextExtensions, idTriple, FTripleCharExtensions);
  end;
end;


// TConfiguration ----------------------------------------------------------------------------------

constructor TConfiguration.Create;
begin
  GetWindowsData;
  FApplicationPath := GetModuleFilePath(hInstance);
  FResourcesPath := FApplicationPath + RESOURCES_DIR;
  FAppDataPath := GetSystemFolderPath(CSIDL_COMMON_APPDATA);
  if DirExists(FAppDataPath) then begin
    FAppDataPath := FAppDataPath + PMPGP + BS;
    if not DirExists(FAppDataPath) and not CreateDir(FAppDataPath) then FAppDataPath := FApplicationPath;
  end
  else FAppDataPath := FApplicationPath;
end;

destructor TConfiguration.Destroy;
begin
  FPGPErrorStrings.Free;
  FPMPGPStrings.Free;
  FTERInterface.Free;
  FPGPInterface.Free;
  FPMInterface.Free;
  FALInterface.Free;
  FPMPGPHelp.Free;
  inherited;
end;

function TConfiguration.GetSupportInfo: String;
begin
  Result := FPMInterface.PMVersion + CRLF + FPGPInterface.PGPVersion + CRLF + FWinVersion + PARA
end;

function TConfiguration.Initialize(PMPGPWindow, ParentWindow: THandle): DWord;
var
  fviPMPGP: TFileVersionInfo;
begin
  Result := $FFFFFFFF;
  FPMPGPWindow := PMPGPWindow;
  FParentWindow := ParentWindow;
  with fviPMPGP do begin
    QueryFile := GetModuleFileName(hInstance);
    QueryLanguage := LANGUAGE_CODE;
    QueryParameters := [fviLegalCopyright, fviProductName, fviProductVersion];
    if StrUtil.FileVersionInfo(fviPMPGP) then begin
      with FPMPGPProductInfo do begin
	Copyright := LegalCopyright;
	Version := ProductVersion;
	Name := ProductName;
      end;
      if ProductVersion <> E then begin
	FPMPGPVersion := PMPGP + SP + ProductVersion;
	Delete(ProductVersion, LastCharPos('/', ProductVersion), MAXINT);
	Delete(ProductVersion, LastCharPos('b', ProductVersion), MAXINT);
	FPMPGPVersionNumber := VersionNumberFromString(ProductVersion);
      end;
    end;
  end;
  FPMInterface := TPMInterface.Create(FParentWindow, FResourcesPath);
  if FPMInterface <> nil then with FPMInterface do begin
    FPGPErrorStrings := TPGPErrorStrings.Create(FResourcesPath + PGP_ERROR_STRINGS + LanguageCode + TXT);
    FPMPGPStrings := TPMPGPStrings.Create(FResourcesPath + PMPGP_STRINGS + '%s' + TXT, LanguageCode);
    FPMPGPProductInfo.MainLanguageID := MainLanguageID;
    PGPErrorStrings := FPGPErrorStrings;
    PMPGPStrings := FPMPGPStrings;
    inherited Create(MailBoxPath);
    LoadTidy(BaseDirPath);
    if FileName = E then begin
      Warn(FPMPGPStrings[USER_HANDLING_HINT]);
      Exit;
    end
    else begin
      FTERInterface := TTERInterface.Create(BaseDirPath, PMVersionNumber > 4730);
      TERInterface := FTERInterface;
      Result := 0;
    end;
    FNewConfig := FNewConfig and (PMUser <> FPMPGPStrings[ALL_PM_USERS]);
    if (FirstLoStrPos('User', ParamStr(1)) = 0) and (ParamStr(1) <> CLEANUP) then begin
      FPMPGPHelp := TPMPGPHelp.Create(FPMPGPWindow, FTERInterface, FResourcesPath, LanguageCode);
    end;
    if FParentWindow <> 0 then FALInterface := TALInterface.Create(MailBoxPath + AUTO_LIST);
  end;
  if not FUninstall then begin
    FPGPInterface := TPGPInterface.Create(FPMPGPHelp, FPMPGPStrings, FPGPErrorStrings);
    if FPGPInterface <> nil then begin
      with FPGPInterface do begin
	with PMPGPProductInfo do begin
	  Version := Version + SP + LB + '+' + SP + PGPVersion + RB;
	  PGPVersionString := Name + SP + Version;
	end;
	if FShowLicenceWarning and (PGPVersionNumber >= 9000)
	and (FPMInterface.PMUser <> FPMPGPStrings[ALL_PM_USERS]) then begin
	  Warn(FPMPGPStrings[PGP9_LICENCE_WARNING]);
	  ShowLicenceWarning := false;
	end;
	GetKeyringPreferences;
	if not FileExists(FileName) then begin
	  Comment := FPGPInterface.Comment;
	  PassphraseCacheSeconds := FPGPInterface.PassphraseCacheSeconds;
	  WipeFilePasses := FPGPInterface.WipeFilePasses;
	end;
	UpdatePGPPreferences;
      end;
      if FALInterface <> nil then with FALInterface do begin
	SetDefaultFlags(AutoEncrypt, AutoSign);
	UpdateList(FPGPInterface);
      end;
    end;
    if FParentWindow <> 0 then Result := LoadCryptorLibrary;
  end;
end;

procedure TConfiguration.RestartPegasusMail(const Command: String; Frame: THandle);
begin
  FRestartingPegasusMail := (FPMInterface <> nil) and
			    FPMInterface.RestartPegasusMail(FApplicationPath + HELPER_EXE, Command, Frame);
end;

procedure TConfiguration.SendPMPGPMessage(Msg: UINT; wParam: WPARAM; lParam: LPARAM);
begin
  SendMessage(FindApplicationWindow(PM_FILE_NAME, HIDDEN_CLASS, E), Msg, wParam, lParam);
end;

procedure TConfiguration.SyncSysTimeCommand(Frame: THandle; Show: Longbool);
var
  sCommand: String;
begin
  if not Show then
    sCommand := SYNCTIME
  else sCommand := SYNCTIME + SP + 'SHOW';
  FileExecute(0, E, FApplicationPath + ADMIN_EXE, sCommand, Show);
end;

procedure TConfiguration.LaunchBrowser(const Command: String);
var
  sBrowser, sCommand: String;
begin
  Impersonate;
  try
    sBrowser := FPMPGPHelp.ManualApplication;
    if (LoStrStart(HTTP, Command) or LoStrStart(FTP, Command) or LoStrStart(MAILTO, Command)) then
      sCommand := Command
    else if LoStrStart(HTM, ExtractFileExt(Command)) then
      sCommand := FILE_PROT + FResourcesPath + Command
    else sCommand := E;
    if LoStrStart(MAILTO, sCommand) and FileExecute(0, E, sCommand, E, true) then Exit;
    if (sCommand <> E) and FileExecute(0, E, sBrowser, sCommand, true) then Exit;
    Error(FPMPGPStrings[FILE_OPEN_ERROR] + PARA + Command);
  finally
    RevertUser;
  end;
end;

procedure TConfiguration.LaunchPGPHelp(const Command: String);
begin
  Impersonate;
  try
    if FPGPInterface <> nil then FPGPInterface.LaunchHelp(Command);
  finally
    RevertUser;
  end;
end;

procedure TConfiguration.ShowHelpManual(const Command: String);
begin
  Impersonate;
  try
    if not FPMPGPHelp.ShowManual(Command) then Error(FPMPGPStrings[HELP_MANUAL_OPEN_ERROR]);
  finally
    RevertUser;
  end;
end;

function TConfiguration.UserSetup(const Commands: String): Longbool;
begin
  Update;
  Result := false;
  Impersonate;
  try
    DeleteStartMenuEntries(OLD_LINK_FOLDER, true);
    if Commands <> E then begin
      CreateStartMenuEntries(NEW_LINK_FOLDER, LoStrAtPos(SM_ALLUSERS, Commands, Length(Commands) - pred(Length(SM_ALLUSERS))));
    end;
  finally
    RevertUser;
  end;
  with FPMInterface do begin
    case CopyUserFiles(FResourcesPath, MailBoxPath) of
      seNone: begin
	Result := true;
	if FNewConfig then begin
	  Impersonate;
	  try
	    Configure;
	  finally
	    RevertUser;
	  end;
	end;
	Info(FPMPGPStrings[USER_INSTALL_SUCCESS] + PMUser);
      end;
      seUser: begin
	Error(Format(FPMPGPStrings[USER_PMSMIME_HINT], [AnsiUpperCase(PMUser)]));
	DeleteUserFiles(FPMInterface.MailBoxPath);
      end;
    else
      Info(FPMPGPStrings[USER_SETUP_FAILURE] + PMUser);
    end;
    if PMUser = FPMPGPStrings[ALL_PM_USERS] then DeleteFile(MailBoxPath + PMPGP_CFG);
  end;
end;

function TConfiguration.UserUpdate(const Commands: String): Longbool;
const
  LSB = '[';
  RSB = ']';
var
  sUsersFile, sMailBoxUser, sCurrentLine, sWindowsUser: String;
  iUser, iLine, iPos: Integer;
  slUsers: TStringList;
  fUserFound: Longbool;
  se: TSetupError;
begin
  Result := false;
  sUsersFile := FAppDataPath + NEW_USERS_CFG;
  if FileExists(sUsersFile) and (FPMInterface.PMUser <> FPMPGPStrings[ALL_PM_USERS]) then begin
    sMailBoxUser := FPMInterface.PMUser + EQ + QU;
    iUser := succ(Length(sMailBoxUser));
    slUsers := TStringList.Create;
    if slUsers <> nil then with slUsers do begin
      try
	fUserFound := false;
	LoadFromFile(sUsersFile);
	iLine := IndexOf(LSB + FWinUser + RSB);
	if iLine > -1 then begin
	  inc(iLine);
	  while iLine < Count do begin
	    sCurrentLine := Strings[iLine];
	    if (sCurrentLine <> E) and (sCurrentLine[1] = LSB) then Break;
	    fUserFound := (Length(sCurrentLine) > iUser) and LoStrStart(sMailBoxUser, sCurrentLine);
	    if fUserFound then Break;
	    inc(iLine);
	  end;
	end;
	if fUserFound or not Cancel(Format(FPMPGPStrings[USER_UPDATE_CONFIRMATION], [FPMInterface.PMUser])) then begin
	  Result := UserSetup(Commands);
	end;
	if (Count > 2) and Confirm(FPMPGPStrings[USER_UPDATE_ALL_CONFIRMATION]) then begin
	  Result := false;
	  for iLine := 0 to pred(Count) do begin
	    sCurrentLine := Strings[iLine];
	    if sCurrentLine <> E then begin
	      if sCurrentLine[1] <> LSB then begin
		iPos := FirstStrPos(EQ + QU, sCurrentLine);
		if iPos > 0 then begin
		  sMailBoxUser := Copy(sCurrentLine, 1, pred(iPos));
		  System.Delete(sCurrentLine, 1, succ(iPos));
		  System.Delete(sCurrentLine, Length(sCurrentLine), 1);
		  if (sCurrentLine <> E) and (AnsiCompareText(sCurrentLine, FPMInterface.MailBoxPath) <> 0) then begin
		    se := CopyUserFiles(FPMInterface.MailBoxPath, sCurrentLine);
		    if se = seOther then se := CopyUserFiles(FResourcesPath, sCurrentLine);
		    case se of
		      seNone: Info(FPMPGPStrings[USER_INSTALL_SUCCESS] + sWindowsUser + SL + sMailBoxUser);
		      seUser: begin
			Error(Format(FPMPGPStrings[USER_PMSMIME_HINT], [AnsiUpperCase(sMailBoxUser)]));
			DeleteUserFiles(FPMInterface.MailBoxPath);
		      end;
		    else
		      Info(FPMPGPStrings[USER_SETUP_FAILURE] + sWindowsUser + SL + sMailBoxUser);
		    end;
		    Result := true;
		  end;
		end;
	      end
	      else begin
		sWindowsUser := sCurrentLine;
		System.Delete(sWindowsUser, Length(sWindowsUser), 1);
		System.Delete(sWindowsUser, 1, 1);
	      end;
	    end;
	  end;
	  if not Result then Info(FPMPGPStrings[USER_UPDATE_ABORTED]);
	end;
      finally
	Free;
      end;
    end;
  end
  else Result := UserSetup(Commands);
end;

function TConfiguration.UserUninstall: Longbool;
begin
  with FPMInterface do begin
    Result := DeleteUserFiles(MailBoxPath);
    if Result then begin
      Info(FPMPGPStrings[USER_UNINSTALL_SUCCESS] + PMUser);
      WriteUserEntry(false);
    end
    else Info(FPMPGPStrings[USER_UNINSTALL_FAILURE] + PMUser);
  end;
end;

procedure TConfiguration.UninstallCleanup;
var
  sOldUsersFile, sNewUsersFile, sCurrentLine: String;
  slUsers: TStringList;
  iLine, iPos: Integer;
begin
  FUninstall := true;
  sNewUsersFile := FAppDataPath + NEW_USERS_CFG;
  if not FileExists(sNewUsersFile) then begin
    sOldUsersFile := FAppDataPath + OLD_USERS_CFG;
    if FileExists(sOldUsersFile) then begin
      MoveFile(sOldUsersFile, sNewUsersFile, true);
      DeleteFile(sOldUsersFile);
    end;
  end;
  if FileExists(sNewUsersFile) then begin
    slUsers := TStringList.Create;
    if slUsers <> nil then with slUsers do begin
      try
	LoadFromFile(sNewUsersFile);
	if Count > 0 then begin
	  for iLine := 0 to pred(Count) do begin
	    sCurrentLine := Strings[iLine];
	    if (sCurrentLine <> E) and (sCurrentLine[1] <> '[') then begin
	      iPos := FirstStrPos(EQ + QU, sCurrentLine);
	      if iPos > 0 then begin
		System.Delete(sCurrentLine, Length(sCurrentLine), 1);
		System.Delete(sCurrentLine, 1, succ(iPos));
		DeleteUserFiles(sCurrentLine);
	      end;
	    end;
	  end;
	end;
	if DeleteFile(sNewUsersFile) then RemoveDir(FAppDataPath);
      finally
	Free;
      end;
    end;
  end;
  DeleteStartMenuEntries(OLD_LINK_FOLDER, true);
  DeleteStartMenuEntries(NEW_LINK_FOLDER, true);
end;

function TConfiguration.WriteUserEntry(AddNew: Longbool): Longbool;
var
  sAppPathUsers, sAppDataUsers, sMailBoxUser, sNewUserLine, sCurrentLine: String;
  iUser, iLine, iLen: Integer;
  slUsers: TStringList;
  fSave: Longbool;
begin
  Result := false;
  sAppPathUsers := FApplicationPath + OLD_USERS_CFG;
  sAppDataUsers := FAppDataPath + NEW_USERS_CFG;
  if (FAppDataPath <> FApplicationPath) and FileExists(sAppPathUsers) and not FileExists(sAppDataUsers) then begin
    MoveFile(sAppPathUsers, sAppDataUsers, true);
    DeleteFile(sAppPathUsers);
  end;
  with FPMInterface do begin
    sMailBoxUser := PMUser + EQ + QU;
    sNewUserLine := sMailBoxUser + MailBoxPath + QU;
  end;
  iUser := succ(Length(sMailBoxUser));
  slUsers := ReadIniFileSection(sAppDataUsers, FWinUser);
  if AddNew and (slUsers = nil) then slUsers := TStringList.Create;
  if slUsers <> nil then with slUsers do begin
    try
      fSave := false;
      iLine := IndexOf(sNewUserLine);
      if iLine = -1 then begin
	inc(iLine);
	while iLine < Count do begin
	  sCurrentLine := Strings[iLine];
	  iLen := Length(sCurrentLine);
	  if (iLen > iUser) and LoStrStart(sMailBoxUser, sCurrentLine) then begin
	    if not FileExists(Copy(sCurrentLine, iUser, iLen - iUser) + PMPGP_CFG) then begin
	      Delete(iLine);
	      Continue;
	    end;
	  end;
	  inc(iLine);
	end;
	if AddNew then begin
	  Add(sNewUserLine);
	  fSave := true;
	end
	else begin
	  Result := true;
	  Exit;
	end;
      end
      else if not AddNew then begin
	Delete(iLine);
	fSave := true;
      end;
      if fSave then Result := WriteIniFileSection(sAppDataUsers, FWinUser, slUsers);
    finally
      Free;
    end;
  end;
end;

procedure TConfiguration.Update;
begin
  Impersonate;
  try
    if PMPGPVersionNumber > StoredVersion then begin
      case StoredVersion of
	0000..5999: DeleteObsoleteFiles;
      end;
    end;
    StoredVersion := PMPGPVersionNumber;
  finally
    RevertUser;
  end;
end;

procedure TConfiguration.GetWindowsData;
var
  hFont: THandle;
  lfWin: TLogFont;
begin
  Impersonate;
  try
    FWinUser := GetUserName;
    if FWinUser = E then FWinUser := DEFAULT_USER;
    FWinVersion := GetWinVersion(false, true, false, true);
    FWinVersionNumber := VersionNumberFromString(FWinVersion);
    hFont := GetStockObject(ANSI_FIXED_FONT);
    if (hFont <> 0) and (GetObject(hFont, SizeOf(TLogFont), @lfWin) <> 0) then FWinFixedFontName := lfWin.lfFaceName;
    if FWinFixedFontName = 'Courier' then FWinFixedFontName := 'Courier New';
  finally
    RevertUser;
  end;
end;

procedure TConfiguration.UpdatePGPPreferences;
begin
  if FPGPInterface <> nil then with FPGPInterface do begin
    with KeyRingPrefs do begin
      if (Secring <> E) and (Secring <> PrivateKeyring) then
	PrivateKeyring := Secring
      else Secring := PrivateKeyring;
      if (Pubring <> E) and (Pubring <> PublicKeyring) then
	PublicKeyring := Pubring
      else Pubring := PublicKeyring;
      if (Groups <> E) and (Groups <> GroupsFile) then
	GroupsFile := Groups
      else Groups := GroupsFile;
      if (DefaultUser <> E) and (DefaultUser <> DefaultKeyID) then
	DefaultKeyID := DefaultUser
      else DefaultUser := DefaultKeyID;
    end;
    FPGPSMimeCapability := Initialize;
    DefaultUser := KeyRingPrefs.DefaultKeyID;
    if KeyServer.Protocol = -1 then GetKeyServer;
    Comment := Self.Comment;
    PassphraseCacheSeconds := Self.PassphraseCacheSeconds;
    WipeFilePasses := Self.WipeFilePasses;
    ResetCaching;
  end;
end;

procedure TConfiguration.GetKeyServer;
begin
  if FPGPInterface <> nil then with FPGPInterface do begin
    GetDefaultKeyServer(FKeyServer);
    ShowError(ErrorCode);
  end;
end;

procedure TConfiguration.SetKeyServer;
begin
  if FPGPInterface <> nil then with FPGPInterface do begin
    SetDefaultKeyServer(FKeyServer);
    ShowError(ErrorCode);
  end;
end;

procedure TConfiguration.GetPMPrefs;
begin
  if FPMInterface <> nil then with FPMInterface do begin
    FPMScanLines := ScanLines;
    FPMEditMargin := EditMargin;
  end;
end;

procedure TConfiguration.SetPMPrefs;
begin
  if FPMInterface <> nil then with FPMInterface do begin
    ScanLines := FPMScanLines;
    EditMargin := FPMEditMargin;
  end;
end;

procedure TConfiguration.DeleteObsoleteFiles;
begin
  if FileExists(PGPSERV + EXE) then begin
    DeleteFiles(FApplicationPath + PGPSERV + DOT + AST);
    DeleteFiles(FApplicationPath + PGPVIEW + DLL);
    DeleteFiles(FApplicationPath + PGPVSTR + DLL);
    DeleteFiles(FApplicationPath + AUTOPUSH + DLL);
    DeleteFiles(FApplicationPath + 'UTF8' + DLL);
    DeleteFiles(FApplicationPath + AST + TXT + AST);
    DeleteFiles(FApplicationPath + AST + FXX);
    DeleteFiles(FApplicationPath + AST + BMP);
    DeleteFiles(FApplicationPath + AST + GIF);
    DeleteFiles(FApplicationPath + AST + GID);
    DeleteFiles(FApplicationPath + AST + HLP);
    DeleteFiles(FApplicationPath + AST + HTM);
  end;
end;

function TConfiguration.LoadCryptorLibrary: DWord;
var
  sCurrentDir: String;
begin
  Result := 0;
  sCurrentDir := GetCurrentDir;
  try
    SetCurrentDir(FApplicationPath);
    if (GetModuleHandle(CRYPTOR_LIB) <> 0) or (LoadLibrary(PChar(FApplicationPath + CRYPTOR_LIB)) <> 0) then Exit;
    Result := GetLastError;
  finally
    SetCurrentDir(sCurrentDir);
  end;
end;

function TConfiguration.Impersonate: THandle;
begin
  if RunningAsAdmin(GetCurrentProcessId) then begin
    Result := GetShellAccessToken;
    UserImpersonate(Result);
    CloseHandle(Result);
  end
  else Result := INVALID_HANDLE_VALUE;
end;

function TConfiguration.RevertUser: Longbool;
begin
  Result := UserRevert;
end;

function TConfiguration.CopyUserFiles(const SourcePath, DestPath: String): TSetupError;
begin
  Impersonate;
  try
    Result := seNone;
    DeleteFormFactFiles(DestPath, PGPVIEW, AUTOPUSH);
    if FileExists(DestPath + 'PMSM*' + FXX) then begin
      Result := seUser;
      Exit;
    end;
    if not CopyFormFactFiles(SourcePath, DestPath) then begin
      Result := seOther;
      Exit;
    end;
    if FNewConfig then CopyFile(PChar(FResourcesPath + AUTO_LIST), PChar(DestPath + AUTO_LIST), true);
  finally
    RevertUser;
  end;
  if SourcePath = FResourcesPath then WriteUserEntry(true);
end;

function TConfiguration.DeleteUserFiles(const MailBoxPath: String): Longbool;
begin
  Impersonate;
  try
    Result := DeleteFile(PChar(MailBoxPath + PMPGP_CFG)) and DeleteFile(PChar(MailBoxPath + AUTO_LIST)) and
	      DeleteFormFactFiles(MailBoxPath, CRYPTOR_NAME, MAIN_NAME);
  finally
    RevertUser;
  end;
end;

function TConfiguration.InsertPMPGPPath(const FormFactFile: String): Longbool;
const
  DLL_PATH = 'Form DLL = ';
var
  slFFFText: TStringList;
  iLine: Integer;
begin
  Result := false;
  slFFFText := TStringList.Create;
  if slFFFText <> nil then with slFFFText do begin
    try
      LoadFromFile(FormFactFile);
      for iLine := 0 to pred(Count) do begin
	if (Strings[iLine] <> E) and (Strings[iLine][1] <> SC)
	and StrStart(DLL_PATH, Strings[iLine]) then begin
	  if LoStrStart(FPMInterface.BaseDirPath, FApplicationPath) then begin
	    Strings[iLine] := DLL_PATH + QU + '~a\' +
			      Copy(FApplicationPath, succ(Length(FPMInterface.BaseDirPath)), MAXINT) +
			      ExtractFileName(Strings[iLine]);
	  end
	  else Strings[iLine] := DLL_PATH + QU + FApplicationPath + ExtractFileName(Strings[iLine]);
	end;
      end;
      SaveToFile(FormFactFile);
      Result := true;
    finally
      Free;
    end;
  end;
end;

function TConfiguration.CopyFormFactFiles(const SourcePath, DestPath: String): Longbool;
var
  hSearch: THandle;
  fdFound: TWin32FindData;
  sFound, sDestFile: String;
begin
  Result := false;
  hSearch := FindFirst(SourcePath + AST + FXX, fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    try
      repeat
	sDestFile := DestPath + fdFound.cFileName;
	if SourcePath = FResourcesPath then begin
	  Result := CopyFile(PChar(SourcePath + fdFound.cFileName), PChar(sDestFile), false);
	  if Result and (SourcePath = FResourcesPath) then Result := InsertPMPGPPath(sDestFile);
	  if not Result then Break;
	end
	else begin
	  sFound := ExtractBaseName(fdFound.cFileName);
	  if (CompareText(sFound, CRYPTOR_NAME) = 0) or (CompareText(sFound, MAIN_NAME) = 0) then begin
	    Result := CopyFile(PChar(SourcePath + fdFound.cFileName), PChar(sDestFile), false);
	    if not Result then Break;
	  end;
	end;
      until not FindNext(hSearch, fdFound);
    finally
      FindClose(hSearch);
    end;
  end;
end;

function TConfiguration.DeleteFormFactFiles(const MailBoxPath, Name1, Name2: String): Longbool;
begin
  Result := DeleteFiles(MailBoxPath + Name1 + FXX) and DeleteFiles(MailBoxPath + Name2 + FXX);
end;

function TConfiguration.CreateStartMenuEntries(const FolderName: String; AllUsers: Longbool): Longbool;

  function CreateShortcuts(const Path: String; AllUsers: Longbool): Longbool;
  var
    sTarget: String;
  begin
    Result := false;
    CreateDir(Path);
    if not AllUsers then begin
      sTarget := FResourcesPath + PMPGP + FPMInterface.LanguageCode + CHM;
      if not FileExists(sTarget) then sTarget := FindFile(FResourcesPath + PMPGP + '??' + CHM);
      if not CreateLinkFile(Path + FPMPGPStrings[LINK_CONTACT], sTarget, E, E) then Exit;
      sTarget := FResourcesPath + PMPGP_LICENCE + FPMInterface.LanguageCode + TXT;
      if not CreateLinkFile(Path + FPMPGPStrings[LINK_LICENCE], sTarget, E, E) then Exit;
      sTarget := FResourcesPath + PMPGP_HELP + FPMInterface.LanguageCode + HTM;
      if not FileExists(sTarget) then sTarget := FindFile(FResourcesPath + PMPGP_HELP + '??' + HTM);
      if not CreateLinkFile(Path + FPMPGPStrings[LINK_MANUAL], sTarget, E, E) then Exit;
      sTarget := RegString(HKEY_LOCAL_MACHINE, REG_KEY_UNINSTALL, REG_ENTRY_UNINSTALL);
      if not CreateLinkFile(Path + FPMPGPStrings[LINK_UNINSTALL], sTarget, E, E) then Exit;
      Result := true;
    end
    else Result := true;
    if Result then begin
      Result := CreateLinkFile(Path + FPMPGPStrings[LINK_USER_SETUP],
			       FApplicationPath + ADMIN_EXE, USER_SETUP, E) and
		CreateLinkFile(Path + FPMPGPStrings[LINK_USER_UNINSTALL],
			       FApplicationPath + ADMIN_EXE, USER_UNINSTALL, E);
    end;
  end;

begin
  DeleteStartMenuEntries(NEW_LINK_FOLDER, AllUsers);
  Impersonate;
  try
    Result := CreateShortcuts(GetSystemFolderPath(CSIDL_PROGRAMS) + NEW_LINK_FOLDER, false);
  finally
    RevertUser;
  end;
  if AllUsers then CreateShortcuts(GetSystemFolderPath(CSIDL_COMMON_PROGRAMS) + NEW_LINK_FOLDER, true);
end;

function TConfiguration.DeleteStartMenuEntries(const FolderName: String; AllUsers: Longbool): Longbool;
begin
  Impersonate;
  try
    Result := DeleteLinkFolder(GetSystemFolderPath(CSIDL_PROGRAMS) + FolderName);
  finally
    RevertUser;
  end;
  if AllUsers then DeleteLinkFolder(GetSystemFolderPath(CSIDL_COMMON_PROGRAMS) + FolderName);
end;


end.

