{$J+,Z4}
unit MessageDialogs;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, Graphics, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls,
  Menus, HotLabel, CustBtnCtrl, CustCheck, Buttons, FlatBtn, FileUtils, WindowUtils,
  PMPGPForm, PMPGPCommon, PMPGPResources;

type
  TMessageDialog = class(TPMPGPForm)
    pnTop: TPanel;
    hlTop: THotLabel;
    pnText: TPanel;
    reText: TRichEdit;
    cbHideDialog: TCustCheckBox;
    pnBottom: TPanel;
    pnButtons: TPanel;
    fbOk: TFlatBtn;
    pnYesNo: TPanel;
    fbYes: TFlatBtn;
    fbNo: TFlatBtn;
    pnCustom: TPanel;
    fbLeft: TFlatBtn;
    fbMiddle: TFlatBtn;
    fbRight: TFlatBtn;
    tmPopup: TTimer;
    pnInput: TPanel;
    pmMessage: TPopupMenu;
    miSelectAll: TMenuItem;
    miCopySelection: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure tmPopupTimer(Sender: TObject);
    procedure reTextMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure pmMessagePopup(Sender: TObject);
    procedure miClick(Sender: TObject);
  private
    FDefaultButton: TFlatBtn;
    FCancelButton: TFlatBtn;
    FCustomIcon: THandle;
    procedure AdjustDialogSize;
    procedure SetDialogType(DialogType: Integer);
    procedure WMContextMenu(var Message: TWMMouse); message WM_CONTEXTMENU;
  protected
  public
  end;

procedure SystemError(const Text: String; Code: Cardinal);
function  Error(const Text: String): Longbool;
function  CustomIconError(Icon: THandle; const Text: String): Longbool;
function  Warn(const Text: String): Longbool;
function  CustomIconWarn(Icon: THandle; const Text: String): Longbool;
function  WarnWithCheckBox(const Text, Option: String): Longbool;
function  Info(const Text: String): Longbool;
function  CustomIconInfo(Icon: THandle; const Text: String): Longbool;
function  InfoWithCheckBox(const Text, Option: String): Longbool;
function  Popup(const Text: String): Longbool;
function  CustomIconPopup(Icon: THandle; const Text: String): Longbool;
function  CustomDataPopup(Icon: THandle; const Title, Text: String): Longbool;
function  PopupWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
function  Confirm(const Text: String): Longbool;
function  ConfirmWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
function  Cancel(const Text: String): Longbool;
function  CancelWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
function  DefYesCancel(const Text: String): TModalResult;
function  DefYesCancelWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
function  DefNoCancel(const Text: String): TModalResult;
function  DefNoCancelWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
function  DefCancelYesNo(const Text: String): TModalResult;
function  DefCancelYesNoWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
function  CustomConfirm(const Title, Text, YesBtn, NoBtn: String): Longbool;
function  CustomConfirmWithCheckBox(const Title, Text, YesBtn, NoBtn, Option: String; var Checked: Longbool): Longbool;
function  CustomCancel(const Title, Text, YesBtn, NoBtn: String): Longbool;
function  CustomCancelWithCheckBox(const Title, Text, YesBtn, NoBtn, Option: String; var Checked: Longbool): Longbool;


implementation

uses
 PMPGPMainClass;

{$R *.DFM}


// Simple messages ---------------------------------------------------------------------------------

function MessageDialog(DialogType: Integer; CustomIcon: THandle; const Title, Msg, Option: String;
		       Popup: Longbool; var Checked: Longbool): Longbool;
var
  frmMessageDlg: TMessageDialog;
begin
  Result := false;
  frmMessageDlg := TMessageDialog.Create(MainClass);
  if frmMessageDlg <> nil then with frmMessageDlg do begin
    try
      if CustomIcon <> 0 then FCustomIcon := CustomIcon;
      if CompareText(ClassString(GetActiveWindow), ClassName) <> 0 then begin
	cbHideDialog.Caption := Option;
	SetDialogType(DialogType);
	if Title <> E then begin
	  tmPopup.Interval := 10000;
	  hlTop.Caption := Title;
	end;
	reText.Lines.Text := Msg;
	tmPopup.Enabled := Popup;
	ShowModal;
	Checked := cbHideDialog.Checked;
      end;
    finally
      Release;
    end;
  end;
end;

procedure SystemError(const Text: String; Code: Cardinal);
var
  fDummy: Longbool;
begin
  if Code <> 0 then MessageDialog(MB_ICONERROR, 0, E, Format(Text, [SysErrorString(Code)]), E, false, fDummy);
end;

function Error(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONERROR, 0, E, Text, E, false, fDummy);
end;

function CustomIconError(Icon: THandle; const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONERROR, Icon, E, Text, E, false, fDummy);
end;

function Warn(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONEXCLAMATION, 0, E, Text, E, false, fDummy);
end;

function CustomIconWarn(Icon: THandle; const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONEXCLAMATION, Icon, E, Text, E, false, fDummy);
end;

function WarnWithCheckBox(const Text, Option: String): Longbool;
begin
  MessageDialog(MB_ICONEXCLAMATION, 0, E, Text, Option, false, Result);
end;

function Info(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, 0, E, Text, E, false, fDummy);
end;

function CustomIconInfo(Icon: THandle; const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, Icon, E, Text, E, false, fDummy);
end;

function InfoWithCheckBox(const Text, Option: String): Longbool;
begin
  MessageDialog(MB_ICONINFORMATION, 0, E, Text, Option, false, Result);
end;

function Popup(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, 0, E, Text, E, true, fDummy);
end;

function CustomIconPopup(Icon: THandle; const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, Icon, E, Text, E, true, fDummy);
end;

function CustomDataPopup(Icon: THandle; const Title, Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, Icon, Title, Text, E, true, fDummy);
end;

function PopupWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
begin
  Result := MessageDialog(MB_ICONINFORMATION, 0, E, Text, Option, true, Checked);
end;


// Simple alternatives -----------------------------------------------------------------------------

function YesNoDialog(DefaultButton: Integer; const Data, Option: String; var Checked: Longbool): TModalResult;
var
  frmMessageDlg: TMessageDialog;
begin
  frmMessageDlg := TMessageDialog.Create(MainClass);
  if frmMessageDlg <> nil then with frmMessageDlg do begin
    try
      SetDialogType(MB_ICONQUESTION or MB_YESNO or DefaultButton);
      cbHideDialog.Caption := Option;
      reText.Lines.Text := Data;
      Result := ShowModal;
      Checked := cbHideDialog.Checked;
    finally
      Release;
    end;
  end
  else Result := mrNone;
end;

function Confirm(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := (YesNoDialog(MB_DEFBUTTON1, Text, E, fDummy) = mrYes);
end;

function ConfirmWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
begin
  Result := (YesNoDialog(MB_DEFBUTTON1, Text, Option, Checked) = mrYes);
end;

function Cancel(const Text: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := (YesNoDialog(MB_DEFBUTTON2, Text, E, fDummy) = mrNo);
end;

function CancelWithCheckBox(const Text, Option: String; var Checked: Longbool): Longbool;
begin
  Result := (YesNoDialog(MB_DEFBUTTON2, Text, Option, Checked) = mrNo);
end;


// Cancel alternatives -----------------------------------------------------------------------------

function YesNoCancelDialog(DefaultButton: Integer; const Data, Option: String; var Checked: Longbool): TModalResult;
var
  frmMessageDlg: TMessageDialog;
begin
  frmMessageDlg := TMessageDialog.Create(MainClass);
  if frmMessageDlg <> nil then with frmMessageDlg do begin
    try
      SetDialogType(MB_ICONQUESTION or MB_YESNOCANCEL or DefaultButton);
      cbHideDialog.Caption := Option;
      reText.Lines.Text := Data;
      Result := ShowModal;
      Checked := cbHideDialog.Checked;
    finally
      Release;
    end;
  end
  else Result := mrNone;
end;

function DefYesCancel(const Text: String): TModalResult;
var
  fDummy: Longbool;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON1, Text, E, fDummy);
end;

function DefYesCancelWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON1, Text, Option, Checked);
end;

function DefNoCancel(const Text: String): TModalResult;
var
  fDummy: Longbool;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON3, Text, E, fDummy);
end;

function DefNoCancelWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON3, Text, Option, Checked);
end;

function DefCancelYesNo(const Text: String): TModalResult;
var
  fDummy: Longbool;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON2, Text, E, fDummy);
end;

function DefCancelYesNoWithCheckBox(const Text, Option: String; var Checked: Longbool): TModalResult;
begin
  Result := YesNoCancelDialog(MB_DEFBUTTON2, Text, Option, Checked);
end;


// Custom alternatives -----------------------------------------------------------------------------

function CustomYesNoDialog(DefaultButton: Integer; const Title, Data, YesBtn, NoBtn, Option: String;
			   var Checked: Longbool): TModalResult;
var
  frmMessageDlg: TMessageDialog;
  iBtnOffset: Integer;
begin
  frmMessageDlg := TMessageDialog.Create(MainClass);
  if frmMessageDlg <> nil then with frmMessageDlg do begin
    try
      case DefaultButton of
	MB_DEFBUTTON1: SetDialogType(MB_ICONQUESTION or MB_YESNOCANCEL or DefaultButton);
	MB_DEFBUTTON3: SetDialogType(MB_ICONEXCLAMATION or MB_YESNOCANCEL or DefaultButton);
      end;
      Caption := PM_PGP + ' - ' + Title;
      reText.Lines.Text := Data;
      fbLeft.Caption := YesBtn;
      fbRight.Caption := NoBtn;
      fbMiddle.Visible := false;
      iBtnOffset := (fbRight.BoundsRect.Left - fbLeft.BoundsRect.Right) div 4;
      with fbLeft do Left := Left + iBtnOffset;
      with fbRight do Left := Left - iBtnOffset;
      cbHideDialog.Caption := Option;
      Result := ShowModal;
      Checked := cbHideDialog.Checked;
    finally
      Release;
    end;
  end
  else Result := mrNone;
end;

function CustomConfirm(const Title, Text, YesBtn, NoBtn: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := (CustomYesNoDialog(MB_DEFBUTTON1, Title, Text, YesBtn, NoBtn, E, fDummy) = mrYes);
end;

function  CustomConfirmWithCheckBox(const Title, Text, YesBtn, NoBtn, Option: String; var Checked: Longbool): Longbool;
begin
  Result := (CustomYesNoDialog(MB_DEFBUTTON1, Title, Text, YesBtn, NoBtn, Option, Checked) = mrYes);
end;

function CustomCancel(const Title, Text, YesBtn, NoBtn: String): Longbool;
var
  fDummy: Longbool;
begin
  Result := (CustomYesNoDialog(MB_DEFBUTTON3, Title, Text, YesBtn, NoBtn, E, fDummy) = mrNo);
end;

function  CustomCancelWithCheckBox(const Title, Text, YesBtn, NoBtn, Option: String; var Checked: Longbool): Longbool;
begin
  Result := (CustomYesNoDialog(MB_DEFBUTTON3, Title, Text, YesBtn, NoBtn, Option, Checked) = mrNo);
end;


// TMessageDialog ----------------------------------------------------------------------------------

procedure TMessageDialog.FormCreate(Sender: TObject);
begin
  Width := 0;
  SetBounds((Screen.Width - Width) div 2, (Screen.Height - Height) div 2, Width, Height);
  fbOk.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_OK);
  fbMiddle.Caption := PMPGPStrings[CANCEL_BUTTON];
  fbYes.Caption := AM + PMPGPStrings[YES_BUTTON];
  fbNo.Caption := AM + PMPGPStrings[NO_BUTTON];
  fbLeft.Caption := fbYes.Caption;
  fbRight.Caption := fbNo.Caption;
  DisableSystemMenu;
end;

procedure TMessageDialog.FormShow(Sender: TObject);
begin
  AdjustDialogSize;
  FDefaultButton.SetFocus;
end;

procedure TMessageDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  iIndex: Integer;
begin
  if ssShift in Shift then Exit;
  case Key of
    VK_RETURN, VK_ESCAPE: begin
      if FDefaultButton = fbOk then
	ModalResult := FDefaultButton.ModalResult
      else if (Key = VK_ESCAPE) then ModalResult := FCancelButton.ModalResult;
      Exit;
    end;
    ord('A'): if Shift = [ssCtrl] then with reText do begin
      SetFocus;
      SelectAll;
      Exit;
    end;
    ord('C'): if Shift = [ssCtrl] then with reText do begin
      if SelLength = 0 then begin
	WriteClipboardData(Lines.Text, CF_TEXT, Self.Handle);
	RemoveKeyMessage;
      end;
      Exit;
    end;
  end;
  if (Key > VK_SPACE) and not (ssCtrl in Shift) then begin
    for iIndex := 0 to pred(ComponentCount) do begin
      if Components[iIndex] is TFlatBtn then begin
	with TFlatBtn(Components[iIndex]) do begin
	  if CanFocus and IsAccel(Key, Caption) then begin
	    RemoveKeyMessage;
	    Click;
	  end;
	end;
      end;
    end;
  end;
end;

procedure TMessageDialog.tmPopupTimer(Sender: TObject);
begin
  tmPopup.Enabled := false;
  Close;
end;

procedure TMessageDialog.reTextMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then begin
    if fbOK.Showing and MainClass.UnknownHexIDPopup(Handle, reText.Text) then Exit;
    with reText.ClientToScreen(Point(X, Y)) do pmMessage.Popup(X, Y);
  end;
end;

procedure TMessageDialog.pmMessagePopup(Sender: TObject);
begin
  miSelectAll.Caption := PMPGPStrings[SELECT_ALL_ITEM];
  miCopySelection.Caption := PMPGPStrings[COPY_SELECTION_ITEM];
end;

procedure TMessageDialog.miClick(Sender: TObject);
begin
  reText.SetFocus;
  if Sender = miSelectAll then
    reText.SelectAll
  else if Sender = miCopySelection then reText.CopyToClipboard;
end;

procedure TMessageDialog.AdjustDialogSize;
var
  iBuffer, iWidth, iHeight, iLines, iIndex, iBorder, iMMWidth: Integer;
  tmText: TTextMetric;
  rParent: TRect;
  sLine: String;
  szText: TSize;
  hDC: THandle;
begin
  iBuffer := 0;
  iWidth := Width;
  iHeight := Height;
  reText.ParentFont := true;
  iLines := reText.Lines.Count;
  GetTextMetrics(Canvas.Handle, tmText);
  rParent := GetWindowRect(ParentHandle);
  for iIndex := 0 to pred(iLines) do begin
    sLine := reText.Lines[iIndex];
    GetTextExtentPoint32(Canvas.Handle, PChar(sLine), succ(Length(sLine)), szText);
    with szText do if iBuffer < CX then iBuffer := CX;
  end;
  with szText, tmText do begin
    CX := iBuffer;
    CY := iLines * (tmHeight + tmExternalLeading);
    if (CX > reText.Width) or (CY > reText.Height) then begin
      inc(iWidth, CX - reText.Width);
      inc(iHeight, CY - reText.Height);
    end;
  end;
  iBorder := (GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXDLGFRAME)) shl 1;
  with pnTop do if hlTop.Caption <> E then begin
    with hlTop do iMMWidth := (8 shl 1) + MinimumWidth;
    if iWidth < iMMWidth then iWidth := iMMWidth;
    Visible := true;
  end
  else begin
    dec(iHeight, Height);
    Visible := false;
  end;
  with pnInput do if cbHideDialog.Caption <> E then begin
    with cbHideDialog do iMMWidth := (Left shl 1) + MinimumWidth;
    if iWidth < iMMWidth then iWidth := iMMWidth;
    Visible := true;
  end
  else begin
    dec(iHeight, Height);
    Visible := false;
  end;
  if pnCustom.Visible and (iWidth < pnCustom.Width) then iWidth := pnCustom.Width;
  hDC := GetWindowDC(Handle);
  with szText do begin
    try
      GetTextExtentPoint32(hDC, PChar(Caption), succ(Length(Caption)), szText);
      inc(CX, GetSystemMetrics(SM_CXSMICON) + GetSystemMetrics(SM_CXSIZE));
      with rParent do iMMWidth := Right - Left;
      if CX > iMMWidth then CX := iMMWidth;
      if iWidth < CX then iWidth := CX;
    finally
      ReleaseDC(Handle, hDC);
    end;
  end;
  SetBounds(Left, Top, iWidth + iBorder, iHeight);
  with pnBottom do begin
    pnButtons.Left := (Width - pnButtons.Width) div 2;
    pnCustom.Left := (Width - pnCustom.Width) div 2;
  end;
  with cbHideDialog do SetBounds((Self.Width - MinimumWidth) div 2, Top, MinimumWidth, Height);
end;

procedure TMessageDialog.SetDialogType(DialogType: Integer);
var
  iIcon, iType, iDefButton: Integer;
begin
  FDefaultButton := fbOk;
  FCancelButton := fbNo;
  Font.Style := [fsBold];
  MessageBeep(DialogType);
  iIcon := DialogType and MB_ICONMASK;
  iType := DialogType and MB_TYPEMASK;
  iDefButton := DialogType and MB_DEFMASK;
  if FCustomIcon <> 0 then Icon.Handle := FCustomIcon;
  Caption := PM_PGP + ' - ';
  case iIcon of
    MB_ICONERROR: begin
      if FCustomIcon = 0 then Icon.Handle := LoadSysIcon(OIC_HAND);
      Caption := Caption + PMPGPStrings[ERROR_DIALOG];
    end;
    MB_ICONQUESTION: begin
      if FCustomIcon = 0 then Icon.Handle := LoadSysIcon(OIC_QUESTION);
      Caption := Caption + PMPGPStrings[CONFIRM_DIALOG];
      fbOk.Visible := false;
    end;
    MB_ICONWARNING: begin
      if FCustomIcon = 0 then Icon.Handle := LoadSysIcon(OIC_WARNING);
      Caption := Caption + PMPGPStrings[WARN_DIALOG];
    end;
    MB_ICONINFORMATION: begin
      if FCustomIcon = 0 then Icon.Handle := LoadSysIcon(OIC_INFORMATION);
      Caption := Caption + PMPGPStrings[INFO_DIALOG];
    end;
  end;
  if iType = 0 then Exit;
  fbOk.Visible := false;
  case iType of
    MB_YESNOCANCEL: begin
      pnCustom.Visible := true;
      case iDefButton of
	MB_DEFBUTTON1: FDefaultButton := fbLeft;
	MB_DEFBUTTON2: FDefaultButton := fbMiddle;
	MB_DEFBUTTON3: FDefaultButton := fbRight;
      end;
      FCancelButton := fbMiddle;
    end;
    MB_YESNO: begin
      pnYesNo.Visible := true;
      case iDefButton of
	MB_DEFBUTTON1: FDefaultButton := fbYes;
	MB_DEFBUTTON2: FDefaultButton := fbNo;
      end;
      FCancelButton := fbNo;
    end;
  end;
end;

procedure TMessageDialog.WMContextMenu(var Message: TWMMouse);
begin
  with Message do if (XPos = -1) and (YPos = -1) then pmMessage.Popup(Left + Width div 2, Top + Height div 2);
end;


end.

