{$J+,Z4}
unit MsgTreeDialog;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, CommCtrl, Classes, SysUtils, StrUtil, Graphics, Forms, Dialogs, Controls,
  ComCtrls, ExtCtrls, StdCtrls, Menus, Buttons, MenuControl, Printers, PrintDlg, ExtCtrlsEx,
  TreeViewEx, ToolTip, HotLabel, FlatBtn, ImageViewer, TextViewer, FileUtils, MailUtils,
  CertStore, PMPGPForm, PMHandler, PGPHandler, PMPGPCommon, PMPGPResources;

type
  TMsgTreeDialog = class(TPMPGPForm)
    mcFile: TMenuControl;
    pmFile: TMCPopupMenu;
    miView: TMenuItem;
    miCopy: TMenuItem;
    miLine: TMenuItem;
    miSave: TMenuItem;
    miClose: TMenuItem;
    pnTop: TPanel;
    pnLeft: TPanel;
    hlSubjectName: THotLabel;
    hlSubjectText: THotLabel;
    hlSentName: THotLabel;
    hlSentText: THotLabel;
    pnTree: TColoredFramePanel;
    pnRight: TPanel;
    pnBottom: TPanel;
    fbView: TFlatBtn;
    fbSave: TFlatBtn;
    fbClose: TFlatBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure mcFileShowPopupMenu(const Position: TPoint; var Cancel: Longbool);
    procedure miViewClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure miSpecialClick(Sender: TObject);
    procedure miCopyClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure miCloseClick(Sender: TObject);
  private
    MailMessageData: String;
    VisibleMsgTree: TMimeMsgTree;
    function  WriteTempFile(const FileName: String; WriteData: Longbool): String;
    procedure MsgTreeCollapsing(Sender: TObject; Node: TTreeNode; var AllowCollapse: Boolean);
    procedure MsgTreeChange(Sender: TObject; Node: TTreeNode);
    procedure MsgTreeDblClick(Sender: TObject);
    procedure MsgTreeMultiSelecting(Node: TTreeNode; SelectAllParam: Longbool; var Cancel: Boolean);
    procedure MsgTreeSaveFiles(Node: TTreeNode; var Cancel: Boolean);
    procedure MsgTreeMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure MsgTreeMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure MsgTreeMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ShowPopupMenu(const TrackPos: TPoint; Alignment: TPopupAlignment);
    procedure OnAdvancedCommand(const FileName: String; var fCancelCommand: Longbool);
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND;
  protected
    procedure SetColors(Default: Longbool); override;
  public
  end;

procedure ShowMsgTree(MainClass: TObject; const MailMsgData: String);


implementation

{$R *.DFM}

uses
  MessageDialogs;

procedure ShowMsgTree(MainClass: TObject; const MailMsgData: String);
var
  frmMsgTreeDlg: TMsgTreeDialog;
begin
  frmMsgTreeDlg := TMsgTreeDialog.Create(MainClass);
  if frmMsgTreeDlg <> nil then with frmMsgTreeDlg do begin
    try
      with MainClass.PrinterPrefs do if not Initialized then begin
	Initialized := true;
	ApplySettings := false;
	StrPLCopy(Printer, PMInterface.PrinterName, pred(SizeOf(TPrinterName)));
	Copies := 1;
	Collate := true;
	PrintRange := prAllPages;
	CustomOption := (ord(PMInterface.PrintHeaders) <> 0);
	with PMInterface.PrinterFont do begin
	  if Name <> E then StrPLCopy(PrinterFontName, Name, pred(SizeOf(TFontName)));
	  if Size <> 0 then PrinterFontSize := Size;
	end;
	MarginFontStyles := [fsBold];
	with PMInterface do begin
	  if PrintHeader then Include(MarginOptions, moPrintHeader);
	  if PrintFooter then Include(MarginOptions, moPrintFooter);
	  Margins := PrintMargins;
	end;
      end;
      with MainClass.PrefsCaptions do begin
	PropsSheet := PMPGPStrings[FORMATTING_SHEET];
	OrientationGroup := PMPGPStrings[ORIENTATION_GROUP];
	PagePortrait := PMPGPStrings[PAGE_PORTRAIT];
	PageLandscape := PMPGPStrings[PAGE_LANDSCAPE];
	MarginsGroup := PMPGPStrings[MARGINS_GROUP];
	HMargins := PMPGPStrings[MARGINS_HORZ_OPTION];
	VMargins := PMPGPStrings[MARGINS_VERT_OPTION];
	FontGroup := PMPGPStrings[FONT_GROUP];
	FontName := PMPGPStrings[FONT_NAME];
	FontSize := PMPGPStrings[FONT_SIZE];
	HeadFootGroup := PMPGPStrings[HEAD_FOOT_GROUP];
	PrintHeader := PMPGPStrings[PRINT_HEADER_OPTION];
	PrintFooter := PMPGPStrings[PRINT_FOOTER_OPTION];
	CustomOption := PMPGPStrings[PRINT_MSG_HEADERS_OPTION];
      end;
      MailMessageData := MailMsgData;
      ShowModal;
    finally
      Release;
    end;
  end;
end;

procedure TMsgTreeDialog.FormCreate(Sender: TObject);
begin
  VisibleMsgTree := TMimeMsgTree.Create(Self);
  if VisibleMsgTree <> nil then with VisibleMsgTree do begin
    Parent := pnTree;
    Align := alClient;
    BorderStyle := bsNone;
    HideSelection := false;
    MultiSelect := false;
    ParentColor := true;
    PopupMenu := pmFile;
    ReadOnly := true;
    ShowHint := true;
    ShowButtons := false;
    OnChange := MsgTreeChange;
    OnCollapsing := MsgTreeCollapsing;
    OnDblClick := MsgTreeDblClick;
    OnMultiSelecting := MsgTreeMultiSelecting;
    OnEnumSelectedNodes := MsgTreeSaveFiles;
    OnMouseMove := MsgTreeMouseMove;
    OnMouseDown := MsgTreeMouseDown;
    OnMouseUp := MsgTreeMouseUp;
  end;
  Caption := PMPGPStrings[MSG_TREE_DIALOG];
  SetHotKeyHint(fbView, PMPGPStrings[VIEW_HINT]);
  SetHotKeyHint(fbSave, PMPGPStrings[SAVE_HINT]);
  SetHotKeyHint(fbClose, PMPGPStrings[CLOSE_HINT]);
  mcFile.Caption := PMPGPStrings[FILE_ITEM];
  miView.Caption := PMPGPStrings[VIEW_ITEM];
  miCopy.Caption := PMPGPStrings[COPY_ITEM];
  miSave.Caption := PMPGPStrings[SAVE_ITEM];
  miClose.Caption := PMPGPStrings[CLOSE_ITEM];
end;

procedure TMsgTreeDialog.FormShow(Sender: TObject);
var
  sDate, sName, sEmail, sDefault, sHeaderName: String;
begin
  try
    with VisibleMsgTree do begin
      MsgTreeData := MainClass.MsgTreeData;
      MimeParse(MailMessageData, 1, 0, MDO_ALL);
      if ContainsData then begin
	AddIcons;
	FullExpand;
	with SectionNodes[0] do begin
	  sDate := ShortUtf8Date;
	  sName := ExtractName(Utf8Sender);
	  sEmail := ExtractEmail(Utf8Sender);
	  with PMInterface do sDefault := GetSenderAddress(CurrentGlobalID);
	  if sEmail = sDefault then begin
	    sName := ExtractName(Utf8Recipient);
	    sEmail := ExtractEmail(Utf8Recipient);
	    sHeaderName := MsgTreeData.RecipientHeaderName;
	  end
	  else sHeaderName := MsgTreeData.SenderHeaderName;
	  if sName = E then sName := sEmail;
	  hlSubjectName.Caption := Utf8ToWide(MsgTreeData.SubjectHeaderName);
	  hlSubjectText.Caption := Utf8ToWide(Utf8Subject);
	  hlSentName.Caption := Utf8ToWide(sHeaderName);
	  hlSentText.Caption := Utf8ToWide(sName + SP + LB + sDate + RB);
	end;
      end;
    end;
  finally
    with VisibleMsgTree do if ContainsData then SectionNodes[0].Selected := true;
  end;
end;

procedure TMsgTreeDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  rSelected: TRect;
begin
  with VisibleMsgTree do begin
    if ContainsData then begin
       if Shift = [ssCtrl] then begin
	case chr(Key) of
	  'C': begin
	    with miCopy do if Enabled then begin
	      RemoveKeyMessage;
	      Click;
	    end;
	  end;
	  'O': begin
	    with fbView do if Enabled then begin
	      RemoveKeyMessage;
	      Click;
	    end;
	  end;
	  'S': begin
	    with fbSave do if Enabled then begin
	      RemoveKeyMessage;
	      Click;
	    end;
	  end;
	end;
      end
      else if (Key = VK_APPS) or ((Shift = [ssShift]) and (Key = VK_F10)) then begin
	if SelectionCount = 1 then begin
	  rSelected := Selected.DisplayRect(true);
	  with rSelected do begin
	    dec(Right, Left);
	    dec(Bottom, Top);
	    TopLeft := ClientToScreen(TopLeft);
	    if not (Selected.SectionType in [mdtMultipartUnknown..mdtMultipartDigest]) then begin
	      ShowPopupMenu(Point(Left + Right div 2, Top + Bottom div 2), paCenter);
	    end;
	  end;
	end;
      end
      else case Key of
	VK_LEFT: if Selected.Parent <> nil then begin
	  Selected := Selected.Parent;
	  Key := 0;
	end;
	VK_RIGHT: if Selected.HasChildren then begin
	  Selected := Selected.GetFirstChild;
	  Key := 0;
	end;
      end;
    end;
  end;
end;

procedure TMsgTreeDialog.FormDestroy(Sender: TObject);
begin
  VisibleMsgTree.Free;
  DelTempFiles(E);
end;

procedure TMsgTreeDialog.mcFileShowPopupMenu(const Position: TPoint; var Cancel: Longbool);
begin
  Cancel := true;
  miLine.Visible := true;
  miClose.Visible := true;
  CreateFileContextMenu(WriteTempFile(TSectionNode(VisibleMsgTree.Selected).FileName , false),
			PMPGPStrings[ADVANCED_ITEM], mcFile.Handle, pmFile.Handle, miCopy.MenuIndex,
			MIN_CONTEXT_MENU_ID, @Position, paLeft, OnAdvancedCommand);
end;

procedure TMsgTreeDialog.miViewClick(Sender: TObject);
var
  Icon: TIcon;
  sText: String;
  hIcon: THandle;
begin
  with VisibleMsgTree do if SelectionCount = 1 then begin
    Icon := TIcon.Create;
    try
      Images.GetIcon(Selected.AbsoluteIndex * 2, Icon);
      if IsGdipImage(Selected.FileExt) then
	ShowGdipImage(MainClass, Selected, Icon)
      else begin
	if Selected.SectionType = mdtSignature then begin
	  sText := Selected.SectionText;
	  hIcon := LoadIcon(PChar(StrToIntDef(SpaceExtract(sText), 0)));
	  CustomIconInfo(hIcon, sText);
	end
	else if TSectionNode(Selected).Encoding in [ePlainKeys..eCerts] then
	  MainClass.PGPInterface.ImportKeys(Selected.SectionData, Self.Handle, false, false)
	else ShowMessageText(MainClass, Selected, Icon, true);
      end;
    finally
      Icon.Free;
    end;
  end;
end;

procedure TMsgTreeDialog.miOpenClick(Sender: TObject);
begin
  FileExecute(0, E, WriteTempFile(E, true), E, true);
end;

procedure TMsgTreeDialog.miSpecialClick(Sender: TObject);
begin
  FileExecute(0, TMenuItem(Sender).Hint, WriteTempFile(E, true), E, true);
end;

procedure TMsgTreeDialog.miCopyClick(Sender: TObject);
var
  sMessageData: String;
begin
  with VisibleMsgTree do if SelectionCount = 1 then begin
    sMessageData := Selected.SectionData;
    if IsGdipImage(Selected.FileExt) then
      CopyGdipImage(sMessageData)
    else begin
      case Selected.SectionType of
	mdtMainHeaders, mdtPlainText, mdtAltText: WriteClipboardData(sMessageData, CF_TEXT, Self.Handle);
	mdtHtmlText: WriteClipboardData(sMessageData, CF_HTML, Self.Handle);
	mdtRtfText: WriteClipboardData(sMessageData, CF_RTF, Self.Handle);
      end;
    end;
  end;
end;

procedure TMsgTreeDialog.miSaveClick(Sender: TObject);
var
  sMessageData, sFileName, sFileExt: String;
  snTest: TSectionNode;
begin
  with VisibleMsgTree do if Selected = SelectedNode then with TSectionNode(Selected) do begin
    sMessageData := SectionData;
    case SectionType of
      mdtSignature: begin
	if Encoding = eSMSigned then begin
	  ShowCertificateDialog(B64Convert(HeaderText), E, false, Self.Handle);
	  Exit;
	end
	else sMessageData := HeaderText;
      end;
      mdtMainHeaders: begin
	sFileName := ViewerFileSave(HashedString(SectionData, 8) + EXT_NEW_MAIL, DefaultMailPath,
				    PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle);
	if sFileName <> E then begin
	  snTest := TSectionNode(GetNext);
	  if snTest <> nil then begin
	    sMessageData := SetPMFlags((snTest.SectionType in [mdtMultipartUnknown..mdtMultipartDigest]),
				       (snTest.SectionType in [mdtMultipartEncrypted..mdtMultipartSigned]) or
				       (snTest.Encoding > eNone),
				       false, false, sFileName);
	    if (sMessageData <> E) and WriteTextToFile(sFileName, sMessageData) then begin
	      Info(PMPGPStrings[DATA_SAVED_TO] + CRLF + sFileName);
	    end;
	  end;
	end;
	Exit;
      end;
    end;
    if sMessageData <> E then begin
      sFileExt := ExtractFileExt(FileName);
      if (CompareText(sFileExt, EXT_NEW_MAIL) = 0)
      or (CompareText(sFileExt, EXT_CNR_MAIL) = 0)
      or (CompareText(sFileExt, EXT_MSO_MAIL) = 0 ) then
	sFileName := ViewerFileSave(ChangeFileExt(FileName, EXT_NEW_MAIL), DefaultMailPath,
				    PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle)
      else sFileName := ViewerFileSave(FileName, E, PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle);
      if (sFileName <> E) and WriteTextToFile(sFileName, sMessageData) then begin
	Info(PMPGPStrings[DATA_SAVED_TO] + CRLF + sFileName);
      end;
    end;
  end
  else EnumSelectedNodes;
end;

procedure TMsgTreeDialog.miCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TMsgTreeDialog.SetColors(Default: Longbool);
begin
  inherited;
  VisibleMsgTree.Color := AWindowColors[Default];
end;

function TMsgTreeDialog.WriteTempFile(const FileName: String; WriteData: Longbool): String;
begin
  Result := E;
  with VisibleMsgTree do if SelectionCount = 1 then begin
    if FileName = E then
      Result := TempFileName(Selected.FileExt)
    else if not WriteData then
      Result := UniqueFile(GetTempDir + FileName)
    else Result := FileName;
    if WriteData then with Selected do begin
      case SectionType of
	mdtSignature: if not WriteTextToFile(Result, HeaderText) then Result := E;
	mdtHtmlText: if not SaveRelatedSectionsAsHtml(Result) then Result := E;
      else
	if not WriteTextToFile(Result, SectionData) then Result := E;
      end;
      if Result = E then MessageDialogs.Error(PMPGPStrings[DATA_WRITE_ERROR]);
    end
    else FileClose(FileCreate(Result));
  end;
end;

procedure TMsgTreeDialog.MsgTreeCollapsing(Sender: TObject; Node: TTreeNode; var AllowCollapse: Boolean);
begin
  AllowCollapse := false;
end;

procedure TMsgTreeDialog.MsgTreeChange(Sender: TObject; Node: TTreeNode);
var
  sFileExt: String;
begin
  if Node <> nil then with VisibleMsgTree, TSectionNode(Node) do begin
    if SelectionCount = 1 then begin
      if (SectionType = mdtSignature) and (Encoding in [eSMEncrypted..eSMInline]) then
	fbSave.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CERT)
      else if SectionType = mdtMainHeaders then
	fbSave.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_MAIL)
      else fbSave.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_SAVE);
      sFileExt := Copy(FileExt, 1, 4);
      case SectionType of
	mdtUnknownData: fbView.Enabled := IsUnknownSectionText(Index);
	mdtMainHeaders..mdtAltText: begin
	  fbView.Enabled := true;
	  if sFileExt = E then begin
	    case SectionType of
	      mdtMainHeaders, mdtPlainText, mdtAltText: sFileExt := TXT;
	      mdtHtmlText: sFileExt := HTM;
	      mdtRtfText: sFileExt := RTF;
	    end;
	  end;
	end;
	mdtFile: begin
	  fbView.Enabled := SingleCharExtensionMatch(sFileExt) or
			    DoubleCharExtensionMatch(sFileExt) or
			    TripleCharExtensionMatch(sFileExt) or
			    IsGdipImage(sFileExt);
	end;
      else
	fbView.Enabled := false;
      end;
      miView.Enabled := fbView.Enabled;
      miView.Default := fbView.Enabled;
      miCopy.Enabled := fbView.Enabled;
      miSave.Default := not fbView.Enabled;
      if not (SectionType in [mdtMultipartUnknown..mdtMultipartDigest]) then begin
	if sFileExt = E then sFileExt := AST;
	fbSave.Default := not fbView.Enabled;
	fbSave.Enabled := true;
      end
      else fbSave.Enabled := false;
      if Showing then mcFile.Enabled := (FileExt <> E);
    end;
  end
  else begin
    mcFile.Enabled := false;
    fbView.Enabled := false;
    fbSave.Enabled := true;
  end;
end;

procedure TMsgTreeDialog.MsgTreeDblClick(Sender: TObject);
begin
  if fbView.Enabled then
    fbView.Click
  else if fbSave.Enabled then fbSave.Click;
end;

procedure TMsgTreeDialog.MsgTreeMultiSelecting(Node: TTreeNode; SelectAllParam: Longbool; var Cancel: Boolean);
begin
  Cancel := (Node = nil) or (TSectionNode(Node).SectionType in [mdtMultipartUnknown..mdtMultipartDigest]) or
	    ((SelectAllParam = true) and (TSectionNode(Node).SectionType <> mdtFile));
end;

procedure TMsgTreeDialog.MsgTreeSaveFiles(Node: TTreeNode; var Cancel: Boolean);
var
  snData: TSectionNode;
  sMessageData, sFileName: String;
begin
  if Node <> nil then with VisibleMsgTree do begin
    snData := TSectionNode(Node);
    if not (snData.SectionType in [mdtMultipartUnknown..mdtMultipartDigest]) then begin
      sMessageData := snData.SectionData;
      if sMessageData <> E then begin
	sFileName := ViewerFileSave(snData.FileName, E, PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle);
	if (sFileName <> E) and WriteTextToFile(sFileName, sMessageData) then begin
	  Info(PMPGPStrings[DATA_SAVED_TO] + CRLF + sFileName);
	end;
      end;
    end;
  end;
end;

procedure TMsgTreeDialog.MsgTreeMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  snFound: TSectionNode;
  sFileExt: String;
begin
  with VisibleMsgTree do begin
    snFound := TSectionNode(GetNodeAt(X, Y));
    if snFound <> nil then begin
      case snFound.SectionType of
	mdtMainHeaders: Hint := PMPGPStrings[SAVE_MSG_HINT];
	mdtSignature: begin
	  if snFound.Encoding in [ePMEncrypted..eSMInline] then
	    Hint := PMPGPStrings[SAVE_CERT_HINT]
	  else Hint := E;
	end;
	mdtFile: begin
	  sFileExt := ExtractFileExt(TSectionNode(snFound).FileName);
	  if (CompareText(sFileExt, EXT_NEW_MAIL) = 0)
	  or (CompareText(sFileExt, EXT_CNR_MAIL) = 0)
	  or (CompareText(sFileExt, EXT_MSO_MAIL) = 0 ) then
	    Hint := PMPGPStrings[SAVE_MSG_FILE_HINT]
	  else if snFound.Encoding in [ePlainKeys..eCerts] then
	    Hint := PMPGPStrings[ADD_KEYS_HINT]
	  else Hint := E;
	end;
      else
	Hint := E;
      end;
      if Hint <> E then with ToolTip do begin
	HintHidePause := 8000;
	ToolRect := (snFound.DisplayRect(true));
	Exit;
      end;
    end;
    ToolTip.ToolRect := TOOLRECT_EMPTY;
  end;
end;

procedure TMsgTreeDialog.MsgTreeMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then with VisibleMsgTree do begin
    if (SelectionCount <= 1) and (GetNodeAt(X, Y) <> nil) then begin
      Selected := TSectionNode(GetNodeAt(X, Y));
    end
    else Beep;
  end;
end;

procedure TMsgTreeDialog.MsgTreeMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then with VisibleMsgTree do if (GetHitTestInfoAt(X, Y) * [htOnLabel]) <> [] then begin
    if not (Selected.SectionType in [mdtMultipartUnknown..mdtMultipartDigest]) then begin
      ShowPopupMenu(Point(-1, -1), paLeft);
    end;
  end;
end;

procedure TMsgTreeDialog.ShowPopupMenu(const TrackPos: TPoint; Alignment: TPopupAlignment);
begin
  miLine.Visible := false;
  miClose.Visible := false;
  CreateFileContextMenu(WriteTempFile(TSectionNode(VisibleMsgTree.Selected).FileName , false),
			PMPGPStrings[ADVANCED_ITEM], Handle, pmFile.Handle, miCopy.MenuIndex,
			MIN_CONTEXT_MENU_ID, @TrackPos, Alignment, OnAdvancedCommand);
end;

procedure TMsgTreeDialog.OnAdvancedCommand(const FileName: String; var fCancelCommand: Longbool);
begin
  fCancelCommand := (WriteTempFile(FileName, true) = E);
end;

procedure TMsgTreeDialog.WMCommand(var Message: TWMCommand);
begin
  with Message do if (Ctl = 0) and (NotifyCode = 0) then pmFile.DispatchCommand(ItemID);
end;


end.

