{$J+,Z4}
unit PMHandler;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, Graphics, Forms, FileUtils, MailUtils, WindowUtils,
  Processes, PMMessages, PMOptions, PMPGPCommon, PMPGPResources, TERHandler;

const // File extensions
  EXT_MSG_FOLDER 	= '.PMM';
  EXT_ADDR_BOOK 	= '.PMR';
  EXT_XMAIL_INDEX 	= '.PMG';
  EXT_XMAIL_BOX  	= '.MBX';
  EXT_PM_TEMP 		= '.PM$';
  EXT_NEW_MAIL   	= '.CNM';
  EXT_CNR_MAIL		= '.CNR';
  EXT_MSO_MAIL		= '.EML';
  EXT_DRAFT_MSG		= '.PMW';
  EXT_FINAL_MSG		= '.PMX';
  EXT_PLAIN_SIG		= '.PMS';
  EXT_FORMAT_SIG	= '.PNS';
  SIG_FILE_INET		= 'InetSig';
  SIG_FILE_LOCAL	= 'LocalSig';

const // Custom
  IDM_MDICHILDREN	= $1000;

const // popup menus
  IDM_MDIREADER_DELMSG	=  0205;
  IDM_MDIPREVIEW_DELMSG	=  1205;

const // Class names
  PM_CLASS_ABOUT	= 'ABOUT';
  PM_CLASS_FRAME	= 'FRAME';
  PM_CLASS_FOLMAN	= 'FOLMAN';
  PM_CLASS_MDI_CLIENT	= 'MDICLIENT';
  PM_CLASS_MDI_EDITOR	= 'EDMAIL';
  PM_CLASS_MDI_FOLDER	= 'FOLDER';
  PM_CLASS_MDI_FOLMAN	= 'FOLMAN';
  PM_CLASS_MDI_FORM	= 'FORM';
  PM_CLASS_MDI_QMAN	= 'QMAN';
  PM_CLASS_MDI_RV3	= 'READER';
  PM_CLASS_MDI_RV4	= 'WPMMDI';
  PM_CLASS_MDI_READER	= PM_CLASS_MDI_RV3 + CM + PM_CLASS_MDI_RV4;
  PM_CLASS_LIST_PARENT	= 'DH_IMBROWSE';
  PM_CLASS_LIST_WINDOW	= 'LDUWIN';
  PM_CLASS_BUTTON_PANEL	= 'WPMDLG_BP';
  PM_CLASS_PREVIEW_DLG	= 'WPMDLG_FLD';
  PM_CLASS_READER_TABS	= 'WPMDLG_MRC';
  PM_CLASS_READER_PANE	= 'WPMDLG_MRD';
  PM_CLASS_STATUS_BAR	= 'WPMDLG_STB';
  PM_CLASS_DH_STATIC	= 'DH_STATIC';
  PM_CLASS_TOOL_BAR	= 'DH_NTLBAR';
  PM_CLASS_BUTTON	= 'BMBNDH';

  PM_CLASS_POPMN_PARENT = 'SVCPHTM';
  PM_CLASS_BEAR_PARENT	= 'THTMLVIEWER';
  PM_CLASS_TER_PARENT	= 'DH_TEXTEDIT';

const // Buttons
  BTN_OK		= ID_OK;
  BTN_CANCEL		= ID_CANCEL;
  BTN_APPLY		= 10;
  BTN_HELP		= 25;

const // Commands
  CMD_FIND_ATTACHED_V3	= $0CE;
  CMD_FIND_ATTACHED_V4	= $4A6;

  CMD_ATTACHED_EXEC_V3	= $099;
  CMD_ATTACHED_SAVE_V3	= $097;
  CMD_ATTACHED_EDIT_V3	= $09B;

  CMD_ATTACHED_EXEC_V4	= $122;
  CMD_ATTACHED_SAVE_V4	= $124;
  CMD_ATTACHED_EDIT_V4	= $12A;

  CMD_ATTACHED_REPLY_V4	= $126;
  CMD_ATTACHED_FWD_V4	= $127;
  CMD_ATTACHED_PRINT_V4	= $128;

  CMD_READER_OPEN_1	= $09C;
  CMD_READER_OPEN_2	= $0C8;
  CMD_READER_REPLY	= $0C9;
  CMD_READER_FWD	= $0CA;

  CMD_FOLMAN_SWITCH	= $135;
  CMD_FOLMAN_READER	= $4B0;
  CMD_FOLMAN_REPLY	= $4B1;
  CMD_FOLMAN_FWD	= $4B2;

  CMD_PRINT     	= $0C3;
  CMD_MOVE		= $0CB;
  CMD_COPY		= $0CC;
  CMD_DELETE		= $0CD;
  CMD_VIEW		= $0D3;

  CMD_NEXT		= $105;
  CMD_PREV		= $106;

  CMD_RAW_SWITCH	= $099;

  CMD_EDITOR_TAB	= $0A7;
  CMD_SPECIAL_TAB	= $097;
  CMD_SEND_MESSAGE	= $08B;

const // Strings
  STR_REPLY_CAPTION	= 021;
  STR_NO_ATTACHMENT	= 038;
  STR_DEFAULT_ID	= 045;
  STR_NEWMAIL_FOLDER_V3	= 015;
  STR_NEWMAIL_FOLDER_V4	= 197;
  STR_FORWARDED_BEG	= 002;
  STR_FORWARDED_END	= 188;
  STR_ORIGINAL_MESSAGE	= 670;
  STR_CHECK_SPELL_SUBJ	= 830;
  STR_PREFS_N_SETTINGS	= 11999;

const // Menus
  MENU_FORM		= 'FORM';

const // Dialogs
  DLG_ATTACHMENT_CHECK	= 'AREMIND';
  DLG_SEARCH_MESSAGE	= 'MSEARCH';
  DLG_PASTE_SPECIAL	= 'PASTE';
  DLG_PREFS_N_SETTINGS	= STR_PREFS_N_SETTINGS;
  DLG_PRINT_MESSAGE	= 'PRINTV4M';
  DLG_QUEUE_CHECK	= 'SENDNOW';
  DLG_REPLY_OPTIONS	= 'REPLY';
  DLG_SPELL_CHECK	= 'SPELL';
  DLG_PRINT_V4M		= 'PRINTV4M';
  DLG_USER_NAME		= 'UNAME3';
  DLG_FMV5		= 'FMV5';
  DLG_ER1		= 'ER1';

const // Dialog controls
  // ABOUT
  CTL_BUTTON_ABOUT_INFO	= 505;
  // AINFO
  CTL_EDIT_ABOUT_INFO	= 101;
  // PREF03
  CTL_MOVE_CONTENTS	= 103;
  CTL_CHANGE_NOW	= 104;
  // PREF07
  CTL_DEFAULT_PASSWORD	= 157;
  CTL_ASK_AT_STARTUP	= 158;
  // CRYPT
  CTL_SELECT_METHOD	= 101;
  CTL_ENTER_PASSWORD	= 102;
  CTL_ENCRYPT_OPTION	= 103;
  CTL_SIGN_OPTION	= 104;
  // E1EC
  CTL_SIGNATURE		= 100;
  CTL_TO		= 101;
  CTL_SUBJECT		= 102;
  CTL_CC		= 103;
  CTL_KEYWORDS		= 104;
  CTL_COPYSELF		= 106;
  CTL_URGENT		= 107;
  CTL_ENCRYPT		= 108;
  CTL_CONFIRM_READING	= 109;
  CTL_CONFIRM_DELIVERY	= 110;
  CTL_RICHTEXT		= 114;
  // E1SC
  CTL_BCC		= 141;
  CTL_REPLIES_TO	= 142;
  CTL_MESSAGE_WIDTH	= 143;
  CTL_DONT_CLOSE	= 144;
  CTL_YEAR		= 145;
  CTL_MONTH		= 146;
  CTL_DAY		= 147;
  CTL_HOUR		= 148;
  CTL_MINUTE		= 149;
  CTL_TAB_WIDTH		= 150;
  CTL_USE_MIME		= 152;
  CTL_UNITS		= 153;
  CTL_GENERATE		= 155;
  CTL_CUSTOM_HEADERS	= 176;
  // E1EC & E1S
  CTL_RICH_TEXT		= 114;
  CTL_IDENTITY		= 440;
  // ER1
  CTL_ER_FROM		= 101;
  CTL_ER_TO		= 102;
  CTL_ER_CC		= 103;
  CTL_ER_SUBJECT	= 104;
  CTL_ER_REPLY_TO	= 105;
  CTL_ER_SENDER		= 106;
  // PASTE
  CTL_TEXT_FORMATTED	= 101;
  CTL_TEXT_PL_WRAPPED	= 102;
  CTL_TEXT_PL_UNWRAPPED	= 103;
  CTL_TEXT_PL_QUOTED	= 104;
  // PREF12
  CTL_FIND_BROWSER	= 104;
  CTL_USE_URLPROXY	= 105;
  // PREF15
  CTL_DISABLE_STYLING	= 158;
  CTL_PASTE_PLAIN	= 159;
  CTL_TABWIDTH		= 181;
  // STATBAR
  CTL_STB_SECTION_BASE	= 050;

const // Button messages
  WM_FIRST_BUTTON	= WM_USER + 1;
  WM_WIPE_PASSPHRASE 	= WM_USER + 1;
  WM_EDIT_AUTOLIST 	= WM_USER + 2;
  WM_ENCRYPT_SIGN  	= WM_USER + 3;
  WM_DECRYPT_MESSAGE  	= WM_USER + 4;
  WM_VERIFY_SIGNATURE   = WM_USER + 5;
  WM_MANAGE_KEYS   	= WM_USER + 6;
  WM_LAUNCH_PGP  	= WM_USER + 7;
  WM_LAST_BUTTON	= WM_USER + 7;

const // PM messages
  PM_TERSETFOCUS	= WM_USER + 10;

const // Print headers
  TILDE			= '~';
  WC_SENDER_FULL	= 'A';
  WC_SENDER_NAME	= 'F';
  WC_SENDER_FIRST	= 'C';
  WC_PERSONAL_NAME	= 'P';
  WC_MSG_FORM_DATE	= 'D';
  WC_MSG_FORM_TIME	= 'T';
  WC_LONG_FORM_DATE	= 'X';
  WC_SUBJECT		= 'S';
  WC_CRLF		= 'L';

const // Miscellaneous
  FOLDER_HEADER_LEN	= 128;
  MAX_PATH_LEN		= 127;
  MAX_HINT_LEN		= 63;

type
  TButton = Record
    Definition: TButtonDef;
    Handle: THandle;
  end;

  PButtons = ^TButtons;
  TButtons = Array[0..pred(MAXINT div SizeOf(TButton))] of TButton;

  TButtonList = Class
  private
    FMDIChild: THandle;
    FBitmapPath: String;
    FButtons: PButtons;
    FCapacity: Integer;
    FCount: Integer;
    procedure	Grow;
    procedure	SetCapacity(NewCapacity: Integer);
    function	GetButtonTitle(Msg: Integer): String;
  protected
  public
    constructor	Create(Parent: THandle; const BitmapPath: String);
    destructor	Destroy; override;
    procedure	AddButton(const Bitmap, Title: String; Owner: THandle; Msg, wParam, lParam: Integer);
    procedure	DeleteButton(Msg: Integer);
    procedure	ClearButtons;
    property	Count: Integer read FCount;
    property	ButtonTitles[Msg: Integer]: String read GetButtonTitle;
  end;

type
  TPMWindow = (wMAINFrame, wMDIClient, wMDIActiveChild, wMDIChildDialog, wToolbar);
  TPrintHeaders = (phNone, phSignificant, phAll);
  TWordWrap = (wwNone, wwReformat, wwSimpleWrap);

  TPMInterface = Class(TButtonList)
  private
    {$HINTS OFF}
    FPMVersion: String;
    FPMVersionNumber: Integer;
    FShellAccessToken: THandle;
    FPMUserMainRegKey: THandle;
    FDlgAttachmentCheck: String;
    FDlgSearchMessage: String;
    FDlgQueueCheck: String;
    FDlgReplyOptions: String;
    FDlgSpellCheck: String;
    FDlgPrintV4M: String;
    FDlgUserName: String;
    FDlgFontInfo: TDlgFont;
    FCommandLine: String;
    FBaseDirPath: String;
    FMailBoxPath: String;
    FNewMailPath: String;
    FLanguageCode: String;
    FLanguageModule: THandle;
    FMainLanguageID: LANGID;
    FTERInterface: TTERInterface;
    FPMPGPStrings: TPMPGPStrings;
    FPGPErrorStrings: TPGPErrorStrings;
    function	GetPMUser: String;
    function	GetBaseDirPath: String;
    function	GetMailBoxPath: String;
    function	GetNewMailPath: String;
    function	GetGlobalAddressBooksPath: String;
    function	GetAddressBooksPathList: String;
    function	GetAddressBookFlagsList: String;
    function	GetMainLanguageID: LANGID;
    function	GetPref(PMOption: TPMOptions): Variant;
    procedure	SetPref(PMOption: TPMOptions; const Value: Variant);
    function	GetPrefString(PMOption: TPMOptions; Size: Integer): String;
    function	GetDirString(DirID: Integer): String;
    function	GetIniFilePref(const Section, ID: String): String;
    function	GetPMIniString(const Section, Value: String): String;
    function	GetEditorIdentity(MdiEditor: THandle): String;
    function	GetIniFileString(const Name: String; Global: Longbool; MdiEditor: THandle): String;
    procedure	SetIniFileString(const Name, Value: String; Global: Longbool; MdiEditor: THandle);
    function	GetReplyHeaderOptions(var CustomBlankLines: Integer): TReplyHeaderOptions;
    function	GetMenuHandle(Index: Integer): THandle;
    function	GetWindowHandle(PMWindow: TPMWindow): THandle;
    function	GetStringResource(Index: Cardinal): String;
    function	GetOriginalMessageSeparator: String;
    function	GetForwardedBegDelimiter: String;
    function	GetForwardedEndDelimiter: String;
    function	GetNewMailFolderCaption: String;
    function	GetNoAttachmentCaption: String;
    function	GetReplyMessageCaption: String;
    function	GetBrowserPreference: String;
    function	GetReloadsWinsock: Longbool;
    function	GetScanLines: Integer;
    procedure	SetScanLines(Value: Integer);
    function	GetEditMargin: Integer;
    procedure	SetEditMargin(Value: Integer);
    function	GetMimeEnclosure: Longbool;
    procedure	SetMimeEnclosure(Value: Longbool);
    function	GetMime8BitEncoding: Longbool;
    procedure	SetMime8BitEncoding(Value: Longbool);
    function	GetOmitAttachmentInfo: Longbool;
    procedure	SetOmitAttachmentInfo(Value: Longbool);
    function	GetEditorAutoSign: Longbool;
    procedure	SetEditorAutoSign(Value: Longbool);
    function	GetEditorFancyStyle: Longbool;
    procedure	SetEditorFancyStyle(Value: Longbool);
    function	GetReaderFancyView: Longbool;
    procedure	SetReaderFancyView(Value: Longbool);
    function	GetReaderHotUrls: Longbool;
    procedure	SetReaderHotUrls(Value: Longbool);
    function	GetReaderWordWrap: TWordWrap;
    procedure	SetReaderWordWrap(Value: TWordWrap);
    function	GetReaderFont: TPMPGPFont;
    function	GetCurrentEditorID: String;
    function	GetCurrentGlobalID: String;
    function	GetPersonalName: String;
    function	GetToRecipients: String;
    function	GetCCRecipients: String;
    function	GetBCCRecipients: String;
    function 	GetMessageHeaderLineCount: Integer;
    function	GetEncryptedMessage: Longbool;
    function	GetPrinterFont: TPMPGPFont;
    function	GetPrinterName: String;
    function	GetPrintBannerHeader: String;
    function	GetPrintHeader: Longbool;
    function	GetPrintFooter: Longbool;
    function	GetPrintHeaders: TPrintHeaders;
    function	GetPrintMargins: TRect;
    procedure	SetMenuInfo(const Text: String);
    procedure	SetStatusBarLongInfo(const Text: String);
    procedure	SetStatusBarShortInfo(const Text: String);
    function	GetMailFolderPath(var UnixFolder: Longbool): String;
  public
    constructor	Create(Parent: THandle; const ResourcesPath: String);
    destructor	Destroy; override;
    function	GetDefaultCharset: String;
    function	GetDefaultBCCAddress(const Identity: String): String;
    function	GlobalResolveAddresses(const Identity: String): Longbool;
    function	RestartPegasusMail(const PMPGPHelper, Command: String; Frame: THandle): Longbool;
    function	NewMailMessage(const Recipient, Subject, Body: String): Longbool;
    function	CopyMailMessage(var MDIEditor: THandle): String;
    function	InsertMessageText(const Text: String): Longbool;
    procedure   SendMailMessage(Simple, ForceMime: Longbool);
    function	GetDummyMessage: String;
    function	RemoveBlankMessages: Integer;
    procedure	RescanMessageQueue;
    procedure	ResetStatusBar;
    procedure	SelectFrameMenuItem(MenuID: Integer);
    function	GetFrameMenuItemChecked(MenuID: Integer): Longbool;
    procedure	SetFrameMenuItemChecked(MenuID: Integer; Value: Longbool);
    function	GetMDIMenuItemChecked(MenuID: Integer): Longbool;
    procedure	SendShutDownLog(TargetWindow: THandle; ForceShutDown: Longbool);
    function	BearOrTer(Parent: THandle; NewReader: Longbool): THandle;
    function	CheckMDIQueueManager(MDIChild: THandle): THandle;
    function	CheckMDIFolman(MDIChild: THandle): THandle;
    function	CheckMDIFolder(MDIChild: THandle): THandle;
    function	CheckMDIEditor(MDIChild: THandle): THandle;
    function	GetEditorMessagePage(Parent: THandle): THandle;
    function	GetEditorSpecialPage(Parent: THandle): THandle;
    function	GetEditorTabWidthTwips(MdiEditor: THandle): Integer;
    function	CheckMDIReader(MDIChild: THandle): THandle;
    function	HasNewReader: Longbool;
    function	IsAttachmentView(MDIReader: THandle): Longbool;
    function	IsBodyView(MDIReader: THandle): Longbool;
    function	IsRawView(MDIReader: THandle): Longbool;
    function	IsReply(MDIEditor: THandle): Longbool;
    function	IsMessageHandlingMDI: Longbool;
    function	CreateUtf8BannerHeader(UTCDateTime: TDateTime; const Utf8Sender, Utf8Subject: String): String;
    function	GetDisplayInfo(MDIWindow: THandle; MessageType: TMessageType): TDisplayInfo;
    function	GetFolderText(var FolderName: String; var UnixFolder: Longbool): String;
    function	CreateUnixMailBoxName(const FolderName: String): String;
    function	GetSenderAddress(const Identity: String): String;
    function	GetSignature: String;
    property	PMUser: String read GetPMUser;
    property	PMVersion: String read FPMVersion;
    property	PMVersionNumber: Integer read FPMVersionNumber;
    property	CommandLine: String read FCommandLine;
    property	DlgAttachmentCheck: String read FDlgAttachmentCheck;
    property	DlgSearchMessage: String read FDlgSearchMessage;
    property	DlgQueueCheck: String read FDlgQueueCheck;
    property	DlgReplyOptions: String read FDlgReplyOptions;
    property	DlgSpellCheck: String read FDlgSpellCheck;
    property	DlgPrintV4M: String read FDlgPrintV4M;
    property	DlgUserName: String read FDlgUserName;
    property	DlgFontInfo: TDlgFont read FDlgFontInfo;
    property	LanguageCode: String read FLanguageCode;
    property	MainLanguageID: LANGID read FMainLanguageID;
    property	LanguageModule: THandle read FLanguageModule;
    property	BaseDirPath: String read GetBaseDirPath;
    property	MailBoxPath: String read GetMailBoxPath;
    property	NewMailPath: String read GetNewMailPath;
    property	GlobalAddressBooksPath: String read GetGlobalAddressBooksPath;
    property	AddressBooksPathList: String read GetAddressBooksPathList;
    property	AddressBookFlagsList: String read GetAddressBookFlagsList;
    property	Menus[Index: Integer]: THandle read GetMenuHandle;
    property	Windows[Index: TPMwindow]: THandle read GetWindowHandle;
    property	Strings[Index: Cardinal]: String read GetStringResource;
    property	NewMailFolderCaption: String read GetNewMailFolderCaption;
    property	NoAttachmentCaption: String read GetNoAttachmentCaption;
    property	ReplyMessageCaption: String read GetReplyMessageCaption;
    property	BrowserPreference: String read GetBrowserPreference;
    property	ReloadsWinsock: Longbool read GetReloadsWinsock;
    property	ScanLines: Integer read GetScanLines write SetScanLines;
    property	EditMargin: Integer read GetEditMargin write SetEditMargin;
    property	MimeEnclosure: Longbool read GetMimeEnclosure write SetMimeEnclosure;
    property	Mime8BitEncoding: Longbool read GetMime8BitEncoding write SetMime8BitEncoding;
    property	OmitAttachmentInfo: Longbool read GetOmitAttachmentInfo write SetOmitAttachmentInfo;
    property	EditorAutoSign: Longbool read GetEditorAutoSign write SetEditorAutoSign;
    property	EditorFancyStyle: Longbool read GetEditorFancyStyle write SetEditorFancyStyle;
    property	ReaderFancyView: Longbool read GetReaderFancyView write SetReaderFancyView;
    property	ReaderHotUrls: Longbool read GetReaderHotUrls write SetReaderHotUrls;
    property	ReaderWordWrap: TWordWrap read GetReaderWordWrap write SetReaderWordWrap;
    property	ReaderFont: TPMPGPFont read GetReaderFont;
    property	CurrentEditorID: String read GetCurrentEditorID;
    property	CurrentGlobalID: String read GetCurrentGlobalID;
    property	PersonalName: String read GetPersonalName;
    property	ToRecipients: String read GetToRecipients;
    property	CCRecipients: String read GetCCRecipients;
    property	BCCRecipients: String read GetBCCRecipients;
    property	MessageHeaderLineCount: Integer read GetMessageHeaderLineCount;
    property	EncryptedMessage: Longbool read GetEncryptedMessage;
    property	PrinterFont: TPMPGPFont read GetPrinterFont;
    property	PrinterName: String read GetPrinterName;
    property	PrintBannerHeader: String read GetPrintBannerHeader;
    property	PrintHeader: Longbool read GetPrintHeader;
    property	PrintFooter: Longbool read GetPrintFooter;
    property	PrintHeaders: TPrintHeaders read GetPrintHeaders;
    property	PrintMargins: TRect read GetPrintMargins;
    property	MenuInfo: String write SetMenuInfo;
    property	StatusBarLongInfo: String write SetStatusBarLongInfo;
    property	StatusBarShortInfo: String write SetStatusBarShortInfo;
    property	TERInterface: TTERInterface read FTERInterface write FTERInterface;
    property	PMPGPStrings: TPMPGPStrings read FPMPGPStrings write FPMPGPStrings;
    property	PGPErrorStrings: TPGPErrorStrings read FPGPErrorStrings write FPGPErrorStrings;
    {$HINTS ON}
  end;


implementation

uses
  MessageDialogs;


// Font utility --------------------------------------------------------------------------------

function PMFont2PMPGPFont(const PMFont: TFontDef): TPMPGPFont;
begin
  with PMFont, Result do begin
    Name := FontName;
    Size := FontSize;
    Height := -MulDiv(FontSize, Screen.PixelsPerInch, 72);
    Styles := FontStyle;
  end;
end;


// TButtonList ---------------------------------------------------------------------------------

constructor TButtonList.Create(Parent: THandle; const BitmapPath: String);
begin
  inherited Create;
  FMDIChild := Parent;
  FBitmapPath := BitmapPath;
  SetCapacity(succ(WM_LAST_BUTTON - WM_FIRST_BUTTON));
end;

destructor TButtonList.Destroy;
begin
  SetCapacity(0);
  FCount := 0;
  inherited;
end;

procedure TButtonList.AddButton(const Bitmap, Title: String; Owner: THandle; Msg, wParam, lParam: Integer);
var
  iIndex: Integer;
begin
  if FCount = FCapacity then Grow;
  iIndex := lParam - WM_FIRST_BUTTON;
  with FButtons^[iIndex] do begin
    StrPLCopy(Definition.BitmapFile, FBitmapPath + Bitmap, MAX_PATH_LEN);
    StrPLCopy(Definition.TitleText, Title, MAX_HINT_LEN);
    Definition.OwnerWindow := Owner;
    Definition.wMsg := Msg;
    Definition.wParam := wParam;
    Definition.lParam := lParam;
    Handle := SendMessage(FMDIChild, WM_F_ADDBUTTON, 0, Longint(@Definition));
  end;
  inc(FCount);
end;

procedure TButtonList.DeleteButton(Msg: Integer);
var
  iIndex: Integer;
begin
  iIndex := Msg - WM_FIRST_BUTTON;
  if iIndex < FCount then begin
    SendMessage(FMDIChild, WM_F_REMOVEBUTTON, FButtons^[iIndex].Handle, 0);
    FButtons^[iIndex].Handle := 0;
    dec(FCount);
  end;
end;

procedure TButtonList.ClearButtons;
var
  iIndex: Integer;
begin
  if FCount <> 0 then begin
    for iIndex := 0 to pred(FCapacity) do begin
      if FButtons^[iIndex].Handle <> 0 then SendMessage(FMDIChild, WM_F_REMOVEBUTTON, FButtons^[iIndex].Handle, 0);
    end;
    SetCapacity(0);
    FCount := 0;
  end;
end;

procedure TButtonList.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 8 then
    Delta := 16
  else if FCapacity > 4 then
    Delta := 8
  else Delta := 4;
  SetCapacity(FCapacity + Delta);
end;

procedure TButtonList.SetCapacity(NewCapacity: Integer);
begin
  ReallocMem(FButtons, NewCapacity * SizeOf(TButton));
  FCapacity := NewCapacity;
end;

function TButtonList.GetButtonTitle(Msg: Integer): String;
var
  iIndex: Integer;
begin
  iIndex := Msg - WM_FIRST_BUTTON;
  Result := FButtons^[iIndex].Definition.TitleText;
end;


// TPMInterface ------------------------------------------------------------------------------------

constructor TPMInterface.Create(Parent: THandle; const ResourcesPath: String);

  procedure GetVersionInfos;
  var
    iVersion: Integer;
  begin
    iVersion := SendMessage(FMDIChild, WM_F_GETVERSION, 0, 0);
    if iVersion = 0 then begin
      iVersion := GetSimpleFileVersion(GetBaseDirPath + PM_FILE_NAME, FALSE);
      iVersion := HiWord(iVersion)shl 8 + MulDiv(LoWord(iVersion), 16, 10);
    end;
    FPMVersion := 'PM' + SP + IntToStr(iVersion shr 8) + DOT + IntToHex(iVersion and $FF, 2);
    FPMVersionNumber := (iVersion and $F) * 10 + ((iVersion shr 4) and $F) * 100 + ((iVersion shr 8) and $F) * 1000;
  end;

  procedure GetLanguageInfos;

    function GetOrLoadModule(const Name: String): THandle;
    begin
      Result := GetModuleHandle(PChar(Name));
      if Result = 0 then Result := LoadLibrary(PChar(Name));
    end;

  const
    SEARCH_PATTERN = 'WPM32*';
  var
    hSearch: THandle;
    fdFound: TWin32FindData;
    sFileName: String;
    iLength: Integer;
  begin
    FLanguageCode := 'en';
    hSearch := FindFirst(GetBaseDirPath + SEARCH_PATTERN + '.DLL', fdFound);
    if hSearch <> INVALID_HANDLE_VALUE then begin
      try
	repeat
	  FLanguageModule := GetModuleHandle(fdFound.cFileName);
	  if FLanguageModule <> 0 then begin
	    sFileName := ExtractFileName(fdFound.cFileName);
	    iLength := Length(sFileName);
	    if (FPMVersionNumber >= 4800) or (iLength <= 12) then begin
	      FLanguageModule := ReloadModule(fdFound.cFileName);
	      if iLength <= 12 then
		FLanguageCode := LowerCase(Copy(sFileName, Length(SEARCH_PATTERN), 2))
	      else FLanguageCode := LowerCase(Copy(sFileName, succ(Length(SEARCH_PATTERN)), 2));
	      Break;
	    end;
	   end
	  else iLength := 0;
	until not FindNext(hSearch, fdFound);
	if (FLanguageModule = 0) or ((iLength > 12) and (FPMVersionNumber < 4800)) then FLanguageModule := GetModuleHandle(NIL);
      finally
	FindClose(hSearch);
      end;
    end;
  end;

  procedure GetDialogCaptions;
  begin
    FDlgAttachmentCheck := DialogCaptionString(FLanguageModule, DLG_ATTACHMENT_CHECK);
    FDlgSearchMessage := DialogCaptionString(FLanguageModule, DLG_SEARCH_MESSAGE);
    FDlgQueueCheck := DialogCaptionString(FLanguageModule, DLG_QUEUE_CHECK);
    FDlgReplyOptions := DialogCaptionString(FLanguageModule, DLG_REPLY_OPTIONS);
    FDlgSpellCheck := GetStringResource(STR_CHECK_SPELL_SUBJ);
    Delete(FDlgSpellCheck, succ(FirstCharPos(SP, FDlgSpellCheck)), MAXINT);
    if FDlgSpellCheck = E then FDlgSpellCheck := DialogCaptionString(FLanguageModule, DLG_SPELL_CHECK);
    FDlgPrintV4M := DialogCaptionString(FLanguageModule, DLG_PRINT_V4M);
    FDlgUserName := DialogCaptionString(FLanguageModule, DLG_USER_NAME);
    FDlgFontInfo := DialogFontInfo(FLanguageModule, DLG_FMV5);
  end;

  procedure GetLanguageOption;
  var
    iLen, iPos: Integer;
    sValue: String;
  begin
    iLen := Length(FCommandLine);
    iPos := FirstLoStrPos('-X', FCommandLine);
    if iPos = 0 then iPos := FirstLoStrPos('/X', FCommandLine);
    if iPos > 0 then begin
      inc(iPos, 3);
      while (iPos <= iLen) and (FCommandLine[iPos] <= SP) do inc(iPos);
      FLanguageCode := LowerCase(Copy(FCommandLine, iPos, 2));
    end
    else begin
      sValue := LowerCase(GetPMIniString(SEC_WPM_32, ID_SET_LANGUAGE));
      if (sValue  <> E) and not CompareTxt(sValue, FLanguageCode) then FLanguageCode := sValue;
    end;
  end;

begin
  inherited Create(Parent, ResourcesPath);
  if RunningAsAdmin(GetCurrentProcessID) then begin
    FShellAccessToken := GetShellAccessToken;
    FPMUserMainRegKey := UserProfileLoad(FShellAccessToken, Application.ExeName);
  end
  else FPMUserMainRegKey := HKEY_CURRENT_USER;
  FCommandLine := RegString(FPMUserMainRegKey, REG_KEY_PM_COMMAND, E);
  if FMDIChild <> 0 then begin
    GetVersionInfos;
    GetLanguageInfos;
    GetDialogCaptions;
  end
  else begin
    GetVersionInfos;
    GetLanguageInfos;
    GetLanguageOption;
  end;
  FMainLanguageID := GetMainLanguageID;
end;

destructor TPMInterface.Destroy;
begin
  if RunningAsAdmin(GetCurrentProcessID) then begin
    UserProfileUnload(FShellAccessToken, FPMUserMainRegKey);
    CloseHandle(FShellAccessToken);
  end;
  inherited;
end;

function TPMInterface.GetDefaultCharset: String;
begin
  Result := GetPrefString(W_PRO_CHARSET, 20);
  if Result = E then Result := 'WINDOWS-' + IntToStr(GetACP);
end;

function TPMInterface.GetDefaultBCCAddress(const Identity: String): String;
begin
  if ANSICompareText(Identity, Strings[STR_DEFAULT_ID]) = 0 then
    Result := GetIniFilePref(SEC_DEFAULT, ID_DEF_BCC_FIELD)
  else Result := GetIniFilePref(SEC_IDENTITIES + Identity, ID_DEF_BCC_FIELD);
end;

function TPMInterface.GlobalResolveAddresses(const Identity: String): Longbool;
var
  sPrefs: String;
  iPos, iIndex: Integer;
begin
  if ANSICompareText(Identity, Strings[STR_DEFAULT_ID]) = 0 then
    sPrefs := GetIniFilePref(SEC_DEFAULT, ID_INTEGER_PREFS_1)
  else sPrefs := GetIniFilePref(SEC_IDENTITIES + Identity, ID_INTEGER_PREFS_1);
  iPos := 1;
  for iIndex := 0 to 2 do iPos := succ(NextCharPos(SP, sPrefs, iPos));
  Result := (StrToIntDef(Copy(sPrefs, iPos, NextCharPos(SP, sPrefs, iPos) - iPos), 0) < 3);
end;

function TPMInterface.RestartPegasusMail(const PMPGPHelper, Command: String; Frame: THandle): Longbool;
var
  hProcess: THandle;
  dwProcessID, dwResult: DWord;
  sPMCommands, sPMPath: String;
  iBegPos, iEndPos, iLen: Integer;
begin
  Result := false;
  if FMDIChild <> 0 then
    FileExecute(0, E, PMPGPHelper, PM_RESTART + SP + Command + SP + IntToStr(Windows[wMAINFrame]), false)
  else begin
    if Frame <> 0 then begin
      GetWindowThreadProcessID(Frame, @dwProcessID);
      hProcess := OpenProcess(SYNCHRONIZE, false, dwProcessID);
      try
	PostMessage(Frame, WM_CLOSE, 0, 0);
	dwResult := WaitForSingleObject(hProcess, 10000);
      finally
	CloseHandle(hProcess);
      end;
    end
    else dwResult := WAIT_ABANDONED;
    if dwResult <> WAIT_TIMEOUT then begin
      if Command <> E then begin
	sPMCommands := FCommandLine;
	sPMPath := GetBaseDirPath + PM_FILE_NAME;
	Delete(sPMCommands, 1, pred(NextCharPos(SP, sPMCommands, succ(Length(sPMPath)))));
	iBegPos := FirstLoStrPos(' -I', sPMCommands);
	if iBegPos = 0 then iBegPos := FirstLoStrPos(' /I', sPMCommands);
	if iBegPos > 0 then begin
	  iEndPos := iBegPos + 4;
	  iLen := Length(sPMCommands);
	  while (iEndPos <= iLen) and (sPMCommands[iEndPos] <= SP) do inc(iEndPos);
	  while (iEndPos <= iLen) and (sPMCommands[iEndPos] > SP) do inc(iEndPos);
	  Delete(sPMCommands, iBegPos, iEndPos - iBegPos);
	end;
	sPMCommands := sPMCommands + ' -I ' + Command;
      end;
      Result := FileExecute(0, E, sPMPath, sPMCommands, true);
    end;
  end;
end;

function TPMInterface.NewMailMessage(const Recipient, Subject, Body: String): Longbool;
var
  hMessage: THandle;
begin
  Result := false;
  hMessage := SendMessage(FMDIChild, WM_F_OPENFILE, 255, 0);
  if hMessage <> 0 then begin
    SendMessage(hMessage, WM_F_SET_FIELD, 0, LPARAM(PChar(Recipient)));
    SendMessage(hMessage, WM_F_SET_FIELD, 1, LPARAM(PChar(Subject)));
    SendMessage(hMessage, WM_F_SET_FIELD, 3, LPARAM(PChar(Body)));
    PostMessage(hMessage, WM_F_SET_FIELD, 103, 0);
    Result := true;
  end;
end;

function TPMInterface.CopyMailMessage(var MDIEditor: THandle): String;
type
  TEditorData = Record
    TERInterface: TTERInterface;
    PMPGPStrings: TPMPGPStrings;
    MessageText: String;
    Recipient: String;
    Subject: String;
    CC: String;
    BCC: String;
    RepliesTo: String;
    KeyWords: String;
    Identity: String;
    CustomHeaders: String;
    CopySelf: Integer;
    Urgent: Integer;
    ConfirmReading: Integer;
    ConfirmDelivery: Integer;
    RichText: Integer;
    Signature: Integer;
    DontClose: Integer;
    UseMime: Integer;
    Units: Integer;
    Generate: Integer;
    Year: String;
    Month: String;
    Day: String;
    Hour: String;
    Minute: String;
    MsgWidth: String;
    TabWidth: String;
  end;

  procedure InitEditorData(var ED: TEditorData);
  begin
    FillByte(ED, SizeOf(TEditorData), 0);
    ED.TERInterface := FTERInterface;
    ED.PMPGPStrings := FPMPGPStrings;
  end;

  function GetEditorData(Window: THandle; var EditorData: TEditorData): Longbool;
  var
    hDialog: THandle;
  begin
    Result := true;
    with EditorData do begin
      MessageText := TERInterface.RtfReadMessage(BearOrTer(Window, false));
      hDialog := GetEditorMessagePage(Window);
      if hDialog <> 0 then begin
	Signature := GetSelectedCBXIndex(GetDlgItem(hDialog, CTL_SIGNATURE));
	Recipient := DialogItemString(hDialog, CTL_TO);
	Subject := DialogItemString(hDialog, CTL_SUBJECT);
	CC := DialogItemString(hDialog, CTL_CC);
	KeyWords := DialogItemString(hDialog, CTL_KEYWORDS);
	CopySelf := GetDlgBtnChecked(hDialog, CTL_COPYSELF);
	Urgent := GetDlgBtnChecked(hDialog, CTL_URGENT);
	ConfirmReading := GetDlgBtnChecked(hDialog, CTL_CONFIRM_READING);
	ConfirmDelivery := GetDlgBtnChecked(hDialog, CTL_CONFIRM_DELIVERY);
	RichText := GetDlgBtnChecked(hDialog, CTL_RICHTEXT);
	Identity := DialogItemString(hDialog, CTL_IDENTITY);
      end;
      hDialog := GetEditorSpecialPage(Window);
      if hDialog <> 0 then begin
	BCC := DialogItemString(hDialog, CTL_BCC);
	RepliesTo := DialogItemString(hDialog, CTL_REPLIES_TO);
	MsgWidth := DialogItemString(hDialog, CTL_MESSAGE_WIDTH);
	DontClose := GetDlgBtnChecked(hDialog, CTL_DONT_CLOSE);
	Year := DialogItemString(hDialog, CTL_YEAR);
	Month := DialogItemString(hDialog, CTL_MONTH);
	Day := DialogItemString(hDialog, CTL_DAY);
	Hour := DialogItemString(hDialog, CTL_HOUR);
	Minute := DialogItemString(hDialog, CTL_MINUTE);
	TabWidth := DialogItemString(hDialog, CTL_TAB_WIDTH);
	UseMime := GetDlgBtnChecked(hDialog, CTL_USE_MIME);
	Units := GetSelectedCBXIndex(GetDlgItem(hDialog, CTL_UNITS));
	Generate := GetDlgBtnChecked(hDialog, CTL_GENERATE);
	CustomHeaders := GetLBXText(GetDlgItem(hDialog, CTL_CUSTOM_HEADERS));
	if Identity = E then Identity := DialogItemString(hDialog, CTL_IDENTITY);
      end;
    end;
  end;

var
  hOldEditor, hNewEditor, hNewTer, hSpecial: THandle;
  edMessage: TEditorData;
  pHeader: PChar;
begin
  MDIEditor := 0;
  Result := GetDummyMessage;
  hOldEditor := CheckMDIEditor(0);
  try
    InitEditorData(edMessage);
    GetEditorData(hOldEditor, edMessage);
    with edMessage do begin
      hNewEditor := SendMessage(FMDIChild, WM_F_OPENFILE, 255, 0);
      if hNewEditor <> 0 then begin
	hNewTer := FTERInterface.FindTerWindow(hNewEditor);
	FTERInterface.MessageClear(hNewTer);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 0, Longint(Recipient));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 1, Longint(Subject));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 2, Longint(CC));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 10, Longint(BCC));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 14, Longint(Identity));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 16, Longint(RepliesTo));
	SetDlgItemText(GetEditorMessagePage(hNewEditor), CTL_KEYWORDS, PChar(KeyWords));
	pHeader := Pointer(CustomHeaders);
	if pHeader <> nil then while pHeader^ <> NULL do begin
	  SendMessage(hNewEditor, WM_F_SET_FIELD, 17, Longint(pHeader));
	  inc(pHeader, succ(FastStrLen(pHeader)));
	end;
	FTERInterface.RtfWriteMessage(hNewTer, MessageText);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 4, CopySelf);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 5, ConfirmDelivery);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 6, ConfirmReading);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 7, Urgent);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 11, pred(Signature));
	SendMessage(hNewEditor, WM_F_SET_FIELD, 19, RichText);
	hSpecial := GetEditorSpecialPage(hNewEditor);
	SetDialogCheckBox(GetDlgItem(hSpecial, CTL_DONT_CLOSE), Longbool(DontClose));
	SetDialogCheckBox(GetDlgItem(hSpecial, CTL_USE_MIME), Longbool(UseMime));
	SetDialogCheckBox(GetDlgItem(hSpecial, CTL_GENERATE), Longbool(Generate));
	SetDlgItemText(hSpecial, CTL_YEAR, PChar(Year));
	SetDlgItemText(hSpecial, CTL_MONTH, PChar(Month));
	SetDlgItemText(hSpecial, CTL_DAY, PChar(Day));
	SetDlgItemText(hSpecial, CTL_HOUR, PChar(Hour));
	SetDlgItemText(hSpecial, CTL_MINUTE, PChar(Minute));
	SetDlgItemText(hSpecial, CTL_MESSAGE_WIDTH, PChar(MsgWidth));
	SetDlgItemText(hSpecial, CTL_TAB_WIDTH, PChar(TabWidth));
	SelectCBXItemIndex(GetDlgItem(hSpecial, CTL_UNITS), Units);
	SendMessage(hNewEditor, WM_F_SET_FIELD, 103, 0);
	FTERInterface.SetCaretPos(hNewTer, 0);
      end;
    end;
    ProcessMessages(0);
    SetFocus(hNewEditor);
    MDIEditor := Windows[wMDIActiveChild];
    Info(FPMPGPStrings[MESSAGE_COPIED_INFO]);
  except
    Result := E;
  end;
end;

function TPMInterface.InsertMessageText(const Text: String): Longbool;
var
  hMDIEditor: THandle;
begin
  hMDIEditor := CheckMDIEditor(0);
  Result := (hMDIEditor <> 0);
  if Result then begin
    SendMessage(hMDIEditor, WM_F_SET_FIELD, 103, 0);
    SendMessage(hMDIEditor, WM_F_SET_FIELD, 12, Longint(Text));
    ProcessMessages(0);
  end;
end;

procedure TPMInterface.SendMailMessage(Simple, ForceMime: Longbool);
var
  hMDIEditor, hMainFrame: THandle;
  fMimeEnclosure, fMime8BitEncoding: Longbool;
begin
  hMDIEditor := CheckMDIEditor(0);
  if hMDIEditor <> 0 then begin
    if not Simple then begin
      hMainFrame := Windows[wMAINFrame];
      PostMessage(hMainFrame, WM_COMMAND, CMD_SPECIAL_TAB, 0);
      PostMessage(hMainFrame, WM_COMMAND, CMD_EDITOR_TAB, 0);
      if ForceMime then SendMessage(hMDIEditor, WM_F_SET_FIELD, 9, 1);
    end;
    fMimeEnclosure := MimeEnclosure;
    fMime8BitEncoding := Mime8BitEncoding;
    try
      if ForceMime then begin
	MimeEnclosure := true;
	Mime8BitEncoding := true;
      end;
      SendMessage(hMDIEditor, WM_COMMAND, CMD_SEND_MESSAGE, 0);
    finally
      Mime8BitEncoding := fMime8BitEncoding;
      MimeEnclosure := fMimeEnclosure;
    end;
  end;
end;

function TPMInterface.GetDummyMessage: String;
begin
  Result := 'To: ' + FPMPGPStrings[TO_DELETE] + CRLF + 'Subject: ' + FPMPGPStrings[SUBJECT_CANCELLED];
end;

function TPMInterface.RemoveBlankMessages: Integer;
var
  sMsgDir, sMsgExt, sMsgFile, sMsgText: String;
  hSearch: THandle;
  fdFound: TWin32FindData;
  fsMessage: TFileStream;
begin
  Result := 0;
  sMsgDir := GetMailBoxPath;
  hSearch := FindFirst(sMsgDir + '*.PM?', fdFound);
  if hSearch <> INVALID_HANDLE_VALUE then begin
    try
      repeat
	sMsgExt := UpperCase(ExtractFileExt(fdFound.cFileName));
	if (sMsgExt = EXT_DRAFT_MSG) or (sMsgExt = EXT_FINAL_MSG) then begin
	  inc(Result);
	  sMsgFile := sMsgDir + fdFound.cFileName;
	  if (fdFound.nFileSizeHigh = 0) and (fdFound.nFileSizeLow < 400) then begin
	    try
	      fsMessage := TFileStream.Create(sMsgFile, fmOpenRead);
	      if fsMessage <> nil then with fsMessage do begin
		try
		  if Size > 0 then begin
		    SetString(sMsgText, nil, Size);
		    Read(Pointer(sMsgText)^, Size);
		  end
		  else sMsgText := E;
		finally
		  fsMessage.Free;
		end;
		if (GetCharCount(LF, sMsgText) <= 6) and DeleteFile(sMsgFile) then dec(Result);
	      end;
	    except
	    end;
	  end;
	end;
      until not FindNext(hSearch, fdFound);
    finally
      FindClose(hSearch);
    end;
  end;
  RescanMessageQueue;
end;

procedure TPMInterface.RescanMessageQueue;
var
  hQueueManager: THandle;
begin
  hQueueManager := FindFirstChild(Windows[wMDIClient], PM_CLASS_MDI_QMAN, E, 0, false);
  if hQueueManager <> 0 then PostMessage(hQueueManager, WM_USER + 7476, 0, 0);
end;

procedure TPMInterface.ResetStatusBar;
begin
  SendMessage(FMDIChild, WM_F_SETSTATUS, 0, 0);
end;

procedure TPMInterface.SelectFrameMenuItem(MenuID: Integer);
begin
  PostMessage(Windows[wMAINFrame], WM_COMMAND, MenuID, 0);
end;

function TPMInterface.GetFrameMenuItemChecked(MenuID: Integer): Longbool;
begin
  Result := (GetMenuState(GetMenu(Windows[wMAINFrame]), MenuID, MF_BYCOMMAND) and MF_CHECKED <> 0);
end;

procedure TPMInterface.SetFrameMenuItemChecked(MenuID: Integer; Value: Longbool);
begin
  CheckMenuItem(GetMenu(Windows[wMAINFrame]), MenuID, Value);
end;

function TPMInterface.GetMDIMenuItemChecked(MenuID: Integer): Longbool;
begin
  Result := (GetMenuState(GetMenu(Windows[wMDIActiveChild]), MenuID, MF_BYCOMMAND) and MF_CHECKED <> 0);
end;

procedure TPMInterface.SendShutDownLog(TargetWindow: THandle; ForceShutDown: Longbool);
begin
  SendMessage(TargetWindow, WM_FM_LOGEVENT, WLOG_SHUTDOWN, ord(not ForceShutDown));
end;

function TPMInterface.BearOrTer(Parent: THandle; NewReader: Longbool): THandle;
begin
  Result := FTERInterface.FindBearOrTerWindow(Parent, NewReader and not IsRawView(0));
end;

function TPMInterface.CheckMDIQueueManager(MDIChild: THandle): THandle;
begin
  Result := MDIChild;
  if Result = 0 then Result := Windows[wMDIActiveChild];
  if (Result <> 0) and (ClassString(Result) <> PM_CLASS_MDI_QMAN) then Result := 0;
end;

function TPMInterface.CheckMDIFolman(MDIChild: THandle): THandle;
begin
  Result := MDIChild;
  if Result = 0 then Result := Windows[wMDIActiveChild];
  if (Result <> 0) and (ClassString(Result) <> PM_CLASS_FOLMAN) then Result := 0;
end;

function TPMInterface.CheckMDIFolder(MDIChild: THandle): THandle;
begin
  Result := MDIChild;
  if Result = 0 then Result := Windows[wMDIActiveChild];
  if (Result <> 0) and (ClassString(Result) <> PM_CLASS_MDI_FOLDER) then Result := 0;
end;

function TPMInterface.CheckMDIEditor(MDIChild: THandle): THandle;
begin
  Result := MDIChild;
  if Result = 0 then Result := Windows[wMDIActiveChild];
  if (Result <> 0) and (ClassString(Result) <> PM_CLASS_MDI_EDITOR) then Result := 0;
end;

function TPMInterface.GetEditorMessagePage(Parent: THandle): THandle;
begin
  Result := GetParent(FindStatic(Parent, PMPGPStrings[SEND_BUTTON]));
end;

function TPMInterface.GetEditorSpecialPage(Parent: THandle): THandle;
begin
  Result := GetParent(FindButton(Parent, PMPGPStrings[EDITOR_SETTINGS_GROUP], 0));
end;

function TPMInterface.GetEditorTabWidthTwips(MdiEditor: THandle): Integer;
begin
  Result := TwipsFromMilliMeters(StrToCardDef(Trim(GetIniFileString(ID_EDITOR_TABWIDTH, false, MdiEditor)), 0));
end;

function TPMInterface.CheckMDIReader(MDIChild: THandle): THandle;
var
  sClassName: String;
begin
  Result := MDIChild;
  if Result = 0 then Result := Windows[wMDIActiveChild];
  if Result <> 0 then begin
    sClassName := ClassString(Result);
    if (FirstStrPos(sClassName, PM_CLASS_MDI_READER) = 0)
    or ((sClassName = PM_CLASS_MDI_RV4)
    and (ClassString(GetWindow(Result, GW_CHILD)) <> PM_CLASS_READER_PANE)) then begin
      Result := 0;
    end;
  end;
end;

function TPMInterface.HasNewReader: Longbool;
begin
  Result := (FPMVersionNumber >= 4000);
end;

function TPMInterface.IsAttachmentView(MDIReader: THandle): Longbool;
var
  lButtons: TList;
begin
  Result := false;
  if MDIReader = 0 then MDIReader := Windows[wMDIActiveChild];
  lButtons := TList.Create;
  if lButtons <> nil then begin
    try
      Result := IsWindowVisible(FindFirstChild(MDIReader, PM_CLASS_LIST_WINDOW, E, 0, false)) and
		(HasNewReader or (FindDisabledChildren(MDIReader, PM_CLASS_BUTTON, lButtons) = 0));
    finally
      lButtons.Free;
    end;
  end;
end;

function TPMInterface.IsBodyView(MDIReader: THandle): Longbool;
var
  lButtons: TList;
  hToolbar: THandle;
begin
  Result := false;
  lButtons := TList.Create;
  if lButtons <> nil then begin
    if MDIReader = 0 then MDIReader := Windows[wMDIActiveChild];
    try
      if not HasNewReader then begin
	Result := (CheckMDIReader(MDIReader) = 0) or (
		    (
		      (FindFirstChild(MDIReader, PM_CLASS_LIST_WINDOW, E, 0, false) = 0) or
		      (FindDisabledChildren(MDIReader, PM_CLASS_BUTTON, lButtons) > 0)
		    )
		    and not GetFrameMenuItemChecked(IDM_SHOWHEADERS)
		  );
      end
      else if CheckMDIReader(MDIReader) <> 0 then begin
	if IsWindowVisible(MDIReader) then begin
	  hToolbar := FindFirstChild(MDIReader, PM_CLASS_TOOL_BAR, E, 0, false);
	  Result := IsWindowVisible(hToolbar) and (FindAllChildren(hToolbar, PM_CLASS_BUTTON, lButtons) > 10);
	end
	else Result := true;
      end;
    finally
      lButtons.Free;
    end;
  end;
end;

function TPMInterface.IsRawView(MDIReader: THandle): Longbool;
begin
  if MDIReader = 0 then MDIReader := Windows[wMDIActiveChild];
  if not HasNewReader then
    Result := (CheckMDIReader(MDIReader) <> 0) and GetFrameMenuItemChecked(IDM_SHOWHEADERS)
  else Result := IsWindowVisible(MDIReader) and not IsWindowVisible(FindFirstChild(MDIReader, PM_CLASS_TOOL_BAR, E, 0, false));
end;

function TPMInterface.IsReply(MDIEditor: THandle): Longbool;
begin
  if MDIEditor = 0 then MDIEditor := Windows[wMDIActiveChild];
  Result := (CheckMDIEditor(MDIEditor) <> 0) and LoStrStart(GetReplyMessageCaption, WndString(MDIEditor));
end;

function TPMInterface.IsMessageHandlingMDI: Longbool;
var
  hMDIWindow: THandle;
  sClassName: String;
begin
  Result := true;
  hMDIWindow := Windows[wMDIActiveChild];
  if hMDIWindow <> 0 then begin
    sClassName := ClassString(hMDIWindow);
    if (FirstStrPos(sClassName, PM_CLASS_FOLMAN + CM + PM_CLASS_MDI_FOLDER + CM +
				PM_CLASS_MDI_EDITOR + CM + PM_CLASS_MDI_READER) = 0)
    or ((sClassName = PM_CLASS_MDI_RV4)
    and (ClassString(GetWindow(hMDIWindow, GW_CHILD)) <> PM_CLASS_READER_PANE)) then begin
      Result := false;
    end;
  end
  else Result := false;
end;

function TPMInterface.CreateUtf8BannerHeader(UTCDateTime: TDateTime; const Utf8Sender, Utf8Subject: String): String;
var
  sl: TStringList;
  iIndex, iSize, iPos: Integer;
  sBuffer: String;
  pBuffer: PChar;
begin
  Result := AnsiToUtf8(GetPrintBannerHeader);
  if Result <> E then begin
    sl := TStringList.Create;
    if sl <> nil then with sl do begin
      try
	iIndex := 0;
	iSize := Length(Result);
	repeat
	  iIndex := NextCharPos(TILDE, Result, succ(iIndex));
	  if (iIndex > 0) and (iIndex < iSize) then begin
	    case UpCase(Result[succ(iIndex)]) of
	      WC_SENDER_FULL: sBuffer := Utf8Sender;
	      WC_SENDER_NAME: sBuffer := ExtractName(Utf8Sender);
	      WC_SENDER_FIRST: begin
		sBuffer := ExtractName(Utf8Sender);
		sBuffer := Copy(sBuffer, 1, pred(FirstCharPos(SP, sBuffer)));
	      end;
	      WC_PERSONAL_NAME: sBuffer := AnsiToUtf8(GetPersonalName);
	      WC_MSG_FORM_DATE: sBuffer := AnsiToUtf8(FormatDateTime('d mmm yyyy', UTCDateTime));
	      WC_MSG_FORM_TIME: sBuffer := AnsiToUtf8(FormatDateTime('h:nn', UTCDateTime) + SP + UTC);
	      WC_LONG_FORM_DATE: sBuffer := AnsiToUtf8(FormatDateTime('dddddd', UTCDateTime));
	      WC_SUBJECT: sBuffer := Utf8Subject;
	      WC_CRLF: sBuffer := CRLF;
	    else
	      Continue;
	    end;
	    if sBuffer = E then sBuffer := AnsiToUtf8(FPMPGPStrings[MISSING_HEADER]);
	    AddObject(sBuffer, Ptr(iIndex));
	  end
	  else Break;
	until iIndex = 0;
	iSize := Length(Result);
	for iIndex := 0 to pred(Count) do inc(iSize, Length(Strings[iIndex]) - 2);
	SetString(sBuffer, nil, iSize);
	pBuffer := Pointer(sBuffer);
	iPos := 1;
	for iIndex := 0 to pred(Count) do begin
	  iSize := Integer(Objects[iIndex]) - iPos;
	  System.Move(Result[iPos], pBuffer^, iSize);
	  inc(pBuffer, iSize);
	  iSize := Length(Strings[iIndex]);
	  System.Move(Pointer(Strings[iIndex])^, pBuffer^, iSize);
	  iPos := Integer(Objects[iIndex]) + 2;
	  inc(pBuffer, iSize);
	end;
	iSize := succ(Length(Result) - iPos);
	System.Move(Result[iPos], pBuffer^, iSize);
	inc(pBuffer, iSize);
	System.Delete(sBuffer, succ(pBuffer - Pointer(sBuffer)), MAXINT);
	Result := sBuffer;
      finally
	Free;
      end;
    end;
  end;
end;

function TPMInterface.GetDisplayInfo(MDIWindow: THandle; MessageType: TMessageType): TDisplayInfo;
begin
  FillChar(Result, SizeOf(TDisplayInfo), 0);
  Result.MessageType := MessageType;
  with Result do begin
    IsNewReader := HasNewReader;
    DefaultFont := ReaderFont;
    case MessageType of
      mtRead: begin
	AttachmentHint := FPMPGPStrings[ATTACHMENT_HINT];
	DigestHint := FPMPGPStrings[DIGEST_HINT];
	ClickHereHint := FPMPGPStrings[CLICK_HERE_HINT];
	UseAttachHint := FPMPGPStrings[USE_MENU_OR_ATTACH_HINT];
	WorkWithPartsHint := FPMPGPStrings[WORK_WITH_PARTS_HINT];
	EncodedBegin := FPMPGPStrings[SECTION_BEGIN] + FPMPGPStrings[ENCODED_SECTION];
	EncodedEnd := FPMPGPStrings[SECTION_END] + FPMPGPStrings[ENCODED_SECTION]
      end;
      mtReply: begin
        Signature := GetSignature;
	EditorTabWidthTwips := GetEditorTabWidthTwips(MDIWindow);
	OriginalMessageSeparator := GetOriginalMessageSeparator;
	ReplyHeaderOptions := GetReplyHeaderOptions(CustomHeaderBlankLines);
      end;
      mtForward: begin
	Signature := GetSignature;
	EditorTabWidthTwips := GetEditorTabWidthTwips(MDIWindow);
	ForwardedBegin := GetForwardedBegDelimiter;
	ForwardedEnd := GetForwardedEndDelimiter;
      end;
    end;
  end;
end;

function TPMInterface.GetFolderText(var FolderName: String; var UnixFolder: Longbool): String;

  function ReadNewMailData(const FolderPath: String): String;
  var
    hSearch: THandle;
    fdFound: TWin32FindData;
    sNewMail: String;
  begin
    Result := E;
    hSearch := FindFirst(FolderPath + AST + EXT_NEW_MAIL, fdFound);
    if hSearch <> INVALID_HANDLE_VALUE then begin
      try
	repeat
	  if ReadTextFromFile(FolderPath + fdFound.cFileName, sNewMail) then Result := Result + sNewMail + PM_MAIL_END;
	until not FindNext(hSearch, fdFound);
      finally
	FindClose(hSearch);
      end;
    end;
  end;

  function ReadFolderData(const FolderPath: String; FolderName, UnixFolder: Longbool): String;
  var
    fsBuffer: TFileStream;
    iSize: Integer;
  begin
    try
      fsBuffer := TFileStream.Create(FolderPath, fmOpenRead or fmShareDenyNone);
      if fsBuffer <> nil then with fsBuffer do begin
	try
	  if FolderName then
	    iSize := MAX_PATH
	  else begin
	    if not UnixFolder then begin
	      iSize := Size - FOLDER_HEADER_LEN;
	      Seek(FOLDER_HEADER_LEN, soFromBeginning);
	    end
	    else iSize := Size;
	  end;
	  SetString(Result, nil, iSize);
	  Read(Pointer(Result)^, iSize);
	  if FolderName then Delete(Result, FirstCharPos(#0, Result), MAXINT);
	finally
	  fsBuffer.Free;
	end;
      end
      else Raise EReadError.Create(E);
    except
      Error(FPMPGPStrings[FOLDER_READ_ERROR]);
      Result := E;
    end;
  end;

  function GetSubDirName(const Path: String): String;
  var
    iBeg, iEnd: Integer;
  begin
    if DirExists(Path) then begin
      iBeg := LastCharPos(BS, Path);
      iEnd := Length(Path);
      if iBeg = iEnd then begin
	dec(iEnd);
	iBeg := PrevCharPos(BS, Path, iEnd)
      end;
      Result := Copy(Path, succ(iBeg), iEnd - iBeg);
    end
    else Result := E;
  end;

var
  sFolderPath: String;
begin
  Result := E;
  sFolderPath := GetMailFolderPath(UnixFolder);
  if sFolderPath <> NULL then begin
    if sFolderPath <> E then begin
      SetCursor(LoadCursor(0, IDC_WAIT));
      try
	if sFolderPath[1] = TAB then begin
	  Delete(sFolderPath, 1, 1);
	  FolderName := Strings[STR_NEWMAIL_FOLDER_V3] + SP + LB + AnsiUpperCase(GetSubDirName(sFolderPath)) + RB;
	  Result := ReadNewMailData(sFolderPath);
	end
	else if UnixFolder then begin
	  FolderName := ReadFolderData(sFolderPath, true, true);
	  if FolderName <> E then Result := ReadFolderData(ChangeFileExt(sFolderPath, EXT_XMAIL_BOX), false, true);
	end
	else begin
	  FolderName := ReadFolderData(sFolderPath, true, false);
	  if FolderName <> E then Result := ReadFolderData(sFolderPath, false, false);
	end;
      finally
	SetCursor(LoadCursor(0, IDC_ARROW));
      end;
      if (FolderName <> E) and (Result = E) then Info(FPMPGPStrings[NO_FOLDER_DATA]);
    end
    else Error(FPMPGPStrings[FOLDER_FIND_ERROR]);
  end;
end;

function TPMInterface.CreateUnixMailBoxName(const FolderName: String): String;
begin
  Result := GetMailBoxPath + FolderName + EXT_XMAIL_BOX;
end;

function TPMInterface.GetSenderAddress(const Identity: String): String;
begin
  if ANSICompareText(Identity, Strings[STR_DEFAULT_ID]) = 0 then begin
    if FPMVersionNumber >= 4100 then
      Result := GetIniFilePref(SEC_DEFAULT, ID_EMAIL_FIELD)
    else Result := GetIniFilePref(SEC_TCP_IP, ID_ALT_FROM_FIELD);
  end
  else begin
    if FPMVersionNumber >= 4100 then
      Result := GetIniFilePref(SEC_IDENTITIES + Identity, ID_EMAIL_FIELD)
    else Result := GetIniFilePref(SEC_IDENTITIES + Identity, ID_ALT_FROM_FIELD);
  end;
end;

function TPMInterface.GetSignature: String;
var
  aSettings: Array[0..15] of Byte;
  iIndex, iSet: Integer;
  sIndex, sFile: String;
begin
  iIndex := GetPref(W_PRO_NOSIG);
  if iIndex > -1 then begin
    if iIndex > 0 then sIndex := DigitToHex(succ(iIndex));
    SendMessage(FMDIChild, WM_F_GETPREF, Longint(W_PRO_DEFSETTINGS), Longint(@aSettings));
    iSet := aSettings[14];
    case iSet of
      1, 2: begin
	sFile := GetMailBoxPath + SIG_FILE_LOCAL + sIndex;
	if (iSet = 1) or not ReadTextFromFile(sFile + EXT_FORMAT_SIG, Result) then begin
	  if ReadTextFromFile(sFile + EXT_PLAIN_SIG, Result) then begin
	    OemToChar(Pointer(Result), Pointer(Result));
	  end;
	end;
	Exit;
      end;
      3, 4: begin
	sFile := GetMailBoxPath + SIG_FILE_INET + sIndex;
	if (iSet = 3) or not ReadTextFromFile(sFile + EXT_FORMAT_SIG, Result) then begin
	  if ReadTextFromFile(sFile + EXT_PLAIN_SIG, Result) then begin
	    OemToChar(Pointer(Result), Pointer(Result));
	  end;
	end;
	Exit;
      end;
    end;
  end;
  Result := E;
end;

function TPMInterface.GetPMUser: String;
begin
  if FMDIChild <> 0 then begin
    SetString(Result, NIL, MAX_PATH);
    SendMessage(FMDIChild, WM_F_GETUSERNAME, 0, Longint(Result));
    Delete(Result, succ(FastStrLen(PChar(Result))), MAXINT);
  end
  else begin
    Result := RegString(FPMUserMainRegKey, REG_KEY_PM_USER_NAME, E);
    if Result = E then Result := FPMPGPStrings[ALL_PM_USERS];
  end;
end;

function TPMInterface.GetBaseDirPath: String;
begin
  if FBaseDirPath = E then begin
    if FMDIChild <> 0 then
      Result := GetDirString(WM_F_GETBASEDIR)
    else begin
      Result := RegString(FPMUserMainRegKey, REG_KEY_PM_BASEDIR, E);
      AddBackslash(Result, Length(Result));
    end;
    FBaseDirPath := Result;
  end
  else Result := FBaseDirPath;
end;

function TPMInterface.GetMailBoxPath: String;
const
  WINPMAIL = 'WinPMail';
  PEGASUS = 'Pegasus';
  PMAIL = 'PMail';
  PM = 'PM';
var
  sProgramFiles, sSystemDrive, sSearchPath: String;
begin
  if FMailBoxPath = E then begin
    if FMDIChild <> 0 then
      Result := GetDirString(WM_F_GETHOMEBOX)
    else begin
      Result := RegString(FPMUserMainRegKey, REG_KEY_PM_MAILBOX, E);
      AddBackslash(Result, Length(Result));
      if Result = E then begin
	sProgramFiles := GetSystemFolderPath(CSIDL_PROGRAM_FILES);
	sSystemDrive := ExtractFileDrive(sProgramFiles) + BS;
	sSearchPath := sProgramFiles + PEGASUS_MAIL + SC + sProgramFiles + PEGASUS + SC +
		       sProgramFiles + PMAIL + SC + sProgramFiles + PM + SC + sProgramFiles + WINPMAIL + SC +
		       sSystemDrive + PEGASUS_MAIL + SC + sSystemDrive + PEGASUS + SC +
		       sSystemDrive + PMAIL + SC + sSystemDrive + PM + SC + sSystemDrive + WINPMAIL + SC;
	repeat
	  Result := FindDirectory(GetDesktopWindow, sSearchPath, PM_FILE_NAME, FPMPGPStrings[FIND_PM_PROMPT]);
	until (Result = E) or FileExists(Result + PM_FILE_NAME);
      end;
    end;
    FMailBoxPath := Result;
  end
  else Result := FMailBoxPath;
end;

function TPMInterface.GetNewMailPath: String;
begin
  if FNewMailPath = E then begin
    if FMDIChild <> 0 then
      Result := GetDirString(WM_F_GETNEWMAILBOX)
    else begin
      Result := RegString(FPMUserMainRegKey, REG_KEY_PM_NEWMAIL, E);
      AddBackslash(Result, Length(Result));
    end;
    FNewMailPath := Result;
  end
  else Result := FNewMailPath;
end;

function TPMInterface.GetGlobalAddressBooksPath: String;
var
  sAddrBookVar: String;
  iLen: Integer;
begin
  sAddrBookVar := Copy(EXT_ADDR_BOOK, 2, MAXINT);
  Result := GetPMIniString(SEC_WPM_32_ENV, sAddrBookVar);
  if Result = E then begin
    SetString(Result, nil, MAX_PATH);
    iLen := GetEnvironmentVariable(Pointer(sAddrBookVar), Pointer(Result), MAX_PATH);
    AddBackslash(Result, iLen);
  end;
end;

function TPMInterface.GetAddressBooksPathList: String;
begin
  Result := Trim(GetMailBoxPath + LF + GetBaseDirPath + LF + GetGlobalAddressBooksPath);
end;

function TPMInterface.GetAddressBookFlagsList: String;
var
  sFlagSection, sPMUser: String;
  iLen, iUser, iStart, iIndex, iCopy: Integer;
  pResult: PChar;
  fUserPrefix: Longbool;
begin
  sFlagSection := ReadIniFileSectionString(GetMailBoxPath + STATE_PMJ, SEC_ADDRESS_BOOKS);
  if sFlagSection <> E then begin
    iLen := Length(sFlagSection);
    SetString(Result, nil, iLen);
    pResult := Pointer(Result);
    sPMUser := GetPMUser + DOT;
    iUser := Length(sPMUser);
    iStart := 1;
    repeat
      iIndex := succ(iStart);
      fUserPrefix := LoStrAtPos(sPMUser, sFlagSection, iStart) and (sFlagSection[iStart + iUser] in ['0'..'9']);
      if fUserPrefix and (FPMVersionNumber >= 4000) or not fUserPrefix and (FPMVersionNumber < 4000) then begin
	if fUserPrefix then begin
	  inc(iStart, Length(sPMUser));
	  while (iStart <= iLen) and (sFlagSection[iStart] <> DOT) do inc(iStart);
	  inc(iStart);
	  iIndex := iStart;
	end;
	while (iIndex <= iLen) and (sFlagSection[iIndex] <> EQ) do inc(iIndex);
	inc(iIndex);
	iCopy := iIndex - iStart;
	Move(sFlagSection[iStart], pResult^, iCopy);
	inc(pResult, iCopy);
	while (iIndex <= iLen) and (sFlagSection[iIndex] <> QU) do inc(iIndex);
	inc(iIndex);
	iStart := iIndex;
	while (iIndex <= iLen) and (sFlagSection[iIndex] <> QU) do inc(iIndex);
	iCopy := iIndex - iStart;
	Move(sFlagSection[iStart], pResult^, iCopy);
	inc(pResult, iCopy);
	pResult^ := LF;
	inc(pResult);
      end;
      while (iIndex <= iLen) and (sFlagSection[iIndex] <> NULL) do inc(iIndex);
      iStart := succ(iIndex);
    until iIndex >= iLen;
    Delete(Result, succ(pResult - Pointer(Result)), MAXINT);
  end;
end;

function TPMInterface.GetMainLanguageID: LANGID;
begin
  Result := GetLangIDFromTLD(FLanguageCode);
end;

function TPMInterface.GetPref(PMOption: TPMOptions): Variant;
begin
  Result := SendMessage(FMDIChild, WM_F_GETPREF, Longint(PMOption), 0);
end;

procedure TPMInterface.SetPref(PMOption: TPMOptions; const Value: Variant);
begin
  SendMessage(FMDIChild, WM_F_SETPREF, Longint(PMOption), Longint(Value));
end;

function TPMInterface.GetPrefString(PMOption: TPMOptions; Size: Integer): String;
begin
  SetString(Result, nil, Size);
  SendMessage(FMDIChild, WM_F_GETPREF, Longint(PMOption), Longint(Result));
  Delete(Result, succ(FastStrLen(Pointer(Result))), MAXINT);
end;

function TPMInterface.GetDirString(DirID: Integer): String;
var
  iLen: Integer;
begin
  SetString(Result, nil, succ(MAX_PATH));
  SendMessage(FMDIChild, DirID, succ(MAX_PATH), Longint(Result));
  iLen := FastStrLen(Pointer(Result));
  AddBackslash(Result, iLen);
end;

function TPMInterface.GetIniFilePref(const Section, ID: String): String;
begin
  Result := Trim(ReadIniFileString(GetNewMailPath + PMAIL_INI, Section, ID));
end;

function TPMInterface.GetPMIniString(const Section, Value: String): String;
begin
  Result := Trim(ReadIniFileString(GetBaseDirPath + PEGASUS_INI, Section, Value));
  if Result = E then Result := Trim(ReadIniFileString(GetBaseDirPath + WINPMAIL_INI, Section, Value));
end;

function TPMInterface.GetEditorIdentity(MdiEditor: THandle): String;

  function GetIdentity(Window: THandle; var Identity: String): Bool; stdcall;
  begin
    if (ClassString(Window) = CLASS_CBX) and (GetDlgCtrlID(Window) = CTL_IDENTITY) then begin
      Identity := WndString(Window);
      Result := false;
      Exit;
    end;
    Result := true;
  end;

begin
  Result := E;
  MDIEditor := CheckMDIEditor(MDIEditor);
  if MDIEditor <> 0 then EnumChildWindows(MDIEditor, @GetIdentity, Longint(@Result));
end;

function TPMInterface.GetIniFileString(const Name: String; Global: Longbool; MdiEditor: THandle): String;
var
  sIniFile, sID: String;
begin
  sIniFile := GetNewMailPath + PMAIL_INI;
  if Global then
    sID := RegString(HKEY_CURRENT_USER, REG_KEY_PM_IDENTITY, E)
  else sID := GetEditorIdentity(MdiEditor);
  if (sID = E) or SameText(sID, ModuleString(FLanguageModule, STR_DEFAULT_ID)) then
    Result := ReadIniFileString(sIniFile, SEC_DEFAULT, Name)
  else Result := ReadIniFileString(sIniFile, SEC_IDENTITIES + sID, Name);
end;

procedure TPMInterface.SetIniFileString(const Name, Value: String; Global: Longbool; MdiEditor: THandle);
var
  sIniFile, sID: String;
begin
  sIniFile := GetNewMailPath + PMAIL_INI;
  if Global then
    sID := RegString(HKEY_CURRENT_USER, REG_KEY_PM_IDENTITY, E)
  else sID := GetEditorIdentity(MdiEditor);
  if (sID = E) or SameText(sID, ModuleString(FLanguageModule, STR_DEFAULT_ID)) then
    WriteIniFileString(sIniFile, SEC_DEFAULT, Name, SP + Value)
  else WriteIniFileString(sIniFile, SEC_IDENTITIES + sID, Name, SP + Value);
end;

function TPMInterface.GetReplyHeaderOptions(var CustomBlankLines: Integer): TReplyHeaderOptions;
const
  CH_BLANK_LINE = TILDE + WC_CRLF + TILDE + WC_CRLF;
var
  sReplyHeader: String;
begin
  Result := [];
  CustomBlankLines := 0;
  if GetPref(W_PRO_OMIT_HEADERS) = false then Include(Result, rhoOriginal);
  if GetPref(W_PRO_USE_CUSTOM_REPLY) then Include(Result, rhoCustom);
  if Result * [rhoCustom] <> [] then begin
    sReplyHeader := GetPrefString(W_PRO_CUSTOM_REPLY, 64);
    if sReplyHeader <> E then
      CustomBlankLines := GetStrCount(CH_BLANK_LINE, sReplyHeader)
    else if Result * [rhoOriginal] = [] then CustomBlankLines := -1;
  end;
end;

function TPMInterface.GetMenuHandle(Index: Integer): THandle;
begin
  Result := SendMessage(FMDIChild, WM_F_GETMENU, Index, 0);
end;

function TPMInterface.GetWindowHandle(PMWindow: TPMWindow): THandle;
begin
  Result := SendMessage(FMDIChild, WM_F_GETWINDOW, ord(PMWindow), 0);
end;

function TPMInterface.GetStringResource(Index: Cardinal): String;
begin
  Result := ModuleString(FLanguageModule, Index);
end;

function TPMInterface.GetOriginalMessageSeparator: String;
begin
  Result := Strings[STR_ORIGINAL_MESSAGE];
end;

function TPMInterface.GetForwardedBegDelimiter: String;
begin
  Result := Strings[STR_FORWARDED_BEG];
end;

function TPMInterface.GetForwardedEndDelimiter: String;
begin
  Result := Strings[STR_FORWARDED_END];
end;

function TPMInterface.GetNewMailFolderCaption: String;
begin
  if FPMVersionNumber >= 4000 then
    Result := Strings[STR_NEWMAIL_FOLDER_V4]
  else Result := Strings[STR_NEWMAIL_FOLDER_V3];
end;

function TPMInterface.GetNoAttachmentCaption: String;
begin
  Result := Strings[STR_NO_ATTACHMENT];
end;

function TPMInterface.GetReplyMessageCaption: String;
begin
  Result := Strings[STR_REPLY_CAPTION];
  Delete(Result, FirstStrPos('%d', Result), MAXINT);
end;

function TPMInterface.GetBrowserPreference: String;
begin
  SetString(Result, nil, MAX_PATH);
  SendMessage(FMDIChild, WM_F_GETPREF, Longint(W_PRO_WEB_BROWSER), Longint(Result));
  Delete(Result, succ(FastStrLen(Pointer(Result))), MAXINT);
end;

function TPMInterface.GetReloadsWinsock: Longbool;
begin
  Result := (GetPref(W_PRO_NO_TCP) = false) and (GetPref(W_PRO_POPOPTS) and $40 <> 0);
end;

function TPMInterface.GetScanLines: Integer;
begin
  Result := GetPref(W_PRO_ATTACH_SCAN_LINES);
end;

procedure TPMInterface.SetScanLines(Value: Integer);
begin
  SetPref(W_PRO_ATTACH_SCAN_LINES, Value);
end;

function TPMInterface.GetEditMargin: Integer;
begin
  Result := GetPref(W_PRO_DEFRMARGIN);
end;

procedure TPMInterface.SetEditMargin(Value: Integer);
begin
  SetPref(W_PRO_DEFRMARGIN, Value);
end;

function TPMInterface.GetMimeEnclosure: Longbool;
begin
  Result := GetPref(W_PRO_BULKMAIL);
end;

procedure TPMInterface.SetMimeEnclosure(Value: Longbool);
begin
  SetPref(W_PRO_BULKMAIL, ord(Value = true));
end;

function TPMInterface.GetMime8BitEncoding: Longbool;
begin
  Result := GetPref(W_PRO_USE_8BIT);
end;

procedure TPMInterface.SetMime8BitEncoding(Value: Longbool);
begin
  SetPref(W_PRO_USE_8BIT, ord(Value = true));
end;

function TPMInterface.GetOmitAttachmentInfo: Longbool;
begin
  Result := (GetPref(W_PRO_EDMAIL_FORMAT) and (1 shl 9) <> 0);
end;

procedure TPMInterface.SetOmitAttachmentInfo(Value: Longbool);
begin
  if Value then
    SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) or (1 shl 9))
  else SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) and not (1 shl 9));
end;

function TPMInterface.GetEditorAutoSign: Longbool;
begin
  Result := (GetPref(W_PRO_EDMAIL_FORMAT) and (1 shl 1) <> 0);
end;

procedure TPMInterface.SetEditorAutoSign(Value: Longbool);
begin
  if Value then
    SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) or (1 shl 1))
  else SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) and not (1 shl 1));
end;

function TPMInterface.GetEditorFancyStyle: Longbool;
begin
  Result := (GetPref(W_PRO_EDMAIL_FORMAT) and (1 shl 20) <> 0);
end;

procedure TPMInterface.SetEditorFancyStyle(Value: Longbool);
begin
  if Value then
    SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) or (1 shl 20))
  else SetPref(W_PRO_EDMAIL_FORMAT, GetPref(W_PRO_EDMAIL_FORMAT) and not (1 shl 20));
end;

function TPMInterface.GetReaderFancyView: Longbool;
begin
  Result := (GetPref(W_PRO_READER_FORMAT) and (1 shl 12) <> 0);
end;

procedure TPMInterface.SetReaderFancyView(Value: Longbool);
begin
  if Value then
    SetPref(W_PRO_READER_FORMAT, GetPref(W_PRO_READER_FORMAT) or (1 shl 12))
  else SetPref(W_PRO_READER_FORMAT, GetPref(W_PRO_READER_FORMAT) and not (1 shl 12));
end;

function TPMInterface.GetReaderHotUrls: Longbool;
begin
  Result := (GetPref(W_PRO_HOT_URLS) and 1 <> 0);
end;

procedure TPMInterface.SetReaderHotUrls(Value: Longbool);
begin
  if Value <> GetReaderHotUrls then SetPref(W_PRO_HOT_URLS, Integer(GetPref(W_PRO_HOT_URLS)) xor Ord(Value = true));
end;

function TPMInterface.GetReaderWordWrap: TWordWrap;
begin
  Result := GetPref(W_PRO_WORDWRAP);
end;

procedure TPMInterface.SetReaderWordWrap(Value: TWordWrap);
begin
  SetPref(W_PRO_WORDWRAP, ord(Value));
end;

function TPMInterface.GetReaderFont: TPMPGPFont;
var
  fdReader: TFontDef;
begin
  SendMessage(FMDIChild, WM_F_GETPREF, Longint(W_PRO_READERFONT), Longint(@fdReader));
  Result := PMFont2PMPGPFont(fdReader);
end;

function TPMInterface.GetCurrentEditorID: String;
begin
  Result := GetEditorIdentity(0);
end;

function TPMInterface.GetCurrentGlobalID: String;
begin
  Result := RegString(FPMUserMainRegKey, REG_KEY_PM_IDENTITY, E);
end;

function TPMInterface.GetPersonalName: String;
begin
  SetString(Result, nil, MAX_PATH);
  SendMessage(FMDIChild, WM_F_GETPREF, Longint(W_PRO_PERSONAL_NAME), Longint(Result));
  Delete(Result, succ(FastStrLen(Pointer(Result))), MAXINT);
end;

function TPMInterface.GetToRecipients: String;
begin
  Result := Trim(PChar(SendMessage(FMDIChild, WM_F_GETMESSAGEFIELD, WMSG_TO, 0)));
end;

function TPMInterface.GetCCRecipients: String;
begin
  Result := Trim(PChar(SendMessage(FMDIChild, WM_F_GETMESSAGEFIELD, WMSG_CC, 0)));
end;

function TPMInterface.GetBCCRecipients: String;
begin
  Result := Trim(PChar(SendMessage(FMDIChild, WM_F_GETMESSAGEFIELD, WMSG_BCC, 0)));
end;

function TPMInterface.GetMessageHeaderLineCount: Integer;
var
  sFolderText, sFolderName, sHeaders: String;
  fUnixFolder: Longbool;
  iPos, iIndex, iCount: Integer;
begin
  Result := 0;
  sFolderText := GetFolderText(sFolderName, fUnixFolder);
  if sFolderText <> E then begin
    iPos := Length(sFolderText);
    if fUnixFolder then begin
      for iIndex := 0 to 9 do begin
	repeat
	  iPos := succ(PrevStrPos(UNIX_MAIL_START, sFolderText, iPos - Length(UNIX_MAIL_START)));
	until (iPos < 3) or StrAtPos(LF + LF, sFolderText, iPos - 2)
	or (iPos < 5) or StrAtPos(PARA, sFolderText, iPos - 4);
	if (iPos = 1) or StrAtPos(LF + LF, sFolderText, iPos - 2) then
	  sHeaders := Copy(sFolderText, iPos, NextStrPos(LF + LF, sFolderText, iPos) - iPos)
	else begin
	  sHeaders := Copy(sFolderText, iPos, NextStrPos(PARA, sFolderText, iPos) - iPos);
	  if sHeaders = E then sHeaders := Copy(sFolderText, iPos, NextStrPos(LF + LF, sFolderText, iPos) - iPos);
	end;
	iCount := GetCharCount(LF, sHeaders) + 2;
	if iCount > Result then Result := iCount;
	if iPos < (Result shl 1) then
	  Break
	else dec(iPos);
      end;
    end
    else begin
      for iIndex := 0 to 9 do begin
	iPos := succ(PrevCharPos(PM_MAIL_END, sFolderText, iPos - Length(PM_MAIL_END)));
	sHeaders := Copy(sFolderText, iPos, NextStrPos(PARA, sFolderText, iPos) - iPos);
	iCount := GetCharCount(LF, sHeaders) + 2;
	if iCount > Result then Result := iCount;
	if iPos < (Result shl 1) then
	  Break
	else dec(iPos);
      end;
    end;
  end;
end;

function TPMInterface.GetEncryptedMessage: Longbool;
begin
  Result := (SendMessage(FMDIChild, WM_F_GETMESSAGEFIELD, WMSG_ENCRYPT, 0) <> 0);
end;

function TPMInterface.GetPrinterFont: TPMPGPFont;
var
  fdPrinter: TFontDef;
begin
  SendMessage(FMDIChild, WM_F_GETPREF, Longint(W_PRO_PRINTERFONT), Longint(@fdPrinter));
  Result := PMFont2PMPGPFont(fdPrinter);
end;

function TPMInterface.GetPrinterName: String;
var
  sGlobalID: String;
begin
  sGlobalID := GetCurrentGlobalID;
  if ANSICompareText(sGlobalID, Strings[STR_DEFAULT_ID]) = 0 then
    Result := GetIniFilePref(SEC_DEFAULT, ID_PRINTER_DEVICE)
  else Result := GetIniFilePref(SEC_IDENTITIES + sGlobalID, ID_PRINTER_DEVICE);
end;

function TPMInterface.GetPrintBannerHeader: String;
var
  sGlobalID: String;
begin
  sGlobalID := GetCurrentGlobalID;
  if ANSICompareText(sGlobalID, Strings[STR_DEFAULT_ID]) = 0 then
    Result := GetIniFilePref(SEC_DEFAULT, ID_PRINTING_BANNER)
  else Result := GetIniFilePref(SEC_IDENTITIES + sGlobalID, ID_PRINTING_BANNER);
end;

function TPMInterface.GetPrintHeader: Longbool;
begin
  Result := (GetPref(W_PRO_HPRINT) and (1 shl 8) = 0);
end;

function TPMInterface.GetPrintFooter: Longbool;
begin
  Result := (GetPref(W_PRO_HPRINT) and (1 shl 6) = 0);
end;

function TPMInterface.GetPrintHeaders: TPrintHeaders;
begin
  case GetPref(W_PRO_HPRINT) and 3 of
    0..1: Result := phSignificant;
    2: Result := phAll;
  else
    Result := phNone;
  end;
end;

function TPMInterface.GetPrintMargins: TRect;
begin
  with Result do begin
    Left := GetPref(W_PRO_LRMARGIN);
    Top := GetPref(W_PRO_TBMARGIN);
    Right := Left;
    Bottom := Top;
  end;
end;

procedure TPMInterface.SetMenuInfo(const Text: String);
begin
  SendMessage(FMDIChild, WM_F_SETSTATUS, 3, Longint(Text));
end;

procedure TPMInterface.SetStatusBarLongInfo(const Text: String);
begin
  if Text <> E then
    SendMessage(FMDIChild, WM_F_SETSTATUS, 3, Longint(Text))
  else begin
    SendMessage(FMDIChild, WM_F_SETSTATUS, 3, 0);
    ResetStatusBar;
  end;
end;

procedure TPMInterface.SetStatusBarShortInfo(const Text: String);
begin
  if Text <> E then
    SendMessage(FMDIChild, WM_F_SETSTATUS, 0, Longint(Text))
  else begin
    SendMessage(FMDIChild, WM_F_SETSTATUS, 0, 0);
    ResetStatusBar;
  end;
end;

function TPMInterface.GetMailFolderPath(var UnixFolder: Longbool): String;

  function GetMailBoxList: TStringList;
  var
    iDefault, iIndex, iPos: Integer;
    sBuffer, sHash: String;
  begin
    Result := TStringList.Create;
    if Result <> nil then with Result do begin
      sBuffer := ReadIniFileSectionString(GetMailBoxPath + STATE_PMJ, SEC_MAILBOX_IDS);
      XChgChars(sBuffer, NULL, LF);
      Text := sBuffer;
      iDefault := -1;
      for iIndex := 0 to pred(Count) do begin
	sBuffer := Names[iIndex];
	sHash := Strings[iIndex];
	iPos := NextCharPos(QU, sHash, Length(sBuffer) + 2);
	System.Delete(sHash, NextCharPos(CL, sHash, succ(iPos)), MAXINT);
	System.Delete(sHash, 1, iPos);
	Objects[iIndex] := Ptr(StrToInt(US + sHash));
	if UpperCase(sBuffer) <> 'DEFAULT' then begin
	  AddBackSlash(sBuffer, Length(sBuffer));
	  Strings[iIndex] := sBuffer;
	end
	else begin
	  Strings[iIndex] := GetMailBoxPath;
	  iDefault := iIndex;
	end;
      end;
      if iDefault > 0 then Move(iDefault, 0);
    end;
  end;

  function FindFolderFile(const BaseName, Hash: String; var UnixFolder: Longbool): String;
  var
    slMailBoxes: TStringList;
    iIndex: Integer;
  begin
    slMailBoxes := GetMailBoxList;
    if slMailBoxes <> nil then begin
      try
	if FirstCharPos(CL, BaseName) = 0 then begin
	  for iIndex := 0 to pred(slMailBoxes.Count) do begin
	    Result := slMailBoxes[iIndex] + BaseName + EXT_MSG_FOLDER;
	    if not FileExists(Result) then begin
	      Result := ChangeFileExt(Result, EXT_XMAIL_INDEX);
	      UnixFolder := FileExists(Result);
	      if not UnixFolder then Result := E;
	    end;
	    if Result <> E then Break;
	  end;
	end
	else with slMailBoxes do begin
	  iIndex := IndexOfObject(Ptr(StrToInt(US + Hash)));
	  if iIndex <> -1 then Result := TAB + Strings[iIndex];
	end;
      finally
	slMailBoxes.Free;
      end;
    end;
  end;

var
  iPos: Integer;
  sHash, sSysFolders: String;
begin
  Result := E;
  UnixFolder := false;
  SetString(Result, nil, MAX_PATH);
  FillChar(Pointer(Result)^, MAX_PATH, 0);
  if SendMessage(FMDIChild, WM_F_SELECTOR, 0, Longint(Result)) <> 0 then begin
    Delete(Result, succ(FastStrLen(PChar(Result))), MAXINT);
    ProcessMessages(0);
    if not StrStart(GetNewMailFolderCaption, Result) then begin
      iPos := FirstCharPos(CL, Result);
      if iPos > 0 then begin
	sHash := Copy(Result, 1, pred(iPos));
	iPos := LastCharPos(BS, Result);
	if iPos = 0 then iPos := LastCharPos(CL, Result);
	Delete(Result, 1, iPos);
	if LastCharPos(DOT, Result) = 0 then Result := FindFolderFile(Result, sHash, UnixFolder);
      end
      else begin
	sSysFolders := GetPMIniString(SEC_SYS_FOLDERS, ID_PUBLIC_TREE);
	sSysFolders := TrimCustom(Copy(sSysFolders, succ(FirstCharPos(CM, sSysFolders)), MAXINT), [SP, QU]);
	sSysFolders := XChgStrings(sSysFolders, BS + BS, BS);
	Result := sSysFolders + BS + Result;
      end;
    end
    else Result := TAB + GetNewMailPath;
  end
  else Result := NULL;
end;


end.

