{$J+,Z4}
unit PMPGPCommon;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, SysUtils, StrUtil, TimeUtils, WindowUtils, WebInterface, KeyPropTypes;

const // Export/Import indices
  FORMINIT_INDEX	=  1;
  FORMPROC_INDEX	=  2;
  ENCRYPT_INDEX		=  3;
  DECRYPT_INDEX		=  4;
  SIGN_INDEX		=  5;
  VERIFY_INDEX		=  6;
  KEY_MANAGE_INDEX	=  7;
  DESTROY_FILE_INDEX	=  8;

  USER_SETUP_INDEX	= 10;
  USER_UNINSTALL_INDEX	= 11;
  USER_UPDATE_INDEX	= 12;
  PM_RESTART_INDEX	= 13;
  SYNCTIME_INDEX	= 14;
  HTML_PAGE_INDEX	= 15;
  PGP_HELP_INDEX	= 16;
  MANUAL_INDEX		= 17;
  CLEANUP_INDEX		= 18;

const // Basic registry strings
  REG_KEY_SOFTWARE_MS_WIN	= 'Software\Microsoft\Windows';
  REG_KEY_CURRENT_VERSION	= 'CurrentVersion';
  REG_KEY_WIN_CUR_VER		= REG_KEY_SOFTWARE_MS_WIN + BS + REG_KEY_CURRENT_VERSION;
  REG_KEY_NT_CUR_VER		= REG_KEY_SOFTWARE_MS_WIN + SP + 'NT' + BS + REG_KEY_CURRENT_VERSION;
  REG_KEY_UNINSTALL		= REG_KEY_WIN_CUR_VER + BS + 'Uninstall\Pegasus Mail PGP Utilities_is1';
  REG_ENTRY_UNINSTALL		= 'UninstallString';
  REG_KEY_IE_ZONE_MY_COMP	= REG_KEY_WIN_CUR_VER + BS + 'Internet Settings\Zones\0';
  REG_ENTRY_ALLOW_META_REFRESH	= '1608';
  REG_KEY_DATETIME_SERVERS 	= REG_KEY_WIN_CUR_VER + BS + 'DateTime\Servers';
  REG_KEY_CURRENTCONTROLSET	= 'System\CurrentControlSet';
  REG_KEY_DATETIME_PARAMS	= 'Services\W32Time\Parameters';
  REG_KEY_CCS_DT_PARAMS		= REG_KEY_CURRENTCONTROLSET + BS + REG_KEY_DATETIME_PARAMS;
  REG_ENTRY_NTPSERVER		= 'NtpServer';

const // Miscellaneous strings
  IDW			= 'idw';
  AIO			= '***';
  FRONTEND_NAME		= 'idw''s PGP-Frontend';
  FRONTEND_ITEM		= 'idw''s &PGP-Frontend';
  FRONTEND_X_PM		= 'idw''s X-PM-PGP Handler';
  PGP_FRONTEND		= '''s PGP-Frontend';
  PMIME_HANDLER		= '''s PGP/MIME Handler';
  SMIME_HANDLER		= '''s S/MIME Handler';
  FRONTEND_INLINE	= IDW + PGP_FRONTEND;
  FRONTEND_INLINE_AIO	= AIO + PGP_FRONTEND;
  FRONTEND_PMIME	= IDW + PMIME_HANDLER;
  FRONTEND_PMIME_AIO	= AIO + PMIME_HANDLER;
  FRONTEND_SMIME	= IDW + SMIME_HANDLER;
  FRONTEND_SMIME_AIO	= AIO + SMIME_HANDLER;

  MAIN_MODULE_NAME	= 'PMPGP Main Module';
  CRYPTOR_LIB		= 'PMPGPCryptor.dll';
  HELPER_EXE		= 'PMPGPHelper.exe';
  ADMIN_EXE		= 'PMPGPAdmin.exe';
  MAIN_LIB		= 'PMPGPMain.dll';
  LOG_FILE		= 'PMPGP.log';
  PMPGP			= 'PMPGP';

  PMIME_INFO		= 'PGP/MIME';
  SMIME_INFO		= 'S/MIME';

  FILE_PROT		= 'file:///';
  MAILTO		= 'mailto';
  HTTP			= 'http';
  FTP			= 'ftp';

  FOLDER		= '.folder';
  CFG			= '.cfg';
  EXE			= '.exe';
  DLL			= '.dll';
  FXX			= '.f??';
  CHM			= '.chm';
  BMP			= '.bmp';
  GIF			= '.gif';
  GID			= '.gid';
  HLP			= '.hlp';
  HTM			= '.htm';
  RTF			= '.rtf';
  TXT			= '.txt';
  ASC			= '.asc';
  PGP			= '.pgp';
  SIG			= '.sig';

  EN			= 'En';

const // Hidden window class
  HIDDEN_CLASS		= 'PMPGP_HIDDEN_CLASS';

const // Helper commands
  USER_SETUP		= 'USERSETUP';
  USER_UNINSTALL	= 'USERUNINSTALL';
  USER_UPDATE		= 'USERUPDATE';
  PM_RESTART		= 'PMRESTART';
  HTML_PAGE		= 'HTMLPAGE';
  PGP_HELP		= 'PGPHELP';
  MANUAL		= 'MANUAL';
  TOPIC			= 'TOPIC';
  SYNCTIME		= 'SYNCTIME';
  CLEANUP		= 'CLEANUP';

const // Helper params
  SM_SHORTCUTS		= 'StartMenuShortcuts';
  SM_ALLUSERS		= 'AllUsers';

const // Signature flags
  SF_NOTHING		=  0;
  SF_NO_SIGNATURE	=  1;
  SF_VALID_TRUSTED	=  2;
  SF_VALID_UNTRUSTED	=  3;
  SF_EXPIRED		=  4;
  SF_INVALID		=  5;
  SF_NO_KEY		=  6;
  SF_ENCRYPTED		=  1 shl 16;

const // Various values
  HEX_MARKER		= '0x';
  PGP_NAME		= 'PGP';
  PM_PGP		= 'PM-PGP';
  PGPSDK_UI		= 'PGPsdkUI';
  ET_AL			= SP + 'et al' + DOT;

type // PChar typecasts
  DChar = Array[0..1] of Char;
  PDChar = ^DChar;
  QChar = Array[0..3] of Char;
  PQChar = ^QChar;

type // PM font data
  TPMPGPFont = Record
    Name: String;
    Size: Integer;
    Height: Integer;
    Styles: Cardinal;
  end;

type // Languages
  TLangIDs = Record
    LangID: LANGID;
    TLD: String;
  end;

const
  ALangIDs: Array[0..3] of TLangIDs = (
    (LangID: LANG_GERMAN; TLD: 'De'),
    (LangID: LANG_ENGLISH; TLD: 'En'),
    (LangID: LANG_FRENCH; TLD: 'Fr'),
    (LangID: LANG_ITALIAN; TLD: 'It')
  );

// Global utilities
procedure ProcessMessages(Window: THandle);
procedure ShowFatalErrorMessage(Code: DWord);
function  GetNTPServerList(List: TStrings): Integer;
function  GetDefaultNTPServer: String;
function  GetWinUserDefaultTLD: String;
function  GetLangIDFromTLD(const TLD: String): LANGID;
function  VersionNumberFromString(const Version: String): Integer;
function  VersionStringFromNumber(Version: Integer): String;
function  GetMainVersion(Version: Integer): Integer;
function  SynchronizeTime(const Server: String; Port: Integer): Cardinal;
function  ExtractKeyIDs(const KeyIDData: String; var UserIDs, HexIDs: String): Integer;
function  ExtractHexID(const Text: String): String;
function  ExtractEmail(const Text: String): String;
function  ExtractName(const Text: String): String;
function  GetDefaultBmp: String;

// Debug utilities
function  WriteToFile(const Text, FileName: String): Longbool;
function  LogToFile(const Line: String): Longbool;
procedure DebugMsg(const Text: String);
procedure DebugBool(Bool: Longbool);
procedure DebugHex(Hex: Integer);
procedure DebugInt(Int: Integer);


implementation

{$IFDEF CONDITIONALEXPRESSIONS}
uses
  Variants;

const
  NULL = StrUtil.NULL;
{$ENDIF}


// Various global utilities ------------------------------------------------------------------------

procedure ProcessMessages(Window: THandle);
var
  Msg: TMsg;
begin
  if Window = 0 then begin
    while PeekMessage(Msg, 0, 0, 0, PM_REMOVE) and (Msg.Message <> WM_QUIT) do begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  end
  else with Msg do begin
    while PeekMessage(Msg, 0, 0, 0, PM_REMOVE)
    and ((hWnd = Window) or IsChild(Window, hWnd))
    and (Message <> WM_QUIT) do begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  end;
end;

procedure ShowFatalErrorMessage(Code: DWord);
begin
  if Code <> 0 then MessageBox(GetDesktopWindow, PChar(SysErrorString(Code)), FRONTEND_NAME, MB_OK or MB_ICONERROR);
end;

function GetNTPServerList(List: TStrings): Integer;
var
  hRegKey: HKEY;
  dwIndex, dwName, dwValue: DWord;
  aName: Array[0..1] of AnsiChar;
  aValue: Array[0..255] of AnsiChar;
begin
  Result := 0;
  if (List <> nil) and (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_KEY_DATETIME_SERVERS, 0, KEY_QUERY_VALUE, hRegKey) = ERROR_SUCCESS) then begin
    try
      dwIndex := 0;
      aName := NULL;
      aValue := NULL;
      repeat
	dwName := SizeOf(aName);
	dwValue := SizeOf(aValue);
	if RegEnumValue(hRegKey, dwIndex, aName, dwName, nil, nil, @aValue, @dwValue) <> ERROR_NO_MORE_ITEMS then begin
	  if FastStrLen(aName) > 0 then begin
	    try
	      dwName := StrToCard(aName);
	      if dwName > DWord(List.Count) then dwName := DWord(List.Count);
	      List.Insert(dwName, aValue);
	    except
	    end;
	  end;
	  inc(dwIndex);
	end
	else Break;
      until Result > 0;
    finally
      RegCloseKey(hRegKey);
    end;
    Result := List.Count;
  end;
end;

function GetDefaultNTPServer: String;
var
  sNTPServerIndex: String;
begin
  Result := RegString(HKEY_LOCAL_MACHINE, REG_KEY_CCS_DT_PARAMS, REG_ENTRY_NTPSERVER);
  if Result = E then begin
    sNTPServerIndex := RegString(HKEY_LOCAL_MACHINE, REG_KEY_DATETIME_SERVERS, E);
    if sNTPServerIndex = E then
      Result := 'time.windows.com'
    else Result := RegString(HKEY_LOCAL_MACHINE, REG_KEY_DATETIME_SERVERS, sNTPServerIndex);
  end
  else Delete(Result, FirstCharPos(CM, Result), MAXINT);
end;

function ConvertLangIDs(const ID: Variant): Variant;
var
  iIndex: Integer;
  sTLD: String;
  dwID: DWord;
begin
  if VarType(ID) and varTypeMask = varString then begin
    sTLD := LowerCase(ID);
    if Length(sTLD) = 2 then for iIndex := 0 to High(ALangIDs) do begin
      if LoStrAtPos(sTLD, ALangIDs[iIndex].TLD, 1) then begin
	Result := ALangIDs[iIndex].LangID;
	Exit;
      end;
    end;
    Result := LANG_ENGLISH;
  end
  else begin
    dwID := ID and $3FF;
    for iIndex := 0 to High(ALangIDs) do begin
      if dwID = ALangIDs[iIndex].LangID then begin
	Result := ALangIDs[iIndex].TLD;
	Exit;
      end;
    end;
    Result := 'En';
  end;
end;

function GetWinUserDefaultTLD: String;
begin
  Result := ConvertLangIDs(GetUserDefaultLangID);
end;

function GetLangIDFromTLD(const TLD: String): LANGID;
begin
  Result := ConvertLangIDs(TLD);
end;

function VersionNumberFromString(const Version: String): Integer;
const
  DIGITS = ['0'..'9'];
var
  sVersion: String;
  pVersion: PChar;
  iIndex: Integer;
begin
  Result := 0;
  if Version <> E then begin
    iIndex := 1;
    while (Version[iIndex] <> NULL) and not (Version[iIndex] in DIGITS) do inc(iIndex);
    if Version[iIndex] = NULL then Exit;
    SetString(sVersion, nil, succ(Length(Version) - iIndex));
    pVersion := Pointer(sVersion);
    while Version[iIndex] in DIGITS do begin
      pVersion^ := Version[iIndex];
      inc(pVersion);
      inc(iIndex);
    end;
    if (Version[iIndex] <> DOT) or not (Version[succ(iIndex)] in DIGITS) then begin
      Delete(sVersion, succ(pVersion - Pointer(sVersion)), MAXINT);
      Result := StrToInt(sVersion) * 1000;
      Exit;
    end
    else inc(iIndex);
    pVersion^ := Version[iIndex];
    inc(pVersion);
    inc(iIndex);
    while Version[iIndex] in DIGITS do inc(iIndex);
    if (Version[iIndex] <> DOT) or not (Version[succ(iIndex)] in DIGITS) then begin
      Delete(sVersion, succ(pVersion - Pointer(sVersion)), MAXINT);
      Result := StrToInt(sVersion) * 100;
      Exit;
    end
    else inc(iIndex);
    pVersion^ := Version[iIndex];
    inc(pVersion);
    inc(iIndex);
    while Version[iIndex] in DIGITS do inc(iIndex);
    if (Version[iIndex] <> DOT) or not (Version[succ(iIndex)] in DIGITS) then begin
      Delete(sVersion, succ(pVersion - Pointer(sVersion)), MAXINT);
      Result := StrToInt(sVersion) * 10;
      Exit;
    end
    else inc(iIndex);
    pVersion^ := Version[iIndex];
    Delete(sVersion, pVersion - Pointer(sVersion) + 2, MAXINT);
    Result := StrToInt(sVersion);
  end;
end;

function VersionStringFromNumber(Version: Integer): String;
var
  sVersion: String;
  pResult: PChar;
  iPos, iLen: Integer;
begin
  if Version > 0 then begin
    sVersion := IntToStr(Version);
    iLen := Length(sVersion);
    SetString(Result, nil, iLen shl 1);
    iPos := 1;
    pResult := Pointer(Result);
    while iLen - iPos > 3 do begin
      pResult^ := sVersion[iPos];
      inc(pResult);
      inc(iPos);
    end;
    for iPos := iPos to pred(iLen) do begin
      pResult^ := sVersion[iPos];
      inc(pResult);
      pResult^ := DOT;
      inc(pResult);
    end;
    pResult^ := sVersion[iLen];
    Delete(Result, pResult - Pointer(Result) + 2, MAXINT);
  end
  else Result := E;
end;

function GetMainVersion(Version: Integer): Integer;
begin
  Result := Version div 1000;
end;

function OpenProcessToken(ProcessHandle: THandle; DesiredAccess: DWord; var TokenHandle: THandle): BOOL; stdcall;
	 external advapi32 name 'OpenProcessToken'; // avoid version incompatibilities

function SynchronizeTime(const Server: String; Port: Integer): Cardinal;

const
  SE_SYSTEMTIME_NAME = 'SeSystemtimePrivilege';
var
  dtLocal: TDateTime;
  hPrivilegeToken: THandle;
  tpSystemTime: TTokenPrivileges;
  dwSize: DWord;
begin
  Result := 0;
  SetCursor(LoadCursor(0, IDC_WAIT));
  try
    dtLocal := NTPTime(Server, Port);
    if dtLocal = 0 then Exit;
    if not OpenProcessToken(GetCurrentProcess, TOKEN_READ or TOKEN_WRITE, hPrivilegeToken) then Exit;
    try
      with tpSystemTime do begin
	if not LookupPrivilegeValue(nil, SE_SYSTEMTIME_NAME, Privileges[0].Luid) then Exit;
	Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
	PrivilegeCount := 1;
      end;
      dwSize := SizeOf(TTokenPrivileges);
      if not AdjustTokenPrivileges(hPrivilegeToken, false, tpSystemTime, dwSize, tpSystemTime, dwSize) then Exit;
      try
	if not SetLocalTime(DateTimeToSystemTime(dtLocal)) then Exit;
	SetLastError(0);
      finally
	AdjustTokenPrivileges(hPrivilegeToken, false, tpSystemTime, dwSize, tpSystemTime, dwSize);
      end;
    finally
      CloseHandle(hPrivilegeToken);
    end;
  finally
    SetCursor(LoadCursor(0, IDC_ARROW));
    Result := GetLastError;
  end;
end;

function ExtractKeyIDs(const KeyIDData: String; var UserIDs, HexIDs: String): Integer;
var
  iLen, iPos: Integer;
begin
  Result := 0;
  if KeyIDData <> E then begin
    iLen := Length(KeyIDData);
    iPos := iLen - DefHexIDLen;
    if (iPos < iLen) and (KeyIDData[iPos] <> BS) then iPos := LastCharPos(BS, KeyIDData);
    if iPos > 0 then begin
      UserIDs := Copy(KeyIDData, 1, pred(iPos));
      HexIDs := Copy(KeyIDData, succ(iPos), MAXINT);
    end
    else begin
      UserIDs := E;
      HexIDs := E;
    end;
    if HexIDs <> E then
      Result := succ(GetCharCount(CM, HexIDs))
    else if UserIDs <> E then
      Result := succ(GetCharCount(LF, UserIDs))
    else Result := 0;
  end;
end;

function ExtractHexID(const Text: String): String;
const
  HEX_CHARS = ['0'..'9', 'A'..'F'];
var
  iLen, iBeg, iEnd: Integer;
begin
  Result := E;
  if Text <> E then begin
    iLen := Length(Text);
    iBeg := 1;
    while not (Text[iBeg] in (HEX_CHARS + [NULL])) do inc(iBeg);
    if (iBeg < iLen) and (ord(Text[succ(iBeg)]) and $DF = ord('X')) then inc(iBeg, 2);
    iEnd := iBeg;
    while (iEnd <= iLen) and (Text[iEnd] in HEX_CHARS) do inc(iEnd);
    iLen := iEnd - iBeg;
    if iLen >= (LongHexIDLen - 2) then
      Result := HEX_MARKER + Copy(Text, iBeg, LongHexIDLen - 2)
    else if iLen >= (ShortHexIDLen - 2) then Result := HEX_MARKER + Copy(Text, iBeg, ShortHexIDLen - 2);
  end;
end;

function ExtractEmail(const Text: String): String;
begin
  Result := KeyPropTypes.ExtractEmail(Text);
end;

function ExtractName(const Text: String): String;
begin
  Result := KeyPropTypes.ExtractName(Text);
end;

function GetDefaultBmp: String;
begin
  Result := GetModuleFilePath(0) + 'DEFAULT.BMP';
end;


// Debug utilities ---------------------------------------------------------------------------------

function WriteToFile(const Text, FileName: String): Longbool;
var
  fsBuffer: TFileStream;
  iLen: Integer;
begin
  Result := false;
  if Text <> E then begin
    try
      fsBuffer := TFileStream.Create(FileName, fmCreate);
      if fsBuffer <> nil then with fsBuffer do begin
	try
	  iLen := Length(Text);
	  Result := (Write(Pointer(Text)^, iLen) = iLen);
	finally
	  fsBuffer.Free;
	end;
      end;
    except
    end;
  end;
end;

function LogToFile(const Line: String): Longbool;
var
  sFileName: String;
  fsBuffer: TFileStream;
  iLen: Integer;
begin
  Result := false;
  if Line <> E then begin
    try
      sFileName := GetSystemFolderPath(CSIDL_DESKTOPDIRECTORY) + LOG_FILE;
      if not FileExists(sFileName) then
	fsBuffer := TFileStream.Create(sFileName, fmCreate)
      else fsBuffer := TFileStream.Create(sFileName, fmOpenReadWrite);
      if fsBuffer <> nil then with fsBuffer do begin
	try
	  Seek(0, soFromEnd);
	  iLen := Length(Line) + Length(CRLF);
	  Result := (Write(Pointer(Line + CRLF)^, iLen) = iLen);
	finally
	  fsBuffer.Free;
	end;
      end;
    except
    end;
  end;
end;

const
  DEBUG = 'DEBUG';

procedure DebugMsg(const Text: String);
begin
  if Text <> E then begin
    if Text[Length(Text)] = NULL then
      MessageBoxW(0, Pointer(Text), DEBUG, 0)
    else MessageBoxA(0, Pointer(Text), DEBUG, 0);
  end;
end;

procedure DebugBool(Bool: Longbool);
begin
  MessageBox(0, PChar(IntToStr(ord(Bool))), DEBUG, 0);
end;

procedure DebugHex(Hex: Integer);
begin
  MessageBox(0, PChar(IntToHex(Hex, 8)), DEBUG, 0);
end;

procedure DebugInt(Int: Integer);
begin
  MessageBox(0, PChar(IntToStr(Int)), DEBUG, 0);
end;


end.

