{$J+,Z4}
unit PMPGPForm;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, Graphics, Controls, ExtCtrls, StdCtrls, Forms,
  HelpCtrl, ToolTip, FlatBtn, PMPGPCommon, PMPGPResources, PMPGPMainClass, PMHandler, PGPHandler;

const // PMPGP colors
  clPMPGPBtnShadow	= $0084B0AF;
  clPMPGPBtnFace	= $00A8C8C7;
  clPMPGPWindow		= $00C2E0E0;

const
  ABtnShadowColors: Array[Boolean] of TColor = (clPMPGPBtnShadow, clBtnShadow);
  ABtnFaceColors: Array[Boolean] of TColor = (clPMPGPBtnFace, clBtnFace);
  AWindowColors: Array[Boolean] of TColor = (clPMPGPWindow, clWindow);

type
  {$IFDEF VER90} {$WARNINGS OFF} {$ENDIF}
  TPMPGPForm = Class(TForm)
  private
    FMainClass: TMainClass;
    FCurrentDir: String;
    FFixedFontName: String;
    FParentHandle: THandle;
    FStandardColors: Longbool;
    FToolTip: TToolTip;
    FPMInterface: TPMInterface;
    FPGPInterface: TPGPInterface;
    FPMPGPStrings: TPMPGPStrings;
    function	GetHelpShowing: Longbool;
    function	OnLinkClick(const Link: String): String;
    procedure	WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure	WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure	WMContextMenu(var Message: TWMMouse); message WM_CONTEXTMENU;
    procedure	WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    procedure	WMHelp(var Message: TWMHelp); message WM_HELP;
    procedure	WMNCActivate(var Message: TWMNCActivate); message WM_NCACTIVATE;
    procedure	WMSysCommand(var Message: TWMSysCommand); message WM_SYSCOMMAND;
  protected
    procedure	DoShow; override;
    procedure	SetColors(Default: Longbool); virtual;
    property	ParentHandle: THandle read FParentHandle;
  public
    constructor	Create(MainClass: TObject); {$IFNDEF VER90} reintroduce; overload; {$ENDIF}
    destructor	Destroy; override;
    procedure	CreateParams(var Params: TCreateParams); override;
    procedure	DisableSystemMenu;
    procedure	NormalWindowSize;
    procedure	Release;
    procedure	RemoveKeyMessage;
    procedure	SetHotKeyHint(const Button: TFlatBtn; const Text: String);
    function	ShowHelp(Control: TWinControl; ID: Integer; PopupOnly: Longbool): Longbool;
    property	FixedFontName: String read FFixedFontName;
    property	HelpShowing: Longbool read GetHelpShowing;
    property	MainClass: TMainClass read FMainClass;
    property	PMInterface: TPMInterface read FPMInterface;
    property	PGPInterface: TPGPInterface read FPGPInterface;
    property	PMPGPStrings: TPMPGPStrings read FPMPGPStrings;
  end;
  {$IFDEF VER90} {$WARNINGS ON} {$ENDIF}

procedure RemoveBoldStyleFromFont(const Font: TFont);


implementation

procedure RemoveBoldStyleFromFont(const Font: TFont);
var
  ftStyle: TFontStyles;
begin
  ftStyle := Font.Style;
  if fsBold in ftStyle then Exclude(ftStyle, fsBold);
  Font.Style := ftStyle;
end;

constructor TPMPGPForm.Create(MainClass: TObject);
begin
  if MainClass is TMainClass then begin
    FMainClass := TMainClass(MainClass);
    FCurrentDir := GetCurrentDir;
    with FMainClass do begin
      FPMInterface := PMInterface;
      FPGPInterface := PGPInterface;
      FPMPGPStrings := PMPGPStrings;
      FFixedFontName := WinFixedFontName;
      FStandardColors := StandardColors;
      FParentHandle := CurrentDialogOwner;
    end;
    inherited Create(nil);
    Icon.Handle := LoadIcon(ICO_SMALL);
    HorzScrollBar.Visible := false;
    VertScrollBar.Visible := false;
    KeyPreview := true;
  end;
end;

destructor TPMPGPForm.Destroy;
begin
  SetCurrentDir(FCurrentDir);
  inherited;
end;

procedure TPMPGPForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.WndParent := FParentHandle;
end;

procedure TPMPGPForm.DisableSystemMenu;
var
  hSystemMenu: THandle;
  iSystemMenu: Integer;
begin
  hSystemMenu := GetSystemMenu(Handle, false);
  DeleteMenu(hSystemMenu, SC_CLOSE, MF_BYCOMMAND);
  iSystemMenu := pred(GetMenuItemCount(hSystemMenu));
  for iSystemMenu := iSystemMenu downto 0 do begin
    if GetMenuState(hSystemMenu, iSystemMenu, MF_BYPOSITION) and MF_GRAYED <> 0 then begin
      DeleteMenu(hSystemMenu, iSystemMenu, MF_BYPOSITION);
    end;
  end;
end;

procedure TPMPGPForm.NormalWindowSize;
var
  rMaximized, rSelf: TRect;
begin
  GetWindowRect(FPMInterface.Windows[wMAINFrame], rMaximized);
  with rMaximized do begin
    rSelf.Right := ((Right - Left) div 4) * 3;
    rSelf.Bottom := ((Bottom - Top) div 8) * 5;
    rSelf.Left := (Right - Left - rSelf.Right) div 2;
    rSelf.Top := (Bottom - Top - rSelf.Bottom) div 4;
  end;
  with rSelf do SetWindowPos(Handle, HWND_TOP, Left, Top, Right, Bottom, 0);
end;

procedure TPMPGPForm.Release;
begin
  FMainClass.CurrentDialogOwner := FParentHandle;
  SetActiveWindow(FParentHandle);
  inherited Release;
  ProcessMessages(0);
end;

procedure TPMPGPForm.RemoveKeyMessage;
var
  msgKey: TMsg;
begin
  if ActiveControl <> nil then PeekMessage(msgKey, ActiveControl.Handle, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE);
end;

procedure TPMPGPForm.SetHotKeyHint(const Button: TFlatBtn; const Text: String);
var
  iPos: Integer;
begin
  with Button do begin
    iPos := 0;
    Hint := Text;
    if Hint <> E then iPos := FirstCharPos(AM, Hint);
    if iPos <> 0 then Caption := Copy(Hint, iPos, 2);
  end;
end;

function TPMPGPForm.ShowHelp(Control: TWinControl; ID: Integer; PopupOnly: Longbool): Longbool;
var
  rControl: TRect;
begin
  Result := false;
  if ID > ord(IDH_POPUP_HELP_START) then begin
    if ID > ord(IDH_CONTEXT_HELP_START) then
      Result := FMainClass.PMPGPHelp.Topic(Control.Handle, ID)
    else if FToolTip <> nil then with FToolTip do begin
      with Control do begin
	rControl.TopLeft := Self.ScreenToClient(ClientOrigin);
	rControl.BottomRight := ClientRect.BottomRight;
      end;
      FToolTip.ToolRect := rControl;
      if GetKeyState(VK_F1) < 0 then
	CenterToolTip := cttSubToolRect
      else CenterToolTip := cttSubMouse;
      AnsiText := FMainClass.PMPGPHelp.GetRichTextByID(pred(ID));
      MaxWidth := Self.ClientWidth;
      TextHeight := FMainClass.PMPGPHelp.GetTextHeightByID(pred(ID));
      Show;
    end;
  end;
end;

procedure TPMPGPForm.DoShow;
begin
  SetColors(FStandardColors);
  inherited;
end;

procedure TPMPGPForm.SetColors(Default: Longbool);
begin
  Color := ABtnFaceColors[Default];
end;

function TPMPGPForm.GetHelpShowing: Longbool;
begin
  Result := (FToolTip <> nil) and FToolTip.Showing;
end;

function TPMPGPForm.OnLinkClick(const Link: String): String;
begin
  if LoStrStart(HTTP, Link) or LoStrStart(FTP, Link) or LoStrStart(MAILTO, Link) then
    FMainClass.LaunchBrowser(Link)
  else if LoStrStart(PGP_HELP + QM, Link) then
    FMainClass.LaunchPGPHelp(Copy(Link, succ(FirstCharPos(QM, Link)), MAXINT))
  else if LoStrStart(MANUAL + QM, Link) then
    FMainClass.ShowHelpManual(DC + Copy(Link, succ(FirstCharPos(QM, Link)), MAXINT))
  else if LoStrStart(TOPIC + QM, Link) and (FToolTip <> nil) then with FToolTip do begin
    Result := FMainClass.PMPGPHelp.GetRichTextByName(Copy(Link, succ(FirstCharPos(QM, Link)), MAXINT));
    Exit;
  end;
  Result := E;
end;

procedure TPMPGPForm.WMCreate(var Message: TWMCreate);
begin
  FMainClass.CurrentDialogOwner := WindowHandle;
  FToolTip := TToolTip.Create(Self);
  if FToolTip <> nil then with FToolTip do begin
    OnLinkClick := Self.OnLinkClick;
    ToolTipStyle := ttsRichText;
    HMargin := 8;
    VMargin := 4;
  end;
end;

procedure TPMPGPForm.WMDestroy(var Message: TWMDestroy);
begin
  FToolTip.Free;
end;

procedure TPMPGPForm.WMContextMenu(var Message: TWMMouse);
var
  wcFound: TWinControl;
begin
  if not GetHelpShowing then begin
    with Message do wcFound := FindControl(WindowFromPoint(Point(XPos, YPos)));
    while (wcFound <> nil) and (wcFound.HelpContext = 0) do wcFound := wcFound.Parent;
    if (wcFound = nil) or (wcFound is TEdit) or (wcFound.HelpContext = 0) then Exit;
    ShowHelp(wcFound, wcFound.HelpContext, true);
  end;
end;

procedure TPMPGPForm.WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo);
var
  rMaximized: TRect;
begin
  GetWindowRect(FPMInterface.Windows[wMAINFrame], rMaximized);
  with Message.MinMaxInfo^, rMaximized do begin
    ptMaxSize := Point(Right - Left, Bottom - Top);
    ptMaxPosition := Point(Left, Top);
  end;
  inherited;
end;

procedure TPMPGPForm.WMHelp(var Message: TWMHelp);
var
  wctFound: TWinControl;
  ctFound: TControl;
begin
  wctFound := FindControl(Message.HelpInfo^.hItemHandle);
  if wctFound <> nil then with wctFound do begin
    ctFound := ControlAtPos(ScreenToClient(Message.HelpInfo^.MousePos), true);
    if (ctFound <> nil) and (ctFound is TCustomLabel) then with TLabel(ctFound) do begin
      if FocusControl <> nil then wctFound := FocusControl;
    end;
  end;
  while (wctFound <> nil) and (wctFound.HelpContext = 0) do wctFound := wctFound.Parent;
  if (wctFound <> nil) and (wctFound.HelpContext <> 0) then ShowHelp(wctFound, wctFound.HelpContext, false);
end;

procedure TPMPGPForm.WMNCActivate(var Message: TWMNCActivate);
begin
  inherited;
  if Message.Active and (WindowState = wsMinimized) then WindowState := wsNormal;
end;

procedure TPMPGPForm.WMSysCommand(var Message: TWMSysCommand);
begin
  if (Message.CmdType = SC_CLOSE) and (GetKeyState(VK_MENU) < 0) then
    Message.CmdType := 0
  else inherited;
  if Message.CmdType = SC_CLOSE then ModalResult := mrIgnore;
end;


end.

