{$A+,B-,C+,D+,E-,F+,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V-,W-,X+,Y+,Z4}
{$IFDEF CONDITIONALEXPRESSIONS} {$WARN SYMBOL_PLATFORM OFF} {$ENDIF}
program PMPGPHelper;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

uses
  Windows, SysUtils, StrUtil, Processes, PMPGPCommon;

{$R *.RES}

function LoadPMPGPMainProc(Index: Integer): Pointer;
var
  hPMPGPMain: THandle;
begin
  Result := nil;
  SetLastError(0);
  hPMPGPMain := LoadLibrary(PChar(GetAppDir + MAIN_LIB));
  if hPMPGPMain <> 0 then Result := GetProcAddress(hPMPGPMain, Pointer(Index));
  if Result = nil then ShowFatalErrorMessage(GetLastError);
end;

procedure FreePMPGPMain;
var
  hPMPGPMain: THandle;
begin
  hPMPGPMain := GetModuleHandle(MAIN_LIB);
  if hPMPGPMain <> 0 then FreeLibrary(hPMPGPMain);
end;

function GetCommands: String;
var
  iIndex: Integer;
begin
  for iIndex := 2 to ParamCount do Result := ParamStr(iIndex) + CM;
  Delete(Result, Length(Result), 1);
end;

procedure ConfigureNewUser;
var
  UserSetup: procedure(Commands: PChar); stdcall;
begin
  UserSetup := LoadPMPGPMainProc(USER_SETUP_INDEX);
  if Assigned(UserSetup) then UserSetup(E);
end;

procedure UninstallCurrentUser;
var
  UserUninstall: procedure; stdcall;
begin
  UserUninstall := LoadPMPGPMainProc(USER_UNINSTALL_INDEX);
  if Assigned(UserUninstall) then UserUninstall;
end;

procedure UpdateCurrentUsers;
var
  UserUpdate: procedure(Commands: PChar); stdcall;
begin
  UserUpdate := LoadPMPGPMainProc(USER_UPDATE_INDEX);
  if Assigned(UserUpdate) then UserUpdate(PChar(GetCommands));
end;

procedure RestartPegasusMail;
var
  PMRestart: procedure(ID: PChar; Frame: THandle); stdcall;
  sCommand: String;
  hFrame: THandle;
begin
  PMRestart := LoadPMPGPMainProc(PM_RESTART_INDEX);
  if Assigned(PMRestart) then begin
    sCommand := GetCommandLine;
    Delete(sCommand, 1, NextLoStrPos(PM_RESTART, sCommand, succ(Length(ParamStr(0)))) + Length(PM_RESTART));
    hFrame := StrToInt(Trim(ExtractStr(sCommand, LastCharPos(SP, sCommand), MAXINT)));
    PMRestart(PChar(sCommand), hFrame);
  end;
end;

procedure SyncSysTime;
var
  SynchronizeTime: procedure(Show: Longbool); stdcall;
begin
  SynchronizeTime := LoadPMPGPMainProc(SYNCTIME_INDEX);
  if Assigned(SynchronizeTime) then SynchronizeTime(ParamStr(2) <> E);
end;

procedure LaunchBrowser;
var
  HTMLPageLoad: procedure(Page: PChar); stdcall;
  sCommand: String;
  iPos: Integer;
begin
  HTMLPageLoad := LoadPMPGPMainProc(HTML_PAGE_INDEX);
  if Assigned(HTMLPageLoad) then begin
    sCommand := GetCommandLine;
    iPos := succ(Length(ParamStr(0)));
    Delete(sCommand, 1, NextLoStrPos(HTML_PAGE, sCommand, iPos) + Length(HTML_PAGE));
    HTMLPageLoad(PChar(sCommand));
  end;
end;

procedure LaunchPGPHelp;
var
  PGPHelpLaunch: procedure(ID: PChar); stdcall;
  sCommand: String;
  iPos: Integer;
begin
  PGPHelpLaunch := LoadPMPGPMainProc(PGP_HELP_INDEX);
  if Assigned(PGPHelpLaunch) then begin
    sCommand := GetCommandLine;
    iPos := succ(Length(ParamStr(0)));
    Delete(sCommand, 1, NextLoStrPos(PGP_HELP, sCommand, iPos) + Length(PGP_HELP));
    PGPHelpLaunch(PChar(sCommand));
  end;
end;

procedure ShowManual;
var
  PMPGPManual: procedure(CMD: PChar); stdcall;
  sCommand: String;
  iPos: Integer;
begin
  PMPGPManual := LoadPMPGPMainProc(MANUAL_INDEX);
  if Assigned(PMPGPManual) then begin
    sCommand := GetCommandLine;
    iPos := succ(Length(ParamStr(0)));
    Delete(sCommand, 1, NextLoStrPos(MANUAL, sCommand, iPos) + pred(Length(MANUAL)));
    PMPGPManual(PChar(sCommand));
  end;
end;

procedure CleanupPMPGP;
var
  PMPGPCleanup: procedure; stdcall;
begin
  PMPGPCleanup := LoadPMPGPMainProc(CLEANUP_INDEX);
  if Assigned(PMPGPCleanup) then PMPGPCleanup;
end;


procedure Run;
var
  sCommand: String;
begin
  sCommand := ParamStr(1);
  if sCommand <> E then begin
    try
      if LoStrAtPos(USER_SETUP, sCommand, 1) then
	ConfigureNewUser
      else if LoStrAtPos(USER_UPDATE, sCommand, 1) then
	UpdateCurrentUsers
      else if LoStrAtPos(USER_UNINSTALL, sCommand, 1) then
	UninstallCurrentUser
      else if LoStrAtPos(PM_RESTART, sCommand, 1) then
	RestartPegasusMail
      else if LoStrAtPos(SYNCTIME, sCommand, 1) then
	SyncSysTime
      else if LoStrAtPos(HTML_PAGE, sCommand, 1) then
	LaunchBrowser
      else if LoStrAtPos(PGP_HELP, sCommand, 1) then
	LaunchPGPHelp
      else if LoStrAtPos(MANUAL, sCommand, 1) then
	ShowManual
      else if LoStrAtPos(CLEANUP, sCommand, 1) then
	CleanupPMPGP;
    finally
      FreePMPGPMain;
    end;
  end;
end;

begin
  if not NewInstance then Run;
end.

