{$A+,B-,C+,D+,E-,F+,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V-,W-,X+,Y+,Z4}
{$IFDEF CONDITIONALEXPRESSIONS} {$WARN SYMBOL_PLATFORM OFF} {$ENDIF}
{$IMAGEBASE $40000000}
library PMPGPMain;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

{$R *.RES}

uses
  Windows, SysUtils, PMMessages, PMPGPCommon, PMPGPMainClass;

  function FORMINIT(wVersion: Word; iVariant: Integer; hParent: THandle; pData: PChar;
		    var hDialog: THandle; pCallback: PChar): Word; stdcall; forward;
  function ENCRYPT(const CryptEx: TCryptEx): Integer; stdcall; forward;
  function DECRYPT(const CryptEx: TCryptEx): Integer; stdcall; forward;
  function SIGN(const CryptEx: TCryptEx): Integer; stdcall; forward;
  function VERIFY(const SourceFile, Text: PChar): Integer; stdcall; forward;
  function MANAGE(const SourceFile: PChar): Integer; stdcall; forward;
  function DESTROY(const SourceFile: PChar): Integer; stdcall; forward;

  procedure USERSETUP(Commands: PChar); stdcall; forward;
  procedure USERUNINSTALL; stdcall; forward;
  procedure USERUPDATE(Commands: PChar); stdcall; forward;
  procedure PMRESTART(ID: PChar; Frame: THandle); stdcall; forward;
  procedure SYNCTIME(Show: Longbool); stdcall; forward;
  procedure HTMLPAGE(Page: PChar); stdcall; forward;
  procedure PGPHELP(ID: PChar); stdcall; forward;
  procedure MANUAL(CMD: PChar); stdcall; forward;
  procedure CLEANUP; stdcall; forward;

exports
  // for PMPGPCryptor
  FORMINIT	index FORMINIT_INDEX,
  ENCRYPT	index ENCRYPT_INDEX,
  DECRYPT	index DECRYPT_INDEX,
  SIGN		index SIGN_INDEX,
  VERIFY	index VERIFY_INDEX,
  MANAGE	index KEY_MANAGE_INDEX,
  DESTROY	index DESTROY_FILE_INDEX,
  // for PMPGPHelper
  USERSETUP	index USER_SETUP_INDEX,
  USERUNINSTALL	index USER_UNINSTALL_INDEX,
  USERUPDATE	index USER_UPDATE_INDEX,
  PMRESTART	index PM_RESTART_INDEX,
  SYNCTIME	index SYNCTIME_INDEX,
  HTMLPAGE	index HTML_PAGE_INDEX,
  PGPHELP	index PGP_HELP_INDEX,
  MANUAL	index MANUAL_INDEX,
  CLEANUP	index CLEANUP_INDEX;


// Exported Functions for PMPGPCryptor -------------------------------------------------------------

function FORMINIT(wVersion: Word; iVariant: Integer; hParent: THandle; pData: PChar;
		  var hDialog: THandle; pCallback: PChar): Word;
begin
  Result := 0;
  if (MainClass <> nil) and ((wVersion and $FF00) <= $0100) then with MainClass do begin
    try
      hDialog := Initialize(hParent, wVersion);
      if hDialog <> 0 then Result := 1;
    except
    end;
  end;
end;

function ENCRYPT(const CryptEx: TCryptEx): Integer;
begin
  MainClass.PMPGPBusy := true;
  try
    with CryptEx do Result := MainClass.EncodeMessage(SourceFile, DestFile, Recipient, FormData);
  finally
    MainClass.PMPGPBusy := false;
  end;
end;

function DECRYPT(const CryptEx: TCryptEx): Integer;
var
  dtContinue: TDateTime;
begin
  dtContinue := Now + EncodeTime(0, 0, 0, GetDoubleClickTime);
  while Now < dtContinue do ProcessMessages(0);
  MainClass.PMPGPBusy := true;
  try
    with CryptEx do Result := MainClass.DecodeMessage(SourceFile, DestFile, Flags);
  finally
    MainClass.PMPGPBusy := false;
  end;
end;

function SIGN(const CryptEx: TCryptEx): Integer;
begin
  MainClass.PMPGPBusy := true;
  try
    with CryptEx do Result := MainClass.EncodeMessage(SourceFile, DestFile, Recipient, FormData);
  finally
    MainClass.PMPGPBusy := false;
  end;
end;

function VERIFY(const SourceFile, Text: PChar): Integer;
begin
  MainClass.PMPGPBusy := true;
  try
    Result := -2;
    MainClass.VerifySignature(SourceFile);
  finally
    MainClass.PMPGPBusy := false;
  end;
end;

function MANAGE(const SourceFile: PChar): Integer;
begin
  MainClass.PMPGPBusy := true;
  try
    Result := 0;
    MainClass.ManageKeys(SourceFile, true);
  finally
    MainClass.PMPGPBusy := false;
  end;
end;

function DESTROY(const SourceFile: PChar): Integer;
begin
  MainClass.PMPGPBusy := true;
  try
    Result := 1;
    try
      MainClass.PGPInterface.WipeFile(SourceFile);
    except
      Result := 0;
    end;
  finally
    MainClass.PMPGPBusy := false;
  end;
end;


// Exported Functions for PMPGPHelper --------------------------------------------------------------

procedure USERSETUP(Commands: PChar);
begin
  with MainClass do if Initialize(0, 0) <> 0 then UserSetup(Commands);
end;

procedure USERUNINSTALL;
begin
  with MainClass do if Initialize(0, 0) <> 0 then UserUninstall;
end;

procedure USERUPDATE(Commands: PChar);
begin
  with MainClass do if Initialize(0, 0) <> 0 then UserUpdate(Commands);
end;

procedure PMRESTART(ID: PChar; Frame: THandle);
begin
  with MainClass do if Initialize(0, 0) <> 0 then RestartPegasusMail(ID, Frame);
end;

procedure SYNCTIME(Show: Longbool);
begin
  with MainClass do if Initialize(0, 0) <> 0 then SynchronizeSystemTime(Show);
end;

procedure HTMLPAGE(Page: PChar);
begin
  with MainClass do if Initialize(0, 0) <> 0 then LaunchBrowser(Page);
end;

procedure PGPHELP(ID: PChar);
begin
  with MainClass do if Initialize(0, 0) <> 0 then LaunchPGPHelp(ID);
end;

procedure MANUAL(CMD: PChar);
begin
  with MainClass do if Initialize(0, 0) <> 0 then ShowHelpManual(CMD);
end;

procedure CLEANUP;
begin
  with MainClass do if Initialize(0, 0) <> 0 then UninstallCleanup;
end;

begin
end.

