{$J+,Z4}
unit PMPGPMainClass;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, CommCtrl, Classes, SysUtils, StrUtil, Graphics, Forms, Controls, PrintDlg, ToolTip,
  RichEd20, HelpCtrl, FileUtils, MailUtils, WindowUtils, PMMessages, PMPGPCommon, PMPGPResources,
  PMHandler, PGPHandler, HtsTer, TERHandler, ConfigHandler, AutoListHandler;

const // Window props
  MDI_READER_DATA_PROP	= 'MdiReaderDataProp';
  MDI_EDITOR_DATA_PROP	= 'MdiEditorDataProp';
  TER_PARENT_PROC_PROP	= 'DHTextEditProcProp';

const // Timer IDs
  TIMER_WIPE_CACHE	= 1;
  TIMER_AUTO_SEND	= 2;
  TIMER_RAW_VIEW	= 3;
  TIMER_HINT		= 4;
  TIMER_TIME		= 5;

const // Menu IDs
  IDM_MDIWINDOWS	= $1000;
  IDM_NEWBUTTONS	= $2000;
  IDM_ID_POPUP		= $3000;
  IDM_MAINMENU		= $4000;
  IDM_INFOMENU		= $5000;
  IDM_EDITMENU		= $6000;
  IDM_ADDRESS_BOOKS	= $7000;

type
  TMenuItems = (
    // Dummy
    miNullEntry,
    // Popup menus
    miAddAddress,
    miKeyServer,
    miKeyIDCopy,
    // Window menu
    miWindow,
    // Main menu
    miWipePassphrase,
    miUsePGPMime,
    miAppendPGPExtension,
    miAutoEncrypt,
    miEncrypt,
    miSign,
    miEncryptSign,
    miMimeConvertMessage,
    miDecryptMessage,
    miDecryptFolder,
    miVerifySignature,
    miShowFingerprint,
    miImportSystemCertificates,
    miAddKeysOrCertificates,
    miDeleteKeys,
    miExtractKeys,
    miDeleteCertificate,
    miExtractCertificate,
    miLaunchPGP,
    // Info submenu
    miConfig,
    miManual,
    miAbout,
    // Edit submenu
    miEnableAutoList,
    miAddRecipients,
    miAddRingAddresses,
    miSetAddressOptions,
    miUpdateAutoList,
    miEditAutoList,
    // Dummy
    miLastEntry
  );

type
  TEncodeFlag = (
    efNone,
    efCancel,
    efOk,
    efAsk,
    efAuto,
    efEncrypt,
    efSign,
    efEncryptSign
  );

type
  TEncodeOptions = Record
    RecipientIDs: TKeyIDData;
    AllInOneBCCs: String;
    ADKWarning: Longbool;
    HideKeyDialog: Longbool;
    ConfirmAttachmentEncoding: Longbool;
    EncodeAttachments: Longbool;
    SendAllUnencoded: Longbool;
    RecipientsGroup: Longbool;
    Aborted: Longbool;
  end;

type
  TDecodeOptions = Record
    Attachment: Longbool;
    DecodeOnly: Longbool;
    Forwarding: Longbool;
    Replying: Longbool;
    Printing: Longbool;
    NewMessage: Longbool;
    FancyReply: Longbool;
    HotUrls: Longbool;
    PrintMessage: Longbool;
    FirstPassDone: Longbool;
    ShowAttachments: Longbool;
  end;

type
  PMDIReaderData = ^TMDIReaderData;
  TMDIReaderData = Record
    ReaderProc: TFNWndProc;
    ReaderIcon: THandle;
    KeepBodyIcon: Longbool;
    RawViewActive: Longbool;
    BodyStatusIcon: THandle;
    AttachmentStatusIcon: THandle;
  end;

type
  PMDIEditorData = ^TMDIEditorData;
  TMDIEditorData = Record
    EditorProc: TFNWndProc;
    SendParentProc: TFNWndProc;
    AutomationFlags: Integer;
    EncodeFlag: TEncodeFlag;
    ShowAIOWarning: Longbool;
    AddressBookList: TAddressBookList;
  end;

type
  TMainClass = Class(TConfiguration)
  private
    // PMPGP general
    FInstance: Integer;
    FPMPGPBusy: Longbool;
    FCleanupDone: Longbool;
    FStandaloneMode: Longbool;
    FShutDownLogImplemented: Longbool;
    // Thread handling
    FWaitEvent: THandle;
    FWaitThreadID: DWord;
    FWaitThreadStop: Longbool;
    // Window handling
    FMDIWindows: TList;
    FEventWindow: THandle;
    FHiddenWindow: THandle;
    FPreviousChild: THandle;
    FCurrentDialogOwner: THandle;
    FCallWndHook: THandle;
    FGetMessageHook: THandle;
    // Global variables
    FAddressBookSelected: Longbool;
    FAutoSending: Longbool;
    FCurrentTime: String;
    FGetAndSendTriggered: Longbool;
    FHideNoAttachmentDialog: Longbool;
    FManageOrVerifyTriggered: Longbool;
    FManageOrVerifyFileName: String;
    FSendingMail: Longbool;
    FStatusBarIconShowing: Longbool;
    FStatusBarFontHandle: THandle;
    FTerLinkClicked: Longbool;
    // Signature verifying
    FCurrentStatusIcon: Longint;
    // Key management
    FIDMenuData: String;
    // Coding
    FMimeMsgTree: TMimeMsgTree;
    FMsgTreeData: TMsgTreeData;
    // Decoding
    FDecodeOptions: TDecodeOptions;
    FReaderWordWrapCache: TWordWrap;
    FSignatureData: TSignatureData;
    // Encoding
    FEncodeOptions: TEncodeOptions;
    FRecipientsData: TRecipientsData;
    FRecipientsKeyData: TKeyIDData;
    FSigningKeyData: TKeyIDData;
    // Print messages
    FPrinterPrefs: TPrinterPrefs;
    FPrefsCaptions: TPrefsCaptions;
    // PM modifications
    FPMFrameWindow: THandle;
    FMDIClientWindow: THandle;
    FPMStatusBarWindow: THandle;
    FPMFrameProc: TFNWndProc;
    FPMStatusBarProc: TFNWndProc;
    FPMStatusBarChildProc: TFNWndProc;
    FPMChangeMbxDlgProc: TFNWndProc;
    FPMChangeUserDlgProc: TFNWndProc;
    FEncryptionDlgProc: TFNWndProc;
    FDecPhraseEditProc: TFNWndProc;
    FMoveMailBoxDialog: THandle;
    FStatusToolTip: THandle;
    FTimeToolTip: THandle;
    FTimeRect: TRect;
    FNewPMUser: String;
    FMainMenu: THandle;
    FInfoMenu: THandle;
    FEditMenu: THandle;
    FEncryptSignMenu: THandle;
    FAddressBookMenu: THandle;
    procedure	AddToolbarButtons;
    procedure	RemoveToolbarButtons;
    procedure	AddEditMenuEntries;
    procedure	CreateMainMenuEntry;
    procedure	RemoveMainMenuEntry;
    procedure	SwitchMainMenuItems;
    procedure	SwitchMainMenuEntry(FrameMenu: THandle);
    procedure	InsertMainMenuEntry(FrameMenu, WindowMenu: THandle);
    function	InsertAddressBookMenu: Longbool;
    procedure	RemoveAddressBookMenu(Window: THandle);
    procedure	EnableUsingAutoList(Confirm: Longbool);
    procedure	ShowAutoListMenu;
    procedure	ShowEncryptSignMenu;
    procedure	ShowIDMenu(Window: THandle; const ID: String; IsHexID, KeyTrigger: Longbool);
    procedure	ShowMenuInfo(Menu: THandle; Item: Integer);
    procedure	ShowMenuHelp(Item, ID: Integer);
    procedure	IDMenuHandler(MenuID: Integer);
    procedure	MainMenuHandler(MenuID: Integer);
    procedure	InfoMenuHandler(MenuID: Integer);
    procedure	EditMenuHandler(MenuID: Integer);
    procedure	AddressBookHandler(MenuID: Integer);
    procedure	UpdateMDIWindowMenu(Menu: THandle);
    function	GetActiveMDIChild: THandle;
    function	CloseReaderWindows: Longbool;
    function	CloseEditorWindows: Longbool;
    procedure   CenterDialog(Dialog, Owner: THandle);
    procedure	SendToBack(Dialog: THandle);
    function	FindReader: THandle;
    function	FindPreviewReader: THandle;
    function	FindReaderWindow(Parent: THandle): THandle;
    function	FindHeaderWindow(Parent: THandle): THandle;
    function	FindEditorWindow(Parent: THandle): THandle;
    function	FindStatusBarChild(const Text: String): THandle;
    function	AddressOrHexIDPopup(Window: THandle; KeyTrigger: Longbool): Longbool;
    function 	GetStatusIconIndex(Status: Integer): Integer;
    procedure	ShowVerificationInfo;
    procedure	ClearVerificationInfo;
    function	GetStatusBarFont: THandle;
    function	GetHexStatusLabel(Window: THandle; const ClassName: String): THandle;
    function 	GetStatusToolTipText(const Text: String): String;
    procedure	AddStatusToolTip(const ToolTip, Parent: THandle; const Text: String);
    procedure	CreateStatusToolTip(const Text: String);
    procedure	DestroyStatusToolTip;
    procedure	CreateTimeToolTip;
    procedure	DestroyTimeToolTip;
    procedure	RefreshStatusBarTime;
    procedure	ShowStatusBarTime(StatusBar: THandle);
    procedure 	DrawStatusBarIcon(StatusLabel: THandle);
    procedure	ClearMDIReaderIcons(MDIReader: THandle);
    procedure	DrawMDIReaderIcon(MDIReader: THandle; Active: Longbool);
    procedure	DrawStatusText(DC: THandle; Text: PChar; Indent, CalcRect: Longbool; var Rect: TRect);
    procedure	CallWndProcCreate(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcDestroy(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcSetText(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcShowWindow(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcInitMenuPopup(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcParentNotify(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcMDIDestroy(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    procedure	CallWndProcMDISetMenu(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    function	GetMessageProcPaint(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcClose(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcKeyDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcKeyUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcSysKeyDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcSysKeyUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcMouseMove(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcLButtonDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcLButtonUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcLButtonDblClk(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcRButtonDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMessageProcPMTerSetFocus(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    procedure	HooksCheckCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM);
    function	HiddenWindowHelp(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowCommand(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowMenuSelect(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowExitMenuLoop(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowInit(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowSendMail(wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowLogEvent(hWindow: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	HiddenWindowCleanup(hWindow: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
    function	NewPMFrameProcCommand(wParam: WPARAM; lParam: LPARAM): UINT;
    function	GetMDIReaderDataPtr(MDIReader: THandle): PMDIReaderData;
    function	GetMDIEditorDataPtr(MDIEditor: THandle): PMDIEditorData;
    procedure	MDIReaderUnSubclass(MDIReader: THandle);
    procedure	MDIEditorUnSubclass(MDIEditor: THandle);
    procedure	TextEditUnSubclass(TextEdit: THandle);
    procedure	MDIReaderSubclass(MDIReader: THandle);
    procedure	MDIEditorSubclass(MDIEditor: THandle);
    procedure	TextEditSubclass(TextEdit: THandle);
    procedure	NDLGEncryptionHandler(Dialog: THandle);
    function	GetEncryptorName(EncryptionDialog: THandle): String;
    function	GetEncryptor(EncryptionDialog: THandle; var AllInOne: Longbool): Longint;
    procedure	SetEncryptor(EncryptionDialog: THandle; EditorFlags: Integer);
    procedure	OnSetCacheTimer(Seconds: Longint; Wiped: Longbool);
    function	CreateWaitThread: THandle;
    function	CloseWaitThread(Thread: THandle): DWord;
    function	ProcessPSMimeMessage(const MessageFile: String; var Subject: String;
				     PreDecode: Longbool; var SaveDecoded: Longbool): String;
    procedure	NewPop3MessageHandler(const MessageFile: String);
    procedure	ShowAboutDialog;
    procedure	AddAddressesToAutoList(const Addresses: String);
    procedure	RemoveKeysFromAutoList(const KIDD: TKeyIDData);
    procedure	AddNewKeysToAutoList(const KIDD: TKeyIDData);
    procedure	GetKeyFromServer(const KeyID: String);
    procedure	MimeConvertMessage;
    procedure	DecryptMessage;
    procedure	DecryptFolder;
    procedure	ShowFingerprint;
    procedure	DeleteKeys;
    procedure	ExtractKeys;
    procedure	ImportCerts;
    procedure	DeleteCert;
    procedure	ExtractCert;
    procedure	LaunchPGP;
    procedure	AddRingAddresses;
    procedure	SetAddressOptions;
    procedure	UpdateAutoList;
    procedure	EditAutoList;
    procedure	InitMimeMessageTree;
    function	GetMimeDecodeOptions: TMimeDecodeOptions;
    function	OnEncodedData(const Data: String; var Charset, SignatureText: String;
			      var SignatureFlags: Integer): String;
    function	GetEncodedSection(const Data: String; Keys: Longbool): String;
    procedure	SetPMPGPBusy(Value: Longbool);
    procedure	InitEncoding(MDIEditor: THandle);
    function	TriggerEncoding(MDIEditor: THandle): Longbool;
    function	GetRecipients(var MixOptions: Longbool): Longbool;
    function	AddDefaultBCCRecipients(MDIEditor: THandle): Longbool;
    function	AddAllInOneBCCRecipients(MDIEditor: THandle): Longbool;
    function	PreProcessMessage(const SourceFile, DestFile: String): Integer;
    function	PostProcessMessage(const SourceFile, Recipient, FormData: String): Integer;
  protected
    procedure	ExecShowTime; override;
    procedure	SwitchUsePGPMime; override;
    procedure	SwitchAppendPGPExtension; override;
    procedure	SwitchAutoEncrypt(Sign: Longbool); override;
    function	Configure: Longbool; override;
  public
    constructor	Create;
    destructor	Destroy; override;
    function	Initialize(ParentWindow: THandle; InterfaceVersion: Integer): THandle;
    function	UnknownHexIDPopup(Window: THandle; const Text: String): Longbool;
    procedure	ManageKeys(const KeyData: String; FileName: Longbool);
    procedure	VerifySignature(const FileName: String);
    procedure	SynchronizeSystemTime(Show: Longbool);
    function	DecodeMessage(const SourceFile, DestFile: String; Flags: Integer): Integer;
    function	EncodeMessage(const SourceFile, DestFile, Recipient, FormData: String): Integer;
    property	CurrentDialogOwner: THandle read FCurrentDialogOwner write FCurrentDialogOwner;
    property	PrinterPrefs: TPrinterPrefs read FPrinterPrefs write FPrinterPrefs;
    property	PrefsCaptions: TPrefsCaptions read FPrefsCaptions write FPrefsCaptions;
    property	MsgTreeData: TMsgTreeData read FMsgTreeData write FMsgTreeData;
    property    PMPGPBusy: Longbool read FPMPGPBusy write SetPMPGPBusy;
  end;

const
  MainClass: TMainClass = nil;


implementation

uses
  PMPGPForm, MessageDialogs, TextViewer, MsgTreeDialog, AutoListDialog, ConfigDialog, AboutDialog;


// Message hook procs ------------------------------------------------------------------------------

function PMCallWndProc(nCode: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    if nCode = HC_ACTION then with PCWPStruct(lParam)^ do begin
      case Message of
	WM_CREATE: CallWndProcCreate(hWnd, wParam, lParam);
	WM_DESTROY: CallWndProcDestroy(hWnd, wParam, lParam);
	WM_SETTEXT: CallWndProcSetText(hWnd, wParam, lParam);
	WM_SHOWWINDOW: CallWndProcShowWindow(hWnd, wParam, lParam);
	WM_INITMENUPOPUP: CallWndProcInitMenuPopup(hWnd, wParam, lParam);
	WM_COMMAND: CallWndProcCommand(hWnd, wParam, lParam);
	WM_PARENTNOTIFY: CallWndProcParentNotify(hWnd, wParam, lParam);
	WM_MDIDESTROY: CallWndProcMDIDestroy(hWnd, wParam, lParam);
	WM_MDISETMENU: CallWndProcMDISetMenu(hWnd, wParam, lParam);
      end;
    end;
    Result := CallNextHookEx(FCallWndHook, nCode, wParam, lParam);
  end;
end;

function PMGetMessageProc(nCode: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    if nCode = HC_ACTION then begin
      if (PMPGPHelp <> nil) and PMPGPHelp.PreTranslateHtmlHelpMsg(PMsg(lParam)) then
	PMsg(lParam)^.Message := WM_NULL
      else with PMsg(lParam)^ do begin
	case Message of
	  WM_PAINT: Message := GetMessageProcPaint(hWnd, wParam, lParam);
	  WM_CLOSE: Message := GetMessageProcClose(hWnd, wParam, lParam);
	  WM_KEYDOWN: Message := GetMessageProcKeyDown(hWnd, wParam, lParam);
	  WM_KEYUP: Message := GetMessageProcKeyUp(hWnd, wParam, lParam);
	  WM_SYSKEYDOWN: Message := GetMessageProcSysKeyDown(hWnd, wParam, lParam);
	  WM_SYSKEYUP: Message := GetMessageProcSysKeyUp(hWnd, wParam, lParam);
	  WM_COMMAND: Message := GetMessageProcCommand(hWnd, wParam, lParam);
	  WM_MOUSEMOVE: Message := GetMessageProcMouseMove(hWnd, wParam, lParam);
	  WM_LBUTTONDOWN: Message := GetMessageProcLButtonDown(hWnd, wParam, lParam);
	  WM_LBUTTONUP: Message := GetMessageProcLButtonUp(hWnd, wParam, lParam);
	  WM_LBUTTONDBLCLK: Message := GetMessageProcLButtonDblClk(hWnd, wParam, lParam);
	  WM_RBUTTONDOWN: Message := GetMessageProcRButtonDown(hWnd, wParam, lParam);
	  PM_TERSETFOCUS: Message := GetMessageProcPMTerSetFocus(hWnd, wParam, lParam);
	end;
      end;
    end;
    Result := CallNextHookEx(FGetMessageHook, nCode, wParam, lParam);
  end;
end;


// Subclass procs ----------------------------------------------------------------------------------

function NewPMFrameProc(hWPMFrame: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    case uMsg of
      WM_CLOSE: if not (FShutDownLogImplemented or FCleanupDone) then begin
	PMInterface.SendShutDownLog(FHiddenWindow, false);
	uMsg := WM_NULL;
      end;
      WM_QUERYENDSESSION: PMInterface.SendShutDownLog(FHiddenWindow, true);
      WM_SHOWWINDOW: if wParam <> 0 then WriteUserEntry(true);
      WM_HELP: with PHelpInfo(lParam)^ do if iContextType = HELPINFO_MENUITEM then begin
	if (hItemHandle = FMainMenu) or (hItemHandle = FInfoMenu)
	or (hItemHandle = FEditMenu) or (hItemHandle = FAddressBookMenu) then begin
	  ShowMenuHelp(hItemHandle, iCtrlID);
	end;
      end;
      WM_COMMAND: uMsg := NewPMFrameProcCommand(wParam, lParam);
      WM_INITMENUPOPUP: begin
	if THandle(wParam) = FMainMenu then
	  SwitchMainMenuItems
	else if (MenuString(wParam, 1, MF_BYPOSITION) = E)
	and (FirstStrPos(DelAmp(PMPGPStrings[SYSTEM_ITEM]), DelAmp(MenuString(wParam, 0, MF_BYPOSITION))) <> 0) then begin
	  UpdateMDIWindowMenu(wParam);
	end;
      end;
      WM_MENUSELECT: if DWord(wParam) <> $FFFF0000 then begin
	if (THandle(lParam) = FMainMenu) or (THandle(lParam) = FInfoMenu) or (THandle(lParam) = FEncryptSignMenu)
	or (THandle(lParam) = FEditMenu) or (THandle(lParam) = FAddressBookMenu) then begin
	  FAddressBookSelected := (THandle(lParam) = FAddressBookMenu);
	  ShowMenuInfo(lParam, LoWord(wParam));
	  Result := 0;
	  Exit;
	end;
      end;
      WM_MENUCHAR: begin
	if (THandle(lParam) = FMainMenu) and (GetKeyState(VK_CONTROL) < 0)
	and IsAccel(LoWord(wParam) + $40,
		    MenuString(lParam, IDM_MAINMENU + ord(miAddKeysOrCertificates), MF_BYCOMMAND)) then begin
	  PMInterface.SelectFrameMenuItem(IDM_MAINMENU + ord(miAddKeysOrCertificates));
	end;
      end;
      WM_EXITMENULOOP: RemoveAddressBookMenu(hWPMFrame);
    end;
    Result := CallWindowProc(FPMFrameProc, hWPMFrame, uMsg, wParam, lParam);
    if (uMsg = WM_COMMAND) and (LoWord(wParam) in [IDM_VERIFY_DSIG, IDM_KEY_MANAGEMENT]) then begin
      FManageOrVerifyTriggered := false;
    end;
  end;
end;

function NewPMStatusBarChildProc(hPMStatusBarChild: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;

  procedure AdjustTooltipOrigin(var LabelRect: TRect);
  begin
    with LabelRect.TopLeft do begin
      inc(X, GetSystemMetrics(SM_CXSMICON) - 2);
      dec(Y, 2);
    end;
  end;

var
  sWndText: String;
  rLabel: TRect;
begin
  with MainClass do begin
    case uMsg of
      WM_SETTEXT: begin
	sWndText := DelQuotes(PChar(lParam));
	if IsBrowserUrl(sWndText) then lParam := Longint(PChar(TerInterface.DecodeUrl(sWndText)));
      end;
      WM_NOTIFY: begin
	case PNMHdr(lParam)^.code of
	  TTN_SHOW: begin
	    if FPMStatusBarChildProc <> NIL then
	      rLabel := GetWindowRect(hPMStatusBarChild)
	    else rLabel := GetWindowRect(FindStatic(hPMStatusBarChild, HEX_MARKER));
	    AdjustTooltipOrigin(rLabel);
	    with rLabel.TopLeft do begin
	      SetWindowPos(FStatusToolTip, 0, X, Y, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
	      if GetCapture <> hPMStatusBarChild then SetCapture(hPMStatusBarChild);
	    end;
	    Result := 1;
	    Exit;
	  end;
	  TTN_POP: begin
	    SetTimer(
		     FHiddenWindow, TIMER_HINT,
		     SendMessage(FStatusToolTip, TTM_GETDELAYTIME, MulDiv(TTDT_RESHOW, 3, 2), 0),
		     nil
		    );
	    ReleaseCapture;
	    Result := 0;
	    Exit;
	  end;
	end;
      end;
    end;
    if FPMStatusBarChildProc = NIL then begin
      Result := 0;
      Exit;
    end;
    Result := CallWindowProc(FPMStatusBarChildProc, hPMStatusBarChild, uMsg, wParam, lParam);
  end;
end;

function NewPMStatusBarProc(hPMStatusBar: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    case uMsg of
      WM_MOVE, WM_SIZE: RefreshStatusBarTime;
      WM_PAINT: if ShowTime > 0 then ShowStatusBarTime(hPMStatusBar);
      WM_NOTIFY: if FPMStatusBarChildProc = NIL then begin
	case PNMHdr(lParam)^.code of
	  TTN_SHOW: begin
	    Result := NewPMStatusBarChildProc(hPMStatusBar, uMsg, wParam, lParam);
	    Exit;
	  end;
	  TTN_POP: begin
	    Result := NewPMStatusBarChildProc(hPMStatusBar, uMsg, wParam, lParam);
	    Exit;
	  end;
	end;
      end;
      WM_LBUTTONDBLCLK: if (FTimeToolTip <> 0) and PtInRect(FTimeRect, Point(LoWord(lParam), HiWord(lParam))) then begin
	SyncSysTimeCommand(FPMFrameWindow, true);
	Result := 0;
	Exit;
      end;
    end;
    Result := CallWindowProc(FPMStatusBarProc, hPMStatusBar, uMsg, wParam, lParam);
  end;
end;

function NewPMTextEditProc(hPMTextEdit: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  pTextEditProc: TFNWndProc;
begin
  pTextEditProc := TFNWndProc(GetProp(hPMTextEdit, TER_PARENT_PROC_PROP));
  with MainClass do begin
    case uMsg of
      TER_LINK: if TERInterface.IsTerReader(GetWindow(hPMTextEdit, GW_CHILD)) then begin
	if StrStart(PMPGP_URL, TERInterface.GetUrl(lParam)) then with PMInterface do begin
	  if not IsChild(CheckMDIFolman(0), hPMTextEdit) then FTerLinkClicked := true;
	  SelectFrameMenuItem(IDM_MAINMENU + ord(miDecryptMessage));
	  Result := 1;
	  Exit;
	end;
      end;
    end;
    Result := CallWindowProc(pTextEditProc, hPMTextEdit, uMsg, wParam, lParam);
    if uMsg = WM_DESTROY then TextEditUnSubclass(hPMTextEdit);
  end;
end;

function NewPMChangeMbxDlgProc(hPMChangeMbxDlg: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    case uMsg of
      WM_DESTROY: SetWindowLong(hPMChangeMbxDlg, GWL_WNDPROC, Longint(FPMChangeMbxDlgProc));
      WM_COMMAND: begin
	if (HiWord(wParam) = BN_CLICKED) and (LoWord(wParam) = CTL_CHANGE_NOW)
	and (PMInterface.PMVersionNumber < 4500) then begin
	  Info(PMPGPStrings[CHANGE_MAILBOX_INFO]);
	  Result := 0;
	  Exit;
	end;
      end;
    end;
    Result := CallWindowProc(FPMChangeMbxDlgProc, hPMChangeMbxDlg, uMsg, wParam, lParam);
  end;
end;

function NewPMChangeUserDlgProc(hPMChangeUserDlg: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    case uMsg of
      WM_DESTROY: begin
	SetWindowLong(hPMChangeUserDlg, GWL_WNDPROC, Longint(FPMChangeUserDlgProc));
	if FNewPMUser <> E then Info(PMPGPStrings[CHANGE_USER_INFO]);
      end;
      WM_COMMAND: begin
	case wParam of
	  ID_OK: if FNewPMUser = E then begin
	    FNewPMUser := WndString(FindEditControl(hPMChangeUserDlg));
	    if AnsiCompareText(FNewPMUser, PMInterface.PMUser) = 0 then FNewPMUser := E;
	    lParam := GetDlgItem(hPMChangeUserDlg, ID_CANCEL);
	    wParam := ID_CANCEL;
	  end;
	  ID_CANCEL: SetWindowText(FindEditControl(hPMChangeUserDlg), nil);
	end;
      end;
    end;
    Result := CallWindowProc(FPMChangeUserDlgProc, hPMChangeUserDlg, uMsg, wParam, lParam);
    if (uMsg = WM_DESTROY) and (FNewPMUser <> E) and (PMInterface.PMVersionNumber < 4500) then begin
      RestartPegasusMail(FNewPMUser, FPMFrameWindow);
    end;
  end;
end;

function NewMDIReaderProc(hMDIReader: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  prd: PMDIReaderData;
begin
  with MainClass do begin
    prd := PMDIReaderData(GetProp(hMDIReader, MDI_READER_DATA_PROP));
    Result := CallWindowProc(prd^.ReaderProc, hMDIReader, uMsg, wParam, lParam);
    case uMsg of
      WM_DESTROY: MDIReaderUnSubclass(hMDIReader);
      WM_PAINT, WM_CHILDACTIVATE: DrawMDIReaderIcon(hMDIReader, true);
      WM_NCACTIVATE: DrawMDIReaderIcon(hMDIReader, wParam <> 0);
    end;
  end;
end;

function NewSendParentProc(hSendParent: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  ped: PMDIEditorData;
begin
  with MainClass do begin
    ped := PMDIEditorData(GetProp(GetMDIParent(hSendParent), MDI_EDITOR_DATA_PROP));
    if (uMsg = WM_COMMAND) and (LoWord(wParam) = CMD_SEND_MESSAGE) then begin
      if not FAutoSending then FRecipientsData.Clear;
      FSendingMail := true;
    end;
    Result := CallWindowProc(ped^.SendParentProc, hSendParent, uMsg, wParam, lParam);
    if (uMsg = WM_COMMAND) and (LoWord(wParam) = CMD_SEND_MESSAGE) then FAutoSending := false;
  end;
end;

function NewMDIEditorProc(hMDIEditor: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  ped: PMDIEditorData;
begin
  with MainClass do begin
    ped := PMDIEditorData(GetProp(hMDIEditor, MDI_EDITOR_DATA_PROP));
    Result := CallWindowProc(ped^.EditorProc, hMDIEditor, uMsg, wParam, lParam);
    case uMsg of
      WM_DESTROY: MDIEditorUnSubclass(hMDIEditor);
      WM_SHOWWINDOW: if wParam <> 0 then begin
	ped^.SendParentProc := Ptr(SetWindowLong(GetParent(FindCtrlByID(hMDIEditor, CMD_SEND_MESSAGE)),
						 GWL_WNDPROC, Longint(@NewSendParentProc)));
      end
      else begin
	SetWindowLong(GetParent(FindCtrlByID(hMDIEditor, CMD_SEND_MESSAGE)),
		      GWL_WNDPROC, Longint(ped^.SendParentProc));
      end;
    end;
  end;
end;

function NewEncryptionDlgProc(hEncryptionDlg: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;

  procedure ApplyAutoFlags(Value: Integer);
  begin
    with MainClass do begin
      SetDialogCheckBox(GetDlgItem(hEncryptionDlg, CTL_ENCRYPT_OPTION), (Value and AUTO_FLAG_ENCRYPT) <> 0);
      SetDialogCheckBox(GetDlgItem(hEncryptionDlg, CTL_SIGN_OPTION), (Value and AUTO_FLAG_SIGN) <> 0);
    end;
  end;

  procedure GetAutoFlags(var Value: Integer; var AddressBookList: TAddressBookList);
  var
    iEncryptor: Integer;
    fAllInOne: Longbool;
  begin
    with MainClass do begin
      Value := (Value and not (AUTO_FLAG_ENCRYPT or AUTO_FLAG_SIGN)) or
	       ord(IsDialogCheckBoxChecked(GetDlgItem(hEncryptionDlg, CTL_ENCRYPT_OPTION)) = true) * AUTO_FLAG_ENCRYPT or
	       ord(IsDialogCheckBoxChecked(GetDlgItem(hEncryptionDlg, CTL_SIGN_OPTION)) = true) * AUTO_FLAG_SIGN;
      iEncryptor := GetEncryptor(hEncryptionDlg, fAllInOne);
      if (FRecipientsData.Count = 0) and (Value and AUTO_FLAG_MIME_FILTER = 0) then begin
	Value := Value or iEncryptor;
	if (Value and AUTO_FLAG_SMIME) = 0 then Value := Value or (ord(UsePGPMime = true) * AUTO_FLAG_PMIME);
      end
      else Value := Value or (FRecipientsData.DefaultFlags and AUTO_FLAG_MIME_FILTER);
      AddressBookList.Free;
      if fAllInOne then with PMInterface do begin
	AddressBookList := TAddressBookList.Create(AddressBooksPathList);
	if AddressBookList <> nil then AddressBookList.CreateAddressCache(AddressBookFlagsList);
      end
      else AddressBookList := nil;
    end;
  end;

begin
  with MainClass do begin
    case uMsg of
      WM_DESTROY: with GetMDIEditorDataPtr(PMInterface.CheckMDIEditor(0))^ do begin
	SetWindowLong(hEncryptionDlg, GWL_WNDPROC, Longint(FEncryptionDlgProc));
	if EncodeFlag > efCancel then GetAutoFlags(AutomationFlags, AddressBookList);
      end;
      WM_SHOWWINDOW: if wParam <> 0 then begin
	with GetMDIEditorDataPtr(PMInterface.CheckMDIEditor(0))^ do begin
	  EnableWindow(GetDlgItem(hEncryptionDlg, CTL_ENCRYPT_OPTION), true);
	  EnableWindow(GetDlgItem(hEncryptionDlg, CTL_SIGN_OPTION), true);
	  EnableWindow(GetDlgItem(hEncryptionDlg, CTL_ENTER_PASSWORD), false);
	  SetEncryptor(hEncryptionDlg, AutomationFlags);
	  case EncodeFlag of
	    efAsk, efAuto: begin
	      if FRecipientsData.Count > 0 then
		ApplyAutoFlags(FRecipientsData.DefaultFlags)
	      else ApplyAutoFlags(AUTO_FLAG_ENCRYPT or (ord(AutoSign = true) * AUTO_FLAG_SIGN));
	    end;
	    efEncrypt: ApplyAutoFlags(AUTO_FLAG_ENCRYPT);
	    efSign: ApplyAutoFlags(AUTO_FLAG_SIGN);
	    efEncryptSign: ApplyAutoFlags(AUTO_FLAG_ENCRYPT or AUTO_FLAG_SIGN);
	  else
	    ApplyAutoFlags(AUTO_FLAG_ENCRYPT or ord(AutoSign = true) * AUTO_FLAG_SIGN);
	  end;
	  if (AutoEncrypt and (EncodeFlag > efAsk)) or (EncodeFlag > efAuto) then ClickDlgBtn(hEncryptionDlg, ID_OK);
	end;
      end;
      WM_COMMAND: with GetMDIEditorDataPtr(PMInterface.CheckMDIEditor(0))^ do begin
	case HiWord(wParam) of
	  CBN_SELCHANGE: begin
	    if SendMessage(lParam, CB_GETCURSEL, 0, 0) > 0 then
	      EnableWindow(FindEditControl(hEncryptionDlg), false)
	    else EnableWindow(FindEditControl(hEncryptionDlg), true);
	  end;
	  CBN_SELENDOK: if ShowAIOWarning and StrStart(AIO, GetEncryptorName(hEncryptionDlg)) then begin
	    AIOWarning := not WarnWithCheckBox(PMPGPStrings[ALL_IN_ONE_WARNING], PMPGPStrings[DONT_WARN_AGAIN_CHECKBOX]);
	    ShowAIOWarning := false;
	  end;
	end;
	if EncodeFlag <= efAuto then begin
	  case LoWord(wParam) of
	    ID_OK: if EncodeFlag in [efNone, efCancel] then EncodeFlag := efOk;
	    ID_CANCEL: EncodeFlag := efCancel;
	  end;
	end;
      end;
    end;
    Result := CallWindowProc(FEncryptionDlgProc, hEncryptionDlg, uMsg, wParam, lParam);
  end;
end;

function NewDecPhraseEditProc(hDecPhraseEdit: THandle; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  with MainClass do begin
    case uMsg of
      WM_DESTROY: SetWindowLong(hDecPhraseEdit, GWL_WNDPROC, Longint(FDecPhraseEditProc));
      WM_KEYDOWN: FWaitThreadStop := true;
    end;
    Result := CallWindowProc(FDecPhraseEditProc, hDecPhraseEdit, uMsg, wParam, lParam);
  end;
end;


// Hidden window handling --------------------------------------------------------------------------

function HiddenWindowProc(hWindow: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  hMDIEditor: THandle;
begin
  Result := 0;
  with MainClass do begin
    case uMsg of
      WM_CREATE: FHiddenWindow := hWindow;
      WM_DESTROY: HiddenWindowCleanup(hWindow, 0, 0);
      WM_TCARD: PMPGPHelp.DontClose := Longbool(wParam);
      WM_HELP: Result := HiddenWindowHelp(wParam, lParam);
      WM_COMMAND: Result := HiddenWindowCommand(wParam, lParam);
      WM_TIMER: begin
	case wParam of
	  TIMER_WIPE_CACHE: begin
	    KillTimer(FHiddenWindow, TIMER_WIPE_CACHE);
	    PGPInterface.WipeCache;
	  end;
	  TIMER_AUTO_SEND: begin
	    FAutoSending := true;
	    KillTimer(FHiddenWindow, TIMER_AUTO_SEND);
	    hMDIEditor := PMInterface.CheckMDIEditor(0);
	    if hMDIEditor <> 0 then with GetMDIEditorDataPtr(hMDIEditor)^ do begin
	      PMInterface.SendMailMessage(false, EncodeFlag > efCancel);
	    end;
	  end;
	  TIMER_RAW_VIEW: begin
	    Sleep(0);
	    ProcessMessages(0);
	    KillTimer(FHiddenWindow, TIMER_RAW_VIEW);
	    PMInterface.SelectFrameMenuItem(IDM_MAINMENU + ord(miDecryptMessage));
	  end;
	  TIMER_HINT: begin
	    KillTimer(FHiddenWindow, TIMER_HINT);
	    ShowVerificationInfo;
	  end;
	  TIMER_TIME: if ShowTime > 0 then ShowStatusBarTime(0);
	end;
      end;
      WM_MENUSELECT: Result := HiddenWindowMenuSelect(wParam, lParam);
      WM_EXITMENULOOP: Result := HiddenWindowExitMenuLoop(wParam, lParam);
      WM_FM_INIT: FCurrentDialogOwner := HiddenWindowInit(wParam, lParam);
      WM_FM_SENDMAIL: Result := HiddenWindowSendMail(wParam, lParam);
      WM_FM_LOGEVENT: Result := HiddenWindowLogEvent(hWindow, wParam, lParam);
    else
      Result := DefWindowProc(hWindow, uMsg, wParam, lParam);
    end;
  end;
end;

procedure RegisterHiddenClass;
var
  rWndClass: TWndClass;
begin
  FillByte(rWndClass, SizeOf(TWndClass), 0);
  rWndClass.lpfnWndProc := @HiddenWindowProc;
  rWndClass.hInstance := hInstance;
  rWndClass.lpszClassName := HIDDEN_CLASS;
  Windows.RegisterClass(rWndClass);
end;


// TMainClass --------------------------------------------------------------------------------------

constructor TMainClass.Create;
begin
  inherited Create;
  RegisterHiddenClass;
  FMDIWindows := TList.Create;
  FMimeMsgTree := TMimeMsgTree.Create(nil);
  FRecipientsData := TRecipientsData.Create(E);
  FRecipientsData.Duplicates := true;
  FCurrentDialogOwner := GetDesktopWindow;
  FCallWndHook := SetWindowsHookEx(WH_CALLWNDPROC, @PMCallWndProc, 0, GetCurrentThreadID);
  FGetMessageHook := SetWindowsHookEx(WH_GETMESSAGE, @PMGetMessageProc, 0, GetCurrentThreadID);
end;

destructor TMainClass.Destroy;
begin
  if IsWindow(FHiddenWindow) then DestroyWindow(FHiddenWindow);
  Windows.UnregisterClass(HIDDEN_CLASS, hInstance);
  UnhookWindowsHookEx(FGetMessageHook);
  UnhookWindowsHookEx(FCallWndHook);
  FRecipientsData.Free;
  FMimeMsgTree.Free;
  FMDIWindows.Free;
  inherited;
end;

function TMainClass.Initialize(ParentWindow: THandle; InterfaceVersion: Integer): THandle;
var
  dwFatalError: DWord;
begin
  if FInstance = 0 then begin
    Result := CreateWindow(HIDDEN_CLASS, E, WS_CHILD * ord(ParentWindow <> 0), 0, 0, 0, 0, ParentWindow, 0, hInstance, nil);
    try
      if Result <> 0 then begin
	FStandaloneMode := (ParentWindow = 0);
	FShutDownLogImplemented := (InterfaceVersion >= $0112);
	if not FStandaloneMode then Application.Handle := Result;
	dwFatalError := inherited Initialize(Result, ParentWindow);
      end
      else dwFatalError := GetLastError;
    except
      dwFatalError := ERROR_DLL_INIT_FAILED;
    end;
    if dwFatalError <> 0 then begin
      if dwFatalError <> $FFFFFFFF then ShowFatalErrorMessage(dwFatalError);
      DestroyWindow(Result);
      Result := 0;
    end;
  end
  else Result := FHiddenWindow;
end;

// Public methods

function TMainClass.UnknownHexIDPopup(Window: THandle; const Text: String): Longbool;
const
  LB = '[ ';
  RB = ' ]';
var
  sHexID: String;
begin
  Result := false;
  if Text = E then begin
    if FPMStatusBarChildProc <> NIL then
      Window := FindStatusBarChild(LB + HEX_MARKER)
    else Window := FindStatic(Window, LB + HEX_MARKER);
    if (Window <> 0) and IsWindowVisible(Window) then sHexID := WndString(Window);
  end
  else sHexID := Text;
  if sHexID <> E then begin
    Delete(sHexID, LastStrPos(RB, sHexID), MAXINT);
    Delete(sHexID, 1, succ(FirstStrPos(LB, sHexID)));
    if StrStart(HEX_MARKER, sHexID) then begin
      ShowIDMenu(Window, sHexID, true, false);
      Result := true;
    end;
  end;
end;

procedure TMainClass.ManageKeys(const KeyData: String; FileName: Longbool);
var
  sKeyData, sFileName, sArmored: String;
  hReader: THandle;
  md: TMessageData;
  kidd: TKeyIDData;
begin
  if KeyData <> E then begin
    FManageOrVerifyTriggered := false;
    if not FileName then
      sKeyData := KeyData
    else if KeyData <> FManageOrVerifyFileName then begin
      FManageOrVerifyFileName := KeyData;
      if not ReadTextFromFile(KeyData, sKeyData) then begin
	Error(PMPGPStrings[FILE_OPEN_ERROR] + PARA + KeyData);
	Exit;
      end;
    end
    else begin
      FManageOrVerifyFileName := E;
      Exit;
    end;
  end
  else begin
    hReader := FindReader;
    if hReader <> 0 then sKeyData := TERInterface.GetKeyData(hReader, FMimeMsgTree, PGPSMimeCapability <> smDisabled);
    if sKeyData = E then begin
      if GetKeyState(VK_CONTROL) < 0 then begin
	sFileName := KeyFileOpen(PMPGPStrings[OPEN_FILE_DIALOG], PMPGPStrings[FROM_CLIPBOARD_BUTTON], FHiddenWindow);
	if sFileName <> E then begin
	  if sFileName = CLIP_NAME then begin
	    ReadClipboardText(sKeyData, false, hReader);
	    sKeyData := AdjustLineBreaks(StripExcessCRs(sKeyData));
	    sArmored := PGPInterface.CheckArmor(sKeyData, md);
	    if md.Encoding = eNone then sArmored := GetEncodedSection(sKeyData, true);
	  end
	  else if not ReadTextFromFile(sFileName, sArmored) then begin
	    Error(PMPGPStrings[FILE_OPEN_ERROR] + PARA + sFileName);
	    Exit;
	  end;
	  with PGPInterface do begin
	    kidd := ImportKeys(sArmored, FCurrentDialogOwner, UseAutoList, false);
	    if not ShowError(ErrorCode) and UseAutoList then AddNewKeysToAutoList(kidd);
	  end;
	end;
      end
      else PMInterface.SelectFrameMenuItem(IDM_KEY_MANAGEMENT);
      Exit;
    end;
  end;
  if sKeyData <> E then with PGPInterface do begin
    sKeyData := AdjustLineBreaks(StripExcessCRs(sKeyData));
    sArmored := CheckArmor(sKeyData, md);
    if md.Encoding = eNone then sArmored := GetEncodedSection(sKeyData, true);
    kidd := FindKeys(sArmored, FCurrentDialogOwner, UseAutoList);
    if ErrorCode <> 0 then kidd := ImportKeys(CheckArmor(kidd.UserIDLineList, md), FCurrentDialogOwner, UseAutoList, false);
    if not ShowError(ErrorCode) and UseAutoList then AddNewKeysToAutoList(kidd);
  end;
end;

procedure TMainClass.VerifySignature(const FileName: String);
var
  sSignatureData, sArmored: String;
  md: TMessageData;
  iPos: Integer;
begin
  if FileName <> E then begin
    if FileName <> FManageOrVerifyFileName then begin
      FManageOrVerifyFileName := FileName;
      FManageOrVerifyTriggered := false;
      if ReadTextFromFile(FileName, sSignatureData) then with PGPInterface do begin
	sSignatureData := AdjustLineBreaks(StripExcessCRs(sSignatureData));
	sArmored := CheckArmor(sSignatureData, md);
	if md.Encoding = eNone then sArmored := GetEncodedSection(sSignatureData, false);
	with md do if Encoding < eCerts then begin
	  iPos := FirstStrPos(PARA, Prefix);
	  if iPos > 0 then begin
	    Delete(Prefix, 1, iPos + pred(Length(PARA)));
	    sArmored := Prefix + sArmored + Suffix;
	  end;
	end;
	FSignatureData := VerifySignature(sArmored, E, FCurrentDialogOwner, KeyServerLookup, true);
	ShowVerificationInfo;
      end
      else Error(PMPGPStrings[FILE_OPEN_ERROR] + PARA + FileName);
    end
    else FManageOrVerifyFileName := E;
  end
  else PMInterface.SelectFrameMenuItem(IDM_VERIFY_DSIG);
end;

procedure TMainClass.SynchronizeSystemTime(Show: Longbool);
var
  cErrorCode: Cardinal;
begin
  cErrorCode := SynchronizeTime(TimeServerName, TimeServerPort);
  if not Show then Exit;
  if cErrorCode = 0 then
    Info(PMPGPStrings[SYNC_CLOCK_INFO])
  else SystemError('%s', cErrorCode);
end;

function TMainClass.DecodeMessage(const SourceFile, DestFile: String; Flags: Integer): Integer;
var
  prdActive: PMDIReaderData;
  fDoDecode: Longbool;
  hMDIChild: THandle;
  sMailMessage, sArmored, sAttachment, sDecoded, sCharset: String;
  mdDecode, mdKeyCheck: TMessageData;
  snKeyData: TSectionNode;
  kdImported: TKeyIDData;
begin
  Result := 0;
  PGPInterface.EnableCaching;
  try
    if SourceFile <> E then begin
      prdActive := nil;
      fDoDecode := true;
      hMDIChild := GetActiveMDIChild;
      with PMInterface do begin
	if CheckMDIFolman(hMDIChild) <> 0 then begin
	  FDecodeOptions.DecodeOnly := true;
	  fDoDecode := GetMDIMenuItemChecked(IDM_PREVIEW_MODE);
	  FMimeMsgTree.MaxLineLength := GetMaxWindowTextWidth(BearOrTer(hMDIChild, false));
	end
	else if CheckMDIReader(hMDIChild) <> 0 then begin
	  FMimeMsgTree.MaxLineLength := GetMaxWindowTextWidth(BearOrTer(hMDIChild, PMInterface.HasNewReader));
	  prdActive := GetMDIReaderDataPtr(hMDIChild);
	end;
      end;
      with FDecodeOptions do PrintMessage := Printing;
      if fDoDecode and not FManageOrVerifyTriggered and ReadTextFromFile(SourceFile, sMailMessage) then begin
	if not FDecodeOptions.ShowAttachments then begin
	  sMailMessage := AdjustLineBreaks(StripExcessCRs(sMailMessage));
	  sArmored := PGPInterface.CheckArmor(sMailMessage, mdDecode);
	  if sArmored <> E then begin
	    sAttachment := IsPMAttachment(SourceFile, sMailMessage, EXT_PM_TEMP);
	    if sAttachment <> E then with mdDecode do begin
	      Prefix := E;
	      Suffix := E;
	      if FDecodeOptions.Attachment then begin
		if (sAttachment = ASC) or (sAttachment = PGP) then begin
		  if WriteTextToFile(DestFile, sArmored) then Result := 2;
		  Exit;
		end
		else if (Encoding in [ePlainSigned, ePMEncrypted..eSMInline]) then begin
		  if WriteTextToFile(DestFile, sMailMessage) then Result := 2;
		  Exit;
		end;
	      end;
	    end;
	    if not (PMInterface.IsRawView(hMDIChild) or FDecodeOptions.FirstPassDone) or FDecodeOptions.DecodeOnly then begin
	      PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + DOTS;
	      sDecoded := PGPInterface.DecodeMessage(sArmored, FCurrentDialogOwner, KeyServerLookup,
						     mdDecode.Encoding < eCerts, true, false,
						     FSignatureData, kdImported, sCharset);
	      if PGPInterface.NoError or (mdDecode.Encoding in [eNone, ePlainSigned, ePMSigned, eSMSigned]) then begin
		if mdDecode.Encoding > eCerts then with FMimeMsgTree do begin
		  Clear;
		  if not FDecodeOptions.ShowAttachments then begin
		    MimeParse(sMailMessage, 1, 0, GetMimeDecodeOptions);
		    if ContainsData then begin
		      BodyEncoding := mdDecode.Encoding;
		      sDecoded := GetMessageText(false).AnsiText;
		      if FSignatureData.Flags = SF_NO_KEY then begin
			snKeyData := FindPSEncodedSection(ePlainKeys);
			if snKeyData <> nil then with PGPInterface do begin
			  FSignatureData := VerifySignature(sArmored, snKeyData.SectionText,
							    FCurrentDialogOwner, KeyServerLookup, false);
			end;
		      end;
		    end;
		    if not FirstTextIsFormatted then FDecodeOptions.FancyReply := false;
		  end;
		end
		else begin
		  if sCharset <> E then
		    sDecoded := ContentCheck(sDecoded, sMailMessage, sCharset, PMInterface.ReaderFont)
		  else if not ((sAttachment <> E) and IsBinary(sDecoded))
		  and (mdDecode.Encoding in [ePlainEncrypted, ePlainSigned]) then begin
		    sDecoded := ContentCheck(sDecoded, sMailMessage, E, PMInterface.ReaderFont);
		  end;
		  FDecodeOptions.FancyReply := false;
		end;
		PGPInterface.InsertExtraDataInfo(FSignatureData, mdDecode);
		if not FDecodeOptions.DecodeOnly then with FSignatureData do begin
		  if not AutoVerify or (InfoOption = vsOnStatusBarOnly) then
		    Text := E
		  else Text := PGPInterface.FormatVerificationInfo(Text);
		end;
		Result := 2;
	      end
	      else begin
		if mdDecode.Encoding >= eSMEncrypted then begin
		  sDecoded := Copy(sMailMessage, FirstStrPos(PARA, sMailMessage) + Length(PARA), MAXINT);
		end;
		ClearSignatureData(FSignatureData);
		with mdDecode do begin
		  Prefix := E;
		  Suffix := E;
		end;
	      end;
	      if Result <> 0 then begin
		with mdDecode do begin
		  if AutoVerify and (Encoding = ePlainSigned) then sDecoded := StripSignatureData(sDecoded);
		  if sDecoded = E then sDecoded := sArmored;
		  if FDecodeOptions.FancyReply then
		    sMailMessage := Prefix
		  else begin
		    if Encoding > eCerts then with FDecodeOptions do begin
		      if Replying then
			Prefix := Utf8ToAnsi(FMimeMsgTree.AnsiHeaders) + PARA
		      else if Forwarding then Prefix := FMimeMsgTree.AnsiHeaders + PARA;
		      sMailMessage := Prefix + sDecoded;
		    end
		    else sMailMessage := Prefix + sDecoded + Suffix;
		  end;
		  with PMInterface do if Encoding > eCerts then begin
		    FDecodeOptions.HotUrls := ReaderHotUrls;
		    ReaderHotUrls := FALSE;
		  end;
		end;
		Result := Result * (ord(WriteTextToFile(DestFile, sMailMessage) = true));
		if CheckForKeyData and not FDecodeOptions.DecodeOnly then begin
		  with PGPInterface do begin
		    if not (mdDecode.Encoding in [ePlainKeys, eCerts]) then
		      sArmored := CheckArmor(sDecoded, mdKeyCheck)
		    else sArmored := CheckArmor(sArmored, mdKeyCheck);
		    if sArmored <> E then kdImported := ImportKeys(sArmored, FCurrentDialogOwner, UseAutoList, false);
		  end;
		end;
	      end;
	      if Result = 0 then
		PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[CANCELLED_INFO]
	      else PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[FINISHED_INFO];
	      if (FSignatureData.Text <> E) and not FDecodeOptions.Printing then ShowVerificationInfo;
	      if Result <> 0 then begin
		if UseAutoList then AddNewKeysToAutoList(kdImported);
		if (mdDecode.Encoding > eCerts) and FDecodeOptions.ShowAttachments then begin
		  with mdDecode do begin
		    if not StrStart(DD, sDecoded) then
		      sMailMessage := sDecoded + Suffix
		    else sMailMessage := Prefix + sDecoded + Suffix;
		  end;
		  SendToBack(hMDIChild);
		  ShowMsgTree(Self, sMailMessage);
		  PostMessage(hMDIChild, WM_CLOSE, 0, 0);
		  FDecodeOptions.ShowAttachments := false;
		end;
	      end
	      else if WriteTextToFile(DestFile, sMailMessage) then Result := 2;
	      if PMInterface.HasNewReader then with FDecodeOptions do begin
		if NewMessage and not FirstPassDone and (prdActive <> nil) and prdActive^.RawViewActive then
		  FirstPassDone := true
		else FirstPassDone := false;
	      end;
	    end
	    else begin
	      FDecodeOptions.FirstPassDone := false;
	      if prdActive <> nil then prdActive^.RawViewActive := true;
	      if WriteTextToFile(DestFile, sMailMessage) then Result := 2;
	    end;
	  end
	  else if WriteTextToFile(DestFile, sMailMessage) then Result := 2;
	end
	else with FMimeMsgTree do begin
	  Clear;
	  MimeParse(sMailMessage, 1, 0, [mdoAnalyseEncoding]);
	  if BodyEncoding > eNone then begin
	    Self.SendToBack(hMDIChild);
	    ShowMsgTree(Self, sMailMessage);
	    PostMessage(hMDIChild, WM_CLOSE, 0, 0);
	    FDecodeOptions.ShowAttachments := false;
	  end;
	end;
      end
      else if CopyFile(PChar(SourceFile), PChar(DestFile), false) then Result := 2;
    end;
  finally
    PGPInterface.ResetCaching;
  end;
end;

function TMainClass.EncodeMessage(const SourceFile, DestFile, Recipient, FormData: String): Integer;
begin
  Result := 0;
  PGPInterface.EnableCaching;
  try
    if SourceFile <> E then begin
      if DestFile <> E then
	Result := PreProcessMessage(SourceFile, DestFile)
      else Result := PostProcessMessage(SourceFile, Recipient, FormData);
    end;
  finally
    PGPInterface.ResetCaching;
  end;
end;

// Switch items

procedure TMainClass.SwitchUsePGPMime;
begin
  inherited;
  CheckMenuItem(FMainMenu, IDM_MAINMENU + ord(miUsePGPMime), UsePGPMime);
end;

procedure TMainClass.SwitchAppendPGPExtension;
begin
  inherited;
  CheckMenuItem(FMainMenu, IDM_MAINMENU + ord(miAppendPGPExtension), AppendPGPExtension);
end;

procedure TMainClass.SwitchAutoEncrypt;
begin
  inherited;
  CheckMenuItem(FMainMenu, IDM_MAINMENU + ord(miAutoEncrypt), AutoEncrypt);
end;

procedure TMainClass.ExecShowTime;
begin
  if ShowTime = 0 then begin
    KillTimer(FHiddenWindow, TIMER_TIME);
    SetRectEmpty(FTimeRect);
    DestroyTimeToolTip;
  end
  else SetTimer(FHiddenWindow, TIMER_TIME, 1000, nil);
  InvalidateRect(FPMStatusBarWindow, nil, true);
end;

// Configure

function TMainClass.Configure: Longbool;
begin
  Result := ConfigDialog.Configure(Self, FStandaloneMode);
end;

// Toolbar buttons

procedure TMainClass.AddToolbarButtons;
var
  sEncryptSignButton: String;
  iPos: Integer;
begin
  if PMInterface <> nil then with PMInterface do begin
    AddButton(BTN_EDIT_AUTOLIST, PMPGPStrings[EDIT_AUTOLIST_BUTTON], FHiddenWindow,
	      WM_COMMAND, 0, WM_EDIT_AUTOLIST);
    if WipePassphraseButton then begin
      AddButton(BTN_WIPE_PASSPHRASE, PMPGPStrings[WIPE_PASSPHRASE_BUTTON], FHiddenWindow,
		WM_COMMAND, 0, WM_WIPE_PASSPHRASE);
    end;
    sEncryptSignButton := PMPGPStrings[ENCRYPT_SIGN_BUTTON];
    if (EncryptSignButton = esShowMenu) then begin
      AddButton(BTN_ENCRYPT_SIGN, sEncryptSignButton, FHiddenWindow,
		WM_COMMAND, 0, WM_ENCRYPT_SIGN);
    end
    else begin
      System.Delete(sEncryptSignButton, pred(FirstCharPos(LB, sEncryptSignButton)), MAXINT);
      if (EncryptSignButton = esEncryptSign) then begin
	AddButton(BTN_ENCRYPT_SIGN, sEncryptSignButton, FHiddenWindow,
		  WM_COMMAND, IDM_NEWBUTTONS + ord(miEncryptSign), WM_ENCRYPT_SIGN);
      end
      else begin
	iPos := FirstCharPos(SL, sEncryptSignButton);
	if EncryptSignButton = esEncrypt then begin
	  System.Delete(sEncryptSignButton, iPos, FirstCharPos(SP, sEncryptSignButton) - iPos);
	  AddButton(BTN_ENCRYPT_ONLY, sEncryptSignButton, FHiddenWindow,
		    WM_COMMAND, IDM_NEWBUTTONS + ord(miEncrypt), WM_ENCRYPT_SIGN);
	end
	else if EncryptSignButton = esSign then begin
	  System.Delete(sEncryptSignButton, 1, iPos);
	  AddButton(BTN_SIGN_ONLY, sEncryptSignButton, FHiddenWindow,
		    WM_COMMAND, IDM_NEWBUTTONS + ord(miSign), WM_ENCRYPT_SIGN);
	end;
      end;
    end;
    if DecryptMessageButton then begin
      AddButton(BTN_DECRYPT_MESSAGE, PMPGPStrings[DECRYPT_MESSAGE_BUTTON], FHiddenWindow,
		WM_COMMAND, 0, WM_DECRYPT_MESSAGE);
    end;
    if VerifySignatureButton then begin
      AddButton(BTN_VERIFY_SIGNATURE, PMPGPStrings[VERIFY_SIGNATURE_BUTTON], FHiddenWindow,
		WM_COMMAND, 0, WM_VERIFY_SIGNATURE);
    end;
    if ManageKeysButton then begin
      AddButton(BTN_MANAGE_KEYS, PMPGPStrings[MANAGE_KEYS_BUTTON], FHiddenWindow,
		WM_COMMAND, 0, WM_MANAGE_KEYS);
    end;
    if LaunchPGPButton then begin
      AddButton(BTN_LAUNCH_PGP, PMPGPStrings[LAUNCH_PGP_BUTTON], FHiddenWindow,
		WM_COMMAND, 0, WM_LAUNCH_PGP);
    end;
  end;
end;

procedure TMainClass.RemoveToolbarButtons;
begin
  if PMInterface <> nil then PMInterface.ClearButtons;
end;

// Main menu

procedure TMainClass.AddEditMenuEntries;
begin
  while GetMenuItemCount(FEditMenu) > 0 do DeleteMenu(FEditMenu, 0, MF_BYPOSITION);
  if UseAutoList then begin
    ALInterface.LoadList;
    AppendMenu(FEditMenu, MF_STRING or (MF_GRAYED * ord(FRecipientsKeyData.KeyCount = 0)),
	       IDM_EDITMENU + ord(miAddRecipients), PChar(PMPGPStrings[ADD_RECIPIENTS_ITEM]));
    AppendMenu(FEditMenu, MF_STRING, IDM_EDITMENU + ord(miAddRingAddresses),
	       PChar(PMPGPStrings[ADD_RING_ADDRESSES_ITEM]));
    AppendMenu(FEditMenu, MF_SEPARATOR, 0, nil);
    AppendMenu(FEditMenu, MF_STRING or (MF_GRAYED * ord(ALInterface.Count = 0 )),
	       IDM_EDITMENU + ord(miSetAddressOptions),
	       PChar(PMPGPStrings[SET_ADRRESS_OPTIONS_ITEM]));
    AppendMenu(FEditMenu, MF_STRING or (MF_GRAYED * ord(ALInterface.Count = 0)),
	       IDM_EDITMENU + ord(miUpdateAutoList),
	       PChar(PMPGPStrings[UPDATE_AUTOLIST_ITEM]));
    AppendMenu(FEditMenu, MF_STRING, IDM_EDITMENU + ord(miEditAutoList),
	       PChar(PMPGPStrings[EDIT_AUTOLIST_ITEM]));
  end
  else begin
    AppendMenu(FEditMenu, MF_STRING, IDM_EDITMENU + ord(miEnableAutoList),
	       PChar(PMPGPStrings[ENABLE_AUTOLIST_ITEM]));
  end;
end;

procedure TMainClass.CreateMainMenuEntry;
var
  dwSMimeGrayed: DWord;
  sAddItem: String;
  iBeg, iEnd: Integer;
begin
  FInfoMenu := CreatePopupMenu;
  if FInfoMenu = 0 then Exit;
  AppendMenu(FInfoMenu, MF_STRING, IDM_INFOMENU + ord(miConfig),
	     PChar(PMPGPStrings[CONFIG_ITEM]));
  AppendMenu(FInfoMenu, MF_STRING, IDM_INFOMENU + ord(miManual),
	     PChar(PMPGPStrings[HELP_ITEM]));
  AppendMenu(FInfoMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FInfoMenu, MF_STRING, IDM_INFOMENU + ord(miAbout),
	     PChar(PMPGPStrings[ABOUT_ITEM]));
  FEditMenu := CreatePopupMenu;
  if FEditMenu = 0 then Exit;
  FMainMenu := CreatePopupMenu;
  if FMainMenu = 0 then Exit;
  AppendMenu(FMainMenu, MF_POPUP, FInfoMenu,
	     PChar(FRONTEND_NAME));
  AppendMenu(FMainMenu, MF_POPUP, FEditMenu,
	     PChar(DelAmp(PMPGPStrings[EDIT_AUTOLIST_ITEM])));
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miWipePassphrase),
	     PChar(PMPGPStrings[WIPE_PASSPHRASE_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FMainMenu, MF_STRING or (MF_CHECKED * ord(UsePGPMime = true)), IDM_MAINMENU + ord(miUsePGPMime),
	     PChar(PMPGPStrings[USE_PGP_MIME_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or (MF_CHECKED * ord(AppendPGPExtension = true)), IDM_MAINMENU + ord(miAppendPGPExtension),
	     PChar(PMPGPStrings[APPEND_PGP_EXTENSION_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or (MF_CHECKED * ord(AutoEncrypt = true)), IDM_MAINMENU + ord(miAutoEncrypt),
	     PChar(PMPGPStrings[AUTO_ENCRYPT_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miEncrypt),
	     PChar(PMPGPStrings[ENCRYPT_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miSign),
	     PChar(PMPGPStrings[SIGN_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miEncryptSign),
	     PChar(PMPGPStrings[ENCRYPT_SIGN_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miMimeConvertMessage),
	     PChar(PMPGPStrings[MIME_CONVERT_MESSAGE_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miDecryptMessage),
	     PChar(PMPGPStrings[DECRYPT_MESSAGE_ITEM]));
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miDecryptFolder),
	     PChar(PMPGPStrings[DECRYPT_FOLDER_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FMainMenu, MF_STRING or MF_GRAYED, IDM_MAINMENU + ord(miVerifySignature),
	     PChar(PMPGPStrings[VERIFY_SIGNATURE_ITEM]));
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miShowFingerprint),
	     PChar(PMPGPStrings[SHOW_FINGERPRINT_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  dwSMimeGrayed := MF_GRAYED * ord(PGPSMimeCapability = smDisabled);
  AppendMenu(FMainMenu, MF_STRING or dwSMimeGrayed, IDM_MAINMENU + ord(miImportSystemCertificates),
	     PChar(PMPGPStrings[IMPORT_SYSTEM_CERTIFICATES_ITEM]));
  sAddItem := PMPGPStrings[ADD_KEYS_OR_CERTIFICATES_ITEM];
  if dwSMimeGrayed <> 0 then begin
    iBeg := LastCharPos(SL, sAddItem);
    iEnd := NextCharPos(SP, sAddItem, succ(iBeg));
    if iEnd > iBeg then
      Delete(sAddItem, iBeg, iEnd - iBeg)
    else Delete(sAddItem, iBeg, MAXINT);
  end;
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miAddKeysOrCertificates),
	     PChar(sAddItem));
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miDeleteKeys),
	     PChar(PMPGPStrings[DELETE_KEYS_ITEM]));
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miExtractKeys),
	     PChar(PMPGPStrings[EXTRACT_KEYS_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or dwSMimeGrayed, IDM_MAINMENU + ord(miDeleteCertificate),
	     PChar(PMPGPStrings[DELETE_CERTIFICATE_ITEM]));
  AppendMenu(FMainMenu, MF_STRING or dwSMimeGrayed, IDM_MAINMENU + ord(miExtractCertificate),
	     PChar(PMPGPStrings[EXTRACT_CERTIFICATE_ITEM]));
  AppendMenu(FMainMenu, MF_SEPARATOR, 0, nil);
  AppendMenu(FMainMenu, MF_STRING, IDM_MAINMENU + ord(miLaunchPGP),
	     PChar(PMPGPStrings[LAUNCH_PGP_ITEM]));
end;

procedure TMainClass.RemoveMainMenuEntry;
var
  iIndex, iCount: Integer;
  hFrameMenu: THandle;
begin
  for iIndex := pred(MN_FRAME) to MN_DLM do begin
    if iIndex < MN_FRAME then
      hFrameMenu := GetMenu(FPMFrameWindow)
    else hFrameMenu := PMInterface.Menus[iIndex];
    RemoveMenu(hFrameMenu, FMainMenu, MF_BYCOMMAND);
    iCount := pred(GetMenuItemCount(hFrameMenu));
    while iCount >= 0 do begin
      if StrStart(DelAmp(PMPGPStrings[WINDOW_ITEM]), DelAmp(MenuString(hFrameMenu, iCount, MF_BYPOSITION))) then begin
	RemoveMenu(GetSubMenu(hFrameMenu, iCount), IDM_MAINMENU + ord(miWindow), MF_BYCOMMAND);
	Break;
      end;
      dec(iCount);
    end;
  end;
  DestroyMenu(FEditMenu);
  DestroyMenu(FInfoMenu);
  DestroyMenu(FMainMenu);
  DrawMenuBar(FPMFrameWindow);
end;

procedure TMainClass.SwitchMainMenuItems;
var
  dwItemFlags: DWord;
begin
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miWipePassphrase),
		 MF_BYCOMMAND or (MF_GRAYED * ord(PGPInterface.CacheEmpty = true)));
  dwItemFlags := MF_BYCOMMAND or (MF_GRAYED * ord((FindReader = 0)));
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miDecryptMessage), dwItemFlags);
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miVerifySignature), dwItemFlags);
  dwItemFlags := MF_BYCOMMAND or (MF_GRAYED * ord(FindEditorWindow(0) = 0));
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miEncrypt), dwItemFlags);
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miSign), dwItemFlags);
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miEncryptSign), dwItemFlags);
  if PGPSMimeCapability > smDisabled then begin
    dwItemFlags := MF_BYCOMMAND or (MF_GRAYED * ord(not PGPInterface.SystemHasPrivateCerts));
    EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miImportSystemCertificates), dwItemFlags);
  end;
  AddEditMenuEntries;
  InsertAddressBookMenu;
  DrawMenuBar(FPMFrameWindow);
end;

procedure TMainClass.SwitchMainMenuEntry(FrameMenu: THandle);
var
  hActiveMDIChild: THandle;
  iCount: Integer;
begin
  hActiveMDIChild := GetActiveMDIChild;
  if FrameMenu = 0 then FrameMenu := GetMenu(FPMFrameWindow);
  for iCount := GetMenuItemCount(FrameMenu) downto 0 do begin
    if MenuString(FrameMenu, iCount, MF_BYPOSITION) = FRONTEND_ITEM then begin
      if hActiveMDIChild <> GetActiveMDIChild then PostMessage(FMDIClientWindow, WM_MDIACTIVATE, hActiveMDIChild, 0);
      EnableMenuItem(FrameMenu, iCount, MF_BYPOSITION or (MF_GRAYED * ord(FPMPGPBusy = true)));
      DrawMenuBar(FPMFrameWindow);
      Exit;
    end;
  end;
end;

procedure TMainClass.InsertMainMenuEntry(FrameMenu, WindowMenu: THandle);
var
  iCount, iIndex: Integer;
begin
  if FrameMenu <> 0 then begin
    if FMainMenu = 0 then CreateMainMenuEntry;
    iCount := pred(GetMenuItemCount(FrameMenu));
    if iCount > 0 then begin
      for iIndex := pred(iCount) downto 0 do if GetSubMenu(FrameMenu, iIndex) = FMainMenu then Exit;
      InsertMenu(FrameMenu, pred(iCount), MF_POPUP or MF_BYPOSITION, FMainMenu, FRONTEND_ITEM);
      if WindowMenu = 0 then begin
	while iCount >= 0 do begin
	  if StrStart(DelAmp(PMPGPStrings[WINDOW_ITEM]), DelAmp(MenuString(FrameMenu, iCount, MF_BYPOSITION))) then begin
	    WindowMenu := GetSubMenu(FrameMenu, iCount);
	    Break;
	  end
	  else dec(iCount);
	end;
      end;
      if WindowMenu <> 0 then begin
	AppendMenu(WindowMenu, MF_STRING or MF_GRAYED * ord(FPMPGPBusy = true),
		   IDM_MAINMENU + ord(miWindow), FRONTEND_ITEM);
      end;
    end;
  end;
end;

// Popup menus

function TMainClass.InsertAddressBookMenu: Longbool;
var
  iIndex: Integer;
begin
  Result := false;
  if UseAutoList then begin
    if ALInterface.AddressBookList = nil then ALInterface.AddressBookListCreate(PMInterface.AddressBooksPathList);
    if ALInterface.AddressBookList <> nil then with ALInterface.AddressBookList do begin
      try
	if Count > 0 then begin
	  FAddressBookMenu := CreatePopupMenu;
	  if FAddressBookMenu <> 0 then begin
	    for iIndex := 0 to pred(Count) do begin
	      AppendMenu(FAddressBookMenu, MF_STRING, IDM_ADDRESS_BOOKS + iIndex, PChar(QU + Names[iIndex] + QU));
	    end;
	    AppendMenu(FAddressBookMenu, MF_SEPARATOR, 0, nil);
	    AppendMenu(FAddressBookMenu, MF_STRING, IDM_ADDRESS_BOOKS + Count, PChar(PMPGPStrings[ADD_ALL_BOOKS_ITEM]));
	    InsertMenu(FEditMenu, 0, MF_BYPOSITION or MF_POPUP, FAddressBookMenu, PChar(PMPGPStrings[ADD_ADDRESS_BOOKS_ITEM]));
	    Result := true;
	  end;
	end;
      finally
	if not Result then ALInterface.AddressBookListFree;
      end;
    end;
  end;
end;

procedure TMainClass.RemoveAddressBookMenu(Window: THandle);
var
  iIndex: Integer;
begin
  ProcessMessages(0);
  if not FAddressBookSelected then ALInterface.AddressBookListFree;
  if FAddressBookMenu <> 0 then begin
    for iIndex := 0 to pred(GetMenuItemCount(FAddressBookMenu)) do begin
      RemoveMenu(FAddressBookMenu, iIndex, MF_BYPOSITION);
    end;
    DeleteMenu(FEditMenu, 0, MF_BYPOSITION);
    FAddressBookMenu := 0;
    DrawMenuBar(Window);
  end;
end;

procedure TMainClass.EnableUsingAutoList(Confirm: Longbool);
begin
  if not Confirm or MessageDialogs.Confirm(PMPGPStrings[USE_AUTOLIST_CONFIRMATION]) then UseAutoList := true;
end;

procedure TMainClass.ShowAutoListMenu;
begin
  if UseAutoList then begin
    AddEditMenuEntries;
    InsertAddressBookMenu;
    with GetCursorPos do TrackPopupMenu(FEditMenu, TPM_LEFTALIGN or TPM_LEFTBUTTON, X, Y, 0, FHiddenWindow, nil);
  end
  else EnableUsingAutoList(true);
end;

procedure TMainClass.ShowEncryptSignMenu;
begin
  if FindEditorWindow(0) <> 0 then begin
    FEncryptSignMenu := CreatePopupMenu;
    if FEncryptSignMenu = 0 then Exit;
    try
      if AppendMenu(FEncryptSignMenu, MF_STRING, IDM_MAINMENU + ord(miEncrypt), PChar(PMPGPStrings[ENCRYPT_ITEM]))
      and AppendMenu(FEncryptSignMenu, MF_STRING, IDM_MAINMENU + ord(miSign), PChar(PMPGPStrings[SIGN_ITEM]))
      and AppendMenu(FEncryptSignMenu, MF_STRING, IDM_MAINMENU + ord(miEncryptSign), PChar(PMPGPStrings[ENCRYPT_SIGN_ITEM]))
      and AppendMenu(FEncryptSignMenu, MF_SEPARATOR, 0, nil)
      and AppendMenu(FEncryptSignMenu, MF_STRING or (MF_CHECKED * ord(UsePGPMime = true)),
		     IDM_MAINMENU + ord(miUsePGPMime), PChar(PMPGPStrings[USE_PGP_MIME_ITEM]))
      and AppendMenu(FEncryptSignMenu, MF_STRING or (MF_CHECKED * ord(AppendPGPExtension = true)),
		     IDM_MAINMENU + ord(miAppendPGPExtension), PChar(PMPGPStrings[APPEND_PGP_EXTENSION_ITEM]))
      and AppendMenu(FEncryptSignMenu, MF_STRING or (MF_CHECKED * ord(AutoEncrypt = true)),
		     IDM_MAINMENU + ord(miAutoEncrypt), PChar(PMPGPStrings[AUTO_ENCRYPT_ITEM])) then begin
	with GetCursorPos do TrackPopupMenu(FEncryptSignMenu, TPM_LEFTALIGN or TPM_LEFTBUTTON, X, Y, 0, FPMFrameWindow, nil);
      end;
    finally
      DestroyMenu(FEncryptSignMenu);
      FEncryptSignMenu := 0;
      ProcessMessages(0);
    end;
  end;
end;

procedure TMainClass.ShowIDMenu(Window: THandle; const ID: String; IsHexID, KeyTrigger: Longbool);
var
  hIDMenu: THandle;
  ptOrigin: TPoint;
begin
  hIDMenu := CreatePopupMenu;
  try
    if (IsHexID or (AutoEncrypt and AppendMenu(hIDMenu, MF_STRING, IDM_ID_POPUP + ord(miAddAddress),
					       PChar(PMPGPStrings[ADD_ADDRESS_ITEM]))))
    and AppendMenu(hIDMenu, MF_STRING, IDM_ID_POPUP + ord(miKeyServer),
		   PChar(PMPGPStrings[KEY_SERVER_ITEM]))
    and AppendMenu(hIDMenu, MF_STRING, IDM_ID_POPUP + ord(miKeyIDCopy),
		   PChar(PMPGPStrings[KEYID_COPY_ITEM])) then begin
      FIDMenuData := ID;
      if KeyTrigger then with TERInterface do begin
	ptOrigin := GetCaretScreenPos(Window);
	inc(ptOrigin.Y, GetCurrentLineHeight(Window));
      end
      else ptOrigin := GetCursorPos;
      with ptOrigin do TrackPopupMenu(hIDMenu, TPM_LEFTALIGN or TPM_LEFTBUTTON, X, Y, 0, FHiddenWindow, nil);
    end;
  finally
    DestroyMenu(hIDMenu);
  end;
end;

// Menu infos

procedure TMainClass.ShowMenuInfo(Menu: THandle; Item: Integer);
var
  sInfo: String;
begin
  if (Menu <> 0) and (Item > IDM_MAINMENU) then begin
    if Item < IDM_INFOMENU then begin
      case TMenuItems(Item - IDM_MAINMENU) of
	miWindow:			sInfo := PMPGPStrings[BRING_TO_FOREGROUND_INFO];
	miWipePassphrase:		sInfo := PMPGPStrings[WIPE_PASSPHRASE_INFO];
	miUsePGPMime:			sInfo := PMPGPStrings[USE_PGP_MIME_INFO];
	miAppendPGPExtension:		sInfo := PMPGPStrings[APPEND_PGP_EXTENSION_INFO];
	miAutoEncrypt:			sInfo := PMPGPStrings[AUTO_ENCRYPT_INFO];
	miEncrypt:			sInfo := PMPGPStrings[ENCRYPT_INFO];
	miSign:				sInfo := PMPGPStrings[SIGN_INFO];
	miEncryptSign:			sInfo := PMPGPStrings[ENCRYPT_SIGN_INFO];
	miMimeConvertMessage: begin
	  sInfo := RemoveSMimeString(PMPGPStrings[MIME_CONVERT_MESSAGE_INFO], PGPSMimeCapability = smDisabled);
	end;
	miDecryptMessage:		sInfo := PMPGPStrings[DECRYPT_MESSAGE_INFO];
	miDecryptFolder:		sInfo := PMPGPStrings[DECRYPT_FOLDER_INFO];
	miVerifySignature:		sInfo := PMPGPStrings[VERIFY_SIGNATURE_INFO];
	miShowFingerprint:		sInfo := PMPGPStrings[SHOW_FINGERPRINT_INFO];
	miImportSystemCertificates:	sInfo := PMPGPStrings[IMPORT_SYSTEM_CERTIFICATES_INFO];
	miAddKeysOrCertificates:	sInfo := PMPGPStrings[ADD_KEYS_OR_CERTIFICATES_INFO];
	miDeleteKeys:			sInfo := PMPGPStrings[DELETE_KEYS_INFO];
	miExtractKeys:			sInfo := PMPGPStrings[EXTRACT_KEYS_INFO];
	miDeleteCertificate:		sInfo := PMPGPStrings[DELETE_CERTIFICATE_INFO];
	miExtractCertificate:		sInfo := PMPGPStrings[EXTRACT_CERTIFICATE_INFO];
	miLaunchPGP:			sInfo := PMPGPStrings[LAUNCH_PGP_INFO];
      end
    end
    else if Item < IDM_EDITMENU then begin
      case TMenuItems(Item - IDM_INFOMENU) of
	miConfig:			sInfo := PMPGPStrings[CONFIG_INFO];
	miManual:			sInfo := PMPGPStrings[HELP_INFO];
	miAbout:			sInfo := PMPGPStrings[ABOUT_INFO];
      end;
    end
    else if Item < IDM_ADDRESS_BOOKS then begin
      case TMenuItems(Item - IDM_EDITMENU) of
	miEnableAutoList:		sInfo := PMPGPStrings[ENABLE_AUTOLIST_INFO];
	miAddRecipients:		sInfo := PMPGPStrings[ADD_RECIPIENTS_INFO];
	miAddRingAddresses:		sInfo := PMPGPStrings[ADD_RING_ADDRESSES_INFO];
	miSetAddressOptions:		sInfo := PMPGPStrings[SET_ADRRESS_OPTIONS_INFO];
	miUpdateAutoList:		sInfo := PMPGPStrings[UPDATE_AUTOLIST_INFO];
	miEditAutoList:			sInfo := PMPGPStrings[EDIT_AUTOLIST_INFO];
      end
    end
    else if ALInterface.AddressBookList <> nil then with ALInterface.AddressBookList do begin
      dec(Item, IDM_ADDRESS_BOOKS);
      if Item < Count then
	sInfo := PMPGPStrings[ADD_BOOK_ADDRESSES_INFO] + QU + Names[Item] + QU
      else if Item = Count then sInfo := PMPGPStrings[ADD_ALL_BOOKS_INFO];
    end;
    PMInterface.MenuInfo := sInfo;
  end
  else PMInterface.MenuInfo := SP;
end;

// Menu item handlers

procedure TMainClass.IDMenuHandler(MenuID: Integer);
begin
  case TMenuItems(MenuID) of
    miAddAddress: AddAddressesToAutoList(FIDMenuData);
    miKeyServer: GetKeyFromServer(FIDMenuData);
    miKeyIDCopy: WriteClipboardData(FIDMenuData, CF_TEXT, FPMFrameWindow);
  else
    Exit;
  end;
  FIDMenuData := E;
end;

procedure TMainClass.MainMenuHandler(MenuID: Integer);
var
  hMDIEditor: THandle;
begin
  if not FPMPGPBusy then begin
    case TMenuItems(MenuID) of
      miWindow: ShowAboutDialog;
      miWipePassphrase: PGPInterface.WipeCache;
      miUsePGPMime: SwitchUsePGPMime;
      miAppendPGPExtension: SwitchAppendPGPExtension;
      miAutoEncrypt: SwitchAutoEncrypt(not AutoEncrypt and Confirm(PMPGPStrings[AUTO_SIGN_CONFIRMATION]));
      miEncrypt..miEncryptSign: with PMInterface do begin
	hMDIEditor := CheckMDIEditor(0);
	if hMDIEditor <> 0 then with GetMDIEditorDataPtr(hMDIEditor)^ do begin
	  case TMenuItems(MenuID) of
	    miEncrypt: EncodeFlag := efEncrypt;
	    miSign: EncodeFlag := efSign;
	    miEncryptSign: EncodeFlag := efEncryptSign;
	  end;
	  PMInterface.SendMailMessage(true, false);
	end;
      end;
      miMimeConvertMessage: MimeConvertMessage;
      miDecryptMessage: DecryptMessage;
      miDecryptFolder: DecryptFolder;
      miVerifySignature: VerifySignature(E);
      miShowFingerprint: ShowFingerprint;
      miImportSystemCertificates: ImportCerts;
      miAddKeysOrCertificates: ManageKeys(E, false);
      miDeleteKeys: DeleteKeys;
      miExtractKeys: ExtractKeys;
      miDeleteCertificate: DeleteCert;
      miExtractCertificate: ExtractCert;
      miLaunchPGP: LaunchPGP;
    end;
  end;
end;

procedure TMainClass.InfoMenuHandler(MenuID: Integer);
begin
  if not FPMPGPBusy then begin
    case TMenuItems(MenuID) of
      miConfig: if Configure then RestartPegasusMail(PMInterface.PMUser, FPMFrameWindow);
      miManual: ShowHelpManual(E);
      miAbout: ShowAboutDialog;
    end;
  end;
end;

procedure TMainClass.EditMenuHandler(MenuID: Integer);
begin
  if not FPMPGPBusy and UseAutoList then begin
    case TMenuItems(MenuID) of
      miAddRecipients: AddAddressesToAutoList(E);
      miAddRingAddresses: AddRingAddresses;
      miSetAddressOptions: SetAddressOptions;
      miUpdateAutoList: UpdateAutoList;
      miEditAutoList: EditAutoList;
    end;
  end
  else EnableUsingAutoList(false);
end;

procedure TMainClass.AddressBookHandler(MenuID: Integer);
begin
  if not FPMPGPBusy then with ALInterface do begin
    if AddressBookList <> nil then begin
      try
	AddressBookList.AddBookIndex := MenuID;
	SetDefaultFlags(AutoEncrypt, AutoSign);
	AddAddressBook(Self, ALInterface, PGPSMimeCapability = smReadWrite);
      finally
	FAddressBookSelected := false;
	AddressBookListFree;
      end;
    end;
  end;
end;

procedure TMainClass.ShowMenuHelp(Item, ID: Integer);
begin
  if not IsMenu(ID) and (ID > IDM_MAINMENU) then begin
    if ID < IDM_EDITMENU then begin
      SendMessage(FPMFrameWindow, WM_CANCELMODE, 0, 0);
      PMPGPHelp.Topic(Item, ord(IDH_MAINMENU));
    end
    else begin
      SendMessage(FHiddenWindow, WM_CANCELMODE, 0, 0);
      PMPGPHelp.Topic(Item, ord(IDH_AUTOLIST));
    end;
  end;
end;

// Window utilities

procedure TMainClass.UpdateMDIWindowMenu(Menu: THandle);
var
  iCount, iIndex: Integer;
  hItem: THandle;
begin
  iCount := pred(GetMenuItemCount(Menu));
  if FMDIWindows <> nil then begin
    for iIndex := iCount downto 7 do DeleteMenu(Menu, iIndex, MF_BYPOSITION);
    AppendMenu(Menu, MF_STRING or MF_GRAYED * ord(FPMPGPBusy = true), IDM_MAINMENU + ord(miWindow), FRONTEND_ITEM);
    AppendMenu(Menu, MF_SEPARATOR, 0, nil);
    with FMDIWindows do begin
      EnumChildWindows(FMDIClientWindow, @ListMDIWindows, LPARAM(FMDIWindows));
      for iIndex := 0 to pred(Count) do begin
	hItem := THandle(Items[iIndex]);
	AppendMenu(Menu, MF_STRING or (MF_CHECKED * ord(hItem = GetActiveMDIChild)),
		   IDM_MDIWINDOWS + iIndex, PChar(AM + IntToStr(succ(iIndex)) + SP + WndString(hItem)));
      end;
    end;
  end;
end;

function TMainClass.GetActiveMDIChild: THandle;
begin
  Result := PMInterface.Windows[wMDIActiveChild];
end;

function TMainClass.CloseReaderWindows: Longbool;
var
  lMDIReaders: TList;
  iOldCount: Integer;
  hMDIReader: THandle;
begin
  Result := false;
  lMDIReaders := TList.Create;
  if (lMDIReaders <> nil) and (FindAllChildren(FMDIClientWindow, PM_CLASS_MDI_READER, lMDIReaders) > 0) then begin
    with lMDIReaders do begin
      while Count > 0 do begin
	iOldCount := Count;
	hMDIReader := THandle(Items[0]);
	if PMInterface.CheckMDIReader(hMDIReader) <> 0 then begin
	  SendMessage(hMDIReader, WM_CLOSE, 0, 0);
	  if not IsWindow(hMDIReader) then Delete(0);
	end
	else Delete(0);
	if Count = iOldCount then Exit;
      end;
    end;
  end;
  Result := true;
end;

function TMainClass.CloseEditorWindows: Longbool;
var
  lMDIEditors: TList;
  iOldCount: Integer;
  hMDIEditor: THandle;
begin
  Result := false;
  lMDIEditors := TList.Create;
  if (lMDIEditors <> nil) and (FindAllChildren(FMDIClientWindow, PM_CLASS_MDI_EDITOR, lMDIEditors) > 0) then begin
    with lMDIEditors do begin
      while Count > 0 do begin
	iOldCount := Count;
	hMDIEditor := THandle(Items[0]);
	if PMInterface.CheckMDIEditor(hMDIEditor) <> 0 then begin
	  SendMessage(hMDIEditor, WM_CLOSE, 0, 0);
	  if not IsWindow(hMDIEditor) then Delete(0);
	end
	else Delete(0);
	if Count = iOldCount then Exit;
      end;
    end;
  end;
  Result := true;
end;

procedure TMainClass.CenterDialog(Dialog, Owner: THandle);
var
  hFocus: THandle;
  rDialog, rOwner: TRect;
begin
  if (Owner = 0) or not IsWindowVisible(Owner) then begin
    if (FPMFrameWindow <> 0) and IsWindowVisible(FPMFrameWindow) then
      Owner := FPMFrameWindow
    else Owner := GetDesktopWindow;
  end;
  if (Dialog <> GetDesktopWindow) and (Owner <> 0) then begin
    rOwner := GetWindowRect(Owner);
    with rOwner do begin
      dec(Right, pred(Left));
      dec(Bottom, pred(Top));
    end;
    rDialog := GetWindowRect(Dialog);
    hFocus := GetFocus;
    with rDialog do begin
      dec(Right, pred(Left));
      dec(Bottom, pred(Top));
      Left := rOwner.Left + (rOwner.Right - Right) div 2;
      Top := rOwner.Top + (rOwner.Bottom - Bottom) div 2;
      if Top < 0 then Top := 0;
      SetWindowPos(Dialog, HWND_TOP or HWND_NOTOPMOST, 0, 0, 0, 0, SWP_DRAWFRAME or SWP_NOMOVE or SWP_NOSIZE);
      if (hFocus <> Dialog) and not IsChild(Dialog, hFocus) then SetFocus(Dialog);
      MoveWindow(Dialog, Left, Top, Right, Bottom, false);
    end;
  end;
end;

procedure TMainClass.SendToBack(Dialog: THandle);
begin
  SetWindowPos(Dialog, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
end;

function TMainClass.FindReader: THandle;
begin
  Result := FindReaderWindow(0);
  if Result = 0 then Result := FindPreviewReader;
end;

function TMainClass.FindPreviewReader: THandle;
var
  hParent: THandle;
begin
  Result := 0;
  with PMInterface do begin
    hParent := CheckMDIFolman(0);
    if hParent <> 0 then Result := BearOrTer(hParent, false);
  end;
end;

function TMainClass.FindReaderWindow(Parent: THandle): THandle;
begin
  Result := 0;
  with PMInterface do begin
    Parent := CheckMDIReader(Parent);
    if Parent <> 0 then Result := BearOrTer(Parent, PMInterface.HasNewReader);
  end;
end;

function TMainClass.FindHeaderWindow(Parent: THandle): THandle;
begin
  Result := 0;
  with PMInterface do begin
    Parent := CheckMDIReader(Parent);
    if Parent <> 0 then Result := BearOrTer(Parent, false);
  end;
end;

function TMainClass.FindEditorWindow(Parent: THandle): THandle;
begin
  Result := 0;
  with PMInterface do begin
    if Parent = 0 then Parent := CheckMDIEditor(Parent);
    if Parent <> 0 then Result := BearOrTer(Parent, false);
  end;
end;

function TMainClass.FindStatusBarChild(const Text: String): THandle;
begin
  if Text = E then begin
    if PMInterface.PMVersionNumber > 4520 then with PMInterface do begin
      StatusBarLongInfo := HEX_MARKER;
      try
	Result := FindFirstChild(FPMStatusBarWindow, PM_CLASS_DH_STATIC, HEX_MARKER, 0, false);
      finally
	StatusBarLongInfo := E;
      end;
    end
    else Result := 0;
  end
  else Result := FindFirstChild(FPMStatusBarWindow, PM_CLASS_DH_STATIC, Text, 0, false);
end;

// Popup menus

function TMainClass.AddressOrHexIDPopup(Window: THandle; KeyTrigger: Longbool): Longbool;
var
  sAnyID: String;
  fHexID: Longbool;
begin
  Result := false;
  with TERInterface do begin
    if IsTerWindow(Window) then begin
      if GetKeyState(VK_CONTROL) < 0 then begin
	sAnyID := GetAddressOrHexID(Window, fHexID);
	Result := (sAnyID <> E);
      end
      else if not KeyTrigger then begin
	sAnyID := GetAnsiLineTextAtCursor(Window);
	Result := UnknownHexIDPopup(Window, sAnyID);
	Exit;
      end;
    end;
  end;
  if Result then begin
    if fHexID then
      ShowIDMenu(Window, sAnyID, true, KeyTrigger)
    else ShowIDMenu(Window, sAnyID, false, KeyTrigger);
  end;
end;

// Decode status handling

function TMainClass.GetStatusIconIndex(Status: Integer): Integer;
begin
  Result := ord(Status and IDW_VERIFY_ENCRYPTED <> 0) * IDW_VERIFY_ICON_COUNT + (Status and $FFFF);
end;

procedure TMainClass.ShowVerificationInfo;
begin
  if FindPreviewReader <> 0 then with FSignatureData do if Text <> E then begin
    FCurrentStatusIcon := GetStatusIconIndex(Flags);
    PMInterface.StatusBarLongInfo := Text;
    CreateStatusToolTip(Text);
    DrawStatusBarIcon(0);
  end;
end;

procedure TMainClass.ClearVerificationInfo;
begin
  if PMInterface.IsMessageHandlingMDI then FillChar(FDecodeOptions, SizeOf(TDecodeOptions), 0);
  FCurrentStatusIcon := IDW_VERIFY_NOTHING;
  ClearSignatureData(FSignatureData);
  FMimeMsgTree.Clear;
  DestroyStatusToolTip;
  if GetHexStatusLabel(0, E) <> 0 then begin
    PMInterface.StatusBarLongInfo := E;
    RedrawWindow(FPMStatusBarWindow, nil, 0, RDW_UPDATENOW);
  end;
end;

// StatusBar ToolTip

function TMainClass.GetStatusBarFont: THandle;
var
  lf: TLogFont;
begin
  if FPMStatusBarChildProc <> NIL then begin
    Result := GetStockObject(DEFAULT_GUI_FONT);
    if (Result <> 0) and (GetObject(Result, SizeOf(TLogFont), @lf) > 0) then begin
      StrPLCopy(lf.lfFaceName, PMInterface.DlgFontInfo.Name, LF_FACESIZE);
      Result := CreateFontIndirect(lf);
      Exit;
    end;
  end
  else begin
    Result := SendMessage(FPMStatusBarWindow, WM_GETFONT, 0, 0);
    if (Result <> 0) and (GetObject(Result, SizeOf(TLogFont), @lf) > 0) then begin
      Result := CreateFontIndirect(lf);
      Exit;
    end;
  end;
  Result := 0;
end;

function TMainClass.GetHexStatusLabel(Window: THandle; const ClassName: String): THandle;
begin
  if (Window = 0) or (Window = FPMStatusBarWindow) then begin
    if FPMStatusBarChildProc <> NIL then
      Result := FindStatusBarChild(HEX_MARKER)
    else Result := FindStatic(FPMStatusBarWindow, HEX_MARKER);
  end
  else if (ClassName = CLASS_STATIC) and (FirstStrPos(HEX_MARKER, WndString(Window)) > 0) then
    Result := Window
  else Result := 0;
end;

function TMainClass.GetStatusToolTipText(const Text: String): String;
var
  hLabel, hLabelDC, hLabelFont: THandle;
  sLabel: String;
  szLabel: TSize;
  rLabel: TRect;
begin
  hLabel := GetHexStatusLabel(0, E);
  if (hLabel <> 0) and IsWindowVisible(hLabel) then begin
    if Text = E then
      sLabel := WndString(hLabel)
    else sLabel := Text;
    sLabel := XChgStrAtPos(sLabel, SP, FirstStrPos(CRLF, sLabel), 2);
    hLabelDC := GetDC(hLabel);
    hLabelFont := 0;
    try
      hLabelFont := SelectObject(hLabelDC, FStatusBarFontHandle);
      GetTextExtentPoint32(hLabelDC, PChar(sLabel), Length(sLabel), szLabel);
    finally
      SelectObject(hLabelDC, hLabelFont);
      ReleaseDC(hLabel, hLabelDC);
    end;
    rLabel := GetWindowRect(hLabel);
    if szLabel.CX > (rLabel.Right - rLabel.Left) then
      Result := TrimLeft(sLabel)
    else Result := E;
  end
  else Result := E;
end;

function GetToolTipCount(const ToolTipHandle: THandle): Integer;
begin
  Result := SendMessage(ToolTipHandle, TTM_GETTOOLCOUNT, 0, 0);
end;

function IsToolTipText(const ToolTipHandle, ParentHandle: THandle; const Text: String): Longbool;
var
  iLen: Integer;
  sToolTip: String;
  tiStatus: TToolInfo;
begin
  iLen := Length(Text);
  SetString(sToolTip, nil, succ(iLen));
  FillChar(tiStatus, SizeOf(TToolInfo), 0);
  with tiStatus do begin
    cbSize := SizeOf(TToolInfo);
    uFlags := TTF_IDISHWND;
    hwnd := ParentHandle;
    uId := ParentHandle;
    lpszText := PChar(sToolTip);
  end;
  SendMessage(ToolTipHandle, TTM_GETTEXT, 0, Longint(@tiStatus));
  Result := CompareTxtMax(sToolTip, Text, iLen);
end;

procedure TMainClass.AddStatusToolTip(const ToolTip, Parent: THandle; const Text: String);
var
  lStatics: TList;
  rS1, rS2: TRect;
  tiStatus: TToolInfo;
begin
  lStatics := TList.Create;
  if lStatics <> nil then begin
    try
      if FindAllChildren(FPMStatusBarWindow, CLASS_STATIC, lStatics) > 2 then begin
	rS1 := GetWindowRect(THandle(lStatics[0]));
	ScreenToClient(FPMStatusBarWindow, rS1.TopLeft);
	rS2 := GetWindowRect(THandle(lStatics[2]));
	ScreenToClient(FPMStatusBarWindow, rS2.BottomRight);
	FillChar(tiStatus, SizeOf(TToolInfo), 0);
	with tiStatus do begin
	  cbSize := SizeOf(TToolInfo);
	  uFlags := TTF_SUBCLASS or TTF_TRANSPARENT;
	  hwnd := Parent;
	  hinst := hInstance;
	  if Parent <> FPMStatusBarWindow then begin
	    uId := Parent;
	    rect := GetClientRect(Parent);
	    uFlags := uFlags or TTF_IDISHWND;
	  end
	  else begin
	    uId := 0;
	    with rect do begin
	      TopLeft := rS1.TopLeft;
	      BottomRight := rS2.BottomRight;
	    end;
	    OffsetRect(rect, 2, 1);
	  end;
	  lpszText := PChar(GetStatusToolTipText(Text));
	  SendMessage(FStatusToolTip, TTM_ADDTOOL, 0, Longint(@tiStatus));
	  SendMessage(FStatusToolTip, TTM_SETMARGIN, 0, Longint(@TOOLRECT_EMPTY));
	end;
      end
      else begin
	DestroyWindow(FStatusToolTip);
	FStatusToolTip := 0;
      end;
    finally
      lStatics.Free;
    end;
  end;
end;

procedure TMainClass.CreateStatusToolTip(const Text: String);
var
  hParent: THandle;
begin
  if FPMStatusBarChildProc <> NIL then
    hParent := FindStatusBarChild(Text)
  else hParent := FPMStatusBarWindow;
  if (FStatusToolTip = 0) or (GetToolTipCount(FStatusToolTip) = 0) then begin
    DestroyStatusToolTip;
    FStatusToolTip := CreateWindow(TOOLTIPS_CLASS, nil, WS_POPUP or TTS_NOPREFIX or TTS_ALWAYSTIP,
				   0, 0, 0, 0, hParent, 0, hInstance, nil);
    if FStatusToolTip <> 0 then begin
      SendMessage(FStatusToolTip, WM_SETFONT, FStatusBarFontHandle, 0);
      SendMessage(FStatusToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, MAX_DELAYTIME);
      AddStatusToolTip(FStatusToolTip, hParent, Text);
    end;
  end
  else if IsToolTipText(FStatusToolTip, FindStatusBarChild(Text), Text) then begin
    AddStatusToolTip(FStatusToolTip, hParent, Text);
  end;
end;

procedure TMainClass.DestroyStatusToolTip;
begin
  if FStatusToolTip <> 0 then begin
    DestroyWindow(FStatusToolTip);
    FStatusToolTip := 0;
  end;
end;

// StatusBar time

procedure TMainClass.CreateTimeToolTip;
var
  tiTimer: TToolInfo;
begin
  with tiTimer do begin
    cbSize := SizeOf(TToolInfo);
    uFlags := TTF_SUBCLASS or TTF_TRANSPARENT;
    hwnd := FPMStatusBarWindow;
    rect := FTimeRect;
    hinst := hInstance;
    uId := 0;
    lpszText := PChar(Format(PMPGPStrings[SYNC_CLOCK_HINT], [TimeServerName]));
  end;
  if FTimeToolTip = 0 then begin
    FTimeToolTip := CreateWindow(TOOLTIPS_CLASS, nil, WS_POPUP or TTS_NOPREFIX or TTS_ALWAYSTIP,
				  0, 0, 0, 0, FPMStatusBarWindow, 0, hInstance, nil);
    if FTimeToolTip <> 0 then begin
      SendMessage(FTimeToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, 5000);
      SendMessage(FTimeToolTip, TTM_ADDTOOL, 0, Longint(@tiTimer));
      SendMessage(FTimeToolTip, WM_SETFONT, FStatusBarFontHandle, 0);
    end;
  end
  else begin
    SendMessage(FTimeToolTip, TTM_UPDATETIPTEXT, 0, Longint(@tiTimer));
    SendMessage(FTimeToolTip, TTM_NEWTOOLRECT, 0, Longint(@tiTimer));
  end;
end;

procedure TMainClass.DestroyTimeToolTip;
begin
  if FTimeToolTip <> 0 then begin
    DestroyWindow(FTimeToolTip);
    FTimeToolTip := 0;
  end;
end;

procedure DrawPolyPolyLine(DC: THandle; ColorIndex: Integer;
			   const LinePoints: Array of TPoint;
			   const PointCounts: Array of DWord);
type
  PPolyLine = ^TPolyLine;
  TPolyLine = Array[0..0] of TPoint;
  PPointCounts = ^TPointCounts;
  TPointCounts = Array[0..0] of DWord;
var
  hPen: THandle;
begin
  hPen := SelectObject(DC, CreatePen(PS_SOLID, 1, GetSysColor(ColorIndex)));
  try
    PolyPolyLine(DC, PPolyLine(@LinePoints)^, PPointCounts(@PointCounts)^, succ(High(PointCounts)));
  finally
    DeleteObject(SelectObject(DC, hPen));
  end;
end;

procedure DrawFrameRect(DC: THandle; ColorIndex: Integer; const Rect: TRect);
var
  hBrush: THandle;
begin
  hBrush := CreateSolidBrush(GetSysColor(ColorIndex));
  try
    FrameRect(DC, Rect, hBrush);
  finally
    DeleteObject(hBrush);
  end;
end;

procedure TMainClass.RefreshStatusBarTime;
begin
  FCurrentTime := E;
  ShowStatusBarTime(0);
end;

procedure TMainClass.ShowStatusBarTime(StatusBar: THandle);
const
  RIGHT_MARGIN = 6;
var
  sSize, sTime: String;
  psStatusBar: TPaintStruct;
  hStatusBarDC, hButtonBar: THandle;
  rStatusBar, rVarious, rText: TRect;
  iTestPos: Integer;
begin
  case ShowTime of
    1: begin
      sSize := FormatDateTime('ddddd, t', 0);
      sTime := FormatDateTime('ddddd, t', Now);
    end;
    2: begin
      sSize := FormatDateTime('t', 0);
      sTime := FormatDateTime('t', Time);
    end;
  end;
  if StatusBar = 0 then begin
    psStatusBar.hdc := 0;
    if sTime = FCurrentTime then Exit;
    hStatusBarDC := GetDC(FPMStatusBarWindow);
  end
  else hStatusBarDC := BeginPaint(StatusBar, psStatusBar);
  FCurrentTime := sTime;
  if hStatusBarDC <> 0 then begin
    try
      rStatusBar := GetClientRect(FPMStatusBarWindow);
      with rStatusBar do begin
	iTestPos := GetSystemMetrics(SM_CYSIZE) shr 1;
	hButtonBar := ChildWindowFromPoint(FPMStatusBarWindow, Point(0, Top + iTestPos));
	if hButtonBar <> FPMStatusBarWindow then begin
	  rVarious := GetClientRect(hButtonBar);
	  inc(Top, rVarious.Bottom + 3);
	end;
	dec(Right, GetSystemMetrics(SM_CXSIZE));
	hButtonBar := ChildWindowFromPoint(FPMStatusBarWindow, Point(Right, Bottom - iTestPos));
	if hButtonBar <> FPMStatusBarWindow then begin
	  rVarious := GetClientRect(hButtonBar);
	  dec(Right, rVarious.Right - 3);
	end;
      end;
      rVarious := GetWindowRect(FindStatic(FPMStatusBarWindow, E));
      with rVarious do begin
	ScreenToClient(FPMStatusBarWindow, TopLeft);
	ScreenToClient(FPMStatusBarWindow, BottomRight);
      end;
      rText := rVarious;
      DrawStatusText(hStatusBarDC, Pointer(sSize), false, true, rText);
      with rText do begin
	OffsetRect(rText, rStatusBar.Right - (Right - Left) - RIGHT_MARGIN, 0);
	Bottom := rVarious.Bottom;
	inc(Right, RIGHT_MARGIN);
      end;
      if ChildWindowFromPoint(FPMStatusBarWindow, rText.TopLeft) = FPMStatusBarWindow then begin
	FTimeRect := rText;
        CreateTimeToolTip;
	DrawEdge(hStatusBarDC, rText, EDGE_ETCHED, BF_RECT or BF_ADJUST);
	DrawStatusText(hStatusBarDC, Pointer(sTime), false, false, rText);
      end;
    finally
      if psStatusBar.hdc = 0 then
	ReleaseDC(FPMStatusBarWindow, hStatusBarDC)
      else EndPaint(StatusBar, psStatusBar);
    end;
  end;
end;

// Status icons

procedure TMainClass.DrawStatusBarIcon(StatusLabel: THandle);
var
  hLabelDC: THandle;
  psLabel: TPaintStruct;
  sLabel: String;
  rLabel: TRect;
  iIconWidth: Integer;
begin
  if StatusLabel = 0 then begin
    StatusLabel := GetHexStatusLabel(0, E);
    hLabelDC := GetDC(StatusLabel);
    psLabel.hdc := 0;
  end
  else hLabelDC := BeginPaint(StatusLabel, psLabel);
  if hLabelDC <> 0 then begin
    try
      sLabel := WndString(StatusLabel);
      rLabel := GetClientRect(StatusLabel);
      with rLabel do begin
	dec(Right, pred(Left));
	dec(Bottom, Top - 2);
	Left := -1 - ord(FPMStatusBarChildProc = NIL);
	Top := -2;
	iIconWidth := GetSystemMetrics(SM_CXSMICON);
	DrawIconEx(hLabelDC, Left, Top, LoadIcon(PChar(FCurrentStatusIcon)), iIconWidth, Bottom, 0, 0, DI_NORMAL);
	if Length(sLabel) > Length(HEX_MARKER) then begin
	  inc(Left, iIconWidth - ord(FPMStatusBarChildProc <> NIL));
	  dec(Bottom);
	  inc(Top, 2);
	  if StrStart(HEX_MARKER, sLabel) then
	    DrawStatusText(hLabelDC, @sLabel[Length(HEX_MARKER) + 2], true, false, rLabel)
	  else DrawStatusText(hLabelDC, @sLabel[1], true, false, rLabel);
	end;
      end;
      FStatusBarIconShowing := true;
    finally
      if psLabel.hdc = 0 then
	ReleaseDC(StatusLabel, hLabelDC)
      else EndPaint(StatusLabel, psLabel);
    end;
  end;
end;

procedure TMainClass.ClearMDIReaderIcons(MDIReader: THandle);
var
  prd: PMDIReaderData;
begin
  prd := GetMDIReaderDataPtr(MDIReader);
  if prd <> nil then with prd^ do begin
    BodyStatusIcon := 0;
    KeepBodyIcon := false;
    AttachmentStatusIcon := 0;
    SendMessage(MDIReader, WM_SETICON, 0, ReaderIcon);
  end;
end;

procedure TMainClass.DrawMDIReaderIcon(MDIReader: THandle; Active: Longbool);
var
  prd: PMDIReaderData;
  hStatusIcon: THandle;
begin
  if not FDecodeOptions.ShowAttachments then begin
    prd := GetMDIReaderDataPtr(MDIReader);
    if prd <> nil then with prd^ do begin
      with PMInterface do begin
	hStatusIcon := IDW_VERIFY_NOTHING;
	if IsBodyView(MDIReader) then
	  hStatusIcon := BodyStatusIcon
	else if IsAttachmentView(MDIReader) then
	  hStatusIcon := AttachmentStatusIcon
	else if Active and IsRawView(MDIReader) then SetFocus(TERInterface.FindTerWindow(MDIReader));
      end;
      if hStatusIcon > IDW_VERIFY_NOTHING then
	SendMessage(MDIReader, WM_SETICON, 0, hStatusIcon)
      else SendMessage(MDIReader, WM_SETICON, 0, ReaderIcon);
      if IsZoomed(MDIReader) then DrawMenuBar(FPMFrameWindow);
    end;
  end;
end;

procedure TMainClass.DrawStatusText(DC: THandle; Text: PChar; Indent, CalcRect: Longbool; var Rect: TRect);
var
  rClr: TRect;
  hdcSB, hFont: THandle;
  clrBackground: TColorRef;
  iCenter: Integer;
  gr: TGradientRect;
  atv: Array[0..1] of TTriVertex;
begin
  rClr := Rect;
  with rClr do begin
    ClientToScreen(WindowFromDC(DC), TopLeft);
    ClientToScreen(WindowFromDC(DC), BottomRight);
    ScreenToClient(FPMStatusBarWindow, TopLeft);
    ScreenToClient(FPMStatusBarWindow, BottomRight);
  end;
  hdcSB := GetDC(FPMStatusBarWindow);
  hFont := SelectObject(DC, FStatusBarFontHandle);
  with Rect do clrBackground := SetBkColor(DC, GetPixel(DC, Left, Top));
  try
    if WindowFromDC(DC) = FPMStatusBarWindow then
      iCenter := DT_CENTER
    else iCenter := 0;
    if not CalcRect and Assigned(GradientFill) then begin
      with gr do begin
	UpperLeft  := 0;
	LowerRight := 1;
      end;
      with rClr do clrBackground := GetPixel(hdcSB, 0, Top);
      with atv[0] do begin
	x := Rect.Left;
	y := Rect.Top;
	Red := GetRValue(clrBackground) shl 8;
	Green := GetGValue(clrBackground) shl 8;
	Blue := GetBValue(clrBackground) shl 8;
	Alpha := 0;
      end;
      with rClr do clrBackground := GetPixel(hdcSB, 0, Bottom);
      with atv[1] do begin
	x := Rect.Right;
	y := Rect.Bottom;
	Red := GetRValue(clrBackground) shl 8;
	Green := GetGValue(clrBackground) shl 8;
	Blue := GetBValue(clrBackground) shl 8;
	Alpha := 0;
      end;
      GradientFill(DC, @atv, 2, @gr, 1, GRADIENT_FILL_RECT_V);
      inc(Rect.Left, ord(Indent = true) + 1);
      SetBkMode(DC, TRANSPARENT);
    end;
    DrawText(DC, Text, -1, Rect, iCenter or DT_SINGLELINE or (DT_CALCRECT * ord(CalcRect = true)));
    if CalcRect then inc(Rect.Right, 2);
  finally
    SelectObject(DC, hFont);
    SetBkColor(DC, clrBackground);
    ReleaseDC(FPMStatusBarWindow, hdcSB);
  end;
end;

// CallWndProc handlers

procedure TMainClass.CallWndProcCreate(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  if ClassString(Window) = PM_CLASS_TER_PARENT then TextEditSubclass(Window);
end;

procedure TMainClass.CallWndProcDestroy(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  if Window = FEventWindow then
    SetEvent(FWaitEvent)
  else if WndString(Window) = PMInterface.DlgSearchMessage then FManageOrVerifyTriggered := false;
end;

procedure TMainClass.CallWndProcSetText(Window: THandle; wParam: WPARAM; lParam: LPARAM);

  function PosInParent(Window: THandle): TPoint;
  var
    rParent, rChild: TRect;
  begin
    if Windows.GetWindowRect(GetParent(Window), rParent) and Windows.GetWindowRect(Window, rChild) then begin
      Result := rChild.TopLeft;
      with Result do begin
	dec(X, rParent.Left);
	dec(Y, rParent.Top);
      end;
    end
    else with Result do begin
      X := 0;
      Y := 0;
    end;
  end;
begin
  if FStatusBarIconShowing and (ClassString(Window) = CLASS_STATIC) and (GetParent(Window) = FPMStatusBarWindow)
  and (PosInParent(Window).X < 100) and (FirstStrPos(HEX_MARKER, PChar(lParam)) = 0) then begin
    InvalidateRect(FPMStatusBarWindow, nil, true);
    FStatusBarIconShowing := false;
    DestroyStatusToolTip;
  end;
  if IsChild(FPMStatusBarWindow, Window) then RefreshStatusBarTime;
end;

procedure TMainClass.CallWndProcShowWindow(Window: THandle; wParam: WPARAM; lParam: LPARAM);
var
  sClassName, sWindowText: String;
  hChildWindow: THandle;
  icoPrint: TIcon;
begin
  if (wParam <> 0) and (lParam = 0) then begin
    if (GetWindowLong(Window, GWL_STYLE) and WS_CHILD) = 0 then begin
      sClassName := ClassString(Window);
      sWindowText := WndString(Window);
      if IsModuleWindow(Window) or StrStart(PGP_NAME, sClassName)
      or ((sClassName = CLASS_DIALOG) and StrStart(PGP_NAME, sWindowText)) then begin
	if (FWaitEvent <> 0) and (DWord(GetWindowLong(Window, GWL_HINSTANCE)) = GetModuleHandle(PGPSDK_UI)) then begin
	  hChildWindow := FindFirstChild(Window, E, PMPGPStrings[ENTER_PASSPHRASE_PROMPT] + PMPGPStrings[FOR_DECRYPTION],
					 0, false);
	  if hChildWindow <> 0 then begin
	    FEventWindow := Window;
	    hChildWindow := GetWindow(hChildWindow, GW_HWNDNEXT);
	    FDecPhraseEditProc := Ptr(SetWindowLong(hChildWindow, GWL_WNDPROC, Longint(@NewDecPhraseEditProc)));
	  end;
	end;
	CenterDialog(Window, GetWindow(Window, GW_OWNER));
      end
      else if sClassName = CLASS_DIALOG then begin
	if (FCurrentDialogOwner = GetDeskTopWindow) and (FindStatic(Window, PMPGPStrings[FIND_PM_PROMPT]) <> 0) then
	  CenterDialog(Window, GetWindow(Window, GW_OWNER))
	else if IsModuleWindow(FCurrentDialogOwner) then
	  CenterDialog(Window, FCurrentDialogOwner)
	else begin
	  if LoStrStart(PMInterface.DlgUserName, sWindowText) then
	    FPMChangeUserDlgProc := Ptr(SetWindowLong(Window, GWL_WNDPROC, Longint(@NewPMChangeUserDlgProc)))
	  else if LoStrStart(PMInterface.DlgPrintV4M, sWindowText) and FDecodeOptions.Printing then begin
	    FDecodeOptions.Printing := false;
	    if FDecodeOptions.PrintMessage then begin
	      hChildWindow := FindReader;
	      FDecodeOptions.PrintMessage := false;
	      if (hChildWindow <> 0) and not TERInterface.IsPrinting(hChildWindow) then begin
		PostMessage(Window, WM_CLOSE, 0, 0);
		with FMimeMsgTree do begin
		  AddIcons;
		  icoPrint := FirstTextSection.SectionIcon;
		  try
		    ShowMessageText(Self, FirstTextSection, icoPrint, true);
		  finally
		    icoPrint.Free;
		  end;
		end;
	      end;
	    end;
	  end
	  else if LoStrStart(PMPGPStrings[NO_DIGITAL_SIGNATURE], sWindowText)
	  or (FAutoSending and LoStrStart(PMInterface.DlgSpellCheck, sWindowText))
	  or (FHideNoAttachmentDialog and LoStrStart(PMInterface.NoAttachmentCaption, sWindowText)) then begin
	    Sleep(0);
	    FHideNoAttachmentDialog := false;
	    PostMessage(Window, WM_CLOSE, 0, 0);
	  end
	  else if (
		   FAutoSending and (
				     LoStrStart(PMInterface.DlgAttachmentCheck, sWindowText) or
				     LoStrStart(PMPGPStrings[MISSING_RECIPIENT_DIALOG], sWindowText) or
				     LoStrStart(PMPGPStrings[MISSING_SUBJECT_DIALOG], sWindowText)
				    )
		  )
	  or (FCleanupDone and LoStrStart(PMInterface.DlgQueueCheck, sWindowText)) then begin
	    Sleep(0);
	    ClickDlgBtn(Window, ID_OK);
	    ClickDlgBtn(Window, ID_YES);
	  end;
	  SetCursor(LoadCursor(0, IDC_ARROW));
	end;
      end
    end
    else if (ClassString(Window) = CLASS_DIALOG) and (FindStatic(Window, PMPGPStrings[ENCRYPT_METHOD]) <> 0) then begin
      hChildWindow := FindComboBox(Window);
      HooksCheckCommand(Window, CBN_SELCHANGE shl 16, hChildWindow);
      DeleteCBXIndex(hChildWindow, GetCBXItemIndex(hChildWindow, FRONTEND_X_PM));
    end;
  end;
end;

procedure TMainClass.CallWndProcInitMenuPopup(Window: THandle; wParam: WPARAM; lParam: LPARAM);
var
  iIndex: Integer;
begin
  if TERInterface.IsTerReader(WindowFromPoint(GetCursorPos))
  and TERInterface.IsHyperlinkAtCursor(PMPGPStrings[WORK_WITH_PARTS_HINT]) then begin
    for iIndex := 0 to pred(GetMenuItemCount(wParam)) do begin
      EnableMenuItem(wParam, iIndex, MF_BYPOSITION or MF_GRAYED);
    end;
  end;
end;

procedure TMainClass.CallWndProcCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  HooksCheckCommand(Window, wParam, lParam);
end;

procedure TMainClass.CallWndProcParentNotify(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  if LoWord(wParam) = WM_CREATE then begin
    if (Window = FMDIClientWindow) and (ClassString(lParam) = PM_CLASS_MDI_EDITOR) then
      MDIEditorSubclass(lParam)
    else if (GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_MDICHILD <> 0)
    and (FirstStrPos(ClassString(Window), PM_CLASS_MDI_READER) <> 0) then begin
      if (not PMInterface.HasNewReader or (ClassString(lParam) = PM_CLASS_READER_PANE))
      and (GetWindowLong(Window, GWL_WNDPROC) <> Longint(@NewMDIReaderProc)) then begin
	MDIReaderSubclass(Window);
      end;
    end;
  end;
end;

procedure TMainClass.CallWndProcMDIDestroy(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  if (Window = FMDIClientWindow) and (FMDIWindows <> nil) then begin
    with FMDIWindows do if IndexOf(Ptr(wParam)) <> -1 then Remove(Ptr(wParam));
  end;
end;

procedure TMainClass.CallWndProcMDISetMenu(Window: THandle; wParam: WPARAM; lParam: LPARAM);
begin
  if Window = FMDIClientWindow then begin
    InsertMainMenuEntry(wParam, lParam);
    SwitchMainMenuEntry(wParam);
  end;
end;

// GetMessageProc handlers

function TMainClass.GetMessageProcPaint(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
var
  sClassName: String;
  hMDIReader: THandle;
  prd: PMDIReaderData;
begin
  Result := WM_PAINT;
  sClassName := ClassString(Window);
  if TERInterface.IsBearClass(sClassName) then begin
    hMDIReader := PMInterface.CheckMDIReader(0);
    if IsChild(hMDIReader, Window) then begin
      prd := GetMDIReaderDataPtr(hMDIReader);
      if (prd <> nil) and (prd^.BodyStatusIcon = 0) then PostMessage(Window, PM_TERSETFOCUS, 0, 0);
    end;
  end
  else begin
    Window := GetHexStatusLabel(Window, sClassName);
    if Window <> 0 then DrawStatusBarIcon(Window);
  end;
end;

function TMainClass.GetMessageProcClose(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_CLOSE;
  if (FMoveMailBoxDialog <> 0) and (GetParent(FMoveMailBoxDialog) = Window) then FMoveMailBoxDialog := 0;
end;

function TMainClass.GetMessageProcKeyDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_KEYDOWN;
  if IsMDIWindowChild(FMDIClientWindow, Window) and (wParam <> VK_TAB) then ClearVerificationInfo;
  if (wParam = VK_APPS) and (GetKeyState(VK_CONTROL) < 0) then Result := WM_NULL;
end;

function TMainClass.GetMessageProcKeyUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_KEYUP;
  if (wParam = VK_APPS) and AddressOrHexIDPopup(Window, true) then Result := WM_NULL;
end;

function TMainClass.GetMessageProcSysKeyDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_SYSKEYDOWN;
  if TERInterface.IsBearWindow(Window) then begin
    PostMessage(FPMFrameWindow, Result, wParam, lParam);
    Result := WM_NULL;
  end
  else if (wParam = VK_F10) and (GetKeyState(VK_CONTROL) < 0) then Result := WM_NULL;
end;

function TMainClass.GetMessageProcSysKeyUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_SYSKEYUP;
  if (wParam = VK_F10) and AddressOrHexIDPopup(Window, true) then Result := WM_NULL;
end;

function TMainClass.GetMessageProcCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_COMMAND;
  HooksCheckCommand(Window, wParam, lParam);
end;

function TMainClass.GetMessageProcMouseMove(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_MOUSEMOVE;
  if FPMStatusBarChildProc = NIL then begin
    if (FStatusToolTip <> 0) and (Window <> FStatusToolTip) then begin
      if Window = FPMStatusBarWindow then begin
	if Window <> FPreviousChild then SendMessage(FStatusToolTip, TTM_ACTIVATE, 1, 0);
      end
      else SendMessage(FStatusToolTip, TTM_ACTIVATE, 0, 0);
    end;
  end;
  FPreviousChild := WindowFromPoint(GetCursorPos);
end;

function TMainClass.GetMessageProcLButtonDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  if IsMDIWindowChild(FMDIClientWindow, Window) then ClearVerificationInfo;
  TERInterface.SetHyperlinkFocus(Window);
  Result := WM_LBUTTONDOWN;
end;

function TMainClass.GetMessageProcLButtonUp(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  TERInterface.SetHyperlinkFocus(Window);
  Result := WM_LBUTTONUP;
end;

function TMainClass.GetMessageProcLButtonDblClk(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := WM_LBUTTONDBLCLK;
end;

function TMainClass.GetMessageProcRButtonDown(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
var
  hCurrent: THandle;
begin
  Result := WM_RBUTTONDOWN;
  hCurrent := WindowFromPoint(GetCursorPos);
  if WndString(hCurrent) = PMInterface.ButtonTitles[WM_ENCRYPT_SIGN] then
    ShowEncryptSignMenu
  else if (Window = FStatusToolTip) or (Window = FPMStatusBarWindow) or IsChild(FPMStatusBarWindow, Window) then begin
    UnknownHexIDPopup(Window, E);
    PMInterface.ResetStatusBar;
  end
  else if AddressOrHexIDPopup(Window, false) then Result := WM_NULL;
end;

function TMainClass.GetMessageProcPMTerSetFocus(Window: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
var
  prd: PMDIReaderData;
  mtEdit: TMessageType;
  hMDIChild, hPreview, hReader, hEditor, hStatusIcon: THandle;
begin
  prd := nil;
  Result := PM_TERSETFOCUS;
  hMDIChild := GetActiveMDIChild;
  hPreview := FindPreviewReader;
  hReader := FindReaderWindow(hMDIChild);
  if (Window = hReader) or (hReader = 0) and IsWindowVisible(Window) then prd := GetMDIReaderDataPtr(hMDIChild);
  if FMimeMsgTree.ContainsData then with PMInterface do begin
    if FDecodeOptions.HotUrls then ReaderHotUrls := TRUE;
    hEditor := FindEditorWindow(CheckMDIEditor(hMDIChild));
    if Window = hEditor then begin
      if IsReply(hMDIChild) then
	mtEdit := mtReply
      else mtEdit := mtForward;
      if EditorFancyStyle and FMimeMsgTree.FirstTextIsFormatted then begin
	TERInterface.InsertMessageBody(Window, FMimeMsgTree, GetDisplayInfo(hMDIChild, mtEdit));
	ClickDialogCheckBoxChecked(GetDlgItem(GetEditorMessagePage(hMDIChild), CTL_RICHTEXT));
	if TERInterface.IsTerWindow(Window) then SetFocus(Window);
      end;
      if mtEdit = mtReply then with GetMDIEditorDataPtr(hMDIChild)^ do begin
	case FMimeMsgTree.BodyEncoding of
	  ePMEncrypted..ePMSigned: AutomationFlags := AutomationFlags or AUTO_FLAG_PMIME;
	  eSMEncrypted..eSMInline: AutomationFlags := AutomationFlags or AUTO_FLAG_SMIME;
	end;
      end;
    end
    else if (Window = hReader) or ((Window = hPreview) and IsChild(hMDIChild, GetFocus)) then begin
      TERInterface.InsertMessageBody(Window, FMimeMsgTree, GetDisplayInfo(THandle(-1), mtRead));
      if TERInterface.IsTerWindow(Window) then SetFocus(Window);
    end;
    RedrawWindow(hMDIChild, nil, 0, RDW_ALLCHILDREN or RDW_UPDATENOW);
  end;
  if (Window = hReader) or (prd <> nil) then with PMInterface, FSignatureData do begin
    if IsBodyView(hMDIChild) and not FDecodeOptions.ShowAttachments then begin
      hStatusIcon := LoadIcon(PChar(GetStatusIconIndex(Flags)));
      if prd <> nil then with prd^ do begin
	if not KeepBodyIcon then begin
	  BodyStatusIcon := hStatusIcon;
	  KeepBodyIcon := true;
	end;
      end;
      if not FDecodeOptions.Printing and (Text <> E) then begin
	try
	  case InfoOption of
	    vsInReaderWindow: if Window = hReader then with TERInterface do begin
	      InsertVerificationInfo(Window, Text, LoWord(Flags), GetFirstTabPos(FindHeaderWindow(hMDIChild)),
				     GetDisplayInfo(THandle(-1), mtRead), HasNewReader);
	    end;
	    vsInExtraWindow: begin
	      if LoWord(Flags) in [SF_VALID_TRUSTED, SF_VALID_UNTRUSTED] then
		CustomIconPopup(hStatusIcon, Text)
	      else CustomIconWarn(hStatusIcon, Text);
	    end;
	  end;
	finally
	  Text := E;
	end;
      end;
    end
    else if IsAttachmentView(hMDIChild) and (prd <> nil) then begin
      prd^.AttachmentStatusIcon := LoadIcon(PChar((Flags)));
    end;
    DrawMDIReaderIcon(hMDIChild, true);
  end;
end;

function TMainClass.NewPMFrameProcCommand(wParam: WPARAM; lParam: LPARAM): UINT;
var
  iID: Integer;
begin
  Result := WM_COMMAND;
  iID := LoWord(wParam);
  if lParam = 0 then begin
    if iID >= IDM_MDIWINDOWS then begin
      if iID < IDM_NEWBUTTONS then begin
	if FMDIWindows <> nil then begin
	  dec(iID, IDM_MDIWINDOWS);
	  with FMDIWindows do if iID < Count then begin
	    PostMessage(FMDIClientWindow, WM_MDIRESTORE, THandle(Items[iID]), 0);
	    PostMessage(FMDIClientWindow, WM_MDIACTIVATE, THandle(Items[iID]), 0);
	  end;
	end;
      end
      else if not IsMenu(iID) and (iID > IDM_MAINMENU) then begin
	if iID < IDM_INFOMENU then
	  MainMenuHandler(iID - IDM_MAINMENU)
	else if iID < IDM_EDITMENU then
	  InfoMenuHandler(iID - IDM_INFOMENU)
	else if iID < IDM_ADDRESS_BOOKS then
	  EditMenuHandler(iID - IDM_EDITMENU)
	else AddressBookHandler(iID - IDM_ADDRESS_BOOKS);
      end;
    end;
  end;
  FGetAndSendTriggered := (iID = IDM_GETANDSEND);
end;

procedure TMainClass.HooksCheckCommand(Window: THandle; wParam: WPARAM; lParam: LPARAM);
var
  wCommand: Word;
  sClassName: String;
begin
  if ((wParam = IDM_PRINT) or (wParam = CMD_PRINT)) and (FindReader <> 0) then FDecodeOptions.Printing := true;
  if (HiWord(wParam) = CBN_SELCHANGE) and (ClassString(Window) = CLASS_DIALOG) then begin
    if FindStatic(Window, PMPGPStrings[ENCRYPT_METHOD]) <> 0 then begin
      if SendMessage(lParam, CB_GETCURSEL, 0, 0) > 0 then begin
	EnableWindow(GetDlgItem(Window, CTL_DEFAULT_PASSWORD), false);
	EnableWindow(GetDlgItem(Window, CTL_ASK_AT_STARTUP), false);
      end
      else begin
	EnableWindow(GetDlgItem(Window, CTL_DEFAULT_PASSWORD), true);
	EnableWindow(GetDlgItem(Window, CTL_ASK_AT_STARTUP), true);
      end;
    end;
  end
  else if (lParam <> 0) or not IsMenu(wParam) then with PMInterface do begin
    wCommand := LoWord(wParam);
    if (Window = FMoveMailBoxDialog) and (HiWord(wParam) = BN_CLICKED) then begin
      if MMWarning and (wCommand = CTL_CHANGE_NOW) and (PMInterface.PMVersionNumber >= 4500)
      and not IsDialogCheckBoxChecked(GetDlgItem(Window, CTL_MOVE_CONTENTS)) then begin
	MMWarning := not WarnWithCheckbox(PMPGPStrings[CHANGE_MAILBOX_WARNING],
					  PMPGPStrings[DONT_WARN_AGAIN_CHECKBOX]);
      end;
    end
    else if (wCommand in [ID_OK, ID_CANCEL]) and (ClassString(Window) = CLASS_DIALOG)
    and LoStrStart(PMInterface.DlgReplyOptions, WndString(Window)) then begin
      case wCommand of
	ID_OK: FDecodeOptions.Replying := true;
	ID_CANCEL: FDecodeOptions.FancyReply := false;
      end;
    end
    else if (CheckMDIFolman(0) <> 0) or (CheckMDIFolder(0) <> 0) or (CheckMDIReader(0) <> 0) then begin
      case wCommand of
	CMD_READER_OPEN_1, CMD_READER_OPEN_2: DrawMDIReaderIcon(CheckMDIReader(0), true);
	IDM_PRINT, CMD_PRINT,
	CMD_READER_REPLY, CMD_READER_FWD,
	CMD_FOLMAN_REPLY, CMD_FOLMAN_FWD,
	CMD_ATTACHED_EXEC_V3, CMD_ATTACHED_SAVE_V3, CMD_ATTACHED_EDIT_V3,
	CMD_ATTACHED_EXEC_V4, CMD_ATTACHED_SAVE_V4, CMD_ATTACHED_EDIT_V4,
	CMD_ATTACHED_REPLY_V4, CMD_ATTACHED_FWD_V4, CMD_ATTACHED_PRINT_V4: begin
	  with FDecodeOptions do begin
	    case wCommand of
	      IDM_PRINT, CMD_PRINT: FDecodeOptions.Printing := true;
	      CMD_READER_FWD, CMD_FOLMAN_FWD: begin
		DecodeOnly := true;
		Forwarding := true;
	      end;
	      CMD_READER_REPLY, CMD_FOLMAN_REPLY: begin
		Replying := true;
		FancyReply := EditorFancyStyle;
	      end;
	      CMD_ATTACHED_EXEC_V3, CMD_ATTACHED_SAVE_V3, CMD_ATTACHED_EDIT_V3,
	      CMD_ATTACHED_EXEC_V4, CMD_ATTACHED_SAVE_V4, CMD_ATTACHED_EDIT_V4: begin
		ShowAttachments := (wCommand = CMD_RAW_SWITCH) and (ClassString(Window) = PM_CLASS_READER_PANE);
		Attachment := true;
	      end;
	    end;
	  end;
	end;
	CMD_MOVE, CMD_DELETE, CMD_NEXT, CMD_PREV: begin
	  FDecodeOptions.NewMessage := true;
	  ClearMDIReaderIcons(CheckMDIReader(0));
	end;
	CMD_FIND_ATTACHED_V3: if not HasNewReader then DecryptMessage;
	CMD_FIND_ATTACHED_V4: begin
	  sClassName := ClassString(Window);
	  if sClassName = PM_CLASS_PREVIEW_DLG then with FDecodeOptions do begin
	    ShowAttachments := true;
	    DecodeOnly := true;
	  end
	  else if sClassName = PM_CLASS_READER_TABS then SendKeys([VK_CONTROL, ord('H')]);
	end;
	IDM_VERIFY_DSIG, IDM_KEY_MANAGEMENT: FManageOrVerifyTriggered := true;
      end;
    end;
  end;
end;

function TMainClass.HiddenWindowHelp(wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := 0;
  with PHelpInfo(lParam)^ do if iContextType = HELPINFO_MENUITEM then ShowMenuHelp(hItemHandle, iCtrlID);
end;

function TMainClass.HiddenWindowCommand(wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := 0;
  if (lParam = 0) and (HiWord(wParam) = 0) then begin

    // Menus items -----------------------------------------

    if not IsMenu(wParam) and (wParam > IDM_ID_POPUP) then begin
      if wParam < IDM_MAINMENU then
	IDMenuHandler(wParam - IDM_ID_POPUP)
      else if wParam > IDM_INFOMENU then begin
	if wParam < IDM_ADDRESS_BOOKS then
	  EditMenuHandler(wParam - IDM_EDITMENU)
	else AddressBookHandler(wParam - IDM_ADDRESS_BOOKS);
      end;
    end;
  end
  else if not FPMPGPBusy then begin

    // Buttons ---------------------------------------------

    if wParam = 0 then begin
      case lParam of
	WM_WIPE_PASSPHRASE: PGPInterface.WipeCache;
	WM_EDIT_AUTOLIST: ShowAutoListMenu;
	WM_ENCRYPT_SIGN: ShowEncryptSignMenu;
	WM_DECRYPT_MESSAGE: DecryptMessage;
	WM_VERIFY_SIGNATURE: VerifySignature(E);
	WM_MANAGE_KEYS: ManageKeys(E, false);
	WM_LAUNCH_PGP: LaunchPGP;
      end;
    end
    else if (lParam = WM_ENCRYPT_SIGN) and (wParam > IDM_NEWBUTTONS) then MainMenuHandler(wParam - IDM_NEWBUTTONS);
  end;
end;

function TMainClass.HiddenWindowMenuSelect(wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := 0;
  if DWord(wParam) <> $FFFF0000 then begin
    FAddressBookSelected := (THandle(lParam) = FAddressBookMenu);
    ShowMenuInfo(lParam, LoWord(wParam));
  end;
end;

function TMainClass.HiddenWindowExitMenuLoop(wParam: WPARAM; lParam: LPARAM): UINT;
begin
  Result := 0;
  RemoveAddressBookMenu(FHiddenWindow);
end;

function TMainClass.HiddenWindowInit(wParam: WPARAM; lParam: LPARAM): THandle;
begin
  TextEditSubclass(GetParent(FindPreviewReader));
  FPMFrameWindow := PMInterface.Windows[wMAINFrame];
  FMDIClientWindow := PMInterface.Windows[wMDIClient];
  FPMStatusBarWindow := FindFirstChild(FPMFrameWindow, PM_CLASS_STATUS_BAR, E, 0, false);
  FPMFrameProc := Ptr(SetWindowLong(FPMFrameWindow, GWL_WNDPROC, Longint(@NewPMFrameProc)));
  FPMStatusBarProc := Ptr(SetWindowLong(FPMStatusBarWindow, GWL_WNDPROC, Longint(@NewPMStatusBarProc)));
  FPMStatusBarChildProc := Ptr(SetWindowLong(FindStatusBarChild(E), GWL_WNDPROC, Longint(@NewPMStatusBarChildProc)));
  SetClassLong(FPMStatusBarWindow, GCL_STYLE, GetClassLong(FPMStatusBarWindow, GCL_STYLE) or CS_DBLCLKS);
  if ShowTime > 0 then SetTimer(FHiddenWindow, TIMER_TIME, 1000, nil);
  if TimeAutoSync then SyncSysTimeCommand(FPMFrameWindow, false);
  PGPInterface.OnSetCacheTimer := OnSetCacheTimer;
  AddToolbarButtons;
  InitMimeMessageTree;
  InsertMainMenuEntry(GetMenu(FPMFrameWindow), 0);
  FStatusBarFontHandle := GetStatusBarFont;
  Result := FPMFrameWindow;
end;

function TMainClass.HiddenWindowSendMail(wParam: WPARAM; lParam: LPARAM): UINT;
var
  hMDIEditor: THandle;
begin
  Result := 0;
  hMDIEditor := PMInterface.CheckMDIEditor(0);
  if FSendingMail and (hMDIEditor <> 0) then begin
    FSendingMail := false;
    if not FAutoSending then begin
      Result := WPM_PROHIBIT;
      FEncodeOptions.Aborted := false;
      if TriggerEncoding(hMDIEditor) then
	SetTimer(FHiddenWindow, TIMER_AUTO_SEND, 100, nil)
      else InitEncoding(hMDIEditor);
    end
    else begin
      if not AddDefaultBCCRecipients(hMDIEditor) then begin
	InitEncoding(hMDIEditor);
	Result := WPM_PROHIBIT;
	Exit;
      end;
      AddAllInOneBCCRecipients(hMDIEditor);
      with FRecipientsData do if UnknownKeys = Count then begin
	with FRecipientsKeyData do begin
	  UserIDLineList := UserIDText;
	  KeyCount := UnknownKeys;
	end;
      end;
    end;
  end;
end;

function TMainClass.HiddenWindowLogEvent(hWindow: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
var
  sFolderPath: String;
begin
  Result := 0;
  case wParam of
    WLOG_OPEN_FOLDER: begin
      sFolderPath := PChar(lParam);
    end;
    WLOG_SMTPSEND: begin
      if PMInterface.RemoveBlankMessages <= 0 then begin
	if FGetAndSendTriggered then PMInterface.SelectFrameMenuItem(IDM_GETMAIL);
	Result := WPM_PROHIBIT;
      end;
    end;
    WLOG_NEWDIALOG: begin
      with PCreationEvent(lParam)^ do begin
	case Selector of
	  NDLG_ENCRYPT: NDLGEncryptionHandler(hDialog);
	  NDLG_HOMEBOX: begin
	    if PMInterface.PMVersionNumber >= 4500 then
	      FMoveMailBoxDialog := hDialog
	    else FPMChangeMbxDlgProc := Ptr(SetWindowLong(hDialog, GWL_WNDPROC, Longint(@NewPMChangeMbxDlgProc)));
	  end;
	end;
      end;
    end;
    WLOG_NEWPOP3MESSAGE: begin
      ClearVerificationInfo;
      NewPop3MessageHandler(PPopData(lParam)^.Name);
    end;
    WLOG_SHUTDOWN: Result := HiddenWindowCleanup(hWindow, wParam, lParam);
  end;
end;

function TMainClass.HiddenWindowCleanup(hWindow: THandle; wParam: WPARAM; lParam: LPARAM): UINT;
begin
  if (CloseReaderWindows and CloseEditorWindows) then begin
    if not FCleanupDone then begin
      if not RestartingPegasusMail and (FMoveMailBoxDialog = 0) then ShowWindow(FPMFrameWindow, SW_HIDE);
      SetWindowLong(FindStatusBarChild(E), GWL_WNDPROC, Longint(FPMStatusBarChildProc));
      SetWindowLong(FPMStatusBarWindow, GWL_WNDPROC, Longint(FPMStatusBarProc));
      SetWindowLong(FPMFrameWindow, GWL_WNDPROC, Longint(FPMFrameProc));
      TextEditUnSubclass(GetParent(FindPreviewReader));
      KillTimer(FHiddenWindow, TIMER_TIME);
      DeleteObject(FStatusBarFontHandle);
      DestroyStatusToolTip;
      DestroyTimeToolTip;
      RemoveMainMenuEntry;
      RemoveToolbarButtons;
      FCleanupDone := true;
      Result := WPM_PROHIBIT;
      PostMessage(FPMFrameWindow, WM_CLOSE, 0, 0);
    end
    else Result := 0;
  end
  else Result := WPM_PROHIBIT;
end;

// MDI child handlers

function TMainClass.GetMDIReaderDataPtr(MDIReader: THandle): PMDIReaderData;
begin
  Result := PMDIReaderData(GetProp(MDIReader, MDI_READER_DATA_PROP));
end;

function TMainClass.GetMDIEditorDataPtr(MDIEditor: THandle): PMDIEditorData;
begin
  Result := PMDIEditorData(GetProp(MDIEditor, MDI_EDITOR_DATA_PROP));
end;

procedure TMainClass.MDIReaderUnSubclass(MDIReader: THandle);
var
  prd: PMDIReaderData;
begin
  prd := GetMDIReaderDataPtr(MDIReader);
  if (prd <> nil) and (prd^.ReaderProc <> nil) then begin
    SetWindowLong(MDIReader, GWL_WNDPROC, Longint(prd^.ReaderProc));
    RemoveProp(MDIReader, MDI_READER_DATA_PROP);
    FreeMem(prd);
  end;
end;

procedure TMainClass.MDIEditorUnSubclass(MDIEditor: THandle);
var
  ped: PMDIEditorData;
begin
  ped := GetMDIEditorDataPtr(MDIEditor);
  if (ped <> nil) and (ped^.EditorProc <> nil) then begin
    SetWindowLong(MDIEditor, GWL_WNDPROC, Longint(ped^.EditorProc));
    RemoveProp(MDIEditor, MDI_EDITOR_DATA_PROP);
    ped^.AddressBookList.Free;
    FreeMem(ped);
  end;
end;

procedure TMainClass.TextEditUnSubclass(TextEdit: THandle);
var
  pTextEditProc: TFNWndProc;
begin
  pTextEditProc := TFNWndProc(GetProp(TextEdit, TER_PARENT_PROC_PROP));
  if pTextEditProc <> nil then begin
    SetWindowLong(TextEdit, GWL_WNDPROC, Longint(pTextEditProc));
    RemoveProp(TextEdit, TER_PARENT_PROC_PROP);
  end;
end;

procedure TMainClass.MDIReaderSubclass(MDIReader: THandle);
var
  prd: PMDIReaderData;
begin
  prd := AllocMem(SizeOf(TMDIReaderData));
  if (prd <> nil) and (SetProp(MDIReader, MDI_READER_DATA_PROP, THandle(prd))) then begin
    prd^.ReaderProc := Ptr(SetWindowLong(MDIReader, GWL_WNDPROC, Longint(@NewMDIReaderProc)));
  end;
end;

procedure TMainClass.MDIEditorSubclass(MDIEditor: THandle);
var
  ped: PMDIEditorData;
begin
  ped := AllocMem(SizeOf(TMDIEditorData));
  if (ped <> nil) and SetProp(MDIEditor, MDI_EDITOR_DATA_PROP, THandle(ped)) then begin
    ped^.EditorProc := Ptr(SetWindowLong(MDIEditor, GWL_WNDPROC, Longint(@NewMDIEditorProc)));
    InitEncoding(MDIEditor);
  end;
end;

procedure TMainClass.TextEditSubclass(TextEdit: THandle);
begin
  if (TextEdit <> 0) and (GetProp(TextEdit, TER_PARENT_PROC_PROP) = 0) then begin
    SetProp(TextEdit, TER_PARENT_PROC_PROP, SetWindowLong(TextEdit, GWL_WNDPROC, Longint(@NewPMTextEditProc)));
  end;
end;

procedure TMainClass.NDLGEncryptionHandler(Dialog: THandle);
begin
  ProcessMessages(0);
  FEncryptionDlgProc := Ptr(SetWindowLong(Dialog, GWL_WNDPROC, Longint(@NewEncryptionDlgProc)));
end;

function TMainClass.GetEncryptorName(EncryptionDialog: THandle): String;
begin
  Result := GetSelectedCBXItem(GetDlgItem(EncryptionDialog, CTL_SELECT_METHOD));
end;

function TMainClass.GetEncryptor(EncryptionDialog: THandle; var AllInOne: Longbool): Longint;
var
  sEncryptor: String;
begin
  sEncryptor := GetEncryptorName(EncryptionDialog);
  Result := ord(FirstStrPos(SMIME_INFO, sEncryptor) > 0) * AUTO_FLAG_SMIME;
  PreferSMime := (Result = AUTO_FLAG_SMIME);
  AllInOne := StrStart(AIO, sEncryptor);
end;

procedure TMainClass.SetEncryptor(EncryptionDialog: THandle; EditorFlags: Integer);
type
  TMethodSelector = (msPGPInline, msPGPMime, msSMime);
var
  hEncryptorCbx: THandle;
  msEncryptor: TMethodSelector;
  iMethodIndex: Integer;

  procedure ChangeSelectorState(Disable: Longbool);
  begin
    if Disable then begin
      EnableWindow(hEncryptorCbx, false);
      SetFocus(GetDlgItem(EncryptionDialog, ID_OK));
    end
    else SetFocus(hEncryptorCbx);
  end;

begin
  hEncryptorCbx := GetDlgItem(EncryptionDialog, CTL_SELECT_METHOD);
  DeleteCBXIndex(hEncryptorCbx, GetCBXItemIndex(hEncryptorCbx, FRONTEND_X_PM));
  with FRecipientsData do begin
    msEncryptor := msPGPInline;
    if ((Count = 0) and (UsePGPMime or (EditorFlags and AUTO_FLAG_PMIME <> 0)))
    or ((Count = 1) and (DefaultFlags and AUTO_FLAG_PMIME <> 0)) then begin
      msEncryptor := msPGPMime;
    end;
    if PGPSMimeCapability = smReadWrite then begin
      if ((Count = 0) and (PreferSMime or (EditorFlags and AUTO_FLAG_SMIME <> 0)))
      or ((Count = 1) and (DefaultFlags and AUTO_FLAG_SMIME <> 0)) then begin
	msEncryptor := msSMime;
      end;
    end
    else begin
      DeleteCBXIndex(hEncryptorCbx, GetCBXItemIndex(hEncryptorCbx, FRONTEND_SMIME_AIO));
      DeleteCBXIndex(hEncryptorCbx, GetCBXItemIndex(hEncryptorCbx, FRONTEND_SMIME));
    end;
    case msEncryptor of
      msPGPMime: iMethodIndex := GetCBXItemIndex(hEncryptorCbx, FRONTEND_PMIME);
      msSMime: iMethodIndex := GetCBXItemIndex(hEncryptorCbx, FRONTEND_SMIME);
    else
      iMethodIndex := GetCBXItemIndex(hEncryptorCbx, FRONTEND_INLINE);
    end;
    SelectCBXIndex(hEncryptorCbx, iMethodIndex);
    ChangeSelectorState(Count > 0);
  end;
end;

// Timer handler

procedure TMainClass.OnSetCacheTimer(Seconds: Longint; Wiped: Longbool);
begin
  KillTimer(FHiddenWindow, TIMER_WIPE_CACHE);
  if Wiped then PMInterface.StatusBarShortInfo := PMPGPStrings[PASSPHRASE_WIPED_STATUS];
  if Seconds > 0 then SetTimer(FHiddenWindow, TIMER_WIPE_CACHE, Seconds * 1000, nil);
  EnableMenuItem(FMainMenu, IDM_MAINMENU + ord(miWipePassphrase),
		 MF_BYCOMMAND or (MF_GRAYED * ord(PGPInterface.CacheEmpty = true)));
end;

// POP3 handler

function WaitThread(MainClass: TMainClass): DWord; stdcall;
begin
  with MainClass do begin
    Result := WaitForSingleObject(FWaitEvent, 20000);
    if FWaitThreadStop then
      Result := WAIT_OBJECT_0
    else if Result = WAIT_TIMEOUT then PostMessage(FEventWindow, WM_CLOSE, 0, 0);
  end;
end;

function TMainClass.CreateWaitThread: THandle;
begin
  FEventWindow := 0;
  FWaitThreadStop := false;
  FWaitEvent := CreateEvent(nil, true, false, nil);
  Result := CreateThread(nil, 0, @WaitThread, Self, 0, FWaitThreadID);
end;

function TMainClass.CloseWaitThread(Thread: THandle): DWord;
begin
  GetExitCodeThread(Thread, Result);
  if Result = STILL_ACTIVE then begin
    if FEventWindow = 0 then SetEvent(FWaitEvent);
    WaitForSingleObject(Thread, INFINITE);
    GetExitCodeThread(Thread, Result);
  end;
  CloseHandle(FWaitEvent);
  CloseHandle(Thread);
  FWaitThreadID := 0;
  FWaitEvent := 0;
end;

function TMainClass.ProcessPSMimeMessage(const MessageFile: String; var Subject: String;
					 PreDecode: Longbool; var SaveDecoded: Longbool): String;
begin
  Result := AddPMFlags(MessageFile, PMScanLines, PGPSMimeCapability <> smDisabled, PreDecode, Subject, SaveDecoded);
end;

procedure TMainClass.NewPop3MessageHandler(const MessageFile: String);
var
  sMessage, sSubject, sArmor, sDecoded, sDummy: String;
  fSaveDecoded: Longbool;
  hWaitThread: THandle;
  md: TMessageData;
  sd: TSignatureData;
  kd: TKeyIDData;
  dwExitCode: DWord;
begin
  sMessage := ProcessPSMimeMessage(MessageFile, sSubject, PreDecodePSMime, fSaveDecoded);
  if sMessage <> E then begin
    PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + DOTS;
    hWaitThread := CreateWaitThread;
    try
      sArmor := PGPInterface.CheckArmor(AdjustLineBreaks(StripExcessCRs(sMessage)), md);
      sDecoded := PGPInterface.DecodeMessage(sArmor, FCurrentDialogOwner, false, false, false, true, sd, kd, sDummy);
      ProcessMessages(0);
    finally
      dwExitCode := CloseWaitThread(hWaitThread);
      if PGPInterface.UserAbort then
	PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[CANCELLED_INFO]
      else PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[FINISHED_INFO];
    end;
    if (dwExitCode = WAIT_OBJECT_0) and PGPInterface.NoError then begin
      if fSaveDecoded then begin
	if StrStart(md.Prefix, sArmor) then
	  WriteTextToFile(MessageFile, sDecoded + md.Suffix)
	else WriteTextToFile(MessageFile, md.Prefix + sDecoded + md.Suffix);
      end
      else WriteTextToFile(MessageFile, sMessage);
      with sd do begin
	if LoWord(Flags) <= SF_NO_SIGNATURE then begin
	  sArmor := PGPInterface.CheckArmor(AdjustLineBreaks(StripExcessCRs(sMessage)), md);
	  sd := PGPInterface.VerifySignature(sArmor, E, FCurrentDialogOwner, false, false);
	end;
	if LoWord(Flags) > SF_NO_SIGNATURE then begin
	  Text := PGPInterface.FormatVerificationInfo(Text);
	  sSubject := Concat([AnsiToWide(PMPGPStrings[SUBJECT_HEADER] + SP), Utf8ToWide(sSubject)]);
	  CustomDataPopup(LoadIcon(PChar(GetStatusIconIndex(Flags))), sSubject, Text);
	end;
      end;
    end
    else ProcessPSMimeMessage(MessageFile, sSubject, PreDecodePSMime, fSaveDecoded);
  end;
end;

// About dialog

procedure TMainClass.ShowAboutDialog;
var
  sKeyData: String;
  fContactSupport: Longbool;
begin
  sKeyData := About(Self, PMPGPProductInfo, PMInterface.ReloadsWinsock, fContactSupport);
  if fContactSupport then PMInterface.NewMailMessage(IDWS_ADDRESS, PMPGPVersion, GetSupportInfo);
  if sKeyData <> E then ManageKeys(sKeyData, false);
end;

// AutoList handling

procedure TMainClass.AddAddressesToAutoList(const Addresses: String);
var
  kidd: TKeyIDData;
begin
  ALInterface.SetDefaultFlags(AutoEncrypt, AutoSign);
  if Addresses <> E then begin
    with kidd do begin
      UserIDLineList := Addresses;
      HexIDCommaList := E;
      KeyCount := 0;
    end;
  end
  else begin
    kidd := FRecipientsKeyData;
    kidd.KeyCount := 0;
  end;
  if AddNewAddresses(Self, kidd, ALInterface, PGPSMimeCapability = smReadWrite) then ClearKeyIDData(FRecipientsKeyData);
end;

procedure TMainClass.RemoveKeysFromAutoList(const KIDD: TKeyIDData);
begin
  with KIDD do begin
    if HexIDCommaList = E then Exit;
    if Confirm(UserIDLineList + PARA + PMPGPStrings[LIST_REMOVE_CONFIRMATION])
    and (ALInterface.RemoveByHexID(HexIDCommaList) > 0) then
      Info(PMPGPStrings[ADDRESSES_AND] + PMPGPStrings[KEYS_CERTS_REMOVED])
    else Info(IntToStr(KIDD.KeyCount) + PMPGPStrings[KEYS_CERTS_REMOVED]);
  end;
end;

procedure TMainClass.AddNewKeysToAutoList(const KIDD: TKeyIDData);
begin
  with KIDD do begin
    if UserIDLineList = E then Exit;
    if Confirm(UserIDLineList + PARA + PMPGPStrings[LIST_ADD_CONFIRMATION]) then begin
      ALInterface.SetDefaultFlags(AutoEncrypt, AutoSign);
      AddNewAddresses(Self, KIDD, ALInterface, PGPSMimeCapability = smReadWrite);
    end
    else if KeyCount > 0 then Info(IntToStr(KeyCount) + PMPGPStrings[KEYS_CERTS_ADDED]);
  end;
end;

// ID menu item execution

procedure TMainClass.GetKeyFromServer(const KeyID: String);
var
  kidd: TKeyIDData;
begin
  kidd := PGPInterface.GetKeyFromServer(KeyID, FCurrentDialogOwner, UseAutoList);
  if UseAutoList then AddNewKeysToAutoList(kidd);
end;

// Main menu item execution

procedure TMainClass.MimeConvertMessage;
var
  sNewMailFolder, sNewMailFile, sMessage, sSubject, sArmor, sDecoded, sDummy: String;
  hSearch, hFolman, hFolder: THandle;
  fdFound: TWin32FindData;
  iConvertCount: Integer;
  fSaveDecoded: Longbool;
  md: TMessageData;
  sd: TSignatureData;
  kd: TKeyIDData;
begin
  sNewMailFolder := PMInterface.NewMailPath;
  hSearch := FindFirst(sNewMailFolder + AST + EXT_NEW_MAIL, fdFound);
  if (hSearch <> INVALID_HANDLE_VALUE)
  and Confirm(RemoveSMimeString(PMPGPStrings[MIME_CONVERSION_WARNING], PGPSMimeCapability = smDisabled)) then begin
    PGPInterface.EnableCaching;
    iConvertCount := 0;
    hFolman := 0;
    try
      hFolder := FindFirstChild(FMDIClientWindow, PM_CLASS_MDI_FOLDER, PMInterface.NewMailFolderCaption, 0, false);
      if hFolder <> 0 then SendMessage(hFolder, WM_SYSCOMMAND, SC_CLOSE, 0);
      hFolman := FindFirstChild(FMDIClientWindow, PM_CLASS_FOLMAN, PMInterface.NewMailFolderCaption, 0, false);
      if hFolman <> 0 then PMInterface.SelectFrameMenuItem(CMD_FOLMAN_SWITCH);
      ProcessMessages(0);
      repeat
	sNewMailFile := sNewMailFolder + fdFound.cFileName;
	sMessage := ProcessPSMimeMessage(sNewMailFile, sSubject, true, fSaveDecoded);
	if sMessage <> E then begin
	  PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + DOTS;
	  sArmor := PGPInterface.CheckArmor(AdjustLineBreaks(StripExcessCRs(sMessage)), md);
	  sDecoded := PGPInterface.DecodeMessage(sArmor, FCurrentDialogOwner, false, false, false, false, sd, kd, sDummy);
	  ProcessMessages(0);
	  if PGPInterface.UserAbort then begin
	    PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[CANCELLED_INFO];
	    Exit;
	  end
	  else PMInterface.StatusBarShortInfo := PMPGPStrings[DECODING_INFO] + PMPGPStrings[FINISHED_INFO];
	  if fSaveDecoded then begin
	    if StrStart(md.Prefix, sArmor) then
	      inc(iConvertCount, ord(WriteTextToFile(sNewMailFile, sDecoded + md.Suffix) = true))
	    else inc(iConvertCount, ord(WriteTextToFile(sNewMailFile, md.Prefix + sDecoded + md.Suffix) = true));
	  end
	  else inc(iConvertCount, ord(WriteTextToFile(sNewMailFile, sMessage) = true));
	end;
      until not FindNext(hSearch, fdFound);
    finally
      FindClose(hSearch);
      PGPInterface.ResetCaching;
      Info(IntToStr(iConvertCount) + PMPGPStrings[MIME_CONVERSION_INFO]);
      if hFolman <> 0 then PMInterface.SelectFrameMenuItem(CMD_FOLMAN_SWITCH);
    end;
  end;
end;

procedure TMainClass.DecryptMessage;
var
  hMDIChild, hReader, hFocused: THandle;
  wwReader: TWordWrap;
  sMailMessage: String;
begin
  hMDIChild := GetActiveMDIChild;
  with PMInterface do begin
    if CheckMDIFolman(hMDIChild) <> 0 then begin
      SelectFrameMenuItem(CMD_FIND_ATTACHED_V4);
      Exit;
    end;
    wwReader := ReaderWordWrap;
    if CheckMDIReader(hMDIChild) <> 0 then begin
      hReader := FindReaderWindow(hMDIChild);
      if IsRawView(hMDIChild) then begin
	try
	  sMailMessage := TERInterface.GetEncodedData(hReader, FMimeMsgTree, PGPSMimeCapability <> smDisabled);
	  if sMailMessage <> E then begin
	    PGPInterface.EnableCaching;
	    try
	      ShowMsgTree(Self, sMailMessage);
	    finally
	      PGPInterface.ResetCaching;
	    end;
	  end
	  else if wwReader <> wwNone then Info(PMPGPStrings[WORD_WRAP_HINT]);
	finally
	  ReaderWordWrap := FReaderWordWrapCache;
	end;
	if not HasNewReader then FHideNoAttachmentDialog := true;
      end
      else if IsBodyView(hMDIChild) or IsAttachmentView(hMDIChild) then begin
	if not HasNewReader then FHideNoAttachmentDialog := true;
	hFocused := GetFocus;
	if FTerLinkClicked or Confirm(PMPGPStrings[RAW_VIEW_HINT] + PMPGPStrings[SWITCH_NOW_CONFIRMATION]) then begin
	  if not HasNewReader then SetFrameMenuItemChecked(IDM_SHOWHEADERS, true);
	  ReaderWordWrap := wwNone;
	  FReaderWordWrapCache := wwReader;
	  SelectFrameMenuItem(CMD_RAW_SWITCH);
	  SetTimer(FHiddenWindow, TIMER_RAW_VIEW, 500, nil);
	end
	else SetFocus(hFocused);
      end;
    end;
  end;
  FTerLinkClicked := false;
end;

procedure TMainClass.DecryptFolder;
begin
  PGPInterface.RingCacheUpdate := false;
  try
    ShowFolderText(Self);
  finally
    PGPInterface.RingCacheUpdate := true;
  end;
end;

procedure TMainClass.ShowFingerprint;
var
  sFingerprintData: String;
begin
  sFingerprintData := PGPInterface.GetFingerprintData(-1, FCurrentDialogOwner, false);
  if (sFingerprintData <> E) and Confirm(sFingerprintData + CRLF + PMPGPStrings[COPY_DATA_CONFIRMATION]) then begin
    WriteClipboardData(sFingerprintData, CF_TEXT, FCurrentDialogOwner);
  end;
end;

procedure TMainClass.DeleteKeys;
var
  kidd: TKeyIDData;
begin
  kidd := PGPInterface.DeleteKeys(FCurrentDialogOwner, E, UseAutoList);
  if UseAutoList then RemoveKeysFromAutoList(kidd);
end;

procedure TMainClass.ExtractKeys;
var
  sKeyData, sFileName: String;
  kidd: TKeyIDData;
begin
  sKeyData := PGPInterface.ExtractKeys(FCurrentDialogOwner, kidd);
  if sKeyData <> E then begin
    if not PMInterface.InsertMessageText(sKeyData) then with kidd do begin
      sFileName := LegalFileName(GetFirstLine(UserIDLineList));
      if KeyCount > 1 then sFileName := sFileName + ET_AL + SP + LB + IntToStr(KeyCount) + RB;
      sFileName := KeyFileSave(PMPGPStrings[SAVE_FILE_DIALOG], PMPGPStrings[TO_CLIPBOARD_BUTTON],
			       sFileName, FHiddenWindow);
      if sFileName <> E then begin
	if sFileName = CLIP_NAME then begin
	  sFileName := PMPGPStrings[CLIPBOARD_NAME];
	  if not WriteClipboardData(sKeyData, CF_TEXT, FCurrentDialogOwner) then sFileName := E;
	end
	else if not WriteTextToFile(sFileName, sKeyData) then sFileName := E;
	if sFileName <> E then begin
	  Info(PMPGPStrings[PUBLIC_KEYS_OF] + CRLF + UserIDLineList + PARA + PMPGPStrings[COPIED_TO] + QU + sFileName + QU);
	end;
      end;
    end;
  end;
end;

procedure TMainClass.ImportCerts;
var
  kidd: TKeyIDData;
begin
  kidd := PGPInterface.ImportCerts(FCurrentDialogOwner, UseAutoList);
  if UseAutoList then AddNewKeysToAutoList(kidd);
end;

procedure TMainClass.DeleteCert;
var
  kidd: TKeyIDData;
begin
  kidd := PGPInterface.DeleteCert(FCurrentDialogOwner, UseAutoList);
  if UseAutoList then RemoveKeysFromAutoList(kidd);
end;

procedure TMainClass.ExtractCert;
var
  sCertData, sFileName: String;
  kidd: TKeyIDData;
begin
  sCertData := PGPInterface.ExtractCert(FCurrentDialogOwner, kidd);
  if sCertData <> E then begin
    if not PMInterface.InsertMessageText(sCertData) then with kidd do begin
      sFileName := GetFirstLine(UserIDLineList);
      sFileName := CertFileSave(PMPGPStrings[SAVE_FILE_DIALOG], PMPGPStrings[TO_CLIPBOARD_BUTTON],
				sFileName, FHiddenWindow);
      if sFileName <> E then begin
	if sFileName = CLIP_NAME then begin
	  sFileName := PMPGPStrings[CLIPBOARD_NAME];
	  if not WriteClipboardData(sCertData, CF_TEXT, FCurrentDialogOwner) then sFileName := E;
	end
	else if not WriteTextToFile(sFileName, sCertData) then sFileName := E;
	if sFileName <> E then begin
	  Info(UserIDLineList + HexIDCommaList + CRLF + PMPGPStrings[COPIED_TO] + QU + sFileName + QU);
	end;
      end;
    end;
  end;
end;

procedure TMainClass.LaunchPGP;
begin
  PGPInterface.LaunchKeyManager;
end;

// Edit menu item execution

procedure TMainClass.AddRingAddresses;
begin
  ALInterface.SetDefaultFlags(AutoEncrypt, AutoSign);
  AddKeyring(Self, ALInterface, PGPSMimeCapability = smReadWrite);
end;

procedure TMainClass.SetAddressOptions;
begin
  SetOptions(Self, ALInterface, PGPSMimeCapability = smReadWrite);
end;

procedure TMainClass.UpdateAutoList;
var
  iCount: Integer;
begin
  PGPInterface.GetAutoListIDs;
  try
    iCount := ALInterface.BalanceKeys(PGPInterface.KeyPropsList);
    MessageDialogs.Info(IntToStr(iCount) + PMPGPStrings[LIST_ENTRIES_REMOVED]);
  finally
    PGPInterface.KeyPropsList.Clear;
  end;
end;

procedure TMainClass.EditAutoList;
begin
  with ALInterface do if not EditAutoListFile then Error(PMPGPStrings[LIST_OPEN_ERROR] + PARA + FileName);
end;

// Encoding related processing

procedure TMainClass.InitMimeMessageTree;
begin
  with FMsgTreeData do begin
    DefaultFontName := PMInterface.ReaderFont.Name;
    DefaultMailPath := PMInterface.NewMailPath;
    DateHeaderName := AnsiToUtf8(PMPGPStrings[DATE_HEADER]);
    SenderHeaderName := AnsiToUtf8(PMPGPStrings[SENDER_HEADER]);
    RecipientHeaderName := AnsiToUtf8(PMPGPStrings[RECIPIENT_HEADER]);
    SubjectHeaderName := AnsiToUtf8(PMPGPStrings[SUBJECT_HEADER]);
    ReplyToHeaderName := AnsiToUtf8(PMPGPStrings[REPLY_TO_HEADER]);
    MissingHeaderText := AnsiToUtf8(PMPGPStrings[MISSING_HEADER]);
    MailMessageName := AnsiToUtf8(PMPGPStrings[MAIL_MESSAGE]);
    HeaderSectionName := PMPGPStrings[HEADER_SECTION];
    TextSectionName := PMPGPStrings[TEXT_SECTION];
    UnknownSectionName := PMPGPStrings[UNKNOWN_SECTION];
    SingleCharExtensions := Self.SingleCharExtensions;
    DoubleCharExtensions := Self.DoubleCharExtensions;
    TripleCharExtensions := Self.TripleCharExtensions;
    OnEncodedData := Self.OnEncodedData;
  end;
  FMimeMsgTree.MsgTreeData := FMsgTreeData;
end;

function TMainClass.GetMimeDecodeOptions: TMimeDecodeOptions;
begin
  Result := [mdoReturnUnicode, mdoReturnRtf, mdoDecode];
  if PMInterface.ReaderFancyView or FDecodeOptions.FancyReply then Include(Result, mdoFancyText);
end;

function TMainClass.OnEncodedData(const Data: String; var Charset, SignatureText: String;
				  var SignatureFlags: Integer): String;
begin
  Result := PGPInterface.DecodeFolderData(Data, FCurrentDialogOwner, true, Charset, SignatureText, SignatureFlags);
  if Result = E then Result := AdjustLineBreaks(StripExcessCRs(Data));
end;

function TMainClass.GetEncodedSection(const Data: String; Keys: Longbool): String;
begin
  with FMimeMsgTree do begin
    Clear;
    MimeParse(Data, 1, 0, [mdoAnalyseEncoding]);
    if IsEncoded(BodyEncoding, Keys) then with PSMimeBounds do
      Result := Copy(Data, BegPos, succ(EndPos - BegPos))
    else Result := E;
  end;
end;

procedure TMainClass.SetPMPGPBusy(Value: Longbool);
begin
  FPMPGPBusy := Value;
  SwitchMainMenuEntry(0);
end;

procedure TMainClass.InitEncoding(MDIEditor: THandle);

begin
  FPMPGPBusy := false;
  with FEncodeOptions do begin
    ClearKeyIDData(RecipientIDs);
    AllInOneBCCs := E;
    ADKWarning := Self.ADKWarning;
    HideKeyDialog := Self.HideKeyDialog;
    ConfirmAttachmentEncoding := not Self.EncryptAttachments;
    EncodeAttachments := Self.EncryptAttachments;
    SendAllUnencoded := false;
    RecipientsGroup := false;
    Aborted := false;
  end;
  ClearKeyIDData(FRecipientsKeyData);
  ClearKeyIDData(FSigningKeyData);
  if MDIEditor <> 0 then with GetMDIEditorDataPtr(MDIEditor)^ do begin
    AutomationFlags := ord(PMInterface.EditorAutoSign = true) * AUTO_FLAG_SIGN;
    EncodeFlag := efNone;
    ShowAIOWarning := AIOWarning;
    AddressBookList.Free;
    AddressBookList := nil;
  end;
end;

function TMainClass.TriggerEncoding(MDIEditor: THandle): Longbool;
var
  fMixOptions, fChecked: Longbool;
begin
  Result := true;
  with GetMDIEditorDataPtr(MDIEditor)^ do begin
    if UseAutoList and not (EncodeFlag in [efCancel, efOk])
    or (EncodeFlag > efAuto) then begin
      if GetRecipients(fMixOptions) then begin
	if EncodeFlag = efSign then fMixOptions := false;
	if (FRecipientsData.DefaultFlags and AUTO_FLAG_BASE_FILTER) <> 0 then begin
	  if EncodeFlag = efNone then begin
	    if (FRecipientsData.DefaultFlags and AUTO_FLAG_ASK) <> 0 then
	      EncodeFlag := efAsk
	    else EncodeFlag := efAuto;
	  end;
	  if fMixOptions and MixOptionsWarning then begin
	    case DefCancelYesNoWithCheckBox(PMPGPStrings[MIX_OPTIONS_WARNING],
					    PMPGPStrings[DONT_WARN_AGAIN_CHECKBOX] +
					    PMPGPStrings[NOT_RECOMMENDED_HINT], fChecked) of
	      ID_CANCEL: EncodeFlag := efCancel;
	      ID_YES: begin
		case EncodeFlag of
		  efAuto, efEncrypt: EncodeFlag := efNone;
		  efSign, efEncryptSign: EncodeFlag := efSign;
		end;
		Exit;
	      end;
	      ID_NO: EncodeFlag := efAsk;
	    end;
	    if fChecked then MixOptionsWarning := false;
	  end;
	end
	else if EncodeFlag = efNone then Exit;
      end
      else EncodeFlag := efCancel;
    end
    else Exit;
    case EncodeFlag of
      efCancel: Result := false;
      efOk: Exit;
    else
      ClickDialogCheckBoxChecked(GetDlgItem(PMInterface.GetEditorMessagePage(MDIEditor), CTL_ENCRYPT));
    end;
  end;
end;

function TMainClass.GetRecipients(var MixOptions: Longbool): Longbool;
var
  asRecipients: TAddressStrings;
  sAddresses: String;
begin
  Result := true;
  MixOptions := false;
  if UseAutoList then with PMInterface do begin
    ALInterface.SetDefaultFlags(AutoEncrypt, AutoSign);
    asRecipients[atTo] := ToRecipients;
    asRecipients[atCc] := CCRecipients;
    asRecipients[atBcc] := BCCRecipients;
    GetTabbedPMAddressList(asRecipients, FRecipientsData.AddressCounts, sAddresses);
    MultiKeySelect(Self, sAddresses, ALInterface, FRecipientsData, PGPSMimeCapability = smReadWrite);
    MixOptions := FRecipientsData.MixOptions;
    Result := (FRecipientsData.Count > 0);
  end;
end;

function TMainClass.AddDefaultBCCRecipients(MDIEditor: THandle): Longbool;
var
  asRecipients: TAddressStrings;
  sAddresses: String;
begin
  Result := true;
  if FRecipientsData.Count > 0 then with PMInterface do begin
    asRecipients[atTo] := E;
    asRecipients[atCc] := E;
    asRecipients[atBcc] := GetDefaultBCCAddress(CurrentEditorID);
    if GetTabbedPMAddressList(asRecipients, FRecipientsData.AddressCounts, sAddresses) > 0 then begin
      if UseAutoList then MultiKeySelect(Self, sAddresses, ALInterface, FRecipientsData, PGPSMimeCapability = smReadWrite);
      Result := (FRecipientsData.Count > 0);
    end;
    with GetMDIEditorDataPtr(GetActiveMDIChild)^ do begin
      if Result and ((EncodeFlag = efAsk) or (EncodeFlag > efAuto)) then FRecipientsData.SetOptionFlags(AutomationFlags);
    end;
  end;
end;

function TMainClass.AddAllInOneBCCRecipients(MDIEditor: THandle): Longbool;
var
  sDefaultBBCs: String;
begin
  Result := false;
  with GetMDIEditorDataPtr(MDIEditor)^ do if AddressBookList <> nil then begin
    with PMInterface do begin
      FEncodeOptions.AllInOneBCCs := BCCRecipients;
      sDefaultBBCs := GetDefaultBCCAddress(CurrentEditorID);
    end;
    with FEncodeOptions do begin
      if sDefaultBBCs <> E then begin
	if AllInOneBCCs = E then
	  AllInOneBCCs := sDefaultBBCs
	else AllInOneBCCs := AllInOneBCCs + CM + sDefaultBBCs;
      end;
      if AllInOneBCCs <> E then AllInOneBCCs := AddressBookList.ResolveNames(AllInOneBCCs);
      Result := (AllInOneBCCs <> E);
    end;
  end;
end;

function TMainClass.PreProcessMessage(const SourceFile, DestFile: String): Integer;
var
  sMessage: String;
begin
  if (UpperCase(ExtractFileExt(SourceFile)) = EXT_PM_TEMP) and ReadTextFromFile(SourceFile, sMessage) then begin
    // make PM use BASE64 encoding for binary message data
    Result := ord(WriteTextToFile(DestFile, sMessage) = true) * (1 + ord(IsBinary(sMessage) = true));
  end
  else begin
    // make PM use BASE64 encoding for any attachments
    Result := ord(CopyFile(PChar(SourceFile), PChar(DestFile), false) = true) * 2;
  end;
  if (Result = 0) and (sMessage = E) then Result := ord(CloseHandle(FileCreate(DestFile)) = true) * 2;
end;

function TMainClass.PostProcessMessage(const SourceFile, Recipient, FormData: String): Integer;

  function GetAllInOne(MDIEditor: THandle): Longbool;
  begin
    Result := (GetMDIEditorDataPtr(MDIEditor)^.AddressBookList <> nil);
  end;

  function GetEncodeFlag(MDIEditor: THandle): TEncodeFlag;
  begin
    Result := GetMDIEditorDataPtr(MDIEditor)^.EncodeFlag;
  end;

  function GetAborted(MDIEditor: THandle): Longbool;
  begin
    Result := FEncodeOptions.Aborted or (GetEncodeFlag(MDIEditor) = efCancel);
  end;

var
  hMDIEditor: THandle;
  sStatusInfo, sMessage, sCharset, sHeaders, sBody, sData, sEncoded, sDoneInfo: String;
  fEncrypt, fSign, fUseSMime, fMimeEncode, fIsText, fChecked: Longbool;
  iAutomationFlags, iXPMHeaderFlags, iAIORecipients, iIndex: Integer;
  kdImported: TKeyIDData;
  hdPSMime: THeaderData;
begin
  Result := 0;
  hMDIEditor := PMInterface.CheckMDIEditor(0);
  sStatusInfo := PMPGPStrings[ENCODING_INFO];
  PMInterface.StatusBarShortInfo := sStatusInfo + DOTS;
  if ReadTextFromFile(SourceFile, sMessage) and (Recipient <> E) then begin
    try
      PGPInterface.WipeFile(SourceFile);
      if GetAborted(hMDIEditor) then begin
	WriteTextToFile(SourceFile, PMInterface.GetDummyMessage);
	Exit;
      end;
      sCharset := PMInterface.GetDefaultCharset;
      iAutomationFlags := GetMDIEditorDataPtr(hMDIEditor)^.AutomationFlags;
      if FEncodeOptions.SendAllUnencoded or (iAutomationFlags = AUTO_FLAG_NOTHING) then begin
	WriteTextToFile(SourceFile, RemovePMCryptoHeaders(sMessage, sCharset, not PMInterface.Mime8BitEncoding));
	Exit;
      end;
      with FEncodeOptions, RecipientIDs do begin
	UserIDLineList := Recipient;
	HexIDCommaList := E;
	KeyCount := 0;
      end;
      if FRecipientsData.Count > 0 then with FRecipientsData do begin
	if (GetEncodeFlag(hMDIEditor) >= efAsk) and (HexIDs[0] <> HEX_ID_NULL) then begin
	  if GetEncodeFlag(hMDIEditor) > efAuto then
	    iAutomationFlags := iAutomationFlags or (Flags[0] and AUTO_FLAG_MIME_FILTER)
	  else iAutomationFlags := Flags[0];
	  with FEncodeOptions.RecipientIDs do begin
	    HexIDCommaList := HexIDs[0];
	    KeyCount := 1;
	  end;
	end;
	Delete(0);
      end;
      fEncrypt := (iAutomationFlags and AUTO_FLAG_ENCRYPT) <> 0;
      fSign := (iAutomationFlags and AUTO_FLAG_SIGN) <> 0;
      if GetEncodeFlag(hMDIEditor) = efOk then begin
	if FormData = SMIME_INFO then begin
	  fMimeEncode := true;
	  fUseSMime := true;
	end
	else if FormData = PMIME_INFO then begin
	  fMimeEncode := true;
	  fUseSMime := false;
	end
	else begin
	  fMimeEncode := false;
	  fUseSMime := false;
	end;
      end
      else begin
	fUseSMime := (iAutomationFlags and AUTO_FLAG_SMIME) <> 0;
	if (FEncodeOptions.RecipientIDs.KeyCount = 0) and not fUseSMime then begin
	  iAutomationFlags := iAutomationFlags or (ord(UsePGPMime = true) * AUTO_FLAG_PMIME);
	end;
	fMimeEncode := (iAutomationFlags and AUTO_FLAG_MIME_FILTER) <> 0;
      end;
      iXPMHeaderFlags := ord(fEncrypt = true) * XPM_FLAG_ENCRYPTED or ord(fSign = true) * XPM_FLAG_SIGNED;
      with PMInterface, FSigningKeyData do begin
	if UseAlways and (HexIDCommaList = E) then HexIDCommaList := DefaultKey;
	UserIDLineList := GetSenderAddress(CurrentEditorID);
      end;
      with PGPInterface do begin
	ErrorCode := 0;
	GetSignKeyData(FSigningKeyData, fUseSMime);
	SetSignAlgorithm(SignAlgorithm, UseRipeMD);
      end;
      kdImported.KeyCount := 0;
      if fMimeEncode then begin
	if fUseSMime then
	  sStatusInfo := sStatusInfo + SP + LB + SMIME_INFO + RB
	else sStatusInfo := sStatusInfo + SP + LB + PMIME_INFO + RB;
	PMInterface.StatusBarShortInfo := sStatusInfo + DOTS;
	hdPSMime := PreparePSMimeEncoding(sMessage, sCharset, iXPMHeaderFlags);
	with hdPSMime do begin
	  sHeaders := Copy(Text, Pos, Len);
	  sBody := Copy(Text, Pos + Len, MAXINT);
	end;
      end;
      iAIORecipients := 0;
      repeat
	if fMimeEncode then with FEncodeOptions do begin
	  if GetAllInOne(hMDIEditor) then with RecipientIDs do begin
	    iAIORecipients := ExtractHeaderAddresses(sHeaders, AllInOneBCCs, UserIDLineList);
	    KeyCount := 0;
	  end;
	  sEncoded := sHeaders + PGPInterface.EncodePSMime(sBody, fEncrypt, fSign, EncryptToSelf, fUseSMime, HideKeyDialog,
							   GetAllInOne(hMDIEditor) and (iAIORecipients > 1),
							   FCurrentDialogOwner, RecipientIDs,
							   FSigningKeyData, kdImported);
	  Result := PGPInterface.ErrorCode;
	end
	else with FMimeMsgTree do begin
	  Clear;
	  DefaultCharset := sCharset;
	  MimeParse(sMessage, 1, 0, [mdoFullParse, mdoDecode]);
	  if Count > 0 then with FEncodeOptions do begin
	    if GetAllInOne(hMDIEditor) then with RecipientIDs do begin
	      iAIORecipients := ExtractHeaderAddresses(SectionData[0], AllInOneBCCs, UserIDLineList);
	      KeyCount := 0;
	    end;
	  end;
	  iIndex := 0;
	  while iIndex < Count do with FEncodeOptions do begin
	    if SectionTypes[iIndex] > mdtSubHeaders then begin
	      fIsText := (SectionTypes[iIndex] <> mdtFile);
	      if fIsText or EncodeAttachments then with SectionNodes[iIndex] do begin
		if fIsText or fEncrypt then begin
		  if fIsText then
		    sData := Utf8ToString(SectionData, Charset)
		  else sData := SectionData;
		  sEncoded := PGPInterface.EncodeInline(sData, Charset, fEncrypt, fSign,
							EncryptToSelf, fIsText, HideKeyDialog,
							GetAllInOne(hMDIEditor) and (iAIORecipients > 1),
							FCurrentDialogOwner, RecipientIDs,
							FSigningKeyData, kdImported,
							RecipientsGroup);
		  if sEncoded <> E then begin
		    if not fEncrypt then begin
		      if fIsText then
			SetEncodedData(sEncoded, CTE_QP, iXPMHeaderFlags, false, false)
		      else SetEncodedData(sEncoded, CTE_B64, iXPMHeaderFlags, false, false);
		    end
		    else begin
		      if fIsText then
			SetEncodedData(sEncoded, CTE_QP, iXPMHeaderFlags, false, false)
		      else SetEncodedData(sEncoded, CTE_7BIT, iXPMHeaderFlags, AppendPGPExtension, false);
		    end;
		  end;
		end
		else begin
		  sEncoded := PGPInterface.DetachedSignData(SectionData, FCurrentDialogOwner, FSigningKeyData);
		  if sEncoded <> E then iIndex := SetEncodedData(sEncoded, CTE_B64, iXPMHeaderFlags, false, true);
		end;
		Result := PGPInterface.ErrorCode;
		if Result <> 0 then Break;
	      end
	      else begin
		if ConfirmAttachmentEncoding then begin
		  EncodeAttachments := ConfirmWithCheckBox(PMPGPStrings[ENCODE_ATTACHMENTS_CONFIRMATION],
							   PMPGPStrings[ALWAYS_ENCODE_ATTACHMENTS_CHECKBOX],
							   fChecked);
		  ConfirmAttachmentEncoding := false;
		  if fChecked then EncryptAttachments := true;
		end;
		if not EncodeAttachments then
		  SectionNodes[iIndex].SetEncodedData(SectionData[iIndex], CTE_B64, 0, false, false)
		else Continue;
	      end;
	    end;
	    inc(iIndex);
	  end;
	  FEncodeOptions.RecipientsGroup := false;
	  if Result = 0 then sEncoded := MessageData;
	end;
	if fEncrypt and PGPInterface.IsEncryptKeyError(Result) then with FEncodeOptions do begin
	  with RecipientIDs do begin
	    HexIDCommaList := E;
	    KeyCount := 0;
	  end;
	  HideKeyDialog := false;
	  Result := 0;
	  Continue;
	end;
	if Result <> 0 then begin
	  if CustomConfirm(PMPGPStrings[SELECT_DIALOG],
			   PGPErrorStrings[Result] + EM + PARA + PMPGPStrings[SEND_UNENCODED_CONFIRMATION],
			   PMPGPStrings[COPY_MSG_BUTTON], PMPGPStrings[SEND_MSG_BUTTON]) then begin
	    WriteTextToFile(SourceFile, PMInterface.CopyMailMessage(hMDIEditor));
	    FEncodeOptions.Aborted := true;
	    Break;
	  end
	  else begin
	    FEncodeOptions.SendAllUnencoded := true;
	    WriteTextToFile(SourceFile, RemovePMCryptoHeaders(sMessage, sCharset, not PMInterface.Mime8BitEncoding));
	  end;
	end
	else with FEncodeOptions do begin
	  with FRecipientsKeyData do begin
	    if (KeyCount > 0) and (HexIDCommaList = E) then begin
	      UserIDLineList := E;
	      KeyCount := 0;
	    end;
	    if UseAutoList and not ALInterface.EntryExists(Recipient, RecipientIDs.HexIDCommaList) then begin
	      if Recipient <> E then UserIDLineList := UserIDLineList + Recipient + CRLF;
	      if RecipientIDs.HexIDCommaList <> E then HexIDCommaList := HexIDCommaList + RecipientIDs.HexIDCommaList + CM;
	      inc(KeyCount);
	    end;
	  end;
	  if fEncrypt and ADKWarning and PGPInterface.GetKeyHasADKProperty(RecipientIDs.HexIDCommaList) then begin
	    if CustomCancelWithCheckBox(PMPGPStrings[WARN_DIALOG], PMPGPStrings[ADK_CONTINUE_CONFIRMATION],
					PMPGPStrings[CONTINUE_BUTTON], PMPGPStrings[COPY_MSG_BUTTON],
					PMPGPStrings[DONT_WARN_AGAIN_CHECKBOX] +
					PMPGPStrings[NOT_RECOMMENDED_HINT],
					fChecked) then begin
	      WriteTextToFile(SourceFile, PMInterface.CopyMailMessage(hMDIEditor));
	      FEncodeOptions.Aborted := true;
	      Break;
	    end
	    else ADKWarning := false;
	    if fChecked then Self.ADKWarning := false;
	  end;
	  WriteTextToFile(SourceFile, sEncoded);
	  Break;
	end;
      until Result <> 0;
      if not GetAborted(hMDIEditor) then begin
	if Result = 0 then begin
	  if fEncrypt and ConfirmEncryption then begin
	    PopupWithCheckBox(PMPGPStrings[ENCRYPTION_SUCCESS_CONFIRMATION],
			      PMPGPStrings[DONT_NOTIFY_AGAIN_CHECKBOX],
			      fChecked);
	    if fChecked then ConfirmEncryption := false;
	  end;
	  if UseAutoList then AddNewKeysToAutoList(kdImported);
	end;
      end;
    finally
      try
	if not GetAborted(hMDIEditor) then
	  sDoneInfo := PMPGPStrings[FINISHED_INFO]
	else sDoneInfo := PMPGPStrings[CANCELLED_INFO];
	with PMInterface do begin
	  StatusBarShortInfo := sStatusInfo + sDoneInfo;
	  RescanMessageQueue;
	  FMimeMsgTree.Clear;
	end;
      finally
	Result := Byte(Result = 0);
      end;
    end;
  end;
end;


initialization
  if MainClass = nil then
    MainClass := TMainClass.Create
  else inc(MainClass.FInstance);

finalization
  if MainClass <> nil then begin
    if MainClass.FInstance > 0 then
      dec(MainClass.FInstance)
    else begin
      MainClass.Free;
      MainClass := nil;
    end;
end;


end.

