{$J+,Z4}
unit PMPGPResources;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, HelpCtrl, KeyPropTypes, pgpErrors, PMPGPCommon;

const // from WinUser.h
  LR_DEFAULTCOLOR		= $0000;
  LR_MONOCHROME			= $0001;
  LR_COLOR			= $0002;
  LR_COPYRETURNORG		= $0004;
  LR_COPYDELETEORG		= $0008;
  LR_LOADFROMFILE		= $0010;
  LR_LOADTRANSPARENT		= $0020;
  LR_DEFAULTSIZE		= $0040;
  LR_VGACOLOR			= $0080;
  LR_LOADMAP3DCOLORS		= $1000;
  LR_CREATEDIBSECTION		= $2000;
  LR_COPYFROMRESOURCE		= $4000;
  LR_SHARED			= $8000;

const // About dialog
  GET_UPDATE			= 'Update';
  IDWS_PGP_KEY			= 'idw''s PGP key';
  IDWS_ADDRESS			= 'idw.doc@t-online.de';

const // Button files
  BTN_EDIT_AUTOLIST		= 'EditAutoList.bmp';
  BTN_WIPE_PASSPHRASE		= 'WipePassphrase.bmp';
  BTN_ENCRYPT_SIGN		= 'EncryptSign.bmp';
  BTN_ENCRYPT_ONLY		= 'EncryptOnly.bmp';
  BTN_SIGN_ONLY			= 'SignOnly.bmp';
  BTN_DECRYPT_MESSAGE		= 'DecryptMessage.bmp';
  BTN_VERIFY_SIGNATURE		= 'VerifySignature.bmp';
  BTN_MANAGE_KEYS		= 'ManageKeys.bmp';
  BTN_LAUNCH_PGP		= 'LaunchPGP.bmp';

const // Window icon
  ICO_SMALL			= 'SMALLICON';

const // Help window
  HELP_WINDOW			= 'SIMPLE';

type // Help IDs
  THelpIDs = (

    IDH_POPUP_HELP_START,
    // PGP Paths
    IDH_KEYSERVER,
    IDH_SECRING_FILE,
    IDH_PUBRING_FILE,
    IDH_GROUPS_FILE,
    // PGP defaults
    IDH_DEFAULT_USER,
    IDH_USE_ALWAYS,
    IDH_USE_RIPEMD,
    IDH_COMMENT,
    // Pegasus Mail
    IDH_SCAN_LINES,
    IDH_EDIT_MARGIN,
    // Encryption
    IDH_USE_AUTOLIST,
    IDH_HIDE_KEY_DIALOG,
    IDH_ENCRYPT_TO_SELF,
    IDH_ENCRYPT_ATTACHMENTS,
    IDH_CONFIRM_ENCRYPTION,
    IDH_MIX_OPTIONS_WARN,
    IDH_ADK_WARN,
    // Decoding
    IDH_CHECK_FOR_KEY_DATA,
    IDH_KEYSERVER_LOOKUP,
    IDH_PRE_DECODE_PSMIME,
    IDH_AUTO_VERIFY,
    // Paranoia
    IDH_PASSPHRASE_CACHE,
    IDH_WIPE_FILE_PASSES,

    // Time
    IDH_SHOW_TIME,
    IDH_TIME_SERVER,
    IDH_TIME_AUTO_SYNC,

    IDH_CONTEXT_HELP_START,
    // Dialogs
    IDH_AUTOLIST_OPTIONS_DLG,
    IDH_ENCRYPT_KEY_DLG,
    IDH_MULTI_ID_DLG,
    // Menus
    IDH_MAINMENU,
    IDH_AUTOLIST,
    // Configuration
    IDH_CONFIGURATION_DLG

  );

const // Icon IDs
  IDW_VERIFY_NOTHING		= 0;
  IDW_VERIFY_NO_SIGNATURE	= 1;
  IDW_VERIFY_VALID_TRUSTED	= 2;
  IDW_VERIFY_VALID_UNTRUSTED	= 3;
  IDW_VERIFY_INVALID		= 4;
  IDW_VERIFY_NO_KEY		= 5;
  IDW_VERIFY_EXPIRED		= 6;
  IDW_VERIFY_ICON_COUNT		= 6;
  IDW_VERIFY_ENCRYPTED		= 1 shl 16;

const // Button IDs
  IDW_BUTTON_OK			= 100;
  IDW_BUTTON_CANCEL		= 101;
  IDW_BUTTON_HELP		= 102;

  IDW_BUTTON_SAVE		= 104;
  IDW_BUTTON_MAIL		= 105;
  IDW_BUTTON_CERT		= 106;

  OPEN_FOLDER_BUTTON		= 108;
  LOAD_IMAGES_BUTTON		= 109;

const // Grid header
  IDW_SORT_ARROWS		= 111;

const // Grid cells
  ADD_ENTRY_BITMAP		= 113;
  ENCRYPT_BITMAP		= 114;
  SIGN_BITMAP			= 115;
  ASK_BITMAP			= 116;
  PMIME_BITMAP			= 117;
  SMIME_BITMAP			= 118;
  DISABLED_BITMAP		= 119;

type
  TPMPGPProductInfo = Record
    MainLanguageID: LANGID;
    Copyright: String;
    Version: String;
    Name: String;
  end;

type
  TPMPGPStringIndex = (

    // Pegasus Mail strings
    WINDOW_ITEM,
    SYSTEM_ITEM,
    MISSING_RECIPIENT_DIALOG,
    MISSING_SUBJECT_DIALOG,
    NO_DIGITAL_SIGNATURE,
    ENCRYPT_METHOD,
    SEND_BUTTON,
    EDITOR_SETTINGS_GROUP,

    // Mail strings
    DATE_HEADER,
    SENDER_HEADER,
    RECIPIENT_HEADER,
    SUBJECT_HEADER,
    REPLY_TO_HEADER,
    MISSING_HEADER,
    HEADER_SECTION,
    TEXT_SECTION,
    UNKNOWN_SECTION,
    MAIL_MESSAGE,

    // Toolbar buttons
    EDIT_AUTOLIST_BUTTON,
    WIPE_PASSPHRASE_BUTTON,
    ENCRYPT_SIGN_BUTTON,
    DECRYPT_MESSAGE_BUTTON,
    VERIFY_SIGNATURE_BUTTON,
    MANAGE_KEYS_BUTTON,
    LAUNCH_PGP_BUTTON,

    // Info submenu items
    CONFIG_ITEM,
    HELP_ITEM,
    ABOUT_ITEM,

    // Edit submenu items
    ADD_ADDRESS_BOOKS_ITEM,
    ENABLE_AUTOLIST_ITEM,
    ADD_RECIPIENTS_ITEM,
    ADD_RING_ADDRESSES_ITEM,
    SET_ADRRESS_OPTIONS_ITEM,
    UPDATE_AUTOLIST_ITEM,
    EDIT_AUTOLIST_ITEM,

    // Main menu items
    WIPE_PASSPHRASE_ITEM,
    USE_PGP_MIME_ITEM,
    APPEND_PGP_EXTENSION_ITEM,
    AUTO_ENCRYPT_ITEM,
    ENCRYPT_ITEM,
    SIGN_ITEM,
    ENCRYPT_SIGN_ITEM,
    MIME_CONVERT_MESSAGE_ITEM,
    DECRYPT_MESSAGE_ITEM,
    DECRYPT_FOLDER_ITEM,
    VERIFY_SIGNATURE_ITEM,
    SHOW_FINGERPRINT_ITEM,
    IMPORT_SYSTEM_CERTIFICATES_ITEM,
    ADD_KEYS_OR_CERTIFICATES_ITEM,
    DELETE_KEYS_ITEM,
    EXTRACT_KEYS_ITEM,
    DELETE_CERTIFICATE_ITEM,
    EXTRACT_CERTIFICATE_ITEM,
    LAUNCH_PGP_ITEM,

    // Address submenu item
    ADD_ALL_BOOKS_ITEM,

    // Popup menu items
    ADD_ADDRESS_ITEM,
    KEY_SERVER_ITEM,
    KEYID_COPY_ITEM,

    // Window menu item info
    BRING_TO_FOREGROUND_INFO,

    // Info submenu item infos
    CONFIG_INFO,
    HELP_INFO,
    ABOUT_INFO,

    // Edit submenu item infos
    ENABLE_AUTOLIST_INFO,
    ADD_RECIPIENTS_INFO,
    ADD_RING_ADDRESSES_INFO,
    SET_ADRRESS_OPTIONS_INFO,
    UPDATE_AUTOLIST_INFO,
    EDIT_AUTOLIST_INFO,

    // Main menu item infos
    WIPE_PASSPHRASE_INFO,
    USE_PGP_MIME_INFO,
    APPEND_PGP_EXTENSION_INFO,
    AUTO_ENCRYPT_INFO,
    ENCRYPT_INFO,
    SIGN_INFO,
    ENCRYPT_SIGN_INFO,
    MIME_CONVERT_MESSAGE_INFO,
    DECRYPT_MESSAGE_INFO,
    DECRYPT_FOLDER_INFO,
    VERIFY_SIGNATURE_INFO,
    SHOW_FINGERPRINT_INFO,
    IMPORT_SYSTEM_CERTIFICATES_INFO,
    ADD_KEYS_OR_CERTIFICATES_INFO,
    DELETE_KEYS_INFO,
    EXTRACT_KEYS_INFO,
    DELETE_CERTIFICATE_INFO,
    EXTRACT_CERTIFICATE_INFO,
    LAUNCH_PGP_INFO,

    // Address submenu item infos
    ADD_ALL_BOOKS_INFO,
    ADD_BOOK_ADDRESSES_INFO,

    // Subclass infos
    CHANGE_USER_INFO,
    CHANGE_MAILBOX_INFO,
    CHANGE_MAILBOX_WARNING,

    // Shortcut names
    LINK_CONTACT,
    LINK_LICENCE,
    LINK_MANUAL,
    LINK_UNINSTALL,
    LINK_USER_SETUP,
    LINK_USER_UNINSTALL,

    // Message dialogs
    SELECT_ALL_ITEM,
    COPY_SELECTION_ITEM,
    ERROR_DIALOG,
    WARN_DIALOG,
    INFO_DIALOG,
    CONFIRM_DIALOG,
    CANCEL_BUTTON,
    YES_BUTTON,
    NO_BUTTON,
    ACCEPT_HINT,
    CANCEL_HINT,

    // Search dialog
    SEARCH_DIALOG,
    CASE_SENSITIVE_OPTION,
    WHOLE_WORDS_OPTION,
    UPWARDS_OPTION,
    TEXT_NOT_FOUND_INFO,
    RESTART_SEARCH_CONFIRMATION,

    // About dialog
    ABOUT_DIALOG,
    SEARCHING_,
    NEW_VERSION,
    NO_NEW_VERSION,
    VERSION_COMPARE_ERROR,
    VERSION_FIND_ERROR,
    WEB_READ_ERROR,
    WEB_LOAD_ERROR,
    WEB_CONNECT_ERROR,
    WEB_LIBRARY_ERROR,

    // ConfigHandler
    USER_HANDLING_HINT,
    PGP9_LICENCE_WARNING,
    HELP_MANUAL_OPEN_ERROR,
    USER_INSTALL_SUCCESS,
    USER_SETUP_FAILURE,
    USER_UNINSTALL_SUCCESS,
    USER_UNINSTALL_FAILURE,
    USER_UPDATE_CONFIRMATION,
    USER_UPDATE_ALL_CONFIRMATION,
    USER_UPDATE_ABORTED,
    USER_PMSMIME_HINT,

    // Config dialog
    CONFIG_DIALOG,
    CONFIG_PGP_PATHS,
    CONFIG_PM_TOOLBAR,
    CONFIG_PMPGP_ENCODING,
    CONFIG_PMPGP_DECODING,
    CONFIG_MISCELLANEOUS,
    RESTART_CONFIRMATION,
    SAVE_CONFIRMATION,
    SAVE_ERROR,

    // Button hints
    CONFIG_HELP_HINT,

    // PGP paths
    KEYSERVER_LABEL,
    SECRING_LABEL,
    PUBRING_LABEL,
    GROUPS_LABEL,
    KEYFILE_DIALOG,

    // PGP miscellaneous
    DEFAULT_USER_LABEL,
    USE_ALWAYS_CHECKBOX,
    SIGN_ALGORITHM_LABEL,
    USE_RIPEMD_CHECKBOX,
    COMMENT_LABEL,

    // PM toolbar buttons
    WIPE_PASSPHRASE_CHECKBOX,
    ENCRYPT_SIGN_OPTIONS_LABEL,
    ENCRYPT_SIGN_OPTIONS_COMBOBOX,
    DECRYPT_MESSAGE_CHECKBOX,
    MANAGE_KEYS_CHECKBOX,
    VERIFY_SIGNATURE_CHECKBOX,
    LAUNCH_PGP_CHECKBOX,

    // PM miscellaneous
    NO_CHANGE_LABEL,
    SCAN_LINES_LABEL,
    CHECK_HEADER_LINES_LABEL,
    ESTIMATED_HEADER_LENGHT,
    SCAN_LINES_ADJUST,
    SCAN_LINES_OK,
    EDIT_MARGIN_LABEL,

    // PMPGP encoding
    USE_AUTOLIST_CHECKBOX,
    HIDE_KEY_DIALOG_CHECKBOX,
    ENCRYPT_TO_SELF_CHECKBOX,
    ENCRYPT_ATTACHMENTS_CHECKBOX,
    ALWAYS_ENCODE_ATTACHMENTS_CHECKBOX,
    CONFIRM_ENCRYPTION_CHECKBOX,
    DONT_NOTIFY_AGAIN_CHECKBOX,
    MIX_OPTIONS_WARNING_CHECKBOX,
    DONT_WARN_AGAIN_CHECKBOX,
    ADK_WARNING_CHECKBOX,
    NOT_RECOMMENDED_HINT,

    // PMPGP decoding
    CHECK_FOR_KEY_DATA_CHECKBOX,
    KEYSERVER_LOOKUP_CHECKBOX,
    PRE_DECODE_PSMIME_CHECKBOX,
    AUTO_VERIFY_CHECKBOX,
    NO_INFO_RADIOBOX,
    IN_READER_WINDOW_RADIOBOX,
    IN_EXTRA_WINDOW_RADIOBOX,

    // PMPGP miscellaneous
    CACHE_WARNING,
    CACHE_PASSPHRASE_LABEL,
    CACHE_DISABLED,
    CACHE_INFINITE,
    CACHE_SECONDS,
    WIPE_FILE_PASSES_LABEL,
    STANDARD_COLORS_CHECKBOX,
    SHOW_TIME_CHECKBOX,
    TIME_ONLY,
    DATE_TIME,
    TIMESERVER_LABEL,
    AUTO_SYNC_CHECKBOX,
    SYNC_CLOCK_HINT,
    SYNC_CLOCK_INFO,
    AUTO_SIGN_CONFIRMATION,
    USE_AUTOLIST_CONFIRMATION,

    // AutoList dialog
    NO_AUTOLIST_ENTRIES,
    OPTIONS_SWITCH_LABEL,
    OPTIONS_FIND_HINT,
    OPTIONS_HELP_HINT,
    OPTIONS_DELETE_HINT,
    ADD_ENTRY_COL_HINT,
    ENCRYPT_COL_HINT,
    SIGN_COL_HINT,
    ASK_COL_HINT,
    PMIME_COL_HINT,
    SMIME_COL_HINT,
    DISABLED_COL_HINT,
    ALGORITHM_COL_HINT,
    HEX_ID_COL_HINT,
    USER_ID_COL_HINT,
    MISSING_KEY,
    CORRUPT_KEY,
    REVOKED_KEY,
    EXPIRED_KEY,
    DISABLED_KEY,
    MULTIPLE_KEYS,
    INVALID_ALGORITHM,
    MISSING_KEY_INFO,
    NONE_SELECTED,
    SELECT_KEY_ID,
    SELECT_KEY_ID_INFO,
    FOR_ADDRESS,
    DELETE_ADDRESS_CONFIRMATION,
    SELECT_KEY_FOR_RECIPIENT_DIALOG,
    SET_OPTIONS_DIALOG,
    ADD_KEYRING_DIALOG,
    ADD_ADDRESS_DIALOG,
    SELECT_OR_CANCEL_INFO,
    NO_NEW_ADDRESSES_INFO,

    // More PMPGP strings
    PUBLIC_KEYS_OF,
    CERTIFICATES_OF,
    COPIED_TO,
    CLIPBOARD_NAME,

    // PGP basics
    PASSPHRASE_WIPED_STATUS,

    // PGP key types
    KEYTYPE_PUBLIC,
    KEYTYPE_SECRET,

    // PGP key trust level
    TRUST_UNDEFINED,
    TRUST_UNKNOWN,
    TRUST_NEVER,
    TRUST_XXX,
    TRUST_YYY,
    TRUST_MARGINAL,
    TRUST_COMPLETE,
    TRUST_ULTIMATE,

    // PGP key validity level
    VALIDITY_UNKNOWN,
    VALIDITY_INVALID,
    VALIDITY_MARGINAL,
    VALIDITY_COMPLETE,

    // PGP key algorithms
    ALGORITHM_INVALID,
    ALGORITHM_RSA,
    ALGORITHM_RSAENCRYPTONLY,
    ALGORITHM_RSASIGNONLY,
    ALGORITHM_DH,
    ALGORITHM_DSS,
    ALGORITHM_DHDSS,

    // PGP hash algorithms
    HASH_INVALID,
    HASH_MD2,
    HASH_MD5,
    HASH_SHA,
    HASH_RIPEMD160,
    HASH_SHA256,
    HASH_SHA384,
    HASH_SHA512,

    // Certificate chain levels
    CHAIN_INCOMPLETE,
    CHAIN_COMPLETE,
    CHAIN_SELF_SIGNED,

    // AutoListHandler
    LIST_INVALID_INDEX,
    LIST_OPEN_ERROR,
    LIST_ADD_CONFIRMATION,
    LIST_REMOVE_CONFIRMATION,
    LIST_ENTRIES_ADDED,
    LIST_ENTRIES_REMOVED,

    // PGPHandler
    GROUP_NAME,
    FROM_KEYSERVER_BUTTON,
    SELECT_AND_ADD_BUTTON,
    SELECTED_KEYS_TEXT,
    SELECT_ALL_BUTTON,
    UNSELECT_BUTTON,
    SHOW_CERT_PROPS_BUTTON,
    CERTIFICATE_NAME,
    KEY_NAME,
    BAD_SIGNATURE,
    EXPIRED_SIGNATURE,
    KEY_DISABLED,
    KEY_EXPIRED,
    KEY_REVOKED,
    KEY_UNKNOWN,
    CERT_EXPIRED,
    CERT_REVOKED,
    CERT_CORRUPT,
    PGP_INIT_RETRY_CONFIRMATION,
    SELECT_FINGERPRINT_KEY,
    CERTIFIED_BY,
    COPY_DATA_CONFIRMATION,
    ENTER_PASSPHRASE_PROMPT,
    FOR_DECRYPTION,
    SELECT_KEY_AND_ENTER_PASSPHRASE_PROMPT,
    SELECT_CERT_OF_PROMPT,
    SELECT_KEY_FOR_PROMPT,
    SHOW_KEY_DETAILS,
    IMPORT_CERTS_PROMPT,
    IMPORT_KEYS_PROMPT,
    KEYS_CERTS_ADDED,
    DELETE_KEYS_PROMPT,
    DELETE_CERT_PROMPT,
    ADDRESSES_AND,
    KEYS_CERTS_REMOVED,
    EXPORT_KEYS_PROMPT,
    EXPORT_CERT_PROMPT,
    CERTIFICATE_CHAIN,
    NO_SIGNATURE,
    GOOD_SIG_INFO,
    BAD_SIG_INFO,
    EXPIRED_SIG_INFO,
    SIGNED_BY,
    NO_KEY,
    UNKNOWN_ALGORITHM,
    FOR_SIGNATURE,
    KEYSERVER_HINT,
    EXTRA_DATA_INFO,

    // PMHandler
    FIND_PM_PROMPT,
    TO_DELETE,
    SUBJECT_CANCELLED,
    MESSAGE_COPIED_INFO,
    FOLDER_FIND_ERROR,
    FOLDER_READ_ERROR,
    NO_FOLDER_DATA,
    ALL_PM_USERS,

    // File dialogs
    OPEN_FILE_DIALOG,
    SAVE_FILE_DIALOG,
    FROM_CLIPBOARD_BUTTON,
    TO_CLIPBOARD_BUTTON,
    FILE_CREATE_ERROR,
    FILE_OPEN_ERROR,
    FILE_FIND_ERROR,
    DATA_READ_ERROR,
    DATA_WRITE_ERROR,

    // MsgTreeDialog
    MSG_TREE_DIALOG,
    FORMATTING_SHEET,
    ORIENTATION_GROUP,
    PAGE_PORTRAIT,
    PAGE_LANDSCAPE,
    MARGINS_GROUP,
    MARGINS_HORZ_OPTION,
    MARGINS_VERT_OPTION,
    FONT_GROUP,
    FONT_NAME,
    FONT_SIZE,
    HEAD_FOOT_GROUP,
    PRINT_HEADER_OPTION,
    PRINT_FOOTER_OPTION,
    PRINT_MSG_HEADERS_OPTION,
    VIEW_ITEM,
    COPY_ITEM,
    SAVE_ITEM,
    SAVE_MSG_HINT,
    SAVE_MSG_FILE_HINT,
    SAVE_CERT_HINT,
    ADD_KEYS_HINT,
    SELECT_FILES_ITEM,
    CLOSE_ITEM,
    FILE_ITEM,
    ADVANCED_ITEM,
    DATA_SAVED_TO,

    // ImageViewer
    IMAGE_VIEWER,
    IMAGE_MENU_HINT,
    DECREASE_ITEM,
    INCREASE_ITEM,
    ROTATE_LEFT_ITEM,
    ROTATE_RIGHT_ITEM,
    PRINT_ITEM,
    INVALID_FILE_FORMAT,

    // TextViewer
    TEXT_VIEWER,
    TEXT_MENU_HINT,
    WRAP_HINT,
    UNWRAP_HINT,
    SORT_HINT,
    FIND_FIRST_HINT,
    FIND_NEXT_HINT,
    OPEN_FOLDER_HINT,
    LOAD_IMAGES_HINT,
    HIDE_IMAGES_HINT,
    PRINT_HINT,
    VIEW_HINT,
    SAVE_HINT,
    CLOSE_HINT,
    COPY_IMAGE,
    COPY_URL,
    COPY_DO,
    OPEN_URL,
    LOADING_FOLDER,
    LOADING_DATA,
    UPDATING_DISPLAY,
    MAIL_FOLDER,
    MAIL_MESSAGES,
    PRINT_PAGE_FOOTER,
    DEFAULT_CHARSET,
    MULTI_LINGUAL,
    RTF_FORMATTED,
    HTML_FORMATTED,
    INCLUDING_HEADERS,
    INCLUDING_IMAGES,
    SAVE_ENCRYPTED,
    PLAIN_TEXT_ONLY,
    REINDEX_OR_RESTART,

    // Decoding
    RAW_VIEW_HINT,
    IN_READER_WINDOW,
    SWITCH_NOW_CONFIRMATION,
    WORD_WRAP_HINT,
    DECODING_INFO,
    CANCELLED_INFO,
    FINISHED_INFO,
    MIME_CONVERSION_WARNING,
    MIME_CONVERSION_INFO,
    ATTACHMENT_HINT,
    DIGEST_HINT,
    CLICK_HERE_HINT,
    USE_MENU_OR_ATTACH_HINT,
    WORK_WITH_PARTS_HINT,
    ENCODED_SECTION,
    SECTION_BEGIN,
    SECTION_END,

    // Encoding
    ALL_IN_ONE_WARNING,
    MIX_OPTIONS_WARNING,
    SELECT_DIALOG,
    SEND_UNENCODED_CONFIRMATION,
    ADK_CONTINUE_CONFIRMATION,
    COPY_MSG_BUTTON,
    SEND_MSG_BUTTON,
    CONTINUE_BUTTON,
    ENCODING_INFO,
    OF_MESSAGE,
    OF_ATTACHMENT,
    ENCODE_ATTACHMENTS_CONFIRMATION,
    ENCRYPTION_SUCCESS_CONFIRMATION

  );

type
  TPMPGPStringList = Class(TStringList)
  private
    FCommentMarker: Char;
    function	GetPMPGPString(Index: TPMPGPStringIndex): String;
    procedure	SetPMPGPString(Index: TPMPGPStringIndex; const Value: String);
  protected
    procedure	SetTextStr(const Value: String); override;
  public
    constructor	Create(const FileName: String);
    destructor	Destroy; override;
    property	CommentMarker: Char read FCommentMarker write FCommentMarker;
    property 	Strings[Index: TPMPGPStringIndex]: String read GetPMPGPString write SetPMPGPString; default;
  end;

type
  TPMPGPStrings = Class(TPMPGPStringList)
  private
    function	GetKeyType(Index: TTypes): String;
    procedure	SetKeyTypeNames;
    function	GetKeyTrust(Index: TTrustLevel): String;
    procedure	SetKeyTrustLevels;
    function	GetKeyValidity(Index: TValidityLevel): String;
    procedure	SetKeyValidityLevels;
    function	GetKeyAlgorithm(Index: TKeyAlgorithm): String;
    procedure	SetKeyAlgorithmNames;
    function	GetHashAlgorithm(Index: THashAlgorithm): String;
    procedure	SetHashAlgorithmNames;
    function	GetChainValidity(Index: TX509ChainValidity): String;
    procedure	SetChainValidityLevels;
  protected
    procedure	CheckUserDefaultLanguage(const BaseFileName, LanguageCode: String);
  public
    KeyTypeNames: TKeyTypes;
    KeyTrustLevels: TKeyTrusts;
    KeyValidityLevels: TKeyValidities;
    KeyAlgorithmNames: TKeyAlgorithms;
    HashAlgorithmNames: THashAlgorithms;
    CertValidityLevels: TCertValidities;
    constructor	Create(const BaseFileName, LanguageCode: String);
    destructor	Destroy; override;
    property	KeyTypes[Index: TTypes]: String read GetKeyType;
    property	KeyTrusts[Index: TTrustLevel]: String read GetKeyTrust;
    property	KeyValidities[Index: TValidityLevel]: String read GetKeyValidity;
    property	KeyAlgorithms[Index: TKeyAlgorithm]: String read GetKeyAlgorithm;
    property	HashAlgorithms[Index: THashAlgorithm]: String read GetHashAlgorithm;
    property	ChainValidities[Index: TX509ChainValidity]: String read GetChainValidity;
  end;

type
  TPGPErrorStrings = Class(TStringList)
  private
    function	GetErrorString(ErrorIndex: Longint): String;
  public
    constructor	Create(const FileName: String);
    destructor	Destroy; override;
    property	PGPErrors[Index: Longint]: String read GetErrorString; default;
  end;

function  LoadIcon(Name: PChar): THandle;
procedure LoadHelpTopicList(TopicList: TTopicList);
function  RemoveSMimeString(const Text: String; SMimeDisabled: Longbool): String;


implementation


function LoadIcon(Name: PChar): THandle;
begin
  Result := LoadImage(hInstance, Name, IMAGE_ICON, 0, 0, LR_SHARED);
end;

procedure LoadHelpTopicList(TopicList: TTopicList);
begin
  if TopicList <> nil then with TopicList do begin
    // Dialogs
    Topics[ord(IDH_AUTOLIST_OPTIONS_DLG)] := 'AUTOLIST_OPTIONS_DLG.htm';
    Topics[ord(IDH_ENCRYPT_KEY_DLG)] := 'ENCRYPT_KEY_DLG.htm';
    Topics[ord(IDH_MULTI_ID_DLG)] := 'MULTI_ID_DLG.htm';
    // Menus
    Topics[ord(IDH_MAINMENU)] := 'MAINMENU.htm';
    Topics[ord(IDH_AUTOLIST)] := 'AUTOLIST.htm';
    // Configuration
    Topics[ord(IDH_CONFIGURATION_DLG)] := 'CONFIGURATION_DLG.htm';
  end;
end;

function RemoveSMimeString(const Text: String; SMimeDisabled: Longbool): String;
var
  iPos, iLen: Integer;
begin
  Result := Text;
  if SMimeDisabled then begin
    iPos := FirstLoStrPos(PMIME_INFO, Result) + Length(PMIME_INFO);
    iLen := FirstLoStrPos(SMIME_INFO, Result) + Length(SMIME_INFO) - iPos;
    Delete(Result, iPos, iLen);
  end;
end;


// TPMPGPStringList --------------------------------------------------------------------------------

constructor TPMPGPStringList.Create(const FileName: String);
begin
  inherited Create;
  FCommentMarker := DC;
  LoadFromFile(FileName);
end;

destructor TPMPGPStringList.Destroy;
begin
  inherited;
end;

procedure TPMPGPStringList.SetTextStr(const Value: String);
var
  pPosition, pStart: PChar;
  fQuoted: Longbool;
  sLine: String;
begin
  BeginUpdate;
  try
    Clear;
    pPosition := Pointer(Value);
    if pPosition <> nil then
      while pPosition^ <> NULL do begin
	while pPosition^ in [LF, CR] do inc(pPosition);
	if (FCommentMarker <> NULL) and (pPosition^ = FCommentMarker) then begin
	  while not (pPosition^ in [NULL, LF, CR]) do inc(pPosition);
	  Continue;
	end;
	fQuoted := (pPosition^ = QU);
	if fQuoted then inc(pPosition);
	pStart := pPosition;
	while not (pPosition^ in [NULL, LF, CR]) do inc(pPosition);
	if fQuoted then begin
	  repeat
	    if PChar(DWord(pPosition) - 1)^ = QU then begin
	      SetString(sLine, pStart, pred(pPosition - pStart));
	      Break;
	    end
	    else begin
	      if pPosition^ = NULL then begin
		SetString(sLine, pStart, pPosition - pStart);
		Break;
	      end
	      else begin
		while (pPosition^ in [LF, CR]) do inc(pPosition);
		while not (pPosition^ in [NULL, LF, CR]) do inc(pPosition);
	      end;
	    end;
	  until pPosition = nil;
	end
	else SetString(sLine, pStart, pPosition - pStart);
	Add(sLine);
      end;
  finally
    EndUpdate;
  end;
end;

function TPMPGPStringList.GetPMPGPString(Index: TPMPGPStringIndex): String;
begin
  if ord(Index) < Count then Result := inherited Strings[ord(Index)];
end;

procedure TPMPGPStringList.SetPMPGPString(Index: TPMPGPStringIndex; const Value: String);
begin
  inherited Strings[ord(Index)] := Value;
end;


// TPMPGPStrings -----------------------------------------------------------------------------------

constructor TPMPGPStrings.Create(const BaseFileName, LanguageCode: String);
begin
  inherited Create(Format(BaseFileName, [LanguageCode]));
  CheckUserDefaultLanguage(BaseFileName, LanguageCode);
  SetKeyTypeNames;
  SetKeyTrustLevels;
  SetKeyValidityLevels;
  SetKeyAlgorithmNames;
  SetHashAlgorithmNames;
  SetChainValidityLevels;
end;

destructor TPMPGPStrings.Destroy;
begin
  inherited;
end;

procedure TPMPGPStrings.CheckUserDefaultLanguage(const BaseFileName, LanguageCode: String);
var
  sWinUserDefaultTLD, sDefaultFileName: String;
  slPMPGPDefault: TPMPGPStringList;
begin
  sWinUserDefaultTLD := GetWinUserDefaultTLD;
  sDefaultFileName := Format(BaseFileName, [sWinUserDefaultTLD]);
  if (sWinUserDefaultTLD <> E) and (CompareText(sWinUserDefaultTLD, LanguageCode) <> 0)
  and FileExists(sDefaultFileName) then begin
    slPMPGPDefault := TPMPGPStringList.Create(sDefaultFileName);
    if slPMPGPDefault <> nil then begin
      try
	Strings[KEYFILE_DIALOG] := slPMPGPDefault[KEYFILE_DIALOG];
	Strings[FIND_PM_PROMPT] := slPMPGPDefault[FIND_PM_PROMPT];
	Strings[OPEN_FILE_DIALOG] := slPMPGPDefault[OPEN_FILE_DIALOG];
	Strings[SAVE_FILE_DIALOG] := slPMPGPDefault[SAVE_FILE_DIALOG];
	Strings[FROM_CLIPBOARD_BUTTON] := slPMPGPDefault[FROM_CLIPBOARD_BUTTON];
	Strings[TO_CLIPBOARD_BUTTON] := slPMPGPDefault[TO_CLIPBOARD_BUTTON];
	Strings[FORMATTING_SHEET] := slPMPGPDefault[FORMATTING_SHEET];
	Strings[ORIENTATION_GROUP] := slPMPGPDefault[ORIENTATION_GROUP];
	Strings[PAGE_PORTRAIT] := slPMPGPDefault[PAGE_PORTRAIT];
	Strings[PAGE_LANDSCAPE] := slPMPGPDefault[PAGE_LANDSCAPE];
	Strings[MARGINS_GROUP] := slPMPGPDefault[MARGINS_GROUP];
	Strings[MARGINS_HORZ_OPTION] := slPMPGPDefault[MARGINS_HORZ_OPTION];
	Strings[MARGINS_VERT_OPTION] := slPMPGPDefault[MARGINS_VERT_OPTION];
	Strings[FONT_GROUP] := slPMPGPDefault[FONT_GROUP];
	Strings[FONT_NAME] := slPMPGPDefault[FONT_NAME];
	Strings[FONT_SIZE] := slPMPGPDefault[FONT_SIZE];
	Strings[HEAD_FOOT_GROUP] := slPMPGPDefault[HEAD_FOOT_GROUP];
	Strings[PRINT_HEADER_OPTION] := slPMPGPDefault[PRINT_HEADER_OPTION];
	Strings[PRINT_FOOTER_OPTION] := slPMPGPDefault[PRINT_FOOTER_OPTION];
	Strings[PRINT_MSG_HEADERS_OPTION] := slPMPGPDefault[PRINT_MSG_HEADERS_OPTION];
      finally
	slPMPGPDefault.Free;
      end;
    end;
  end;
end;

function TPMPGPStrings.GetKeyType(Index: TTypes): String;
begin
  Result := inherited Strings[TPMPGPStringIndex(ord(KEYTYPE_PUBLIC) + ord(Index))];
end;

procedure TPMPGPStrings.SetKeyTypeNames;
var
  iIndex: TTypes;
begin
  for iIndex := Low(TTypes) to High(TTypes) do begin
    KeyTypeNames[iIndex] := GetKeyType(iIndex);
  end;
end;

function TPMPGPStrings.GetKeyTrust(Index: TTrustLevel): String;
begin
  Result := inherited Strings[TPMPGPStringIndex(ord(TRUST_UNDEFINED) + ord(Index))];
end;

procedure TPMPGPStrings.SetKeyTrustLevels;
var
  iIndex: TTrustLevel;
begin
  for iIndex := Low(TTrustLevel) to High(TTrustLevel) do begin
    KeyTrustLevels[iIndex] := GetKeyTrust(iIndex);
  end;
end;

function TPMPGPStrings.GetKeyValidity(Index: TValidityLevel): String;
begin
  Result := inherited Strings[TPMPGPStringIndex(ord(VALIDITY_UNKNOWN) + ord(Index))];
end;

procedure TPMPGPStrings.SetKeyValidityLevels;
var
  iIndex: TValidityLevel;
begin
  for iIndex := Low(TValidityLevel) to High(TValidityLevel) do begin
    KeyValidityLevels[iIndex] := GetKeyValidity(iIndex);
  end;
end;

function TPMPGPStrings.GetKeyAlgorithm(Index: TKeyAlgorithm): String;
begin
  if Index < KeyAlgorithm_Reserved01 then
    Result := inherited Strings[TPMPGPStringIndex(ord(ALGORITHM_INVALID) + ord(Index) - ord(KeyAlgorithm_Invalid))]
  else if Index > KeyAlgorithm_Reserved12 then
    Result := inherited Strings[TPMPGPStringIndex(ord(ALGORITHM_DH) + ord(Index) - ord(KeyAlgorithm_DH))]
  else Result := E;
end;

procedure TPMPGPStrings.SetKeyAlgorithmNames;
var
  iIndex: TKeyAlgorithm;
begin
  for iIndex := Low(TKeyAlgorithm) to pred(KeyAlgorithm_Reserved01) do begin
    KeyAlgorithmNames[iIndex] := GetKeyAlgorithm(iIndex);
  end;
  for iIndex := succ(KeyAlgorithm_Reserved12) to High(TKeyAlgorithm) do begin
    KeyAlgorithmNames[iIndex] := GetKeyAlgorithm(iIndex);
  end;
end;

function TPMPGPStrings.GetHashAlgorithm(Index: THashAlgorithm): String;
begin
  Result := inherited Strings[TPMPGPStringIndex(ord(HASH_INVALID) + ord(Index))];
end;

procedure TPMPGPStrings.SetHashAlgorithmNames;
var
  iIndex: THashAlgorithm;
begin
  for iIndex := Low(THashAlgorithm) to High(THashAlgorithm) do begin
    HashAlgorithmNames[iIndex] := GetHashAlgorithm(iIndex);
  end;
end;

function TPMPGPStrings.GetChainValidity(Index: TX509ChainValidity): String;
begin
  Result := inherited Strings[TPMPGPStringIndex(ord(CHAIN_INCOMPLETE) + ord(Index))];
end;

procedure TPMPGPStrings.SetChainValidityLevels;
var
  iIndex: TX509ChainValidity;
begin
  for iIndex := Low(TX509ChainValidity) to High(TX509ChainValidity) do begin
    CertValidityLevels[iIndex] := GetChainValidity(iIndex);
  end;
end;


// TPGPErrorStrings --------------------------------------------------------------------------------

constructor TPGPErrorStrings.Create(const FileName: String);
begin
  inherited Create;
  LoadFromFile(FileName);
end;

destructor TPGPErrorStrings.Destroy;
begin
  inherited;
end;

function TPGPErrorStrings.GetErrorString(ErrorIndex: Longint): String;
var
  iIndex: Longint;
begin
  iIndex:= ErrorIndex - kPGPError_BadParams;
  if (iIndex >= 0) and (iIndex < Count) then
    Result := Strings[iIndex]
  else Result := DC + IntToHex(ErrorIndex, 8);
end;


end.

