{------------------------------------------------------------------------------}
{        This code is copyright (C) 2009-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}
{$J+,Z4}
unit PasswordDialog;

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, Graphics, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Buttons,
  ExtCtrlsEx, CustBtnCtrl, CustCheck, FlatBtn, MaskEdEx, UxThemes, PMPGPForm, PMPGPCommon, PMPGPResources;

type
  TPasswordDialog = class(TPMPGPForm)
    pnTop: TPanel;
    pnOptions: TPanel;
    cbDisplay: TCustCheckBox;
    sePassword: TSecureEdit;
    pnLeft: TPanel;
    pnRight: TPanel;
    pnButtons: TPanel;
    fbOk: TFlatBtn;
    fbCancel: TFlatBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure cbDisplayChanged(Sender: TObject);
  private
  protected
    procedure SetColors(Default: Longbool); override;
  public
  end;

function GetPassword(Password: PWideChar; MaxLength: Integer): Integer;


implementation

uses
  PMPGPMainClass,
  MessageDialogs;

{$R *.DFM}

function GetPassword(Password: PWideChar; MaxLength: Integer): Integer;
var
  frmPasswordDlg: TPasswordDialog;
begin
  Result := 0;
  with MainClass do frmPasswordDlg := TPasswordDialog.Create(MainClass);
  if frmPasswordDlg <> nil then with frmPasswordDlg do begin
    try
      sePassword.SetTextBuffer(Password, MaxLength);
      ActiveControl := sePassword;
      ShowModal;
      case ModalResult of
	mrOk: Result := sePassword.TextLength;
	mrCancel: FillChar(Password^, MaxLength shl 1, 0);
      end;
    finally
      Release;
    end;
  end;
end;

procedure TPasswordDialog.FormCreate(Sender: TObject);
begin
  //Caption := MainClass.PMPGPStrings[PASSWORD_DIALOG];
  //cbDisplay.Caption := PMPGPStrings[SHOW_PASSWORD];
  fbOk.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_OK);
  fbCancel.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CANCEL);
end;

procedure TPasswordDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN: fbOk.Click;
    VK_ESCAPE: fbCancel.Click;
  end;
end;

procedure TPasswordDialog.cbDisplayChanged(Sender: TObject);
begin
  with sePassword do begin
    if PasswordChar = #0 then
      PasswordChar := #32
    else PasswordChar := #0;
  end;
end;

procedure TPasswordDialog.SetColors(Default: Longbool);
begin
  inherited;
  sePassword.Color := AWindowColors[Default];
end;


end.

