{$J+,Z4}
unit SearchDialog;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes, SysUtils, StrUtil, Messages, Graphics, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Buttons,
  ExtCtrlsEx, CustBtnCtrl, CustCheck, FlatBtn, DropEdit, DropGrid, UxThemes, PMPGPForm, PMPGPCommon, PMPGPResources;

type
  TSearchType = (stDown, stWholeWord, stMatchCase, stAnsiChar, stShowOptions);
  TSearchTypes = Set of TSearchType;
  TSearchDialog = class(TPMPGPForm)
    pnTop: TPanel;
    pnOptions: TPanel;
    cbCase: TCustCheckBox;
    cbWord: TCustCheckBox;
    cbUpwards: TCustCheckBox;
    pnLeft: TPanel;
    pnEdit: TColoredFramePanel;
    deAnswer: TDropEdit;
    pnRight: TPanel;
    pnButtons: TPanel;
    fbOk: TFlatBtn;
    fbCancel: TFlatBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure deAnswerEnter(Sender: TObject);
    procedure cbChanged(Sender: TObject);
    procedure cbExit(Sender: TObject);
  private
    Editing: Longbool;
    Changed: Longbool;
  protected
    procedure SetColors(Default: Longbool); override;
  public
  end;

function EditSearchText(const Find, FontName: String; var How: DropEdit.TSearchTypes): String;
function GridSearchText(const Find, FontName: String; var How: DropGrid.TSearchTypes): String;


implementation

uses
  PMPGPMainClass,
  MessageDialogs;

{$R *.DFM}

function SearchText(const Find, FontName: String; var How: TSearchTypes): String;
var
  frmSearchDlg: TSearchDialog;
begin
  with MainClass do frmSearchDlg := TSearchDialog.Create(MainClass);
  if frmSearchDlg <> nil then with frmSearchDlg do begin
    try
      //deAnswer.Font.Name := FontName;
      with deAnswer do begin
	SearchOptions := DropEdit.TSearchTypes(How);
	Text := Find;
      end;
      if stShowOptions in How then begin
	cbCase.Checked := (stMatchCase in How);
	cbWord.Checked := (stWholeWord in How);
	cbUpwards.Checked := not (stDown in How);
      end;
      ActiveControl := deAnswer;
      ShowModal;
      case ModalResult of
	mrOk: begin
	  if stShowOptions in How then begin
	    case cbCase.Checked of
	      false: Exclude(How, stMatchCase);
	      true: Include(How, stMatchCase);
	    end;
	    case cbWord.Checked of
	      false: Exclude(How, stWholeWord);
	      true: Include(How, stWholeWord);
	    end;
	    case cbUpwards.Checked of
	      false: Include(How, stDown);
	      true: Exclude(How, stDown);
	    end;
	  end;
	  if stAnsiChar in How then
	    Result := deAnswer.AnsiText
	  else Result := deAnswer.Text;
	end;
	mrCancel: Result := E;
      end;
    finally
      Release;
    end;
  end;
end;

function EditSearchText(const Find, FontName: String; var How: DropEdit.TSearchTypes): String;
begin
  Result := SearchText(Find, FontName, TSearchTypes(How));
end;

function GridSearchText(const Find, FontName: String; var How: DropGrid.TSearchTypes): String;
begin
  Result := SearchText(Find, FontName, TSearchTypes(How));
end;

procedure TSearchDialog.FormCreate(Sender: TObject);
begin
  Caption := MainClass.PMPGPStrings[SEARCH_DIALOG];
  fbOk.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_OK);
  fbCancel.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CANCEL);
end;

procedure TSearchDialog.FormShow(Sender: TObject);
begin
  pnOptions.Visible := DropEdit.stShowOptions in deAnswer.SearchOptions;
  if pnOptions.Visible then with MainClass do begin
    cbCase.Caption := PMPGPStrings[CASE_SENSITIVE_OPTION];
    cbWord.Caption := PMPGPStrings[WHOLE_WORDS_OPTION];
    cbUpwards.Caption := PMPGPStrings[UPWARDS_OPTION];
  end;
end;

procedure TSearchDialog.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN: fbOk.Click;
    VK_ESCAPE: fbCancel.Click;
    VK_SPACE: Editing := false;
    ord('A'): if Shift = [ssCtrl] then begin
      deAnswer.SelectAll;
      RemoveKeyMessage;
    end;
  end;
end;

procedure TSearchDialog.deAnswerEnter(Sender: TObject);
begin
  if not Changed then deAnswer.SelectAll;
  Changed := false;
  Editing := true;
end;

procedure TSearchDialog.cbChanged(Sender: TObject);
begin
  if Editing then begin
    Changed := true;
    deAnswer.SetFocus;
  end;
end;

procedure TSearchDialog.cbExit(Sender: TObject);
begin
  Changed := true;
end;

procedure TSearchDialog.SetColors(Default: Longbool);
begin
  inherited;
  deAnswer.Color := AWindowColors[Default];
end;


end.

