{$J+,Z4}
unit TERHandler;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, SysUtils, StrUtil, RichEdit, RichEd20, Graphics, HtsTer,
  Utf8, MailUtils, WindowUtils, PMMessages, PMOptions, PMPGPCommon, PMPGPResources;

const
  IE_CLASS	= 'INTERNET EXPLORER_SERVER';
  BEAR_CLASS	= 'THTMLVIEWER';
  FONT_FIXED	= 'Lucida Console';
  PMPGP_URL	= PMPGP + '_URL:';
  MAX_WIDTH	= 2000;


// TerHandler Section ----------------------------------------------------------

type
  TMessageType = (mtRead, mtReply, mtForward);
  TReplyHeaderOption = (rhoOriginal, rhoCustom);
  TReplyHeaderOptions = Set of TReplyHeaderOption;

  TDisplayInfo = Record
    MessageType: TMessageType;
    IsNewReader: Longbool;
    DefaultFont: TPMPGPFont;
    AttachmentHint: String;
    DigestHint: String;
    ClickHereHint: String;
    UseAttachHint: String;
    WorkWithPartsHint: String;
    EncodedBegin: String;
    EncodedEnd: String;
    ForwardedBegin: String;
    ForwardedEnd: String;
    Signature: String;
    EditorTabWidthTwips: Integer;
    OriginalMessageSeparator: String;
    ReplyHeaderOptions: TReplyHeaderOptions;
    CustomHeaderBlankLines: Integer;
  end;

  TTERInterface = Class
  private
    FTerVersion: String;
    FTerWindow: THandle;
    FHtsTerConverter: THtsTerConverter;
    HtsInitialize: function(hTerWnd, hParentWnd: THandle): Bool; stdcall;
    HtsClearWindow: function(hTerWnd: THandle): Bool; stdcall;
    HtsClose: function(hTerWnd: THandle; OutputTo: Integer; FileName: PChar;
		     var hBuf: THandle; var BufLen: Integer): Bool; stdcall;
    HtsRead: function(hTerWnd: THandle; InputType: Integer; FileName: PChar;
		      hBuf: THandle; BufLen: Integer; TagName: PChar): Bool; stdcall;
    HtsSave: function(hTerWnd: THandle; OutputTo: Integer; FileName: PChar;
		      var hBuf: THandle; var BufLen: Integer): Bool; stdcall;
    HtsSetPicture: function(hTerWnd: THandle; PictId, PictType: Integer; PictFile: PChar): Bool; stdcall;
    TERCreateWindowAlt: function(Left, Top, Width, Height: Integer; IsContained: Bool;
				 ParentWnd: THandle; Styles: Cardinal): THandle; stdcall;
    CloseTer: function(hTerWnd: THandle; ForcedClose: Bool): Bool; stdcall;
    TerPrint: function(hWnd: THandle; Dialog: Longbool): Bool; stdcall;
    TerIsPrinting: function(hWnd: THandle): Bool; stdcall;
    TerCommand: function(hWnd: THandle; CommandId: Integer): Bool; stdcall;
    GetTerFields: function(hTerWnd: THandle; var Field: TStrTerField): Bool; stdcall;
    SetTerFields: function(hTerWnd: THandle; const Field: TStrTerField): Bool; stdcall;
    TERGetReadOnly: function(hTerWnd: THandle): Bool; stdcall;
    TERSetFlags: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags2: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags3: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags4: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags5: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags6: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags7: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags8: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags9: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags10: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERSetFlags11: function(hTerWnd: THandle; SetFlag: Bool; FlagValue: DWord): DWord; stdcall;
    TERGetCurrentFont: function(hTerWnd: THandle; iLine, iColumn: Integer): Integer; stdcall;
    TERSetDefaultFont: function(hTerWnd: THandle; fnTypeFace: PChar; iPointSize: Integer;
				dwStyles: DWord; crColor: TColorRef; fRepaint: Bool): Bool; stdcall;
    TERSetFont: function(hTerWnd: THandle; fnTypeFace: PChar; fRepaint: Bool): Bool; stdcall;
    TERGetFontInfo: function(hTerWnd: THandle; iFontId: Integer; fnTypeFace: PChar;
			     var iPointSize: Integer; var dwStyles: DWord): Bool; stdcall;
    TERSetFontSize: function(hTerWnd: THandle; iPointSize: Integer; fRepaint: Bool): Bool; stdcall;
    TERGetParaInfo: function(hTerWnd: THandle; iLine: Integer;
			     var iLeftIndent, iRightIndent, iFirstIndent, iTabId, iStyleId, iAuxId: Integer;
			     var iShading, iPFlags, iSpaceBefore, iSpaceAfter, iSpaceBetween: Integer;
			     var dwStyles: DWord): Bool; stdcall;
    TERSetParaStyles: function(hTerWnd: THandle; wStyles: Word; fOnOff, fRepaint: Bool): Bool; stdcall;
    TERResetParagraph: function(hTerWnd: THandle; fRepaint: Bool): Bool; stdcall;
    TERClearTabStop: function(hTerWnd: THandle; iTabPos: Integer; fRepaint: Bool): Bool; stdcall;
    TERGetTabStop: function(hTerWnd: THandle; iLine, iIndex: Integer;
			   var iTabPos, iTabType, iTabLeader: Integer): Integer; stdcall;
    TERSetTabStop: function(hTerWnd: THandle; iTabType, iTabPos, iTabLeader: Integer; fRepaint: Bool): Bool; stdcall;
    TerSetDefTabWidth: function(hTerWnd: THandle; iWidth: Integer; fRepaint: Bool): Integer; stdcall;
    TERGetSelectedRtf: function (hTerWnd: THandle; var iSize: Integer): THandle; stdcall;
    TERGetSelectedText: function(hTerWnd: THandle; var iTextLen: Integer): THandle; stdcall;
    TERGetSelectedTextU: function(hTerWnd: THandle; var iTextLen: Integer): THandle; stdcall;
    TERInsertRtf: function(hTerWnd: THandle; pRtf: PChar; iLength, iLine, iColumn: Integer; fRepaint: Bool): Bool; stdcall;
    TERInsertText: function(hTerWnd: THandle; pText: PChar; iFont, iPara: Integer; fRepaint: Bool): Bool; stdcall;
    TERInsertTextU: function(hTerWnd: THandle; pText: PWideChar; iFont, iPara: Integer; fRepaint: Bool): Bool; stdcall;
    TERGetText: function(hTerWnd: THandle; var iTextLen: Integer): THandle; stdcall;
    TERSetText: function(hTerWnd: THandle; hText: THandle; iTextLen: Integer; pTitle: PChar; fRelease: Bool): Bool; stdcall;
    TERDeleteText: function(hTerWnd: THandle; fRepaint: Bool): Bool; stdcall;
    TERGetLine: function(hTerWnd: THandle; iLine: Integer; pTextLine: PChar; pFont: Pointer): Integer; stdcall;
    TERGetLineU: function(hTerWnd: THandle; iLine: Integer; pTextLine: PWideChar; pFont: Pointer): Integer; stdcall;
    TERSetLine: function(hTerWnd: THandle; iLine: Integer; pTextLine: PChar; pFont: Pointer): Bool; stdcall;
    TERPictLinkName: function(hTerWnd: THandle; ID: Integer; Get: Bool; FileName: PChar): Bool; stdcall;
    TERFindHlinkField: function(hTerWnd: THandle; pUrlPart1, pUrlPart2: PChar; var iLine, iColumn: Integer): Bool; stdcall;
    TERInsertHyperlink: function(hTerWnd: THandle; pText, pCode: PChar; iPictID: Integer; fRepaint: Bool): Integer; stdcall;
    TERInsertHyperlinkU: function(hTerWnd: THandle; pText, pCode: PWideChar; iPictID: Integer; fRepaint: Bool): Integer; stdcall;
    TERLocateFieldChar: function(hTerWnd: THandle; iFieldId: Integer; FieldCode: PChar; fPresent: Bool;
				 var StartLine: Integer; var StartCol: Integer; fForward: Bool): Bool; stdcall;
    TERInsertDrawObject: function(hTerWnd: THandle; iType, iXPos, iYPos, iWidth, iHeight: Integer): Integer; stdcall;
    TERCreateTable: function(hTerWnd: THandle; iRow, iColumn: Integer; fRepaint: Bool): Bool; stdcall;
    TERCellBorder: function(hTerWnd: THandle; iSelection, iTopWidth, iBotWidth,
			    iLeftWidth, iRightWidth: Integer; fRepaint: Bool): Bool; stdcall;
    TERCellWidth: function(hTerWnd: THandle; iSelection, iWidth, iMargin: Integer; fRepaint: Bool): Bool; stdcall;
    TERSetTableColWidth: function(hTerWnd: THandle; iWidth: Integer; fRepaint: Bool): Bool; stdcall;
    TERGetMarginEx: function(hTerWnd: THandle; iSection: Integer; piLeft, piRight, piTop, piBottom,
			     piHeader, piFooter: PInteger): Integer; stdcall;
    TERGetCursorPos: function(hTerWnd: THandle; var iLine, iColumn: Integer): Bool; stdcall;
    TERSetCursorPos: function(hTerWnd: THandle; iLine, iColumn: Integer; fRepaint: Bool): Bool; stdcall;
    TERGetTextColors: function(hTerWnd: THandle; iFont: Integer; var dwTextColor, dwTextBKColor: DWord): Bool; stdcall;
    TERSetTextColor: function(hTerWnd: THandle; dwTextColor: DWord; fRepaint: Bool): Bool; stdcall;
    TERSetTextBKColor: function(hTerWnd: THandle; dwTextBkColor: DWord; fRepaint: Bool): Bool; stdcall;
    TERSetPageBkColor: function(hTerWnd: THandle; dwPageBkColor: DWord): Bool; stdcall;
    TERTextPosToPix: function(hTerWnd: THandle; iRelativeTo, iLine, iColumn: Integer;
			      var lXPos, lYPos: Longint): Bool; stdcall;
    TERPixToTextPos: function(hTerWnd: THandle; iRelativeTo: Integer; lXPos, lYPos: Longint;
			      var iLine, iColumn: Integer): Bool; stdcall;
    TEREngageCaret: function(hTerWnd: THandle; AtCursor: Bool): Bool; stdcall;
    TERGetCaretPos: function(hTerWnd: THandle): Longint; stdcall;
    TERSetCaretPos: function(hTerWnd: THandle; iPos: Longint): Bool; stdcall;
    TERPosToRowCol: procedure(hTerWnd: THandle; lPos: Longint; var iLine, iColumn: Integer); stdcall;
    TERRowColToPos: function(hTerWnd: THandle; iLine, iColumn: Integer): Longint; stdcall;
    TERLineHeight: function(hTerWnd: THandle; iLine: Integer): Integer; stdcall;
    TERSearchReplace: function(hTerWnd: THandle; pSearch, pReplace: PChar; lFlags, lStartPos: Longint;
			       var lEndPos, lBufSize: Longint): Bool; stdcall;
    TERSearchReplaceU: function(hTerWnd: THandle; pSearch, pReplace: PChar; lFlags, lStartPos: Longint;
				var lEndPos, lBufSize: Longint): Bool; stdcall;
    TERSelect: function(hTerWnd: THandle; iFirstLine, iFirstCol, iLastLine, iLastCol: Integer; fRepaint: Bool): Bool; stdcall;
    TERDeselect: function(hTerWnd: THandle; fRepaint: Bool): Bool; stdcall;
    TERGetSelection: function(hTerWnd: THandle; var iFirstLine, iFirstCol, iLastLine, iLastCol: Integer): Bool; stdcall;
    TEREnableRefresh: function(hTerWnd: THandle; fEnable: Bool): Bool; stdcall;
    TERNormalizeBlock: function(hTerWnd: THandle): Bool; stdcall;
    TERRepaint: function(hTerWnd: THandle; fClearBackground: Bool): Bool; stdcall;
    {$HINTS OFF}
    function	GetCurrentLine: Integer;
    function	GetCursorPos(var Pos: Integer): Longbool;
    function	SetCursorPos(Pos: Integer): Longbool;
    function	GetCurrentFont: Integer;
    function	SetDefaultFont(const Font: TPMPGPFont; const Name: String): Longbool;
    function	GetFontFace(FontID: Integer): String;
    function	SetFontFace(const FontFace: String): Longbool;
    function	GetFontSize(FontID: Integer): Integer;
    function	SetFontSize(Points: Integer): Longbool;
    function	GetParaStyles(var ParaStyles: DWord): Longbool;
    function	SetParaStyles(ParaStyles: DWord): Longbool;
    function	GetTextColor(var TextColor: DWord): Longbool;
    function	SetTextColor(TextColor: DWord): Longbool;
    function	GetTextBkColor(var TextBkColor: DWord): Longbool;
    function	SetTextBkColor(TextBkColor: DWord): Longbool;
    function	SetPageBkColor(PageBkColor: DWord): Longbool;
    function	Get0x0TabPosAndLeader(var TabPos: Integer; var Leader: Integer): Longbool;
    function	Set0x0TabPosAndLeader(TabPos, Leader: Integer): Longbool;
    function	SetFirstTabInCurrentLine(TabPos: Integer): Integer;
    function	FindTextA(const Text: String; StartPos: Longint; Backwards: Longbool): Longint;
    function	FindTextW(const Text: String; StartPos: Longint; Backwards: Longbool): Longint;
    function	ReplaceTextA(const Text: String; StartPos, EndPos: Longint): Longint;
    function	ReplaceTextW(const Text: String; StartPos, EndPos: Longint): Longint;
    function	SelectAll(var BegPos, EndPos: Integer; DoSelect: Longbool): Longbool;
    function	SelectText(BegPos, EndPos: Integer): Longbool;
    function	DeleteAll: Longbool;
    function	DeleteText(BegPos, EndPos: Integer): Longbool;
    function	GetAnsiLine(Line: Integer): String;
    function	GetUnicodeLine(Line: Integer): String;
    function	GetAnsiText(Selection, ClearScreen: Longbool): String;
    function	GetRtfText(Selection, ClearScreen: Longbool): String;
    function	GetUnicodeText(Selection, ClearScreen: Longbool): String;
    function	SetAnsiText(const Text: String; Redraw: Longbool): Longbool;
    function	SetRtfText(const Text: String; Redraw: Longbool): Longbool;
    function	SetTerText(const Text: String): Longbool;
    function	SetUnicodeText(const Text: String; Redraw: Longbool): Longbool;
    function	FindHyperlink(const Url: String; var Line, Column: Integer): Longbool;
    function	SetHyperlink(const Text, Url: String; Redraw: Longbool): Longbool;
    function	GetTerClientWidth: Integer;
    function	DrawHorizontalLine: Longbool;
    procedure	DisableTerGridLines(Disable: Longbool);
    procedure	DisableWebAccess(Disable: Longbool);
    procedure	EnableScreenWrap(Enable: Longbool);
    procedure	EnableRefresh(Enable: Longbool);
    procedure	Repaint(Clear: Longbool);
    {$HINTS ON}
  protected
  public
    constructor	Create(const LibraryPath: String; CheckVersions: Longbool);
    destructor Destroy; override;
    procedure Print(Window: THandle; Dialog: Longbool);
    function IsPrinting(Window: THandle): Longbool;
    function GetTextSize(Window: THandle): Integer;
    function GetSelStart(Window: THandle): Integer;
    function GetSelLength(Window: THandle): Integer;
    function GetTerFlags(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags2(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags2(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags3(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags3(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags4(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags4(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags5(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags5(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags6(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags6(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags7(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags7(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags8(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags8(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags9(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags9(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags10(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags10(Window: THandle; FlagValue: Cardinal): Cardinal;
    function GetTerFlags11(Window: THandle; FlagValue: Cardinal): Cardinal;
    function SetTerFlags11(Window: THandle; FlagValue: Cardinal): Cardinal;
    function FindBearOrTerWindow(Parent: THandle; NewReader: Longbool): THandle;
    function FindTerWindow(Parent: THandle): THandle;
    function SetCaretPos(TerWnd: THandle; Pos: Integer): Longbool;
    function GetCaretScreenPos(TerWnd: THandle): TPoint;
    function SetCaretAtCursorPos(TerWnd: THandle): TPoint;
    function GetCurrentLineHeight(TerWnd: THandle): Integer;
    function GetFirstTabPos(TerWnd: THandle): Integer;
    function GetLineFromPos(TerWnd: THandle; Pos: Integer): Integer;
    function GetPosFromLine(TerWnd: THandle; Line: Integer): Integer;
    function GetKeyData(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree; SMimeCapable: Longbool): String;
    function GetEncodedData(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree; SMimeCapable: Longbool): String;
    function GetAddressOrHexID(TerWnd: THandle; var HexID: Longbool): String;
    function GetAnsiLineText(TerWnd: THandle; Line: Integer): String;
    function GetUnicodeLineText(TerWnd: THandle; Line: Integer): String;
    function GetAnsiLineTextAtCursor(TerWnd: THandle): String;
    function GetUnicodeLineTextAtCursor(TerWnd: THandle): String;
    function GetUrl(HyperlinkData: LPARAM): String;
    function DecodeUrl(const Url: String): String;
    function InsertTextVariant(TerWnd: THandle; const Text: String; const RtfFont: TRtfFont;
			       const FontName: String; TextColor, Position: Integer): Longbool;
    function InsertMessageBody(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree;
			       const DisplayInfo: TDisplayInfo): Longbool;
    function InsertVerificationInfo(TerWnd: THandle; const Text: String;
				    Flags: Integer; TabPos: Integer;
				    const DisplayInfo: TDisplayInfo;
				    NewReader: Longbool): Longbool;
    function IsHyperlinkAtCursor(const Text: String): Longbool;
    function IsHyperlinkFormat(const Text: String): Longbool;
    function IsBearClass(const ClassName: String): Longbool;
    function IsBearWindow(Window: THandle): Longbool;
    function IsReaderBlank(TerWnd: THandle): Longbool;
    function IsTerClass(const ClassName: String): Longbool;
    function IsTerReader(Window: THandle): Longbool;
    function IsTerWindow(Window: THandle): Longbool;
    function MessageClear(TerWnd: THandle): Longbool;
    function RtfReadMessage(TerWnd: THandle): String;
    function RtfWriteMessage(TerWnd: THandle; const Text: String): Longbool;
    function HtmlToRtf(const Html: String; MaxWidth: Integer; var TextHeight: Integer): String;
    function RtfToHtml(const Rtf: String; MaxWidth: Integer; var TextHeight: Integer): String;
    procedure SetHyperlinkFocus(TerWnd: THandle);
  end;


implementation

uses
  MessageDialogs;


const
  HTML_FILE = 0;
  HTML_BUF = 1;

  HTS_LIB = 'HTS??.DLL';
  HTS_32  = 'HTS32.DLL';

  TER_LIB = 'TER??.DLL';
  TER_32  = 'TER32.DLL';

  DEFAULT_BMP   = 'DEFAULT.BMP';

  TER_VER_CLASS = 'TER%sCLASS';

  TER_XX_CLASS  = 'TER??CLASS';

  TER_DH_CLASS  = 'TERDHCLASS';

  HTSTER_CLASS  = 'HTSTER';

  COLOR_GRAYTEXT = clSilver;


// TTERInterface -----------------------------------------------------------------------------------

const // font styles
  ULINE				= $001;
  BOLD				= $002;
  ITALIC			= $004;
  STRIKE			= $008;

const // para styles
  CENTER			= $001;
  RIGHT_JUSTIFY			= $002;
  LEFT_JUSTIFY			= $400;
  JUSTIFY			= $800;

const // tab align
  TAB_LEFT			= 0;
  TAB_RIGHT			= 1;
  TAB_CENTER			= 2;
  TAB_DECIMAL			= 3;

const // tab leaders
  TAB_NONE			= 0;
  TAB_DOT			= 1;
  TAB_HYPH			= 2;
  TAB_ULINE			= 4;

const // draw objects
  DOB_TEXT_BOX			= 1;
  DOB_RECT			= 2;
  DOB_LINE			= 3;

const // cell pos
  POS_BEG                       = 0;
  POS_END                       = 1;

const // pixel positions
  REL_SCREEN			= 0;
  REL_WINDOW			= 1;
  REL_TEXT_BOX			= 2;
  REL_TEXT_ORIG			= 3;

const // global sections
   SECT_ALL			= -1;
   SECT_CUR			= -2;

const // cell selections
  SEL_CELLS     		= 1;
  SEL_COLS      		= 2;
  SEL_ROWS      		= 3;
  SEL_ALL       		= 4;

const // search/replace flags
  SRCH_SCROLL			= $001;
  SRCH_SEARCH			= $002;
  SRCH_REPLACE			= $004;
  SRCH_RETRIEVE			= $008;
  SRCH_CASE			= $010;
  SRCH_WORD			= $020;
  SRCH_SKIP_HIDDEN_TEXT		= $040;
  SRCH_BACK			= $080;
  SRCH_NO_REPLACE_PROT_TEXT	= $100;

const // search expressions
  TER_TAB = '^t';
  TER_CRLF = '^p';
  TER_PARA = TER_CRLF + TER_CRLF;


type
  PTerStrHyperlink = ^TTerStrHyperlink;
  TTerStrHyperlink = Record
    hWnd: THandle;					// handle of the TER window
    code: Array[0..pred(MAX_WIDTH + 2)] of AnsiChar;	// link code
    text: Array[0..pred(MAX_WIDTH + 2)] of AnsiChar;	// link text
    DoubleClick: Bool;					// always true
    codeW: Array[0..pred(MAX_WIDTH + 2)] of WideChar;	// hyperlink code in unicode format
    textW: Array[0..pred(MAX_WIDTH + 2)] of WideChar;	// hyperlink text in unicode format
  end;


function HtmlToClipboard(const HtmlString: String): Longbool;
var
  cfHtml, uHtmlBeg, uHtmlEnd: UINT;
  sHtmlBeg, sHtmlEnd: String;
  hHtml, hText: HGLOBAL;
begin
  Result := false;
  cfHtml := RegisterClipboardFormat('HTML Format');
  if cfHtml = 0 then Exit;
  if not OpenClipboard(0) then Exit;
  try
    EmptyClipboard;
    uHtmlBeg := 105 + pred(Pos('<html', LowerCase(HtmlString)));
    if uHtmlBeg < 105 then uHtmlBeg := 105;
    uHtmlEnd := uHtmlBeg + UINT(Length(HtmlString));
    sHtmlBeg := Format('%.10d', [uHtmlBeg]) + CRLF;
    sHtmlEnd := Format('%.10d', [uHtmlEnd]) + CRLF;
    hHtml := GetStringHandleA(
      'Version:0.9' + CRLF +
      'StartHTML:' + sHtmlBeg +
      'EndHTML:' + sHtmlEnd +
      'StartFragment:' + sHtmlBeg +
      'EndFragment:' + sHtmlEnd +
      HtmlString, 0, false);
    hText := GetStringHandleA(HtmlString, 0, false);
    Result := (hHtml <> 0) and (SetClipboardData(cfHtml, hHtml) <> 0) and
	      (hText <> 0) and (SetClipboardData(CF_TEXT, hText) <> 0);
  finally
    CloseClipboard;
  end;
end;

function RtfToClipboard(const RtfString: String): Longbool;
var
  cfRtf: UINT;
  hRtf, hText: HGLOBAL;
begin
  Result := false;
  cfRtf := RegisterClipboardFormat(RichEdit.CF_RTF);
  if cfRtf = 0 then Exit;
  if not OpenClipboard(0) then Exit;
  try
    EmptyClipboard;
    hRtf := GetStringHandleA(RtfString, 0, false);
    hText := GetStringHandleA(RtfString, 0, false);
    Result := (hRtf <> 0) and (SetClipboardData(cfRtf, hRtf) <> 0) and
	      (hText <> 0) and (SetClipboardData(CF_TEXT, hText) <> 0);
  finally
    CloseClipboard;
  end;
end;


// TTERInterface ---------------------------------------------------------------

constructor TTERInterface.Create(const LibraryPath: String; CheckVersions: Longbool);

  function FindMostRecentVersion(const ModulePath, PatternName, File32Name: String; CheckVersions: Longbool): String;
  var
    hSearch: THandle;
    fdFile: TWin32FindData;
    s32File, sPatternFile: String;
  begin
    Result := E;
    if not CheckVersions then begin
      Result := ModulePath + File32Name;
      Exit;
    end;
    hSearch := FindFirstFile(PChar(ModulePath + PatternName), fdFile);
    if hSearch <> INVALID_HANDLE_VALUE then begin
      try
	repeat
	  if (fdFile.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then begin
	    if CompareTxt(fdFile.cFileName, File32Name) then
	      s32File := ModulePath + File32Name
	    else sPatternFile := ModulePath + fdFile.cFileName;
	  end;
	until not FindNextFile(hSearch, fdFile);
      finally
	Windows.FindClose(hSearch);
      end;
    end;
    if GetModuleHandle(PChar(s32File)) <> 0 then
      Result := ExtractFileName(s32File)
    else if GetModuleHandle(PChar(sPatternFile)) <> 0 then
      Result := ExtractFileName(sPatternFile)
    else Result := ModulePath + File32Name;
  end;

var
  hTER, hHTS: HMODULE;
begin
  inherited Create;
  hTER := LoadLibrary(Pointer(FindMostRecentVersion(LibraryPath, TER_LIB, TER_32, CheckVersions)));
  if hTER <> 0 then begin
    TERCreateWindowAlt := GetProcAddress(hTER, 'TerCreateWindowAlt');
    CloseTer := GetProcAddress(hTER, 'CloseTer');
    TerPrint := GetProcAddress(hTER, 'TerPrint');
    TerCommand := GetProcAddress(hTER, 'TerCommand');
    TerIsPrinting := GetProcAddress(hTER, 'TerIsPrinting');
    GetTerFields := GetProcAddress(hTER, 'GetTerFields');
    SetTerFields := GetProcAddress(hTER, 'SetTerFields');
    TERGetReadOnly := GetProcAddress(hTER, 'TerGetReadOnly');
    TERSetFlags	:= GetProcAddress(hTER, 'TerSetFlags');
    TERSetFlags2 := GetProcAddress(hTER, 'TerSetFlags2');
    TERSetFlags3 := GetProcAddress(hTER, 'TerSetFlags3');
    TERSetFlags4 := GetProcAddress(hTER, 'TerSetFlags4');
    TERSetFlags5 := GetProcAddress(hTER, 'TerSetFlags5');
    TERSetFlags6 := GetProcAddress(hTER, 'TerSetFlags6');
    TERSetFlags7 := GetProcAddress(hTER, 'TerSetFlags7');
    TERSetFlags8 := GetProcAddress(hTER, 'TerSetFlags8');
    TERSetFlags9 := GetProcAddress(hTER, 'TerSetFlags9');
    TERSetFlags10 := GetProcAddress(hTER, 'TerSetFlags10');
    TERSetFlags11 := GetProcAddress(hTER, 'TerSetFlags11');
    TERGetCurrentFont := GetProcAddress(hTER, 'TerGetCurFont');
    TERSetDefaultFont := GetProcAddress(hTER, 'SetTerDefaultFont');
    TERSetFont := GetProcAddress(hTER, 'SetTerFont');
    TERGetFontInfo := GetProcAddress(hTER, 'GetFontInfo');
    TERSetFontSize := GetProcAddress(hTER, 'SetTerPointSize');
    TERGetParaInfo := GetProcAddress(hTER, 'TerGetParaInfo');
    TERSetParaStyles := GetProcAddress(hTER, 'SetTerParaFmt');
    TERResetParagraph := GetProcAddress(hTER, 'ParaNormal');
    TERClearTabStop := GetProcAddress(hTER, 'ClearTab');
    TERGetTabStop := GetProcAddress(hTER, 'TerGetTabStop');
    TERSetTabStop := GetProcAddress(hTER, 'TerSetTab');
    TerSetDefTabWidth := GetProcAddress(hTER, 'TerSetDefTabWidth');
    TERGetSelectedRtf := GetProcAddress(hTER, 'TerGetRtfSel');
    TERGetSelectedText := GetProcAddress(hTER, 'TerGetTextSel');
    TERGetSelectedTextU := GetProcAddress(hTER, 'TerGetTextSelU');
    TERInsertRtf := GetProcAddress(hTER, 'InsertRtfBuf');
    TERInsertText := GetProcAddress(hTER, 'TerInsertText');
    TERInsertTextU := GetProcAddress(hTER, 'TerInsertTextU');
    TERGetText := GetProcAddress(hTER, 'GetTerBuffer');
    TERSetText := GetProcAddress(hTER, 'SetTerBuffer');
    TERDeleteText := GetProcAddress(hTER, 'TerDeleteBlock');
    TERGetLine := GetProcAddress(hTER, 'TerGetLine');
    TERGetLineU := GetProcAddress(hTER, 'TerGetLineU');
    TERSetLine := GetProcAddress(hTER, 'TerSetLine');
    TERPictLinkName := GetProcAddress(hTER, 'TerPictLinkName');
    TERFindHlinkField := GetProcAddress(hTER, 'TerFindHlinkField');
    TERInsertHyperlink := GetProcAddress(hTER, 'TerInsertHyperlink');
    TERInsertHyperlinkU := GetProcAddress(hTER, 'TerInsertHyperlinkU');
    TERLocateFieldChar := GetProcAddress(hTER, 'TerLocateFieldChar');
    TERInsertDrawObject := GetProcAddress(hTER, 'TerInsertDrawObject');
    TERCreateTable := GetProcAddress(hTER, 'TerCreateTable');
    TERCellBorder := GetProcAddress(hTER, 'TerCellBorder');
    TERCellWidth := GetProcAddress(hTER, 'TerCellWidth');
    TERSetTableColWidth := GetProcAddress(hTER, 'TerSetTableColWidth');
    TERGetMarginEx := GetProcAddress(hTER, 'TerGetMarginEx');
    TERGetCursorPos := GetProcAddress(hTER, 'GetTerCursorPos');
    TERSetCursorPos := GetProcAddress(hTER, 'SetTerCursorPos');
    TERGetTextColors := GetProcAddress(hTER, 'TerGetTextColor');
    TERSetTextColor := GetProcAddress(hTER, 'SetTerColor');
    TERSetTextBKColor := GetProcAddress(hTER, 'SetTerBkColor');
    TERSetPageBkColor := GetProcAddress(hTER, 'TerSetPageBkColor');
    TERTextPosToPix := GetProcAddress(hTER, 'TerTextPosToPix');
    TERPixToTextPos := GetProcAddress(hTER, 'TerPixToTextPos');
    TEREngageCaret := GetProcAddress(hTER, 'TerEngageCaret');
    TERGetCaretPos := GetProcAddress(hTER, 'TerGetCaretPos');
    TERSetCaretPos := GetProcAddress(hTER, 'TerSetCaretPos');
    TERPosToRowCol := GetProcAddress(hTER, 'TerAbsToRowCol');
    TERRowColToPos := GetProcAddress(hTER, 'TerRowColToAbs');
    TERLineHeight := GetProcAddress(hTER, 'TerScrLineHeight');
    TERSearchReplace := GetProcAddress(hTER, 'TerSearchReplace');
    TERSearchReplaceU := GetProcAddress(hTER, 'TerSearchReplaceU');
    TERSelect := GetProcAddress(hTER, 'SelectTerText');
    TERDeselect := GetProcAddress(hTER, 'DeselectTerText');
    TERGetSelection := GetProcAddress(hTER, 'TerGetSelection');
    TEREnableRefresh := GetProcAddress(hTER, 'TerEnableRefresh');
    TERNormalizeBlock := GetProcAddress(hTER, 'TerNormalizeBlock');
    TERRepaint := GetProcAddress(hTER, 'TerRepaint');
  end;
  hHTS := LoadLibrary(Pointer(FindMostRecentVersion(LibraryPath, HTS_LIB, HTS_32, CheckVersions)));
  if hHTS <> 0 then begin
    HtsInitialize := GetProcAddress(hHTS, 'HtsInitialize');
    HtsClearWindow := GetProcAddress(hHTS, 'HtsClearWindow');
    HtsClose := GetProcAddress(hHTS, 'HtsClose');
    HtsRead := GetProcAddress(hHTS, 'HtsRead');
    HtsSave := GetProcAddress(hHTS, 'HtsSave');
    HtsSetPicture := GetProcAddress(hHTS, 'HtsSetPicture');
  end;
  FHtsTerConverter := THtsTerConverter.Create(LibraryPath, CheckVersions, 0);
  FTerVersion := IntToStr(GetSimpleFileVersion(GetModuleFileName(hTER), true));
end;

destructor TTERInterface.Destroy;
begin
  FHtsTerConverter.Free;
  inherited;
end;

procedure TTERInterface.Print(Window: THandle; Dialog: Longbool);
begin
  TerPrint(Window, Dialog);
end;

function TTERInterface.IsPrinting(Window: THandle): Longbool;
begin
  Result := TerIsPrinting(Window);
end;

function TTERInterface.GetTextSize(Window: THandle): Integer;
var
  iCurrentPos: Integer;
begin
  GetCursorPos(iCurrentPos);
  try
    SetCursorPos(MAXINT);
    GetCursorPos(Result);
  finally
    SetCursorPos(iCurrentPos);
  end;
end;

function TTERInterface.GetSelStart(Window: THandle): Integer;
var
  FirstLine, FirstCol, EndLine, EndCol: Integer;
begin
  TERNormalizeBlock(Window);
  if TERGetSelection(Window, FirstLine, FirstCol, EndLine, EndCol) then begin
    Result := TERRowColToPos(Window, FirstLine, FirstCol);
    Exit;
  end;
  Result := -1;
end;

function TTERInterface.GetSelLength(Window: THandle): Integer;
var
 FirstLine, FirstCol, EndLine, EndCol: Integer;
begin
  TERNormalizeBlock(Window);
  if TERGetSelection(Window, FirstLine, FirstCol, EndLine, EndCol) then begin
    Result := pred(TERRowColToPos(Window, EndLine, EndCol)) - TERRowColToPos(Window, FirstLine, FirstCol);
    Exit;
  end;
  Result := 0;
end;

function TTERInterface.GetTerFlags(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags2(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags2(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags2(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags2(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags3(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags3(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags3(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags3(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags4(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags4(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags4(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags4(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags5(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags5(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags5(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags5(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags6(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags6(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags6(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags6(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags7(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags7(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags7(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags7(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags8(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags8(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags8(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags8(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags9(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags9(Window, FALSE, FlagValue);
end;

function TTERInterface.SetTerFlags9(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  Result := TerSetFlags9(Window, TRUE, FlagValue);
end;

function TTERInterface.GetTerFlags10(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  if Assigned(TerSetFlags10) then
    Result := TerSetFlags10(Window, FALSE, FlagValue)
  else Result := 0;
end;

function TTERInterface.SetTerFlags10(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  if Assigned(TerSetFlags10) then
    Result := TerSetFlags10(Window, TRUE, FlagValue)
  else Result := 0;
end;

function TTERInterface.GetTerFlags11(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  if Assigned(TerSetFlags11) then
    Result := TerSetFlags11(Window, FALSE, FlagValue)
  else Result := 0;
end;

function TTERInterface.SetTerFlags11(Window: THandle; FlagValue: Cardinal): Cardinal;
begin
  if Assigned(TerSetFlags11) then
    Result := TerSetFlags11(Window, TRUE, FlagValue)
  else Result := 0;
end;

function TTERInterface.FindBearOrTerWindow(Parent: THandle; NewReader: Longbool): THandle;
begin
  if NewReader then begin
    Result := FindFirstChild(Parent, BEAR_CLASS, E, 0, false);
    if Result = 0 then Result := FindFirstChild(Parent, IE_CLASS, E, 0, true);
    if Result = 0 then Result := FindNextChild(Parent, TER_XX_CLASS, E, 0, true);
  end
  else Result := FindFirstChild(Parent, BEAR_CLASS + CM + IE_CLASS + CM + TER_XX_CLASS, E, 0, true);
end;

function TTERInterface.FindTerWindow(Parent: THandle): THandle;
begin
  Result := FindFirstChild(Parent, TER_XX_CLASS, E, 0, true);
end;

function TTERInterface.SetCaretPos(TerWnd: THandle; Pos: Integer): Longbool;
begin
  Result := TERSetCursorPos(FTerWindow, Pos, -1, false);
end;

function TTERInterface.GetCaretScreenPos(TerWnd: THandle): TPoint;
begin
  with Result do begin
    X := 0;
    Y := 0;
  end;
  if (TerWnd <> 0) then TERTextPosToPix(TerWnd, REL_SCREEN, -1, -1, Result.X, Result.Y);
end;

function TTERInterface.SetCaretAtCursorPos(TerWnd: THandle): TPoint;
var
  ptCursor: TPoint;
begin
  if (TerWnd <> 0) then begin
    Windows.GetCursorPos(ptCursor);
    if TERPixToTextPos(TerWnd, REL_SCREEN, ptCursor.X, ptCursor.Y, Integer(Result.Y), Integer(Result.X)) then begin
      TERSetCursorPos(TerWnd, Result.Y, Result.X, true);
    end;
  end;
end;

function TTERInterface.GetCurrentLineHeight(TerWnd: THandle): Integer;
var
  iLine, iColumn: Integer;
begin
  Result := 0;
  if (TerWnd <> 0) then begin
    TERPosToRowCol(TerWnd, TERGetCaretPos(TerWnd), iLine, iColumn);
    Result := TERLineHeight(TerWnd, iLine);
  end;
end;

function TTERInterface.GetFirstTabPos(TerWnd: THandle): Integer;
var
  iType, iFlag: Integer;
begin
  Result := 0;
  if (TerWnd <> 0) then TERGetTabStop(TerWnd, 0, 0, Result, iType, iFlag);
end;

function TTERInterface.GetLineFromPos(TerWnd: THandle; Pos: Integer): Integer;
var
  iColumn: Integer;
begin
  Result := -1;
  if (TerWnd <> 0) then TERPosToRowCol(TerWnd, Pos, Result, iColumn);
end;

function TTERInterface.GetPosFromLine(TerWnd: THandle; Line: Integer): Integer;
begin
  Result := TERRowColToPos(TerWnd, Line, 0);
end;

function TTERInterface.GetKeyData(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree; SMimeCapable: Longbool): String;
var
  hd: THeaderData;
begin
  if (TerWnd <> 0) then begin
    FTerWindow := TerWnd;
    Result := GetAnsiText(true, false);
    if Result = E then Result := GetAnsiText(false, false);
    if Result <> E then begin
      if not IsEncoded(GetEncoding(Result, hd), true) then begin
	with MimeMsgTree do begin
	  Clear;
	  MimeParse(Result, 1, 0, [mdoAnalyseEncoding]);
	  hd.Encoding := BodyEncoding;
	end;
      end;
      case hd.Encoding of
	eNone: Result := E;
	eCerts, eSMEncrypted..eSMInline: if not SMimeCapable then Result := E;
      end;
    end;
  end
  else Result := E;
end;

function TTERInterface.GetEncodedData(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree; SMimeCapable: Longbool): String;
var
  hd: THeaderData;
begin
  if (TerWnd <> 0) then begin
    FTerWindow := TerWnd;
    Result := GetAnsiText(false, false);
    if Result <> E then begin
      if not IsEncoded(GetEncoding(Result, hd), false) then begin
	with MimeMsgTree do begin
	  Clear;
	  MimeParse(Result, 1, 0, [mdoAnalyseEncoding]);
	  hd.Encoding := BodyEncoding;
	end;
      end;
      case hd.Encoding of
	eNone, ePlainKeys, eCerts: Result := E;
	eSMEncrypted..eSMInline: if not SMimeCapable then Result := E;
      end;
    end;
  end
  else Result := E;
end;

function TTERInterface.GetAddressOrHexID(TerWnd: THandle; var HexID: Longbool): String;
begin
  HexID := false;
  if (TerWnd <> 0) then begin
    FTerWindow := TerWnd;
    Result := GetAnsiText(true, false);
    if Length(Result) > 0 then begin
      if FirstCharPos(AT, Result) > 0 then
	Result := ExtractEmail(Result)
      else begin
	Result := ExtractHexID(Result);
	HexID := (Result <> E);
      end;
    end
    else Result := E;
  end
  else Result := E;
end;

function TTERInterface.GetAnsiLineText(TerWnd: THandle; Line: Integer): String;
begin
  if (TerWnd <> 0) then begin
    FTerWindow := TerWnd;
    if Line = -1 then Line := GetCurrentLine;
    if Line >= 0 then begin
      Result := GetAnsiLine(Line);
      Exit;
    end;
  end;
  Result := E;
end;

function TTERInterface.GetUnicodeLineText(TerWnd: THandle; Line: Integer): String;
begin
  if (TerWnd <> 0) then begin
    FTerWindow := TerWnd;
    if Line = -1 then Line := GetCurrentLine;
    if Line >= 0 then begin
      Result := GetUnicodeLine(Line);
      Exit;
    end;
  end;
  Result := E;
end;

function TTERInterface.GetAnsiLineTextAtCursor(TerWnd: THandle): String;
var
  ptCursor: TPoint;
  iLine, iColumn: Integer;
begin
  if (TerWnd <> 0) then begin
    iLine := 0;
    iColumn := 0;
    Windows.GetCursorPos(ptCursor);
    if TERPixToTextPos(TerWnd, REL_SCREEN, ptCursor.X, ptCursor.Y, iLine, iColumn) then begin
      Result := GetAnsiLine(iLine);
      Exit;
    end;
  end;
  Result := E;
end;

function TTERInterface.GetUnicodeLineTextAtCursor(TerWnd: THandle): String;
var
  ptCursor: TPoint;
  iLine, iColumn: Integer;
begin
  if (TerWnd <> 0) then begin
    iLine := 0;
    iColumn := 0;
    Windows.GetCursorPos(ptCursor);
    if TERPixToTextPos(TerWnd, REL_SCREEN, ptCursor.X, ptCursor.Y, iLine, iColumn) then begin
      Result := GetUnicodeLine(iLine);
      Exit;
    end;
  end;
  Result := E;
end;

function TTERInterface.GetUrl(HyperlinkData: LPARAM): String;
begin
  Result := DelQuotes(PTerStrHyperlink(HyperlinkData)^.code);
end;

function TTERInterface.DecodeUrl(const Url: String): String;
var
  sDecoded: String;
begin
  sDecoded := Unescape(MimeDecodeHeaders(DelQuotes(Url)), false, PC);
  if (sDecoded <> E) and (sDecoded <> Url) then
    Result := XChgStrings(sDecoded, '&amp;', AM)
  else Result := Url;
end;

function TTERInterface.InsertTextVariant(TerWnd: THandle; const Text: String; const RtfFont: TRtfFont;
					 const FontName: String; TextColor, Position: Integer): Longbool;
var
  sRtfText, sFontFace: String;
  iBeg, iEnd: Integer;
begin
  if not LoStrStart(RTF_MARKER, Text) then
    sRtfText := TextToRtf(Text, RtfFont, false)
  else sRtfText := Text;
  iBeg := TERGetCaretPos(TerWnd);
  Result := SetRtfText(sRtfText, false);
  iEnd := TERGetCaretPos(TerWnd);
  if (iEnd > iBeg) and ((TextColor > -1) or (Position > -1)) then begin
    SelectText(iBeg, iEnd);
    sFontFace := GetFontFace(-1);
    try
      if FontName <> E then SetFontFace(FontName);
      if TextColor > -1 then SetTextColor(TextColor);
      if Position > -1 then SetParaStyles(Position);
      SetFontSize(RtfFont.Size);
      TERDeselect(TerWnd, false);
    finally
      if sFontFace <> E then SetFontFace(sFontFace);
    end;
  end;
end;

function TTERInterface.InsertMessageBody(TerWnd: THandle; const MimeMsgTree: TMimeMsgTree;
					 const DisplayInfo: TDisplayInfo): Longbool;

  function GetLineText(TerWnd: THandle; Line: Integer): String;
  begin
    Result := GetUnicodeLineText(TerWnd, Line);
  end;

  function GetTextBuffer: String;
  begin
    Result := Trim(WideToUtf8(GetUnicodeText(false, true)));
  end;

  function InsertHeaderText(HeaderText: String; const DisplayInfo: TDisplayInfo;
			    BlankLineCount: Integer; AdjustHeaders: Longbool): String;
  var
    iIndex, iLen: Integer;
    sCustomHeader, sHeader, sFormat: String;
    saHeaders: TStringArray;
    pSource, pDest, pPos: PChar;
  begin
    Result := E;
    iIndex := 0;
    for BlankLineCount := BlankLineCount downto 0 do iIndex := NextStrPos(PARA, HeaderText, iIndex + Length(PARA));
    if iIndex > 0 then begin
      if AdjustHeaders then begin
	Result := HeaderText;
	HeaderText := ExtractStr(Result, 1, pred(iIndex));
	iIndex := FirstStrPos(PARA, HeaderText);
	if iIndex > 0 then begin
	  inc(iIndex, pred(Length(PARA)));
	  sCustomHeader := TrimLeft(ExtractStr(HeaderText, 1, iIndex));
	end
	else if DisplayInfo.MessageType = mtForward then begin
	  iIndex := FirstStrPos(CRLF, HeaderText) + pred(Length(CRLF));
	  sCustomHeader := PARA + ExtractStr(HeaderText, 1, iIndex) + CRLF;
	end;
	with saHeaders do begin
	  try
	    StringAdjust := MimeDecodeHeaders;
	    New;
	    AddText(HeaderText);
	    for iIndex := pred(Count) downto 0 do begin
	      sHeader := GetString(iIndex);
	      pSource := Pointer(sHeader);
	      pPos := pSource;
	      while not (pPos^ in [NULL, CL]) do inc(pPos);
	      if pPos^ = CL then begin
		inc(pPos);
		iLen := pPos - pSource;
		System.SetString(sFormat, nil, Length(sHeader) + 16 - iLen);
		pDest := Pointer(sFormat);
		pDest := StrLAppend(pDest, pSource, iLen, false);
		iLen := 16 - iLen;
		if iLen > 0 then begin
		  System.FillChar(pDest^, iLen, SP);
		  inc(pDest, iLen);
		end;
		pDest^ := TAB;
		inc(pDest);
		while pPos^ in [TAB, SP] do inc(pPos);
		pSource := pPos;
		while not (pPos^ in [NULL, LF, CR]) do inc(pPos);
		pDest := StrLAppend(pDest, pSource, succ(pPos - pSource), false);
		Delete(sFormat, pDest - Pointer(sFormat), MAXINT);
		SetString(iIndex, sFormat);
	      end;
	    end;
	    HeaderText := sCustomHeader + GetText + CRLF;
	  finally
	    Clear;
	  end;
	end;
      end
      else Delete(HeaderText, iIndex + Length(PARA), MAXINT);
      InsertTextVariant(TerWnd, Utf8ToWide(HeaderText), TRtfFont(DisplayInfo.DefaultFont), E, -1, -1);
    end;
  end;

  procedure SetSigAndForwardMsgTerminator(const DisplayInfo: TDisplayInfo);
  begin
    with DisplayInfo do begin
      SetAnsiText(CRLF, false);
      InsertTextVariant(TerWnd, ForwardedEnd, TRtfFont(DefaultFont), E, -1, -1);
      if Signature <> E then begin
	SetAnsiText(CRLF, false);
	InsertTextVariant(TerWnd, Signature, TRtfFont(DefaultFont), E, -1, -1);
      end;
      SetAnsiText(CRLF, false);
    end;
  end;

  procedure SetSigAndOrgMsgSeparator(const DisplayInfo: TDisplayInfo);
  begin
    with DisplayInfo do begin
      SetAnsiText(PARA, false);
      if Signature <> E then begin
	InsertTextVariant(TerWnd, Signature, TRtfFont(DefaultFont), E, -1, -1);
	SetAnsiText(PARA, false);
      end;
      InsertTextVariant(TerWnd, OriginalMessageSeparator, TRtfFont(DefaultFont), E, -1, -1);
    end;
  end;

  procedure SetEncodingSeparator(const DisplayInfo: TDisplayInfo; MessageText: TTextList;
				 Index: Integer; var Encoded: Longbool);
  const
    AENCODING: Array[false..true] of String = (PMIME_INFO, SMIME_INFO);
  const
    SEPARATOR = '%s%s';
    SEP_L = LT + SP + SEPARATOR + SP;
    SEP_R = SP + SEPARATOR + SP + GT + CRLF;
  var
    fSMimeEncoded: Boolean;
    sSeparator, sFontFace: String;
    dwTextColor: DWord;
    iFontSize: Integer;
  begin
    with MessageText do if MimeMsgTree.UnencodedMimeData then begin
      fSMimeEncoded := (MimeMsgTree.BodyEncoding >= eSMEncrypted);
      if not Encoded then begin
	if (Index = Count) or not SectionNodes[Index].IsPSMEncoded then Exit;
	sSeparator := Format(SEP_L, [E, E]) +
		      Format(DisplayInfo.EncodedBegin, [AENCODING[fSMimeEncoded]]) +
		      Format(SEP_R, [E, E]);
	Encoded := true;
      end
      else begin
	if (Index < Count) and SectionNodes[Index].IsPSMEncoded then Exit;
	sSeparator := Format(SEP_L, [E, '']) +
		      Format(DisplayInfo.EncodedEnd, [AENCODING[fSMimeEncoded]]) +
		      Format(SEP_R, ['', E]);
	Encoded := false;
      end;
      iFontSize := GetFontSize(GetCurrentFont);
      sFontFace := GetFontFace(-1);
      GetTextColor(dwTextColor);
      try
	SetFontSize(10);
	SetFontFace(FONT_FIXED);
	SetTextColor(COLOR_GRAYTEXT);
	InsertTextVariant(TerWnd, sSeparator, TRtfFont(DisplayInfo.DefaultFont), FONT_FIXED, COLOR_GRAYTEXT, -1);
      finally
	SetTextColor(dwTextColor);
	SetFontFace(sFontFace);
	SetFontSize(iFontSize);
      end;
    end;
  end;

const
  SEPARATOR  = '';
var
  RtfFont: TRtfFont;
  iPos, iBeg, iEnd, iIndex, iCurrent: Integer;
  dwTextColor, dwParaStyles: DWord;
  sTextBuffer, sMessageText, sTheresMoreHint: String;
  tlMessage: TTextList;
  fEncoded: Longbool;
begin
  Result := false;
  if (TerWnd <> 0) and MimeMsgTree.ContainsData then begin
    RtfFont := TRtfFont(DisplayInfo.DefaultFont);
    iPos := 0;
    iBeg := MAXINT;
    iEnd := MAXINT;
    FTerWindow := TerWnd;
    EnableScreenWrap(true);
    DisableWebAccess(true);
    DisableTerGridLines(true);
    GetTextColor(dwTextColor);
    GetParaStyles(dwParaStyles);
    try
      if DisplayInfo.MessageType > mtRead then with DisplayInfo do begin
	if MessageType = mtReply then begin
	  EnableRefresh(false);
	  sTextBuffer := GetTextBuffer;
	  if ReplyHeaderOptions * [rhoCustom] <> [] then begin
	    SetAnsiText(PARA, false);
	    if Signature <> E then InsertTextVariant(TerWnd, Signature, RtfFont, E, -1, -1);
	    GetCursorPos(iPos);
	    DrawHorizontalLine;
	    SetCursorPos(succ(iPos));
	    SetAnsiText(PARA, false);
	    if ReplyHeaderOptions * [rhoOriginal] <> [] then
	      InsertHeaderText(sTextBuffer, DisplayInfo, succ(CustomHeaderBlankLines), true)
	    else InsertHeaderText(sTextBuffer, DisplayInfo, CustomHeaderBlankLines, false);
	  end
	  else if ReplyHeaderOptions = [rhoOriginal] then begin
	    SetSigAndOrgMsgSeparator(DisplayInfo);
	    InsertHeaderText(sTextBuffer, DisplayInfo, 0, true);
	  end
	  else begin
	    DeleteAll;
	    SetSigAndOrgMsgSeparator(DisplayInfo);
	  end;
	  SetCursorPos(iBeg);
	end
	else begin
	  EnableRefresh(false);
	  sTextBuffer := GetTextBuffer;
	  InsertHeaderText(sTextBuffer, DisplayInfo, CustomHeaderBlankLines, true);
	  GetCursorPos(iPos);
	  iBeg := iPos;
	end;
      end
      else if not DisplayInfo.IsNewReader then begin
	iBeg := FindTextA(TER_PARA, 0, false);
	if iBeg > -1 then
	  inc(iBeg, Length(CRLF))
	else iBeg := 0;
      end
      else iBeg := 0;
      if (iBeg = iEnd) or DeleteText(iBeg, iEnd) then begin
	if DisplayInfo.MessageType = mtRead then EnableRefresh(false);
	with MimeMsgTree do tlMessage := GetMessageText(DisplayInfo.IsNewReader and (mdoFancyText in MimeDecodeOptions));
	with tlMessage do begin
	  SetDefaultFont(DisplayInfo.DefaultFont, E);
	  if DisplayInfo.IsNewReader then begin
	    fEncoded := false;
	    for iIndex := 0 to pred(Count) do begin
	      iPos := -1;
	      sMessageText := Strings[iIndex];
	      if sMessageText <> E then begin
		SetEncodingSeparator(DisplayInfo, tlMessage, iIndex, fEncoded);
		Result := InsertTextVariant(TerWnd, sMessageText, RtfFont, E, -1, -1);
		iCurrent := GetCurrentLine;
		if GetLineText(TerWnd, iCurrent) = E then begin
		  TERSetCursorPos(TerWnd, iCurrent, 0, true);
		  DeleteText(TERGetCaretPos(TerWnd), MAXINT);
		end
		else SetAnsiText(CRLF, false);
	      end;
	    end;
	    SetEncodingSeparator(DisplayInfo, tlMessage, Count, fEncoded);
	  end
	  else InsertTextVariant(TerWnd, AnsiText, RtfFont, E, -1, -1);
	  if DisplayInfo.MessageType = mtForward then begin
	    SetSigAndForwardMsgTerminator(DisplayInfo);
	    SetCursorPos(0);
	  end;
	end;
	if (DisplayInfo.MessageType = mtRead) and MimeMsgTree.HasAttachments or MimeMsgTree.IsMessageDigest then begin
	  with DisplayInfo do begin
	    SetParaStyles(CENTER);
	    SetTextColor(COLOR_GRAYTEXT);
	    InsertTextVariant(TerWnd, CRLF + SEPARATOR, RtfFont, FONT_FIXED, COLOR_GRAYTEXT, CENTER);
	    if MimeMsgTree.HasAttachments then
	      sTheresMoreHint := AttachmentHint
	    else sTheresMoreHint := DigestHint;
	    InsertTextVariant(TerWnd, sTheresMoreHint, RtfFont, ShellDlgFont(0, 0), COLOR_GRAYTEXT, CENTER);
	    iBeg := TERGetCaretPos(TerWnd);
	    SetTextColor($008080FF);
	    SetFontSize(RtfFont.Size);
	    SetDefaultFont(DisplayInfo.DefaultFont, E);
	    SetHyperlink(ClickHereHint + WorkWithPartsHint + DOT, PMPGP_URL + WorkWithPartsHint, false);
	    iEnd := TERGetCaretPos(TerWnd);
	    SelectText(iBeg, iEnd);
	    SetParaStyles(CENTER);
	  end;
	end;
      end;
    finally
      SetParaStyles(dwParaStyles);
      SetTextColor(dwTextColor);
      EnableRefresh(true);
      MimeMsgTree.Clear;
      SetCursorPos(0);
    end;
  end;
end;

function TTERInterface.InsertVerificationInfo(TerWnd: THandle; const Text: String;
					      Flags: Integer; TabPos: Integer;
					      const DisplayInfo: TDisplayInfo;
					      NewReader: Longbool): Longbool;

  function SetText(const Text: String; TextColor: Integer): Longbool;
  begin
    Result := InsertTextVariant(TerWnd, Text, TRtfFont(DisplayInfo.DefaultFont), E, TextColor, -1);
  end;

const
  DTAB = TAB + TAB;
  PGP_INFO	= 'PGP-Info:';
  COLOR_VALID	= $00008000;
  COLOR_INVALID	= $000000FF;
  COLOR_WARNING	= $000080FF;
var
  sInfo, sDelimiter, sReaderText: String;
  iLine, iPos, iTab, iLeader: Integer;
  dwSigColor, dwTextColor: DWord;
begin
  Result := false;
  if (TerWnd <> 0) and (Text <> E) then begin
    FTerWindow := TerWnd;
    iLine := 0;
    case Flags of
      SF_VALID_TRUSTED..SF_EXPIRED: dwSigColor := COLOR_VALID;
      SF_INVALID: dwSigColor := COLOR_INVALID;
      SF_NO_KEY: dwSigColor := COLOR_WARNING;
    else
      dwSigColor := 0;
    end;
    Result := true;
    if (dwSigColor <> 0) and not LoStrStart(PGP_INFO, GetAnsiLineText(TerWnd, iLine)) then begin
      sDelimiter := DTAB;
      sInfo := Text;
      iPos := 1;
      repeat
	iPos := NextStrPos(CRLF, sInfo, iPos);
	if iPos > 0 then begin
	  inc(iPos, Length(CRLF));
	  Insert(sDelimiter, sInfo, iPos);
	end;
      until iPos = 0;
      iPos := -1;
      if NewReader then sReaderText := GetRtfText(true, true);
      Get0x0TabPosAndLeader(iTab, iLeader);
      GetTextColor(dwTextColor);
      TERSetCaretPos(TerWnd, 0);
      Result := (
	TERResetParagraph(TerWnd, false) and
	TERCreateTable(TerWnd, 1, 1, false) and
	TERCellWidth(TerWnd, SEL_ALL, -1, 0, false) and
	TERSetTableColWidth(TerWnd, GetTerClientWidth, false) and
	Set0x0TabPosAndLeader(TabPos, TAB_NONE) and
	SetText(PGP_INFO, -1) and
	SetTextColor(dwSigColor) and
	SetText(DTAB + sInfo, dwSigColor) and
	Set0x0TabPosAndLeader(TabPos, TAB_ULINE) and
	SetTextColor(0) and
	SetAnsiText(sDelimiter + CRLF, false) and
	GetCursorPos(iPos) and
	SetTextColor(dwTextColor) and
	Set0x0TabPosAndLeader(iTab, iLeader) and
	(not NewReader or SetRtfText(sReaderText, false)) and
	TERSetCaretPos(TerWnd, iPos)
      );
      Repaint(false);
    end;
  end;
end;

function TTERInterface.IsHyperlinkAtCursor(const Text: String): Longbool;
var
  ptCursor: TPoint;
  iLineIn, iColumnIn: Integer;
  iLineOut, iColumnOut: Integer;
begin
  Result := false;
  if Text <> E then begin
    iLineIn := 0;
    iColumnIn := 0;
    Windows.GetCursorPos(ptCursor);
    if TERPixToTextPos(FTerWindow, REL_SCREEN, ptCursor.X, ptCursor.Y, iLineIn, iColumnIn) then begin
      iLineOut := iLineIn;
      iColumnOut := iColumnIn;
      Result := FindHyperlink(Text, iLineOut, iColumnOut) and
      		(iLineOut = iLineIn) and (iColumnOut - iColumnIn < Length(Text));
    end;
  end;
end;

function TTERInterface.IsHyperlinkFormat(const Text: String): Longbool;
var
  ptCursor: TPoint;
  iLineIn, iColumnIn: Integer;
  iLineOut, iColumnOut: Integer;
begin
  Result := false;
  if Text <> E then begin
    iLineIn := 0;
    iColumnIn := 0;
    Windows.GetCursorPos(ptCursor);
    if TERPixToTextPos(FTerWindow, REL_SCREEN, ptCursor.X, ptCursor.Y, iLineIn, iColumnIn) then begin
      iLineOut := iLineIn;
      iColumnOut := iColumnIn;
      Result := TERLocateFieldChar(FTerWindow, FIELD_HLINK, nil, true, iLineOut, iColumnOut, true) and
      		(iLineOut = iLineIn) and (iColumnOut - iColumnIn < Length(Text));
    end;
  end;
end;

function TTERInterface.IsBearClass(const ClassName: String): Longbool;
begin
  Result := (ClassName = BEAR_CLASS) or (ClassName = IE_CLASS);
end;

function TTERInterface.IsBearWindow(Window: THandle): Longbool;
var
 sClassName: String;
begin
  sClassName := ClassString(Window);
  Result := (sClassName = BEAR_CLASS) or (sClassName = IE_CLASS);
end;

function TTERInterface.IsReaderBlank(TerWnd: THandle): Longbool;
begin
  Result := (TerWnd = 0) or (GetWindowTextLength(TerWnd) <= Length(CRLF));
end;

function TTERInterface.IsTerClass(const ClassName: String): Longbool;
begin
  Result := CheckQMRegEx(ClassName, TER_XX_CLASS, Length(TER_XX_CLASS));
end;

function TTERInterface.IsTerReader(Window: THandle): Longbool;
begin
  Result := (IsTerWindow(Window) and TERGetReadOnly(Window));
end;

function TTERInterface.IsTerWindow(Window: THandle): Longbool;
begin
  Result := CheckQMRegEx(ClassString(Window), TER_XX_CLASS, Length(TER_XX_CLASS));
end;

function TTERInterface.MessageClear(TerWnd: THandle): Longbool;
begin
  FTerWindow := TerWnd;
  Result := (TerWnd <> 0) and DeleteAll;
end;

function TTERInterface.RtfReadMessage(TerWnd: THandle): String;
begin
  FTerWindow := TerWnd;
  if (TerWnd <> 0) then
    Result := GetRtfText(false, false)
  else Result := E;
end;

function TTERInterface.RtfWriteMessage(TerWnd: THandle; const Text: String): Longbool;
begin
  FTerWindow := TerWnd;
  Result := (TerWnd <> 0) and SetRtfText(Text, true);
end;

function TTERInterface.HtmlToRtf(const Html: String; MaxWidth: Integer; var TextHeight: Integer): String;
begin
  Result := FHtsTerConverter.HtmlToRtf(Html, MaxWidth, TextHeight);
end;

function TTERInterface.RtfToHtml(const Rtf: String; MaxWidth: Integer; var TextHeight: Integer): String;
begin
  Result := FHtsTerConverter.RtfToHtml(Rtf, MaxWidth, TextHeight);
end;

procedure TTERInterface.SetHyperlinkFocus(TerWnd: THandle);
begin
  if IsTerReader(TerWnd) and IsHyperlinkFormat(GetAnsiLineTextAtCursor(TerWnd)) then begin
    SetCaretAtCursorPos(TerWnd);
    SetFocus(TerWnd);
  end;
end;


// private methods -------------------------------------------------------------

function TTERInterface.GetCurrentLine: Integer;
var
 iColumn: Integer;
begin
  Result := -1;
  iColumn := 0;
  TERGetCursorPos(FTerWindow, Result, iColumn);
end;

function TTERInterface.GetCursorPos(var Pos: Integer): Longbool;
var
 iColumn: Integer;
begin
  Pos := -1;
  iColumn := -1;
  Result := TERGetCursorPos(FTerWindow, Pos, iColumn);
end;

function TTERInterface.SetCursorPos(Pos: Integer): Longbool;
begin
  Result := TERSetCursorPos(FTerWindow, Pos, -1, false);
end;

function TTERInterface.GetCurrentFont: Integer;
begin
  Result := TERGetCurrentFont(FTerWindow, -1, 0);
end;

function TTERInterface.SetDefaultFont(const Font: TPMPGPFont; const Name: String): Longbool;

  function SetNewFont(const Font: TPMPGPFont; const Name: String; Style: Integer): Longbool;
  begin
    Result := TERSetDefaultFont(FTerWindow, PChar(Name), Font.Size, Style, 0, false) and
	      TERSetFont(FTerWindow, PChar(Name), false);
  end;

var
  iStyle: Integer;
begin
  iStyle := 0;
  with Font do if Styles <> 0 then begin
    if Styles and CFE_BOLD <> 0 then iStyle := iStyle or BOLD;
    if Styles and CFE_ITALIC <> 0 then iStyle := iStyle or ITALIC;
    if Styles and CFE_UNDERLINE <> 0 then iStyle := iStyle or ULINE;
    if Styles and CFE_STRIKEOUT <> 0 then iStyle := iStyle or STRIKE;
  end;
  if Name <> E then
    Result := SetNewFont(Font, Name, iStyle)
  else Result := SetNewFont(Font, Font.Name, iStyle);
end;

function TTERInterface.GetFontFace(FontID: Integer): String;
var
  iSize: Integer;
  dwStyles: DWord;
begin
  SetString(Result, nil, SizeOf(TFontName));
  if TERGetFontInfo(FTerWindow, FontId, PChar(Result), iSize, dwStyles) then begin
    Delete(Result, succ(StrLen(PChar(Result))), MAXINT);
    Exit;
  end;
  Result := E;
end;

function TTERInterface.SetFontFace(const FontFace: String): Longbool;
begin
  Result := TERSetFont(FTerWindow, PChar(FontFace), false);
end;

function TTERInterface.GetFontSize(FontID: Integer): Integer;
var
  sDummy: String;
  dwDummy: DWord;
begin
  Result := 0;
  SetString(sDummy, nil, LF_FACESIZE);
  TERGetFontInfo(FTerWindow, FontID, Pointer(sDummy), Result, dwDummy);
end;

function TTERInterface.SetFontSize(Points: Integer): Longbool;
begin
  Result := TERSetFontSize(FTerWindow, Points, false);
end;

function TTERInterface.GetParaStyles(var ParaStyles: DWord): Longbool;
var
  iDummy: Integer;
begin
  ParaStyles := 0;
  Result := TERGetParaInfo(FTerWindow, GetCurrentLine, iDummy, iDummy, iDummy, iDummy, iDummy,
			   iDummy, iDummy, iDummy, iDummy, iDummy, iDummy, ParaStyles);
end;

function TTERInterface.SetParaStyles(ParaStyles: DWord): Longbool;
begin
  Result := TERSetParaStyles(FTerWindow, ParaStyles, true, false);
end;

function TTERInterface.GetTextColor(var TextColor: DWord): Longbool;
var
  dwTextBkColor: DWord;
begin
  dwTextBkColor := 0;
  TextColor := GetSysColor(COLOR_WINDOWTEXT);
  Result := TERGetTextColors(FTerWindow, GetCurrentFont, TextColor, dwTextBkColor);
end;

function TTERInterface.SetTextColor(TextColor: DWord): Longbool;
begin
  Result := TERSetTextColor(FTerWindow, TextColor, false);
end;

function TTERInterface.GetTextBkColor(var TextBkColor: DWord): Longbool;
var
  dwTextColor: DWord;
begin
  dwTextColor := 0;
  TextBkColor := GetSysColor(COLOR_WINDOW);
  Result := TERGetTextColors(FTerWindow, GetCurrentFont, dwTextColor, TextBkColor);
end;

function TTERInterface.SetTextBkColor(TextBkColor: DWord): Longbool;
begin
  Result := TERSetTextBkColor(FTerWindow, TextBkColor, false);
end;

function TTERInterface.SetPageBkColor(PageBkColor: DWord): Longbool;
begin
  Result := TERSetPageBkColor(FTerWindow, PageBkColor);
end;

function TTERInterface.Get0x0TabPosAndLeader(var TabPos, Leader: Integer): Longbool;
var
  iType: Integer;
begin
  TabPos := 0;
  Leader := TAB_NONE;
  Result := (TERGetTabStop(FTerWindow, 0, 0, TabPos, iType, Leader) > 0);
end;

function TTERInterface.Set0x0TabPosAndLeader(TabPos, Leader: Integer): Longbool;
var
  iPos, iType, iLeader: Integer;
begin
  if TERGetTabStop(FTerWindow, 0, 0, iPos, iType, iLeader) = 0 then
    Result := TERClearTabStop(FTerWindow, TabPos, false)
  else Result := TERSetTabStop(FTerWindow, iType, TabPos, Leader, false);
end;

function TTERInterface.SetFirstTabInCurrentLine(TabPos: Integer): Integer;
var
  iType, iLeader: Integer;
begin
  if TERGetTabStop(FTerWindow, -1, 0, Result, iType, iLeader) = -1 then Result := 0;
  TERSetTabStop(FTerWindow, iType, TabPos, iLeader, false);
end;

function TTERInterface.FindTextA(const Text: String; StartPos: Longint; Backwards: Longbool): Longint;
var
  lFlags, lDummy: Longint;
begin
  Result := -1;
  if Text <> E then begin
    lFlags := SRCH_SEARCH;
    if Backwards then lFlags := lFlags or SRCH_BACK;
    if not TERSearchReplace(FTerWindow, Pointer(Text), nil, lFlags, StartPos, Result, lDummy) then Result := -1;
  end;
end;

function TTERInterface.FindTextW(const Text: String; StartPos: Longint; Backwards: Longbool): Longint;
var
  lFlags, lDummy: Longint;
begin
  Result := -1;
  if Text <> E then begin
    lFlags := SRCH_SEARCH;
    if Backwards then lFlags := lFlags or SRCH_BACK;
    if not TERSearchReplaceU(FTerWindow, Pointer(Text), nil, lFlags, StartPos, Result, lDummy) then Result := -1;
  end;
end;

function TTERInterface.ReplaceTextA(const Text: String; StartPos, EndPos: Longint): Longint;
var
  lFlags, lDummy: Longint;
begin
  try
    Result := EndPos;
    if Text <> E then begin
      lFlags := SRCH_REPLACE;
      if not TERSearchReplace(FTerWindow, nil, Pointer(Text), lFlags, StartPos, Result, lDummy) then Result := -1;
    end;
  except
  end;
end;

function TTERInterface.ReplaceTextW(const Text: String; StartPos, EndPos: Longint): Longint;
var
  lFlags, lDummy: Longint;
begin
  try
    Result := EndPos;
    if Text <> E then begin
      lFlags := SRCH_REPLACE;
      if not TERSearchReplaceU(FTerWindow, nil, Pointer(Text), lFlags, StartPos, Result, lDummy) then Result := -1;
    end;
  except
  end;
end;

function TTERInterface.SelectAll(var BegPos, EndPos: Integer; DoSelect: Longbool): Longbool;
var
  iBegCol, iEndCol: Integer;
begin
  Result := TERSelect(FTerWindow, BegPos, -1, EndPos, -1, false);
  if TERGetSelection(FTerWindow, BegPos, iBegCol, EndPos, iEndCol) then begin
    BegPos := TERRowColToPos(FTerWindow, BegPos, iBegCol);
    EndPos := TERRowColToPos(FTerWindow, EndPos, iEndCol);
    if not DoSelect then begin
      TerDeselect(FTerWindow, false);
      Exit;
    end
    else Result := TERSelect(FTerWindow, BegPos, -1, EndPos, -1, false);
  end;
end;

function TTERInterface.SelectText(BegPos, EndPos: Integer): Longbool;
begin
  if (BegPos = 0) and (EndPos = MAXINT) then
    Result := SelectAll(BegPos, EndPos, true)
  else Result := TERSelect(FTerWindow, BegPos, -1, EndPos, -1, false);
end;

function TTERInterface.DeleteAll: Longbool;
begin
  Result := SetTerText(SP) and SelectText(0, 1) and TERDeleteText(FTerWindow, TRUE);
end;

function TTERInterface.DeleteText(BegPos, EndPos: Integer): Longbool;
begin
  if (BegPos = 0) and (EndPos = MAXINT) then
    Result := DeleteAll
  else Result := (IsReaderBlank(FTerWindow) or (SelectText(BegPos, EndPos) and TERDeleteText(FTerWindow, false)));
end;

function TTERInterface.GetAnsiLine(Line: Integer): String;
var
  iPos: Integer;
  pResult: PAnsiChar;
begin
  SetString(Result, nil, TERGetLine(FTerWindow, Line, nil, nil));
  iPos := pred(TERGetLine(FTerWindow, Line, Pointer(Result), nil));
  if iPos > -1 then begin
    pResult := Pointer(Result);
    while (iPos >= 0) and (pResult[iPos] <= SP) do dec(iPos);
    Delete(Result, iPos + 2, MAXINT);
  end
  else Result:= E;
end;

function TTERInterface.GetUnicodeLine(Line: Integer): String;
var
  iPos: Integer;
  pResult: PWideChar;
begin
  SetString(Result, nil, (TERGetLineU(FTerWindow, Line, nil, nil) shl 1) + Length(wNul));
  iPos := pred(TERGetLineU(FTerWindow, Line, Pointer(Result), nil));
  if iPos > -1 then begin
    pResult := Pointer(Result);
    while (iPos >= 0) and (pResult[iPos] <= SP) do dec(iPos);
    Delete(Result, iPos + 2, MAXINT);
  end
  else Result:= E;
end;

function TTERInterface.GetAnsiText(Selection, ClearScreen: Longbool): String;
var
  hText: THandle;
  iLen: Integer;
begin
  Result := E;
  if not Selection then SelectText(0, MAXINT);
  hText := TERGetSelectedText(FTerWindow, iLen);
  try
    if (hText <> 0) and (iLen > 0) then begin
      SetString(Result, PChar(GlobalLock(hText)), iLen);
      if ClearScreen then
	TERDeleteText(FTerWindow, ClearScreen)
      else if not Selection then TERDeselect(FTerWindow, true);
      Result := TrimLeft(Result);
    end;
  finally
    GlobalUnlock(hText);
    GlobalFree(hText);
  end;
end;

function TTERInterface.GetRtfText(Selection, ClearScreen: Longbool): String;
var
  hText: THandle;
  iLen: Integer;
begin
  Result := E;
  if not Selection then SelectText(0, MAXINT);
  hText := TERGetSelectedRtf(FTerWindow, iLen);
  try
    if (hText <> 0) and (iLen > 0) then begin
      SetString(Result, PChar(GlobalLock(hText)), iLen);
      if ClearScreen then
	TERDeleteText(FTerWindow, ClearScreen)
      else if not Selection then TERDeselect(FTerWindow, true);
    end;
  finally
    GlobalUnlock(hText);
    GlobalFree(hText);
  end;
end;

function TTERInterface.GetUnicodeText(Selection, ClearScreen: Longbool): String;
var
  hText: THandle;
  iLen: Integer;
begin
  Result := E;
  if not Selection then SelectText(0, MAXINT);
  hText := TERGetSelectedTextU(FTerWindow, iLen);
  try
    if (hText <> 0) and (iLen > 0) then begin
      SetString(Result, PChar(GlobalLock(hText)), iLen);
      if ClearScreen then
	TERDeleteText(FTerWindow, ClearScreen)
      else if not Selection then TERDeselect(FTerWindow, true);
    end;
  finally
    GlobalUnlock(hText);
    GlobalFree(hText);
  end;
end;

function TTERInterface.SetAnsiText(const Text: String; Redraw: Longbool): Longbool;
begin
  Result := false;
  if Text <> E then Result := TERInsertText(FTerWindow, Pointer(Text), -1, -1, Redraw);
end;

function TTERInterface.SetRtfText(const Text: String; Redraw: Longbool): Longbool;
begin
  Result := false;
  if Text <> E then begin
    Result := TERInsertRtf(FTerWindow, Pointer(Text), Length(Text), -1, -1, Redraw);
  end;
end;

function TTERInterface.SetUnicodeText(const Text: String; Redraw: Longbool): Longbool;
begin
  Result := false;
  if Text <> E then Result := TERInsertTextU(FTerWindow, Pointer(Text), -1, -1, Redraw);
end;

function TTERInterface.SetTerText(const Text: String): Longbool;
begin
  Result := false;
  if Text <> E then Result := TERSetText(FTerWindow, GetStringHandleA(Text, 0, false), Length(Text), E, true);
end;

function TTERInterface.FindHyperlink(const Url: String; var Line, Column: Integer): Longbool;
begin
  Result := false;
  if Url <> E then begin
    if (Line < 0) or (Column < 0) then begin
      Line := -1;
      Column := -1;
      TERGetCursorPos(FTerWindow, Line, Column);
    end;
    Result := TERFindHlinkField(FTerWindow, Pointer(Url), nil, Line, Column);
  end;
end;

function TTERInterface.SetHyperlink(const Text, Url: String; Redraw: Longbool): Longbool;
var
  stf: TStrTerField;
  clrLink: TColorRef;
begin
  if (Text <> E) and (Url <> E) then begin
    GetTerFields(FTerWindow, stf);
    clrLink := stf.LinkColor;
    try
      stf.LinkColor := $008080FF;
      SetTerFields(FTerWindow, stf);
      Result := (TERInsertHyperlink(FTerWindow, Pointer(Text), Pointer(Url), 0, Redraw) <> -1);
    finally
      stf.LinkColor := clrLink;
      SetTerFields(FTerWindow, stf);
    end;
    Exit;
  end;
  Result := false;
end;

function TTERInterface.GetTerClientWidth: Integer;
begin
  Result := GetWidthTwips(FTerWindow, GetClientRect(FTerWindow).Right - GetSystemMetrics(SM_CXVSCROLL));
end;

function TTERInterface.DrawHorizontalLine: Longbool;
begin
  Result := (TERInsertDrawObject(FTerWindow, DOB_LINE, -1, -1, GetTerClientWidth, 0) <> 0);
end;

procedure TTERInterface.DisableTerGridLines(Disable: Longbool);
var
  field: TStrTerField;
begin
  if GetTerFields(FTerWindow, field) then begin
    field.ShowTableGridLines := not Disable;
    SetTerFields(FTerWindow, field);
  end;
end;

procedure TTERInterface.DisableWebAccess(Disable: Longbool);
begin
  SetTerFlags7(FTerWindow, GetTerFlags7(FTerWindow, 0) or TFLAG7_NO_INTERNET);
end;

procedure TTERInterface.EnableRefresh(Enable: Longbool);
begin
  TEREnableRefresh(FTerWindow, Enable);
end;

procedure TTERInterface.EnableScreenWrap(Enable: Longbool);
begin
  SetTerFlags(FTerWindow, GetTerFlags(FTerWindow, 0) or TFLAG_KEEP_PICT_ASPECT);
  SetTerFlags6(FTerWindow, GetTerFlags6(FTerWindow, 0) or TFLAG6_WRAP_AT_WINDOW_WIDTH);
  SetTerFlags7(FTerWindow, GetTerFlags7(FTerWindow, 0) or TFLAG7_SHRINK_PICT_TO_PAGE);
  SetTerFlags8(FTerWindow, GetTerFlags8(FTerWindow, 0) or TFLAG8_AUTO_REFORMAT_TABLES);
  SetTerFlags9(FTerWindow, GetTerFlags9(FTerWindow, 0) or TFLAG9_WRAP_LONG_HTML_WORD);
  SetTerFlags10(FTerWindow, GetTerFlags10(FTerWindow, 0) or TFLAG10_SHRINK_RTF_IMAGES);
  SetTerFlags11(FTerWindow, GetTerFlags11(FTerWindow, 0) or TFLAG11_SHRINK_SUBTABLE_TO_FIT);
end;

procedure TTERInterface.Repaint(Clear: Longbool);
begin
  TERRepaint(FTerWindow, Clear);
end;


end.

