{$J+,Z4}
unit TextViewer;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2023 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, SysUtils, StrUtil, Graphics, Forms, Dialogs, Controls, ComCtrls, ExtCtrls, StdCtrls, Menus,
  ExtCtrlsEx, Buttons, FlatBtn, Gauges3d, ColGrip, RichEdit, RichEd20, DropEdit, DatedStrings, SearchDialog, PrintDlg,
  UxThemes, WindowUtils, FileUtils, MailUtils, PMPGPForm, PMPGPCommon, PMPGPResources;


type
  TTextViewer = class(TPMPGPForm)
    pnLeft: TPanel;
    pnLeftTop: TPanel;
    pnLeftLeft: TPanel;
    pnText: TColoredFramePanel;
    deText: TDropEdit;
    pnLeftRight: TPanel;
    pnLeftBottom: TPanel;
    pnProgress: TPanel;
    pnLabel: TPanel;
    pnBottom: TPanel;
    pnBar: TPanel;
    gProgress: TGauge3d;
    fbAbort: TFlatBtn;
    pnButtons: TPanel;
    pnTop: TPanel;
    fbWrap: TFlatBtn;
    fbSort: TFlatBtn;
    fbFindFirst: TFlatBtn;
    fbFindNext: TFlatBtn;
    fbSave: TFlatBtn;
    fbLoad: TFlatBtn;
    fbPrint: TFlatBtn;
    pnClose: TPanel;
    fbClose: TFlatBtn;
    cgTextViewer: TColoredGrip;
    pmText: TPopupMenu;
    miSelectAll: TMenuItem;
    miCopySelection: TMenuItem;
    miLine: TMenuItem;
    miCopyLink: TMenuItem;
    miOpenLink: TMenuItem;
    pmSave: TPopupMenu;
    miSaveAs: TMenuItem;
    miHeader: TMenuItem;
    miText: TMenuItem;
    miUnicode: TMenuItem;
    miRichText: TMenuItem;
    miHtmlText: TMenuItem;
    miHtmlWithImages: TMenuItem;
    miSeparator1: TMenuItem;
    miEncryptedText: TMenuItem;
    miEncryptedUnicode: TMenuItem;
    miEncryptedRichText: TMenuItem;
    miSeparator2: TMenuItem;
    miUnixMailbox: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure fbWrapClick(Sender: TObject);
    procedure fbSortClick(Sender: TObject);
    procedure fbFindFirstClick(Sender: TObject);
    procedure fbFindNextClick(Sender: TObject);
    procedure fbSaveClick(Sender: TObject);
    procedure fbLoadClick(Sender: TObject);
    procedure fbPrintClick(Sender: TObject);
    procedure fbCloseClick(Sender: TObject);
    procedure fbWrapToggleToolbarButton(Sender: TObject);
    procedure fbLoadToggleToolbarButton(Sender: TObject);
    procedure deTextAfterSetPrintText(PrintEdit: TDropEdit; const PrinterPrefs: TPrinterPrefs);
    procedure deTextUrlClick(const Url: String; Button: TMouseButton; X, Y: Longint);
    procedure deTextUrlDblClick(const Url: String);
    procedure miSelectAllClick(Sender: TObject);
    procedure miCopySelectionClick(Sender: TObject);
    procedure miCopyLinkClick(Sender: TObject);
    procedure miOpenLinkClick(Sender: TObject);
    procedure pmTextPopup(Sender: TObject);
    procedure pmSavePopup(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
  private
    CreationBounds: TRect;
    FolderData: TDatedMessages;
    FolderMode: Longbool;
    FolderName: String;
    JustStarted: Longbool;
    MailAddress: String;
    MailToUrl: String;
    TextSection: TSectionNode;
    UrlClicked: String;
    procedure DisplayData;
    function  ProgressBarMove(Value: Integer): Longbool;
    function  ProgressBarShow(Total: Integer): Longbool;
    procedure Search(Again: Longbool);
    procedure SetFont(ToggleMonoSpaced: Longbool);
    procedure SizeAdjust;
    procedure SwitchButtons;
    procedure WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  protected
    procedure SetColors(Default: Longbool); override;
  public
  end;

procedure ShowFolderText(MainClass: TObject);
procedure ShowMessageText(MainClass: TObject; Section: TSectionNode; SectionIcon: TIcon; Wrapped: Longbool);


implementation

uses
  MessageDialogs;

{$R *.DFM}

procedure ShowFolderText(MainClass: TObject);
var
  frmTextViewer: TTextViewer;
begin
  frmTextViewer := TTextViewer.Create(MainClass);
  if frmTextViewer <> nil then with frmTextViewer do begin
    try
      fbLoad.Glyph.LoadFromResourceID(hInstance, OPEN_FOLDER_BUTTON);
      SetHotKeyHint(fbLoad, PMPGPStrings[OPEN_FOLDER_HINT]);
      fbSort.Enabled := true;
      fbLoad.Enabled := true;
      FolderMode := true;
      ShowModal;
    finally
      Release;
    end;
  end;
end;

procedure ShowMessageText(MainClass: TObject; Section: TSectionNode; SectionIcon: TIcon; Wrapped: Longbool);
var
  frmTextViewer: TTextViewer;
begin
  frmTextViewer := TTextViewer.Create(MainClass);
  if frmTextViewer <> nil then with frmTextViewer do begin
    try
      with MainClass.PrinterPrefs do if not Initialized then begin
	Initialized := true;
	ApplySettings := false;
	StrPLCopy(Printer, PMInterface.PrinterName, pred(SizeOf(TPrinterName)));
	Copies := 1;
	Collate := true;
	PrintRange := prAllPages;
	CustomOption := (ord(PMInterface.PrintHeaders) <> 0);
	with PMInterface.PrinterFont do begin
	  if Name <> E then StrPLCopy(PrinterFontName, Name, pred(SizeOf(TFontName)));
	  if Size <> 0 then PrinterFontSize := Size;
	end;
	MarginFontStyles := [fsBold];
	with PMInterface do begin
	  if PrintHeader then Include(MarginOptions, moPrintHeader);
	  if PrintFooter then Include(MarginOptions, moPrintFooter);
	  Margins := PrintMargins;
	end;
      end;
      with MainClass.PrefsCaptions do begin
	PropsSheet := PMPGPStrings[FORMATTING_SHEET];
	OrientationGroup := PMPGPStrings[ORIENTATION_GROUP];
	PagePortrait := PMPGPStrings[PAGE_PORTRAIT];
	PageLandscape := PMPGPStrings[PAGE_LANDSCAPE];
	MarginsGroup := PMPGPStrings[MARGINS_GROUP];
	HMargins := PMPGPStrings[MARGINS_HORZ_OPTION];
	VMargins := PMPGPStrings[MARGINS_VERT_OPTION];
	FontGroup := PMPGPStrings[FONT_GROUP];
	FontName := PMPGPStrings[FONT_NAME];
	FontSize := PMPGPStrings[FONT_SIZE];
	HeadFootGroup := PMPGPStrings[HEAD_FOOT_GROUP];
	PrintHeader := PMPGPStrings[PRINT_HEADER_OPTION];
	PrintFooter := PMPGPStrings[PRINT_FOOTER_OPTION];
	CustomOption := PMPGPStrings[PRINT_MSG_HEADERS_OPTION];
      end;
      TextSection := Section;
      Caption := TextSection.SectionName;
      with TextSection.MimeMsgTree do begin
	if deText.RichEditV3 then
	  MimeDecodeOptions := MimeDecodeOptions + [mdoReturnUnicode]
	else MimeDecodeOptions := MimeDecodeOptions - [mdoReturnUnicode];
      end;
      deText.ShowObjects := PMInterface.ReaderFancyView;
      fbLoad.Glyph.LoadFromResourceID(hInstance, LOAD_IMAGES_BUTTON);
      with fbLoad do begin
	Enabled := (Section.SectionType in [mdtHtmlText..mdtRtfText]) and (Section.Count > 0);
	if Enabled then begin
	  BtnDown := Section.IncludeImages;
	  ToolbarStyle := tbsToolbarButton;
	end
	else ToolbarStyle := tbsNone;
	fbLoadToggleToolbarButton(nil);
      end;
      if Wrapped then fbWrap.Click;
      Icon := SectionIcon;
      ShowModal;
    finally
      Release;
    end;
  end;
end;

procedure TTextViewer.FormCreate(Sender: TObject);
var
  iPos: Integer;
begin
  fbAbort.Glyph.LoadFromResourceID(hInstance, IDW_BUTTON_CANCEL);
  CreationBounds := Rect(Left, Top, Width, Height);
  with deText do begin
    DatedStrings.DateTimeSorted := dtSorted;
    FolderData := TDatedMessages.Create(DatedStrings);
    if FolderData <> nil then with FolderData do begin
      AttachmentHint := PMPGPStrings[ATTACHMENT_HINT];
      DigestHint := PMPGPStrings[DIGEST_HINT];
      MsgTreeData := MainClass.MsgTreeData;
      OnShowProgress := ProgressBarMove;
    end;
    GetSearchText := EditSearchText;
    WordWrap := (ord(PMInterface.ReaderWordWrap) > 0);
    fbWrap.BtnDown := WordWrap;
  end;
  NormalWindowSize;
  JustStarted := true;
  Caption := PMPGPStrings[TEXT_VIEWER];
  pnLeftTop.Caption := PMPGPStrings[TEXT_MENU_HINT];
  SetHotKeyHint(fbSort, PMPGPStrings[SORT_HINT]);
  SetHotKeyHint(fbFindFirst, PMPGPStrings[FIND_FIRST_HINT]);
  SetHotKeyHint(fbFindNext, PMPGPStrings[FIND_NEXT_HINT]);
  SetHotKeyHint(fbSave, PMPGPStrings[SAVE_HINT]);
  SetHotKeyHint(fbPrint, PMPGPStrings[PRINT_HINT]);
  SetHotKeyHint(fbClose, PMPGPStrings[CLOSE_HINT]);
  miSelectAll.Caption := PMPGPStrings[SELECT_ALL_ITEM];
  miCopySelection.Caption := PMPGPStrings[COPY_SELECTION_ITEM];
  miCopyLink.Caption := Format(PMPGPStrings[COPY_DO], [PMPGPStrings[COPY_URL]]);
  miOpenLink.Caption := Format(PMPGPStrings[OPEN_URL], [PMPGPStrings[COPY_URL]]);
  miSaveAs.Caption := PMPGPStrings[SAVE_FILE_DIALOG];
  with miHtmlText do Caption := Caption + PMPGPStrings[HTML_FORMATTED];
  with miHtmlWithImages do begin
    iPos := FirstCharPos(SP, PMPGPStrings[INCLUDING_IMAGES]);
    Caption := Caption + SP + Copy(PMPGPStrings[INCLUDING_IMAGES], 1, iPos) +
	       AM + Copy(PMPGPStrings[INCLUDING_IMAGES], succ(iPos), MAXINT);
  end;
  miEncryptedText.Caption := DelAmp(miText.Caption) + PMPGPStrings[SAVE_ENCRYPTED];
  miEncryptedUnicode.Caption := DelAmp(miUnicode.Caption) + PMPGPStrings[SAVE_ENCRYPTED];
  miEncryptedRichText.Caption := DelAmp(miRichText.Caption) + PMPGPStrings[SAVE_ENCRYPTED];
  with miUnixMailbox do Caption := Caption + PMPGPStrings[PLAIN_TEXT_ONLY];
end;

procedure TTextViewer.FormShow(Sender: TObject);
var
  iDistance: Integer;
begin
  SizeAdjust;
  deText.PlainText := false;
  deText.Font.Name := Self.Font.Name;
  if FolderMode then begin
    fbPrint.Visible := false;
    iDistance := fbPrint.Top - fbLoad.Top - fbLoad.Height;
    dec(CreationBounds.Bottom, fbPrint.Height + iDistance * 2);
    miHtmlText.Visible := false;
    miHtmlWithImages.Visible := false;
    miSeparator1.Visible := true;
    miEncryptedText.Visible := true;
    miEncryptedUnicode.Visible := true;
    miEncryptedRichText.Visible := true;
    miSeparator2.Visible := true;
    miUnixMailbox.Visible := true;
  end;
end;

procedure TTextViewer.FormPaint(Sender: TObject);
begin
  if JustStarted then begin
    JustStarted := false;
    ProcessMessages(0);
    SetFont(false);
    Sleep(0);
    if FolderMode then
      fbLoad.Click
    else DisplayData;
  end;
end;

procedure TTextViewer.FormResize(Sender: TObject);
var
  iWidth, iLeft, iTop: Integer;
begin
  with pnProgress do begin
    iWidth := deText.ClientWidth - (GetSystemMetrics(SM_CXVSCROLL) * 2);
    fbAbort.Left := (iWidth - fbAbort.Width) div 2 - BevelWidth;
    iLeft := deText.Left + ((deText.Width - iWidth) div 2);
    iTop := deText.Top + ((deText.Height - Height) div 2);
    SetBounds(iLeft, iTop, iWidth, Height);
  end;
end;

procedure TTextViewer.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if ssCtrl in Shift then begin
    case Char(Key) of
      'F': Search(false);
      'G': Search(true);
      'O': with fbLoad do if Enabled then Click;
      'P': with fbPrint do if Visible then Click;
      'S': with fbSave do if Enabled then Click;
      'V', 'X': Beep;
    end;
  end
  else begin
    case Key of
      VK_TAB: SelectNext(ActiveControl, not (ssShift in Shift), true);
      VK_ESCAPE: begin
	if pnProgress.Showing then
	  ModalResult := mrAbort
	else ModalResult := mrOk;
      end;
      VK_F3: if not (ssAlt in Shift) then Search(true);
      VK_F12: SetFont(true);
    end;
  end;
end;

procedure TTextViewer.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := pnButtons.Enabled and (ModalResult <> mrAbort) and not (pnProgress.Showing or deText.Searching);
  if not CanClose then ModalResult := mrNone;
end;

procedure TTextViewer.FormDestroy(Sender: TObject);
begin
  FolderData.Free;
end;

procedure TTextViewer.fbWrapClick(Sender: TObject);
begin
  with deText do begin
    SwitchButtons;
    try
      WordWrap := not WordWrap;
      SetFocus;
    finally
      SwitchButtons;
    end;
  end;
end;

procedure TTextViewer.fbSortClick(Sender: TObject);
begin
  with deText do if DatedStrings.Count > 0 then begin
    SwitchButtons;
    ProgressBarShow(-1);
    try
      with DatedStrings do DateTimeSort(DateTimeSorted <> dtReverse);
      Text := DatedStrings.Text;
    finally
      ProgressBarShow(0);
      SwitchButtons;
    end;
  end;
end;

procedure TTextViewer.fbFindFirstClick(Sender: TObject);
begin
  Search(false);
end;

procedure TTextViewer.fbFindNextClick(Sender: TObject);
begin
  Search(true);
end;

procedure TTextViewer.fbSaveClick(Sender: TObject);
var
  ptMouse: TPoint;
  rButton: TRect;
begin
  if deText.ContainsText then begin
    RemoveKeyMessage;
    ptMouse := GetCursorPos;
    rButton := GetWindowRect(fbSave.Handle);
    if PtInRect(rButton, ptMouse) then
      with ptMouse do pmSave.Popup(X, Y)
    else with rButton do pmSave.Popup(Left + (Right - Left) div 2, Top + (Bottom - Top) div 2);
  end
  else Beep;
end;

procedure TTextViewer.fbLoadClick(Sender: TObject);

  function FolderRange: String;
  var
    sFirstDate, sLastDate: String;
  begin
    with deText.DatedStrings do begin
      if DateTimeSorted = dtSorted then begin
	sFirstDate := FormatDateTime(ShortDateFormat, DateTimes[0]);
	sLastDate := FormatDateTime(ShortDateFormat, DateTimes[pred(Count)]);
      end
      else begin
	sFirstDate := FormatDateTime(ShortDateFormat, DateTimes[pred(Count)]);
	sLastDate := FormatDateTime(ShortDateFormat, DateTimes[0]);
      end;
      Result := IntToStr(Count) + PMPGPStrings[MAIL_MESSAGES] + CM + SP + sFirstDate + ' - ' + sLastDate;
    end;
  end;

var
  sCaption, sData, sName: String;
  cDefault: TCursor;
  fUnix: Longbool;
begin
  with deText do begin
    if FolderMode then begin
      SwitchButtons;
      PGPInterface.EnableCaching;
      try
	sCaption := Caption;
	Self.Enabled := false;
	cDefault := Screen.Cursor;
	Screen.Cursor := crHourglass;
	try
	  ProcessMessages(0);
	  Caption := PMPGPStrings[LOADING_FOLDER] + DOTS;
	  sData := AdjustLineBreaks(StripExcessCRs(PMInterface.GetFolderText(FolderName, fUnix)));
	finally
	  Screen.Cursor := cDefault;
	  Self.Enabled := true;
	end;
	if sData <> E then begin
	  sName := '[' + FolderName + ']';
	  Caption := PMPGPStrings[LOADING_FOLDER] + sName + SP + DOTS;
	  ProgressBarShow(Length(sData));
	  FolderData.ProcessFolder(sData, fUnix);
	end;
      finally
	PGPInterface.ResetCaching;
	try
	  if (sData <> E) and (DatedStrings.Count > 0) then begin
	    pnLabel.Caption := PMPGPStrings[UPDATING_DISPLAY] + DOTS;
	    deText.ShowDatedStrings;
	    SetFont(false);
	    Caption := PMPGPStrings[MAIL_FOLDER] + sName + CL + SP + FolderRange;
	  end
	  else Caption := sCaption;
	finally
	  ProgressBarShow(0);
	  RemoveKeyMessage;
	  SwitchButtons;
	  SetFocus;
	end;
      end;
    end
    else begin
      OnResizeRequest := nil;
      SwitchButtons;
      try
	ShowObjects := not ShowObjects;
	DisplayData;
      finally
	SwitchButtons;
      end;
    end;
  end;
end;

procedure TTextViewer.fbPrintClick(Sender: TObject);
var
  pdText: TTextPrinterData;
  sFileExt: String;
begin
  FillByte(pdText, SizeOf(TTextPrinterData), 0);
  with pdText do begin
    with PD do begin
      OwnerWindow := Self.Handle;
      PrintJobName := Caption;
    end;
    with TextSection do begin
      sFileExt := FileExt;
      PreformattedText := (SectionType in [mdtHtmlText..mdtRtfText]) or
			  ((SectionType = mdtFile) and ((sFileExt = HTM) or (sFileExt = RTF)));
    end;
    with TextSection.MimeMsgTree.SectionNodes[0] do begin
      Utf8HeaderText := PMInterface.CreateUtf8BannerHeader(UTCDateTime, Utf8Sender, Utf8Subject);
      Utf8FooterText := AnsiToUtf8(PMPGPStrings[PRINT_PAGE_FOOTER]);
    end;
  end;
  pdText.PD.PrinterPrefs := MainClass.PrinterPrefs;
  pdText.PD.PrefsCaptions := MainClass.PrefsCaptions;
  deText.PrintWithMargins(pdText);
  MainClass.PrinterPrefs := pdText.PD.PrinterPrefs;
end;

procedure TTextViewer.fbCloseClick(Sender: TObject);
begin
  if fbClose.CanFocus then Close;
end;

procedure TTextViewer.SetColors(Default: Longbool);
begin
  inherited;
  with gProgress do begin
    BackColor := Self.Color;
    BorderWidth := ord(Default = false);
  end;
end;

procedure TTextViewer.fbWrapToggleToolbarButton(Sender: TObject);
begin
  if fbWrap.BtnDown then
    SetHotKeyHint(fbWrap, PMPGPStrings[UNWRAP_HINT])
  else SetHotKeyHint(fbWrap, PMPGPStrings[WRAP_HINT]);
end;

procedure TTextViewer.fbLoadToggleToolbarButton(Sender: TObject);
begin
  if fbLoad.BtnDown then
    SetHotKeyHint(fbLoad, PMPGPStrings[HIDE_IMAGES_HINT])
  else SetHotKeyHint(fbLoad, PMPGPStrings[LOAD_IMAGES_HINT]);
end;

procedure TTextViewer.deTextAfterSetPrintText(PrintEdit: TDropEdit; const PrinterPrefs: TPrinterPrefs);
var
  iIndex, iPos, iBeg, iEnd: Integer;
  aLine: TLineArray;
begin
  if PrinterPrefs.CustomOption then with PrintEdit do begin
    SetSel(0, 0);
    SelText := TextSection.MimeMsgTree.MainHeaders;
    if GetTextLen > 0 then for iIndex := 0 to pred(Lines) do begin
      aLine := GetLine(iIndex);
      if not IsLineBlank(aLine) then begin
	iPos := 0;
	while (aLine[iPos] > NULL) and (aLine[iPos] <> TAB) do inc(iPos);
	if aLine[iPos] = TAB then begin
	  iBeg := iPos;
	  while aLine[iPos] > NULL do inc(iPos);
	  iEnd := iPos;
	  if iEnd > iBeg then begin
	    iPos := GetLineStartIndex(iIndex);
	    inc(iBeg, succ(iPos));
	    inc(iEnd, iPos);
	    SetSel(iBeg, iEnd - iBeg);
	    SelAttributes.Style := [fsBold];
	    Paragraph.Alignment := taLeftJustify;
	  end;
	end;
      end
      else Break;
    end;
  end;
end;

procedure TTextViewer.deTextUrlClick(const Url: String; Button: TMouseButton; X, Y: Longint);
begin
  if Button = mbRight then begin
    UrlClicked := Url;
    miLine.Visible := true;
    miCopyLink.Visible := true;
    miOpenLink.Visible := true;
    try
      with deText.ClientToScreen(Point(X, Y)) do pmText.Popup(X, Y);
    finally
      miLine.Visible := false;
      miCopyLink.Visible := false;
      miOpenLink.Visible := false;
    end;
    if LoStrStart(MAILTO + CL, Url) then begin
      MailAddress := Copy(Url, succ(Length(MAILTO) + Length(CL)), MAXINT);
      MailToUrl := Url;
    end
    else begin
      MailAddress := E;
      MailToUrl := E;
    end;
  end;
end;

procedure TTextViewer.deTextUrlDblClick(const Url: String);
begin
  if Length(Url) > 3 then MainClass.LaunchBrowser(Url);
end;

procedure TTextViewer.miSelectAllClick(Sender: TObject);
begin
  deText.SelectAll;
end;

procedure TTextViewer.miCopySelectionClick(Sender: TObject);
begin
  deText.Copy;
end;

procedure TTextViewer.miCopyLinkClick(Sender: TObject);
begin
  if UrlClicked <> E then WriteClipboardData(UrlClicked, CF_TEXT, Self.Handle);
end;

procedure TTextViewer.miOpenLinkClick(Sender: TObject);
begin
  if Length(UrlClicked) > 3 then MainClass.LaunchBrowser(UrlClicked);
end;

procedure TTextViewer.pmTextPopup(Sender: TObject);
begin
  miCopySelection.Enabled := (deText.SelText <> E);
end;

procedure TTextViewer.pmSavePopup(Sender: TObject);
var
  sFileExt: String;
begin
  if (TextSection <> nil) and (TextSection = TextSection.MimeMsgTree.FirstTextSection) then begin
    miText.Caption := '&Text' + PMPGPStrings[INCLUDING_HEADERS];
    miUnicode.Caption := '&Unicode' + PMPGPStrings[INCLUDING_HEADERS];
    if deText.ObjectCount = 0 then
      miRichText.Caption := '&Rich Text' + PMPGPStrings[RTF_FORMATTED]
    else miRichText.Caption := '&Rich Text' + SP + LB + PMPGPStrings[INCLUDING_IMAGES] + RB;
  end
  else begin
    miText.Caption := '&Text' + PMPGPStrings[DEFAULT_CHARSET];
    miUnicode.Caption := '&Unicode' + PMPGPStrings[MULTI_LINGUAL];
    miRichText.Caption := '&Rich Text' + PMPGPStrings[RTF_FORMATTED]
  end;
  miUnicode.Enabled := deText.RichEditV3;
  miRichText.Enabled := FolderMode;
  if TextSection <> nil then with TextSection do begin
    sFileExt := FileExt;
    miRichText.Enabled := (SectionType in [mdtHtmlText..mdtRtfText]) or
			  ((SectionType = mdtFile) and ((sFileExt = HTM) or (sFileExt = RTF)));
    miHtmlText.Enabled := (SectionType = mdtHtmlText) or ((SectionType = mdtFile) and (sFileExt = HTM));
    miHtmlWithImages.Enabled := miHtmlText.Enabled and (deText.ObjectCount > 0);
  end;
end;

procedure TTextViewer.miSaveClick(Sender: TObject);

  function SaveEncrypted(const Data, FileName: String): Longbool;
  begin
    Result := WriteTextToFile(FileName, PGPInterface.EncodeFolderData(Data, MainClass.DefaultKey, Handle));
  end;

type
  TFileFormat = (ffText, ffUnicode, ffRichText, ffHtmlText, ffHtmlWithImages,
		 ffEncryptedText, ffEncryptedUnicode, ffEncryptedRichText, ffUnixMailbox);
var
  ffToSave: TFileFormat;
  sFileName, sText: String;
begin
  if Sender = miText then
    ffToSave := ffText
  else if Sender = miUnicode then
    ffToSave := ffUnicode
  else if Sender = miRichText then
    ffToSave := ffRichText
  else if Sender = miHtmlText then
    ffToSave := ffHtmlText
  else if Sender = miHtmlWithImages then
    ffToSave := ffHtmlWithImages
  else if Sender = miEncryptedText then
    ffToSave := ffEncryptedText
  else if Sender = miEncryptedUnicode then
    ffToSave := ffEncryptedUnicode
  else if Sender = miEncryptedRichText then
    ffToSave := ffEncryptedRichText
  else if Sender = miUnixMailbox then
    ffToSave := ffUnixMailbox
  else Exit;
  if FolderMode then begin
    sFileName := LegalFilename(Caption);
    case ffToSave of
      ffText, ffUnicode: sFileName := sFileName + TXT;
      ffRichText: sFileName := sFileName + RTF;
      ffEncryptedText, ffEncryptedUnicode: sFileName := sFileName + TXT + PGP;
      ffEncryptedRichText: sFileName := sFileName + RTF + PGP;
      ffUnixMailbox: sFileName := PMInterface.CreateUnixMailboxName(LegalFileName(FolderName));
    end;
    sFileName := ViewerFileSave(Utf8Check(sFileName), E, PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle);
    if sFileName = E then Exit;
    try
      case ffToSave of
	ffText: if not WriteTextToFile(sFileName, deText.AnsiText) then sFileName := E;
	ffUnicode: if not WriteTextToFile(sFileName, deText.WideText) then sFileName := E;
	ffRichText: if not WriteTextToFile(sFileName, deText.RtfText) then sFileName := E;
	ffEncryptedText: if not SaveEncrypted(deText.AnsiText, sFileName) then Exit;
	ffEncryptedUnicode: if not SaveEncrypted(deText.WideText, sFileName) then Exit;
	ffEncryptedRichText: if not SaveEncrypted(deText.RtfText, sFileName) then Exit;
	ffUnixMailbox: if not FolderData.SaveAsUnixMailbox(sFileName) then sFileName := E;
      end;
    except
      sFileName := E;
    end;
  end
  else if (TextSection <> nil) and (TextSection.SectionData <> E) then begin
    with TextSection.MimeMsgTree do begin
      if TextSection = FirstTextSection then begin
	sFileName := SectionNodes[0].FileNameFromHeaders;
	case ffToSave of
	  ffText, ffUnicode: sFileName := sFileName + TXT;
	  ffRichText: sFileName := sFileName + RTF;
	  ffHtmlText, ffHtmlWithImages: sFileName := sFileName + HTM;
	end;
      end
      else begin
	sFileName := TextSection.FileName;
	case ffToSave of
	  ffText, ffUnicode: sFileName := ChangeFileExt(sFileName, TXT);
	  ffRichText: sFileName := ChangeFileExt(sFileName, RTF);
	  ffHtmlText, ffHtmlWithImages: sFileName := ChangeFileExt(sFileName, HTM);
	end;
      end;
    end;
    sFileName := ViewerFileSave(Utf8Check(sFileName), E, PMPGPStrings[SAVE_FILE_DIALOG], Self.Handle);
    if sFileName = E then Exit;
    with TextSection.MimeMsgTree do begin
      case ffToSave of
	ffText: begin
	  if TextSection = FirstTextSection then
	    sText := Concat([Utf8Check(SectionNodes[0].Utf8MainHeaders + PARA), deText.AnsiText])
	  else sText := deText.AnsiText;
	end;
	ffUnicode: begin
	  if TextSection = FirstTextSection then
	    sText := Concat([Utf8ToWide(SectionNodes[0].Utf8MainHeaders + PARA), deText.WideText])
	  else sText := deText.WideText;
	end;
	ffRichText: sText := deText.RtfText;
	ffHtmlText: sText := TextSection.SectionData;
	ffHtmlWithImages: if not TextSection.SaveRelatedSectionsAsHtml(sFileName) then sFileName := E;
      end;
      if (ffToSave <> ffHtmlWithImages) and not WriteTextToFile(sFileName, sText) then sFileName := E;
    end;
  end
  else Exit;
  if sFileName <> E then begin
    if ffToSave = ffUnixMailbox then begin
      if Confirm(PMPGPStrings[DATA_SAVED_TO] + CRLF + sFileName + DOT + PARA + PMPGPStrings[REINDEX_OR_RESTART]) then begin
        PostMessage(Handle, WM_SYSCOMMAND, SC_CLOSE, 0);
        MainClass.RestartPegasusMail(PMInterface.PMUser, 0);
      end;
    end
    else Info(PMPGPStrings[DATA_SAVED_TO] + CRLF + sFileName);
  end
  else Error(PMPGPStrings[DATA_WRITE_ERROR]);
end;

procedure TTextViewer.DisplayData;
begin
  if TextSection <> nil then with TextSection do begin
    MimeMsgTree.MaxLineLength := GetMaxWindowTextWidth(deText.Handle);
    IncludeImages := deText.ShowObjects;
    deText.Text := SectionText;
    with fbLoad do if not Enabled then begin
      Enabled := (SectionType in [mdtHtmlText..mdtRtfText]) and (IncludeImages or (deText.ObjectCount > 0));
      if Enabled then begin
	BtnDown := IncludeImages;
	ToolbarStyle := tbsToolbarButton;
      end
      else ToolbarStyle := tbsNone;
    end;
  end;
end;

function TTextViewer.ProgressBarMove(Value: Integer): Longbool;
begin
  Sleep(0);
  ProcessMessages(0);
  gProgress.AddProgress(Value);
  Result := (ModalResult <> mrAbort);
end;

function TTextViewer.ProgressBarShow(Total: Integer): Longbool;
begin
  if Total > 0 then begin
    pnLabel.Caption := PMPGPStrings[LOADING_DATA] + DOTS;
    with gProgress do begin
      Progress := 0;
      MaxValue := Total;
      Visible := true;
    end;
    fbAbort.Visible := true;
    pnProgress.Show;
    ProcessMessages(0);
  end
  else if Total < 0 then begin
    pnBar.Caption := PMPGPStrings[UPDATING_DISPLAY] + DOTS;
    gProgress.Visible := false;
    fbAbort.Visible := false;
    pnProgress.Show;
    ProcessMessages(0);
  end
  else begin
    ProcessMessages(0);
    pnProgress.Hide;
    pnBar.Caption := E;
    pnLabel.Caption := E;
  end;
  Result := pnProgress.Visible;
  deText.Refresh;
end;

procedure TTextViewer.SetFont(ToggleMonoSpaced: Longbool);
var
  fs: TFontStyles;
begin
  ProcessMessages(0);
  with deText do begin
    SwitchButtons;
    try
      with PMInterface.ReaderFont do begin
	if ToggleMonoSpaced then begin
	  if FolderMode then ProgressBarShow(-1);
	  try
	    if Font.Name = Name then
	      Font.Name := FixedFontName
	    else Font.Name := Name;
	  finally
	    if FolderMode then ProgressBarShow(0);
	  end;
	end
	else begin
	  Font.Name := Name;
	  Font.Size := Size;
	  fs := [];
	  if Styles <> 0 then begin
	    if Styles and CFE_BOLD <> 0 then Include(fs, fsBold);
	    if Styles and CFE_ITALIC <> 0 then Include(fs, fsItalic);
	    if Styles and CFE_UNDERLINE <> 0 then Include(fs, fsUnderline);
	    if Styles and CFE_STRIKEOUT <> 0 then Include(fs, fsStrikeout);
	  end;
	  Font.Style := fs;
	end;
      end;
    finally
      SwitchButtons;
    end;
  end;
end;

procedure TTextViewer.Search(Again: Longbool);
var
  fShift: Longbool;
begin
  if not (pnProgress.Showing or deText.Searching) then with deText do begin
    ReSearch := Again;
    fShift := ((GetKeyState(VK_SHIFT) < 0) and RichEditV3);
    if fShift then
      SearchOptions := SearchOptions - [DropEdit.stDown]
    else SearchOptions := SearchOptions + [DropEdit.stDown];
    if not (ReSearch and fShift) then begin
      if not SearchText then
	Info(PMPGPStrings[TEXT_NOT_FOUND_INFO])
      else if ReSearch then begin
	if Confirm(PMPGPStrings[TEXT_NOT_FOUND_INFO] + PMPGPStrings[RESTART_SEARCH_CONFIRMATION]) then begin
	  SetSel(0, 0);
	  Search(true);
	end
	else ResetSel;
      end;
    end
    else if not SearchText then Info(PMPGPStrings[TEXT_NOT_FOUND_INFO]);
    Invalidate;
  end;
end;

procedure TTextViewer.SizeAdjust;

  function GetA4TextWidth(Canvas: TCanvas): Integer;
  var
    hDC: THandle;
  begin
    hDC := Canvas.Handle;
    Result := MulDiv(180, GetDeviceCaps(hDC, HORZRES), GetDeviceCaps(hDC, HORZSIZE));
  end;

var
  iDiff, iLeft, iWidth: Integer;
begin
  if not deText.PlainText then begin
    iDiff := GetA4TextWidth(Canvas) - deText.Width;
    if iDiff > 0 then begin
      iLeft := Left - iDiff div 2;
      iWidth := Width + iDiff;
      if (iLeft >= 0) and (iWidth <= Screen.Width) then SetBounds(iLeft, Top, iWidth, Height);
    end;
  end;
end;

procedure TTextViewer.SwitchButtons;
begin
  if Showing then begin
    with pnButtons do Enabled := not Enabled;
    with deText do begin
      HideCaret := not HideCaret;
      if HideCaret then
	BeginUpdate
      else EndUpdate;
    end;
  end;
end;

procedure TTextViewer.WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo);
begin
  inherited;
  with CreationBounds do Message.MinMaxInfo.ptMinTrackSize := Point(Right, Bottom);
end;

procedure TTextViewer.WMSize(var Msg: TWMSize);
begin
  if deText <> nil then deText.Invalidate;
  ProcessMessages(0);
  inherited;
end;

end.

