{$J+,Z4}
unit WindowUtils;
{------------------------------------------------------------------------------}
{        This code is copyright (C) 2006-2018 by Michael in der Wiesche        }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, Classes, Controls, Forms, Graphics, SysUtils, StrUtil, Processes, PMPGPCommon;

const
  BUTTON	= 'BUTTON';
  COMBOBOX	= 'COMBOBOX';
  DIALOG	= '#32770';
  EDIT		= 'EDIT';
  LISTBOX	= 'LISTBOX';
  STATIC	= 'STATIC';

type
  COLOR16 = Word;

  PTriVertex = ^TTriVertex;
  TTriVertex = Packed Record
    x: Longint;
    y: Longint;
    Red: COLOR16;
    Green: COLOR16;
    Blue: COLOR16;
    Alpha: COLOR16;
  end;

  PGradientRect = ^TGradientRect;
  TGradientRect = Packed Record
    UpperLeft: ULONG;
    LowerRight: ULONG;
  end;

const
  GRADIENT_FILL_RECT_H = $00000000;
  GRADIENT_FILL_RECT_V = $00000001;

var GradientFill: function(DC: HDC; Vertex: PTriVertex; NumVertex: ULONG;
			   Mesh: Pointer; NumMesh, Mode: ULONG): BOOL; stdcall;

function  GetCursorPos: TPoint;
function  GetWidthTwips(Window: THandle; const Data: Variant): Integer;
function  ListMDIWindows(Window: THandle; const MDIWindows: TList): Bool; stdcall;
function  FindApplicationWindow(const FileName, ChildClass, ChildText: String): THandle;
function  FindAllChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
function  FindDisabledChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
function  FindInvisibleChild(ParentWindow: THandle; const ClassName, WindowText: String): THandle;
function  FindFirstChild(ParentWindow: THandle; const ClassName, WindowText: String; CtlID: Integer; RegExClass: Longbool): THandle;
function  FindNextChild(ParentWindow: THandle; const ClassName, WindowText: String; CtlID: Integer; RegExClass: Longbool): THandle;
function  FindButton(ParentWindow: THandle; const WindowText: String; Style: DWord): THandle;
function  FindCheckBox(Dialog: THandle; const WindowText: String): THandle;
function  FindComboBox(Dialog: THandle): THandle;
function  FindCtrlByID(Dialog: THandle; CtlID: Integer): THandle;
function  FindEditControl(Dialog: THandle): THandle;
function  FindStatic(Dialog: THandle; const WindowText: String): THandle;
procedure ClickDlgBtn(Dialog: THandle; BtnID: Integer);
function  GetDlgBtnChecked(Dialog: THandle; BtnID: Integer): Integer;
function  SetDialogCheckBox(CheckBox: THandle; Check: Longbool): Longbool;
function  IsDialogCheckBoxChecked(CheckBox: THandle): Longbool;
function  ClickDialogCheckBoxChecked(CheckBox: THandle): Longbool;
function  CheckMenuItem(Menu: THandle; ItemID: Integer; Check: Longbool): Longbool;
function  SelectCBXItemIndex(ComboBox: THandle; Index: Integer): Longbool;
function  GetCBXItemIndex(ComboBox: THandle; const Item: String): Integer;
function  DeleteCBXIndex(ComboBox: THandle; Index: Integer): Integer;
function  SelectCBXIndex(ComboBox: THandle; Index: Integer): Integer;
function  GetSelectedCBXIndex(ComboBox: THandle): Integer;
function  GetSelectedCBXItem(ComboBox: THandle): String;
function  GetLBXText(ListBox: THandle): String;
function  GetMDIParent(Window: THandle): THandle;
function  IsMDIWindowChild(MDIClient, Window: THandle): Longbool;
function  IsModuleWindow(Window: THandle): Longbool;
procedure SendKey(Window: THandle; Key: Integer);


implementation

{$IFDEF CONDITIONALEXPRESSIONS}
uses
  Variants;

const
  NULL = StrUtil.NULL;
{$ENDIF}

type
  TFindCase = (fcFirst, fcNext, fcAll);
  PWindowParams = ^TWindowParams;
  TWindowParams = Record
    wpWindowText: String;
    wpClassName: String;
    wpWindowList: TList;
    wpButtonStyle: DWord;
    wpControlID: Integer;
    wpDisabled: Longbool;
    wpInvisible: Longbool;
    wpCheckRegEx: Longbool;
    fcType: TFindCase;
    wpHandle: THandle;
  end;

function GetCursorPos: TPoint;
begin
  if not Windows.GetCursorPos(Result) then Result := Point(-1, -1);
end;

function GetWidthTwips(Window: THandle; const Data: Variant): Integer;
var
  hDC, hFont: THandle;
  sData: String;
  szText: TSize;
begin
  hDC := GetDC(Window);
  hFont := 0;
  try
    if VarType(Data) and varTypeMask = varString then begin
      sData := Data;
      hFont := SelectObject(hDC, SendMessage(Window, WM_GETFONT, 0, 0));
      GetTextExtentPoint32(hDC, PChar(sData), Length(sData), szText);
    end
    else szText.cx := Data;
    Result := MulDiv(szText.cx, 1440, GetDeviceCaps(hDC, LOGPIXELSX));
  finally
    if hFont <> 0 then SelectObject(hDC, hFont);
    ReleaseDC(Window, hDC);
  end;
end;

function ListMDIWindows(Window: THandle; const MDIWindows: TList): Bool; stdcall;
begin
  Result := true;
  with MDIWindows do begin
    if IsWindowVisible(Window)
    and ((GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_MDICHILD) <> 0)
    and (IndexOf(Ptr(Window)) = -1) then begin
      Insert(0, Ptr(Window));
    end;
  end;
end;

function FindApplicationWindow(const FileName, ChildClass, ChildText: String): THandle;
begin
  Result := FindProcessNameInfo(PChar(ExtractFileName(FileName)), true).piWindow;
  if (Result = 0) or ((ChildClass = E) and (ChildText = E)) then Exit;
  Result := FindFirstChild(Result, ChildClass, ChildText, 0, false);
end;

function CheckChildWindows(FoundWindow: THandle; WP: PWindowParams): Bool; stdcall;

  function IsDisabled(FoundWindow: THandle): Longbool;
  begin
    Result := IsWindowVisible(FoundWindow) and not IsWindowEnabled(FoundWindow);
  end;

var
  sFound: String;
  pSearch: PChar;
  iSearch, iBeg, iEnd, iLen: Integer;
begin
  Result := true;
  with WP^ do begin
    if (not wpDisabled or IsDisabled(FoundWindow)) and not (wpInvisible and IsWindowVisible(FoundWindow)) then begin
      if (wpControlID <> 0) and (GetDlgCtrlID(FoundWindow) = wpControlID) then begin
	wpHandle := FoundWindow;
	Result := false;
	Exit;
      end;
      if wpWindowText <> E then begin
	if FirstStrPos(wpWindowText, DelAmp(WndString(FoundWindow))) <> 0 then begin
	  wpHandle := FoundWindow;
	  if wpClassName = E then begin
	    Result := false;
	    Exit;
	  end;
	end;
      end;
      if wpClassName <> E then begin
	sFound := ClassString(FoundWindow);
	pSearch := Pointer(wpClassName);
	iSearch := Length(wpClassName);
	iBeg := 1;
	while iBeg <= iSearch do begin
	  inc(pSearch, pred(iBeg));
	  iEnd := NextCharPos(CM, wpClassName, iBeg);
	  if iEnd = 0 then iEnd := succ(iSearch);
	  iLen := iEnd - iBeg;
	  if iLen = Length(sFound) then begin
	    if (wpCheckRegEx and CheckQMRegEx(sFound, pSearch, iLen))
	    or (CompareString(LOCALE_USER_DEFAULT, 0, Pointer(sFound), -1, pSearch, iLen) = 2) then begin
	      if (wpHandle <> 0) or ((wpWindowText = E) and (fcType = fcFirst)) then begin
		wpHandle := FoundWindow;
		Result := false;
	      end
	      else begin
		case fcType of
		  fcNext: begin
		    fcType := fcFirst;
		    wpHandle := 0;
		  end;
		  fcAll: begin
		    if wpWindowList <> nil then wpWindowList.Add(Ptr(FoundWindow));
		    wpHandle := 0;
		  end;
		end;
	      end;
	      Exit;
	    end;
	  end;
	  dec(pSearch, pred(iBeg));
	  iBeg := succ(iEnd);
	end;
	if Result = true then wpHandle := 0;
      end;
    end;
  end;
end;

function FindAllChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowList := Children;
      fcType := fcAll;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpWindowList.Count;
  end;
end;

function FindDisabledChildren(ParentWindow: THandle; const ClassName: String; const Children: TList): Longint;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowList := Children;
      wpDisabled := true;
      fcType := fcAll;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpWindowList.Count;
  end;
end;

function FindInvisibleChild(ParentWindow: THandle; const ClassName, WindowText: String): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0  then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpClassName := UpperCase(ClassName);
      wpWindowText := WindowText;
      wpInvisible := true;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function FindFirstChild(ParentWindow: THandle; const ClassName, WindowText: String;
			CtlID: Integer; RegExClass: Longbool): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0 then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpWindowText := DelAmp(WindowText);
      wpClassName := UpperCase(ClassName);
      wpControlID := CtlID;
      wpCheckRegEx := RegExClass;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function FindNextChild(ParentWindow: THandle; const ClassName, WindowText: String;
		       CtlID: Integer; RegExClass: Longbool): THandle;
var
  wpSearch: TWindowParams;
begin
  Result := 0;
  if ParentWindow <> 0 then begin
    FillByte(wpSearch, SizeOf(TWindowParams), 0);
    with wpSearch do begin
      wpWindowText := DelAmp(WindowText);
      wpClassName := UpperCase(ClassName);
      wpControlID := CtlID;
      wpCheckRegEx := RegExClass;
      fcType := fcNext;
    end;
    EnumChildWindows(ParentWindow, @CheckChildWindows, LPARAM(@wpSearch));
    Result := wpSearch.wpHandle;
  end;
end;

function CheckButtons(FoundWindow: THandle; WP: PWindowParams): Bool; stdcall;
begin
  Result := true;
  with WP^ do begin
    if (GetWindowLong(FoundWindow, GWL_STYLE) and wpButtonStyle <> 0) and (ClassString(FoundWindow) = BUTTON)
    and ((wpWindowText = E) or (FirstStrPos(wpWindowText, DelAmp(WndString(FoundWindow))) <> 0)) then begin
      wpHandle := FoundWindow;
      Result := false;
    end;
  end;
end;

function FindButton(ParentWindow: THandle; const WindowText: String; Style: DWord): THandle;
var
  wpSearch: TWindowParams;
begin
  if WindowText <> E then
    Result := FindFirstChild(ParentWindow, BUTTON, WindowText, 0, false)
  else begin
    Result := 0;
    if ParentWindow <> 0 then begin
      FillByte(wpSearch, SizeOf(TWindowParams), 0);
      with wpSearch do begin
	wpWindowText := DelAmp(WindowText);
	wpButtonStyle := Style;
      end;
      EnumChildWindows(ParentWindow, @CheckButtons, LPARAM(@wpSearch));
      Result := wpSearch.wpHandle;
    end;
  end;
end;

function FindCheckBox(Dialog: THandle; const WindowText: String): THandle;
begin
  Result := FindButton(Dialog, WindowText, BS_CHECKBOX);
end;

function FindComboBox(Dialog: THandle): THandle;
begin
  Result := FindFirstChild(Dialog, COMBOBOX, E, 0, false);
end;

function  FindCtrlByID(Dialog: THandle; CtlID: Integer): THandle;
begin
  Result := FindFirstChild(Dialog, EDIT, E, 0, false);
end;

function FindEditControl(Dialog: THandle): THandle;
begin
  Result := FindFirstChild(Dialog, EDIT, E, 0, false);
end;

function FindStatic(Dialog: THandle; const WindowText: String): THandle;
begin
  Result := FindFirstChild(Dialog, STATIC, WindowText, 0, false);
end;

procedure ClickDlgBtn(Dialog: THandle; BtnID: Integer);
begin
  SendDlgItemMessage(Dialog, BtnID, BM_CLICK, 0, 0);
end;

function GetDlgBtnChecked(Dialog: THandle; BtnID: Integer): Integer;
begin
  Result := SendDlgItemMessage(Dialog, BtnID, BM_GETCHECK, 0, 0);
end;

function SetDialogCheckBox(CheckBox: THandle; Check: Longbool): Longbool;
begin
  Result := true;
  if Check then
    SendMessage(CheckBox, BM_SETCHECK, BST_CHECKED, 0)
  else SendMessage(CheckBox, BM_SETCHECK, BST_UNCHECKED, 0);
end;

function IsDialogCheckBoxChecked(CheckBox: THandle): Longbool;
begin
  Result := (SendMessage(CheckBox, BM_GETCHECK, 0, 0) = BST_CHECKED);
end;

function ClickDialogCheckBoxChecked(CheckBox: THandle): Longbool;
begin
  Result := true;
  if SendMessage(CheckBox, BM_GETCHECK, 0, 0) = BST_CHECKED then SendMessage(CheckBox, BM_SETCHECK, BST_UNCHECKED, 0);
  SendMessage(CheckBox, BM_CLICK, 0, 0);
end;

function CheckMenuItem(Menu: THandle; ItemID: Integer; Check: Longbool): Longbool;
begin
  Result := (Windows.CheckMenuItem(Menu, ItemID, MF_BYCOMMAND or (MF_CHECKED * (ord(Check = true)))) <> $FFFFFFFF)
end;

function SelectCBXItemIndex(ComboBox: THandle; Index: Integer): Longbool;
begin
  Result := (SendMessage(ComboBox, CB_SETCURSEL, Index, 0) <> CB_ERR);
end;

function GetCBXItemIndex(ComboBox: THandle; const Item: String): Integer;
begin
  Result := CB_ERR;
  if Item <> E then Result := ord(SendMessage(ComboBox, CB_FINDSTRINGEXACT, -1, Longint(Item)));
end;

function DeleteCBXIndex(ComboBox: THandle; Index: Integer): Integer;
begin
  Result := CB_ERR;
  if Index <> CB_ERR then Result := SendMessage(ComboBox, CB_DELETESTRING, Index, 0);
end;

function SelectCBXIndex(ComboBox: THandle; Index: Integer): Integer;
begin
  Result := CB_ERR;
  if Index <> CB_ERR then Result := SendMessage(ComboBox, CB_SETCURSEL, Index, 0);
end;

function GetSelectedCBXIndex(ComboBox: THandle): Integer;
begin
  Result := SendMessage(ComboBox, CB_GETCURSEL, 0, 0);
end;

function GetSelectedCBXItem(ComboBox: THandle): String;
var
  iSelected: Integer;
begin
  iSelected := SendMessage(ComboBox, CB_GETCURSEL, 0, 0);
  if iSelected > 0 then begin
    SetString(Result, nil, SendMessage(ComboBox, CB_GETLBTEXTLEN, iSelected, 0));
    Delete(Result, succ(SendMessage(ComboBox, CB_GETLBTEXT, iSelected, Longint(Result))), MAXINT);
  end;
end;

function GetLBXText(ListBox: THandle): String;
var
  iCount, iLength, iIndex: Integer;
  pBuffer: PChar;
begin
  iCount := SendMessage(ListBox, LB_GETCOUNT, 0, 0);
  if iCount > 0 then begin
    iLength := 0;
    for iIndex := 0 to pred(iCount) do inc(iLength, SendMessage(ListBox, LB_GETTEXTLEN, iIndex, 0));
    SetString(Result, nil, iLength + iCount);
    pBuffer := Pointer(Result);
    for iIndex := 0 to pred(iCount) do begin
      iLength := SendMessage(ListBox, LB_GETTEXT, iIndex, Longint(pBuffer));
      if iLength <> LB_ERR then begin
	inc(pBuffer, iLength);
	pBuffer^ := NULL;
	inc(pBuffer);
      end;
    end;
    Delete(Result, succ(pBuffer - Pointer(Result)), MAXINT);
    Exit;
  end;
  Result := E;
end;

function GetMDIParent(Window: THandle): THandle;
begin
  Result := Window;
  while (Result <> 0) and (GetWindowLong(Result, GWL_EXSTYLE) and WS_EX_MDICHILD = 0) do begin
    Result := GetParent(Result);
  end;
end;

function IsMDIWindowChild(MDIClient, Window: THandle): Longbool;
begin
  Result := (GetWindowLong(Window, GWL_EXSTYLE) and WS_EX_MDICHILD <> 0) or
	    IsChild(SendMessage(MDIClient, WM_MDIGETACTIVE, 0, 0), Window);

end;

function IsModuleWindow(Window: THandle): Longbool;
begin
  Result := (THandle(GetWindowLong(Window, GWL_HINSTANCE)) = hInstance);
end;

procedure SendKey(Window: THandle; Key: Integer);
begin
  SendMessage(Window, WM_KEYDOWN, Key, 0);
  SendMessage(Window, WM_KEYUP, Key, LPARAM($C0000000));
end;

begin
  GradientFill := GetProcAddress(GetModuleHandle('MSIMG32.DLL'), 'GradientFill');
end.

