{$J+,Z4}
unit KeyDialog;

{------------------------------------------------------------------------------}
{                                                                              }
{                                 This code is                                 }
{              Copyright (C) 2001-2023 by Michael in der Wiesche               }
{                                                                              }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, CommCtrl, Classes, Controls, Graphics, SysUtils, StrTools, WindowUtils, StrUtil,
  RichEd20, TreeViewEx, HelpCtrl, ToolTip, KeyPropTypes, KeyFuncs, X509Types, X509Funcs, CertStore;

type
  TSelKeyResult = (skCancel, skOk, skServer, skUseAdd);
  TRangeItem = Record
    Handle: HTreeItem;
    Param: Integer;
  end;
  TSelRange = Record
    FirstItem: TRangeItem;
    LastItem: TRangeItem;
  end;
  TSelKeyDialog = class(TObject)
  private
    FBoldFont: THandle;
    FPrompt: String;
    FShowDetails: String;
    FEditText: String;
    FGroupText: String;
    FServerBtnText: String;
    FUseAddBtnText: String;
    FCancelBtnText: String;
    FOkBtnText: String;
    FSelectedKeyText: String;
    FSelectAllBtnText: String;
    FUnselectBtnText: String;
    FRevokedKeyText: String;
    FDisabledKeyText: String;
    FExpiredKeyText: String;
    FCertificateText: String;
    FCertifiedText: String;
    FLegacyKeyText: String;
    FVerifiedText: String;
    FInputID: String;
    FOutputID: String;
    FHelpHandler: THelpHandler;
    FHelpIndex: Integer;
    FSMime: Longbool;
    FSelRange: TSelRange;
    FSelecting: Longbool;
    FMultiSelect: Longbool;
    FInitialized: Longbool;
    FTVMsgProc: Pointer;
    FImages: TImageList;
    FStateImages: TImageList;
    FToolTip: THandle;
    FTreeView: THandle;
    FDialogHandle: THandle;
    FParentHandle: THandle;
    FLTextLabelHandle: THandle;
    FRTextLabelHandle: THandle;
    FEditHandle: THandle;
    FServerBtnHandle: THandle;
    FUseAddBtnHandle: THandle;
    FCancelBtnHandle: THandle;
    FOkBtnHandle: THandle;
    FPreSelected: HTreeItem;
    FKeyPropsList: TKeyPropsList;
    FGroupHexIDs: TStringList;
    FGroupSelected: Longbool;
    function  GetSelectionRange: TSelRange;
    function  GetSelectedCount: Integer;
    function  GetSelectedItem: HTreeItem;
    function  GetFirstItem: HTreeItem;
    function  GetLastItem: HTreeItem;
    function  GetNextItem(hItem: HTreeItem): HTreeItem;
    function  GetPrevItem(hItem: HTreeItem): HTreeItem;
    function  GetValidItem(hItem: HTreeItem): HTreeItem;
    function  ItemParam(hItem: HTreeItem): Longint;
    function  ItemSelected(hItem: HTreeItem): Longbool;
    function  GetAlias(const sUserID: String): String;
    function  GetEmail(var sUserID: String): String;
    function  Cooked(const sUserID: String): String;
    function  FindCookedItem: Pointer;
    procedure UpdateItemCount;
    procedure ProcessMessages;
    procedure CenterDialog;
    procedure CenterItem;
    procedure ShowHelp;
    procedure CollapseAll;
    procedure SelectAll;
    procedure UnselectAll(hFocus: HTreeItem);
    procedure MoveFocus(lDirection: Longint);
    procedure ShiftSelect(hNewTarget: HTreeItem);
    procedure ControlSelect(hNewTarget: HTreeItem);
    procedure SelectRange(hFirst, hLast: HTreeItem);
    procedure SelectSingleItem(hNewTarget: HTreeItem);
    procedure UnselectRange(hFirst, hLast: HTreeItem);
    function  InitChange(Old, New: TTVItem): TNMTreeView;
    function  Changing(Old, New: HTreeItem): Longbool;
    function  Changed(Old, New: HTreeItem): Longbool;
    function  TVKeyDown(var Message: TWMKey): Longbool;
    function  TVKeyUp(var Message: TWMKey): Longbool;
    function  TVLeftDown(var Message: TWMMouse): Longbool;
    function  TVLeftUp(var Message: TWMMouse): Longbool;
    function  TVRightDown(var Message: TWMMouse): Longbool;
    function  TVRightUp(var Message: TWMMouse): Longbool;
    function  TVDoubleClick(var Message: TWMMouse): Longbool;
    function  GetKeyInfo(iKeyIndex: Integer; hItem: HTreeItem; ShowCertDialog: Longbool): String;
    function  GetGroupInfo(hGroupItem: HTreeItem): String;
    procedure ShowFixedToolTip(pptPos: PPoint);
    procedure HideToolTip;
    function  CreateTreeView: HTreeItem;
    function  DestroyTreeView: HTreeItem;
    function  LoadEntries: HTreeItem;
    function  ShowKeyDialog: Longint;
    function  InitDialog(hDialog: THandle): Longint;
    function  NotifyDialog(lData: Longint): Longint;
    function  ButtonInit(hButton: THandle): Longint;
    function  ButtonCommand(lButtonID: Longint): Longint;
  protected
  public
    KeyTrustsText: TKeyTrusts;
    KeyValiditiesText: TKeyValidities;
    KeyAlgorithmsText: TKeyAlgorithms;
    HashAlgorithmsText: THashAlgorithms;
    CertValitidiesText: TCertValidities;
    constructor CreateDialog(hParent: THandle);
    destructor Destroy; override;
    function  SelectEncryptKey(var EncryptKeyID: String;
			       const KeyPropsList: TKeyPropsList;
			       var GroupSelected: Longbool): TSelKeyResult;
    property Prompt: String read FPrompt write FPrompt;
    property ShowDetails: String read FShowDetails write FShowDetails;
    property EditText: String read FEditText write FEditText;
    property GroupText: String read FGroupText write FGroupText;
    property ServerBtnText: String read FServerBtnText write FServerBtnText;
    property UseAddBtnText: String read FUseAddBtnText write FUseAddBtnText;
    property CancelBtnText: String read FCancelBtnText write FCancelBtnText;
    property OkBtnText: String read FOkBtnText write FOkBtnText;
    property SelectedKeyText: String read FSelectedKeyText write FSelectedKeyText;
    property SelectAllBtnText: String read FSelectAllBtnText write FSelectAllBtnText;
    property UnselectBtnText: String read FUnselectBtnText write FUnselectBtnText;
    property RevokedKeyText: String read FRevokedKeyText write FRevokedKeyText;
    property DisabledKeyText: String read FDisabledKeyText write FDisabledKeyText;
    property ExpiredKeyText: String read FExpiredKeyText write FExpiredKeyText;
    property CertificateText: String read FCertificateText write FCertificateText;
    property CertifiedText: String read FCertifiedText write FCertifiedText;
    property HelpHandler: THelpHandler read FHelpHandler write FHelpHandler;
    property HelpIndex: Integer read FHelpIndex write FHelpIndex;
    property MultiSelect: Longbool read FMultiSelect write FMultiSelect;
    property SMime: Longbool read FSMime write FSMime;
  end;


implementation

{$R *.RES}
{$I *.INC}

function ShiftState: Integer;
begin
  Result := (ord(GetKeyState(VK_SHIFT) < 0) * MK_SHIFT) + (ord(GetKeyState(VK_CONTROL) < 0) * MK_CONTROL);
end;

constructor TSelKeyDialog.CreateDialog(hParent: THandle);
begin
  inherited Create;
  InitCommonControls;
  FParentHandle := hParent;
  FImages := TImageList.CreateSize(18, 13);
  FStateImages := TImageList.CreateSize(13, 13);
end;

destructor TSelKeyDialog.Destroy;
begin
  FStateImages.Free;
  FImages.Free;
  inherited;
end;

function GetRangeItem(AHandle: HTreeItem; AParam: Integer): TRangeItem;
begin
  with Result do begin
    Handle := AHandle;
    Param := AParam;
  end;
end;

function TSelKeyDialog.GetSelectionRange: TSelRange;

  function InitSelection: TSelRange;
  var
    hItem: HTreeItem;
    lParam: Longint;
  begin
    with Result do begin
      hItem := GetValidItem(GetSelectedItem);
      if hItem <> nil then begin
	lParam := ItemParam(hItem);
	FirstItem := GetRangeItem(hItem, lParam);
	LastItem := GetRangeItem(hItem, lParam);
      end
      else begin
	FirstItem := GetRangeItem(nil, -1);
	LastItem := GetRangeItem(nil, -1);
      end;
    end;
  end;

var
  tvNext, tvPrev: HTreeItem;
begin
  Result := InitSelection;
  tvNext := GetFirstItem;
  while tvNext <> nil do begin
    if ItemSelected(tvNext) then begin
      Result.FirstItem := GetRangeItem(tvNext, ItemParam(tvNext));
      Break;
    end;
    tvNext := GetNextItem(tvNext);
  end;
  tvPrev := GetLastItem;
  while tvPrev <> nil do begin
    if ItemSelected(tvPrev) then begin
      Result.LastItem := GetRangeItem(tvPrev, ItemParam(tvPrev));
      Break;
    end;
    tvPrev := GetPrevItem(tvPrev);
  end;
end;

function TSelKeyDialog.GetSelectedCount: Integer;
var
  tvItem: TTVItem;
  hParent: HTreeItem;
begin
  Result := 0;
  ProcessMessages;
  with tvItem do begin
    Mask := TVIF_HANDLE or TVIF_STATE;
    StateMask := TVIS_EXPANDED or TVIS_SELECTED;
    hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_ROOT, 0));
    while hItem <> nil do begin
      SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem));
      if (State and TVIS_SELECTED) <> 0 then
	inc(Result)
      else if (State and TVIS_EXPANDED) <> 0 then begin
	hParent := hItem;
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CHILD, Longint(hItem)));
	while hItem <> nil do begin
	  SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem));
	  if (State and TVIS_SELECTED) <> 0 then begin
	    inc(Result);
	    Break;
	  end;
	  hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)));
	end;
	hItem := hParent;
      end;
      hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)));
    end;
  end;
end;

function TSelKeyDialog.GetSelectedItem: HTreeItem;
begin
  Result := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CARET, 0));
end;

function TSelKeyDialog.GetFirstItem: HTreeItem;
begin
  Result := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_ROOT, 0));
end;

function TSelKeyDialog.GetLastItem: HTreeItem;
var
  hItem: HTreeItem;
begin
  Result := nil;
  hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_ROOT, 0));
  while hItem <> nil do begin
    Result := hItem;
    hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(Result)));
  end;
end;

function TSelKeyDialog.GetNextItem(hItem: HTreeItem): HTreeItem;
begin
  Result := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(GetValidItem(hItem))));
end;

function TSelKeyDialog.GetPrevItem(hItem: HTreeItem): HTreeItem;
begin
  Result := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_PREVIOUS, Longint(GetValidItem(hItem))));
end;

function TSelKeyDialog.GetValidItem(hItem: HTreeItem): HTreeItem;
begin
  Result := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_PARENT, Longint(hItem)));
  if Result = nil then Result := hItem;
end;

function TSelKeyDialog.ItemParam(hItem: HTreeItem): Longint;
var
  tvItem: TTVItem;
begin
  tvItem.hItem := hItem;
  tvItem.Mask := TVIF_PARAM;
  if SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0 then
    Result := tvItem.lParam
  else Result := -1;
end;

function TSelKeyDialog.ItemSelected(hItem: HTreeItem): Longbool;
var
  tvItem: TTVItem;
begin
  tvItem.hItem := hItem;
  tvItem.Mask := TVIF_STATE;
  tvItem.StateMask := TVIS_SELECTED;
  if SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0 then
    Result := (tvItem.State and TVIS_SELECTED <> 0)
  else Result := false;
end;

function TSelKeyDialog.GetAlias(const sUserID: String): String;
var
  iPos: Integer;
begin
  iPos := FirstCharPos(AT, sUserID);
  if iPos > 1 then
    Result := Copy(sUserID, 1, pred(iPos))
  else Result := sUserID;
end;

function TSelKeyDialog.GetEmail(var sUserID: String): String;
var
  iBegPos, iEndPos, iLen: Integer;
begin
  iBegPos := FirstCharPos(LT, sUserID);
  if iBegPos > 0 then begin
    iEndPos := LastCharPos(GT, sUserID);
    if iEndPos > iBegPos then begin
      Result := Copy(sUserID, succ(iBegPos), pred(iEndPos - iBegPos));
      iEndPos := pred(iBegPos);
      while (iEndPos > 0) and (sUserID[iEndPos] <= SP) do dec(iEndPos);
      iBegPos := 1;
      while (iBegPos < iEndPos) and (sUserID[iBegPos] <= SP) do inc(iBegPos);
      Delete(sUserID, succ(iEndPos), MAXINT);
      if iBegPos > 1 then Delete(sUserID, 1, pred(iBegPos));
    end
    else Result := E;
  end
  else Result := E;
  if Result = E then begin
    iBegPos := FirstCharPos(AT, sUserID);
    if iBegPos > 0 then begin
      iLen := Length(sUserID);
      iEndPos := succ(iBegPos);
      while (iEndPos <= iLen) and (sUserID[iEndPos] > SP) do inc(iEndPos);
      while (iBegPos > 0) and (sUserID[iBegPos] > SP) do dec(iBegPos);
      Result := Copy(sUserID, succ(iBegPos), iEndPos - iBegPos);
    end;
  end;
end;

function TSelKeyDialog.Cooked(const sUserID: String): String;
var
  iPos: Integer;
  pResult: PChar;
begin
  if sUserID <> E then begin
    Result := ANSILowerCase(sUserID);
    pResult := Pointer(Result);
    for iPos := 0 to pred(Length(Result)) do if not IsCharAlphaNumeric(pResult[iPos]) then pResult[iPos] := SP;
  end
  else Result := E;
end;

function TSelKeyDialog.FindCookedItem: Pointer;
var
  iCount, iIndex, iPos: Integer;
  slEmail, slAlias: TStringList;
  sCooked, sBuffer, sFirstName: String;
begin
  Result := nil;
  iCount := FKeyPropsList.Count;
  sCooked := Cooked(GetEmail(FInputID));
  if (sCooked <> E) and (iCount <> 0) then begin
    slEmail := nil;
    slAlias := nil;
    try
      slEmail := TStringList.Create;
      slAlias := TStringList.Create;
      if (slEmail <> nil) and (slAlias <> nil) then begin
	with slEmail do begin
	  Duplicates := dupAccept;
	  Sorted := true;
	end;
	with slAlias do begin
	  Duplicates := dupAccept;
	  Sorted := true;
	end;
	for iIndex := 0 to pred(iCount) do with pKeyPropsRec(FKeyPropsList.Objects[iIndex])^.kUserIDList do begin
	  for iPos := 0 to pred(Count) do begin
	    sBuffer := CertOwners[iPos];
	    slEmail.AddObject(Cooked(GetEmail(sBuffer)), Objects[iPos]);
	    slAlias.AddObject(Cooked(sBuffer), Objects[iPos]);
	  end;
	end;
	if slEmail.Find(sCooked, iIndex) then
	  Result := slEmail.Objects[iIndex]
	else begin
	  sCooked := Cooked(GetAlias(FInputID));
	  if slEmail.Find(sCooked, iIndex) then
	    Result := slEmail.Objects[iIndex]
	  else with slAlias do begin
	    if Find(sCooked, iIndex) then
	      Result := Objects[iIndex]
	    else begin
	      repeat
		iIndex := 0;
		while iIndex < Count do begin
		  sBuffer := Strings[iIndex];
		  if LoStrAtPos(sCooked, sBuffer, 1) then begin
		    Result := Objects[iIndex];
		    Exit;
		  end
		  else if Length(sCooked) > 1 then begin
		    iPos := LastCharPos(SP, sBuffer);
		    if LoStrAtPos(sCooked, sBuffer, succ(iPos)) then begin
		      Result := Objects[iIndex];
		      Exit;
		    end
		    else inc(iIndex);
		  end
		  else inc(iIndex);
		end;
		iPos := LastCharPos(SP, sCooked);
		if iPos <> 0 then begin
		  if sFirstName = E then begin
		    sFirstName := Copy(sCooked, 1, pred(iPos));
		    System.Delete(sCooked, 1, iPos);
		  end
		  else begin
		    System.Delete(sCooked, iPos, MAXINT);
		    sFirstName := sCooked;
		  end;
		end
		else begin
		  if sCooked <> sFirstName then begin
		    sCooked := sFirstName;
		    if sCooked <> E then
		      Continue
		    else Exit;
		  end;
		  iPos := Length(sCooked);
		  if iPos > 1 then begin
		    System.Delete(sCooked, iPos, 1);
		    sFirstName := sCooked;
		  end
		  else Exit;
		end;
	      until Result <> nil;
	    end;
	  end;
	end;
      end;
    finally
      slEmail.Free;
      slAlias.Free;
    end;
  end;
end;

procedure TSelKeyDialog.UpdateItemCount;
var
  iSelCount: Integer;
begin
  iSelCount := GetSelectedCount;
  if not FMultiSelect then EnableWindow(FUseAddBtnHandle, iSelCount > 0);
  EnableWindow(FOkBtnHandle, iSelCount > 0);
  SendMessage(FServerBtnHandle, BM_SETSTYLE, 0, 1);
  SendMessage(FUseAddBtnHandle, BM_SETSTYLE, 0, 1);
  SendMessage(FOkBtnHandle, BM_SETSTYLE, BS_DEFPUSHBUTTON, 1);
  if FMultiSelect then SetWindowText(FRTextLabelHandle, PChar(FSelectedKeyText + SP + Format('%.4d', [iSelCount])));
end;

procedure TSelKeyDialog.ProcessMessages;
var
  Msg: TMsg;
begin
  while PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
end;

procedure TSelKeyDialog.CenterDialog;
var
  rDialog, rParent: TRect;
begin
  if FParentHandle = 0 then FParentHandle := GetDesktopWindow;
  rParent := GetWindowRect(FParentHandle);
  with rParent do begin
    dec(Right, pred(Left));
    dec(Bottom, pred(Top));
  end;
  rDialog := GetWindowRect(FDialogHandle);
  with rDialog do begin
    dec(Right, pred(Left));
    dec(Bottom, pred(Top));
    Left := rParent.Left + (rParent.Right - Right) div 2;
    Top := rParent.Top + (rParent.Bottom - Bottom) div 2;
    SetWindowPos(FDialogHandle, HWND_TOP, Left, Top, 0, 0, SWP_DRAWFRAME or SWP_NOSIZE);
  end;
end;

procedure TSelKeyDialog.CenterItem;
var
  iHalfPageDown: Integer;
  hSelected: HTreeItem;
begin
  hSelected := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CARET, 0));
  if hSelected = HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_FIRSTVISIBLE, 0)) then begin
    iHalfPageDown := GetScrollPos(FTreeView, SB_VERT) - (SendMessage(FTreeView, TVM_GETVISIBLECOUNT, 0, 0) div 2);
    if iHalfPageDown < 0 then iHalfPageDown := 0;
    PostMessage(FTreeView, WM_VSCROLL, SB_THUMBPOSITION + (iHalfPageDown shl 16), 0);
  end
  else if hSelected <> nil then PostMessage(FTreeView, WM_VSCROLL, SB_PAGEDOWN, 0);
end;

procedure TSelKeyDialog.ShowHelp;
begin
  if FHelpHandler <> nil then FHelpHandler.Topic(FDialogHandle, FHelpIndex);
end;

procedure TSelKeyDialog.CollapseAll;
var
  hItem: HTreeItem;
begin
  if FMultiSelect then begin
    hItem := GetFirstItem;
    while hItem <> nil do begin
      SendMessage(FTreeView, TVM_EXPAND, TVE_COLLAPSE, Longint(hItem));
      hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)))
    end;
  end;
end;

procedure TSelKeyDialog.SelectAll;
begin
  if FMultiSelect then begin
    if Changing(nil, nil) then begin
      try
	SelectRange(GetFirstItem, nil);
	UpdateItemCount;
      finally
	Changed(nil, nil);
      end;
    end;
  end;
end;

procedure TSelKeyDialog.UnselectAll(hFocus: HTreeItem);
begin
  if FMultiSelect then begin
    if Changing(nil, nil) then begin
      try
	UnselectRange(GetFirstItem, nil);
	TreeView_SelectItem(FTreeView, nil);
	TreeView_SelectItem(FTreeView, hFocus);
	UpdateItemCount;
      finally
	Changed(nil, nil);
      end;
    end;
  end;
end;

function InitItemData(Handle: HTreeItem; Selected: Cardinal): TTVItem;
begin
  with Result do begin
    Mask := TVIF_HANDLE or TVIF_STATE;
    StateMask := TVIS_SELECTED;
    State := Selected;
    hItem := Handle;
  end;
end;

procedure TSelKeyDialog.MoveFocus(lDirection: Longint);
var
  tvTarget, tvOrigin: TTVItem;
begin
  tvOrigin := InitItemData(GetValidItem(GetSelectedItem), TVIS_SELECTED);
  if tvOrigin.hItem <> nil then begin
    case lDirection of
      TVGN_NEXT: with tvOrigin do begin
	if hItem = GetLastItem then Exit;
	tvTarget := InitItemData(GetNextItem(hItem), TVIS_SELECTED);
      end;
      TVGN_PREVIOUS: with tvOrigin do begin
	if hItem = GetFirstItem then Exit;
	tvTarget := InitItemData(GetPrevItem(hItem), TVIS_SELECTED);
      end;
    else
      Exit;
    end;
    if Changing(nil, nil) then begin
      try
	TreeView_GetItem(FTreeView, tvOrigin);
	TreeView_GetItem(FTreeView, tvTarget);
	TreeView_SelectItem(FTreeView, nil);
	TreeView_SelectItem(FTreeView, tvTarget.hItem);
	TreeView_EnsureVisible(FTreeView, tvTarget.hItem);
	TreeView_SetItem(FTreeView, tvTarget);
	TreeView_SetItem(FTreeView, tvOrigin);
      finally
	Changed(nil, nil);
      end;
    end;
  end;
end;

procedure TSelKeyDialog.ShiftSelect(hNewTarget: HTreeItem);
var
  tvFirst, tvLast: HTreeItem;
  lParam: Longint;
begin
  if hNewTarget <> nil then begin
    if Changing(nil,  nil) then begin
      try
	hNewTarget := GetValidItem(hNewTarget);
	lParam := ItemParam(hNewTarget);
	FSelRange := GetSelectionRange;
	with FSelRange do begin
	  if lParam < FirstItem.Param then
	    FirstItem := GetRangeItem(hNewTarget, lParam)
	  else if lParam > LastItem.Param then
	    LastItem := GetRangeItem(hNewTarget, lParam)
	  else begin
	    if lParam > ItemParam(GetValidItem(GetSelectedItem)) then
	      FirstItem := GetRangeItem(hNewTarget, lParam)
	    else LastItem := GetRangeItem(hNewTarget, lParam);
	  end;
	  tvFirst := GetFirstItem;
	  tvLast := GetLastItem;
	  if FirstItem.Param > ItemParam(tvFirst) then UnselectRange(tvFirst, GetPrevItem(FirstItem.Handle));
	  if LastItem.Param < ItemParam(tvLast) then UnselectRange(GetNextItem(LastItem.Handle), tvLast);
	  TreeView_SelectItem(FTreeView, nil);
	  TreeView_SelectItem(FTreeView, hNewTarget);
	  TreeView_EnsureVisible(FTreeView, hNewTarget);
	  SelectRange(FirstItem.Handle, LastItem.Handle);
	end;
      finally
	Changed(nil, nil);
      end;
    end;
  end;
end;

procedure TSelKeyDialog.ControlSelect(hNewTarget: HTreeItem);
var
  tvOrigin, tvTarget: TTVItem;
begin
  if hNewTarget <> nil then begin
    if Changing(nil, nil) then begin
      try
	hNewTarget := GetValidItem(hNewTarget);
	tvOrigin := InitItemData(GetValidItem(GetSelectedItem), 0);
	tvTarget := InitItemData(hNewTarget, 0);
	TreeView_GetItem(FTreeView, tvOrigin);
	TreeView_GetItem(FTreeView, tvTarget);
	TreeView_SelectItem(FTreeView, nil);
	TreeView_SelectItem(FTreeView, hNewTarget);
	TreeView_EnsureVisible(FTreeView, hNewTarget);
	with tvTarget do State := (State xor TVIS_SELECTED);
	TreeView_SetItem(FTreeView, tvTarget);
	if tvOrigin.hItem <> tvTarget.hItem then TreeView_SetItem(FTreeView, tvOrigin);
      finally
	Changed(nil, nil);
      end;
    end;
  end;
end;

procedure TSelKeyDialog.SelectRange(hFirst, hLast: HTreeItem);
var
  tvItem: TTVItem;
begin
  tvItem := InitItemData(GetValidItem(hFirst), TVIS_SELECTED);
  hLast := GetValidItem(hLast);
  with tvItem do begin
    while hItem <> nil do begin
      SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
      if hItem <> hLast then
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)))
      else Break;
    end;
  end;
end;

procedure TSelKeyDialog.SelectSingleItem(hNewTarget: HTreeItem);
begin
  if hNewTarget = GetSelectedItem then Exit;
  if Changing(nil, nil) then begin
    try
      TreeView_SelectItem(FTreeView, nil);
      TreeView_SelectItem(FTreeView, hNewTarget);
    finally
      Changed(nil, nil);
    end;
  end;
end;

procedure TSelKeyDialog.UnselectRange(hFirst, hLast: HTreeItem);
var
  tvItem: TTVItem;
  hParent: HTreeItem;
begin
  tvItem := InitItemData(hFirst, 0);
  with tvItem do begin
    while hItem <> nil do begin
      SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
      hParent := hItem;
      try
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CHILD, Longint(hItem)));
	while hItem <> nil do begin
	  SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
	  if hItem <> hLast then
	    hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)))
	  else Break;
	end;
      finally
	hItem := hParent;
      end;
      if hItem <> hLast then
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)))
      else Break;
    end;
  end;
end;

function InitChangeData(Item: HTreeItem): TTVItem;
begin
  with Result do begin
    Mask := TVIF_HANDLE or TVIF_PARAM or TVIF_STATE;
    StateMask := TVIS_FOCUSED or TVIS_SELECTED;
    hItem := Item;
    lParam := 0;
    State := 0;
  end;
end;

function TSelKeyDialog.InitChange(Old, New: TTVItem): TNMTreeView;
begin
  with Result do begin
    with hdr do begin
      hwndFrom := FTreeView;
      idFrom := GetDlgCtrlID(FTreeView);
      code := 0;
    end;
    if GetKeyState(VK_LBUTTON) < 0 then
      action := TVC_BYMOUSE
    else action := TVC_BYKEYBOARD;
    itemOld := Old;
    itemNew := New;
    ptDrag := GetCursorPos;
    ScreenToClient(FTreeView, ptDrag);
  end;
end;

function TSelKeyDialog.Changing(Old, New: HTreeItem): Longbool;
var
  tvOld, tvNew: TTVItem;
  nmtv: TNMTreeView;
begin
  tvOld := InitChangeData(Old);
  tvNew := InitChangeData(New);
  nmtv := InitChange(tvOld, tvNew);
  nmtv.hdr.code := TVN_SELCHANGING;
  Result := (SendMessage(GetParent(FTreeView), WM_NOTIFY, nmtv.hdr.idFrom, Longint(@nmtv)) = 0);
  if Result then begin
    SendMessage(FTreeView, WM_SETREDRAW, 0, 0);
    FSelecting := true;
    CollapseAll;
  end;
end;

function TSelKeyDialog.Changed(Old, New: HTreeItem): Longbool;
var
  tvOld, tvNew: TTVItem;
  nmtv: TNMTreeView;
begin
  Result := (New <> Old);
  tvOld := InitChangeData(Old);
  tvNew := InitChangeData(New);
  nmtv := InitChange(tvOld, tvNew);
  nmtv.hdr.code := TVN_SELCHANGED;
  FSelecting := false;
  SendMessage(FTreeView, WM_SETREDRAW, 1, 0);
  SendMessage(GetParent(FTreeView), WM_NOTIFY, nmtv.hdr.idFrom, Longint(@nmtv));
end;

function TSelKeyDialog.TVKeyDown(var Message: TWMKey): Longbool;

  procedure RemoveKeyMsg;
  var
    msgKey: TMsg;
  begin
    PeekMessage(msgKey, FTreeView, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE);
  end;

begin
  HideToolTip;
  Result := false;
  if FMultiSelect then begin
    case ShiftState of
      0: UnselectAll(GetSelectedItem);
      MK_SHIFT: if Message.CharCode <> VK_SHIFT then begin
	case Message.CharCode of
	  VK_UP: ShiftSelect(GetPrevItem(GetSelectedItem));
	  VK_DOWN: ShiftSelect(GetNextItem(GetSelectedItem));
	end;
	Result := true;
	Exit;
      end;
      MK_CONTROL: begin
	case Message.CharCode of
	  VK_SPACE: begin
	    RemoveKeyMsg;
	    Result := true;
	    ControlSelect(GetValidItem(GetSelectedItem));
	    Exit;
	  end;
	  VK_UP: begin
	    MoveFocus(TVGN_PREVIOUS);
	    Result := true;
	    Exit;
	  end;
	  VK_DOWN: begin
	    MoveFocus(TVGN_NEXT);
	    Result := true;
	    Exit;
	  end;
	  ord('A'): begin
	    RemoveKeyMsg;
	    Result := true;
	    SelectAll;
	    Exit;
	  end;
	  ord('U'): begin
	    RemoveKeyMsg;
	    Result := true;
	    UnselectAll(nil);
	    Exit;
	  end;
	end;
      end;
    end;
  end;
  if (GetKeyState(VK_CONTROL) < 0) and (Message.CharCode = ord('I')) then begin
    RemoveKeyMsg;
    Result := true;
    ShowFixedToolTip(nil);
  end;
end;

function TSelKeyDialog.TVKeyUp(var Message: TWMKey): Longbool;
begin
  Result := false;
  UpdateItemCount;
end;

function TSelKeyDialog.TVLeftDown(var Message: TWMMouse): Longbool;
var
  htInfo: TTVHitTestInfo;
begin
  HideToolTip;
  Result := true;
  with htInfo do begin
    with Message do pt := Point(XPos, YPos);
    SendMessage(FTreeView, TVM_HITTEST, 0, Longint(@htInfo));
    case flags of
      TVHT_ONITEMLABEL: begin
	SetFocus(FTreeView);
	if FMultiSelect then begin
	  case ShiftState of
	    0: UnselectAll(hItem);
	    MK_SHIFT: begin
	      ShiftSelect(hItem);
	      Exit;
	    end;
	    MK_CONTROL: begin
	      ControlSelect(hItem);
	      Exit;
	    end;
	  else
	    UnselectAll(hItem);
	  end;
	end
	else SelectSingleItem(hItem);
      end;
      TVHT_ONITEMBUTTON: SendMessage(FTreeView, TVM_EXPAND, TVE_TOGGLE, Longint(hItem));
    end;
  end;
end;

function TSelKeyDialog.TVLeftUp(var Message: TWMMouse): Longbool;
begin
  Result := true;
  UpdateItemCount;
end;

function TSelKeyDialog.TVRightDown(var Message: TWMMouse): Longbool;
var
  ptPos: TPoint;
begin
  Result := true;
  with Message do begin
    ptPos := Point(XPos, YPos);
    ShowFixedToolTip(@ptPos);
  end;
end;

function TSelKeyDialog.TVRightUp(var Message: TWMMouse): Longbool;
begin
  Result := true;
end;

function TSelKeyDialog.TVDoubleClick(var Message: TWMMouse): Longbool;
var
  htInfo: TTVHitTestInfo;
begin
  Result := true;
  with htInfo do begin
    with Message do pt := Point(XPos, YPos);
    SendMessage(FTreeView, TVM_HITTEST, 0, Longint(@htInfo));
    case flags of
      TVHT_ONITEMLABEL: begin
	if FMultiSelect then begin
	  UnselectAll(GetValidItem(hItem));
	end
	else begin
	  SendMessage(FTreeView, TVM_SELECTITEM, TVGN_CARET, Longint(hItem));
	  PostMessage(GetParent(FTreeView), WM_COMMAND, ID_OK, 0);
	end;
      end;
      TVHT_ONITEMICON: SendMessage(FTreeView, TVM_EXPAND, TVE_TOGGLE, Longint(hItem));
    end;
  end;
end;

function TSelKeyDialog.GetKeyInfo(iKeyIndex: Integer; hItem: HTreeItem; ShowCertDialog: Longbool): String;

  function ExtractDate(const DateTime: String): String;
  var
    iBegPos, iEndPos: Integer;
  begin
    iBegPos := succ(FirstCharPos(CM, DateTime));
    iEndPos := pred(LastCharPos(CM, DateTime));
    while (iEndPos > iBegPos) and (DateTime[iEndPos] <= SP) do dec(iEndPos);
    while (iBegPos < iEndPos) and (DateTime[iBegPos] <= SP) do inc(iBegPos);
    Result := Copy(DateTime, iBegPos, succ(iEndPos - iBegPos));
  end;

  function CertificateInfo(const Key, User: String; ShowDialog: Longbool): String;
  var
    lCertProps: TX509CertPropsList;
    sCertIASN: String;
    sCertData: String;
    iCount: Integer;
    sKeyData: String;
    sValidity: String;
  begin
    Result := E;
    lCertProps := nil;
    try
      if FindX509CertProps(User, Key, E, KeyFilterFlag_X509Cert, false, lCertProps) > 0 then with lCertProps do begin
	if ShowDialog then begin
	  sCertIASN := Strings[0];
	  Clear;
	  if (ExportX509Certificate(E, sCertIASN, E, false, false, false, false, sCertData, lCertProps, 0) = 0) then begin
	    if (ShowCertificateDialog(sCertData, E, false, FDialogHandle) = 0) then Exit;
	  end;
	end;
	for iCount := 0 to pred(Count) do with pX509CertPropsRec(Objects[iCount])^ do begin
	  if x509IsRoot then
	    sValidity := CertValitidiesText[CertChain_Root]
	  else if x509IsVerified then
	    sValidity := CertValitidiesText[CertChain_Valid]
	  else sValidity := CertValitidiesText[CertChain_Invalid];
	  Result := Result + GetUserFromLongName(x509OwnerLongName, true) + CRLF + FCertifiedText + CRLF +
		    GetUserFromLongName(x509IssuerLongName, true) + PARA +
		    ExtractDate(x509CreaTimeStr) + ' - ' + ExtractDate(x509ExpTimeStr) + PARA +
		    HashAlgorithmsText[x509IssuerHashAlgorithm] + PARA +
		    sValidity + PARA;
	end;
	with pKeyPropsRec(FKeyPropsList.Objects[iKeyIndex])^ do begin
	  sKeyData := KeyAlgorithmsText[kKeyAlgorithm] + SP + LB + kHexID + RB + ' - ' + kSize  + PARA + kFingerprint;
	end;
	Result := Result + sKeyData;
      end;
    finally
      lCertProps.Free;
    end;
  end;

var
  sUserID: String;
  sValidTrust: String;
  iUserIndex: Integer;
begin
  if iKeyIndex >= 0 then with pKeyPropsRec(FKeyPropsList.Objects[iKeyIndex])^ do begin
    iUserIndex := kUserIDList.IndexOfObject(TObject(hItem));
    if iUserIndex = -1 then iUserIndex := 0;
    if FSMime and (kX509Cert > BadCert) then
      Result := CertificateInfo(FKeyPropsList[iKeyIndex], kUserIDList.CertOwners[iUserIndex], ShowCertDialog)
    else begin
      if kHasADK <> NoADK then
	sUserID := kUserIDList.CertOwners[iUserIndex] + CRLF + '>>>>>>>>>> A D K <<<<<<<<<<' + CRLF
      else sUserID := kUserIDList.CertOwners[iUserIndex] + CRLF;
      sValidTrust := KeyValiditiesText[kValidity] + CRLF + KeyTrustsText[kTrust] + PARA;
      Result := sUserID + FKeyPropsList[iKeyIndex] + CRLF + LB + kHexID + RB + PARA +
		KeyAlgorithmsText[kKeyAlgorithm] + ' - ' + kSize  + ' - ' + ExtractDate(kCreaTimeStr) + PARA +
		sValidTrust + kFingerprint;
    end;
  end;
end;

function TSelKeyDialog.GetGroupInfo(hGroupItem: HTreeItem): String;
var
  tvItem: TTVItem;
  iIndex: Integer;
begin
  tvItem.Mask := TVIF_PARAM;
  tvItem.hItem := hGroupItem;
  if (SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0) and (tvItem.lParam < 0) then begin
    with FGroupHexIDs do begin
      iIndex := tvItem.lParam + Count;
      Result := ANSIUpperCase(FGroupText) + CRLF + Copy(Strings[iIndex], succ(LastCharPos(TAB, Strings[iIndex])), MAXINT) + CRLF;
    end;
    with tvItem do begin
      hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CHILD, Longint(hItem)));
      while (hItem <> nil) and (SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0) do begin
	Result := Result + CRLF + pKeyPropsRec(FKeyPropsList.Objects[lParam])^.kUserID;
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)));
      end;
    end;
  end;
end;

procedure TSelKeyDialog.ShowFixedToolTip(pptPos: PPoint);
var
  htInfo: TTVHitTestInfo;
  tvitem: TTVItem;
  tvRect: TRect;
  sInfo: String;
  td: TTitleData;
begin
  HideToolTip;
  with htInfo do begin
    if pptPos <> nil then begin
      pt := pptPos^;
      if (TreeView_HitTest(FTreeView, htInfo) = nil) or (flags <> TVHT_ONITEMLABEL) then Exit;
    end
    else hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CARET, 0));
    tvItem.hItem := hItem;
    tvItem.Mask := TVIF_PARAM;
    if TreeView_GetItem(FTreeView, tvItem) and TreeView_GetItemRect(FTreeView, hItem, tvRect, TRUE) then begin
      if tvItem.lParam >= 0 then
	sInfo := GetKeyInfo(tvItem.lParam, hItem, true)
      else sInfo := GetGroupInfo(hItem);
      td.tdIcon := TTI_NONE;
      td.tdTitle := E;
      with tvRect do begin
	ClientToScreen(FTreeView, pt);
	FToolTip := CreateTrackingToolTip(FTreeView, FTreeView, tvRect, sInfo, GetClientWidth(FTreeView),
					  td, NIL, true, false, false);
	pt := Point(Left, Bottom);
	ClientToScreen(FTreeView, pt);
	SendMessage(FToolTip, WM_SETFONT, SendMessage(FTreeView, WM_GETFONT, 0, 0), 0);
	if FToolTip <> 0 then ShowTrackingToolTip(FToolTip, FTreeView, 0, E, pt, NIL);
      end;
    end;
  end;
end;

procedure TSelKeyDialog.HideToolTip;
begin
  if FToolTip <> 0 then begin
    DestroyTrackingToolTip(FToolTip, FTreeView, 0);
    FToolTip := 0;
  end;
end;

function TVMsgHook(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  Message: TMessage;
begin
  Result := 0;
  Message.Msg := Msg;
  Message.wParam := wParam;
  Message.lParam := lParam;
  Message.Result := Result;
  with TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)) do begin
    try
      case Msg of
	WM_KEYDOWN: if TVKeyDown(TWMKey(Message)) then Exit;
	WM_KEYUP: if TVKeyUp(TWMKey(Message)) then Exit;
	WM_LBUTTONDOWN: if TVLeftDown(TWMMouse(Message)) then Exit;
	WM_LBUTTONUP: if TVLeftUp(TWMMouse(Message)) then Exit;
	WM_LBUTTONDBLCLK: if TVDoubleClick(TWMMouse(Message)) then Exit;
	WM_RBUTTONDOWN: if TVRightDown(TWMMouse(Message)) then Exit;
	WM_RBUTTONUP: if TVRightUp(TWMMouse(Message)) then Exit;
      end;
      Message.Result := CallWindowProc(FTVMsgProc, hWnd, Msg, wParam, lParam);
    finally
      Result := Message.Result;
    end;
  end;
end;

function TSelKeyDialog.CreateTreeView: HTreeItem;
var
  rClient, rEdit, rOk: TRect;
  bmpBuffer: TBitmap;
begin
  Result := nil;
  rClient := GetClientRect(FDialogHandle);
  rEdit := GetWindowRect(FEditHandle);
  rOk := GetWindowRect(FOkBtnHandle);
  ClientToScreen(FDialogHandle, rClient.TopLeft);
  with rEdit do begin
    rClient.TopLeft := Point(Left - rClient.Left, Bottom - rClient.Top + 12);
    rClient.BottomRight := Point(Right - Left, rOk.Top - Bottom - 24);
  end;
  with rClient do begin
    FTreeView := CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, nil,
				WS_CHILD or WS_TABSTOP or WS_VISIBLE or TVS_DISABLEDRAGDROP or
				ord(not FSMime) * TVS_HASBUTTONS or TVS_LINESATROOT or TVS_SHOWSELALWAYS,
				Left, Top, Right, Bottom, FDialogHandle, 0, hInstance, nil);
  end;
  if FTreeView <> 0 then begin
    SetWindowLong(FTreeView, GWL_USERDATA, Longint(Self));
    FTVMsgProc := Ptr(SetWindowLong(FTreeView, GWL_WNDPROC, Longint(@TVMsgHook)));
    SendMessage(FTreeView, WM_SETFONT, SendMessage(FDialogHandle, WM_GETFONT, 0, 0), 0);
    if (FImages <> nil) and (FStateImages <> nil) then begin
      with FImages do begin
	AllocBy := 7;
	BkColor := clWindow;
      end;
      with FStateImages do begin
	AllocBy := 4;
	BkColor := clWindow;
      end;
      bmpBuffer := TBitmap.Create;
      if bmpBuffer <> nil then with bmpBuffer, Canvas do begin
	try
	  Width := 13;
	  Height := 13;
	  FStateImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, INVALID);
	  FStateImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, MIXED);
	  FStateImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, VALID);
	  FStateImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, GROUP);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, RSAL_KEY);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, RSA_KEY);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, DSA_KEY);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, RSA_USER);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, DSA_USER);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	  LoadFromResourceID(hInstance, CERT);
	  FImages.AddMasked(bmpBuffer, TransparentColor);
	finally
	  bmpBuffer.Free;
	end;
	Result := LoadEntries;
	SendMessage(FTreeView, TVM_SETIMAGELIST, TVSIL_NORMAL, FImages.Handle);
	SendMessage(FTreeView, TVM_SETIMAGELIST, TVSIL_STATE, FStateImages.Handle);
	rClient := GetClientRect(FTreeView);
      end;
    end;
  end;
end;

function TSelKeyDialog.DestroyTreeView: HTreeItem;
var
  tvItem: TTVItem;
  iCharPos: Integer;
  hParentItem: HTreeItem;
begin
  HideToolTip;
  Result := nil;
  FOutputID := E;
  DeleteObject(FBoldFont);
  if FTreeView <> 0 then begin
    with tvItem do begin
      if FMultiSelect then with FKeyPropsList do begin
	Mask := TVIF_HANDLE or TVIF_PARAM or TVIF_STATE;
	StateMask := TVIS_SELECTED;
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_ROOT, 0));
	while hItem <> nil do begin
	  SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem));
	  if (State and TVIS_SELECTED) <> 0 then begin
	    if lParam >= 0 then
	      FOutputID := FOutputID + Strings[lParam] + CM
	    else with FGroupHexIDs do begin
	      FOutputID := FOutputID + Strings[lParam + Count];
	      iCharPos := LastCharPos(TAB, FOutputID);
	      System.Delete(FOutputID, succ(iCharPos), MAXINT);
	      FOutputID[iCharPos] := CM;
	      FGroupSelected := true;
	    end;
	  end;
	  hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_NEXT, Longint(hItem)));
	end;
	System.Delete(FOutputID, Length(FOutputID), 1);
      end
      else begin
	Mask := TVIF_PARAM;
	hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CARET, 0));
	hParentItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_PARENT, Longint(hItem)));
	if hParentItem <> nil then hItem := hParentItem;
	if SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0 then begin
	  if lParam >= 0 then
	    FOutputID := FKeyPropsList[lParam]
	  else with FGroupHexIDs do begin
	    FOutputID := Strings[lParam + Count];
	    System.Delete(FOutputID, LastCharPos(TAB, FOutputID), MAXINT);
	    FGroupSelected := true;
	  end;
	  Result := hItem;
	end;
      end;
    end;
    DestroyWindow(FTreeView);
  end;
  BringWindowToTop(FParentHandle);
end;

function TSelKeyDialog.LoadEntries: HTreeItem;

  function GetAlgImageIndex(const rKeyProps: TKeyPropsRec): Longint;
  begin
    with rKeyProps do begin
      case kKeyAlgorithm of
	KeyAlgorithm_RSA..KeyAlgorithm_RSASignOnly: begin
	  if kLegacyKey then
	    Result := RSAL_KEY - GROUP
	  else Result := RSA_KEY - GROUP;
	end;
	KeyAlgorithm_DH..KeyAlgorithm_DHDSS: Result := DSA_KEY - GROUP;
      else
	Result := -1;
      end;
    end;
  end;

  procedure AddUserIDs(iIndex: Integer; var tvInsert: TTVInsertStruct; var hSelItem: HTreeItem);
  var
    tvItem: TTVItem;
    hNewItem: HTreeItem;
    iSubIndex: Integer;
    fMixedValidity: Longbool;
    pKeyProps: PKeyPropsRec;
    sKeyUser, sCertUser: String;
    cvCertificate: TX509ChainValidity;
  begin
    tvItem := tvInsert.Item;
    hNewItem := tvInsert.hParent;
    with tvItem do begin
      iSubIndex := 0;
      fMixedValidity := false;
      pKeyProps := pKeyPropsRec(FKeyPropsList.Objects[iIndex]);
      with pKeyProps^ do while iSubIndex < kUserIDList.Count do begin
	sKeyUser := kUserIDList[iSubIndex];
	if kUserIDList.X509Types[iSubIndex] > BadCert then begin
	  with kUserIDList do begin
	    sKeyUser := CertUserIDs[iSubIndex];
	    sCertUser := CertOwners[iSubIndex];
	    kUserIDList[iSubIndex] := sKeyUser;
	    cvCertificate := ChainValidities[iSubIndex];
	    if (cvCertificate = CertChain_Root) and (KeyValidities[iSubIndex] = Validity_Complete) then begin
	      cvCertificate := CertChain_Valid;
	    end;
	  end;
	  iImage := CERT - GROUP;
	  iSelectedImage := iImage;
	  case cvCertificate of
	    CertChain_Invalid: begin
	      State := IndexToStateImageMask(ord(Validity_Invalid));
	      if kUserIDList.Count = 1 then kValidity := Validity_Invalid;
	    end;
	    CertChain_Valid: begin
	      State := IndexToStateImageMask(ord(Validity_Complete));
	      if kUserIDList.Count = 1 then kValidity := Validity_Complete;
	    end;
	  else
	    State := IndexToStateImageMask(ord(Validity_Marginal));
	    if kUserIDList.Count = 1 then kValidity := Validity_Marginal;
	  end;
	  if (kUserIDList.Count <> 1) and (State <> DWord(IndexToStateImageMask(ord(kValidity)))) then begin
	    fMixedValidity := true;
	  end;
	end
	else with kUserIDList do begin
	  sCertUser := E;
	  State := IndexToStateImageMask(ord(KeyValidities[iSubIndex]));
	  if KeyValidities[iSubIndex] <> kValidity then fMixedValidity := true;
	end;
	if (sCertUser <> E) or not FSMime then begin
	  if FSMime then
	    pszText := PChar(sCertUser)
	  else pszText := PChar(sKeyUser);
	  tvInsert.Item := tvItem;
	  tvItem.hItem := hNewItem;
	  // as we overwrite all certificate properties we made sure kUserIDList only contains standard UserIDs
	  kUserIDList.Objects[iSubIndex] := TObject(SendMessage(FTreeView, TVM_INSERTITEM, 0, Longint(@tvInsert)));
	  if (Result = nil) and (CompareText(FInputID, sKeyUser) = 0) then Result := hNewItem;
	end
	else begin
	  kUserIDList.Delete(iSubIndex);
	  Continue;
	end;
	inc(iSubIndex);
      end;
      if hNewItem <> TVI_ROOT then begin
	hItem := hNewItem;
	Mask := TVIF_STATE;
	if fMixedValidity then begin
	  State := IndexToStateImageMask(MIXED);
	  pKeyProps^.kValidity := Validity_Marginal;
	end
	else State := IndexToStateImageMask(ord(pKeyProps^.kValidity));
	SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
      end;
    end;
  end;

  procedure AddGroups(var tvInsert: TTVInsertStruct; var hSelItem: HTreeItem);
  var
    iIndex, iSubIndex, iKeyIndex, iTabPos: Integer;
    slGroupIDs: TStringList;
    pKeyProps: PKeyPropsRec;
    sGroupID: String;
    tvItem: TTVItem;
    hNewItem: HTreeItem;
    iGroupValidity: Integer;
  begin
    FGroupHexIDs.Sort;
    slGroupIDs := TStringList.Create;
    iIndex := pred(FGroupHexIDs.Count);
    try
      while iIndex >= 0 do begin
	iTabPos := FirstCharPos(TAB, FGroupHexIDs[iIndex]);
	sGroupID := Copy(FGroupHexIDs[iIndex], 1, pred(iTabPos));
	slGroupIDs.CommaText := Copy(FGroupHexIDs[iIndex], succ(iTabPos), MAXINT);
	tvItem := tvInsert.Item;
	if slGroupIDs.Count > 0 then with tvItem do begin
	  // add groups by group ID
	  Mask := TVIF_IMAGE or TVIF_PARAM or TVIF_SELECTEDIMAGE or TVIF_STATE or TVIF_TEXT;
	  StateMask := TVIS_STATEIMAGEMASK;
	  State := IndexToStateImageMask(MIXED);
	  pszText := PChar(sGroupID + SP + LB + FGroupText + RB);
	  lParam := iIndex - FGroupHexIDs.Count;
	  iImage := GROUP - GROUP;
	  iSelectedImage := iImage;
	  tvInsert.hParent := TVI_ROOT;
	  tvInsert.Item := tvItem;
	  hNewItem := HTreeItem(SendMessage(FTreeView, TVM_INSERTITEM, 0, Longint(@tvInsert)));
	  if hNewItem <> nil then begin
	    if (Result = nil) and (Length(FInputID) > 1) and (FirstLoStrPos(FInputID, pszText) <> 0) then begin
	      Result := hNewItem;
	    end;
	    iSubIndex := 0;
	    iGroupValidity := 0;
	    // add keys by primary user ID
	    while iSubIndex < slGroupIDs.Count do begin
	      iKeyIndex := FKeyPropsList.IndexOf(slGroupIDs[iSubIndex]);
	      if iKeyIndex <> -1 then begin
		tvInsert.hParent := hNewItem;
		lParam := iKeyIndex;
		pKeyProps := pKeyPropsRec(FKeyPropsList.Objects[iKeyIndex]);
		with pKeyProps^ do begin
		  inc(iGroupValidity, ord(kValidity = Validity_Complete));
		  State := IndexToStateImageMask(ord(kValidity));
		  pszText := PChar(kUserID + SP + LB + kSize + RB);
		end;
		iImage := GetAlgImageIndex(pKeyProps^);
		iSelectedImage := iImage;
		tvInsert.Item := tvItem;
		SendMessage(FTreeView, TVM_INSERTITEM, 0, Longint(@tvInsert));
		inc(iSubIndex);
	      end
	      else slGroupIDs.Delete(iSubIndex);
	    end;
	    hItem := hNewItem;
	    Mask := TVIF_STATE;
	    if iGroupValidity = 0 then begin
	      State := IndexToStateImageMask(INVALID);
	      SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
	    end
	    else if iGroupValidity = slGroupIDs.Count then begin
	      State := IndexToStateImageMask(VALID);
	      SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
	    end;
	    SendMessage(FTreeView, TVM_SORTCHILDREN, 0, Longint(hNewItem));
	    FGroupHexIDs[iIndex] := slGroupIDs.CommaText + TAB + sGroupID;
	  end;
	end;
	dec(iIndex);
      end;
    finally
      slGroupIDs.Free;
    end;
  end;

  var
  iIndex: Integer;
  tvInsert: TTVInsertStruct;
  tvItem: TTVItem;
  pKeyProps: PKeyPropsRec;
  hNewItem: HTreeItem;
begin
  iIndex := 0;
  Result := nil;
  tvInsert.hInsertAfter := TVI_LAST;
  while iIndex < FKeyPropsList.Count do begin
    with tvItem do begin
      Mask := TVIF_IMAGE or TVIF_PARAM or TVIF_SELECTEDIMAGE or TVIF_STATE or TVIF_TEXT;
      StateMask := TVIS_STATEIMAGEMASK;
      tvInsert.hParent := TVI_ROOT;
      lParam := iIndex;
      pKeyProps := pKeyPropsRec(FKeyPropsList.Objects[iIndex]);
      with pKeyProps^ do begin
	State := IndexToStateImageMask(ord(kValidity));
	pszText := PChar(kUserID + SP + LB + kSize + RB);
      end;
      iImage := GetAlgImageIndex(pKeyProps^);
      iSelectedImage := iImage;
      tvInsert.Item := tvItem;
      if not FSMime then begin
	hNewItem := HTreeItem(SendMessage(FTreeView, TVM_INSERTITEM, 0, Longint(@tvInsert)));
	if not FMultiSelect and (FKeyPropsList[iIndex] = FInputID) then Result := hNewItem;
      end
      else hNewItem := TVI_ROOT;
      if hNewItem <> nil then begin
	if iImage = RSAL_KEY - GROUP then
	  inc(iImage, 3)
	else if iImage = DSA_KEY - GROUP then inc(iImage, 2);
	iSelectedImage := iImage;
	tvInsert.hParent := hNewItem;
	AddUserIDs(iIndex, tvInsert, Result);
      end;
    end;
    inc(iIndex);
  end;
  tvInsert.hInsertAfter := TVI_FIRST;
  if FGroupHexIDs <> nil then AddGroups(tvInsert, Result);
  if FSMime then SendMessage(FTreeView, TVM_SORTCHILDREN, 0, Longint(TVI_ROOT));
  if not FMultiSelect and (Result = nil) and (FInputID <> E) and (IsHexID(FInputID) = 0) then Result := FindCookedItem;
  SetFocus(FTreeView);
  if Result <> nil then begin
    SendMessage(FTreeView, TVM_EXPAND, TVE_EXPAND, Longint(Result));
    SendMessage(FTreeView, TVM_SELECTITEM, TVGN_CARET, Longint(Result));
    SendMessage(FTreeView, TVM_SELECTITEM, TVGN_FIRSTVISIBLE, Longint(Result));
  end
  else with tvItem do begin
    Mask := TVIF_HANDLE or TVIF_STATE;
    StateMask := TVIS_SELECTED;
    State := 0;
    hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_CARET, 0));
    SendMessage(FTreeView, TVM_SETITEM, 0, Longint(@tvItem));
  end;
  UpdateItemCount;
end;

function DrawTextControl(const DrawItemStruct: TDrawItemStruct; Control: THandle; AlignRight: Boolean): Longbool;
var
  rcText: TRect;
  dwAlign: DWord;
  clrText: TColorRef;
begin
  with DrawItemStruct do begin
    rcText := rcItem;
    dwAlign := (Byte(AlignRight) * DT_RIGHT);
    case CtlID of
      TEXT_L: clrText := RGB(0, 0, 255);
      TEXT_R: clrText := GetSysColor(COLOR_GRAYTEXT);
    else
      clrText := GetSysColor(COLOR_WINDOWTEXT);
    end;
    SetBkMode(hDC, TRANSPARENT);
    SetTextColor(hDC, TColorRef(clrText));
    DrawText(hDC, PChar(WndString(Control)), -1, rcText, dwAlign or DT_SINGLELINE or DT_VCENTER or DT_WORD_ELLIPSIS);
  end;
  Result := true;
end;

function KeyDialogProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;

begin
  Result := 0;
  case Msg of
    WM_DESTROY: TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).DestroyTreeView;
    WM_SHOWWINDOW: if wParam <> 0 then with TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)) do begin
      CenterDialog;
      CenterItem;
    end;
    WM_DRAWITEM: with TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)) do begin
      if PDrawItemStruct(lParam)^.CtlType = ODT_STATIC then begin
	case wParam of
	  TEXT_L: Result := Ord(DrawTextControl(PDrawItemStruct(lParam)^, FLTextLabelHandle, false));
	  TEXT_R: Result := Ord(DrawTextControl(PDrawItemStruct(lParam)^, FRTextLabelHandle, true));
	end;
      end;
    end;
    WM_NOTIFY: if PNMHdr(lParam)^.hWndFrom = TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).FTreeView then begin
      Result := TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).NotifyDialog(lParam);
    end;
    WM_HELP: TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).ShowHelp;
    WM_INITDIALOG: Result := TSelKeyDialog(lParam).InitDialog(hWnd);
    WM_COMMAND: if HiWord(wParam) = EN_SETFOCUS then
      SendMessage(lParam, EM_SETSEL, 0, 0)
    else Result := TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).ButtonCommand(LoWord(wParam));
    WM_SYSCOMMAND: begin
      case wParam of
	SC_CLOSE: Result := ord(EndDialog(hWnd, IDCANCEL));
	SC_CONTEXTHELP: PostMessage(hWnd, WM_HELP, 0, 0);
      end;
    end;
    WM_CTLCOLORBTN: Result := TSelKeyDialog(GetWindowLong(hWnd, GWL_USERDATA)).ButtonInit(THandle(lParam));
  end;
end;

function TSelKeyDialog.ShowKeyDialog: Longint;
begin
  FLegacyKeyText := KeyAlgorithmsText[KeyAlgorithm_RSA];
  if FLegacyKeyText <> E then Insert('-Legacy', FLegacyKeyText, FirstStrPos('RSA', FLegacyKeyText) + 3);
  FVerifiedText := KeyValiditiesText[TValidityLevel(INVALID)];
  Delete(FVerifiedText, 1, FirstCharPos(SP, FVerifiedText));
  Result := DialogBoxParam(hInstance, PChar(KEY_DIALOG), FParentHandle, @KeyDialogProc, Longint(Self));
end;

function TSelKeyDialog.InitDialog(hDialog: THandle): Longint;
begin
  Result := 0;
  FDialogHandle := hDialog;
  FLTextLabelHandle := GetDlgItem(FDialogHandle, TEXT_L);
  FRTextLabelHandle := GetDlgItem(FDialogHandle, TEXT_R);
  FEditHandle := GetDlgItem(FDialogHandle, TEXT_EDIT);
  FServerBtnHandle := GetDlgItem(FDialogHandle, IDRETRY);
  FUseAddBtnHandle := GetDlgItem(FDialogHandle, IDYES);
  FCancelBtnHandle := GetDlgItem(FDialogHandle, IDCANCEL);
  FOkBtnHandle := GetDlgItem(FDialogHandle, IDOK);
  SetWindowText(FDialogHandle, 'PGP');
  SetWindowLong(FDialogHandle, GWL_USERDATA, Longint(Self));
  FBoldFont := CreateFontFromHandle(SendMessage(FDialogHandle, WM_GETFONT, 0, 0), E, -1, [fsBold]);
  SendMessage(FLTextLabelHandle, WM_SETFONT, FBoldFont, 0);
  SetWindowText(FLTextLabelHandle, PChar(FPrompt));
  SetWindowText(FRTextLabelHandle, PChar(FShowDetails));
  SetWindowText(FCancelBtnHandle, PChar(FCancelBtnText));
  SetWindowText(FOkBtnHandle, PChar(FOkBtnText));
  if not FMultiSelect then begin
    SetWindowLong(FEditHandle, GWL_STYLE, GetWindowLong(FEditHandle, GWL_STYLE) and not WS_TABSTOP);
    SetWindowText(FServerBtnHandle, PChar(FServerBtnText));
    SetWindowText(FUseAddBtnHandle, PChar(FUseAddBtnText));
  end
  else begin
    SetWindowText(FServerBtnHandle, PChar(FSelectAllBtnText));
    SetWindowText(FUseAddBtnHandle, PChar(FUnselectBtnText));
  end;
  if FEditText = E then
    EnableWindow(FEditHandle, false)
  else SetWindowText(FEditHandle, PChar(FEditText));
  FPreSelected := CreateTreeView;
end;

function TSelKeyDialog.NotifyDialog(lData: Longint): Longint;
var
  tvItem: TTVItem;
begin
  Result := 0;
  case PNMHdr(lData)^.code of
    NM_CUSTOMDRAW: with PNMTVCustomDraw(lData)^ do begin
      case nmcd.dwDrawStage of
	CDDS_PREPAINT: begin
	  SetWindowLong(FDialogHandle, DWL_MSGRESULT, CDRF_NOTIFYITEMDRAW);
	  Result := 1;
	end;
	CDDS_ITEMPREPAINT: begin
	  if nmcd.uItemState = CDIS_SELECTED then begin
	    SetWindowLong(FDialogHandle, DWL_MSGRESULT, CDRF_NEWFONT);
	    clrText := ColorToRGB(clHighlightText);
	    clrTextBk := ColorToRGB(clHighlight);
	    Result := 1;
	  end;
	  with tvItem do begin
	    Mask := TVIF_PARAM;
	    hItem := HTreeItem(nmcd.dwItemSpec);
	    if (SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0) and (lParam >= 0) then begin
	      hItem := HTreeItem(SendMessage(FTreeView, TVM_GETNEXTITEM, TVGN_PARENT, Longint(hItem)));
	      if hItem <> nil then begin
		if (SendMessage(FTreeView, TVM_GETITEM, 0, Longint(@tvItem)) <> 0) and (lParam < 0) then begin
		  SetWindowLong(FDialogHandle, DWL_MSGRESULT, CDRF_NEWFONT);
		  clrText := ColorToRGB(clGrayText);
		  Result := 1;
		end;
	      end;
	      with pKeyPropsRec(FKeyPropsList.Objects[nmcd.lItemlParam])^ do begin
		if kRevoked or kDisabled or kExpired then begin
		  SetWindowLong(FDialogHandle, DWL_MSGRESULT, CDRF_NEWFONT);
		  clrText := ColorToRGB(clGrayText);
		  Result := 1;
		end
		else if kHasADK <> NoADK then begin
		  SetWindowLong(FDialogHandle, DWL_MSGRESULT, CDRF_NEWFONT);
		  clrText := ColorToRGB(clRed);
		  Result := 1;
		end;
	      end;
	    end;
	  end;
	end;
      end;
    end;
    TVN_SELCHANGING: if not FSelecting then begin
    end;
    TVN_SELCHANGED: if not FSelecting then begin
    end;
  end;
end;

function TSelKeyDialog.ButtonInit(hButton: THandle): Longint;
begin
  Result := 0;
  if not (FInitialized or IsWindowEnabled(hButton)) then begin
    FInitialized := true;
    if not (FSMime or FMultiSelect) then EnableWindow(FServerBtnHandle, FInputID <> E);
    if FPreSelected <> nil then begin
      EnableWindow(FOkBtnHandle, true);
      SendMessage(FOkBtnHandle, BM_SETSTYLE, BS_DEFPUSHBUTTON, 1);
      if not FMultiSelect then EnableWindow(FUseAddBtnHandle, Longint(hButton) >= 0);
    end
    else if IsWindowEnabled(FServerBtnHandle) then begin
      SendMessage(FServerBtnHandle, BM_SETSTYLE, BS_DEFPUSHBUTTON, 1);
      SetFocus(FServerBtnHandle);
    end;
    if FMultiSelect then begin
      EnableWindow(FServerBtnHandle, true);
      EnableWindow(FUseAddBtnHandle, true);
    end;
  end;
end;

function TSelKeyDialog.ButtonCommand(lButtonID: Longint): Longint;
begin
  Result := 0;
  case lButtonID of
    ID_OK: begin
      if GetKeyState(VK_CONTROL) < 0 then begin
	if not FMultiSelect and IsWindowEnabled(FUseAddBtnHandle) then Result := ord(EndDialog(FDialogHandle, IDYES));
      end
      else if IsWindowEnabled(FOkBtnHandle) then Result := ord(EndDialog(FDialogHandle, IDOK));
      if FMultiSelect then FGroupSelected := true;
    end;
    ID_CANCEL: begin
      if FToolTip <> 0 then
	HideToolTip
      else Result := ord(EndDialog(FDialogHandle, IDCANCEL));
    end;
    ID_RETRY: begin
      if not FMultiSelect then
	Result := ord(EndDialog(FDialogHandle, IDRETRY))
      else begin
	SetFocus(FTreeView);
	SelectAll;
      end;
    end;
    ID_YES: begin
      if not FMultiSelect then begin
	if FSMime then begin
	  ShowFixedToolTip(nil);
	  SetFocus(FTreeView);
	end
	else Result := ord(EndDialog(FDialogHandle, IDYES));
      end
      else begin
	SetFocus(FTreeView);
	UnselectAll(nil);
      end;
    end;
  end;
end;

function TSelKeyDialog.SelectEncryptKey(var EncryptKeyID: String;
					const KeyPropsList: TKeyPropsList;
					var GroupSelected: Longbool): TSelKeyResult;
begin
  Result := skCancel;
  FInputID := EncryptKeyID;
  FKeyPropsList := KeyPropsList;
  FGroupHexIDs := KeyPropsList.GroupsList;
  case ShowKeyDialog of
    IDOK: begin
      EncryptKeyID := FOutputID;
      Result := skOk;
    end;
    IDCANCEL: EncryptKeyID := E;
    IDRETRY: Result := skServer;
    IDYES: begin
      EncryptKeyID := FOutputID + FInputID;
      Result := skUseAdd;
    end;
  end;
  GroupSelected := FGroupSelected;
  ProcessMessages;
end;


end.

