{$J+,Z4}
unit StrTools;

{------------------------------------------------------------------------------}
{                                                                              }
{                                 This code is                                 }
{              Copyright (C) 2004-2023 by Michael in der Wiesche               }
{                                                                              }
{------------------------------------------------------------------------------}

interface

uses
  Windows;

const
  E = '';
  SP = ' ';
  CR = #13;
  LF = #10;
  AM = '&';
  AT = '@';
  QU = '"';
  EQ = '=';
  BS = '\';
  SL = '/';
  GT = '>';
  LT = '<';
  LB = '(';
  RB = ')';
  CL = ':';
  CM = ',';
  SC = ';';
  NUL = #0;
  TAB = #9;
  DOT = '.';
  CRLF = CR + LF;
  PARA = CRLF + CRLF;

function StrToWideStr(const Str: String): String;

function GetCharCount(Find: Char; const Text: String): Integer;

function ExtractStr(var Source: String; Pos, Len: Integer): String;

function StrAtPos(const Find, Dest: String; Pos: Integer): Boolean;
function NextStrPos(const Find, Dest: String; BegPos: Integer): Integer;

function LoStrAtPos(const Find, Dest: String; Pos: Integer): Boolean;
function NextLoStrPos(const Find, Dest: String; BegPos: Integer): Integer;

function FirstStrPos(const Find, Dest: String): Integer;
function LastStrPos(const Find, Dest: String): Integer;

function FirstLoStrPos(const Find, Dest: String): Integer;
function LastLoStrPos(const Find, Dest: String): Integer;

function FirstCharPos(Find: Char; const Dest: String): Integer;
function LastCharPos(Find: Char; const Dest: String): Integer;

function NextCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer;
function PrevCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer;

procedure XChgChars(var Str: String; Old, New: Char);

function HexInsertSpaces(const Hex: String): String;
function HexStringToByteString(const HexString: String): String;
function BytesToString(Bytes: PChar; Size: DWord): String;
function ReverseBytesToString(Bytes: PChar; Size: DWord): String;


implementation

function StrToWideStr(const Str: String): String;
var
  iPCharLen: Integer;
begin
  iPCharLen := Length(Str) + 2;
  SetString(Result, nil, iPCharLen shl 1);
  StringToWideChar(Str, Pointer(Result), iPCharLen);
end;

function GetCharCount(Find: Char; const Text: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Text
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EBX
  MOV	ECX,[EDX - 04h]
  XOR	EBX,EBX
  @LOOP:
  DEC	ECX
  JL	@END
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  INC	EBX
  JMP	@LOOP
  @END:
  MOV	EAX,EBX
  POP	EBX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function ExtractStr(var Source: String; Pos, Len: Integer): String; register; assembler;
asm	// EAX = @pSource, EDX = Pos, ECX = Len, [EBP + 08h] = @Result
  PUSH	EBX
  PUSH	EDI
  PUSH	ESI
  MOV	EBX,EAX
  MOV	EDI,EDX
  MOV	ESI,ECX
  MOV	EAX,[EBP + 08h]
  CALL	SYSTEM.@LSTRCLR
  OR	DWORD PTR [EBX],0
  JE	@EXIT
  OR	EDI,EDI
  JLE	@EXIT
  OR	ESI,ESI
  JLE	@EXIT
  MOV	EAX,[EBX]
  CMP	[EAX - 04h],EDI
  JL	@EXIT
  DEC	EDI
  MOV	EAX,[EAX - 04h]
  SUB	EAX,EDI
  CMP	EAX,ESI
  JLE	@ALLOC
  MOV	EAX,ESI
  @ALLOC:
  CALL	SYSTEM.@NEWANSISTRING
  MOV	EDX,[EBP + 08h]
  MOV	[EDX],EAX
  MOV	ECX,[EAX - 04h]
  MOV	EBP,[EBX]
  MOV	ESI,ECX
  ADD	EBP,EDI
  TEST	ECX,03h
  JE	@FASTCOPY
  @COPY:
  DEC	ECX
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  TEST	ECX,03h
  JE	@STOPCOPY
  DEC	ECX
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  TEST	ECX,03h
  JE	@STOPCOPY
  DEC	ECX
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  @STOPCOPY:
  OR	ECX,ECX
  JE	@NOCOPY
  @FASTCOPY:
  SUB	ECX,04h
  MOV	EDX,[EBP + ECX]
  MOV	[EAX + ECX],EDX
  JNE	@FASTCOPY
  @NOCOPY:
  MOV	EAX,[EBX]
  MOV	EDX,EBP
  ADD	EDX,ESI
  SUB	EDX,EAX
  CMP	EDX,[EAX - 04h]
  JNZ	@DELETE
  MOV	EAX,EBX
  CALL	UniqueString
  SUB	[EAX - 04h],ESI
  MOV	BYTE PTR [EAX + EDI],0
  JNZ	@EXIT
  MOV	EAX,EBX
  CALL	SYSTEM.@LSTRCLR
  JMP	@EXIT
  @DELETE:
  MOV	EAX,EBX
  CALL	UniqueString
  SUB	[EAX - 04h],ESI
  MOV	ECX,[EAX - 04h]
  ADD	EAX,EDI
  SUB	ECX,EDI
  MOV	EBP,EAX
  ADD	EAX,ECX
  ADD	EBP,ECX
  ADD	EBP,ESI
  NEG	ECX
  TEST	ECX,03h
  JE	@FASTMOVE
  @MOVE:
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  INC	ECX
  TEST	ECX,03h
  JE	@STOPMOVE
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  INC	ECX
  TEST	ECX,03h
  JE	@STOPMOVE
  MOV	DL,[EBP + ECX]
  MOV	[EAX + ECX],DL
  INC	ECX
  @STOPMOVE:
  OR	ECX,ECX
  JE	@NOMOVE
  @FASTMOVE:
  MOV	EDX,[EBP + ECX]
  MOV	[EAX + ECX],EDX
  ADD	ECX,04h
  JLE	@FASTMOVE
  @NOMOVE:
  MOV	BYTE PTR [EAX],0
  @EXIT:
  POP	ESI
  POP	EDI
  POP	EBX
end;

function StrAtPos(const Find, Dest: String; Pos: Integer): Boolean; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = Pos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EDX - 04h]
  SUB	EBX,ECX
  INC	EBX
  CMP	EBX,[EAX - 04h]
  JL	@END
  ADD	EDX,ECX
  MOV	ECX,[EAX - 04h]
  DEC	EDX
  @LOOP:
  DEC	ECX
  JL	@MATCH
  MOV	BL,[EAX + ECX]
  CMP	BL,[EDX + ECX]
  JE	@LOOP
  POP	EBX
  XOR	EAX,EAX
  RET
  @MATCH:
  POP	EBX
  MOV	EAX,true
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
end;

function NextStrPos(const Find, Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = BegPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EAX - 04h]
  SUB	ECX,[EDX - 04h]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,[EDX - 04h]
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[EBP + EBX],AH
  JE	@CMP
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - 04h]
  POP	EDI
  MOV	EAX,ECX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  POP	EBX
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - 04h]
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function LoStrAtPos(const Find, Dest: String; Pos: Integer): Boolean; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = Pos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EDX - 04h]
  SUB	EBX,ECX
  INC	EBX
  CMP	EBX,[EAX - 04h]
  JL	@END
  ADD	EDX,ECX
  MOV	ECX,[EAX - 04h]
  DEC	EDX
  @LOOP:
  DEC	ECX
  JL	@MATCH
  MOV	BL,[EDX + ECX]
  XOR	BL,[EAX + ECX]
  JE	@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@END
  TEST	BYTE PTR [EAX + ECX],040h
  JE	@END
  AND	BL,0DFh
  JE	@LOOP
  POP	EBX
  XOR	EAX,EAX
  RET
  @MATCH:
  POP	EBX
  MOV	EAX,true
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
end;

function NextLoStrPos(const Find, Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest, ECX = BegPos
  OR	EAX,EAX
  JE	@EXIT
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  PUSH	EBX
  MOV	EBX,[EAX - 04h]
  SUB	ECX,[EDX - 04h]
  ADD	EBX,ECX
  CMP	EBX,1
  JG	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,[EDX - 04h]
  MOV	AL,[ESI]
  DEC	EDI
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[EBP + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - 04h]
  POP	EDI
  MOV	EAX,ECX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  POP	EBX
  @EXIT:
  XOR	EAX,EAX
  RET
  @CHAR:
  POP	EBX
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - 04h]
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function FirstStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  MOV	ECX,[EDX - 04h]
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,ECX
  MOV	AL,[ESI]
  NEG	ECX
  @LOOP:
  CMP	[EDI + ECX],AL
  JE	@MATCH
  ADD	ECX,[ESI - 04h]
  JE	@NOT
  SUB	ECX,[ESI - 04h]
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  CMP	[EBP + EBX],AH
  JE	@CMP
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - 04h]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  PUSH	EDX
  MOV	ECX,[EDX - 04h]
  ADD	EDX,[EDX - 04h]
  NEG	ECX
  MOV	AL,[EAX]
  @@LOOP:
  CMP	[EDX + ECX],AL
  JE	@@MATCH
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function LastStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  MOV	ECX,[EDX - 04h]
  CMP	ECX,[EAX - 04h]
  JL	@END
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - 04h]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  CMP	[EDI + ECX - 01h],AL
  JE	@MATCH
  DEC	ECX
  CMP	ECX,[ESI - 04h]
  JNL	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  CMP	[EBP + EBX - 01h],AH
  JE	@CMP
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - 04h]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	ECX,[EDX - 04h]
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  CMP	[EDX + ECX],AL
  JNE	@@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function FirstLoStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  MOV	ECX,[EDX - 04h]
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	EDI,EDX
  MOV	ESI,EAX
  ADD	EDI,ECX
  MOV	AL,[ESI]
  NEG	ECX
  @LOOP:
  MOV	AH,[EDI + ECX]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  ADD	ECX,[ESI - 04h]
  JE	@NOT
  SUB	ECX,[ESI - 04h]
  INC	ECX
  JNE	@LOOP
  @NOT:
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,EBX
  ADD	EBP,EBX
  ADD	EDI,ECX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX]
  XOR	AH,[EBP + EBX]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  INC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  ADD	ECX,[EDX - 04h]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  PUSH	EDX
  MOV	ECX,[EDX - 04h]
  ADD	EDX,[EDX - 04h]
  NEG	ECX
  MOV	AL,[EAX]
  @@LOOP:
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@NEXT
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@NEXT
  AND	AH,0DFh
  JE	@@MATCH
  @@NEXT:
  INC	ECX
  JNE	@@LOOP
  POP	EDX
  XOR	EAX,EAX
  RET
  @@MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function LastLoStrPos(const Find, Dest: String): Integer; register; assembler;
asm	// EAX = @Find, EDX = @Dest
  OR	EAX,EAX
  JE	@END
  OR	EDX,EDX
  JE	@END
  CMP	DWORD PTR [EAX - 04h],1
  JE	@CHAR
  MOV	ECX,[EDX - 04h]
  CMP	ECX,[EAX - 04h]
  JL	@END
  PUSH	EBX
  PUSH	EBP
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EBX,[EAX - 04h]
  MOV	EDI,EDX
  MOV	AL,[ESI + EBX - 01h]
  @LOOP:
  MOV	AH,[EDI + ECX - 01h]
  XOR	AH,AL
  JE	@MATCH
  TEST	AL,040h
  JE	@NEXT
  TEST	BYTE PTR [EDI + ECX - 01h],040h
  JE	@NEXT
  AND	AH,0DFh
  JE	@MATCH
  @NEXT:
  DEC	ECX
  CMP	ECX,[ESI - 04h]
  JNL	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @MATCH:
  PUSH	EDI
  MOV	EBX,[ESI - 04h]
  MOV	EBP,ESI
  ADD	EDI,ECX
  ADD	EBP,EBX
  NEG	EBX
  @CMP:
  INC	EBX
  JE	@YES
  MOV	AH,[EDI + EBX - 01h]
  XOR	AH,[EBP + EBX - 01h]
  JE	@CMP
  TEST	BYTE PTR [EDI + EBX - 01h],040h
  JE	@BREAK
  TEST	BYTE PTR [EBP + EBX - 01h],040h
  JE	@BREAK
  AND	AH,0DFh
  JE	@CMP
  @BREAK:
  POP	EDI
  DEC	ECX
  JNE	@LOOP
  XOR	EAX,EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @YES:
  SUB	ECX,[ESI - 04h]
  POP	EDI
  MOV	EAX,ECX
  INC	EAX
  POP	ESI
  POP	EDI
  POP	EBP
  POP	EBX
  RET
  @END:
  XOR	EAX,EAX
  RET
  @CHAR:
  MOV	ECX,[EDX - 04h]
  MOV	AL,[EAX]
  @@LOOP:
  DEC	ECX
  JL	@@EXIT
  MOV	AH,[EDX + ECX]
  XOR	AH,AL
  JE	@@MATCH
  TEST	AL,040h
  JE	@@LOOP
  TEST	BYTE PTR [EDX + ECX],040h
  JE	@@LOOP
  AND	AH,0DFh
  JNE	@@LOOP
  @@MATCH:
  MOV	EAX,ECX
  INC	EAX
  RET
  @@EXIT:
  XOR	EAX,EAX
end;

function FirstCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  PUSH	EDX
  MOV	ECX,[EDX - 04h]
  ADD	EDX,[EDX - 04h]
  NEG	ECX
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function LastCharPos(Find: Char; const Dest: String): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest
  OR	EDX,EDX
  JE	@EXIT
  MOV	ECX,[EDX - 04h]
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

function NextCharPos(Find: Char; const Dest: String; BegPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = BegPos
  OR	EDX,EDX
  JE	@EXIT
  OR	ECX,ECX
  JLE	@EXIT
  SUB	ECX,[EDX - 04h]
  JG	@EXIT
  PUSH	EDX
  DEC	ECX
  ADD	EDX,[EDX - 04h]
  @LOOP:
  CMP	[EDX + ECX],AL
  JE	@MATCH
  INC	ECX
  JNE	@LOOP
  POP	EDX
  @EXIT:
  XOR	EAX,EAX
  RET
  @MATCH:
  INC	ECX
  POP	EDX
  MOV	EAX,ECX
  ADD	EAX,[EDX - 04h]
end;

function PrevCharPos(Find: Char; const Dest: String; EndPos: Integer): Integer; register; assembler;
asm	// EAX = Find, EDX = @Dest, ECX = EndPos
  OR	EDX,EDX
  JE	@EXIT
  CMP	ECX,[EDX - 04h]
  JG	@EXIT
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	[EDX + ECX],AL
  JNE	@LOOP
  MOV	EAX,ECX
  INC	EAX
  RET
  @EXIT:
  XOR	EAX,EAX
end;

procedure XChgChars(var Str: String; Old, New: Char); register; assembler;
asm	// EAX = @pStr, EDX = Old, ECX = New
  OR	DWORD PTR [EAX],0
  JE	@EXIT
  PUSH	EAX
  PUSH	EDX
  PUSH	ECX
  CALL	UniqueString
  POP	ECX
  POP	EDX
  POP	EAX
  MOV	DH,CL
  MOV	EAX,[EAX]
  MOV	ECX,[EAX - 04h]
  @LOOP:
  DEC	ECX
  JL	@EXIT
  CMP	DL,[EAX + ECX]
  JNE	@LOOP
  MOV	[EAX + ECX],DH
  JMP	@LOOP
  @EXIT:
end;

function HexInsertSpaces(const Hex: String): String; register; assembler;
asm	// EAX = @Hex, EDX = @Result
  OR	EAX,EAX
  JE	@EXIT
  MOV	ECX,[EAX - 4]
  TEST	ECX,1
  JNE	@EXIT
  PUSH	EDI
  SHR	ECX,1
  PUSH	ESI
  DEC	ECX
  PUSH	EAX
  PUSH	ECX
  PUSH	EDX
  MOV	EAX,[EAX - 4]
  ADD	EAX,ECX
  CALL	SYSTEM.@NEWANSISTRING
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDI,EAX
  SUB	EDI,2
  POP	ECX
  POP	ESI
  ADD	EDI,ECX
  MOV	EDX,[ESI - 4]
  SUB	ESI,2
  @LOOP:
  MOV	AX,[ESI + EDX]
  MOV	[EDI + EDX],AX
  SUB	EDX,2
  JE	@END
  MOV	BYTE PTR [EDI + EDX + 1],' '
  DEC	EDI
  JMP	@LOOP
  @END:
  POP	ESI
  POP	EDI
  RET
  @EXIT:
  MOV	EAX,EDX
  CALL  SYSTEM.@LSTRCLR
end;

function HexStringToByteString(const HexString: String): String; register; assembler;
asm	// EAX = @HexString, EDX = @Result
  OR	EAX,EAX
  JE	@EXIT
  PUSH	EBX
  PUSH	EDX
  PUSH	EAX
  PUSH	EDX
  MOV	EAX,[EAX - 4]
  SHR	EAX,1
  CALL	SYSTEM.@NEWANSISTRING
  POP	EDX
  MOV	[EDX],EAX
  MOV	EDX,EAX
  POP	EAX
  MOV	BYTE PTR [EDX],0
  MOV	EBX,[EAX - 4]
  ADD	EAX,EBX
  NEG	EBX
  @LOOP:
  MOV	CL,[EAX + EBX]
  SUB	CL,'0'
  JB	@ERROR
  CMP	CL,09h
  JA	@HEX
  JMP	@STORE
  @HEX:
  AND	CL,0DFh
  SUB	CL,'A' - '0'
  JB	@ERROR
  CMP	CL,05h
  JA	@ERROR
  ADD	CL,0Ah
  @STORE:
  SHL	BYTE PTR [EDX],4
  ADD	BYTE PTR [EDX],CL
  INC	EBX
  JE	@END
  BT	EBX,0
  JC	@LOOP
  INC	EDX
  MOV	BYTE PTR [EDX],0
  JMP	@LOOP
  @END:
  MOV	EAX,EDX
  POP	EDX
  MOV	ECX,[EDX]
  SUB	EAX,ECX
  INC	EAX
  POP	EBX
  MOV	[ECX-4],EAX
  RET
  @ERROR:
  POP	EDX
  POP	EBX
  @EXIT:
  MOV	EAX,EDX
  CALL  SYSTEM.@LSTRCLR
end;

function BytesToString(Bytes: PChar; Size: DWord): String; register; assembler;
asm	// EAX = @Bytes, EDX = Size, ECX = @Result
  PUSH	EBX
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EDI,ECX
  MOV	EAX,EDX
  MOV	EBX,EDX
  SHL	EAX,1
  DEC	EDX
  ADD	EBX,EAX
  PUSH	EDX
  DEC	EBX
  MOV	EAX,EBX
  CALL	SYSTEM.@NEWANSISTRING
  MOV	[EDI],EAX
  MOV	EDI,EAX
  SUB	EBX,2
  POP	ECX
  @LOOP:
  XOR	EAX,EAX
  MOV	AL,[ESI + ECX]
  MOV	EDX,EAX
  AND	EAX,0Fh
  SHR	EDX,4
  MOV	AH,BYTE PTR[@HEXCHARS + EAX]
  MOV	AL,BYTE PTR[@HEXCHARS + EDX]
  MOV	[EDI + EBX],AX
  SUB	EBX,3
  DEC	ECX
  JS	@END
  MOV	BYTE PTR[EDI + EBX + 2],' '
  JMP	@LOOP
  @END:
  POP	ESI
  POP	EDI
  POP	EBX
  RET
  @HEXCHARS:
  DB	'0123456789ABCDEF';
end;

function ReverseBytesToString(Bytes: PChar; Size: DWord): String; register; assembler;
asm	// EAX = @Bytes, EDX = Size, ECX = @Result
  PUSH	EBX
  PUSH	EDI
  PUSH	ESI
  MOV	ESI,EAX
  MOV	EDI,ECX
  MOV	EAX,EDX
  MOV	EBX,EDX
  SHL	EAX,1
  DEC	EDX
  ADD	EBX,EAX
  PUSH	EDX
  DEC	EBX
  MOV	EAX,EBX
  CALL	SYSTEM.@NEWANSISTRING
  MOV	[EDI],EAX
  MOV	EDI,EAX
  SUB	EBX,2
  POP	ECX
  ADD	ESI,ECX
  NEG	ECX
  @LOOP:
  XOR	EAX,EAX
  MOV	AL,[ESI + ECX]
  MOV	EDX,EAX
  AND	EAX,0Fh
  SHR	EDX,4
  MOV	AH,BYTE PTR[@HEXCHARS + EAX]
  MOV	AL,BYTE PTR[@HEXCHARS + EDX]
  MOV	[EDI + EBX],AX
  SUB	EBX,3
  INC	ECX
  JG	@END
  MOV	BYTE PTR[EDI + EBX + 2],' '
  JMP	@LOOP
  @END:
  POP	ESI
  POP	EDI
  POP	EBX
  RET
  @HEXCHARS:
  DB	'0123456789ABCDEF';
end;


end.

