unit ifo_html;

interface

uses
  Windows, Messages, SysUtils, StrUtil;

//
//  IFO_HTML.C
//  Interface Service functions for WinPMail v4.0.
//
//  Pegasus Mail System,
//  Copyright (c) 1990-2004, David Harris, all rights reserved.
//
//  This module implements an HTML display object based on Dave Baldwin's
//  Delphi HTML Display Classes (http://www.pbear.com), as implemented by
//  Martin Ireland. If "BEARHTML.DLL" is available in the directory where
//  Pegasus Mail is installed, this control will report a display competency
//  of 5 for HTML data types, allowing it to override the bug-ridden HTS
//  default display object for HTML data.
//
//  Name:            "HTMLReader"
//  Type:            Multiple instances OK, attributes are read/write
//  Initializable:   No
//
//  Attributes:
//  ---------------------------------------------------------------------
//  "Parent"      HWND           Parent window for display object
//  "Rect"        RECT *         Bounding rectangle of the area in which
//                               the object is to display message data
//  "IOModel"     int            (Read-only) 0 for IMESSAGE, 1 for file
//  "Message"     char *         Name of file to display for IOModel=1
//  "IMessage"    IMESSAGE *     IMESSAGE to display for IOModel=0
//  "Mflags"      int            IMESSAGE flags
//  "FontName"    char *         Name of the default font to use
//  "FontSize"    int            Name of the default font size to use
//  "RType"       int            Unique "type" ID for this object (2)
//
//  Methods:
//  ---------------------------------------------------------------------
//  "Create"      No parameters  WinPMail calls this method when it has
//                               filled out the attributes for the control
//                               and is ready for it to instantiate on the
//                               parent's client area.
//

type
  INT_32 = Longint;
  UINT_32 = Cardinal;

  TMsgFunc = function(ref: UINT_32; wMsg: UINT_32; wParam: UINT_32; lParam: UINT_32): UINT_32; stdcall;
  TCreateViewer = function(const bounds: TRect; hParent: THandle; id: INT_32; bgcolour: COLORREF;
                           cfunc: TMsgFunc; flags: UINT_32; pmail_data: UINT_32): UINT_32; stdcall;
  TBearApi = Record
    CreateViewer:  TCreateViewer;
    ViewerCommand: TMsgFunc;
  end;

const

//  VO_SETRECT - set the bounding rectangle for the control (resize)
//
  VO_SETRECT = (WM_USER + 2660);
//      wParam:  always set to 0
//      lParam:  pointer to a RECT structure indicating the new bounding
//               rectangle for the control in coordinates local to the host
//               window.
//      Returns: 1 on success, 0 on failure


//  VO_DESTROY - clean up the viewer object, removing any memory, controls
//  and files it may have created during operation.
//
  VO_DESTROY = (WM_USER + 2675);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


//  VO_SETFILE - set the content displayed by the viewer object
//
  VO_SETFILE = (WM_USER + 2661);
//      wParam:  Pointer to a string containing charset meta-information,
//               if available. This will typically be a character set name
//               as found in the MIME headers of a message, such as
//               "ISO-8859-1", "US-ASCII" or "UTF-8".
//      lParam:  (char *) pointer, indicating the address of a C-style
//               (nul-terminated) array of characters indicating a filename
//               containing HTML data to be rendered by the viewer object.
//               Any preprocessing required on the HTML data (for instance,
//               converting CID graphic references to local filenames) will
//               already have been done, and the viewer object can assume
//               that the file will be available either until it is
//               destroyed, or until another VO_SETFILE message is sent.
//               Note that this message can be sent multiple times during
//               the life of a viewer object, referencing a different file
//               each time.
//      Returns: 1 on success, 0 on failure


//  VO_RENDER - controls the redraw state of the viewer object
//
  VO_RENDER = (WM_USER + 2662);
//      wParam:  1 if the object can draw and update, 0 if not. When
//               initially created, the viewer object must assume that its
//               rendering state is set to 0 - Pegasus Mail will send it a
//               VO_RENDER message when it has finished setting it up. The
//               viewer object should update the display as soon as it gets
//               a VO_RENDER message. Receipt of a VO_RENDER message with
//               "wParam" set to 0 can be taken to mean that a different
//               HTML content file is about to be specified and loaded.
//      lParam:  unused, always set to 0


//  VO_SETFOCUS - set focus to the viewer object's content. Pegasus Mail will
//  send this message to the viewer object whenever it wants the viewer
//  object to dispay a caret or other focus indicator.
//
  VO_SETFOCUS = (WM_USER + 2663);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


//  VO_PAGEDOWN - scroll down one page in the content
//
  VO_PAGEDOWN = (WM_USER + 2664);
//      wParam:  if 1, scroll down a single line instead of a page
//      lParam:  unused, always set to 0


//  VO_PAGEUP - scroll up one page in the content
//
  VO_PAGEUP = (WM_USER + 2665);
//      wParam:  if 1, scroll up a single line instead of a page
//      lParam:  unused, always set to 0


//  VO_SETPRINTHEADER - set a text string to be printed as the header on each
//  page when the HTML content is printed via VO_PRINT.
//
  VO_SETPRINTHEADER = (WM_USER + 2666);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to C-type string containing text to be
//               printed as a header on each page.


//  VO_SETPRINTFOOTER - set a text string to be printed as the footer on each
//  page when the HTML content is printed via VO_PRINT.
//
  VO_SETPRINTFOOTER = (WM_USER + 2667);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to C-type string containing text to be
//               printed as a header on each page.


//  VO_SETPRINTFLAGS - adjusts behaviour when printing via VO_PRINT.
//
  VO_SETPRINTFLAGS = (WM_USER + 2668);
//      wParam:  UINT_32 flag values to be decided
//      lParam:  unused, always set to 0


//  VO_SETPRINTFONT - set a default font that should be used when printing
//  the content via VO_PRINT (note that the viewer object may be unable to
//  comply with this, but I'm including it for the sake of completeness)
//
  VO_SETPRINTFONT = (WM_USER + 2673);
//      wParam:  Point size for font
//      lParam:  (char *) points to name of font to use for printing


//  VO_PRINT - print the current HTML content
//
  VO_PRINT = (WM_USER + 2669);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


//  VO_GETSELECT - get either the length of the current selection in the
//  viewer object, or the contents of the selected area
//
  VO_GETSELECT = (WM_USER + 2670);
//      wParam:  If wParam==1, then the object should return only the
//               length of the currently-selected area in the content. If
//               wParam==0, then the object should assume that lParam
//               points to a suitable-sized buffer and copy the selected
//               content into that buffer.
//      lParam:  If wParam==0, then this points to a buffer.


//  VO_SELECTALL - perform a "Select all" operation in the content
//
  VO_SELECTALL = (WM_USER + 2671);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


//  VO_COPY - perform a "Copy to clipboard" operation to copy the
//  currently-selected area of the content to the Windows clipboard.
//
  VO_COPY = (WM_USER + 2672);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


//  VO_SETFONT - set a default font that should be used when displaying the
//  content on-screen (note that the viewer object may be unable to comply
//  with this, but I'm including it for the sake of completeness)
//
  VO_SETFONT = (WM_USER + 2674);
//      wParam:  Point size for font
//      lParam:  (char *) points to name of font to use for display


//  VO_SEARCH - asks the viewer object to perform a search operation.
//
  VO_SEARCH = (WM_USER + 2676);
//      wParam:  Search flags:
//               If (wParam & 1), then perform a case-sensitive search
//               If (wParam & 2), then search for whole words only
//               If (wParam & 4), then search backwards from the cursor
//               If (wParam & 8), then search from the start of the buffer
//      lParam:  (char *) points to the search string


//  VO_SHOWIMAGES - turns on/off generation of images (default is Off)
//
  VO_SHOWIMAGES = (WM_USER + 2678);
//    wParam: turn on = 1
//            turn off = 0
//
//  If ShowImages is off, Htmlviewer will format pages without reference to an image.
//  If ShowImages is on, an Htmlviewer UrlRequest will be issued for the caller to obtain
//  the image file (Jpg or Gif).  A local file must be identified and supplied in full pathname
//  format. The Htmlviewer page will reserve space on the page for the image
//
//  At present the only images expected to be displayed will be inline graphics from an Mhtml
//  compliant message.

//  VO_TIDYFILE - Apply TIDY.DLL processing to a file
//
  VO_TIDYFILE = (WM_USER + 2679);
//    wParam: always 0
//    lParam: pointer to a TIDYRQ structure

type
  TFileName = Array[0..255] of AnsiChar;
  TFileRequest = Record
    size:   UINT_32;
    source: TFileName;
    dest:   TFileName;
    flags:  UINT_32;
  end;
  TStringRequest = Record
    size:     UINT_32;
    hInput:   HGlobal;
    hOutput:  HGlobal;
    BodyOnly: Bool;
  end;
  PTidyRequest = ^TTidyRequest;
  TTidyRequest = Record
    size: UINT_32;
    case Boolean of
      TRUE: (
	source:  TFileName;
	dest:    TFileName;
	flags:   UINT_32);
      FALSE: (
	hInput:   HGlobal;
	hOutput:  HGlobal;
	BodyOnly: Bool);
  end;

//  The TIDYRQ structure contains a reference to a source file containing the
//  HTML to be processed, and a destination file, where the tidied HTML data
//  is to be written. "ssize" is always set to the size of the structure before
//  the call, and should be used to check version matching. "flags" contains
//  a bitmap of setting requests that control the type of processing Pegasus
//  Mail would like performed on the data: by default, BearHTML should use
//  whatever settings are appropriate to convert the HTML data to valid
//  HTML v3 format.
//
//  BearHTML should return 1 if the conversion is successful, or 0 if it is
//  not. The destination file need not be deleted if the operation is
//  unsuccessful (Pegasus Mail will do that).


//
//  A viewer object can send the following messages to Pegasus Mail:
//


const

//  VN_LINK - a hyperlink has been clicked by the user
//
  VN_LINK = (WM_USER + 2760);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to buffer containing the full URL clicked
//               by the user.


const

//  VN_MOUSEOVER - the mouse has entered a hyperlinked or rollover area in
//  the HTML content. The viewer object should only send this message once
//  for any hyperlinked area the mouse enters.
//
  VN_MOUSEOVER = (WM_USER + 2761);
//      wParam:  (char *) points to a buffer containing any alternate text
//               associated with the hyperlinked area.
//      lParam:  (char *) points to a buffer containing the full URL
//               contained in the hyperlinked area.


const

//  VN_MOUSELEAVE - the mouse has left a hyperlinked or rollover area
//  previously notified using VN_MOUSEOVER
//
  VN_MOUSELEAVE = (WM_USER + 2762);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


type
  TBearLink = Record
    url:       PAnsiChar;
    target:    PAnsiChar;
    image:     Pointer;
    imageX:    Integer;
    imageY:    Integer;
    clickWord: PAnsiChar;
  end;

const

//  VN_RCLICK - the user has right-clicked within the viewer's area
//
  VN_RCLICK = (WM_USER + 2763);
//      wParam:  unused, always set to 0
//      lParam:  (BEAR_RLINK *) points to a BEAR_RLINK structure
//               indicating whether a link has been right-clicked.


const

//  VN_URLREQUEST - a IMAGE or CSS hyperlink URL data is required by the viewer
//
  VN_URLREQUEST = (WM_USER + 2764);
//      wParam:  Sends Either 0 for Image, or 1 for CSS
//      lParam:  (char *) points to buffer containing the full URL.

//     Returns:
//       lParam: (char *) points to same buffer now containing the local filename
//               for the requested data
//       Returncode value indicates processing is to occur:
//               0: Use default graphic (blank grey box)
  REQ_DEFAULT = 0;
//               1: Use local filename (CID: inline graphic)
  REQ_LOCALFILE = 1;
//               2: Fetch and load remote graphic
  REQ_REMOTEIMAGE = 2;

//    In either case the recipient is expected to convert the URL into a local filename.
//    A default file must be returned where the original request cannot be honoured.


const

// VN_WARN - an unsupported HTML feature has been discovered
//
  VN_WARN = (WM_USER + 2765);
//      wParam:  Currently defined
//               1:  Script language, ie Javascript
  warnScript = 1;
//               2:  LINK verb to an alternative full URL
  warnLink = 2;
//               3:  Meta data REDIRECT to alternative full URL
  warnRedirect = 3;
//      lParam:  (char *) points to buffer containing the full URL.
//
//   In the case of SCRIPT warning this can be because of SCRIPT tag that contains
//   inline code, or that points via a SRC= to a URL location containing source code.
//   For inline source, lParam will be NIL.
//   In all cases the verb will be skipped in the HTML source.


const

// VN_CHAR - a keyboard stroke has been made by the user
//
  VN_CHAR = (WM_USER + 2766);
//      wParam:  Keyboard char as VK_ value
//      lParam:  Shift/Control/Alt keyboard state
//               1 = Shift
  charShift = 1;
//               2 = Control
  charCtrl = 2;
//               4 = Alt
  charAlt = 4;
//    Combinations of lparam values are allowed.


const

// VN_PRINTHEADER - provide HTML to be used as the header on each
// page when the HTML content is printed via VO_PRINT.
//
  VN_PRINTHEADER = (WM_USER + 2767);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to C-type string containing text to be
//               printed as a header on each page.


const

// VN_PRINTFOOTER - provide HTML to be used as the footer on each
// page when the HTML content is printed via VO_PRINT.
//
  VN_PRINTFOOTER = (WM_USER + 2768);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to C-type string containing text to be
//               printed as a header on each page.


const

//  VN_GETOPTIONS - Pegasus Mail will return a 32-bit quantity representing
//  options that apply to the HTML Renderer. The following values are
//  currently defined:
//
//     Bit 0:  If non-zero, BearHTML should NOT perform phishing checks.
//
  VN_GETOPTIONS = (WM_USER + 2769);
//      wParam:  unused, always set to 0
//      lParam:  unused, always set to 0


const

//  VN_SETSTATUS - "lParam" is a pointer to a C-style string which Pegasus
//  Mail will display in its status bar. If the string is zero-length, the
//  status bar will be reset to its normal condition (so, once you have
//  finished whatever you are doing, send one of these messages with an
//  empty string to clear your last message).
//
  VN_SETSTATUS = (WM_USER + 2770);
//      wParam:  unused, always set to 0
//      lParam:  (char *) points to C-type string containing text to be
//               displayed on the status bar.


const

//  VN_TCPSETTINGS - If "wParam" is 0, WinPMail will return a value indicating
//  the current method it is using to load the Windows Sockets layer:
//
//      1 = never
  tcpNever = 1;
//      2 = on demand
  tcpOnDemand = 2;
//      3 = always
  tcpAlways = 3;
//
//  If wParam is 1, then lParam is taken to be one of the values shown above,
//  and the program will change its Windows sockets loading method to comply
//  with the requested setting. Note that this may require a program restart
//  to take effect.
//
  VN_TCPSETTINGS = (WM_USER + 2771);
//      wParam:  0 to query, 1 to set
//      lParam:  see above


const

//  VN_GETVARIABLE - Get an internal WinPMail operating variable. "wParam"
//  selects the variable to retrieve, while lParam is a pointer to a buffer
//  where the value will be placed. The space that must be allocated for
//  the buffer pointed to by "lParam" is determined implicitly by the
//  value of wParam (which is not ideal, but we don't have enough parameters
//  to pass a length indicator around)
//
//  wParam can have the following values:

  VNGV_USERNAME = 1;          //  128 bytes, the WinPMail username for the current user
  VNGV_FRAMEWINDOW = 2;       //  4 bytes, the HWND for the WinPMail frame window
  VNGV_BASEDIR = 3;           //  256 bytes, the WinPMail base directory
  VNGV_EXEDIR = 4;            //  256 bytes, the directory where WINPM-32.EXE is located
  VNGV_SCRATCHDIR = 5;        //  256 bytes, the recommended scratch directory
  VNGV_HOMEBOX = 6;           //  256 bytes, path to the user's home mailbox directory
  VNGV_NEWMAILBOX = 7;        //  256 bytes, path to the user's new mail folder directory
  VNGV_VERSION = 8;           //  4 bytes, the internal binary version
  VNGV_BUILDNO = 9;           //  4 bytes, the internal build number
  VNGV_VERSTRING = 10;        //  128 bytes, the display string for the current version
  VNGV_OIFHUB = 11;           //  V5 object interface hub block

//  Note that you should not assume that VNGV_BASEDIR and VNGV_EXEDIR will be
//  the same: VNGV_BASEDIR should be where you look for any data item - the only
//  reason you would use VNGV_EXEDIR would be if you need to locate a DLL for
//  loading, or if for some reason you need to know exactly where the program's
//  actual executable files are located. Similarly, although VNGV_NEWMAILBOX and
//  VNGV_HOMEBOX will usually be the same, you should not assume this: use the
//  one that most exactly fits your needs.
//
  VN_GETVARIABLE = (WM_USER + 2772);
//      wParam - a selector from the list shown above
//      lParam - a pointer to a buffer of the size shown above
//      Returns: 1 on success, 0 on failure


function TidyHtml(const Html: String; BodyOnly: Longbool): String;


implementation


function LoadIERenderer(var BearApi: TBearApi): Longbool;
const
  IER = 'IERenderer.dll';
begin
  Result := (GetModuleHandle(PChar(IER)) <> 0) and (Assigned(BearApi.CreateViewer) and Assigned(BearApi.ViewerCommand));
  if not Result then begin
    Result := Longbool(GetModuleHandle(PChar(IER)));
    if Result then with BearApi do begin
      CreateViewer := GetProcAddress(THandle(Result), 'CreateViewer');
      ViewerCommand := GetProcAddress(THandle(Result), 'ViewerCommand');
      Result := (Assigned(BearApi.CreateViewer) and Assigned(BearApi.ViewerCommand));
    end;
  end;
end;

function TidyHtml(const Html: String; BodyOnly: Longbool): String;
var
  tr: TTidyRequest;
  ba: TBearApi;
begin
  FillChar(ba, SizeOf(TBearApi), 0);
  if LoadIERenderer(ba) then begin
    with tr do begin
      size := SizeOf(TStringRequest);
      hInput := GetStringHandle(Html, 0);
      hOutput := GetStringHandle('PMPGP', 0);
    end;
    tr.BodyOnly := BodyOnly;
    try
      if ba.ViewerCommand(0, VO_TIDYFILE, 0, Longint(@tr)) > 0 then begin
        Result := GetHandleString(tr.hOutput, false);
        Exit;
      end;
    finally
      GlobalFree(tr.hOutput);
      GlobalFree(tr.hInput);
    end;
  end;
  Result := Html;
end;


end.

